// TODO DEBUGGING, not in any DWG
#define DWG_TYPE DWG_TYPE_PLACEHOLDER
#include "tests_common.c"

void
api_process (dwg_object *obj)
{
  int error;
  dwg_obj_placeholder *_obj = dwg_object_to_PLACEHOLDER (obj);
}
