/* ex: set ro ft=c: -*- mode: c; buffer-read-only: t -*- */
/* generated by make regen-unknown, do not modify */

// code, value, bits, pre_bits, num_bits, type, name, num, pos[]
/* ASSOCDIMDEPENDENCYBODY C36 in test/test-data/2010/civil_example-imperial.dxf */
static struct _unknown_field unknown_dxf_ASSOCDIMDEPENDENCYBODY_723[] = {
  //{ 5, "C36", NULL, 0, BITS_UNKNOWN, "handle", 1, {-1,-1,-1,-1,-1} },
    { 330, "C35", NULL, 0, BITS_UNKNOWN, "ownerhandle", 1, {-1,-1,-1,-1,-1} },
    { 100, "AcDbAssocDependencyBody", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 100, "AcDbImpAssocDimDependencyBodyBase", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 1, "ROW=60", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 100, "AcDbAssocDimDependencyBody", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 0, NULL, NULL, 0, BITS_UNKNOWN, NULL, 0, {-1,-1,-1,-1,-1}}
};
/* ASSOCDIMDEPENDENCYBODY C4C in test/test-data/2010/civil_example-imperial.dxf */
static struct _unknown_field unknown_dxf_ASSOCDIMDEPENDENCYBODY_724[] = {
  //{ 5, "C4C", NULL, 0, BITS_UNKNOWN, "handle", 1, {-1,-1,-1,-1,-1} },
    { 330, "C4B", NULL, 0, BITS_UNKNOWN, "ownerhandle", 1, {-1,-1,-1,-1,-1} },
    { 100, "AcDbAssocDependencyBody", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 100, "AcDbImpAssocDimDependencyBodyBase", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 1, "UENorth=5", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 100, "AcDbAssocDimDependencyBody", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 0, NULL, NULL, 0, BITS_UNKNOWN, NULL, 0, {-1,-1,-1,-1,-1}}
};
/* ASSOCDIMDEPENDENCYBODY BB20 in test/test-data/2018/Dynblocks.dxf */
static struct _unknown_field unknown_dxf_ASSOCDIMDEPENDENCYBODY_725[] = {
  //{ 5, "BB20", NULL, 0, BITS_UNKNOWN, "handle", 1, {-1,-1,-1,-1,-1} },
    { 330, "BB1F", NULL, 0, BITS_UNKNOWN, "ownerhandle", 1, {-1,-1,-1,-1,-1} },
    { 100, "AcDbAssocDependencyBody", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 100, "AcDbImpAssocDimDependencyBodyBase", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 1, "4.0000", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 100, "AcDbAssocDimDependencyBody", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 0, NULL, NULL, 0, BITS_UNKNOWN, NULL, 0, {-1,-1,-1,-1,-1}}
};
/* ASSOCDIMDEPENDENCYBODY BB28 in test/test-data/2018/Dynblocks.dxf */
static struct _unknown_field unknown_dxf_ASSOCDIMDEPENDENCYBODY_726[] = {
  //{ 5, "BB28", NULL, 0, BITS_UNKNOWN, "handle", 1, {-1,-1,-1,-1,-1} },
    { 330, "BB27", NULL, 0, BITS_UNKNOWN, "ownerhandle", 1, {-1,-1,-1,-1,-1} },
    { 100, "AcDbAssocDependencyBody", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 100, "AcDbImpAssocDimDependencyBodyBase", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 1, "9.0230", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 100, "AcDbAssocDimDependencyBody", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 0, NULL, NULL, 0, BITS_UNKNOWN, NULL, 0, {-1,-1,-1,-1,-1}}
};
/* ASSOCDIMDEPENDENCYBODY 49B in test/test-data/2010/mechanical_example-imperial.dxf */
static struct _unknown_field unknown_dxf_ASSOCDIMDEPENDENCYBODY_727[] = {
  //{ 5, "49B", NULL, 0, BITS_UNKNOWN, "handle", 1, {-1,-1,-1,-1,-1} },
    { 330, "49A", NULL, 0, BITS_UNKNOWN, "ownerhandle", 1, {-1,-1,-1,-1,-1} },
    { 100, "AcDbAssocDependencyBody", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 100, "AcDbImpAssocDimDependencyBodyBase", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 1, "BOLTHOLE=1\\\"", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 100, "AcDbAssocDimDependencyBody", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 3, {-1,-1,-1,-1,-1} },
    { 0, NULL, NULL, 0, BITS_UNKNOWN, NULL, 0, {-1,-1,-1,-1,-1}}
};
