// TODO DEBUGGING
#define DWG_TYPE DWG_TYPE_TABLECONTENT
#include "tests_common.c"

void
api_process (dwg_object *obj)
{
  int error;
#ifdef DEBUG_CLASSES
  dwg_obj_tablecontent *_obj = dwg_object_to_TABLECONTENT (obj);
#endif
}
