# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.3.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
# Import the low-level C/C++ module
if getattr(globals().get("__spec__"), "parent", None) or globals().get("__package__") or "." in __name__:
    from . import _LibreDWG
else:
    import _LibreDWG

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "this":
            set(self, name, value)
        elif name == "thisown":
            self.this.own(value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def new_Dwg_Object_Array(nelements):
    return _LibreDWG.new_Dwg_Object_Array(nelements)

def delete_Dwg_Object_Array(ary):
    return _LibreDWG.delete_Dwg_Object_Array(ary)

def Dwg_Object_Array_getitem(ary, index):
    return _LibreDWG.Dwg_Object_Array_getitem(ary, index)

def Dwg_Object_Array_setitem(ary, index, value):
    return _LibreDWG.Dwg_Object_Array_setitem(ary, index, value)
LIBREDWG_VERSION_MAJOR = _LibreDWG.LIBREDWG_VERSION_MAJOR
LIBREDWG_VERSION_MINOR = _LibreDWG.LIBREDWG_VERSION_MINOR
LIBREDWG_VERSION = _LibreDWG.LIBREDWG_VERSION
FORMAT_RC = _LibreDWG.FORMAT_RC
SCANF_2X = _LibreDWG.SCANF_2X
FORMAT_RCd = _LibreDWG.FORMAT_RCd
FORMAT_RCu = _LibreDWG.FORMAT_RCu
FORMAT_RCx = _LibreDWG.FORMAT_RCx
FORMAT_B = _LibreDWG.FORMAT_B
FORMAT_BB = _LibreDWG.FORMAT_BB
FORMAT_3B = _LibreDWG.FORMAT_3B
FORMAT_RD = _LibreDWG.FORMAT_RD
FORMAT_TF = _LibreDWG.FORMAT_TF
FORMAT_TV = _LibreDWG.FORMAT_TV
FORMAT_T16 = _LibreDWG.FORMAT_T16
FORMAT_T32 = _LibreDWG.FORMAT_T32
FORMAT_TU32 = _LibreDWG.FORMAT_TU32
FORMAT_BT = _LibreDWG.FORMAT_BT
FORMAT_DD = _LibreDWG.FORMAT_DD
FORMAT_BD = _LibreDWG.FORMAT_BD
FORMAT_4BITS = _LibreDWG.FORMAT_4BITS
FORMAT_D2T = _LibreDWG.FORMAT_D2T
FORMAT_TU = _LibreDWG.FORMAT_TU
class Dwg_Bitcode_TimeBLL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    days = property(_LibreDWG.Dwg_Bitcode_TimeBLL_days_get, _LibreDWG.Dwg_Bitcode_TimeBLL_days_set)
    ms = property(_LibreDWG.Dwg_Bitcode_TimeBLL_ms_get, _LibreDWG.Dwg_Bitcode_TimeBLL_ms_set)
    value = property(_LibreDWG.Dwg_Bitcode_TimeBLL_value_get, _LibreDWG.Dwg_Bitcode_TimeBLL_value_set)

    def __init__(self):
        _LibreDWG.Dwg_Bitcode_TimeBLL_swiginit(self, _LibreDWG.new_Dwg_Bitcode_TimeBLL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Bitcode_TimeBLL

# Register Dwg_Bitcode_TimeBLL in _LibreDWG:
_LibreDWG.Dwg_Bitcode_TimeBLL_swigregister(Dwg_Bitcode_TimeBLL)
class Dwg_Bitcode_2RD(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_LibreDWG.Dwg_Bitcode_2RD_x_get, _LibreDWG.Dwg_Bitcode_2RD_x_set)
    y = property(_LibreDWG.Dwg_Bitcode_2RD_y_get, _LibreDWG.Dwg_Bitcode_2RD_y_set)

    def __init__(self):
        _LibreDWG.Dwg_Bitcode_2RD_swiginit(self, _LibreDWG.new_Dwg_Bitcode_2RD())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Bitcode_2RD

# Register Dwg_Bitcode_2RD in _LibreDWG:
_LibreDWG.Dwg_Bitcode_2RD_swigregister(Dwg_Bitcode_2RD)
class Dwg_Bitcode_2BD(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_LibreDWG.Dwg_Bitcode_2BD_x_get, _LibreDWG.Dwg_Bitcode_2BD_x_set)
    y = property(_LibreDWG.Dwg_Bitcode_2BD_y_get, _LibreDWG.Dwg_Bitcode_2BD_y_set)

    def __init__(self):
        _LibreDWG.Dwg_Bitcode_2BD_swiginit(self, _LibreDWG.new_Dwg_Bitcode_2BD())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Bitcode_2BD

# Register Dwg_Bitcode_2BD in _LibreDWG:
_LibreDWG.Dwg_Bitcode_2BD_swigregister(Dwg_Bitcode_2BD)
class Dwg_Bitcode_3RD(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_LibreDWG.Dwg_Bitcode_3RD_x_get, _LibreDWG.Dwg_Bitcode_3RD_x_set)
    y = property(_LibreDWG.Dwg_Bitcode_3RD_y_get, _LibreDWG.Dwg_Bitcode_3RD_y_set)
    z = property(_LibreDWG.Dwg_Bitcode_3RD_z_get, _LibreDWG.Dwg_Bitcode_3RD_z_set)

    def __init__(self):
        _LibreDWG.Dwg_Bitcode_3RD_swiginit(self, _LibreDWG.new_Dwg_Bitcode_3RD())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Bitcode_3RD

# Register Dwg_Bitcode_3RD in _LibreDWG:
_LibreDWG.Dwg_Bitcode_3RD_swigregister(Dwg_Bitcode_3RD)
class Dwg_Bitcode_3BD(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_LibreDWG.Dwg_Bitcode_3BD_x_get, _LibreDWG.Dwg_Bitcode_3BD_x_set)
    y = property(_LibreDWG.Dwg_Bitcode_3BD_y_get, _LibreDWG.Dwg_Bitcode_3BD_y_set)
    z = property(_LibreDWG.Dwg_Bitcode_3BD_z_get, _LibreDWG.Dwg_Bitcode_3BD_z_set)

    def __init__(self):
        _LibreDWG.Dwg_Bitcode_3BD_swiginit(self, _LibreDWG.new_Dwg_Bitcode_3BD())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Bitcode_3BD

# Register Dwg_Bitcode_3BD in _LibreDWG:
_LibreDWG.Dwg_Bitcode_3BD_swigregister(Dwg_Bitcode_3BD)
R_INVALID = _LibreDWG.R_INVALID
R_1_1 = _LibreDWG.R_1_1
R_1_2 = _LibreDWG.R_1_2
R_1_3 = _LibreDWG.R_1_3
R_1_4 = _LibreDWG.R_1_4
R_2_0b = _LibreDWG.R_2_0b
R_2_0 = _LibreDWG.R_2_0
R_2_10 = _LibreDWG.R_2_10
R_2_21 = _LibreDWG.R_2_21
R_2_22 = _LibreDWG.R_2_22
R_2_4 = _LibreDWG.R_2_4
R_2_5 = _LibreDWG.R_2_5
R_2_6 = _LibreDWG.R_2_6
R_9 = _LibreDWG.R_9
R_9c1 = _LibreDWG.R_9c1
R_10 = _LibreDWG.R_10
R_11b1 = _LibreDWG.R_11b1
R_11b2 = _LibreDWG.R_11b2
R_11 = _LibreDWG.R_11
R_12 = _LibreDWG.R_12
R_13b1 = _LibreDWG.R_13b1
R_13b2 = _LibreDWG.R_13b2
R_13 = _LibreDWG.R_13
R_13c3 = _LibreDWG.R_13c3
R_14 = _LibreDWG.R_14
R_2000b = _LibreDWG.R_2000b
R_2000 = _LibreDWG.R_2000
R_2000i = _LibreDWG.R_2000i
R_2002 = _LibreDWG.R_2002
R_2004a = _LibreDWG.R_2004a
R_2004b = _LibreDWG.R_2004b
R_2004c = _LibreDWG.R_2004c
R_2004 = _LibreDWG.R_2004
R_2007a = _LibreDWG.R_2007a
R_2007b = _LibreDWG.R_2007b
R_2007 = _LibreDWG.R_2007
R_2010b = _LibreDWG.R_2010b
R_2010 = _LibreDWG.R_2010
R_2013b = _LibreDWG.R_2013b
R_2013 = _LibreDWG.R_2013
R_2018b = _LibreDWG.R_2018b
R_2018 = _LibreDWG.R_2018
R_2022b = _LibreDWG.R_2022b
R_AFTER = _LibreDWG.R_AFTER
class Dwg_Versions(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    r = property(_LibreDWG.Dwg_Versions_r_get, _LibreDWG.Dwg_Versions_r_set)
    type = property(_LibreDWG.Dwg_Versions_type_get)
    hdr = property(_LibreDWG.Dwg_Versions_hdr_get)
    desc = property(_LibreDWG.Dwg_Versions_desc_get)
    dwg_version = property(_LibreDWG.Dwg_Versions_dwg_version_get, _LibreDWG.Dwg_Versions_dwg_version_set)
    maint_rel_version = property(_LibreDWG.Dwg_Versions_maint_rel_version_get, _LibreDWG.Dwg_Versions_maint_rel_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Versions_swiginit(self, _LibreDWG.new_Dwg_Versions())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Versions

# Register Dwg_Versions in _LibreDWG:
_LibreDWG.Dwg_Versions_swigregister(Dwg_Versions)
DWG_CLASS_STABLE = _LibreDWG.DWG_CLASS_STABLE
DWG_CLASS_UNSTABLE = _LibreDWG.DWG_CLASS_UNSTABLE
DWG_CLASS_DEBUGGING = _LibreDWG.DWG_CLASS_DEBUGGING
DWG_CLASS_UNHANDLED = _LibreDWG.DWG_CLASS_UNHANDLED
DWG_ENTITY_SECTION = _LibreDWG.DWG_ENTITY_SECTION
DWG_BLOCKS_SECTION = _LibreDWG.DWG_BLOCKS_SECTION
DWG_EXTRA_SECTION = _LibreDWG.DWG_EXTRA_SECTION
DWG_SUPERTYPE_ENTITY = _LibreDWG.DWG_SUPERTYPE_ENTITY
DWG_SUPERTYPE_OBJECT = _LibreDWG.DWG_SUPERTYPE_OBJECT
DWG_TYPE_UNUSED = _LibreDWG.DWG_TYPE_UNUSED
DWG_TYPE_TEXT = _LibreDWG.DWG_TYPE_TEXT
DWG_TYPE_ATTRIB = _LibreDWG.DWG_TYPE_ATTRIB
DWG_TYPE_ATTDEF = _LibreDWG.DWG_TYPE_ATTDEF
DWG_TYPE_BLOCK = _LibreDWG.DWG_TYPE_BLOCK
DWG_TYPE_ENDBLK = _LibreDWG.DWG_TYPE_ENDBLK
DWG_TYPE_SEQEND = _LibreDWG.DWG_TYPE_SEQEND
DWG_TYPE_INSERT = _LibreDWG.DWG_TYPE_INSERT
DWG_TYPE_MINSERT = _LibreDWG.DWG_TYPE_MINSERT
DWG_TYPE_VERTEX_2D = _LibreDWG.DWG_TYPE_VERTEX_2D
DWG_TYPE_VERTEX_3D = _LibreDWG.DWG_TYPE_VERTEX_3D
DWG_TYPE_VERTEX_MESH = _LibreDWG.DWG_TYPE_VERTEX_MESH
DWG_TYPE_VERTEX_PFACE = _LibreDWG.DWG_TYPE_VERTEX_PFACE
DWG_TYPE_VERTEX_PFACE_FACE = _LibreDWG.DWG_TYPE_VERTEX_PFACE_FACE
DWG_TYPE_POLYLINE_2D = _LibreDWG.DWG_TYPE_POLYLINE_2D
DWG_TYPE_POLYLINE_3D = _LibreDWG.DWG_TYPE_POLYLINE_3D
DWG_TYPE_ARC = _LibreDWG.DWG_TYPE_ARC
DWG_TYPE_CIRCLE = _LibreDWG.DWG_TYPE_CIRCLE
DWG_TYPE_LINE = _LibreDWG.DWG_TYPE_LINE
DWG_TYPE_DIMENSION_ORDINATE = _LibreDWG.DWG_TYPE_DIMENSION_ORDINATE
DWG_TYPE_DIMENSION_LINEAR = _LibreDWG.DWG_TYPE_DIMENSION_LINEAR
DWG_TYPE_DIMENSION_ALIGNED = _LibreDWG.DWG_TYPE_DIMENSION_ALIGNED
DWG_TYPE_DIMENSION_ANG3PT = _LibreDWG.DWG_TYPE_DIMENSION_ANG3PT
DWG_TYPE_DIMENSION_ANG2LN = _LibreDWG.DWG_TYPE_DIMENSION_ANG2LN
DWG_TYPE_DIMENSION_RADIUS = _LibreDWG.DWG_TYPE_DIMENSION_RADIUS
DWG_TYPE_DIMENSION_DIAMETER = _LibreDWG.DWG_TYPE_DIMENSION_DIAMETER
DWG_TYPE_POINT = _LibreDWG.DWG_TYPE_POINT
DWG_TYPE__3DFACE = _LibreDWG.DWG_TYPE__3DFACE
DWG_TYPE_POLYLINE_PFACE = _LibreDWG.DWG_TYPE_POLYLINE_PFACE
DWG_TYPE_POLYLINE_MESH = _LibreDWG.DWG_TYPE_POLYLINE_MESH
DWG_TYPE_SOLID = _LibreDWG.DWG_TYPE_SOLID
DWG_TYPE_TRACE = _LibreDWG.DWG_TYPE_TRACE
DWG_TYPE_SHAPE = _LibreDWG.DWG_TYPE_SHAPE
DWG_TYPE_VIEWPORT = _LibreDWG.DWG_TYPE_VIEWPORT
DWG_TYPE_ELLIPSE = _LibreDWG.DWG_TYPE_ELLIPSE
DWG_TYPE_SPLINE = _LibreDWG.DWG_TYPE_SPLINE
DWG_TYPE_REGION = _LibreDWG.DWG_TYPE_REGION
DWG_TYPE__3DSOLID = _LibreDWG.DWG_TYPE__3DSOLID
DWG_TYPE_BODY = _LibreDWG.DWG_TYPE_BODY
DWG_TYPE_RAY = _LibreDWG.DWG_TYPE_RAY
DWG_TYPE_XLINE = _LibreDWG.DWG_TYPE_XLINE
DWG_TYPE_DICTIONARY = _LibreDWG.DWG_TYPE_DICTIONARY
DWG_TYPE_OLEFRAME = _LibreDWG.DWG_TYPE_OLEFRAME
DWG_TYPE_MTEXT = _LibreDWG.DWG_TYPE_MTEXT
DWG_TYPE_LEADER = _LibreDWG.DWG_TYPE_LEADER
DWG_TYPE_TOLERANCE = _LibreDWG.DWG_TYPE_TOLERANCE
DWG_TYPE_MLINE = _LibreDWG.DWG_TYPE_MLINE
DWG_TYPE_BLOCK_CONTROL = _LibreDWG.DWG_TYPE_BLOCK_CONTROL
DWG_TYPE_BLOCK_HEADER = _LibreDWG.DWG_TYPE_BLOCK_HEADER
DWG_TYPE_LAYER_CONTROL = _LibreDWG.DWG_TYPE_LAYER_CONTROL
DWG_TYPE_LAYER = _LibreDWG.DWG_TYPE_LAYER
DWG_TYPE_STYLE_CONTROL = _LibreDWG.DWG_TYPE_STYLE_CONTROL
DWG_TYPE_STYLE = _LibreDWG.DWG_TYPE_STYLE
DWG_TYPE_LTYPE_CONTROL = _LibreDWG.DWG_TYPE_LTYPE_CONTROL
DWG_TYPE_LTYPE = _LibreDWG.DWG_TYPE_LTYPE
DWG_TYPE_VIEW_CONTROL = _LibreDWG.DWG_TYPE_VIEW_CONTROL
DWG_TYPE_VIEW = _LibreDWG.DWG_TYPE_VIEW
DWG_TYPE_UCS_CONTROL = _LibreDWG.DWG_TYPE_UCS_CONTROL
DWG_TYPE_UCS = _LibreDWG.DWG_TYPE_UCS
DWG_TYPE_VPORT_CONTROL = _LibreDWG.DWG_TYPE_VPORT_CONTROL
DWG_TYPE_VPORT = _LibreDWG.DWG_TYPE_VPORT
DWG_TYPE_APPID_CONTROL = _LibreDWG.DWG_TYPE_APPID_CONTROL
DWG_TYPE_APPID = _LibreDWG.DWG_TYPE_APPID
DWG_TYPE_DIMSTYLE_CONTROL = _LibreDWG.DWG_TYPE_DIMSTYLE_CONTROL
DWG_TYPE_DIMSTYLE = _LibreDWG.DWG_TYPE_DIMSTYLE
DWG_TYPE_VX_CONTROL = _LibreDWG.DWG_TYPE_VX_CONTROL
DWG_TYPE_VX_TABLE_RECORD = _LibreDWG.DWG_TYPE_VX_TABLE_RECORD
DWG_TYPE_GROUP = _LibreDWG.DWG_TYPE_GROUP
DWG_TYPE_MLINESTYLE = _LibreDWG.DWG_TYPE_MLINESTYLE
DWG_TYPE_OLE2FRAME = _LibreDWG.DWG_TYPE_OLE2FRAME
DWG_TYPE_DUMMY = _LibreDWG.DWG_TYPE_DUMMY
DWG_TYPE_LONG_TRANSACTION = _LibreDWG.DWG_TYPE_LONG_TRANSACTION
DWG_TYPE_LWPOLYLINE = _LibreDWG.DWG_TYPE_LWPOLYLINE
DWG_TYPE_HATCH = _LibreDWG.DWG_TYPE_HATCH
DWG_TYPE_XRECORD = _LibreDWG.DWG_TYPE_XRECORD
DWG_TYPE_PLACEHOLDER = _LibreDWG.DWG_TYPE_PLACEHOLDER
DWG_TYPE_VBA_PROJECT = _LibreDWG.DWG_TYPE_VBA_PROJECT
DWG_TYPE_LAYOUT = _LibreDWG.DWG_TYPE_LAYOUT
DWG_TYPE_PROXY_ENTITY = _LibreDWG.DWG_TYPE_PROXY_ENTITY
DWG_TYPE_PROXY_OBJECT = _LibreDWG.DWG_TYPE_PROXY_OBJECT
DWG_TYPE_ACDSRECORD = _LibreDWG.DWG_TYPE_ACDSRECORD
DWG_TYPE_ACDSSCHEMA = _LibreDWG.DWG_TYPE_ACDSSCHEMA
DWG_TYPE_ACMECOMMANDHISTORY = _LibreDWG.DWG_TYPE_ACMECOMMANDHISTORY
DWG_TYPE_ACMESCOPE = _LibreDWG.DWG_TYPE_ACMESCOPE
DWG_TYPE_ACMESTATEMGR = _LibreDWG.DWG_TYPE_ACMESTATEMGR
DWG_TYPE_ACSH_BOOLEAN_CLASS = _LibreDWG.DWG_TYPE_ACSH_BOOLEAN_CLASS
DWG_TYPE_ACSH_BOX_CLASS = _LibreDWG.DWG_TYPE_ACSH_BOX_CLASS
DWG_TYPE_ACSH_BREP_CLASS = _LibreDWG.DWG_TYPE_ACSH_BREP_CLASS
DWG_TYPE_ACSH_CHAMFER_CLASS = _LibreDWG.DWG_TYPE_ACSH_CHAMFER_CLASS
DWG_TYPE_ACSH_CONE_CLASS = _LibreDWG.DWG_TYPE_ACSH_CONE_CLASS
DWG_TYPE_ACSH_CYLINDER_CLASS = _LibreDWG.DWG_TYPE_ACSH_CYLINDER_CLASS
DWG_TYPE_ACSH_EXTRUSION_CLASS = _LibreDWG.DWG_TYPE_ACSH_EXTRUSION_CLASS
DWG_TYPE_ACSH_FILLET_CLASS = _LibreDWG.DWG_TYPE_ACSH_FILLET_CLASS
DWG_TYPE_ACSH_HISTORY_CLASS = _LibreDWG.DWG_TYPE_ACSH_HISTORY_CLASS
DWG_TYPE_ACSH_LOFT_CLASS = _LibreDWG.DWG_TYPE_ACSH_LOFT_CLASS
DWG_TYPE_ACSH_PYRAMID_CLASS = _LibreDWG.DWG_TYPE_ACSH_PYRAMID_CLASS
DWG_TYPE_ACSH_REVOLVE_CLASS = _LibreDWG.DWG_TYPE_ACSH_REVOLVE_CLASS
DWG_TYPE_ACSH_SPHERE_CLASS = _LibreDWG.DWG_TYPE_ACSH_SPHERE_CLASS
DWG_TYPE_ACSH_SWEEP_CLASS = _LibreDWG.DWG_TYPE_ACSH_SWEEP_CLASS
DWG_TYPE_ACSH_TORUS_CLASS = _LibreDWG.DWG_TYPE_ACSH_TORUS_CLASS
DWG_TYPE_ACSH_WEDGE_CLASS = _LibreDWG.DWG_TYPE_ACSH_WEDGE_CLASS
DWG_TYPE_ALDIMOBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_ALDIMOBJECTCONTEXTDATA
DWG_TYPE_ALIGNMENTPARAMETERENTITY = _LibreDWG.DWG_TYPE_ALIGNMENTPARAMETERENTITY
DWG_TYPE_ANGDIMOBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_ANGDIMOBJECTCONTEXTDATA
DWG_TYPE_ANNOTSCALEOBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_ANNOTSCALEOBJECTCONTEXTDATA
DWG_TYPE_ARCALIGNEDTEXT = _LibreDWG.DWG_TYPE_ARCALIGNEDTEXT
DWG_TYPE_ARC_DIMENSION = _LibreDWG.DWG_TYPE_ARC_DIMENSION
DWG_TYPE_ASSOC2DCONSTRAINTGROUP = _LibreDWG.DWG_TYPE_ASSOC2DCONSTRAINTGROUP
DWG_TYPE_ASSOC3POINTANGULARDIMACTIONBODY = _LibreDWG.DWG_TYPE_ASSOC3POINTANGULARDIMACTIONBODY
DWG_TYPE_ASSOCACTION = _LibreDWG.DWG_TYPE_ASSOCACTION
DWG_TYPE_ASSOCACTIONPARAM = _LibreDWG.DWG_TYPE_ASSOCACTIONPARAM
DWG_TYPE_ASSOCALIGNEDDIMACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCALIGNEDDIMACTIONBODY
DWG_TYPE_ASSOCARRAYACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCARRAYACTIONBODY
DWG_TYPE_ASSOCARRAYMODIFYACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCARRAYMODIFYACTIONBODY
DWG_TYPE_ASSOCARRAYMODIFYPARAMETERS = _LibreDWG.DWG_TYPE_ASSOCARRAYMODIFYPARAMETERS
DWG_TYPE_ASSOCARRAYPATHPARAMETERS = _LibreDWG.DWG_TYPE_ASSOCARRAYPATHPARAMETERS
DWG_TYPE_ASSOCARRAYPOLARPARAMETERS = _LibreDWG.DWG_TYPE_ASSOCARRAYPOLARPARAMETERS
DWG_TYPE_ASSOCARRAYRECTANGULARPARAMETERS = _LibreDWG.DWG_TYPE_ASSOCARRAYRECTANGULARPARAMETERS
DWG_TYPE_ASSOCASMBODYACTIONPARAM = _LibreDWG.DWG_TYPE_ASSOCASMBODYACTIONPARAM
DWG_TYPE_ASSOCBLENDSURFACEACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCBLENDSURFACEACTIONBODY
DWG_TYPE_ASSOCCOMPOUNDACTIONPARAM = _LibreDWG.DWG_TYPE_ASSOCCOMPOUNDACTIONPARAM
DWG_TYPE_ASSOCDEPENDENCY = _LibreDWG.DWG_TYPE_ASSOCDEPENDENCY
DWG_TYPE_ASSOCDIMDEPENDENCYBODY = _LibreDWG.DWG_TYPE_ASSOCDIMDEPENDENCYBODY
DWG_TYPE_ASSOCEDGEACTIONPARAM = _LibreDWG.DWG_TYPE_ASSOCEDGEACTIONPARAM
DWG_TYPE_ASSOCEDGECHAMFERACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCEDGECHAMFERACTIONBODY
DWG_TYPE_ASSOCEDGEFILLETACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCEDGEFILLETACTIONBODY
DWG_TYPE_ASSOCEXTENDSURFACEACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCEXTENDSURFACEACTIONBODY
DWG_TYPE_ASSOCEXTRUDEDSURFACEACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCEXTRUDEDSURFACEACTIONBODY
DWG_TYPE_ASSOCFACEACTIONPARAM = _LibreDWG.DWG_TYPE_ASSOCFACEACTIONPARAM
DWG_TYPE_ASSOCFILLETSURFACEACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCFILLETSURFACEACTIONBODY
DWG_TYPE_ASSOCGEOMDEPENDENCY = _LibreDWG.DWG_TYPE_ASSOCGEOMDEPENDENCY
DWG_TYPE_ASSOCLOFTEDSURFACEACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCLOFTEDSURFACEACTIONBODY
DWG_TYPE_ASSOCMLEADERACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCMLEADERACTIONBODY
DWG_TYPE_ASSOCNETWORK = _LibreDWG.DWG_TYPE_ASSOCNETWORK
DWG_TYPE_ASSOCNETWORKSURFACEACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCNETWORKSURFACEACTIONBODY
DWG_TYPE_ASSOCOBJECTACTIONPARAM = _LibreDWG.DWG_TYPE_ASSOCOBJECTACTIONPARAM
DWG_TYPE_ASSOCOFFSETSURFACEACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCOFFSETSURFACEACTIONBODY
DWG_TYPE_ASSOCORDINATEDIMACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCORDINATEDIMACTIONBODY
DWG_TYPE_ASSOCOSNAPPOINTREFACTIONPARAM = _LibreDWG.DWG_TYPE_ASSOCOSNAPPOINTREFACTIONPARAM
DWG_TYPE_ASSOCPATCHSURFACEACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCPATCHSURFACEACTIONBODY
DWG_TYPE_ASSOCPATHACTIONPARAM = _LibreDWG.DWG_TYPE_ASSOCPATHACTIONPARAM
DWG_TYPE_ASSOCPERSSUBENTMANAGER = _LibreDWG.DWG_TYPE_ASSOCPERSSUBENTMANAGER
DWG_TYPE_ASSOCPLANESURFACEACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCPLANESURFACEACTIONBODY
DWG_TYPE_ASSOCPOINTREFACTIONPARAM = _LibreDWG.DWG_TYPE_ASSOCPOINTREFACTIONPARAM
DWG_TYPE_ASSOCRESTOREENTITYSTATEACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCRESTOREENTITYSTATEACTIONBODY
DWG_TYPE_ASSOCREVOLVEDSURFACEACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCREVOLVEDSURFACEACTIONBODY
DWG_TYPE_ASSOCROTATEDDIMACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCROTATEDDIMACTIONBODY
DWG_TYPE_ASSOCSWEPTSURFACEACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCSWEPTSURFACEACTIONBODY
DWG_TYPE_ASSOCTRIMSURFACEACTIONBODY = _LibreDWG.DWG_TYPE_ASSOCTRIMSURFACEACTIONBODY
DWG_TYPE_ASSOCVALUEDEPENDENCY = _LibreDWG.DWG_TYPE_ASSOCVALUEDEPENDENCY
DWG_TYPE_ASSOCVARIABLE = _LibreDWG.DWG_TYPE_ASSOCVARIABLE
DWG_TYPE_ASSOCVERTEXACTIONPARAM = _LibreDWG.DWG_TYPE_ASSOCVERTEXACTIONPARAM
DWG_TYPE_BASEPOINTPARAMETERENTITY = _LibreDWG.DWG_TYPE_BASEPOINTPARAMETERENTITY
DWG_TYPE_BLKREFOBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_BLKREFOBJECTCONTEXTDATA
DWG_TYPE_BLOCKALIGNEDCONSTRAINTPARAMETER = _LibreDWG.DWG_TYPE_BLOCKALIGNEDCONSTRAINTPARAMETER
DWG_TYPE_BLOCKALIGNMENTGRIP = _LibreDWG.DWG_TYPE_BLOCKALIGNMENTGRIP
DWG_TYPE_BLOCKALIGNMENTPARAMETER = _LibreDWG.DWG_TYPE_BLOCKALIGNMENTPARAMETER
DWG_TYPE_BLOCKANGULARCONSTRAINTPARAMETER = _LibreDWG.DWG_TYPE_BLOCKANGULARCONSTRAINTPARAMETER
DWG_TYPE_BLOCKARRAYACTION = _LibreDWG.DWG_TYPE_BLOCKARRAYACTION
DWG_TYPE_BLOCKBASEPOINTPARAMETER = _LibreDWG.DWG_TYPE_BLOCKBASEPOINTPARAMETER
DWG_TYPE_BLOCKDIAMETRICCONSTRAINTPARAMETER = _LibreDWG.DWG_TYPE_BLOCKDIAMETRICCONSTRAINTPARAMETER
DWG_TYPE_BLOCKFLIPACTION = _LibreDWG.DWG_TYPE_BLOCKFLIPACTION
DWG_TYPE_BLOCKFLIPGRIP = _LibreDWG.DWG_TYPE_BLOCKFLIPGRIP
DWG_TYPE_BLOCKFLIPPARAMETER = _LibreDWG.DWG_TYPE_BLOCKFLIPPARAMETER
DWG_TYPE_BLOCKGRIPLOCATIONCOMPONENT = _LibreDWG.DWG_TYPE_BLOCKGRIPLOCATIONCOMPONENT
DWG_TYPE_BLOCKHORIZONTALCONSTRAINTPARAMETER = _LibreDWG.DWG_TYPE_BLOCKHORIZONTALCONSTRAINTPARAMETER
DWG_TYPE_BLOCKLINEARCONSTRAINTPARAMETER = _LibreDWG.DWG_TYPE_BLOCKLINEARCONSTRAINTPARAMETER
DWG_TYPE_BLOCKLINEARGRIP = _LibreDWG.DWG_TYPE_BLOCKLINEARGRIP
DWG_TYPE_BLOCKLINEARPARAMETER = _LibreDWG.DWG_TYPE_BLOCKLINEARPARAMETER
DWG_TYPE_BLOCKLOOKUPACTION = _LibreDWG.DWG_TYPE_BLOCKLOOKUPACTION
DWG_TYPE_BLOCKLOOKUPGRIP = _LibreDWG.DWG_TYPE_BLOCKLOOKUPGRIP
DWG_TYPE_BLOCKLOOKUPPARAMETER = _LibreDWG.DWG_TYPE_BLOCKLOOKUPPARAMETER
DWG_TYPE_BLOCKMOVEACTION = _LibreDWG.DWG_TYPE_BLOCKMOVEACTION
DWG_TYPE_BLOCKPARAMDEPENDENCYBODY = _LibreDWG.DWG_TYPE_BLOCKPARAMDEPENDENCYBODY
DWG_TYPE_BLOCKPOINTPARAMETER = _LibreDWG.DWG_TYPE_BLOCKPOINTPARAMETER
DWG_TYPE_BLOCKPOLARGRIP = _LibreDWG.DWG_TYPE_BLOCKPOLARGRIP
DWG_TYPE_BLOCKPOLARPARAMETER = _LibreDWG.DWG_TYPE_BLOCKPOLARPARAMETER
DWG_TYPE_BLOCKPOLARSTRETCHACTION = _LibreDWG.DWG_TYPE_BLOCKPOLARSTRETCHACTION
DWG_TYPE_BLOCKPROPERTIESTABLE = _LibreDWG.DWG_TYPE_BLOCKPROPERTIESTABLE
DWG_TYPE_BLOCKPROPERTIESTABLEGRIP = _LibreDWG.DWG_TYPE_BLOCKPROPERTIESTABLEGRIP
DWG_TYPE_BLOCKRADIALCONSTRAINTPARAMETER = _LibreDWG.DWG_TYPE_BLOCKRADIALCONSTRAINTPARAMETER
DWG_TYPE_BLOCKREPRESENTATION = _LibreDWG.DWG_TYPE_BLOCKREPRESENTATION
DWG_TYPE_BLOCKROTATEACTION = _LibreDWG.DWG_TYPE_BLOCKROTATEACTION
DWG_TYPE_BLOCKROTATIONGRIP = _LibreDWG.DWG_TYPE_BLOCKROTATIONGRIP
DWG_TYPE_BLOCKROTATIONPARAMETER = _LibreDWG.DWG_TYPE_BLOCKROTATIONPARAMETER
DWG_TYPE_BLOCKSCALEACTION = _LibreDWG.DWG_TYPE_BLOCKSCALEACTION
DWG_TYPE_BLOCKSTRETCHACTION = _LibreDWG.DWG_TYPE_BLOCKSTRETCHACTION
DWG_TYPE_BLOCKUSERPARAMETER = _LibreDWG.DWG_TYPE_BLOCKUSERPARAMETER
DWG_TYPE_BLOCKVERTICALCONSTRAINTPARAMETER = _LibreDWG.DWG_TYPE_BLOCKVERTICALCONSTRAINTPARAMETER
DWG_TYPE_BLOCKVISIBILITYGRIP = _LibreDWG.DWG_TYPE_BLOCKVISIBILITYGRIP
DWG_TYPE_BLOCKVISIBILITYPARAMETER = _LibreDWG.DWG_TYPE_BLOCKVISIBILITYPARAMETER
DWG_TYPE_BLOCKXYGRIP = _LibreDWG.DWG_TYPE_BLOCKXYGRIP
DWG_TYPE_BLOCKXYPARAMETER = _LibreDWG.DWG_TYPE_BLOCKXYPARAMETER
DWG_TYPE_CAMERA = _LibreDWG.DWG_TYPE_CAMERA
DWG_TYPE_CELLSTYLEMAP = _LibreDWG.DWG_TYPE_CELLSTYLEMAP
DWG_TYPE_CONTEXTDATAMANAGER = _LibreDWG.DWG_TYPE_CONTEXTDATAMANAGER
DWG_TYPE_CSACDOCUMENTOPTIONS = _LibreDWG.DWG_TYPE_CSACDOCUMENTOPTIONS
DWG_TYPE_CURVEPATH = _LibreDWG.DWG_TYPE_CURVEPATH
DWG_TYPE_DATALINK = _LibreDWG.DWG_TYPE_DATALINK
DWG_TYPE_DATATABLE = _LibreDWG.DWG_TYPE_DATATABLE
DWG_TYPE_DBCOLOR = _LibreDWG.DWG_TYPE_DBCOLOR
DWG_TYPE_DETAILVIEWSTYLE = _LibreDWG.DWG_TYPE_DETAILVIEWSTYLE
DWG_TYPE_DGNDEFINITION = _LibreDWG.DWG_TYPE_DGNDEFINITION
DWG_TYPE_DGNUNDERLAY = _LibreDWG.DWG_TYPE_DGNUNDERLAY
DWG_TYPE_DICTIONARYVAR = _LibreDWG.DWG_TYPE_DICTIONARYVAR
DWG_TYPE_DICTIONARYWDFLT = _LibreDWG.DWG_TYPE_DICTIONARYWDFLT
DWG_TYPE_DIMASSOC = _LibreDWG.DWG_TYPE_DIMASSOC
DWG_TYPE_DMDIMOBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_DMDIMOBJECTCONTEXTDATA
DWG_TYPE_DWFDEFINITION = _LibreDWG.DWG_TYPE_DWFDEFINITION
DWG_TYPE_DWFUNDERLAY = _LibreDWG.DWG_TYPE_DWFUNDERLAY
DWG_TYPE_DYNAMICBLOCKPROXYNODE = _LibreDWG.DWG_TYPE_DYNAMICBLOCKPROXYNODE
DWG_TYPE_DYNAMICBLOCKPURGEPREVENTER = _LibreDWG.DWG_TYPE_DYNAMICBLOCKPURGEPREVENTER
DWG_TYPE_EVALUATION_GRAPH = _LibreDWG.DWG_TYPE_EVALUATION_GRAPH
DWG_TYPE_EXTRUDEDSURFACE = _LibreDWG.DWG_TYPE_EXTRUDEDSURFACE
DWG_TYPE_FCFOBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_FCFOBJECTCONTEXTDATA
DWG_TYPE_FIELD = _LibreDWG.DWG_TYPE_FIELD
DWG_TYPE_FIELDLIST = _LibreDWG.DWG_TYPE_FIELDLIST
DWG_TYPE_FLIPPARAMETERENTITY = _LibreDWG.DWG_TYPE_FLIPPARAMETERENTITY
DWG_TYPE_FLIPACTIONENTITY = _LibreDWG.DWG_TYPE_FLIPACTIONENTITY
DWG_TYPE_GEODATA = _LibreDWG.DWG_TYPE_GEODATA
DWG_TYPE_GEOMAPIMAGE = _LibreDWG.DWG_TYPE_GEOMAPIMAGE
DWG_TYPE_GEOPOSITIONMARKER = _LibreDWG.DWG_TYPE_GEOPOSITIONMARKER
DWG_TYPE_GRADIENT_BACKGROUND = _LibreDWG.DWG_TYPE_GRADIENT_BACKGROUND
DWG_TYPE_GROUND_PLANE_BACKGROUND = _LibreDWG.DWG_TYPE_GROUND_PLANE_BACKGROUND
DWG_TYPE_HELIX = _LibreDWG.DWG_TYPE_HELIX
DWG_TYPE_IBL_BACKGROUND = _LibreDWG.DWG_TYPE_IBL_BACKGROUND
DWG_TYPE_IDBUFFER = _LibreDWG.DWG_TYPE_IDBUFFER
DWG_TYPE_IMAGE = _LibreDWG.DWG_TYPE_IMAGE
DWG_TYPE_IMAGEDEF = _LibreDWG.DWG_TYPE_IMAGEDEF
DWG_TYPE_IMAGEDEF_REACTOR = _LibreDWG.DWG_TYPE_IMAGEDEF_REACTOR
DWG_TYPE_IMAGE_BACKGROUND = _LibreDWG.DWG_TYPE_IMAGE_BACKGROUND
DWG_TYPE_INDEX = _LibreDWG.DWG_TYPE_INDEX
DWG_TYPE_LARGE_RADIAL_DIMENSION = _LibreDWG.DWG_TYPE_LARGE_RADIAL_DIMENSION
DWG_TYPE_LAYERFILTER = _LibreDWG.DWG_TYPE_LAYERFILTER
DWG_TYPE_LAYER_INDEX = _LibreDWG.DWG_TYPE_LAYER_INDEX
DWG_TYPE_LAYOUTPRINTCONFIG = _LibreDWG.DWG_TYPE_LAYOUTPRINTCONFIG
DWG_TYPE_LEADEROBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_LEADEROBJECTCONTEXTDATA
DWG_TYPE_LIGHT = _LibreDWG.DWG_TYPE_LIGHT
DWG_TYPE_LIGHTLIST = _LibreDWG.DWG_TYPE_LIGHTLIST
DWG_TYPE_LINEARPARAMETERENTITY = _LibreDWG.DWG_TYPE_LINEARPARAMETERENTITY
DWG_TYPE_LOFTEDSURFACE = _LibreDWG.DWG_TYPE_LOFTEDSURFACE
DWG_TYPE_MATERIAL = _LibreDWG.DWG_TYPE_MATERIAL
DWG_TYPE_MENTALRAYRENDERSETTINGS = _LibreDWG.DWG_TYPE_MENTALRAYRENDERSETTINGS
DWG_TYPE_MESH = _LibreDWG.DWG_TYPE_MESH
DWG_TYPE_MLEADEROBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_MLEADEROBJECTCONTEXTDATA
DWG_TYPE_MLEADERSTYLE = _LibreDWG.DWG_TYPE_MLEADERSTYLE
DWG_TYPE_MOVEACTIONENTITY = _LibreDWG.DWG_TYPE_MOVEACTIONENTITY
DWG_TYPE_MOTIONPATH = _LibreDWG.DWG_TYPE_MOTIONPATH
DWG_TYPE_MPOLYGON = _LibreDWG.DWG_TYPE_MPOLYGON
DWG_TYPE_MTEXTATTRIBUTEOBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_MTEXTATTRIBUTEOBJECTCONTEXTDATA
DWG_TYPE_MTEXTOBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_MTEXTOBJECTCONTEXTDATA
DWG_TYPE_MULTILEADER = _LibreDWG.DWG_TYPE_MULTILEADER
DWG_TYPE_NAVISWORKSMODEL = _LibreDWG.DWG_TYPE_NAVISWORKSMODEL
DWG_TYPE_NAVISWORKSMODELDEF = _LibreDWG.DWG_TYPE_NAVISWORKSMODELDEF
DWG_TYPE_NPOCOLLECTION = _LibreDWG.DWG_TYPE_NPOCOLLECTION
DWG_TYPE_NURBSURFACE = _LibreDWG.DWG_TYPE_NURBSURFACE
DWG_TYPE_OBJECT_PTR = _LibreDWG.DWG_TYPE_OBJECT_PTR
DWG_TYPE_ORDDIMOBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_ORDDIMOBJECTCONTEXTDATA
DWG_TYPE_PARTIAL_VIEWING_INDEX = _LibreDWG.DWG_TYPE_PARTIAL_VIEWING_INDEX
DWG_TYPE_PDFDEFINITION = _LibreDWG.DWG_TYPE_PDFDEFINITION
DWG_TYPE_PDFUNDERLAY = _LibreDWG.DWG_TYPE_PDFUNDERLAY
DWG_TYPE_PERSUBENTMGR = _LibreDWG.DWG_TYPE_PERSUBENTMGR
DWG_TYPE_PLANESURFACE = _LibreDWG.DWG_TYPE_PLANESURFACE
DWG_TYPE_PLOTSETTINGS = _LibreDWG.DWG_TYPE_PLOTSETTINGS
DWG_TYPE_POINTCLOUD = _LibreDWG.DWG_TYPE_POINTCLOUD
DWG_TYPE_POINTCLOUDCOLORMAP = _LibreDWG.DWG_TYPE_POINTCLOUDCOLORMAP
DWG_TYPE_POINTCLOUDDEF = _LibreDWG.DWG_TYPE_POINTCLOUDDEF
DWG_TYPE_POINTCLOUDDEFEX = _LibreDWG.DWG_TYPE_POINTCLOUDDEFEX
DWG_TYPE_POINTCLOUDDEF_REACTOR = _LibreDWG.DWG_TYPE_POINTCLOUDDEF_REACTOR
DWG_TYPE_POINTCLOUDDEF_REACTOR_EX = _LibreDWG.DWG_TYPE_POINTCLOUDDEF_REACTOR_EX
DWG_TYPE_POINTCLOUDEX = _LibreDWG.DWG_TYPE_POINTCLOUDEX
DWG_TYPE_POINTPARAMETERENTITY = _LibreDWG.DWG_TYPE_POINTPARAMETERENTITY
DWG_TYPE_POINTPATH = _LibreDWG.DWG_TYPE_POINTPATH
DWG_TYPE_POLARGRIPENTITY = _LibreDWG.DWG_TYPE_POLARGRIPENTITY
DWG_TYPE_RADIMLGOBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_RADIMLGOBJECTCONTEXTDATA
DWG_TYPE_RADIMOBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_RADIMOBJECTCONTEXTDATA
DWG_TYPE_RAPIDRTRENDERSETTINGS = _LibreDWG.DWG_TYPE_RAPIDRTRENDERSETTINGS
DWG_TYPE_RASTERVARIABLES = _LibreDWG.DWG_TYPE_RASTERVARIABLES
DWG_TYPE_RENDERENTRY = _LibreDWG.DWG_TYPE_RENDERENTRY
DWG_TYPE_RENDERENVIRONMENT = _LibreDWG.DWG_TYPE_RENDERENVIRONMENT
DWG_TYPE_RENDERGLOBAL = _LibreDWG.DWG_TYPE_RENDERGLOBAL
DWG_TYPE_RENDERSETTINGS = _LibreDWG.DWG_TYPE_RENDERSETTINGS
DWG_TYPE_REVOLVEDSURFACE = _LibreDWG.DWG_TYPE_REVOLVEDSURFACE
DWG_TYPE_ROTATIONPARAMETERENTITY = _LibreDWG.DWG_TYPE_ROTATIONPARAMETERENTITY
DWG_TYPE_ROTATEACTIONENTITY = _LibreDWG.DWG_TYPE_ROTATEACTIONENTITY
DWG_TYPE_RTEXT = _LibreDWG.DWG_TYPE_RTEXT
DWG_TYPE_SCALE = _LibreDWG.DWG_TYPE_SCALE
DWG_TYPE_SCALEACTIONENTITY = _LibreDWG.DWG_TYPE_SCALEACTIONENTITY
DWG_TYPE_SECTIONOBJECT = _LibreDWG.DWG_TYPE_SECTIONOBJECT
DWG_TYPE_SECTIONVIEWSTYLE = _LibreDWG.DWG_TYPE_SECTIONVIEWSTYLE
DWG_TYPE_SECTION_MANAGER = _LibreDWG.DWG_TYPE_SECTION_MANAGER
DWG_TYPE_SECTION_SETTINGS = _LibreDWG.DWG_TYPE_SECTION_SETTINGS
DWG_TYPE_SKYLIGHT_BACKGROUND = _LibreDWG.DWG_TYPE_SKYLIGHT_BACKGROUND
DWG_TYPE_SOLID_BACKGROUND = _LibreDWG.DWG_TYPE_SOLID_BACKGROUND
DWG_TYPE_SORTENTSTABLE = _LibreDWG.DWG_TYPE_SORTENTSTABLE
DWG_TYPE_SPATIAL_FILTER = _LibreDWG.DWG_TYPE_SPATIAL_FILTER
DWG_TYPE_SPATIAL_INDEX = _LibreDWG.DWG_TYPE_SPATIAL_INDEX
DWG_TYPE_STRETCHACTIONENTITY = _LibreDWG.DWG_TYPE_STRETCHACTIONENTITY
DWG_TYPE_SUN = _LibreDWG.DWG_TYPE_SUN
DWG_TYPE_SUNSTUDY = _LibreDWG.DWG_TYPE_SUNSTUDY
DWG_TYPE_SWEPTSURFACE = _LibreDWG.DWG_TYPE_SWEPTSURFACE
DWG_TYPE_TABLE = _LibreDWG.DWG_TYPE_TABLE
DWG_TYPE_TABLECONTENT = _LibreDWG.DWG_TYPE_TABLECONTENT
DWG_TYPE_TABLEGEOMETRY = _LibreDWG.DWG_TYPE_TABLEGEOMETRY
DWG_TYPE_TABLESTYLE = _LibreDWG.DWG_TYPE_TABLESTYLE
DWG_TYPE_TEXTOBJECTCONTEXTDATA = _LibreDWG.DWG_TYPE_TEXTOBJECTCONTEXTDATA
DWG_TYPE_TVDEVICEPROPERTIES = _LibreDWG.DWG_TYPE_TVDEVICEPROPERTIES
DWG_TYPE_VISIBILITYGRIPENTITY = _LibreDWG.DWG_TYPE_VISIBILITYGRIPENTITY
DWG_TYPE_VISIBILITYPARAMETERENTITY = _LibreDWG.DWG_TYPE_VISIBILITYPARAMETERENTITY
DWG_TYPE_VISUALSTYLE = _LibreDWG.DWG_TYPE_VISUALSTYLE
DWG_TYPE_WIPEOUT = _LibreDWG.DWG_TYPE_WIPEOUT
DWG_TYPE_WIPEOUTVARIABLES = _LibreDWG.DWG_TYPE_WIPEOUTVARIABLES
DWG_TYPE_XREFPANELOBJECT = _LibreDWG.DWG_TYPE_XREFPANELOBJECT
DWG_TYPE_XYPARAMETERENTITY = _LibreDWG.DWG_TYPE_XYPARAMETERENTITY
DWG_TYPE_BREAKDATA = _LibreDWG.DWG_TYPE_BREAKDATA
DWG_TYPE_BREAKPOINTREF = _LibreDWG.DWG_TYPE_BREAKPOINTREF
DWG_TYPE_FLIPGRIPENTITY = _LibreDWG.DWG_TYPE_FLIPGRIPENTITY
DWG_TYPE_LINEARGRIPENTITY = _LibreDWG.DWG_TYPE_LINEARGRIPENTITY
DWG_TYPE_ROTATIONGRIPENTITY = _LibreDWG.DWG_TYPE_ROTATIONGRIPENTITY
DWG_TYPE_XYGRIPENTITY = _LibreDWG.DWG_TYPE_XYGRIPENTITY
DWG_TYPE__3DLINE = _LibreDWG.DWG_TYPE__3DLINE
DWG_TYPE_REPEAT = _LibreDWG.DWG_TYPE_REPEAT
DWG_TYPE_ENDREP = _LibreDWG.DWG_TYPE_ENDREP
DWG_TYPE_JUMP = _LibreDWG.DWG_TYPE_JUMP
DWG_TYPE_LOAD = _LibreDWG.DWG_TYPE_LOAD
DWG_TYPE_ABSHDRAWINGSETTINGS = _LibreDWG.DWG_TYPE_ABSHDRAWINGSETTINGS
DWG_TYPE_ACAECUSTOBJ = _LibreDWG.DWG_TYPE_ACAECUSTOBJ
DWG_TYPE_ACAEEEMGROBJ = _LibreDWG.DWG_TYPE_ACAEEEMGROBJ
DWG_TYPE_ACAMCOMP = _LibreDWG.DWG_TYPE_ACAMCOMP
DWG_TYPE_ACAMCOMPDEF = _LibreDWG.DWG_TYPE_ACAMCOMPDEF
DWG_TYPE_ACAMCOMPDEFMGR = _LibreDWG.DWG_TYPE_ACAMCOMPDEFMGR
DWG_TYPE_ACAMCONTEXTMODELER = _LibreDWG.DWG_TYPE_ACAMCONTEXTMODELER
DWG_TYPE_ACAMGDIMSTD = _LibreDWG.DWG_TYPE_ACAMGDIMSTD
DWG_TYPE_ACAMGFILTERDAT = _LibreDWG.DWG_TYPE_ACAMGFILTERDAT
DWG_TYPE_ACAMGHOLECHARTSTDCSN = _LibreDWG.DWG_TYPE_ACAMGHOLECHARTSTDCSN
DWG_TYPE_ACAMGHOLECHARTSTDDIN = _LibreDWG.DWG_TYPE_ACAMGHOLECHARTSTDDIN
DWG_TYPE_ACAMGHOLECHARTSTDISO = _LibreDWG.DWG_TYPE_ACAMGHOLECHARTSTDISO
DWG_TYPE_ACAMGLAYSTD = _LibreDWG.DWG_TYPE_ACAMGLAYSTD
DWG_TYPE_ACAMGRCOMPDEF = _LibreDWG.DWG_TYPE_ACAMGRCOMPDEF
DWG_TYPE_ACAMGRCOMPDEFSET = _LibreDWG.DWG_TYPE_ACAMGRCOMPDEFSET
DWG_TYPE_ACAMGTITLESTD = _LibreDWG.DWG_TYPE_ACAMGTITLESTD
DWG_TYPE_ACAMMVDBACKUPOBJECT = _LibreDWG.DWG_TYPE_ACAMMVDBACKUPOBJECT
DWG_TYPE_ACAMPROJECT = _LibreDWG.DWG_TYPE_ACAMPROJECT
DWG_TYPE_ACAMSHAFTCOMPDEF = _LibreDWG.DWG_TYPE_ACAMSHAFTCOMPDEF
DWG_TYPE_ACAMSTDPCOMPDEF = _LibreDWG.DWG_TYPE_ACAMSTDPCOMPDEF
DWG_TYPE_ACAMWBLOCKTEMPENTS = _LibreDWG.DWG_TYPE_ACAMWBLOCKTEMPENTS
DWG_TYPE_ACARRAYJIGENTITY = _LibreDWG.DWG_TYPE_ACARRAYJIGENTITY
DWG_TYPE_ACCMCONTEXT = _LibreDWG.DWG_TYPE_ACCMCONTEXT
DWG_TYPE_ACDBCIRCARCRES = _LibreDWG.DWG_TYPE_ACDBCIRCARCRES
DWG_TYPE_ACDBDIMENSIONRES = _LibreDWG.DWG_TYPE_ACDBDIMENSIONRES
DWG_TYPE_ACDBENTITYCACHE = _LibreDWG.DWG_TYPE_ACDBENTITYCACHE
DWG_TYPE_ACDBLINERES = _LibreDWG.DWG_TYPE_ACDBLINERES
DWG_TYPE_ACDBSTDPARTRES_ARC = _LibreDWG.DWG_TYPE_ACDBSTDPARTRES_ARC
DWG_TYPE_ACDBSTDPARTRES_LINE = _LibreDWG.DWG_TYPE_ACDBSTDPARTRES_LINE
DWG_TYPE_ACDB_HATCHSCALECONTEXTDATA_CLASS = _LibreDWG.DWG_TYPE_ACDB_HATCHSCALECONTEXTDATA_CLASS
DWG_TYPE_ACDB_HATCHVIEWCONTEXTDATA_CLASS = _LibreDWG.DWG_TYPE_ACDB_HATCHVIEWCONTEXTDATA_CLASS
DWG_TYPE_ACDB_PROXY_ENTITY_DATA = _LibreDWG.DWG_TYPE_ACDB_PROXY_ENTITY_DATA
DWG_TYPE_ACGREFACADMASTER = _LibreDWG.DWG_TYPE_ACGREFACADMASTER
DWG_TYPE_ACGREFMASTER = _LibreDWG.DWG_TYPE_ACGREFMASTER
DWG_TYPE_ACIMINTSYSVAR = _LibreDWG.DWG_TYPE_ACIMINTSYSVAR
DWG_TYPE_ACIMREALSYSVAR = _LibreDWG.DWG_TYPE_ACIMREALSYSVAR
DWG_TYPE_ACIMSTRSYSVAR = _LibreDWG.DWG_TYPE_ACIMSTRSYSVAR
DWG_TYPE_ACIMSYSVARMAN = _LibreDWG.DWG_TYPE_ACIMSYSVARMAN
DWG_TYPE_ACMANOOTATIONVIEWSTANDARDANSI = _LibreDWG.DWG_TYPE_ACMANOOTATIONVIEWSTANDARDANSI
DWG_TYPE_ACMANOOTATIONVIEWSTANDARDCSN = _LibreDWG.DWG_TYPE_ACMANOOTATIONVIEWSTANDARDCSN
DWG_TYPE_ACMANOOTATIONVIEWSTANDARDDIN = _LibreDWG.DWG_TYPE_ACMANOOTATIONVIEWSTANDARDDIN
DWG_TYPE_ACMANOOTATIONVIEWSTANDARDISO = _LibreDWG.DWG_TYPE_ACMANOOTATIONVIEWSTANDARDISO
DWG_TYPE_ACMAPLEGENDDBOBJECT = _LibreDWG.DWG_TYPE_ACMAPLEGENDDBOBJECT
DWG_TYPE_ACMAPLEGENDITEMDBOBJECT = _LibreDWG.DWG_TYPE_ACMAPLEGENDITEMDBOBJECT
DWG_TYPE_ACMAPMAPVIEWPORTDBOBJECT = _LibreDWG.DWG_TYPE_ACMAPMAPVIEWPORTDBOBJECT
DWG_TYPE_ACMAPPRINTLAYOUTELEMENTDBOBJECTCONTAINER = _LibreDWG.DWG_TYPE_ACMAPPRINTLAYOUTELEMENTDBOBJECTCONTAINER
DWG_TYPE_ACMBALLOON = _LibreDWG.DWG_TYPE_ACMBALLOON
DWG_TYPE_ACMBOM = _LibreDWG.DWG_TYPE_ACMBOM
DWG_TYPE_ACMBOMROW = _LibreDWG.DWG_TYPE_ACMBOMROW
DWG_TYPE_ACMBOMROWSTRUCT = _LibreDWG.DWG_TYPE_ACMBOMROWSTRUCT
DWG_TYPE_ACMBOMSTANDARDANSI = _LibreDWG.DWG_TYPE_ACMBOMSTANDARDANSI
DWG_TYPE_ACMBOMSTANDARDCSN = _LibreDWG.DWG_TYPE_ACMBOMSTANDARDCSN
DWG_TYPE_ACMBOMSTANDARDDIN = _LibreDWG.DWG_TYPE_ACMBOMSTANDARDDIN
DWG_TYPE_ACMBOMSTANDARDISO = _LibreDWG.DWG_TYPE_ACMBOMSTANDARDISO
DWG_TYPE_ACMCENTERLINESTANDARDANSI = _LibreDWG.DWG_TYPE_ACMCENTERLINESTANDARDANSI
DWG_TYPE_ACMCENTERLINESTANDARDCSN = _LibreDWG.DWG_TYPE_ACMCENTERLINESTANDARDCSN
DWG_TYPE_ACMCENTERLINESTANDARDDIN = _LibreDWG.DWG_TYPE_ACMCENTERLINESTANDARDDIN
DWG_TYPE_ACMCENTERLINESTANDARDISO = _LibreDWG.DWG_TYPE_ACMCENTERLINESTANDARDISO
DWG_TYPE_ACMDATADICTIONARY = _LibreDWG.DWG_TYPE_ACMDATADICTIONARY
DWG_TYPE_ACMDATAENTRY = _LibreDWG.DWG_TYPE_ACMDATAENTRY
DWG_TYPE_ACMDATAENTRYBLOCK = _LibreDWG.DWG_TYPE_ACMDATAENTRYBLOCK
DWG_TYPE_ACMDATUMID = _LibreDWG.DWG_TYPE_ACMDATUMID
DWG_TYPE_ACMDATUMSTANDARDANSI = _LibreDWG.DWG_TYPE_ACMDATUMSTANDARDANSI
DWG_TYPE_ACMDATUMSTANDARDCSN = _LibreDWG.DWG_TYPE_ACMDATUMSTANDARDCSN
DWG_TYPE_ACMDATUMSTANDARDDIN = _LibreDWG.DWG_TYPE_ACMDATUMSTANDARDDIN
DWG_TYPE_ACMDATUMSTANDARDISO = _LibreDWG.DWG_TYPE_ACMDATUMSTANDARDISO
DWG_TYPE_ACMDATUMSTANDARDISO2012 = _LibreDWG.DWG_TYPE_ACMDATUMSTANDARDISO2012
DWG_TYPE_ACMDETAILSTANDARDANSI = _LibreDWG.DWG_TYPE_ACMDETAILSTANDARDANSI
DWG_TYPE_ACMDETAILSTANDARDCSN = _LibreDWG.DWG_TYPE_ACMDETAILSTANDARDCSN
DWG_TYPE_ACMDETAILSTANDARDDIN = _LibreDWG.DWG_TYPE_ACMDETAILSTANDARDDIN
DWG_TYPE_ACMDETAILSTANDARDISO = _LibreDWG.DWG_TYPE_ACMDETAILSTANDARDISO
DWG_TYPE_ACMDETAILTANDARDCUSTOM = _LibreDWG.DWG_TYPE_ACMDETAILTANDARDCUSTOM
DWG_TYPE_ACMDIMBREAKPERSREACTOR = _LibreDWG.DWG_TYPE_ACMDIMBREAKPERSREACTOR
DWG_TYPE_ACMEDRAWINGMAN = _LibreDWG.DWG_TYPE_ACMEDRAWINGMAN
DWG_TYPE_ACMEVIEW = _LibreDWG.DWG_TYPE_ACMEVIEW
DWG_TYPE_ACME_DATABASE = _LibreDWG.DWG_TYPE_ACME_DATABASE
DWG_TYPE_ACME_DOCUMENT = _LibreDWG.DWG_TYPE_ACME_DOCUMENT
DWG_TYPE_ACMFCFRAME = _LibreDWG.DWG_TYPE_ACMFCFRAME
DWG_TYPE_ACMFCFSTANDARDANSI = _LibreDWG.DWG_TYPE_ACMFCFSTANDARDANSI
DWG_TYPE_ACMFCFSTANDARDCSN = _LibreDWG.DWG_TYPE_ACMFCFSTANDARDCSN
DWG_TYPE_ACMFCFSTANDARDDIN = _LibreDWG.DWG_TYPE_ACMFCFSTANDARDDIN
DWG_TYPE_ACMFCFSTANDARDISO = _LibreDWG.DWG_TYPE_ACMFCFSTANDARDISO
DWG_TYPE_ACMFCFSTANDARDISO2004 = _LibreDWG.DWG_TYPE_ACMFCFSTANDARDISO2004
DWG_TYPE_ACMFCFSTANDARDISO2012 = _LibreDWG.DWG_TYPE_ACMFCFSTANDARDISO2012
DWG_TYPE_ACMIDSTANDARDANSI = _LibreDWG.DWG_TYPE_ACMIDSTANDARDANSI
DWG_TYPE_ACMIDSTANDARDCSN = _LibreDWG.DWG_TYPE_ACMIDSTANDARDCSN
DWG_TYPE_ACMIDSTANDARDDIN = _LibreDWG.DWG_TYPE_ACMIDSTANDARDDIN
DWG_TYPE_ACMIDSTANDARDISO = _LibreDWG.DWG_TYPE_ACMIDSTANDARDISO
DWG_TYPE_ACMIDSTANDARDISO2004 = _LibreDWG.DWG_TYPE_ACMIDSTANDARDISO2004
DWG_TYPE_ACMIDSTANDARDISO2012 = _LibreDWG.DWG_TYPE_ACMIDSTANDARDISO2012
DWG_TYPE_ACMNOTESTANDARDANSI = _LibreDWG.DWG_TYPE_ACMNOTESTANDARDANSI
DWG_TYPE_ACMNOTESTANDARDCSN = _LibreDWG.DWG_TYPE_ACMNOTESTANDARDCSN
DWG_TYPE_ACMNOTESTANDARDDIN = _LibreDWG.DWG_TYPE_ACMNOTESTANDARDDIN
DWG_TYPE_ACMNOTESTANDARDISO = _LibreDWG.DWG_TYPE_ACMNOTESTANDARDISO
DWG_TYPE_ACMPARTLIST = _LibreDWG.DWG_TYPE_ACMPARTLIST
DWG_TYPE_ACMPICKOBJ = _LibreDWG.DWG_TYPE_ACMPICKOBJ
DWG_TYPE_ACMSECTIONSTANDARDANSI = _LibreDWG.DWG_TYPE_ACMSECTIONSTANDARDANSI
DWG_TYPE_ACMSECTIONSTANDARDCSN2002 = _LibreDWG.DWG_TYPE_ACMSECTIONSTANDARDCSN2002
DWG_TYPE_ACMSECTIONSTANDARDCUSTOM = _LibreDWG.DWG_TYPE_ACMSECTIONSTANDARDCUSTOM
DWG_TYPE_ACMSECTIONSTANDARDDIN = _LibreDWG.DWG_TYPE_ACMSECTIONSTANDARDDIN
DWG_TYPE_ACMSECTIONSTANDARDISO = _LibreDWG.DWG_TYPE_ACMSECTIONSTANDARDISO
DWG_TYPE_ACMSECTIONSTANDARDISO2001 = _LibreDWG.DWG_TYPE_ACMSECTIONSTANDARDISO2001
DWG_TYPE_ACMSTANDARDANSI = _LibreDWG.DWG_TYPE_ACMSTANDARDANSI
DWG_TYPE_ACMSTANDARDCSN = _LibreDWG.DWG_TYPE_ACMSTANDARDCSN
DWG_TYPE_ACMSTANDARDDIN = _LibreDWG.DWG_TYPE_ACMSTANDARDDIN
DWG_TYPE_ACMSTANDARDISO = _LibreDWG.DWG_TYPE_ACMSTANDARDISO
DWG_TYPE_ACMSURFSTANDARDANSI = _LibreDWG.DWG_TYPE_ACMSURFSTANDARDANSI
DWG_TYPE_ACMSURFSTANDARDCSN = _LibreDWG.DWG_TYPE_ACMSURFSTANDARDCSN
DWG_TYPE_ACMSURFSTANDARDDIN = _LibreDWG.DWG_TYPE_ACMSURFSTANDARDDIN
DWG_TYPE_ACMSURFSTANDARDISO = _LibreDWG.DWG_TYPE_ACMSURFSTANDARDISO
DWG_TYPE_ACMSURFSTANDARDISO2002 = _LibreDWG.DWG_TYPE_ACMSURFSTANDARDISO2002
DWG_TYPE_ACMSURFSYM = _LibreDWG.DWG_TYPE_ACMSURFSYM
DWG_TYPE_ACMTAPERSTANDARDANSI = _LibreDWG.DWG_TYPE_ACMTAPERSTANDARDANSI
DWG_TYPE_ACMTAPERSTANDARDCSN = _LibreDWG.DWG_TYPE_ACMTAPERSTANDARDCSN
DWG_TYPE_ACMTAPERSTANDARDDIN = _LibreDWG.DWG_TYPE_ACMTAPERSTANDARDDIN
DWG_TYPE_ACMTAPERSTANDARDISO = _LibreDWG.DWG_TYPE_ACMTAPERSTANDARDISO
DWG_TYPE_ACMTHREADLINESTANDARDANSI = _LibreDWG.DWG_TYPE_ACMTHREADLINESTANDARDANSI
DWG_TYPE_ACMTHREADLINESTANDARDCSN = _LibreDWG.DWG_TYPE_ACMTHREADLINESTANDARDCSN
DWG_TYPE_ACMTHREADLINESTANDARDDIN = _LibreDWG.DWG_TYPE_ACMTHREADLINESTANDARDDIN
DWG_TYPE_ACMTHREADLINESTANDARDISO = _LibreDWG.DWG_TYPE_ACMTHREADLINESTANDARDISO
DWG_TYPE_ACMWELDSTANDARDANSI = _LibreDWG.DWG_TYPE_ACMWELDSTANDARDANSI
DWG_TYPE_ACMWELDSTANDARDCSN = _LibreDWG.DWG_TYPE_ACMWELDSTANDARDCSN
DWG_TYPE_ACMWELDSTANDARDDIN = _LibreDWG.DWG_TYPE_ACMWELDSTANDARDDIN
DWG_TYPE_ACMWELDSTANDARDISO = _LibreDWG.DWG_TYPE_ACMWELDSTANDARDISO
DWG_TYPE_ACMWELDSYM = _LibreDWG.DWG_TYPE_ACMWELDSYM
DWG_TYPE_ACRFATTGENMGR = _LibreDWG.DWG_TYPE_ACRFATTGENMGR
DWG_TYPE_ACRFINSADJ = _LibreDWG.DWG_TYPE_ACRFINSADJ
DWG_TYPE_ACRFINSADJUSTERMGR = _LibreDWG.DWG_TYPE_ACRFINSADJUSTERMGR
DWG_TYPE_ACRFMCADAPIATTHOLDER = _LibreDWG.DWG_TYPE_ACRFMCADAPIATTHOLDER
DWG_TYPE_ACRFOBJATTMGR = _LibreDWG.DWG_TYPE_ACRFOBJATTMGR
DWG_TYPE_ACSH_SUBENT_MATERIAL_CLASS = _LibreDWG.DWG_TYPE_ACSH_SUBENT_MATERIAL_CLASS
DWG_TYPE_AC_AM_2D_XREF_MGR = _LibreDWG.DWG_TYPE_AC_AM_2D_XREF_MGR
DWG_TYPE_AC_AM_BASIC_VIEW = _LibreDWG.DWG_TYPE_AC_AM_BASIC_VIEW
DWG_TYPE_AC_AM_BASIC_VIEW_DEF = _LibreDWG.DWG_TYPE_AC_AM_BASIC_VIEW_DEF
DWG_TYPE_AC_AM_COMPLEX_HIDE_SITUATION = _LibreDWG.DWG_TYPE_AC_AM_COMPLEX_HIDE_SITUATION
DWG_TYPE_AC_AM_COMP_VIEW_DEF = _LibreDWG.DWG_TYPE_AC_AM_COMP_VIEW_DEF
DWG_TYPE_AC_AM_COMP_VIEW_INST = _LibreDWG.DWG_TYPE_AC_AM_COMP_VIEW_INST
DWG_TYPE_AC_AM_DIRTY_NODES = _LibreDWG.DWG_TYPE_AC_AM_DIRTY_NODES
DWG_TYPE_AC_AM_HIDE_SITUATION = _LibreDWG.DWG_TYPE_AC_AM_HIDE_SITUATION
DWG_TYPE_AC_AM_MAPPER_CACHE = _LibreDWG.DWG_TYPE_AC_AM_MAPPER_CACHE
DWG_TYPE_AC_AM_MASTER_VIEW_DEF = _LibreDWG.DWG_TYPE_AC_AM_MASTER_VIEW_DEF
DWG_TYPE_AC_AM_MVD_DEP_MGR = _LibreDWG.DWG_TYPE_AC_AM_MVD_DEP_MGR
DWG_TYPE_AC_AM_OVERRIDE_FILTER = _LibreDWG.DWG_TYPE_AC_AM_OVERRIDE_FILTER
DWG_TYPE_AC_AM_PROPS_OVERRIDE = _LibreDWG.DWG_TYPE_AC_AM_PROPS_OVERRIDE
DWG_TYPE_AC_AM_SHAFT_HIDE_SITUATION = _LibreDWG.DWG_TYPE_AC_AM_SHAFT_HIDE_SITUATION
DWG_TYPE_AC_AM_STDP_VIEW_DEF = _LibreDWG.DWG_TYPE_AC_AM_STDP_VIEW_DEF
DWG_TYPE_AC_AM_TRANSFORM_GHOST = _LibreDWG.DWG_TYPE_AC_AM_TRANSFORM_GHOST
DWG_TYPE_ADAPPL = _LibreDWG.DWG_TYPE_ADAPPL
DWG_TYPE_AECC_ALIGNMENT_DESIGN_CHECK_SET = _LibreDWG.DWG_TYPE_AECC_ALIGNMENT_DESIGN_CHECK_SET
DWG_TYPE_AECC_ALIGNMENT_LABEL_SET = _LibreDWG.DWG_TYPE_AECC_ALIGNMENT_LABEL_SET
DWG_TYPE_AECC_ALIGNMENT_LABEL_SET_EXT = _LibreDWG.DWG_TYPE_AECC_ALIGNMENT_LABEL_SET_EXT
DWG_TYPE_AECC_ALIGNMENT_PARCEL_NODE = _LibreDWG.DWG_TYPE_AECC_ALIGNMENT_PARCEL_NODE
DWG_TYPE_AECC_ALIGNMENT_STYLE = _LibreDWG.DWG_TYPE_AECC_ALIGNMENT_STYLE
DWG_TYPE_AECC_APPURTENANCE_STYLE = _LibreDWG.DWG_TYPE_AECC_APPURTENANCE_STYLE
DWG_TYPE_AECC_ASSEMBLY_STYLE = _LibreDWG.DWG_TYPE_AECC_ASSEMBLY_STYLE
DWG_TYPE_AECC_BUILDING_SITE_STYLE = _LibreDWG.DWG_TYPE_AECC_BUILDING_SITE_STYLE
DWG_TYPE_AECC_CANT_DIAGRAM_VIEW_STYLE = _LibreDWG.DWG_TYPE_AECC_CANT_DIAGRAM_VIEW_STYLE
DWG_TYPE_AECC_CATCHMENT_STYLE = _LibreDWG.DWG_TYPE_AECC_CATCHMENT_STYLE
DWG_TYPE_AECC_CLASS_NODE = _LibreDWG.DWG_TYPE_AECC_CLASS_NODE
DWG_TYPE_AECC_CONTOURVIEW = _LibreDWG.DWG_TYPE_AECC_CONTOURVIEW
DWG_TYPE_AECC_CORRIDOR_STYLE = _LibreDWG.DWG_TYPE_AECC_CORRIDOR_STYLE
DWG_TYPE_AECC_DISP_REP_ALIGNMENT = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CANT_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CANT_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CSV = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CSV
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CURVE_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CURVE_LABEL
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_DESIGNSPEED_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_DESIGNSPEED_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_GEOMPT_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_GEOMPT_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_INDEXED_PI_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_INDEXED_PI_LABEL
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_MINOR_STATION_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_MINOR_STATION_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_PI_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_PI_LABEL
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_SPIRAL_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_SPIRAL_LABEL
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STAEQU_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STAEQU_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STATION_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STATION_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STATION_OFFSET_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STATION_OFFSET_LABEL
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_SUPERELEVATION_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_SUPERELEVATION_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_TABLE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_TABLE
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_TANGENT_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_TANGENT_LABEL
DWG_TYPE_AECC_DISP_REP_ALIGNMENT_VERTICAL_GEOMPT_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ALIGNMENT_VERTICAL_GEOMPT_LABELING
DWG_TYPE_AECC_DISP_REP_APPURTENANCE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_APPURTENANCE
DWG_TYPE_AECC_DISP_REP_APPURTENANCE_CSV = _LibreDWG.DWG_TYPE_AECC_DISP_REP_APPURTENANCE_CSV
DWG_TYPE_AECC_DISP_REP_APPURTENANCE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_APPURTENANCE_LABELING
DWG_TYPE_AECC_DISP_REP_APPURTENANCE_PROFILE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_APPURTENANCE_PROFILE_LABELING
DWG_TYPE_AECC_DISP_REP_ASSEMBLY = _LibreDWG.DWG_TYPE_AECC_DISP_REP_ASSEMBLY
DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE
DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_PROFILE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_PROFILE
DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_SECTION = _LibreDWG.DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_SECTION
DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE
DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_CSV = _LibreDWG.DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_CSV
DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_PROFILE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_PROFILE
DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_SECTION = _LibreDWG.DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_SECTION
DWG_TYPE_AECC_DISP_REP_BUILDINGSITE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_BUILDINGSITE
DWG_TYPE_AECC_DISP_REP_BUILDINGUTIL_CONNECTOR = _LibreDWG.DWG_TYPE_AECC_DISP_REP_BUILDINGUTIL_CONNECTOR
DWG_TYPE_AECC_DISP_REP_CANT_DIAGRAM_VIEW = _LibreDWG.DWG_TYPE_AECC_DISP_REP_CANT_DIAGRAM_VIEW
DWG_TYPE_AECC_DISP_REP_CATCHMENT_AREA = _LibreDWG.DWG_TYPE_AECC_DISP_REP_CATCHMENT_AREA
DWG_TYPE_AECC_DISP_REP_CATCHMENT_AREA_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_CATCHMENT_AREA_LABEL
DWG_TYPE_AECC_DISP_REP_CORRIDOR = _LibreDWG.DWG_TYPE_AECC_DISP_REP_CORRIDOR
DWG_TYPE_AECC_DISP_REP_CROSSING_PIPE_PROFILE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_CROSSING_PIPE_PROFILE_LABELING
DWG_TYPE_AECC_DISP_REP_CROSSING_PRESSURE_PIPE_PROFILE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_CROSSING_PRESSURE_PIPE_PROFILE_LABELING
DWG_TYPE_AECC_DISP_REP_CSVSTATIONSLIDER = _LibreDWG.DWG_TYPE_AECC_DISP_REP_CSVSTATIONSLIDER
DWG_TYPE_AECC_DISP_REP_FACE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_FACE
DWG_TYPE_AECC_DISP_REP_FEATURE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_FEATURE
DWG_TYPE_AECC_DISP_REP_FEATURE_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_FEATURE_LABEL
DWG_TYPE_AECC_DISP_REP_FEATURE_LINE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_FEATURE_LINE
DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_CSV = _LibreDWG.DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_CSV
DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_PROFILE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_PROFILE
DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_SECTION = _LibreDWG.DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_SECTION
DWG_TYPE_AECC_DISP_REP_FITTING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_FITTING
DWG_TYPE_AECC_DISP_REP_FITTING_CSV = _LibreDWG.DWG_TYPE_AECC_DISP_REP_FITTING_CSV
DWG_TYPE_AECC_DISP_REP_FITTING_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_FITTING_LABELING
DWG_TYPE_AECC_DISP_REP_FITTING_PROFILE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_FITTING_PROFILE_LABELING
DWG_TYPE_AECC_DISP_REP_FLOW_SEGMENT_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_FLOW_SEGMENT_LABEL
DWG_TYPE_AECC_DISP_REP_GENERAL_SEGMENT_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_GENERAL_SEGMENT_LABEL
DWG_TYPE_AECC_DISP_REP_GRADING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_GRADING
DWG_TYPE_AECC_DISP_REP_GRAPH = _LibreDWG.DWG_TYPE_AECC_DISP_REP_GRAPH
DWG_TYPE_AECC_DISP_REP_GRAPHPROFILE_NETWORKPART = _LibreDWG.DWG_TYPE_AECC_DISP_REP_GRAPHPROFILE_NETWORKPART
DWG_TYPE_AECC_DISP_REP_GRAPHPROFILE_PRESSUREPART = _LibreDWG.DWG_TYPE_AECC_DISP_REP_GRAPHPROFILE_PRESSUREPART
DWG_TYPE_AECC_DISP_REP_GRID_SURFACE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_GRID_SURFACE
DWG_TYPE_AECC_DISP_REP_GRID_SURFACE_CSV = _LibreDWG.DWG_TYPE_AECC_DISP_REP_GRID_SURFACE_CSV
DWG_TYPE_AECC_DISP_REP_HORGEOMETRY_BAND_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_HORGEOMETRY_BAND_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_HYDRO_REGION = _LibreDWG.DWG_TYPE_AECC_DISP_REP_HYDRO_REGION
DWG_TYPE_AECC_DISP_REP_INTERFERENCE_CHECK = _LibreDWG.DWG_TYPE_AECC_DISP_REP_INTERFERENCE_CHECK
DWG_TYPE_AECC_DISP_REP_INTERFERENCE_PART = _LibreDWG.DWG_TYPE_AECC_DISP_REP_INTERFERENCE_PART
DWG_TYPE_AECC_DISP_REP_INTERFERENCE_PART_SECTION = _LibreDWG.DWG_TYPE_AECC_DISP_REP_INTERFERENCE_PART_SECTION
DWG_TYPE_AECC_DISP_REP_INTERSECTION = _LibreDWG.DWG_TYPE_AECC_DISP_REP_INTERSECTION
DWG_TYPE_AECC_DISP_REP_INTERSECTION_LOCATION_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_INTERSECTION_LOCATION_LABELING
DWG_TYPE_AECC_DISP_REP_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_LABELING
DWG_TYPE_AECC_DISP_REP_LEGEND_TABLE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_LEGEND_TABLE
DWG_TYPE_AECC_DISP_REP_LINE_BETWEEN_POINTS_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_LINE_BETWEEN_POINTS_LABEL
DWG_TYPE_AECC_DISP_REP_LOTLINE_CSV = _LibreDWG.DWG_TYPE_AECC_DISP_REP_LOTLINE_CSV
DWG_TYPE_AECC_DISP_REP_MASSHAULLINE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_MASSHAULLINE
DWG_TYPE_AECC_DISP_REP_MASS_HAUL_VIEW = _LibreDWG.DWG_TYPE_AECC_DISP_REP_MASS_HAUL_VIEW
DWG_TYPE_AECC_DISP_REP_MATCHLINE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_MATCHLINE_LABELING
DWG_TYPE_AECC_DISP_REP_MATCH_LINE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_MATCH_LINE
DWG_TYPE_AECC_DISP_REP_MATERIAL_SECTION = _LibreDWG.DWG_TYPE_AECC_DISP_REP_MATERIAL_SECTION
DWG_TYPE_AECC_DISP_REP_NETWORK = _LibreDWG.DWG_TYPE_AECC_DISP_REP_NETWORK
DWG_TYPE_AECC_DISP_REP_NOTE_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_NOTE_LABEL
DWG_TYPE_AECC_DISP_REP_OFFSET_ELEV_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_OFFSET_ELEV_LABEL
DWG_TYPE_AECC_DISP_REP_PARCEL_BOUNDARY = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PARCEL_BOUNDARY
DWG_TYPE_AECC_DISP_REP_PARCEL_FACE_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PARCEL_FACE_LABEL
DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT
DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT_LABEL
DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT_TABLE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT_TABLE
DWG_TYPE_AECC_DISP_REP_PARCEL_TABLE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PARCEL_TABLE
DWG_TYPE_AECC_DISP_REP_PIPE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PIPE
DWG_TYPE_AECC_DISP_REP_PIPENETWORK_BAND_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PIPENETWORK_BAND_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_PIPE_CSV = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PIPE_CSV
DWG_TYPE_AECC_DISP_REP_PIPE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PIPE_LABELING
DWG_TYPE_AECC_DISP_REP_PIPE_PROFILE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PIPE_PROFILE_LABELING
DWG_TYPE_AECC_DISP_REP_PIPE_SECTION_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PIPE_SECTION_LABELING
DWG_TYPE_AECC_DISP_REP_PIPE_TABLE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PIPE_TABLE
DWG_TYPE_AECC_DISP_REP_POINT_ENT = _LibreDWG.DWG_TYPE_AECC_DISP_REP_POINT_ENT
DWG_TYPE_AECC_DISP_REP_POINT_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_POINT_GROUP
DWG_TYPE_AECC_DISP_REP_POINT_TABLE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_POINT_TABLE
DWG_TYPE_AECC_DISP_REP_PRESSUREPIPENETWORK = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PRESSUREPIPENETWORK
DWG_TYPE_AECC_DISP_REP_PRESSURE_PART_TABLE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PRESSURE_PART_TABLE
DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE
DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_CSV = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_CSV
DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_LABELING
DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_PROFILE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_PROFILE_LABELING
DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_SECTION_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_SECTION_LABELING
DWG_TYPE_AECC_DISP_REP_PROFILE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PROFILE
DWG_TYPE_AECC_DISP_REP_PROFILEDATA_BAND_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PROFILEDATA_BAND_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_PROFILE_PROJECTION = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PROFILE_PROJECTION
DWG_TYPE_AECC_DISP_REP_PROFILE_PROJECTION_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PROFILE_PROJECTION_LABEL
DWG_TYPE_AECC_DISP_REP_PROFILE_VIEW = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PROFILE_VIEW
DWG_TYPE_AECC_DISP_REP_PROFILE_VIEW_DEPTH_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_PROFILE_VIEW_DEPTH_LABEL
DWG_TYPE_AECC_DISP_REP_QUANTITY_TAKEOFF_AGGREGATE_EARTHWORK_TABLE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_QUANTITY_TAKEOFF_AGGREGATE_EARTHWORK_TABLE
DWG_TYPE_AECC_DISP_REP_RIGHT_OF_WAY = _LibreDWG.DWG_TYPE_AECC_DISP_REP_RIGHT_OF_WAY
DWG_TYPE_AECC_DISP_REP_SAMPLELINE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SAMPLELINE_LABELING
DWG_TYPE_AECC_DISP_REP_SAMPLE_LINE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SAMPLE_LINE
DWG_TYPE_AECC_DISP_REP_SAMPLE_LINE_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SAMPLE_LINE_GROUP
DWG_TYPE_AECC_DISP_REP_SECTION = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION
DWG_TYPE_AECC_DISP_REP_SECTIONALDATA_BAND_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTIONALDATA_BAND_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_SECTIONDATA_BAND_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTIONDATA_BAND_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_SECTIONSEGMENT_BAND_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTIONSEGMENT_BAND_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_SECTION_CORRIDOR = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION_CORRIDOR
DWG_TYPE_AECC_DISP_REP_SECTION_CORRIDOR_POINT_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION_CORRIDOR_POINT_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_SECTION_GRADEBREAK_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION_GRADEBREAK_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_SECTION_MINOR_OFFSET_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION_MINOR_OFFSET_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_SECTION_OFFSET_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION_OFFSET_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_SECTION_PIPENETWORK = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION_PIPENETWORK
DWG_TYPE_AECC_DISP_REP_SECTION_PRESSUREPIPENETWORK = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION_PRESSUREPIPENETWORK
DWG_TYPE_AECC_DISP_REP_SECTION_PROJECTION = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION_PROJECTION
DWG_TYPE_AECC_DISP_REP_SECTION_PROJECTION_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION_PROJECTION_LABEL
DWG_TYPE_AECC_DISP_REP_SECTION_SEGMENT_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION_SEGMENT_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_SECTION_VIEW = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION_VIEW
DWG_TYPE_AECC_DISP_REP_SECTION_VIEW_DEPTH_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION_VIEW_DEPTH_LABEL
DWG_TYPE_AECC_DISP_REP_SECTION_VIEW_QUANTITY_TAKEOFF_TABLE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SECTION_VIEW_QUANTITY_TAKEOFF_TABLE
DWG_TYPE_AECC_DISP_REP_SHEET = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SHEET
DWG_TYPE_AECC_DISP_REP_SPANNING_PIPE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SPANNING_PIPE_LABELING
DWG_TYPE_AECC_DISP_REP_SPANNING_PIPE_PROFILE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SPANNING_PIPE_PROFILE_LABELING
DWG_TYPE_AECC_DISP_REP_STATION_ELEV_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_STATION_ELEV_LABEL
DWG_TYPE_AECC_DISP_REP_STRUCTURE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_STRUCTURE
DWG_TYPE_AECC_DISP_REP_STRUCTURE_CSV = _LibreDWG.DWG_TYPE_AECC_DISP_REP_STRUCTURE_CSV
DWG_TYPE_AECC_DISP_REP_STRUCTURE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_STRUCTURE_LABELING
DWG_TYPE_AECC_DISP_REP_STRUCTURE_PROFILE_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_STRUCTURE_PROFILE_LABELING
DWG_TYPE_AECC_DISP_REP_STRUCTURE_SECTION_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_STRUCTURE_SECTION_LABELING
DWG_TYPE_AECC_DISP_REP_SUBASSEMBLY = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SUBASSEMBLY
DWG_TYPE_AECC_DISP_REP_SUPERELEVATION_BAND_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SUPERELEVATION_BAND_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_SUPERELEVATION_DIAGRAM_VIEW = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SUPERELEVATION_DIAGRAM_VIEW
DWG_TYPE_AECC_DISP_REP_SURFACE_CONTOUR_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SURFACE_CONTOUR_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_SURFACE_ELEVATION_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SURFACE_ELEVATION_LABEL
DWG_TYPE_AECC_DISP_REP_SURFACE_SLOPE_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SURFACE_SLOPE_LABEL
DWG_TYPE_AECC_DISP_REP_SURVEY_FIGURE_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SURVEY_FIGURE_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_SVFIGURE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SVFIGURE
DWG_TYPE_AECC_DISP_REP_SVFIGURE_CSV = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SVFIGURE_CSV
DWG_TYPE_AECC_DISP_REP_SVFIGURE_PROFILE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SVFIGURE_PROFILE
DWG_TYPE_AECC_DISP_REP_SVFIGURE_SECTION = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SVFIGURE_SECTION
DWG_TYPE_AECC_DISP_REP_SVFIGURE_SEGMENT_LABEL = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SVFIGURE_SEGMENT_LABEL
DWG_TYPE_AECC_DISP_REP_SVNETWORK = _LibreDWG.DWG_TYPE_AECC_DISP_REP_SVNETWORK
DWG_TYPE_AECC_DISP_REP_TANGENT_INTERSECTION_TABLE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_TANGENT_INTERSECTION_TABLE
DWG_TYPE_AECC_DISP_REP_TIN_SURFACE = _LibreDWG.DWG_TYPE_AECC_DISP_REP_TIN_SURFACE
DWG_TYPE_AECC_DISP_REP_TIN_SURFACE_CSV = _LibreDWG.DWG_TYPE_AECC_DISP_REP_TIN_SURFACE_CSV
DWG_TYPE_AECC_DISP_REP_VALIGNMENT_CRESTCURVE_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_VALIGNMENT_CRESTCURVE_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_VALIGNMENT_CSV = _LibreDWG.DWG_TYPE_AECC_DISP_REP_VALIGNMENT_CSV
DWG_TYPE_AECC_DISP_REP_VALIGNMENT_HAGEOMPT_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_VALIGNMENT_HAGEOMPT_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_VALIGNMENT_LINE_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_VALIGNMENT_LINE_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_VALIGNMENT_MINOR_STATION_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_VALIGNMENT_MINOR_STATION_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_VALIGNMENT_PVI_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_VALIGNMENT_PVI_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_VALIGNMENT_SAGCURVE_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_VALIGNMENT_SAGCURVE_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_VALIGNMENT_STATION_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_VALIGNMENT_STATION_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_VERTICALGEOMETRY_BAND_LABEL_GROUP = _LibreDWG.DWG_TYPE_AECC_DISP_REP_VERTICALGEOMETRY_BAND_LABEL_GROUP
DWG_TYPE_AECC_DISP_REP_VIEWFRAME_LABELING = _LibreDWG.DWG_TYPE_AECC_DISP_REP_VIEWFRAME_LABELING
DWG_TYPE_AECC_DISP_REP_VIEW_FRAME = _LibreDWG.DWG_TYPE_AECC_DISP_REP_VIEW_FRAME
DWG_TYPE_AECC_FEATURELINE_STYLE = _LibreDWG.DWG_TYPE_AECC_FEATURELINE_STYLE
DWG_TYPE_AECC_FEATURE_STYLE = _LibreDWG.DWG_TYPE_AECC_FEATURE_STYLE
DWG_TYPE_AECC_FITTING_STYLE = _LibreDWG.DWG_TYPE_AECC_FITTING_STYLE
DWG_TYPE_AECC_FORMAT_MANAGER_OBJECT = _LibreDWG.DWG_TYPE_AECC_FORMAT_MANAGER_OBJECT
DWG_TYPE_AECC_GRADEVIEW = _LibreDWG.DWG_TYPE_AECC_GRADEVIEW
DWG_TYPE_AECC_GRADING_CRITERIA = _LibreDWG.DWG_TYPE_AECC_GRADING_CRITERIA
DWG_TYPE_AECC_GRADING_CRITERIA_SET = _LibreDWG.DWG_TYPE_AECC_GRADING_CRITERIA_SET
DWG_TYPE_AECC_GRADING_GROUP = _LibreDWG.DWG_TYPE_AECC_GRADING_GROUP
DWG_TYPE_AECC_GRADING_STYLE = _LibreDWG.DWG_TYPE_AECC_GRADING_STYLE
DWG_TYPE_AECC_IMPORT_STORM_SEWER_DEFAULTS = _LibreDWG.DWG_TYPE_AECC_IMPORT_STORM_SEWER_DEFAULTS
DWG_TYPE_AECC_INTERFERENCE_STYLE = _LibreDWG.DWG_TYPE_AECC_INTERFERENCE_STYLE
DWG_TYPE_AECC_INTERSECTION_STYLE = _LibreDWG.DWG_TYPE_AECC_INTERSECTION_STYLE
DWG_TYPE_AECC_LABEL_COLLECTOR_STYLE = _LibreDWG.DWG_TYPE_AECC_LABEL_COLLECTOR_STYLE
DWG_TYPE_AECC_LABEL_NODE = _LibreDWG.DWG_TYPE_AECC_LABEL_NODE
DWG_TYPE_AECC_LABEL_RADIAL_LINE_STYLE = _LibreDWG.DWG_TYPE_AECC_LABEL_RADIAL_LINE_STYLE
DWG_TYPE_AECC_LABEL_TEXT_ITERATOR_CURVE_OR_SPIRAL_STYLE = _LibreDWG.DWG_TYPE_AECC_LABEL_TEXT_ITERATOR_CURVE_OR_SPIRAL_STYLE
DWG_TYPE_AECC_LABEL_TEXT_ITERATOR_STYLE = _LibreDWG.DWG_TYPE_AECC_LABEL_TEXT_ITERATOR_STYLE
DWG_TYPE_AECC_LABEL_TEXT_STYLE = _LibreDWG.DWG_TYPE_AECC_LABEL_TEXT_STYLE
DWG_TYPE_AECC_LABEL_VECTOR_ARROW_STYLE = _LibreDWG.DWG_TYPE_AECC_LABEL_VECTOR_ARROW_STYLE
DWG_TYPE_AECC_LEGEND_TABLE_STYLE = _LibreDWG.DWG_TYPE_AECC_LEGEND_TABLE_STYLE
DWG_TYPE_AECC_MASS_HAUL_LINE_STYLE = _LibreDWG.DWG_TYPE_AECC_MASS_HAUL_LINE_STYLE
DWG_TYPE_AECC_MASS_HAUL_VIEW_STYLE = _LibreDWG.DWG_TYPE_AECC_MASS_HAUL_VIEW_STYLE
DWG_TYPE_AECC_MATCHLINE_STYLE = _LibreDWG.DWG_TYPE_AECC_MATCHLINE_STYLE
DWG_TYPE_AECC_MATERIAL_STYLE = _LibreDWG.DWG_TYPE_AECC_MATERIAL_STYLE
DWG_TYPE_AECC_NETWORK_PART_CATALOG_DEF_NODE = _LibreDWG.DWG_TYPE_AECC_NETWORK_PART_CATALOG_DEF_NODE
DWG_TYPE_AECC_NETWORK_PART_FAMILY_ITEM = _LibreDWG.DWG_TYPE_AECC_NETWORK_PART_FAMILY_ITEM
DWG_TYPE_AECC_NETWORK_PART_LIST = _LibreDWG.DWG_TYPE_AECC_NETWORK_PART_LIST
DWG_TYPE_AECC_NETWORK_RULE = _LibreDWG.DWG_TYPE_AECC_NETWORK_RULE
DWG_TYPE_AECC_PARCEL_NODE = _LibreDWG.DWG_TYPE_AECC_PARCEL_NODE
DWG_TYPE_AECC_PARCEL_STYLE = _LibreDWG.DWG_TYPE_AECC_PARCEL_STYLE
DWG_TYPE_AECC_PART_SIZE_FILTER = _LibreDWG.DWG_TYPE_AECC_PART_SIZE_FILTER
DWG_TYPE_AECC_PIPE_RULES = _LibreDWG.DWG_TYPE_AECC_PIPE_RULES
DWG_TYPE_AECC_PIPE_STYLE = _LibreDWG.DWG_TYPE_AECC_PIPE_STYLE
DWG_TYPE_AECC_PIPE_STYLE_EXTENSION = _LibreDWG.DWG_TYPE_AECC_PIPE_STYLE_EXTENSION
DWG_TYPE_AECC_POINTCLOUD_STYLE = _LibreDWG.DWG_TYPE_AECC_POINTCLOUD_STYLE
DWG_TYPE_AECC_POINTVIEW = _LibreDWG.DWG_TYPE_AECC_POINTVIEW
DWG_TYPE_AECC_POINT_STYLE = _LibreDWG.DWG_TYPE_AECC_POINT_STYLE
DWG_TYPE_AECC_PRESSURE_PART_LIST = _LibreDWG.DWG_TYPE_AECC_PRESSURE_PART_LIST
DWG_TYPE_AECC_PRESSURE_PIPE_STYLE = _LibreDWG.DWG_TYPE_AECC_PRESSURE_PIPE_STYLE
DWG_TYPE_AECC_PROFILESECTIONENTITY_STYLE = _LibreDWG.DWG_TYPE_AECC_PROFILESECTIONENTITY_STYLE
DWG_TYPE_AECC_PROFILE_DESIGN_CHECK_SET = _LibreDWG.DWG_TYPE_AECC_PROFILE_DESIGN_CHECK_SET
DWG_TYPE_AECC_PROFILE_LABEL_SET = _LibreDWG.DWG_TYPE_AECC_PROFILE_LABEL_SET
DWG_TYPE_AECC_PROFILE_STYLE = _LibreDWG.DWG_TYPE_AECC_PROFILE_STYLE
DWG_TYPE_AECC_PROFILE_VIEW_BAND_STYLE_SET = _LibreDWG.DWG_TYPE_AECC_PROFILE_VIEW_BAND_STYLE_SET
DWG_TYPE_AECC_PROFILE_VIEW_DATA_BAND_STYLE = _LibreDWG.DWG_TYPE_AECC_PROFILE_VIEW_DATA_BAND_STYLE
DWG_TYPE_AECC_PROFILE_VIEW_HORIZONTAL_GEOMETRY_BAND_STYLE = _LibreDWG.DWG_TYPE_AECC_PROFILE_VIEW_HORIZONTAL_GEOMETRY_BAND_STYLE
DWG_TYPE_AECC_PROFILE_VIEW_PIPE_NETWORK_BAND_STYLE = _LibreDWG.DWG_TYPE_AECC_PROFILE_VIEW_PIPE_NETWORK_BAND_STYLE
DWG_TYPE_AECC_PROFILE_VIEW_SECTIONAL_DATA_BAND_STYLE = _LibreDWG.DWG_TYPE_AECC_PROFILE_VIEW_SECTIONAL_DATA_BAND_STYLE
DWG_TYPE_AECC_PROFILE_VIEW_STYLE = _LibreDWG.DWG_TYPE_AECC_PROFILE_VIEW_STYLE
DWG_TYPE_AECC_PROFILE_VIEW_SUPERELEVATION_DIAGRAM_BAND_STYLE = _LibreDWG.DWG_TYPE_AECC_PROFILE_VIEW_SUPERELEVATION_DIAGRAM_BAND_STYLE
DWG_TYPE_AECC_PROFILE_VIEW_VERTICAL_GEOMETRY_BAND_STYLE = _LibreDWG.DWG_TYPE_AECC_PROFILE_VIEW_VERTICAL_GEOMETRY_BAND_STYLE
DWG_TYPE_AECC_QUANTITY_TAKEOFF_CRITERIA = _LibreDWG.DWG_TYPE_AECC_QUANTITY_TAKEOFF_CRITERIA
DWG_TYPE_AECC_ROADWAYLINK_STYLE = _LibreDWG.DWG_TYPE_AECC_ROADWAYLINK_STYLE
DWG_TYPE_AECC_ROADWAYMARKER_STYLE = _LibreDWG.DWG_TYPE_AECC_ROADWAYMARKER_STYLE
DWG_TYPE_AECC_ROADWAYSHAPE_STYLE = _LibreDWG.DWG_TYPE_AECC_ROADWAYSHAPE_STYLE
DWG_TYPE_AECC_ROADWAY_STYLE_SET = _LibreDWG.DWG_TYPE_AECC_ROADWAY_STYLE_SET
DWG_TYPE_AECC_ROOT_SETTINGS_NODE = _LibreDWG.DWG_TYPE_AECC_ROOT_SETTINGS_NODE
DWG_TYPE_AECC_SAMPLE_LINE_GROUP_STYLE = _LibreDWG.DWG_TYPE_AECC_SAMPLE_LINE_GROUP_STYLE
DWG_TYPE_AECC_SAMPLE_LINE_STYLE = _LibreDWG.DWG_TYPE_AECC_SAMPLE_LINE_STYLE
DWG_TYPE_AECC_SECTION_LABEL_SET = _LibreDWG.DWG_TYPE_AECC_SECTION_LABEL_SET
DWG_TYPE_AECC_SECTION_STYLE = _LibreDWG.DWG_TYPE_AECC_SECTION_STYLE
DWG_TYPE_AECC_SECTION_VIEW_BAND_STYLE_SET = _LibreDWG.DWG_TYPE_AECC_SECTION_VIEW_BAND_STYLE_SET
DWG_TYPE_AECC_SECTION_VIEW_DATA_BAND_STYLE = _LibreDWG.DWG_TYPE_AECC_SECTION_VIEW_DATA_BAND_STYLE
DWG_TYPE_AECC_SECTION_VIEW_ROAD_SURFACE_BAND_STYLE = _LibreDWG.DWG_TYPE_AECC_SECTION_VIEW_ROAD_SURFACE_BAND_STYLE
DWG_TYPE_AECC_SECTION_VIEW_STYLE = _LibreDWG.DWG_TYPE_AECC_SECTION_VIEW_STYLE
DWG_TYPE_AECC_SETTINGS_NODE = _LibreDWG.DWG_TYPE_AECC_SETTINGS_NODE
DWG_TYPE_AECC_SHEET_STYLE = _LibreDWG.DWG_TYPE_AECC_SHEET_STYLE
DWG_TYPE_AECC_SLOPE_PATTERN_STYLE = _LibreDWG.DWG_TYPE_AECC_SLOPE_PATTERN_STYLE
DWG_TYPE_AECC_STATION_FORMAT_STYLE = _LibreDWG.DWG_TYPE_AECC_STATION_FORMAT_STYLE
DWG_TYPE_AECC_STRUCTURE_RULES = _LibreDWG.DWG_TYPE_AECC_STRUCTURE_RULES
DWG_TYPE_AECC_STUCTURE_STYLE = _LibreDWG.DWG_TYPE_AECC_STUCTURE_STYLE
DWG_TYPE_AECC_SUPERELEVATION_DIAGRAM_VIEW_STYLE = _LibreDWG.DWG_TYPE_AECC_SUPERELEVATION_DIAGRAM_VIEW_STYLE
DWG_TYPE_AECC_SURFACE_STYLE = _LibreDWG.DWG_TYPE_AECC_SURFACE_STYLE
DWG_TYPE_AECC_SVFIGURE_STYLE = _LibreDWG.DWG_TYPE_AECC_SVFIGURE_STYLE
DWG_TYPE_AECC_SVNETWORK_STYLE = _LibreDWG.DWG_TYPE_AECC_SVNETWORK_STYLE
DWG_TYPE_AECC_TABLE_STYLE = _LibreDWG.DWG_TYPE_AECC_TABLE_STYLE
DWG_TYPE_AECC_TAG_MANAGER = _LibreDWG.DWG_TYPE_AECC_TAG_MANAGER
DWG_TYPE_AECC_TREE_NODE = _LibreDWG.DWG_TYPE_AECC_TREE_NODE
DWG_TYPE_AECC_USER_DEFINED_ATTRIBUTE_CLASSIFICATION = _LibreDWG.DWG_TYPE_AECC_USER_DEFINED_ATTRIBUTE_CLASSIFICATION
DWG_TYPE_AECC_VALIGNMENT_STYLE_EXTENSION = _LibreDWG.DWG_TYPE_AECC_VALIGNMENT_STYLE_EXTENSION
DWG_TYPE_AECC_VIEW_FRAME_STYLE = _LibreDWG.DWG_TYPE_AECC_VIEW_FRAME_STYLE
DWG_TYPE_AECS_DISP_PROPS_MEMBER = _LibreDWG.DWG_TYPE_AECS_DISP_PROPS_MEMBER
DWG_TYPE_AECS_DISP_PROPS_MEMBER_LOGICAL = _LibreDWG.DWG_TYPE_AECS_DISP_PROPS_MEMBER_LOGICAL
DWG_TYPE_AECS_DISP_PROPS_MEMBER_PLAN = _LibreDWG.DWG_TYPE_AECS_DISP_PROPS_MEMBER_PLAN
DWG_TYPE_AECS_DISP_PROPS_MEMBER_PLAN_SKETCH = _LibreDWG.DWG_TYPE_AECS_DISP_PROPS_MEMBER_PLAN_SKETCH
DWG_TYPE_AECS_DISP_PROPS_MEMBER_PROJECTED = _LibreDWG.DWG_TYPE_AECS_DISP_PROPS_MEMBER_PROJECTED
DWG_TYPE_AECS_DISP_REP_MEMBER_ELEVATION_DESIGN = _LibreDWG.DWG_TYPE_AECS_DISP_REP_MEMBER_ELEVATION_DESIGN
DWG_TYPE_AECS_DISP_REP_MEMBER_ELEVATION_DETAIL = _LibreDWG.DWG_TYPE_AECS_DISP_REP_MEMBER_ELEVATION_DETAIL
DWG_TYPE_AECS_DISP_REP_MEMBER_LOGICAL = _LibreDWG.DWG_TYPE_AECS_DISP_REP_MEMBER_LOGICAL
DWG_TYPE_AECS_DISP_REP_MEMBER_MODEL_DESIGN = _LibreDWG.DWG_TYPE_AECS_DISP_REP_MEMBER_MODEL_DESIGN
DWG_TYPE_AECS_DISP_REP_MEMBER_MODEL_DETAIL = _LibreDWG.DWG_TYPE_AECS_DISP_REP_MEMBER_MODEL_DETAIL
DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_DESIGN = _LibreDWG.DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_DESIGN
DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_DETAIL = _LibreDWG.DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_DETAIL
DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_SKETCH = _LibreDWG.DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_SKETCH
DWG_TYPE_AECS_MEMBER_NODE_SHAPE = _LibreDWG.DWG_TYPE_AECS_MEMBER_NODE_SHAPE
DWG_TYPE_AECS_MEMBER_STYLE = _LibreDWG.DWG_TYPE_AECS_MEMBER_STYLE
DWG_TYPE_AEC_2DSECTION_STYLE = _LibreDWG.DWG_TYPE_AEC_2DSECTION_STYLE
DWG_TYPE_AEC_AECDBDISPREPBDGELEVLINEPLAN100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPBDGELEVLINEPLAN100
DWG_TYPE_AEC_AECDBDISPREPBDGELEVLINEPLAN50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPBDGELEVLINEPLAN50
DWG_TYPE_AEC_AECDBDISPREPBDGSECTIONLINEPLAN100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPBDGSECTIONLINEPLAN100
DWG_TYPE_AEC_AECDBDISPREPBDGSECTIONLINEPLAN50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPBDGSECTIONLINEPLAN50
DWG_TYPE_AEC_AECDBDISPREPCEILINGGRIDPLAN100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPCEILINGGRIDPLAN100
DWG_TYPE_AEC_AECDBDISPREPCEILINGGRIDPLAN50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPCEILINGGRIDPLAN50
DWG_TYPE_AEC_AECDBDISPREPCOLUMNGRIDPLAN100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPCOLUMNGRIDPLAN100
DWG_TYPE_AEC_AECDBDISPREPCOLUMNGRIDPLAN50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPCOLUMNGRIDPLAN50
DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN100
DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN50
DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLUNITPLAN100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLUNITPLAN100
DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLUNITPLAN50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLUNITPLAN50
DWG_TYPE_AEC_AECDBDISPREPMVBLOCKREFPLAN100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPMVBLOCKREFPLAN100
DWG_TYPE_AEC_AECDBDISPREPMVBLOCKREFPLAN50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPMVBLOCKREFPLAN50
DWG_TYPE_AEC_AECDBDISPREPROOFPLAN100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPROOFPLAN100
DWG_TYPE_AEC_AECDBDISPREPROOFPLAN50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPROOFPLAN50
DWG_TYPE_AEC_AECDBDISPREPROOFSLABPLAN100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPROOFSLABPLAN100
DWG_TYPE_AEC_AECDBDISPREPROOFSLABPLAN50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPROOFSLABPLAN50
DWG_TYPE_AEC_AECDBDISPREPSLABPLAN100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPSLABPLAN100
DWG_TYPE_AEC_AECDBDISPREPSLABPLAN50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPSLABPLAN50
DWG_TYPE_AEC_AECDBDISPREPSPACEPLAN100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPSPACEPLAN100
DWG_TYPE_AEC_AECDBDISPREPSPACEPLAN50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPSPACEPLAN50
DWG_TYPE_AEC_AECDBDISPREPWALLPLAN100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPWALLPLAN100
DWG_TYPE_AEC_AECDBDISPREPWALLPLAN50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPWALLPLAN50
DWG_TYPE_AEC_AECDBDISPREPWINDOWASSEMBLYPLAN100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPWINDOWASSEMBLYPLAN100
DWG_TYPE_AEC_AECDBDISPREPWINDOWASSEMBLYPLAN50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPWINDOWASSEMBLYPLAN50
DWG_TYPE_AEC_AECDBDISPREPZONE100 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPZONE100
DWG_TYPE_AEC_AECDBDISPREPZONE50 = _LibreDWG.DWG_TYPE_AEC_AECDBDISPREPZONE50
DWG_TYPE_AEC_AECDBZONEDEF = _LibreDWG.DWG_TYPE_AEC_AECDBZONEDEF
DWG_TYPE_AEC_AECDBZONESTYLE = _LibreDWG.DWG_TYPE_AEC_AECDBZONESTYLE
DWG_TYPE_AEC_ANCHOR_OPENINGBASE_TO_WALL = _LibreDWG.DWG_TYPE_AEC_ANCHOR_OPENINGBASE_TO_WALL
DWG_TYPE_AEC_CLASSIFICATION_DEF = _LibreDWG.DWG_TYPE_AEC_CLASSIFICATION_DEF
DWG_TYPE_AEC_CLASSIFICATION_SYSTEM_DEF = _LibreDWG.DWG_TYPE_AEC_CLASSIFICATION_SYSTEM_DEF
DWG_TYPE_AEC_CLEANUP_GROUP_DEF = _LibreDWG.DWG_TYPE_AEC_CLEANUP_GROUP_DEF
DWG_TYPE_AEC_CURTAIN_WALL_LAYOUT_STYLE = _LibreDWG.DWG_TYPE_AEC_CURTAIN_WALL_LAYOUT_STYLE
DWG_TYPE_AEC_CURTAIN_WALL_UNIT_STYLE = _LibreDWG.DWG_TYPE_AEC_CURTAIN_WALL_UNIT_STYLE
DWG_TYPE_AEC_CVSECTIONVIEW = _LibreDWG.DWG_TYPE_AEC_CVSECTIONVIEW
DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN = _LibreDWG.DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN
DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN100 = _LibreDWG.DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN100
DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN50 = _LibreDWG.DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN50
DWG_TYPE_AEC_DIM_STYLE = _LibreDWG.DWG_TYPE_AEC_DIM_STYLE
DWG_TYPE_AEC_DISPLAYTHEME_STYLE = _LibreDWG.DWG_TYPE_AEC_DISPLAYTHEME_STYLE
DWG_TYPE_AEC_DISP_PROPS_2D_SECTION = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_2D_SECTION
DWG_TYPE_AEC_DISP_PROPS_CLIP_VOLUME = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_CLIP_VOLUME
DWG_TYPE_AEC_DISP_PROPS_CLIP_VOLUME_RESULT = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_CLIP_VOLUME_RESULT
DWG_TYPE_AEC_DISP_PROPS_DIM = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_DIM
DWG_TYPE_AEC_DISP_PROPS_DISPLAYTHEME = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_DISPLAYTHEME
DWG_TYPE_AEC_DISP_PROPS_DOOR = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_DOOR
DWG_TYPE_AEC_DISP_PROPS_DOOR_NOMINAL = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_DOOR_NOMINAL
DWG_TYPE_AEC_DISP_PROPS_DOOR_PLAN_100 = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_DOOR_PLAN_100
DWG_TYPE_AEC_DISP_PROPS_DOOR_PLAN_50 = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_DOOR_PLAN_50
DWG_TYPE_AEC_DISP_PROPS_DOOR_THRESHOLD_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_DOOR_THRESHOLD_PLAN
DWG_TYPE_AEC_DISP_PROPS_DOOR_THRESHOLD_SYMBOL_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_DOOR_THRESHOLD_SYMBOL_PLAN
DWG_TYPE_AEC_DISP_PROPS_EDITINPLACEPROFILE_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_EDITINPLACEPROFILE_MODEL
DWG_TYPE_AEC_DISP_PROPS_ENT = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_ENT
DWG_TYPE_AEC_DISP_PROPS_ENT_REF = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_ENT_REF
DWG_TYPE_AEC_DISP_PROPS_GRID_ASSEMBLY_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_GRID_ASSEMBLY_MODEL
DWG_TYPE_AEC_DISP_PROPS_GRID_ASSEMBLY_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_GRID_ASSEMBLY_PLAN
DWG_TYPE_AEC_DISP_PROPS_LAYOUT_CURVE = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_LAYOUT_CURVE
DWG_TYPE_AEC_DISP_PROPS_LAYOUT_GRID2D = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_LAYOUT_GRID2D
DWG_TYPE_AEC_DISP_PROPS_LAYOUT_GRID3D = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_LAYOUT_GRID3D
DWG_TYPE_AEC_DISP_PROPS_MASKBLOCK = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_MASKBLOCK
DWG_TYPE_AEC_DISP_PROPS_MASS_ELEM_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_MASS_ELEM_MODEL
DWG_TYPE_AEC_DISP_PROPS_MASS_GROUP = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_MASS_GROUP
DWG_TYPE_AEC_DISP_PROPS_MATERIAL = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_MATERIAL
DWG_TYPE_AEC_DISP_PROPS_OPENING = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_OPENING
DWG_TYPE_AEC_DISP_PROPS_POLYGON_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_POLYGON_MODEL
DWG_TYPE_AEC_DISP_PROPS_POLYGON_TRUECOLOUR = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_POLYGON_TRUECOLOUR
DWG_TYPE_AEC_DISP_PROPS_RAILING_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_RAILING_MODEL
DWG_TYPE_AEC_DISP_PROPS_RAILING_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_RAILING_PLAN
DWG_TYPE_AEC_DISP_PROPS_ROOF = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_ROOF
DWG_TYPE_AEC_DISP_PROPS_ROOFSLAB = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_ROOFSLAB
DWG_TYPE_AEC_DISP_PROPS_ROOFSLAB_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_ROOFSLAB_PLAN
DWG_TYPE_AEC_DISP_PROPS_SCHEDULE_TABLE = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_SCHEDULE_TABLE
DWG_TYPE_AEC_DISP_PROPS_SLAB = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_SLAB
DWG_TYPE_AEC_DISP_PROPS_SLAB_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_SLAB_PLAN
DWG_TYPE_AEC_DISP_PROPS_SLICE = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_SLICE
DWG_TYPE_AEC_DISP_PROPS_SPACE_DECOMPOSED = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_SPACE_DECOMPOSED
DWG_TYPE_AEC_DISP_PROPS_SPACE_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_SPACE_MODEL
DWG_TYPE_AEC_DISP_PROPS_SPACE_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_SPACE_PLAN
DWG_TYPE_AEC_DISP_PROPS_STAIR_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_STAIR_MODEL
DWG_TYPE_AEC_DISP_PROPS_STAIR_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_STAIR_PLAN
DWG_TYPE_AEC_DISP_PROPS_STAIR_PLAN_OVERLAPPING = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_STAIR_PLAN_OVERLAPPING
DWG_TYPE_AEC_DISP_PROPS_WALL_GRAPH = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_WALL_GRAPH
DWG_TYPE_AEC_DISP_PROPS_WALL_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_WALL_MODEL
DWG_TYPE_AEC_DISP_PROPS_WALL_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_WALL_PLAN
DWG_TYPE_AEC_DISP_PROPS_WALL_SCHEM = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_WALL_SCHEM
DWG_TYPE_AEC_DISP_PROPS_WINDOW = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_WINDOW
DWG_TYPE_AEC_DISP_PROPS_WINDOW_ASSEMBLY_SILL_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_WINDOW_ASSEMBLY_SILL_PLAN
DWG_TYPE_AEC_DISP_PROPS_WINDOW_NOMINAL = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_WINDOW_NOMINAL
DWG_TYPE_AEC_DISP_PROPS_WINDOW_PLAN_100 = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_WINDOW_PLAN_100
DWG_TYPE_AEC_DISP_PROPS_WINDOW_PLAN_50 = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_WINDOW_PLAN_50
DWG_TYPE_AEC_DISP_PROPS_WINDOW_SILL_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_WINDOW_SILL_PLAN
DWG_TYPE_AEC_DISP_PROPS_ZONE = _LibreDWG.DWG_TYPE_AEC_DISP_PROPS_ZONE
DWG_TYPE_AEC_DISP_REP_2D_SECTION = _LibreDWG.DWG_TYPE_AEC_DISP_REP_2D_SECTION
DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSELEMPLAN100 = _LibreDWG.DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSELEMPLAN100
DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSELEMPLAN50 = _LibreDWG.DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSELEMPLAN50
DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSGROUPPLAN100 = _LibreDWG.DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSGROUPPLAN100
DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSGROUPPLAN50 = _LibreDWG.DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSGROUPPLAN50
DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLAN100 = _LibreDWG.DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLAN100
DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLAN50 = _LibreDWG.DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLAN50
DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLANREFLECTED = _LibreDWG.DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLANREFLECTED
DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGSILLPLAN = _LibreDWG.DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGSILLPLAN
DWG_TYPE_AEC_DISP_REP_ANCHOR = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ANCHOR
DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID
DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID_MODEL
DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID_RCP
DWG_TYPE_AEC_DISP_REP_ANCHOR_ENT_TO_NODE = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ANCHOR_ENT_TO_NODE
DWG_TYPE_AEC_DISP_REP_ANCHOR_EXT_TAG_TO_ENT = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ANCHOR_EXT_TAG_TO_ENT
DWG_TYPE_AEC_DISP_REP_ANCHOR_TAG_TO_ENT = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ANCHOR_TAG_TO_ENT
DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_MODEL
DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_PLAN
DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_RCP
DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_MODEL
DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_PLAN
DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_RCP
DWG_TYPE_AEC_DISP_REP_BDG_SECTION_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_BDG_SECTION_MODEL
DWG_TYPE_AEC_DISP_REP_BDG_SECTION_SUBDIV = _LibreDWG.DWG_TYPE_AEC_DISP_REP_BDG_SECTION_SUBDIV
DWG_TYPE_AEC_DISP_REP_CEILING_GRID = _LibreDWG.DWG_TYPE_AEC_DISP_REP_CEILING_GRID
DWG_TYPE_AEC_DISP_REP_CEILING_GRID_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_CEILING_GRID_MODEL
DWG_TYPE_AEC_DISP_REP_CEILING_GRID_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_CEILING_GRID_RCP
DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_MODEL
DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_PLAN
DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_RESULT = _LibreDWG.DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_RESULT
DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_RESULT_SUBDIV = _LibreDWG.DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_RESULT_SUBDIV
DWG_TYPE_AEC_DISP_REP_COL_BLOCK = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_BLOCK
DWG_TYPE_AEC_DISP_REP_COL_CIRCARC2D = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_CIRCARC2D
DWG_TYPE_AEC_DISP_REP_COL_CONCOINCIDENT = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_CONCOINCIDENT
DWG_TYPE_AEC_DISP_REP_COL_CONCONCENTRIC = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_CONCONCENTRIC
DWG_TYPE_AEC_DISP_REP_COL_CONEQUALDISTANCE = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_CONEQUALDISTANCE
DWG_TYPE_AEC_DISP_REP_COL_CONMIDPOINT = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_CONMIDPOINT
DWG_TYPE_AEC_DISP_REP_COL_CONNECTOR = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_CONNECTOR
DWG_TYPE_AEC_DISP_REP_COL_CONNORMAL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_CONNORMAL
DWG_TYPE_AEC_DISP_REP_COL_CONPARALLEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_CONPARALLEL
DWG_TYPE_AEC_DISP_REP_COL_CONPERPENDICULAR = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_CONPERPENDICULAR
DWG_TYPE_AEC_DISP_REP_COL_CONSYMMETRIC = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_CONSYMMETRIC
DWG_TYPE_AEC_DISP_REP_COL_CONTANGENT = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_CONTANGENT
DWG_TYPE_AEC_DISP_REP_COL_DIMANGLE = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_DIMANGLE
DWG_TYPE_AEC_DISP_REP_COL_DIMDIAMETER = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_DIMDIAMETER
DWG_TYPE_AEC_DISP_REP_COL_DIMDISTANCE = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_DIMDISTANCE
DWG_TYPE_AEC_DISP_REP_COL_DIMLENGTH = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_DIMLENGTH
DWG_TYPE_AEC_DISP_REP_COL_DIMMAJORRADIUS = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_DIMMAJORRADIUS
DWG_TYPE_AEC_DISP_REP_COL_DIMMINORRADIUS = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_DIMMINORRADIUS
DWG_TYPE_AEC_DISP_REP_COL_ELLIPARC2D = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_ELLIPARC2D
DWG_TYPE_AEC_DISP_REP_COL_LAYOUTDATA = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_LAYOUTDATA
DWG_TYPE_AEC_DISP_REP_COL_LINE2D = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_LINE2D
DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_ADD = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_ADD
DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_CUTPLANE = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_CUTPLANE
DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_EXTRUSION = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_EXTRUSION
DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_GROUP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_GROUP
DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_LOFT = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_LOFT
DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_PATH = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_PATH
DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_REVOLVE = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_REVOLVE
DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_SUBTRACT = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_SUBTRACT
DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_TRANSITION = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_TRANSITION
DWG_TYPE_AEC_DISP_REP_COL_POINT2D = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_POINT2D
DWG_TYPE_AEC_DISP_REP_COL_PROFILE = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_PROFILE
DWG_TYPE_AEC_DISP_REP_COLUMN_GRID = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COLUMN_GRID
DWG_TYPE_AEC_DISP_REP_COLUMN_GRID_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COLUMN_GRID_MODEL
DWG_TYPE_AEC_DISP_REP_COLUMN_GRID_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COLUMN_GRID_RCP
DWG_TYPE_AEC_DISP_REP_COL_WORKPLANE = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_WORKPLANE
DWG_TYPE_AEC_DISP_REP_COL_WORKPLANE_REF = _LibreDWG.DWG_TYPE_AEC_DISP_REP_COL_WORKPLANE_REF
DWG_TYPE_AEC_DISP_REP_CONFIG = _LibreDWG.DWG_TYPE_AEC_DISP_REP_CONFIG
DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_LAYOUT_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_LAYOUT_MODEL
DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_LAYOUT_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_LAYOUT_PLAN
DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_UNIT_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_UNIT_MODEL
DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_UNIT_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_UNIT_PLAN
DWG_TYPE_AEC_DISP_REP_DCM_DIMRADIUS = _LibreDWG.DWG_TYPE_AEC_DISP_REP_DCM_DIMRADIUS
DWG_TYPE_AEC_DISP_REP_DISPLAYTHEME = _LibreDWG.DWG_TYPE_AEC_DISP_REP_DISPLAYTHEME
DWG_TYPE_AEC_DISP_REP_DOOR_ELEV = _LibreDWG.DWG_TYPE_AEC_DISP_REP_DOOR_ELEV
DWG_TYPE_AEC_DISP_REP_DOOR_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_DOOR_MODEL
DWG_TYPE_AEC_DISP_REP_DOOR_NOMINAL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_DOOR_NOMINAL
DWG_TYPE_AEC_DISP_REP_DOOR_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_DOOR_PLAN
DWG_TYPE_AEC_DISP_REP_DOOR_PLAN_50 = _LibreDWG.DWG_TYPE_AEC_DISP_REP_DOOR_PLAN_50
DWG_TYPE_AEC_DISP_REP_DOOR_PLAN_HEKTO = _LibreDWG.DWG_TYPE_AEC_DISP_REP_DOOR_PLAN_HEKTO
DWG_TYPE_AEC_DISP_REP_DOOR_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_DOOR_RCP
DWG_TYPE_AEC_DISP_REP_DOOR_THRESHOLD_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_DOOR_THRESHOLD_PLAN
DWG_TYPE_AEC_DISP_REP_DOOR_THRESHOLD_SYMBOL_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_DOOR_THRESHOLD_SYMBOL_PLAN
DWG_TYPE_AEC_DISP_REP_EDITINPLACEPROFILE = _LibreDWG.DWG_TYPE_AEC_DISP_REP_EDITINPLACEPROFILE
DWG_TYPE_AEC_DISP_REP_ENT_REF = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ENT_REF
DWG_TYPE_AEC_DISP_REP_LAYOUT_CURVE = _LibreDWG.DWG_TYPE_AEC_DISP_REP_LAYOUT_CURVE
DWG_TYPE_AEC_DISP_REP_LAYOUT_GRID2D = _LibreDWG.DWG_TYPE_AEC_DISP_REP_LAYOUT_GRID2D
DWG_TYPE_AEC_DISP_REP_LAYOUT_GRID3D = _LibreDWG.DWG_TYPE_AEC_DISP_REP_LAYOUT_GRID3D
DWG_TYPE_AEC_DISP_REP_MASKBLOCK_REF = _LibreDWG.DWG_TYPE_AEC_DISP_REP_MASKBLOCK_REF
DWG_TYPE_AEC_DISP_REP_MASKBLOCK_REF_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_MASKBLOCK_REF_RCP
DWG_TYPE_AEC_DISP_REP_MASS_ELEM_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_MASS_ELEM_MODEL
DWG_TYPE_AEC_DISP_REP_MASS_ELEM_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_MASS_ELEM_RCP
DWG_TYPE_AEC_DISP_REP_MASS_ELEM_SCHEM = _LibreDWG.DWG_TYPE_AEC_DISP_REP_MASS_ELEM_SCHEM
DWG_TYPE_AEC_DISP_REP_MASS_GROUP_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_MASS_GROUP_MODEL
DWG_TYPE_AEC_DISP_REP_MASS_GROUP_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_MASS_GROUP_PLAN
DWG_TYPE_AEC_DISP_REP_MASS_GROUP_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_MASS_GROUP_RCP
DWG_TYPE_AEC_DISP_REP_MATERIAL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_MATERIAL
DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF = _LibreDWG.DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF
DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF_MODEL
DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF_RCP
DWG_TYPE_AEC_DISP_REP_OPENING = _LibreDWG.DWG_TYPE_AEC_DISP_REP_OPENING
DWG_TYPE_AEC_DISP_REP_OPENING_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_OPENING_MODEL
DWG_TYPE_AEC_DISP_REP_POLYGON_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_POLYGON_MODEL
DWG_TYPE_AEC_DISP_REP_POLYGON_TRUECOLOUR = _LibreDWG.DWG_TYPE_AEC_DISP_REP_POLYGON_TRUECOLOUR
DWG_TYPE_AEC_DISP_REP_RAILING_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_RAILING_MODEL
DWG_TYPE_AEC_DISP_REP_RAILING_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_RAILING_PLAN
DWG_TYPE_AEC_DISP_REP_RAILING_PLAN_100 = _LibreDWG.DWG_TYPE_AEC_DISP_REP_RAILING_PLAN_100
DWG_TYPE_AEC_DISP_REP_RAILING_PLAN_50 = _LibreDWG.DWG_TYPE_AEC_DISP_REP_RAILING_PLAN_50
DWG_TYPE_AEC_DISP_REP_ROOF_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ROOF_MODEL
DWG_TYPE_AEC_DISP_REP_ROOF_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ROOF_PLAN
DWG_TYPE_AEC_DISP_REP_ROOF_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ROOF_RCP
DWG_TYPE_AEC_DISP_REP_ROOFSLAB_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ROOFSLAB_MODEL
DWG_TYPE_AEC_DISP_REP_ROOFSLAB_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ROOFSLAB_PLAN
DWG_TYPE_AEC_DISP_REP_SCHEDULE_TABLE = _LibreDWG.DWG_TYPE_AEC_DISP_REP_SCHEDULE_TABLE
DWG_TYPE_AEC_DISP_REP_SET = _LibreDWG.DWG_TYPE_AEC_DISP_REP_SET
DWG_TYPE_AEC_DISP_REP_SLAB_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_SLAB_MODEL
DWG_TYPE_AEC_DISP_REP_SLAB_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_SLAB_PLAN
DWG_TYPE_AEC_DISP_REP_SLICE = _LibreDWG.DWG_TYPE_AEC_DISP_REP_SLICE
DWG_TYPE_AEC_DISP_REP_SPACE_DECOMPOSED = _LibreDWG.DWG_TYPE_AEC_DISP_REP_SPACE_DECOMPOSED
DWG_TYPE_AEC_DISP_REP_SPACE_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_SPACE_MODEL
DWG_TYPE_AEC_DISP_REP_SPACE_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_SPACE_PLAN
DWG_TYPE_AEC_DISP_REP_SPACE_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_SPACE_RCP
DWG_TYPE_AEC_DISP_REP_SPACE_VOLUME = _LibreDWG.DWG_TYPE_AEC_DISP_REP_SPACE_VOLUME
DWG_TYPE_AEC_DISP_REP_STAIR_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_STAIR_MODEL
DWG_TYPE_AEC_DISP_REP_STAIR_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_STAIR_PLAN
DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_100 = _LibreDWG.DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_100
DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_50 = _LibreDWG.DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_50
DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_OVERLAPPING = _LibreDWG.DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_OVERLAPPING
DWG_TYPE_AEC_DISP_REP_STAIR_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_STAIR_RCP
DWG_TYPE_AEC_DISP_REP_WALL_GRAPH = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WALL_GRAPH
DWG_TYPE_AEC_DISP_REP_WALL_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WALL_MODEL
DWG_TYPE_AEC_DISP_REP_WALL_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WALL_PLAN
DWG_TYPE_AEC_DISP_REP_WALL_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WALL_RCP
DWG_TYPE_AEC_DISP_REP_WALL_SCHEM = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WALL_SCHEM
DWG_TYPE_AEC_DISP_REP_WINDOW_ASSEMBLY_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WINDOW_ASSEMBLY_MODEL
DWG_TYPE_AEC_DISP_REP_WINDOW_ASSEMBLY_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WINDOW_ASSEMBLY_PLAN
DWG_TYPE_AEC_DISP_REP_WINDOWASSEMBLY_SILL_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WINDOWASSEMBLY_SILL_PLAN
DWG_TYPE_AEC_DISP_REP_WINDOW_ELEV = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WINDOW_ELEV
DWG_TYPE_AEC_DISP_REP_WINDOW_MODEL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WINDOW_MODEL
DWG_TYPE_AEC_DISP_REP_WINDOW_NOMINAL = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WINDOW_NOMINAL
DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN
DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN_100 = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN_100
DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN_50 = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN_50
DWG_TYPE_AEC_DISP_REP_WINDOW_RCP = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WINDOW_RCP
DWG_TYPE_AEC_DISP_REP_WINDOW_SILL_PLAN = _LibreDWG.DWG_TYPE_AEC_DISP_REP_WINDOW_SILL_PLAN
DWG_TYPE_AEC_DISP_REP_ZONE = _LibreDWG.DWG_TYPE_AEC_DISP_REP_ZONE
DWG_TYPE_AEC_DISPROPSMASSELEMPLANCOMMON = _LibreDWG.DWG_TYPE_AEC_DISPROPSMASSELEMPLANCOMMON
DWG_TYPE_AEC_DISPROPSMASSGROUPPLANCOMMON = _LibreDWG.DWG_TYPE_AEC_DISPROPSMASSGROUPPLANCOMMON
DWG_TYPE_AEC_DISPROPSOPENINGPLANCOMMON = _LibreDWG.DWG_TYPE_AEC_DISPROPSOPENINGPLANCOMMON
DWG_TYPE_AEC_DISPROPSOPENINGPLANCOMMONHATCHED = _LibreDWG.DWG_TYPE_AEC_DISPROPSOPENINGPLANCOMMONHATCHED
DWG_TYPE_AEC_DISPROPSOPENINGSILLPLAN = _LibreDWG.DWG_TYPE_AEC_DISPROPSOPENINGSILLPLAN
DWG_TYPE_AEC_DISP_ROPS_RAILING_PLAN_100 = _LibreDWG.DWG_TYPE_AEC_DISP_ROPS_RAILING_PLAN_100
DWG_TYPE_AEC_DISP_ROPS_RAILING_PLAN_50 = _LibreDWG.DWG_TYPE_AEC_DISP_ROPS_RAILING_PLAN_50
DWG_TYPE_AEC_DISP_ROPS_STAIR_PLAN_100 = _LibreDWG.DWG_TYPE_AEC_DISP_ROPS_STAIR_PLAN_100
DWG_TYPE_AEC_DISP_ROPS_STAIR_PLAN_50 = _LibreDWG.DWG_TYPE_AEC_DISP_ROPS_STAIR_PLAN_50
DWG_TYPE_AEC_DOOR_STYLE = _LibreDWG.DWG_TYPE_AEC_DOOR_STYLE
DWG_TYPE_AEC_ENDCAP_STYLE = _LibreDWG.DWG_TYPE_AEC_ENDCAP_STYLE
DWG_TYPE_AEC_FRAME_DEF = _LibreDWG.DWG_TYPE_AEC_FRAME_DEF
DWG_TYPE_AEC_LAYERKEY_STYLE = _LibreDWG.DWG_TYPE_AEC_LAYERKEY_STYLE
DWG_TYPE_AEC_LIST_DEF = _LibreDWG.DWG_TYPE_AEC_LIST_DEF
DWG_TYPE_AEC_MASKBLOCK_DEF = _LibreDWG.DWG_TYPE_AEC_MASKBLOCK_DEF
DWG_TYPE_AEC_MASS_ELEM_STYLE = _LibreDWG.DWG_TYPE_AEC_MASS_ELEM_STYLE
DWG_TYPE_AEC_MATERIAL_DEF = _LibreDWG.DWG_TYPE_AEC_MATERIAL_DEF
DWG_TYPE_AEC_MVBLOCK_DEF = _LibreDWG.DWG_TYPE_AEC_MVBLOCK_DEF
DWG_TYPE_AEC_MVBLOCK_REF = _LibreDWG.DWG_TYPE_AEC_MVBLOCK_REF
DWG_TYPE_AEC_NOTIFICATION_TRACKER = _LibreDWG.DWG_TYPE_AEC_NOTIFICATION_TRACKER
DWG_TYPE_AEC_POLYGON = _LibreDWG.DWG_TYPE_AEC_POLYGON
DWG_TYPE_AEC_POLYGON_STYLE = _LibreDWG.DWG_TYPE_AEC_POLYGON_STYLE
DWG_TYPE_AEC_PROPERTY_SET_DEF = _LibreDWG.DWG_TYPE_AEC_PROPERTY_SET_DEF
DWG_TYPE_AEC_RAILING_STYLE = _LibreDWG.DWG_TYPE_AEC_RAILING_STYLE
DWG_TYPE_AEC_REFEDIT_STATUS_TRACKER = _LibreDWG.DWG_TYPE_AEC_REFEDIT_STATUS_TRACKER
DWG_TYPE_AEC_ROOFSLABEDGE_STYLE = _LibreDWG.DWG_TYPE_AEC_ROOFSLABEDGE_STYLE
DWG_TYPE_AEC_ROOFSLAB_STYLE = _LibreDWG.DWG_TYPE_AEC_ROOFSLAB_STYLE
DWG_TYPE_AEC_SCHEDULE_DATA_FORMAT = _LibreDWG.DWG_TYPE_AEC_SCHEDULE_DATA_FORMAT
DWG_TYPE_AEC_SLABEDGE_STYLE = _LibreDWG.DWG_TYPE_AEC_SLABEDGE_STYLE
DWG_TYPE_AEC_SLAB_STYLE = _LibreDWG.DWG_TYPE_AEC_SLAB_STYLE
DWG_TYPE_AEC_SPACE_STYLES = _LibreDWG.DWG_TYPE_AEC_SPACE_STYLES
DWG_TYPE_AEC_STAIR_STYLE = _LibreDWG.DWG_TYPE_AEC_STAIR_STYLE
DWG_TYPE_AEC_STAIR_WINDER_STYLE = _LibreDWG.DWG_TYPE_AEC_STAIR_WINDER_STYLE
DWG_TYPE_AEC_STAIR_WINDER_TYPE_BALANCED = _LibreDWG.DWG_TYPE_AEC_STAIR_WINDER_TYPE_BALANCED
DWG_TYPE_AEC_STAIR_WINDER_TYPE_MANUAL = _LibreDWG.DWG_TYPE_AEC_STAIR_WINDER_TYPE_MANUAL
DWG_TYPE_AEC_STAIR_WINDER_TYPE_SINGLE_POINT = _LibreDWG.DWG_TYPE_AEC_STAIR_WINDER_TYPE_SINGLE_POINT
DWG_TYPE_AEC_VARS_AECBBLDSRV = _LibreDWG.DWG_TYPE_AEC_VARS_AECBBLDSRV
DWG_TYPE_AEC_VARS_ARCHBASE = _LibreDWG.DWG_TYPE_AEC_VARS_ARCHBASE
DWG_TYPE_AEC_VARS_DWG_SETUP = _LibreDWG.DWG_TYPE_AEC_VARS_DWG_SETUP
DWG_TYPE_AEC_VARS_MUNICH = _LibreDWG.DWG_TYPE_AEC_VARS_MUNICH
DWG_TYPE_AEC_VARS_STRUCTUREBASE = _LibreDWG.DWG_TYPE_AEC_VARS_STRUCTUREBASE
DWG_TYPE_AEC_WALLMOD_STYLE = _LibreDWG.DWG_TYPE_AEC_WALLMOD_STYLE
DWG_TYPE_AEC_WALL_STYLE = _LibreDWG.DWG_TYPE_AEC_WALL_STYLE
DWG_TYPE_AEC_WINDOW_ASSEMBLY_STYLE = _LibreDWG.DWG_TYPE_AEC_WINDOW_ASSEMBLY_STYLE
DWG_TYPE_AEC_WINDOW_STYLE = _LibreDWG.DWG_TYPE_AEC_WINDOW_STYLE
DWG_TYPE_ALIGNMENTGRIPENTITY = _LibreDWG.DWG_TYPE_ALIGNMENTGRIPENTITY
DWG_TYPE_AMCONTEXTMGR = _LibreDWG.DWG_TYPE_AMCONTEXTMGR
DWG_TYPE_AMDTADMENUSTATE = _LibreDWG.DWG_TYPE_AMDTADMENUSTATE
DWG_TYPE_AMDTAMMENUSTATE = _LibreDWG.DWG_TYPE_AMDTAMMENUSTATE
DWG_TYPE_AMDTBROWSERDBTAB = _LibreDWG.DWG_TYPE_AMDTBROWSERDBTAB
DWG_TYPE_AMDTDMMENUSTATE = _LibreDWG.DWG_TYPE_AMDTDMMENUSTATE
DWG_TYPE_AMDTEDGESTANDARDDIN = _LibreDWG.DWG_TYPE_AMDTEDGESTANDARDDIN
DWG_TYPE_AMDTEDGESTANDARDDIN13715 = _LibreDWG.DWG_TYPE_AMDTEDGESTANDARDDIN13715
DWG_TYPE_AMDTEDGESTANDARDISO = _LibreDWG.DWG_TYPE_AMDTEDGESTANDARDISO
DWG_TYPE_AMDTEDGESTANDARDISO13715 = _LibreDWG.DWG_TYPE_AMDTEDGESTANDARDISO13715
DWG_TYPE_AMDTFORMULAUPDATEDISPATCHER = _LibreDWG.DWG_TYPE_AMDTFORMULAUPDATEDISPATCHER
DWG_TYPE_AMDTINTERNALREACTOR = _LibreDWG.DWG_TYPE_AMDTINTERNALREACTOR
DWG_TYPE_AMDTMCOMMENUSTATE = _LibreDWG.DWG_TYPE_AMDTMCOMMENUSTATE
DWG_TYPE_AMDTMENUSTATEMGR = _LibreDWG.DWG_TYPE_AMDTMENUSTATEMGR
DWG_TYPE_AMDTNOTE = _LibreDWG.DWG_TYPE_AMDTNOTE
DWG_TYPE_AMDTNOTETEMPLATEDB = _LibreDWG.DWG_TYPE_AMDTNOTETEMPLATEDB
DWG_TYPE_AMDTSECTIONSYM = _LibreDWG.DWG_TYPE_AMDTSECTIONSYM
DWG_TYPE_AMDTSECTIONSYMLABEL = _LibreDWG.DWG_TYPE_AMDTSECTIONSYMLABEL
DWG_TYPE_AMDTSYSATTR = _LibreDWG.DWG_TYPE_AMDTSYSATTR
DWG_TYPE_AMGOBJPROPCFG = _LibreDWG.DWG_TYPE_AMGOBJPROPCFG
DWG_TYPE_AMGSETTINGSOBJ = _LibreDWG.DWG_TYPE_AMGSETTINGSOBJ
DWG_TYPE_AMIMASTER = _LibreDWG.DWG_TYPE_AMIMASTER
DWG_TYPE_AM_DRAWING_MGR = _LibreDWG.DWG_TYPE_AM_DRAWING_MGR
DWG_TYPE_AM_DWGMGR_NAME = _LibreDWG.DWG_TYPE_AM_DWGMGR_NAME
DWG_TYPE_AM_DWG_DOCUMENT = _LibreDWG.DWG_TYPE_AM_DWG_DOCUMENT
DWG_TYPE_AM_DWG_SHEET = _LibreDWG.DWG_TYPE_AM_DWG_SHEET
DWG_TYPE_AM_VIEWDIMPARMAP = _LibreDWG.DWG_TYPE_AM_VIEWDIMPARMAP
DWG_TYPE_BINRECORD = _LibreDWG.DWG_TYPE_BINRECORD
DWG_TYPE_CAMSCATALOGAPPOBJECT = _LibreDWG.DWG_TYPE_CAMSCATALOGAPPOBJECT
DWG_TYPE_CAMSSTRUCTBTNSTATE = _LibreDWG.DWG_TYPE_CAMSSTRUCTBTNSTATE
DWG_TYPE_CATALOGSTATE = _LibreDWG.DWG_TYPE_CATALOGSTATE
DWG_TYPE_CBROWSERAPPOBJECT = _LibreDWG.DWG_TYPE_CBROWSERAPPOBJECT
DWG_TYPE_DEPMGR = _LibreDWG.DWG_TYPE_DEPMGR
DWG_TYPE_DMBASEELEMENT = _LibreDWG.DWG_TYPE_DMBASEELEMENT
DWG_TYPE_DMDEFAULTSTYLE = _LibreDWG.DWG_TYPE_DMDEFAULTSTYLE
DWG_TYPE_DMLEGEND = _LibreDWG.DWG_TYPE_DMLEGEND
DWG_TYPE_DMMAP = _LibreDWG.DWG_TYPE_DMMAP
DWG_TYPE_DMMAPMANAGER = _LibreDWG.DWG_TYPE_DMMAPMANAGER
DWG_TYPE_DMSTYLECATEGORY = _LibreDWG.DWG_TYPE_DMSTYLECATEGORY
DWG_TYPE_DMSTYLELIBRARY = _LibreDWG.DWG_TYPE_DMSTYLELIBRARY
DWG_TYPE_DMSTYLEREFERENCE = _LibreDWG.DWG_TYPE_DMSTYLEREFERENCE
DWG_TYPE_DMSTYLIZEDENTITIESTABLE = _LibreDWG.DWG_TYPE_DMSTYLIZEDENTITIESTABLE
DWG_TYPE_DMSURROGATESTYLESETS = _LibreDWG.DWG_TYPE_DMSURROGATESTYLESETS
DWG_TYPE_DM_PLACEHOLDER = _LibreDWG.DWG_TYPE_DM_PLACEHOLDER
DWG_TYPE_EXACTERMXREFMAP = _LibreDWG.DWG_TYPE_EXACTERMXREFMAP
DWG_TYPE_EXACXREFPANELOBJECT = _LibreDWG.DWG_TYPE_EXACXREFPANELOBJECT
DWG_TYPE_EXPO_NOTIFYBLOCK = _LibreDWG.DWG_TYPE_EXPO_NOTIFYBLOCK
DWG_TYPE_EXPO_NOTIFYHALL = _LibreDWG.DWG_TYPE_EXPO_NOTIFYHALL
DWG_TYPE_EXPO_NOTIFYPILLAR = _LibreDWG.DWG_TYPE_EXPO_NOTIFYPILLAR
DWG_TYPE_EXPO_NOTIFYSTAND = _LibreDWG.DWG_TYPE_EXPO_NOTIFYSTAND
DWG_TYPE_EXPO_NOTIFYSTANDNOPOLY = _LibreDWG.DWG_TYPE_EXPO_NOTIFYSTANDNOPOLY
DWG_TYPE_GSMANAGER = _LibreDWG.DWG_TYPE_GSMANAGER
DWG_TYPE_IRD_DSC_DICT = _LibreDWG.DWG_TYPE_IRD_DSC_DICT
DWG_TYPE_IRD_DSC_RECORD = _LibreDWG.DWG_TYPE_IRD_DSC_RECORD
DWG_TYPE_IRD_OBJ_RECORD = _LibreDWG.DWG_TYPE_IRD_OBJ_RECORD
DWG_TYPE_MAPFSMRVOBJECT = _LibreDWG.DWG_TYPE_MAPFSMRVOBJECT
DWG_TYPE_MAPGWSUNDOOBJECT = _LibreDWG.DWG_TYPE_MAPGWSUNDOOBJECT
DWG_TYPE_MAPIAMMOUDLE = _LibreDWG.DWG_TYPE_MAPIAMMOUDLE
DWG_TYPE_MAPMETADATAOBJECT = _LibreDWG.DWG_TYPE_MAPMETADATAOBJECT
DWG_TYPE_MAPRESOURCEMANAGEROBJECT = _LibreDWG.DWG_TYPE_MAPRESOURCEMANAGEROBJECT
DWG_TYPE_McDbContainer2 = _LibreDWG.DWG_TYPE_McDbContainer2
DWG_TYPE_McDbMarker = _LibreDWG.DWG_TYPE_McDbMarker
DWG_TYPE_NAMEDAPPL = _LibreDWG.DWG_TYPE_NAMEDAPPL
DWG_TYPE_NEWSTDPARTPARLIST = _LibreDWG.DWG_TYPE_NEWSTDPARTPARLIST
DWG_TYPE_NOTEPOSITION = _LibreDWG.DWG_TYPE_NOTEPOSITION
DWG_TYPE_OBJCLONER = _LibreDWG.DWG_TYPE_OBJCLONER
DWG_TYPE_PARAMMGR = _LibreDWG.DWG_TYPE_PARAMMGR
DWG_TYPE_PARAMSCOPE = _LibreDWG.DWG_TYPE_PARAMSCOPE
DWG_TYPE_PILLAR = _LibreDWG.DWG_TYPE_PILLAR
DWG_TYPE_STDPART2D = _LibreDWG.DWG_TYPE_STDPART2D
DWG_TYPE_TCH_ARROW = _LibreDWG.DWG_TYPE_TCH_ARROW
DWG_TYPE_TCH_AXIS_LABEL = _LibreDWG.DWG_TYPE_TCH_AXIS_LABEL
DWG_TYPE_TCH_BLOCK_INSERT = _LibreDWG.DWG_TYPE_TCH_BLOCK_INSERT
DWG_TYPE_TCH_COLUMN = _LibreDWG.DWG_TYPE_TCH_COLUMN
DWG_TYPE_TCH_DBCONFIG = _LibreDWG.DWG_TYPE_TCH_DBCONFIG
DWG_TYPE_TCH_DIMENSION2 = _LibreDWG.DWG_TYPE_TCH_DIMENSION2
DWG_TYPE_TCH_DRAWINGINDEX = _LibreDWG.DWG_TYPE_TCH_DRAWINGINDEX
DWG_TYPE_TCH_HANDRAIL = _LibreDWG.DWG_TYPE_TCH_HANDRAIL
DWG_TYPE_TCH_LINESTAIR = _LibreDWG.DWG_TYPE_TCH_LINESTAIR
DWG_TYPE_TCH_OPENING = _LibreDWG.DWG_TYPE_TCH_OPENING
DWG_TYPE_TCH_RECTSTAIR = _LibreDWG.DWG_TYPE_TCH_RECTSTAIR
DWG_TYPE_TCH_SLAB = _LibreDWG.DWG_TYPE_TCH_SLAB
DWG_TYPE_TCH_SPACE = _LibreDWG.DWG_TYPE_TCH_SPACE
DWG_TYPE_TCH_TEXT = _LibreDWG.DWG_TYPE_TCH_TEXT
DWG_TYPE_TCH_WALL = _LibreDWG.DWG_TYPE_TCH_WALL
DWG_TYPE_TGrupoPuntos = _LibreDWG.DWG_TYPE_TGrupoPuntos
DWG_TYPE_VAACIMAGEINVENTORY = _LibreDWG.DWG_TYPE_VAACIMAGEINVENTORY
DWG_TYPE_VAACXREFPANELOBJECT = _LibreDWG.DWG_TYPE_VAACXREFPANELOBJECT
DWG_TYPE_FREED = _LibreDWG.DWG_TYPE_FREED
DWG_TYPE_UNKNOWN_ENT = _LibreDWG.DWG_TYPE_UNKNOWN_ENT
DWG_TYPE_UNKNOWN_OBJ = _LibreDWG.DWG_TYPE_UNKNOWN_OBJ
DWG_TYPE_UNUSED_r11 = _LibreDWG.DWG_TYPE_UNUSED_r11
DWG_TYPE_LINE_r11 = _LibreDWG.DWG_TYPE_LINE_r11
DWG_TYPE_POINT_r11 = _LibreDWG.DWG_TYPE_POINT_r11
DWG_TYPE_CIRCLE_r11 = _LibreDWG.DWG_TYPE_CIRCLE_r11
DWG_TYPE_SHAPE_r11 = _LibreDWG.DWG_TYPE_SHAPE_r11
DWG_TYPE_REPEAT_r11 = _LibreDWG.DWG_TYPE_REPEAT_r11
DWG_TYPE_ENDREP_r11 = _LibreDWG.DWG_TYPE_ENDREP_r11
DWG_TYPE_TEXT_r11 = _LibreDWG.DWG_TYPE_TEXT_r11
DWG_TYPE_ARC_r11 = _LibreDWG.DWG_TYPE_ARC_r11
DWG_TYPE_TRACE_r11 = _LibreDWG.DWG_TYPE_TRACE_r11
DWG_TYPE_LOAD_r11 = _LibreDWG.DWG_TYPE_LOAD_r11
DWG_TYPE_SOLID_r11 = _LibreDWG.DWG_TYPE_SOLID_r11
DWG_TYPE_BLOCK_r11 = _LibreDWG.DWG_TYPE_BLOCK_r11
DWG_TYPE_ENDBLK_r11 = _LibreDWG.DWG_TYPE_ENDBLK_r11
DWG_TYPE_INSERT_r11 = _LibreDWG.DWG_TYPE_INSERT_r11
DWG_TYPE_ATTDEF_r11 = _LibreDWG.DWG_TYPE_ATTDEF_r11
DWG_TYPE_ATTRIB_r11 = _LibreDWG.DWG_TYPE_ATTRIB_r11
DWG_TYPE_SEQEND_r11 = _LibreDWG.DWG_TYPE_SEQEND_r11
DWG_TYPE_JUMP_r11 = _LibreDWG.DWG_TYPE_JUMP_r11
DWG_TYPE_POLYLINE_r11 = _LibreDWG.DWG_TYPE_POLYLINE_r11
DWG_TYPE_VERTEX_r11 = _LibreDWG.DWG_TYPE_VERTEX_r11
DWG_TYPE_3DLINE_r11 = _LibreDWG.DWG_TYPE_3DLINE_r11
DWG_TYPE_3DFACE_r11 = _LibreDWG.DWG_TYPE_3DFACE_r11
DWG_TYPE_DIMENSION_r11 = _LibreDWG.DWG_TYPE_DIMENSION_r11
DWG_TYPE_VIEWPORT_r11 = _LibreDWG.DWG_TYPE_VIEWPORT_r11
DWG_TYPE_UNKNOWN_r11 = _LibreDWG.DWG_TYPE_UNKNOWN_r11
DWG_NOERR = _LibreDWG.DWG_NOERR
DWG_ERR_WRONGCRC = _LibreDWG.DWG_ERR_WRONGCRC
DWG_ERR_NOTYETSUPPORTED = _LibreDWG.DWG_ERR_NOTYETSUPPORTED
DWG_ERR_UNHANDLEDCLASS = _LibreDWG.DWG_ERR_UNHANDLEDCLASS
DWG_ERR_INVALIDTYPE = _LibreDWG.DWG_ERR_INVALIDTYPE
DWG_ERR_INVALIDHANDLE = _LibreDWG.DWG_ERR_INVALIDHANDLE
DWG_ERR_INVALIDEED = _LibreDWG.DWG_ERR_INVALIDEED
DWG_ERR_VALUEOUTOFBOUNDS = _LibreDWG.DWG_ERR_VALUEOUTOFBOUNDS
DWG_ERR_CLASSESNOTFOUND = _LibreDWG.DWG_ERR_CLASSESNOTFOUND
DWG_ERR_SECTIONNOTFOUND = _LibreDWG.DWG_ERR_SECTIONNOTFOUND
DWG_ERR_PAGENOTFOUND = _LibreDWG.DWG_ERR_PAGENOTFOUND
DWG_ERR_INTERNALERROR = _LibreDWG.DWG_ERR_INTERNALERROR
DWG_ERR_INVALIDDWG = _LibreDWG.DWG_ERR_INVALIDDWG
DWG_ERR_IOERROR = _LibreDWG.DWG_ERR_IOERROR
DWG_ERR_OUTOFMEM = _LibreDWG.DWG_ERR_OUTOFMEM
class Dwg_Handle(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    code = property(_LibreDWG.Dwg_Handle_code_get, _LibreDWG.Dwg_Handle_code_set)
    size = property(_LibreDWG.Dwg_Handle_size_get, _LibreDWG.Dwg_Handle_size_set)
    value = property(_LibreDWG.Dwg_Handle_value_get, _LibreDWG.Dwg_Handle_value_set)
    is_global = property(_LibreDWG.Dwg_Handle_is_global_get, _LibreDWG.Dwg_Handle_is_global_set)

    def __init__(self):
        _LibreDWG.Dwg_Handle_swiginit(self, _LibreDWG.new_Dwg_Handle())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Handle

# Register Dwg_Handle in _LibreDWG:
_LibreDWG.Dwg_Handle_swigregister(Dwg_Handle)
class Dwg_Object_Ref(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    obj = property(_LibreDWG.Dwg_Object_Ref_obj_get, _LibreDWG.Dwg_Object_Ref_obj_set)
    handleref = property(_LibreDWG.Dwg_Object_Ref_handleref_get, _LibreDWG.Dwg_Object_Ref_handleref_set)
    absolute_ref = property(_LibreDWG.Dwg_Object_Ref_absolute_ref_get, _LibreDWG.Dwg_Object_Ref_absolute_ref_set)
    r11_idx = property(_LibreDWG.Dwg_Object_Ref_r11_idx_get, _LibreDWG.Dwg_Object_Ref_r11_idx_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_Ref_swiginit(self, _LibreDWG.new_Dwg_Object_Ref())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_Ref

# Register Dwg_Object_Ref in _LibreDWG:
_LibreDWG.Dwg_Object_Ref_swigregister(Dwg_Object_Ref)
DWG_HDL_OWNER = _LibreDWG.DWG_HDL_OWNER
DWG_HDL_SOFTOWN = _LibreDWG.DWG_HDL_SOFTOWN
DWG_HDL_HARDOWN = _LibreDWG.DWG_HDL_HARDOWN
DWG_HDL_SOFTPTR = _LibreDWG.DWG_HDL_SOFTPTR
DWG_HDL_HARDPTR = _LibreDWG.DWG_HDL_HARDPTR
DWG_COLOR_METHOD_VOID = _LibreDWG.DWG_COLOR_METHOD_VOID
DWG_COLOR_METHOD_BYLAYER = _LibreDWG.DWG_COLOR_METHOD_BYLAYER
DWG_COLOR_METHOD_BYBLOCK = _LibreDWG.DWG_COLOR_METHOD_BYBLOCK
DWG_COLOR_METHOD_ACI = _LibreDWG.DWG_COLOR_METHOD_ACI
DWG_COLOR_METHOD_TRUECOLOR = _LibreDWG.DWG_COLOR_METHOD_TRUECOLOR
DWG_COLOR_METHOD_FGCOLOR = _LibreDWG.DWG_COLOR_METHOD_FGCOLOR
DWG_COLOR_METHOD_NONE = _LibreDWG.DWG_COLOR_METHOD_NONE
class Dwg_Color(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    index = property(_LibreDWG.Dwg_Color_index_get, _LibreDWG.Dwg_Color_index_set)
    flag = property(_LibreDWG.Dwg_Color_flag_get, _LibreDWG.Dwg_Color_flag_set)
    raw = property(_LibreDWG.Dwg_Color_raw_get, _LibreDWG.Dwg_Color_raw_set)
    rgb = property(_LibreDWG.Dwg_Color_rgb_get, _LibreDWG.Dwg_Color_rgb_set)
    method = property(_LibreDWG.Dwg_Color_method_get, _LibreDWG.Dwg_Color_method_set)
    name = property(_LibreDWG.Dwg_Color_name_get, _LibreDWG.Dwg_Color_name_set)
    book_name = property(_LibreDWG.Dwg_Color_book_name_get, _LibreDWG.Dwg_Color_book_name_set)
    handle = property(_LibreDWG.Dwg_Color_handle_get, _LibreDWG.Dwg_Color_handle_set)
    alpha_raw = property(_LibreDWG.Dwg_Color_alpha_raw_get, _LibreDWG.Dwg_Color_alpha_raw_set)
    alpha_type = property(_LibreDWG.Dwg_Color_alpha_type_get, _LibreDWG.Dwg_Color_alpha_type_set)
    alpha = property(_LibreDWG.Dwg_Color_alpha_get, _LibreDWG.Dwg_Color_alpha_set)

    def __init__(self):
        _LibreDWG.Dwg_Color_swiginit(self, _LibreDWG.new_Dwg_Color())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Color

# Register Dwg_Color in _LibreDWG:
_LibreDWG.Dwg_Color_swigregister(Dwg_Color)

def dwg_color_method_name(method):
    return _LibreDWG.dwg_color_method_name(method)
class _dwg_binary_chunk(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    size = property(_LibreDWG._dwg_binary_chunk_size_get, _LibreDWG._dwg_binary_chunk_size_set)
    codepage = property(_LibreDWG._dwg_binary_chunk_codepage_get, _LibreDWG._dwg_binary_chunk_codepage_set)
    is_tu = property(_LibreDWG._dwg_binary_chunk_is_tu_get, _LibreDWG._dwg_binary_chunk_is_tu_set)
    u = property(_LibreDWG._dwg_binary_chunk_u_get)

    def __init__(self):
        _LibreDWG._dwg_binary_chunk_swiginit(self, _LibreDWG.new__dwg_binary_chunk())
    __swig_destroy__ = _LibreDWG.delete__dwg_binary_chunk

# Register _dwg_binary_chunk in _LibreDWG:
_LibreDWG._dwg_binary_chunk_swigregister(_dwg_binary_chunk)
class _dwg_binary_chunk_u(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    data = property(_LibreDWG._dwg_binary_chunk_u_data_get, _LibreDWG._dwg_binary_chunk_u_data_set)
    wdata = property(_LibreDWG._dwg_binary_chunk_u_wdata_get, _LibreDWG._dwg_binary_chunk_u_wdata_set)

    def __init__(self):
        _LibreDWG._dwg_binary_chunk_u_swiginit(self, _LibreDWG.new__dwg_binary_chunk_u())
    __swig_destroy__ = _LibreDWG.delete__dwg_binary_chunk_u

# Register _dwg_binary_chunk_u in _LibreDWG:
_LibreDWG._dwg_binary_chunk_u_swigregister(_dwg_binary_chunk_u)
class Dwg_Resbuf(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    type = property(_LibreDWG.Dwg_Resbuf_type_get, _LibreDWG.Dwg_Resbuf_type_set)
    value = property(_LibreDWG.Dwg_Resbuf_value_get)
    nextrb = property(_LibreDWG.Dwg_Resbuf_nextrb_get, _LibreDWG.Dwg_Resbuf_nextrb_set)

    def __init__(self):
        _LibreDWG.Dwg_Resbuf_swiginit(self, _LibreDWG.new_Dwg_Resbuf())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Resbuf

# Register Dwg_Resbuf in _LibreDWG:
_LibreDWG.Dwg_Resbuf_swigregister(Dwg_Resbuf)
class _dwg_resbuf_value(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    pt = property(_LibreDWG._dwg_resbuf_value_pt_get, _LibreDWG._dwg_resbuf_value_pt_set)
    i8 = property(_LibreDWG._dwg_resbuf_value_i8_get, _LibreDWG._dwg_resbuf_value_i8_set)
    i16 = property(_LibreDWG._dwg_resbuf_value_i16_get, _LibreDWG._dwg_resbuf_value_i16_set)
    i32 = property(_LibreDWG._dwg_resbuf_value_i32_get, _LibreDWG._dwg_resbuf_value_i32_set)
    i64 = property(_LibreDWG._dwg_resbuf_value_i64_get, _LibreDWG._dwg_resbuf_value_i64_set)
    absref = property(_LibreDWG._dwg_resbuf_value_absref_get, _LibreDWG._dwg_resbuf_value_absref_set)
    dbl = property(_LibreDWG._dwg_resbuf_value_dbl_get, _LibreDWG._dwg_resbuf_value_dbl_set)
    str = property(_LibreDWG._dwg_resbuf_value_str_get, _LibreDWG._dwg_resbuf_value_str_set)

    def __init__(self):
        _LibreDWG._dwg_resbuf_value_swiginit(self, _LibreDWG.new__dwg_resbuf_value())
    __swig_destroy__ = _LibreDWG.delete__dwg_resbuf_value

# Register _dwg_resbuf_value in _LibreDWG:
_LibreDWG._dwg_resbuf_value_swigregister(_dwg_resbuf_value)
class Dwg_Header_Variables(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    size = property(_LibreDWG.Dwg_Header_Variables_size_get, _LibreDWG.Dwg_Header_Variables_size_set)
    bitsize_hi = property(_LibreDWG.Dwg_Header_Variables_bitsize_hi_get, _LibreDWG.Dwg_Header_Variables_bitsize_hi_set)
    bitsize = property(_LibreDWG.Dwg_Header_Variables_bitsize_get, _LibreDWG.Dwg_Header_Variables_bitsize_set)
    ACADMAINTVER = property(_LibreDWG.Dwg_Header_Variables_ACADMAINTVER_get, _LibreDWG.Dwg_Header_Variables_ACADMAINTVER_set)
    REQUIREDVERSIONS = property(_LibreDWG.Dwg_Header_Variables_REQUIREDVERSIONS_get, _LibreDWG.Dwg_Header_Variables_REQUIREDVERSIONS_set)
    DWGCODEPAGE = property(_LibreDWG.Dwg_Header_Variables_DWGCODEPAGE_get, _LibreDWG.Dwg_Header_Variables_DWGCODEPAGE_set)
    codepage = property(_LibreDWG.Dwg_Header_Variables_codepage_get, _LibreDWG.Dwg_Header_Variables_codepage_set)
    unit1_ratio = property(_LibreDWG.Dwg_Header_Variables_unit1_ratio_get, _LibreDWG.Dwg_Header_Variables_unit1_ratio_set)
    unit2_ratio = property(_LibreDWG.Dwg_Header_Variables_unit2_ratio_get, _LibreDWG.Dwg_Header_Variables_unit2_ratio_set)
    unit3_ratio = property(_LibreDWG.Dwg_Header_Variables_unit3_ratio_get, _LibreDWG.Dwg_Header_Variables_unit3_ratio_set)
    unit4_ratio = property(_LibreDWG.Dwg_Header_Variables_unit4_ratio_get, _LibreDWG.Dwg_Header_Variables_unit4_ratio_set)
    unit1_name = property(_LibreDWG.Dwg_Header_Variables_unit1_name_get, _LibreDWG.Dwg_Header_Variables_unit1_name_set)
    unit2_name = property(_LibreDWG.Dwg_Header_Variables_unit2_name_get, _LibreDWG.Dwg_Header_Variables_unit2_name_set)
    unit3_name = property(_LibreDWG.Dwg_Header_Variables_unit3_name_get, _LibreDWG.Dwg_Header_Variables_unit3_name_set)
    unit4_name = property(_LibreDWG.Dwg_Header_Variables_unit4_name_get, _LibreDWG.Dwg_Header_Variables_unit4_name_set)
    unknown_8 = property(_LibreDWG.Dwg_Header_Variables_unknown_8_get, _LibreDWG.Dwg_Header_Variables_unknown_8_set)
    unknown_9 = property(_LibreDWG.Dwg_Header_Variables_unknown_9_get, _LibreDWG.Dwg_Header_Variables_unknown_9_set)
    unknown_10 = property(_LibreDWG.Dwg_Header_Variables_unknown_10_get, _LibreDWG.Dwg_Header_Variables_unknown_10_set)
    VX_TABLE_RECORD = property(_LibreDWG.Dwg_Header_Variables_VX_TABLE_RECORD_get, _LibreDWG.Dwg_Header_Variables_VX_TABLE_RECORD_set)
    DIMASO = property(_LibreDWG.Dwg_Header_Variables_DIMASO_get, _LibreDWG.Dwg_Header_Variables_DIMASO_set)
    DIMSHO = property(_LibreDWG.Dwg_Header_Variables_DIMSHO_get, _LibreDWG.Dwg_Header_Variables_DIMSHO_set)
    DIMSAV = property(_LibreDWG.Dwg_Header_Variables_DIMSAV_get, _LibreDWG.Dwg_Header_Variables_DIMSAV_set)
    PLINEGEN = property(_LibreDWG.Dwg_Header_Variables_PLINEGEN_get, _LibreDWG.Dwg_Header_Variables_PLINEGEN_set)
    ORTHOMODE = property(_LibreDWG.Dwg_Header_Variables_ORTHOMODE_get, _LibreDWG.Dwg_Header_Variables_ORTHOMODE_set)
    REGENMODE = property(_LibreDWG.Dwg_Header_Variables_REGENMODE_get, _LibreDWG.Dwg_Header_Variables_REGENMODE_set)
    FILLMODE = property(_LibreDWG.Dwg_Header_Variables_FILLMODE_get, _LibreDWG.Dwg_Header_Variables_FILLMODE_set)
    QTEXTMODE = property(_LibreDWG.Dwg_Header_Variables_QTEXTMODE_get, _LibreDWG.Dwg_Header_Variables_QTEXTMODE_set)
    PSLTSCALE = property(_LibreDWG.Dwg_Header_Variables_PSLTSCALE_get, _LibreDWG.Dwg_Header_Variables_PSLTSCALE_set)
    LIMCHECK = property(_LibreDWG.Dwg_Header_Variables_LIMCHECK_get, _LibreDWG.Dwg_Header_Variables_LIMCHECK_set)
    MENUEXT = property(_LibreDWG.Dwg_Header_Variables_MENUEXT_get, _LibreDWG.Dwg_Header_Variables_MENUEXT_set)
    BLIPMODE = property(_LibreDWG.Dwg_Header_Variables_BLIPMODE_get, _LibreDWG.Dwg_Header_Variables_BLIPMODE_set)
    unknown_11 = property(_LibreDWG.Dwg_Header_Variables_unknown_11_get, _LibreDWG.Dwg_Header_Variables_unknown_11_set)
    USRTIMER = property(_LibreDWG.Dwg_Header_Variables_USRTIMER_get, _LibreDWG.Dwg_Header_Variables_USRTIMER_set)
    FASTZOOM = property(_LibreDWG.Dwg_Header_Variables_FASTZOOM_get, _LibreDWG.Dwg_Header_Variables_FASTZOOM_set)
    FLATLAND = property(_LibreDWG.Dwg_Header_Variables_FLATLAND_get, _LibreDWG.Dwg_Header_Variables_FLATLAND_set)
    VIEWMODE = property(_LibreDWG.Dwg_Header_Variables_VIEWMODE_get, _LibreDWG.Dwg_Header_Variables_VIEWMODE_set)
    SKPOLY = property(_LibreDWG.Dwg_Header_Variables_SKPOLY_get, _LibreDWG.Dwg_Header_Variables_SKPOLY_set)
    unknown_mon = property(_LibreDWG.Dwg_Header_Variables_unknown_mon_get, _LibreDWG.Dwg_Header_Variables_unknown_mon_set)
    unknown_day = property(_LibreDWG.Dwg_Header_Variables_unknown_day_get, _LibreDWG.Dwg_Header_Variables_unknown_day_set)
    unknown_year = property(_LibreDWG.Dwg_Header_Variables_unknown_year_get, _LibreDWG.Dwg_Header_Variables_unknown_year_set)
    unknown_hour = property(_LibreDWG.Dwg_Header_Variables_unknown_hour_get, _LibreDWG.Dwg_Header_Variables_unknown_hour_set)
    unknown_min = property(_LibreDWG.Dwg_Header_Variables_unknown_min_get, _LibreDWG.Dwg_Header_Variables_unknown_min_set)
    unknown_sec = property(_LibreDWG.Dwg_Header_Variables_unknown_sec_get, _LibreDWG.Dwg_Header_Variables_unknown_sec_set)
    unknown_ms = property(_LibreDWG.Dwg_Header_Variables_unknown_ms_get, _LibreDWG.Dwg_Header_Variables_unknown_ms_set)
    ANGDIR = property(_LibreDWG.Dwg_Header_Variables_ANGDIR_get, _LibreDWG.Dwg_Header_Variables_ANGDIR_set)
    SPLFRAME = property(_LibreDWG.Dwg_Header_Variables_SPLFRAME_get, _LibreDWG.Dwg_Header_Variables_SPLFRAME_set)
    ATTREQ = property(_LibreDWG.Dwg_Header_Variables_ATTREQ_get, _LibreDWG.Dwg_Header_Variables_ATTREQ_set)
    ATTDIA = property(_LibreDWG.Dwg_Header_Variables_ATTDIA_get, _LibreDWG.Dwg_Header_Variables_ATTDIA_set)
    MIRRTEXT = property(_LibreDWG.Dwg_Header_Variables_MIRRTEXT_get, _LibreDWG.Dwg_Header_Variables_MIRRTEXT_set)
    WORLDVIEW = property(_LibreDWG.Dwg_Header_Variables_WORLDVIEW_get, _LibreDWG.Dwg_Header_Variables_WORLDVIEW_set)
    WIREFRAME = property(_LibreDWG.Dwg_Header_Variables_WIREFRAME_get, _LibreDWG.Dwg_Header_Variables_WIREFRAME_set)
    TILEMODE = property(_LibreDWG.Dwg_Header_Variables_TILEMODE_get, _LibreDWG.Dwg_Header_Variables_TILEMODE_set)
    PLIMCHECK = property(_LibreDWG.Dwg_Header_Variables_PLIMCHECK_get, _LibreDWG.Dwg_Header_Variables_PLIMCHECK_set)
    VISRETAIN = property(_LibreDWG.Dwg_Header_Variables_VISRETAIN_get, _LibreDWG.Dwg_Header_Variables_VISRETAIN_set)
    DELOBJ = property(_LibreDWG.Dwg_Header_Variables_DELOBJ_get, _LibreDWG.Dwg_Header_Variables_DELOBJ_set)
    DISPSILH = property(_LibreDWG.Dwg_Header_Variables_DISPSILH_get, _LibreDWG.Dwg_Header_Variables_DISPSILH_set)
    PELLIPSE = property(_LibreDWG.Dwg_Header_Variables_PELLIPSE_get, _LibreDWG.Dwg_Header_Variables_PELLIPSE_set)
    SAVEIMAGES = property(_LibreDWG.Dwg_Header_Variables_SAVEIMAGES_get, _LibreDWG.Dwg_Header_Variables_SAVEIMAGES_set)
    PROXYGRAPHICS = property(_LibreDWG.Dwg_Header_Variables_PROXYGRAPHICS_get, _LibreDWG.Dwg_Header_Variables_PROXYGRAPHICS_set)
    MEASUREMENT = property(_LibreDWG.Dwg_Header_Variables_MEASUREMENT_get, _LibreDWG.Dwg_Header_Variables_MEASUREMENT_set)
    DRAGMODE = property(_LibreDWG.Dwg_Header_Variables_DRAGMODE_get, _LibreDWG.Dwg_Header_Variables_DRAGMODE_set)
    TREEDEPTH = property(_LibreDWG.Dwg_Header_Variables_TREEDEPTH_get, _LibreDWG.Dwg_Header_Variables_TREEDEPTH_set)
    LUNITS = property(_LibreDWG.Dwg_Header_Variables_LUNITS_get, _LibreDWG.Dwg_Header_Variables_LUNITS_set)
    LUPREC = property(_LibreDWG.Dwg_Header_Variables_LUPREC_get, _LibreDWG.Dwg_Header_Variables_LUPREC_set)
    AUNITS = property(_LibreDWG.Dwg_Header_Variables_AUNITS_get, _LibreDWG.Dwg_Header_Variables_AUNITS_set)
    AUPREC = property(_LibreDWG.Dwg_Header_Variables_AUPREC_get, _LibreDWG.Dwg_Header_Variables_AUPREC_set)
    ATTMODE = property(_LibreDWG.Dwg_Header_Variables_ATTMODE_get, _LibreDWG.Dwg_Header_Variables_ATTMODE_set)
    COORDS = property(_LibreDWG.Dwg_Header_Variables_COORDS_get, _LibreDWG.Dwg_Header_Variables_COORDS_set)
    PDMODE = property(_LibreDWG.Dwg_Header_Variables_PDMODE_get, _LibreDWG.Dwg_Header_Variables_PDMODE_set)
    PICKSTYLE = property(_LibreDWG.Dwg_Header_Variables_PICKSTYLE_get, _LibreDWG.Dwg_Header_Variables_PICKSTYLE_set)
    OSMODE = property(_LibreDWG.Dwg_Header_Variables_OSMODE_get, _LibreDWG.Dwg_Header_Variables_OSMODE_set)
    unknown_12 = property(_LibreDWG.Dwg_Header_Variables_unknown_12_get, _LibreDWG.Dwg_Header_Variables_unknown_12_set)
    unknown_13 = property(_LibreDWG.Dwg_Header_Variables_unknown_13_get, _LibreDWG.Dwg_Header_Variables_unknown_13_set)
    unknown_14 = property(_LibreDWG.Dwg_Header_Variables_unknown_14_get, _LibreDWG.Dwg_Header_Variables_unknown_14_set)
    USERI1 = property(_LibreDWG.Dwg_Header_Variables_USERI1_get, _LibreDWG.Dwg_Header_Variables_USERI1_set)
    USERI2 = property(_LibreDWG.Dwg_Header_Variables_USERI2_get, _LibreDWG.Dwg_Header_Variables_USERI2_set)
    USERI3 = property(_LibreDWG.Dwg_Header_Variables_USERI3_get, _LibreDWG.Dwg_Header_Variables_USERI3_set)
    USERI4 = property(_LibreDWG.Dwg_Header_Variables_USERI4_get, _LibreDWG.Dwg_Header_Variables_USERI4_set)
    USERI5 = property(_LibreDWG.Dwg_Header_Variables_USERI5_get, _LibreDWG.Dwg_Header_Variables_USERI5_set)
    SPLINESEGS = property(_LibreDWG.Dwg_Header_Variables_SPLINESEGS_get, _LibreDWG.Dwg_Header_Variables_SPLINESEGS_set)
    SURFU = property(_LibreDWG.Dwg_Header_Variables_SURFU_get, _LibreDWG.Dwg_Header_Variables_SURFU_set)
    SURFV = property(_LibreDWG.Dwg_Header_Variables_SURFV_get, _LibreDWG.Dwg_Header_Variables_SURFV_set)
    SURFTYPE = property(_LibreDWG.Dwg_Header_Variables_SURFTYPE_get, _LibreDWG.Dwg_Header_Variables_SURFTYPE_set)
    SURFTAB1 = property(_LibreDWG.Dwg_Header_Variables_SURFTAB1_get, _LibreDWG.Dwg_Header_Variables_SURFTAB1_set)
    SURFTAB2 = property(_LibreDWG.Dwg_Header_Variables_SURFTAB2_get, _LibreDWG.Dwg_Header_Variables_SURFTAB2_set)
    SPLINETYPE = property(_LibreDWG.Dwg_Header_Variables_SPLINETYPE_get, _LibreDWG.Dwg_Header_Variables_SPLINETYPE_set)
    SHADEDGE = property(_LibreDWG.Dwg_Header_Variables_SHADEDGE_get, _LibreDWG.Dwg_Header_Variables_SHADEDGE_set)
    SHADEDIF = property(_LibreDWG.Dwg_Header_Variables_SHADEDIF_get, _LibreDWG.Dwg_Header_Variables_SHADEDIF_set)
    UNITMODE = property(_LibreDWG.Dwg_Header_Variables_UNITMODE_get, _LibreDWG.Dwg_Header_Variables_UNITMODE_set)
    MAXACTVP = property(_LibreDWG.Dwg_Header_Variables_MAXACTVP_get, _LibreDWG.Dwg_Header_Variables_MAXACTVP_set)
    ISOLINES = property(_LibreDWG.Dwg_Header_Variables_ISOLINES_get, _LibreDWG.Dwg_Header_Variables_ISOLINES_set)
    CMLJUST = property(_LibreDWG.Dwg_Header_Variables_CMLJUST_get, _LibreDWG.Dwg_Header_Variables_CMLJUST_set)
    TEXTQLTY = property(_LibreDWG.Dwg_Header_Variables_TEXTQLTY_get, _LibreDWG.Dwg_Header_Variables_TEXTQLTY_set)
    unknown_14b = property(_LibreDWG.Dwg_Header_Variables_unknown_14b_get, _LibreDWG.Dwg_Header_Variables_unknown_14b_set)
    LTSCALE = property(_LibreDWG.Dwg_Header_Variables_LTSCALE_get, _LibreDWG.Dwg_Header_Variables_LTSCALE_set)
    TEXTSIZE = property(_LibreDWG.Dwg_Header_Variables_TEXTSIZE_get, _LibreDWG.Dwg_Header_Variables_TEXTSIZE_set)
    TRACEWID = property(_LibreDWG.Dwg_Header_Variables_TRACEWID_get, _LibreDWG.Dwg_Header_Variables_TRACEWID_set)
    SKETCHINC = property(_LibreDWG.Dwg_Header_Variables_SKETCHINC_get, _LibreDWG.Dwg_Header_Variables_SKETCHINC_set)
    FILLETRAD = property(_LibreDWG.Dwg_Header_Variables_FILLETRAD_get, _LibreDWG.Dwg_Header_Variables_FILLETRAD_set)
    THICKNESS = property(_LibreDWG.Dwg_Header_Variables_THICKNESS_get, _LibreDWG.Dwg_Header_Variables_THICKNESS_set)
    ANGBASE = property(_LibreDWG.Dwg_Header_Variables_ANGBASE_get, _LibreDWG.Dwg_Header_Variables_ANGBASE_set)
    PDSIZE = property(_LibreDWG.Dwg_Header_Variables_PDSIZE_get, _LibreDWG.Dwg_Header_Variables_PDSIZE_set)
    PLINEWID = property(_LibreDWG.Dwg_Header_Variables_PLINEWID_get, _LibreDWG.Dwg_Header_Variables_PLINEWID_set)
    USERR1 = property(_LibreDWG.Dwg_Header_Variables_USERR1_get, _LibreDWG.Dwg_Header_Variables_USERR1_set)
    USERR2 = property(_LibreDWG.Dwg_Header_Variables_USERR2_get, _LibreDWG.Dwg_Header_Variables_USERR2_set)
    USERR3 = property(_LibreDWG.Dwg_Header_Variables_USERR3_get, _LibreDWG.Dwg_Header_Variables_USERR3_set)
    USERR4 = property(_LibreDWG.Dwg_Header_Variables_USERR4_get, _LibreDWG.Dwg_Header_Variables_USERR4_set)
    USERR5 = property(_LibreDWG.Dwg_Header_Variables_USERR5_get, _LibreDWG.Dwg_Header_Variables_USERR5_set)
    CHAMFERA = property(_LibreDWG.Dwg_Header_Variables_CHAMFERA_get, _LibreDWG.Dwg_Header_Variables_CHAMFERA_set)
    CHAMFERB = property(_LibreDWG.Dwg_Header_Variables_CHAMFERB_get, _LibreDWG.Dwg_Header_Variables_CHAMFERB_set)
    CHAMFERC = property(_LibreDWG.Dwg_Header_Variables_CHAMFERC_get, _LibreDWG.Dwg_Header_Variables_CHAMFERC_set)
    CHAMFERD = property(_LibreDWG.Dwg_Header_Variables_CHAMFERD_get, _LibreDWG.Dwg_Header_Variables_CHAMFERD_set)
    FACETRES = property(_LibreDWG.Dwg_Header_Variables_FACETRES_get, _LibreDWG.Dwg_Header_Variables_FACETRES_set)
    CMLSCALE = property(_LibreDWG.Dwg_Header_Variables_CMLSCALE_get, _LibreDWG.Dwg_Header_Variables_CMLSCALE_set)
    CELTSCALE = property(_LibreDWG.Dwg_Header_Variables_CELTSCALE_get, _LibreDWG.Dwg_Header_Variables_CELTSCALE_set)
    VIEWTWIST = property(_LibreDWG.Dwg_Header_Variables_VIEWTWIST_get, _LibreDWG.Dwg_Header_Variables_VIEWTWIST_set)
    MENU = property(_LibreDWG.Dwg_Header_Variables_MENU_get, _LibreDWG.Dwg_Header_Variables_MENU_set)
    TDCREATE = property(_LibreDWG.Dwg_Header_Variables_TDCREATE_get, _LibreDWG.Dwg_Header_Variables_TDCREATE_set)
    TDUPDATE = property(_LibreDWG.Dwg_Header_Variables_TDUPDATE_get, _LibreDWG.Dwg_Header_Variables_TDUPDATE_set)
    TDUCREATE = property(_LibreDWG.Dwg_Header_Variables_TDUCREATE_get, _LibreDWG.Dwg_Header_Variables_TDUCREATE_set)
    TDUUPDATE = property(_LibreDWG.Dwg_Header_Variables_TDUUPDATE_get, _LibreDWG.Dwg_Header_Variables_TDUUPDATE_set)
    unknown_15 = property(_LibreDWG.Dwg_Header_Variables_unknown_15_get, _LibreDWG.Dwg_Header_Variables_unknown_15_set)
    unknown_16 = property(_LibreDWG.Dwg_Header_Variables_unknown_16_get, _LibreDWG.Dwg_Header_Variables_unknown_16_set)
    unknown_17 = property(_LibreDWG.Dwg_Header_Variables_unknown_17_get, _LibreDWG.Dwg_Header_Variables_unknown_17_set)
    TDINDWG = property(_LibreDWG.Dwg_Header_Variables_TDINDWG_get, _LibreDWG.Dwg_Header_Variables_TDINDWG_set)
    TDUSRTIMER = property(_LibreDWG.Dwg_Header_Variables_TDUSRTIMER_get, _LibreDWG.Dwg_Header_Variables_TDUSRTIMER_set)
    CECOLOR = property(_LibreDWG.Dwg_Header_Variables_CECOLOR_get, _LibreDWG.Dwg_Header_Variables_CECOLOR_set)
    HANDLING = property(_LibreDWG.Dwg_Header_Variables_HANDLING_get, _LibreDWG.Dwg_Header_Variables_HANDLING_set)
    HANDSEED = property(_LibreDWG.Dwg_Header_Variables_HANDSEED_get, _LibreDWG.Dwg_Header_Variables_HANDSEED_set)
    unknown_4f2 = property(_LibreDWG.Dwg_Header_Variables_unknown_4f2_get, _LibreDWG.Dwg_Header_Variables_unknown_4f2_set)
    unknown_5 = property(_LibreDWG.Dwg_Header_Variables_unknown_5_get, _LibreDWG.Dwg_Header_Variables_unknown_5_set)
    unknown_6 = property(_LibreDWG.Dwg_Header_Variables_unknown_6_get, _LibreDWG.Dwg_Header_Variables_unknown_6_set)
    unknown_6a = property(_LibreDWG.Dwg_Header_Variables_unknown_6a_get, _LibreDWG.Dwg_Header_Variables_unknown_6a_set)
    unknown_6b = property(_LibreDWG.Dwg_Header_Variables_unknown_6b_get, _LibreDWG.Dwg_Header_Variables_unknown_6b_set)
    unknown_6c = property(_LibreDWG.Dwg_Header_Variables_unknown_6c_get, _LibreDWG.Dwg_Header_Variables_unknown_6c_set)
    CLAYER = property(_LibreDWG.Dwg_Header_Variables_CLAYER_get, _LibreDWG.Dwg_Header_Variables_CLAYER_set)
    TEXTSTYLE = property(_LibreDWG.Dwg_Header_Variables_TEXTSTYLE_get, _LibreDWG.Dwg_Header_Variables_TEXTSTYLE_set)
    CELTYPE = property(_LibreDWG.Dwg_Header_Variables_CELTYPE_get, _LibreDWG.Dwg_Header_Variables_CELTYPE_set)
    CMATERIAL = property(_LibreDWG.Dwg_Header_Variables_CMATERIAL_get, _LibreDWG.Dwg_Header_Variables_CMATERIAL_set)
    DIMSTYLE = property(_LibreDWG.Dwg_Header_Variables_DIMSTYLE_get, _LibreDWG.Dwg_Header_Variables_DIMSTYLE_set)
    CMLSTYLE = property(_LibreDWG.Dwg_Header_Variables_CMLSTYLE_get, _LibreDWG.Dwg_Header_Variables_CMLSTYLE_set)
    PSVPSCALE = property(_LibreDWG.Dwg_Header_Variables_PSVPSCALE_get, _LibreDWG.Dwg_Header_Variables_PSVPSCALE_set)
    PINSBASE = property(_LibreDWG.Dwg_Header_Variables_PINSBASE_get, _LibreDWG.Dwg_Header_Variables_PINSBASE_set)
    PEXTMIN = property(_LibreDWG.Dwg_Header_Variables_PEXTMIN_get, _LibreDWG.Dwg_Header_Variables_PEXTMIN_set)
    PEXTMAX = property(_LibreDWG.Dwg_Header_Variables_PEXTMAX_get, _LibreDWG.Dwg_Header_Variables_PEXTMAX_set)
    PLIMMIN = property(_LibreDWG.Dwg_Header_Variables_PLIMMIN_get, _LibreDWG.Dwg_Header_Variables_PLIMMIN_set)
    PLIMMAX = property(_LibreDWG.Dwg_Header_Variables_PLIMMAX_get, _LibreDWG.Dwg_Header_Variables_PLIMMAX_set)
    PELEVATION = property(_LibreDWG.Dwg_Header_Variables_PELEVATION_get, _LibreDWG.Dwg_Header_Variables_PELEVATION_set)
    PUCSORG = property(_LibreDWG.Dwg_Header_Variables_PUCSORG_get, _LibreDWG.Dwg_Header_Variables_PUCSORG_set)
    PUCSXDIR = property(_LibreDWG.Dwg_Header_Variables_PUCSXDIR_get, _LibreDWG.Dwg_Header_Variables_PUCSXDIR_set)
    PUCSYDIR = property(_LibreDWG.Dwg_Header_Variables_PUCSYDIR_get, _LibreDWG.Dwg_Header_Variables_PUCSYDIR_set)
    PUCSNAME = property(_LibreDWG.Dwg_Header_Variables_PUCSNAME_get, _LibreDWG.Dwg_Header_Variables_PUCSNAME_set)
    PUCSBASE = property(_LibreDWG.Dwg_Header_Variables_PUCSBASE_get, _LibreDWG.Dwg_Header_Variables_PUCSBASE_set)
    PUCSORTHOREF = property(_LibreDWG.Dwg_Header_Variables_PUCSORTHOREF_get, _LibreDWG.Dwg_Header_Variables_PUCSORTHOREF_set)
    PUCSORTHOVIEW = property(_LibreDWG.Dwg_Header_Variables_PUCSORTHOVIEW_get, _LibreDWG.Dwg_Header_Variables_PUCSORTHOVIEW_set)
    PUCSORGTOP = property(_LibreDWG.Dwg_Header_Variables_PUCSORGTOP_get, _LibreDWG.Dwg_Header_Variables_PUCSORGTOP_set)
    PUCSORGBOTTOM = property(_LibreDWG.Dwg_Header_Variables_PUCSORGBOTTOM_get, _LibreDWG.Dwg_Header_Variables_PUCSORGBOTTOM_set)
    PUCSORGLEFT = property(_LibreDWG.Dwg_Header_Variables_PUCSORGLEFT_get, _LibreDWG.Dwg_Header_Variables_PUCSORGLEFT_set)
    PUCSORGRIGHT = property(_LibreDWG.Dwg_Header_Variables_PUCSORGRIGHT_get, _LibreDWG.Dwg_Header_Variables_PUCSORGRIGHT_set)
    PUCSORGFRONT = property(_LibreDWG.Dwg_Header_Variables_PUCSORGFRONT_get, _LibreDWG.Dwg_Header_Variables_PUCSORGFRONT_set)
    PUCSORGBACK = property(_LibreDWG.Dwg_Header_Variables_PUCSORGBACK_get, _LibreDWG.Dwg_Header_Variables_PUCSORGBACK_set)
    INSBASE = property(_LibreDWG.Dwg_Header_Variables_INSBASE_get, _LibreDWG.Dwg_Header_Variables_INSBASE_set)
    EXTMIN = property(_LibreDWG.Dwg_Header_Variables_EXTMIN_get, _LibreDWG.Dwg_Header_Variables_EXTMIN_set)
    EXTMAX = property(_LibreDWG.Dwg_Header_Variables_EXTMAX_get, _LibreDWG.Dwg_Header_Variables_EXTMAX_set)
    VIEWDIR = property(_LibreDWG.Dwg_Header_Variables_VIEWDIR_get, _LibreDWG.Dwg_Header_Variables_VIEWDIR_set)
    TARGET = property(_LibreDWG.Dwg_Header_Variables_TARGET_get, _LibreDWG.Dwg_Header_Variables_TARGET_set)
    LIMMIN = property(_LibreDWG.Dwg_Header_Variables_LIMMIN_get, _LibreDWG.Dwg_Header_Variables_LIMMIN_set)
    LIMMAX = property(_LibreDWG.Dwg_Header_Variables_LIMMAX_get, _LibreDWG.Dwg_Header_Variables_LIMMAX_set)
    VIEWCTR = property(_LibreDWG.Dwg_Header_Variables_VIEWCTR_get, _LibreDWG.Dwg_Header_Variables_VIEWCTR_set)
    ELEVATION = property(_LibreDWG.Dwg_Header_Variables_ELEVATION_get, _LibreDWG.Dwg_Header_Variables_ELEVATION_set)
    VIEWSIZE = property(_LibreDWG.Dwg_Header_Variables_VIEWSIZE_get, _LibreDWG.Dwg_Header_Variables_VIEWSIZE_set)
    SNAPMODE = property(_LibreDWG.Dwg_Header_Variables_SNAPMODE_get, _LibreDWG.Dwg_Header_Variables_SNAPMODE_set)
    SNAPUNIT = property(_LibreDWG.Dwg_Header_Variables_SNAPUNIT_get, _LibreDWG.Dwg_Header_Variables_SNAPUNIT_set)
    SNAPBASE = property(_LibreDWG.Dwg_Header_Variables_SNAPBASE_get, _LibreDWG.Dwg_Header_Variables_SNAPBASE_set)
    SNAPANG = property(_LibreDWG.Dwg_Header_Variables_SNAPANG_get, _LibreDWG.Dwg_Header_Variables_SNAPANG_set)
    SNAPSTYLE = property(_LibreDWG.Dwg_Header_Variables_SNAPSTYLE_get, _LibreDWG.Dwg_Header_Variables_SNAPSTYLE_set)
    SNAPISOPAIR = property(_LibreDWG.Dwg_Header_Variables_SNAPISOPAIR_get, _LibreDWG.Dwg_Header_Variables_SNAPISOPAIR_set)
    GRIDMODE = property(_LibreDWG.Dwg_Header_Variables_GRIDMODE_get, _LibreDWG.Dwg_Header_Variables_GRIDMODE_set)
    GRIDUNIT = property(_LibreDWG.Dwg_Header_Variables_GRIDUNIT_get, _LibreDWG.Dwg_Header_Variables_GRIDUNIT_set)
    AXISMODE = property(_LibreDWG.Dwg_Header_Variables_AXISMODE_get, _LibreDWG.Dwg_Header_Variables_AXISMODE_set)
    AXISUNIT = property(_LibreDWG.Dwg_Header_Variables_AXISUNIT_get, _LibreDWG.Dwg_Header_Variables_AXISUNIT_set)
    VPOINTX = property(_LibreDWG.Dwg_Header_Variables_VPOINTX_get, _LibreDWG.Dwg_Header_Variables_VPOINTX_set)
    VPOINTY = property(_LibreDWG.Dwg_Header_Variables_VPOINTY_get, _LibreDWG.Dwg_Header_Variables_VPOINTY_set)
    VPOINTZ = property(_LibreDWG.Dwg_Header_Variables_VPOINTZ_get, _LibreDWG.Dwg_Header_Variables_VPOINTZ_set)
    VPOINTXALT = property(_LibreDWG.Dwg_Header_Variables_VPOINTXALT_get, _LibreDWG.Dwg_Header_Variables_VPOINTXALT_set)
    VPOINTYALT = property(_LibreDWG.Dwg_Header_Variables_VPOINTYALT_get, _LibreDWG.Dwg_Header_Variables_VPOINTYALT_set)
    VPOINTZALT = property(_LibreDWG.Dwg_Header_Variables_VPOINTZALT_get, _LibreDWG.Dwg_Header_Variables_VPOINTZALT_set)
    flag_3d = property(_LibreDWG.Dwg_Header_Variables_flag_3d_get, _LibreDWG.Dwg_Header_Variables_flag_3d_set)
    UCSORG = property(_LibreDWG.Dwg_Header_Variables_UCSORG_get, _LibreDWG.Dwg_Header_Variables_UCSORG_set)
    UCSXDIR = property(_LibreDWG.Dwg_Header_Variables_UCSXDIR_get, _LibreDWG.Dwg_Header_Variables_UCSXDIR_set)
    UCSYDIR = property(_LibreDWG.Dwg_Header_Variables_UCSYDIR_get, _LibreDWG.Dwg_Header_Variables_UCSYDIR_set)
    UCSNAME = property(_LibreDWG.Dwg_Header_Variables_UCSNAME_get, _LibreDWG.Dwg_Header_Variables_UCSNAME_set)
    UCSBASE = property(_LibreDWG.Dwg_Header_Variables_UCSBASE_get, _LibreDWG.Dwg_Header_Variables_UCSBASE_set)
    UCSORTHOVIEW = property(_LibreDWG.Dwg_Header_Variables_UCSORTHOVIEW_get, _LibreDWG.Dwg_Header_Variables_UCSORTHOVIEW_set)
    UCSORTHOREF = property(_LibreDWG.Dwg_Header_Variables_UCSORTHOREF_get, _LibreDWG.Dwg_Header_Variables_UCSORTHOREF_set)
    UCSORGTOP = property(_LibreDWG.Dwg_Header_Variables_UCSORGTOP_get, _LibreDWG.Dwg_Header_Variables_UCSORGTOP_set)
    UCSORGBOTTOM = property(_LibreDWG.Dwg_Header_Variables_UCSORGBOTTOM_get, _LibreDWG.Dwg_Header_Variables_UCSORGBOTTOM_set)
    UCSORGLEFT = property(_LibreDWG.Dwg_Header_Variables_UCSORGLEFT_get, _LibreDWG.Dwg_Header_Variables_UCSORGLEFT_set)
    UCSORGRIGHT = property(_LibreDWG.Dwg_Header_Variables_UCSORGRIGHT_get, _LibreDWG.Dwg_Header_Variables_UCSORGRIGHT_set)
    UCSORGFRONT = property(_LibreDWG.Dwg_Header_Variables_UCSORGFRONT_get, _LibreDWG.Dwg_Header_Variables_UCSORGFRONT_set)
    UCSORGBACK = property(_LibreDWG.Dwg_Header_Variables_UCSORGBACK_get, _LibreDWG.Dwg_Header_Variables_UCSORGBACK_set)
    DIMPOST = property(_LibreDWG.Dwg_Header_Variables_DIMPOST_get, _LibreDWG.Dwg_Header_Variables_DIMPOST_set)
    DIMAPOST = property(_LibreDWG.Dwg_Header_Variables_DIMAPOST_get, _LibreDWG.Dwg_Header_Variables_DIMAPOST_set)
    DIMTOL = property(_LibreDWG.Dwg_Header_Variables_DIMTOL_get, _LibreDWG.Dwg_Header_Variables_DIMTOL_set)
    DIMLIM = property(_LibreDWG.Dwg_Header_Variables_DIMLIM_get, _LibreDWG.Dwg_Header_Variables_DIMLIM_set)
    DIMTIH = property(_LibreDWG.Dwg_Header_Variables_DIMTIH_get, _LibreDWG.Dwg_Header_Variables_DIMTIH_set)
    DIMTOH = property(_LibreDWG.Dwg_Header_Variables_DIMTOH_get, _LibreDWG.Dwg_Header_Variables_DIMTOH_set)
    DIMSE1 = property(_LibreDWG.Dwg_Header_Variables_DIMSE1_get, _LibreDWG.Dwg_Header_Variables_DIMSE1_set)
    DIMSE2 = property(_LibreDWG.Dwg_Header_Variables_DIMSE2_get, _LibreDWG.Dwg_Header_Variables_DIMSE2_set)
    DIMALT = property(_LibreDWG.Dwg_Header_Variables_DIMALT_get, _LibreDWG.Dwg_Header_Variables_DIMALT_set)
    DIMTOFL = property(_LibreDWG.Dwg_Header_Variables_DIMTOFL_get, _LibreDWG.Dwg_Header_Variables_DIMTOFL_set)
    DIMSAH = property(_LibreDWG.Dwg_Header_Variables_DIMSAH_get, _LibreDWG.Dwg_Header_Variables_DIMSAH_set)
    DIMTIX = property(_LibreDWG.Dwg_Header_Variables_DIMTIX_get, _LibreDWG.Dwg_Header_Variables_DIMTIX_set)
    DIMSOXD = property(_LibreDWG.Dwg_Header_Variables_DIMSOXD_get, _LibreDWG.Dwg_Header_Variables_DIMSOXD_set)
    DIMALTD = property(_LibreDWG.Dwg_Header_Variables_DIMALTD_get, _LibreDWG.Dwg_Header_Variables_DIMALTD_set)
    DIMZIN = property(_LibreDWG.Dwg_Header_Variables_DIMZIN_get, _LibreDWG.Dwg_Header_Variables_DIMZIN_set)
    DIMSD1 = property(_LibreDWG.Dwg_Header_Variables_DIMSD1_get, _LibreDWG.Dwg_Header_Variables_DIMSD1_set)
    DIMSD2 = property(_LibreDWG.Dwg_Header_Variables_DIMSD2_get, _LibreDWG.Dwg_Header_Variables_DIMSD2_set)
    DIMTOLJ = property(_LibreDWG.Dwg_Header_Variables_DIMTOLJ_get, _LibreDWG.Dwg_Header_Variables_DIMTOLJ_set)
    DIMJUST = property(_LibreDWG.Dwg_Header_Variables_DIMJUST_get, _LibreDWG.Dwg_Header_Variables_DIMJUST_set)
    DIMFIT = property(_LibreDWG.Dwg_Header_Variables_DIMFIT_get, _LibreDWG.Dwg_Header_Variables_DIMFIT_set)
    DIMUPT = property(_LibreDWG.Dwg_Header_Variables_DIMUPT_get, _LibreDWG.Dwg_Header_Variables_DIMUPT_set)
    DIMTZIN = property(_LibreDWG.Dwg_Header_Variables_DIMTZIN_get, _LibreDWG.Dwg_Header_Variables_DIMTZIN_set)
    DIMTAD = property(_LibreDWG.Dwg_Header_Variables_DIMTAD_get, _LibreDWG.Dwg_Header_Variables_DIMTAD_set)
    DIMUNIT = property(_LibreDWG.Dwg_Header_Variables_DIMUNIT_get, _LibreDWG.Dwg_Header_Variables_DIMUNIT_set)
    DIMAUNIT = property(_LibreDWG.Dwg_Header_Variables_DIMAUNIT_get, _LibreDWG.Dwg_Header_Variables_DIMAUNIT_set)
    DIMDEC = property(_LibreDWG.Dwg_Header_Variables_DIMDEC_get, _LibreDWG.Dwg_Header_Variables_DIMDEC_set)
    DIMTDEC = property(_LibreDWG.Dwg_Header_Variables_DIMTDEC_get, _LibreDWG.Dwg_Header_Variables_DIMTDEC_set)
    DIMALTU = property(_LibreDWG.Dwg_Header_Variables_DIMALTU_get, _LibreDWG.Dwg_Header_Variables_DIMALTU_set)
    DIMALTTD = property(_LibreDWG.Dwg_Header_Variables_DIMALTTD_get, _LibreDWG.Dwg_Header_Variables_DIMALTTD_set)
    DIMTXSTY = property(_LibreDWG.Dwg_Header_Variables_DIMTXSTY_get, _LibreDWG.Dwg_Header_Variables_DIMTXSTY_set)
    DIMSCALE = property(_LibreDWG.Dwg_Header_Variables_DIMSCALE_get, _LibreDWG.Dwg_Header_Variables_DIMSCALE_set)
    DIMARROW = property(_LibreDWG.Dwg_Header_Variables_DIMARROW_get, _LibreDWG.Dwg_Header_Variables_DIMARROW_set)
    DIMASZ = property(_LibreDWG.Dwg_Header_Variables_DIMASZ_get, _LibreDWG.Dwg_Header_Variables_DIMASZ_set)
    DIMEXO = property(_LibreDWG.Dwg_Header_Variables_DIMEXO_get, _LibreDWG.Dwg_Header_Variables_DIMEXO_set)
    DIMDLI = property(_LibreDWG.Dwg_Header_Variables_DIMDLI_get, _LibreDWG.Dwg_Header_Variables_DIMDLI_set)
    DIMEXE = property(_LibreDWG.Dwg_Header_Variables_DIMEXE_get, _LibreDWG.Dwg_Header_Variables_DIMEXE_set)
    DIMRND = property(_LibreDWG.Dwg_Header_Variables_DIMRND_get, _LibreDWG.Dwg_Header_Variables_DIMRND_set)
    DIMDLE = property(_LibreDWG.Dwg_Header_Variables_DIMDLE_get, _LibreDWG.Dwg_Header_Variables_DIMDLE_set)
    DIMTP = property(_LibreDWG.Dwg_Header_Variables_DIMTP_get, _LibreDWG.Dwg_Header_Variables_DIMTP_set)
    DIMTM = property(_LibreDWG.Dwg_Header_Variables_DIMTM_get, _LibreDWG.Dwg_Header_Variables_DIMTM_set)
    DIMFXL = property(_LibreDWG.Dwg_Header_Variables_DIMFXL_get, _LibreDWG.Dwg_Header_Variables_DIMFXL_set)
    DIMJOGANG = property(_LibreDWG.Dwg_Header_Variables_DIMJOGANG_get, _LibreDWG.Dwg_Header_Variables_DIMJOGANG_set)
    DIMTFILL = property(_LibreDWG.Dwg_Header_Variables_DIMTFILL_get, _LibreDWG.Dwg_Header_Variables_DIMTFILL_set)
    DIMTFILLCLR = property(_LibreDWG.Dwg_Header_Variables_DIMTFILLCLR_get, _LibreDWG.Dwg_Header_Variables_DIMTFILLCLR_set)
    DIMAZIN = property(_LibreDWG.Dwg_Header_Variables_DIMAZIN_get, _LibreDWG.Dwg_Header_Variables_DIMAZIN_set)
    DIMARCSYM = property(_LibreDWG.Dwg_Header_Variables_DIMARCSYM_get, _LibreDWG.Dwg_Header_Variables_DIMARCSYM_set)
    DIMTXT = property(_LibreDWG.Dwg_Header_Variables_DIMTXT_get, _LibreDWG.Dwg_Header_Variables_DIMTXT_set)
    DIMCEN = property(_LibreDWG.Dwg_Header_Variables_DIMCEN_get, _LibreDWG.Dwg_Header_Variables_DIMCEN_set)
    DIMTSZ = property(_LibreDWG.Dwg_Header_Variables_DIMTSZ_get, _LibreDWG.Dwg_Header_Variables_DIMTSZ_set)
    DIMALTF = property(_LibreDWG.Dwg_Header_Variables_DIMALTF_get, _LibreDWG.Dwg_Header_Variables_DIMALTF_set)
    DIMLFAC = property(_LibreDWG.Dwg_Header_Variables_DIMLFAC_get, _LibreDWG.Dwg_Header_Variables_DIMLFAC_set)
    DIMTVP = property(_LibreDWG.Dwg_Header_Variables_DIMTVP_get, _LibreDWG.Dwg_Header_Variables_DIMTVP_set)
    DIMTFAC = property(_LibreDWG.Dwg_Header_Variables_DIMTFAC_get, _LibreDWG.Dwg_Header_Variables_DIMTFAC_set)
    DIMGAP = property(_LibreDWG.Dwg_Header_Variables_DIMGAP_get, _LibreDWG.Dwg_Header_Variables_DIMGAP_set)
    DIMPOST_T = property(_LibreDWG.Dwg_Header_Variables_DIMPOST_T_get, _LibreDWG.Dwg_Header_Variables_DIMPOST_T_set)
    DIMAPOST_T = property(_LibreDWG.Dwg_Header_Variables_DIMAPOST_T_get, _LibreDWG.Dwg_Header_Variables_DIMAPOST_T_set)
    DIMBLK_T = property(_LibreDWG.Dwg_Header_Variables_DIMBLK_T_get, _LibreDWG.Dwg_Header_Variables_DIMBLK_T_set)
    DIMBLK1_T = property(_LibreDWG.Dwg_Header_Variables_DIMBLK1_T_get, _LibreDWG.Dwg_Header_Variables_DIMBLK1_T_set)
    DIMBLK2_T = property(_LibreDWG.Dwg_Header_Variables_DIMBLK2_T_get, _LibreDWG.Dwg_Header_Variables_DIMBLK2_T_set)
    unknown_string = property(_LibreDWG.Dwg_Header_Variables_unknown_string_get, _LibreDWG.Dwg_Header_Variables_unknown_string_set)
    DIMALTRND = property(_LibreDWG.Dwg_Header_Variables_DIMALTRND_get, _LibreDWG.Dwg_Header_Variables_DIMALTRND_set)
    DIMCLRD_C = property(_LibreDWG.Dwg_Header_Variables_DIMCLRD_C_get, _LibreDWG.Dwg_Header_Variables_DIMCLRD_C_set)
    DIMCLRE_C = property(_LibreDWG.Dwg_Header_Variables_DIMCLRE_C_get, _LibreDWG.Dwg_Header_Variables_DIMCLRE_C_set)
    DIMCLRT_C = property(_LibreDWG.Dwg_Header_Variables_DIMCLRT_C_get, _LibreDWG.Dwg_Header_Variables_DIMCLRT_C_set)
    DIMCLRD = property(_LibreDWG.Dwg_Header_Variables_DIMCLRD_get, _LibreDWG.Dwg_Header_Variables_DIMCLRD_set)
    DIMCLRE = property(_LibreDWG.Dwg_Header_Variables_DIMCLRE_get, _LibreDWG.Dwg_Header_Variables_DIMCLRE_set)
    DIMCLRT = property(_LibreDWG.Dwg_Header_Variables_DIMCLRT_get, _LibreDWG.Dwg_Header_Variables_DIMCLRT_set)
    DIMADEC = property(_LibreDWG.Dwg_Header_Variables_DIMADEC_get, _LibreDWG.Dwg_Header_Variables_DIMADEC_set)
    DIMFRAC = property(_LibreDWG.Dwg_Header_Variables_DIMFRAC_get, _LibreDWG.Dwg_Header_Variables_DIMFRAC_set)
    DIMLUNIT = property(_LibreDWG.Dwg_Header_Variables_DIMLUNIT_get, _LibreDWG.Dwg_Header_Variables_DIMLUNIT_set)
    DIMDSEP = property(_LibreDWG.Dwg_Header_Variables_DIMDSEP_get, _LibreDWG.Dwg_Header_Variables_DIMDSEP_set)
    DIMTMOVE = property(_LibreDWG.Dwg_Header_Variables_DIMTMOVE_get, _LibreDWG.Dwg_Header_Variables_DIMTMOVE_set)
    DIMALTZ = property(_LibreDWG.Dwg_Header_Variables_DIMALTZ_get, _LibreDWG.Dwg_Header_Variables_DIMALTZ_set)
    DIMALTTZ = property(_LibreDWG.Dwg_Header_Variables_DIMALTTZ_get, _LibreDWG.Dwg_Header_Variables_DIMALTTZ_set)
    DIMATFIT = property(_LibreDWG.Dwg_Header_Variables_DIMATFIT_get, _LibreDWG.Dwg_Header_Variables_DIMATFIT_set)
    DIMFXLON = property(_LibreDWG.Dwg_Header_Variables_DIMFXLON_get, _LibreDWG.Dwg_Header_Variables_DIMFXLON_set)
    DIMTXTDIRECTION = property(_LibreDWG.Dwg_Header_Variables_DIMTXTDIRECTION_get, _LibreDWG.Dwg_Header_Variables_DIMTXTDIRECTION_set)
    DIMALTMZF = property(_LibreDWG.Dwg_Header_Variables_DIMALTMZF_get, _LibreDWG.Dwg_Header_Variables_DIMALTMZF_set)
    DIMALTMZS = property(_LibreDWG.Dwg_Header_Variables_DIMALTMZS_get, _LibreDWG.Dwg_Header_Variables_DIMALTMZS_set)
    DIMMZF = property(_LibreDWG.Dwg_Header_Variables_DIMMZF_get, _LibreDWG.Dwg_Header_Variables_DIMMZF_set)
    DIMMZS = property(_LibreDWG.Dwg_Header_Variables_DIMMZS_get, _LibreDWG.Dwg_Header_Variables_DIMMZS_set)
    DIMLDRBLK = property(_LibreDWG.Dwg_Header_Variables_DIMLDRBLK_get, _LibreDWG.Dwg_Header_Variables_DIMLDRBLK_set)
    DIMBLK = property(_LibreDWG.Dwg_Header_Variables_DIMBLK_get, _LibreDWG.Dwg_Header_Variables_DIMBLK_set)
    DIMBLK1 = property(_LibreDWG.Dwg_Header_Variables_DIMBLK1_get, _LibreDWG.Dwg_Header_Variables_DIMBLK1_set)
    DIMBLK2 = property(_LibreDWG.Dwg_Header_Variables_DIMBLK2_get, _LibreDWG.Dwg_Header_Variables_DIMBLK2_set)
    DIMLTYPE = property(_LibreDWG.Dwg_Header_Variables_DIMLTYPE_get, _LibreDWG.Dwg_Header_Variables_DIMLTYPE_set)
    DIMLTEX1 = property(_LibreDWG.Dwg_Header_Variables_DIMLTEX1_get, _LibreDWG.Dwg_Header_Variables_DIMLTEX1_set)
    DIMLTEX2 = property(_LibreDWG.Dwg_Header_Variables_DIMLTEX2_get, _LibreDWG.Dwg_Header_Variables_DIMLTEX2_set)
    DIMLWD = property(_LibreDWG.Dwg_Header_Variables_DIMLWD_get, _LibreDWG.Dwg_Header_Variables_DIMLWD_set)
    DIMLWE = property(_LibreDWG.Dwg_Header_Variables_DIMLWE_get, _LibreDWG.Dwg_Header_Variables_DIMLWE_set)
    BLOCK_CONTROL_OBJECT = property(_LibreDWG.Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_get, _LibreDWG.Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_set)
    LAYER_CONTROL_OBJECT = property(_LibreDWG.Dwg_Header_Variables_LAYER_CONTROL_OBJECT_get, _LibreDWG.Dwg_Header_Variables_LAYER_CONTROL_OBJECT_set)
    STYLE_CONTROL_OBJECT = property(_LibreDWG.Dwg_Header_Variables_STYLE_CONTROL_OBJECT_get, _LibreDWG.Dwg_Header_Variables_STYLE_CONTROL_OBJECT_set)
    LTYPE_CONTROL_OBJECT = property(_LibreDWG.Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_get, _LibreDWG.Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_set)
    VIEW_CONTROL_OBJECT = property(_LibreDWG.Dwg_Header_Variables_VIEW_CONTROL_OBJECT_get, _LibreDWG.Dwg_Header_Variables_VIEW_CONTROL_OBJECT_set)
    UCS_CONTROL_OBJECT = property(_LibreDWG.Dwg_Header_Variables_UCS_CONTROL_OBJECT_get, _LibreDWG.Dwg_Header_Variables_UCS_CONTROL_OBJECT_set)
    VPORT_CONTROL_OBJECT = property(_LibreDWG.Dwg_Header_Variables_VPORT_CONTROL_OBJECT_get, _LibreDWG.Dwg_Header_Variables_VPORT_CONTROL_OBJECT_set)
    APPID_CONTROL_OBJECT = property(_LibreDWG.Dwg_Header_Variables_APPID_CONTROL_OBJECT_get, _LibreDWG.Dwg_Header_Variables_APPID_CONTROL_OBJECT_set)
    DIMSTYLE_CONTROL_OBJECT = property(_LibreDWG.Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_get, _LibreDWG.Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_set)
    VX_CONTROL_OBJECT = property(_LibreDWG.Dwg_Header_Variables_VX_CONTROL_OBJECT_get, _LibreDWG.Dwg_Header_Variables_VX_CONTROL_OBJECT_set)
    DICTIONARY_ACAD_GROUP = property(_LibreDWG.Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_get, _LibreDWG.Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_set)
    DICTIONARY_ACAD_MLINESTYLE = property(_LibreDWG.Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_get, _LibreDWG.Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_set)
    DICTIONARY_NAMED_OBJECT = property(_LibreDWG.Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_get, _LibreDWG.Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_set)
    TSTACKALIGN = property(_LibreDWG.Dwg_Header_Variables_TSTACKALIGN_get, _LibreDWG.Dwg_Header_Variables_TSTACKALIGN_set)
    TSTACKSIZE = property(_LibreDWG.Dwg_Header_Variables_TSTACKSIZE_get, _LibreDWG.Dwg_Header_Variables_TSTACKSIZE_set)
    HYPERLINKBASE = property(_LibreDWG.Dwg_Header_Variables_HYPERLINKBASE_get, _LibreDWG.Dwg_Header_Variables_HYPERLINKBASE_set)
    STYLESHEET = property(_LibreDWG.Dwg_Header_Variables_STYLESHEET_get, _LibreDWG.Dwg_Header_Variables_STYLESHEET_set)
    DICTIONARY_LAYOUT = property(_LibreDWG.Dwg_Header_Variables_DICTIONARY_LAYOUT_get, _LibreDWG.Dwg_Header_Variables_DICTIONARY_LAYOUT_set)
    DICTIONARY_PLOTSETTINGS = property(_LibreDWG.Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_get, _LibreDWG.Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_set)
    DICTIONARY_PLOTSTYLENAME = property(_LibreDWG.Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_get, _LibreDWG.Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_set)
    DICTIONARY_MATERIAL = property(_LibreDWG.Dwg_Header_Variables_DICTIONARY_MATERIAL_get, _LibreDWG.Dwg_Header_Variables_DICTIONARY_MATERIAL_set)
    DICTIONARY_COLOR = property(_LibreDWG.Dwg_Header_Variables_DICTIONARY_COLOR_get, _LibreDWG.Dwg_Header_Variables_DICTIONARY_COLOR_set)
    DICTIONARY_VISUALSTYLE = property(_LibreDWG.Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_get, _LibreDWG.Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_set)
    DICTIONARY_LIGHTLIST = property(_LibreDWG.Dwg_Header_Variables_DICTIONARY_LIGHTLIST_get, _LibreDWG.Dwg_Header_Variables_DICTIONARY_LIGHTLIST_set)
    unknown_20 = property(_LibreDWG.Dwg_Header_Variables_unknown_20_get, _LibreDWG.Dwg_Header_Variables_unknown_20_set)
    FLAGS = property(_LibreDWG.Dwg_Header_Variables_FLAGS_get, _LibreDWG.Dwg_Header_Variables_FLAGS_set)
    CELWEIGHT = property(_LibreDWG.Dwg_Header_Variables_CELWEIGHT_get, _LibreDWG.Dwg_Header_Variables_CELWEIGHT_set)
    ENDCAPS = property(_LibreDWG.Dwg_Header_Variables_ENDCAPS_get, _LibreDWG.Dwg_Header_Variables_ENDCAPS_set)
    JOINSTYLE = property(_LibreDWG.Dwg_Header_Variables_JOINSTYLE_get, _LibreDWG.Dwg_Header_Variables_JOINSTYLE_set)
    LWDISPLAY = property(_LibreDWG.Dwg_Header_Variables_LWDISPLAY_get, _LibreDWG.Dwg_Header_Variables_LWDISPLAY_set)
    XEDIT = property(_LibreDWG.Dwg_Header_Variables_XEDIT_get, _LibreDWG.Dwg_Header_Variables_XEDIT_set)
    EXTNAMES = property(_LibreDWG.Dwg_Header_Variables_EXTNAMES_get, _LibreDWG.Dwg_Header_Variables_EXTNAMES_set)
    PSTYLEMODE = property(_LibreDWG.Dwg_Header_Variables_PSTYLEMODE_get, _LibreDWG.Dwg_Header_Variables_PSTYLEMODE_set)
    OLESTARTUP = property(_LibreDWG.Dwg_Header_Variables_OLESTARTUP_get, _LibreDWG.Dwg_Header_Variables_OLESTARTUP_set)
    INSUNITS = property(_LibreDWG.Dwg_Header_Variables_INSUNITS_get, _LibreDWG.Dwg_Header_Variables_INSUNITS_set)
    CEPSNTYPE = property(_LibreDWG.Dwg_Header_Variables_CEPSNTYPE_get, _LibreDWG.Dwg_Header_Variables_CEPSNTYPE_set)
    CPSNID = property(_LibreDWG.Dwg_Header_Variables_CPSNID_get, _LibreDWG.Dwg_Header_Variables_CPSNID_set)
    FINGERPRINTGUID = property(_LibreDWG.Dwg_Header_Variables_FINGERPRINTGUID_get, _LibreDWG.Dwg_Header_Variables_FINGERPRINTGUID_set)
    VERSIONGUID = property(_LibreDWG.Dwg_Header_Variables_VERSIONGUID_get, _LibreDWG.Dwg_Header_Variables_VERSIONGUID_set)
    SORTENTS = property(_LibreDWG.Dwg_Header_Variables_SORTENTS_get, _LibreDWG.Dwg_Header_Variables_SORTENTS_set)
    INDEXCTL = property(_LibreDWG.Dwg_Header_Variables_INDEXCTL_get, _LibreDWG.Dwg_Header_Variables_INDEXCTL_set)
    HIDETEXT = property(_LibreDWG.Dwg_Header_Variables_HIDETEXT_get, _LibreDWG.Dwg_Header_Variables_HIDETEXT_set)
    XCLIPFRAME = property(_LibreDWG.Dwg_Header_Variables_XCLIPFRAME_get, _LibreDWG.Dwg_Header_Variables_XCLIPFRAME_set)
    DIMASSOC = property(_LibreDWG.Dwg_Header_Variables_DIMASSOC_get, _LibreDWG.Dwg_Header_Variables_DIMASSOC_set)
    HALOGAP = property(_LibreDWG.Dwg_Header_Variables_HALOGAP_get, _LibreDWG.Dwg_Header_Variables_HALOGAP_set)
    OBSCOLOR = property(_LibreDWG.Dwg_Header_Variables_OBSCOLOR_get, _LibreDWG.Dwg_Header_Variables_OBSCOLOR_set)
    INTERSECTIONCOLOR = property(_LibreDWG.Dwg_Header_Variables_INTERSECTIONCOLOR_get, _LibreDWG.Dwg_Header_Variables_INTERSECTIONCOLOR_set)
    OBSLTYPE = property(_LibreDWG.Dwg_Header_Variables_OBSLTYPE_get, _LibreDWG.Dwg_Header_Variables_OBSLTYPE_set)
    INTERSECTIONDISPLAY = property(_LibreDWG.Dwg_Header_Variables_INTERSECTIONDISPLAY_get, _LibreDWG.Dwg_Header_Variables_INTERSECTIONDISPLAY_set)
    PROJECTNAME = property(_LibreDWG.Dwg_Header_Variables_PROJECTNAME_get, _LibreDWG.Dwg_Header_Variables_PROJECTNAME_set)
    BLOCK_RECORD_PSPACE = property(_LibreDWG.Dwg_Header_Variables_BLOCK_RECORD_PSPACE_get, _LibreDWG.Dwg_Header_Variables_BLOCK_RECORD_PSPACE_set)
    BLOCK_RECORD_MSPACE = property(_LibreDWG.Dwg_Header_Variables_BLOCK_RECORD_MSPACE_get, _LibreDWG.Dwg_Header_Variables_BLOCK_RECORD_MSPACE_set)
    LTYPE_BYLAYER = property(_LibreDWG.Dwg_Header_Variables_LTYPE_BYLAYER_get, _LibreDWG.Dwg_Header_Variables_LTYPE_BYLAYER_set)
    LTYPE_BYBLOCK = property(_LibreDWG.Dwg_Header_Variables_LTYPE_BYBLOCK_get, _LibreDWG.Dwg_Header_Variables_LTYPE_BYBLOCK_set)
    LTYPE_CONTINUOUS = property(_LibreDWG.Dwg_Header_Variables_LTYPE_CONTINUOUS_get, _LibreDWG.Dwg_Header_Variables_LTYPE_CONTINUOUS_set)
    CAMERADISPLAY = property(_LibreDWG.Dwg_Header_Variables_CAMERADISPLAY_get, _LibreDWG.Dwg_Header_Variables_CAMERADISPLAY_set)
    unknown_21 = property(_LibreDWG.Dwg_Header_Variables_unknown_21_get, _LibreDWG.Dwg_Header_Variables_unknown_21_set)
    unknown_22 = property(_LibreDWG.Dwg_Header_Variables_unknown_22_get, _LibreDWG.Dwg_Header_Variables_unknown_22_set)
    unknown_23 = property(_LibreDWG.Dwg_Header_Variables_unknown_23_get, _LibreDWG.Dwg_Header_Variables_unknown_23_set)
    STEPSPERSEC = property(_LibreDWG.Dwg_Header_Variables_STEPSPERSEC_get, _LibreDWG.Dwg_Header_Variables_STEPSPERSEC_set)
    STEPSIZE = property(_LibreDWG.Dwg_Header_Variables_STEPSIZE_get, _LibreDWG.Dwg_Header_Variables_STEPSIZE_set)
    _3DDWFPREC = property(_LibreDWG.Dwg_Header_Variables__3DDWFPREC_get, _LibreDWG.Dwg_Header_Variables__3DDWFPREC_set)
    LENSLENGTH = property(_LibreDWG.Dwg_Header_Variables_LENSLENGTH_get, _LibreDWG.Dwg_Header_Variables_LENSLENGTH_set)
    CAMERAHEIGHT = property(_LibreDWG.Dwg_Header_Variables_CAMERAHEIGHT_get, _LibreDWG.Dwg_Header_Variables_CAMERAHEIGHT_set)
    SOLIDHIST = property(_LibreDWG.Dwg_Header_Variables_SOLIDHIST_get, _LibreDWG.Dwg_Header_Variables_SOLIDHIST_set)
    SHOWHIST = property(_LibreDWG.Dwg_Header_Variables_SHOWHIST_get, _LibreDWG.Dwg_Header_Variables_SHOWHIST_set)
    PSOLWIDTH = property(_LibreDWG.Dwg_Header_Variables_PSOLWIDTH_get, _LibreDWG.Dwg_Header_Variables_PSOLWIDTH_set)
    PSOLHEIGHT = property(_LibreDWG.Dwg_Header_Variables_PSOLHEIGHT_get, _LibreDWG.Dwg_Header_Variables_PSOLHEIGHT_set)
    LOFTANG1 = property(_LibreDWG.Dwg_Header_Variables_LOFTANG1_get, _LibreDWG.Dwg_Header_Variables_LOFTANG1_set)
    LOFTANG2 = property(_LibreDWG.Dwg_Header_Variables_LOFTANG2_get, _LibreDWG.Dwg_Header_Variables_LOFTANG2_set)
    LOFTMAG1 = property(_LibreDWG.Dwg_Header_Variables_LOFTMAG1_get, _LibreDWG.Dwg_Header_Variables_LOFTMAG1_set)
    LOFTMAG2 = property(_LibreDWG.Dwg_Header_Variables_LOFTMAG2_get, _LibreDWG.Dwg_Header_Variables_LOFTMAG2_set)
    LOFTPARAM = property(_LibreDWG.Dwg_Header_Variables_LOFTPARAM_get, _LibreDWG.Dwg_Header_Variables_LOFTPARAM_set)
    LOFTNORMALS = property(_LibreDWG.Dwg_Header_Variables_LOFTNORMALS_get, _LibreDWG.Dwg_Header_Variables_LOFTNORMALS_set)
    LATITUDE = property(_LibreDWG.Dwg_Header_Variables_LATITUDE_get, _LibreDWG.Dwg_Header_Variables_LATITUDE_set)
    LONGITUDE = property(_LibreDWG.Dwg_Header_Variables_LONGITUDE_get, _LibreDWG.Dwg_Header_Variables_LONGITUDE_set)
    NORTHDIRECTION = property(_LibreDWG.Dwg_Header_Variables_NORTHDIRECTION_get, _LibreDWG.Dwg_Header_Variables_NORTHDIRECTION_set)
    TIMEZONE = property(_LibreDWG.Dwg_Header_Variables_TIMEZONE_get, _LibreDWG.Dwg_Header_Variables_TIMEZONE_set)
    LIGHTGLYPHDISPLAY = property(_LibreDWG.Dwg_Header_Variables_LIGHTGLYPHDISPLAY_get, _LibreDWG.Dwg_Header_Variables_LIGHTGLYPHDISPLAY_set)
    TILEMODELIGHTSYNCH = property(_LibreDWG.Dwg_Header_Variables_TILEMODELIGHTSYNCH_get, _LibreDWG.Dwg_Header_Variables_TILEMODELIGHTSYNCH_set)
    DWFFRAME = property(_LibreDWG.Dwg_Header_Variables_DWFFRAME_get, _LibreDWG.Dwg_Header_Variables_DWFFRAME_set)
    DGNFRAME = property(_LibreDWG.Dwg_Header_Variables_DGNFRAME_get, _LibreDWG.Dwg_Header_Variables_DGNFRAME_set)
    REALWORLDSCALE = property(_LibreDWG.Dwg_Header_Variables_REALWORLDSCALE_get, _LibreDWG.Dwg_Header_Variables_REALWORLDSCALE_set)
    INTERFERECOLOR = property(_LibreDWG.Dwg_Header_Variables_INTERFERECOLOR_get, _LibreDWG.Dwg_Header_Variables_INTERFERECOLOR_set)
    INTERFEREOBJVS = property(_LibreDWG.Dwg_Header_Variables_INTERFEREOBJVS_get, _LibreDWG.Dwg_Header_Variables_INTERFEREOBJVS_set)
    INTERFEREVPVS = property(_LibreDWG.Dwg_Header_Variables_INTERFEREVPVS_get, _LibreDWG.Dwg_Header_Variables_INTERFEREVPVS_set)
    DRAGVS = property(_LibreDWG.Dwg_Header_Variables_DRAGVS_get, _LibreDWG.Dwg_Header_Variables_DRAGVS_set)
    CSHADOW = property(_LibreDWG.Dwg_Header_Variables_CSHADOW_get, _LibreDWG.Dwg_Header_Variables_CSHADOW_set)
    SHADOWPLANELOCATION = property(_LibreDWG.Dwg_Header_Variables_SHADOWPLANELOCATION_get, _LibreDWG.Dwg_Header_Variables_SHADOWPLANELOCATION_set)
    unknown_54 = property(_LibreDWG.Dwg_Header_Variables_unknown_54_get, _LibreDWG.Dwg_Header_Variables_unknown_54_set)
    unknown_55 = property(_LibreDWG.Dwg_Header_Variables_unknown_55_get, _LibreDWG.Dwg_Header_Variables_unknown_55_set)
    unknown_56 = property(_LibreDWG.Dwg_Header_Variables_unknown_56_get, _LibreDWG.Dwg_Header_Variables_unknown_56_set)
    unknown_57 = property(_LibreDWG.Dwg_Header_Variables_unknown_57_get, _LibreDWG.Dwg_Header_Variables_unknown_57_set)
    dwg_size = property(_LibreDWG.Dwg_Header_Variables_dwg_size_get, _LibreDWG.Dwg_Header_Variables_dwg_size_set)
    numentities = property(_LibreDWG.Dwg_Header_Variables_numentities_get, _LibreDWG.Dwg_Header_Variables_numentities_set)
    circle_zoom = property(_LibreDWG.Dwg_Header_Variables_circle_zoom_get, _LibreDWG.Dwg_Header_Variables_circle_zoom_set)
    FRONTZ = property(_LibreDWG.Dwg_Header_Variables_FRONTZ_get, _LibreDWG.Dwg_Header_Variables_FRONTZ_set)
    BACKZ = property(_LibreDWG.Dwg_Header_Variables_BACKZ_get, _LibreDWG.Dwg_Header_Variables_BACKZ_set)
    UCSICON = property(_LibreDWG.Dwg_Header_Variables_UCSICON_get, _LibreDWG.Dwg_Header_Variables_UCSICON_set)
    oldCECOLOR_hi = property(_LibreDWG.Dwg_Header_Variables_oldCECOLOR_hi_get, _LibreDWG.Dwg_Header_Variables_oldCECOLOR_hi_set)
    oldCECOLOR_lo = property(_LibreDWG.Dwg_Header_Variables_oldCECOLOR_lo_get, _LibreDWG.Dwg_Header_Variables_oldCECOLOR_lo_set)
    layer_colors = property(_LibreDWG.Dwg_Header_Variables_layer_colors_get, _LibreDWG.Dwg_Header_Variables_layer_colors_set)
    unknown_51e = property(_LibreDWG.Dwg_Header_Variables_unknown_51e_get, _LibreDWG.Dwg_Header_Variables_unknown_51e_set)
    unknown_520 = property(_LibreDWG.Dwg_Header_Variables_unknown_520_get, _LibreDWG.Dwg_Header_Variables_unknown_520_set)
    unknown_52c = property(_LibreDWG.Dwg_Header_Variables_unknown_52c_get, _LibreDWG.Dwg_Header_Variables_unknown_52c_set)
    unknown_52e = property(_LibreDWG.Dwg_Header_Variables_unknown_52e_get, _LibreDWG.Dwg_Header_Variables_unknown_52e_set)
    unknown_530 = property(_LibreDWG.Dwg_Header_Variables_unknown_530_get, _LibreDWG.Dwg_Header_Variables_unknown_530_set)
    unknown_59 = property(_LibreDWG.Dwg_Header_Variables_unknown_59_get, _LibreDWG.Dwg_Header_Variables_unknown_59_set)
    aspect_ratio = property(_LibreDWG.Dwg_Header_Variables_aspect_ratio_get, _LibreDWG.Dwg_Header_Variables_aspect_ratio_set)

    def __init__(self):
        _LibreDWG.Dwg_Header_Variables_swiginit(self, _LibreDWG.new_Dwg_Header_Variables())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Header_Variables

# Register Dwg_Header_Variables in _LibreDWG:
_LibreDWG.Dwg_Header_Variables_swigregister(Dwg_Header_Variables)
class Dwg_Entity_TEXT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_TEXT_parent_get, _LibreDWG.Dwg_Entity_TEXT_parent_set)
    dataflags = property(_LibreDWG.Dwg_Entity_TEXT_dataflags_get, _LibreDWG.Dwg_Entity_TEXT_dataflags_set)
    elevation = property(_LibreDWG.Dwg_Entity_TEXT_elevation_get, _LibreDWG.Dwg_Entity_TEXT_elevation_set)
    ins_pt = property(_LibreDWG.Dwg_Entity_TEXT_ins_pt_get, _LibreDWG.Dwg_Entity_TEXT_ins_pt_set)
    alignment_pt = property(_LibreDWG.Dwg_Entity_TEXT_alignment_pt_get, _LibreDWG.Dwg_Entity_TEXT_alignment_pt_set)
    extrusion = property(_LibreDWG.Dwg_Entity_TEXT_extrusion_get, _LibreDWG.Dwg_Entity_TEXT_extrusion_set)
    thickness = property(_LibreDWG.Dwg_Entity_TEXT_thickness_get, _LibreDWG.Dwg_Entity_TEXT_thickness_set)
    oblique_angle = property(_LibreDWG.Dwg_Entity_TEXT_oblique_angle_get, _LibreDWG.Dwg_Entity_TEXT_oblique_angle_set)
    rotation = property(_LibreDWG.Dwg_Entity_TEXT_rotation_get, _LibreDWG.Dwg_Entity_TEXT_rotation_set)
    height = property(_LibreDWG.Dwg_Entity_TEXT_height_get, _LibreDWG.Dwg_Entity_TEXT_height_set)
    width_factor = property(_LibreDWG.Dwg_Entity_TEXT_width_factor_get, _LibreDWG.Dwg_Entity_TEXT_width_factor_set)
    text_value = property(_LibreDWG.Dwg_Entity_TEXT_text_value_get, _LibreDWG.Dwg_Entity_TEXT_text_value_set)
    generation = property(_LibreDWG.Dwg_Entity_TEXT_generation_get, _LibreDWG.Dwg_Entity_TEXT_generation_set)
    horiz_alignment = property(_LibreDWG.Dwg_Entity_TEXT_horiz_alignment_get, _LibreDWG.Dwg_Entity_TEXT_horiz_alignment_set)
    vert_alignment = property(_LibreDWG.Dwg_Entity_TEXT_vert_alignment_get, _LibreDWG.Dwg_Entity_TEXT_vert_alignment_set)
    style = property(_LibreDWG.Dwg_Entity_TEXT_style_get, _LibreDWG.Dwg_Entity_TEXT_style_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_TEXT_swiginit(self, _LibreDWG.new_Dwg_Entity_TEXT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_TEXT

# Register Dwg_Entity_TEXT in _LibreDWG:
_LibreDWG.Dwg_Entity_TEXT_swigregister(Dwg_Entity_TEXT)
class Dwg_AcDbMTextObjectEmbedded(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    entmode = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_entmode_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_entmode_set)
    num_reactors = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_num_reactors_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_num_reactors_set)
    has_ds_data = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_has_ds_data_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_has_ds_data_set)
    color = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_color_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_color_set)
    is_xdic_missing = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_is_xdic_missing_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_is_xdic_missing_set)
    ltype_scale = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_ltype_scale_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_ltype_scale_set)
    ltype_flags = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_ltype_flags_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_ltype_flags_set)
    plotstyle_flags = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_plotstyle_flags_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_plotstyle_flags_set)
    material_flags = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_material_flags_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_material_flags_set)
    shadow_flags = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_shadow_flags_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_shadow_flags_set)
    has_full_visualstyle = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_has_full_visualstyle_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_has_full_visualstyle_set)
    has_face_visualstyle = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_has_face_visualstyle_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_has_face_visualstyle_set)
    has_edge_visualstyle = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_has_edge_visualstyle_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_has_edge_visualstyle_set)
    invisible = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_invisible_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_invisible_set)
    linewt = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_linewt_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_linewt_set)
    layer = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_layer_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_layer_set)
    ins_pt = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_ins_pt_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_ins_pt_set)
    extrusion = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_extrusion_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_extrusion_set)
    x_axis_dir = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_x_axis_dir_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_x_axis_dir_set)
    rect_height = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_rect_height_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_rect_height_set)
    rect_width = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_rect_width_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_rect_width_set)
    text_height = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_text_height_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_text_height_set)
    attachment = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_attachment_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_attachment_set)
    flow_dir = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_flow_dir_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_flow_dir_set)
    extents_width = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_extents_width_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_extents_width_set)
    extents_height = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_extents_height_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_extents_height_set)
    text = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_text_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_text_set)
    style = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_style_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_style_set)
    linespace_style = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_linespace_style_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_linespace_style_set)
    linespace_factor = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_linespace_factor_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_linespace_factor_set)
    unknown_b0 = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_unknown_b0_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_unknown_b0_set)
    bg_fill_flag = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_bg_fill_flag_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_bg_fill_flag_set)
    bg_fill_scale = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_bg_fill_scale_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_bg_fill_scale_set)
    bg_fill_color = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_bg_fill_color_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_bg_fill_color_set)
    bg_fill_trans = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_bg_fill_trans_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_bg_fill_trans_set)
    is_not_annotative = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_is_not_annotative_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_is_not_annotative_set)
    class_version = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_class_version_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_class_version_set)
    default_flag = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_default_flag_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_default_flag_set)
    appid = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_appid_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_appid_set)
    ignore_attachment = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_ignore_attachment_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_ignore_attachment_set)
    column_type = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_column_type_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_column_type_set)
    numfragments = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_numfragments_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_numfragments_set)
    column_width = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_column_width_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_column_width_set)
    gutter = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_gutter_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_gutter_set)
    auto_height = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_auto_height_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_auto_height_set)
    flow_reversed = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_flow_reversed_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_flow_reversed_set)
    num_column_heights = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_num_column_heights_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_num_column_heights_set)
    column_heights = property(_LibreDWG.Dwg_AcDbMTextObjectEmbedded_column_heights_get, _LibreDWG.Dwg_AcDbMTextObjectEmbedded_column_heights_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDbMTextObjectEmbedded_swiginit(self, _LibreDWG.new_Dwg_AcDbMTextObjectEmbedded())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDbMTextObjectEmbedded

# Register Dwg_AcDbMTextObjectEmbedded in _LibreDWG:
_LibreDWG.Dwg_AcDbMTextObjectEmbedded_swigregister(Dwg_AcDbMTextObjectEmbedded)
class Dwg_Entity_ATTRIB(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_ATTRIB_parent_get, _LibreDWG.Dwg_Entity_ATTRIB_parent_set)
    elevation = property(_LibreDWG.Dwg_Entity_ATTRIB_elevation_get, _LibreDWG.Dwg_Entity_ATTRIB_elevation_set)
    ins_pt = property(_LibreDWG.Dwg_Entity_ATTRIB_ins_pt_get, _LibreDWG.Dwg_Entity_ATTRIB_ins_pt_set)
    alignment_pt = property(_LibreDWG.Dwg_Entity_ATTRIB_alignment_pt_get, _LibreDWG.Dwg_Entity_ATTRIB_alignment_pt_set)
    extrusion = property(_LibreDWG.Dwg_Entity_ATTRIB_extrusion_get, _LibreDWG.Dwg_Entity_ATTRIB_extrusion_set)
    thickness = property(_LibreDWG.Dwg_Entity_ATTRIB_thickness_get, _LibreDWG.Dwg_Entity_ATTRIB_thickness_set)
    oblique_angle = property(_LibreDWG.Dwg_Entity_ATTRIB_oblique_angle_get, _LibreDWG.Dwg_Entity_ATTRIB_oblique_angle_set)
    rotation = property(_LibreDWG.Dwg_Entity_ATTRIB_rotation_get, _LibreDWG.Dwg_Entity_ATTRIB_rotation_set)
    height = property(_LibreDWG.Dwg_Entity_ATTRIB_height_get, _LibreDWG.Dwg_Entity_ATTRIB_height_set)
    width_factor = property(_LibreDWG.Dwg_Entity_ATTRIB_width_factor_get, _LibreDWG.Dwg_Entity_ATTRIB_width_factor_set)
    text_value = property(_LibreDWG.Dwg_Entity_ATTRIB_text_value_get, _LibreDWG.Dwg_Entity_ATTRIB_text_value_set)
    generation = property(_LibreDWG.Dwg_Entity_ATTRIB_generation_get, _LibreDWG.Dwg_Entity_ATTRIB_generation_set)
    horiz_alignment = property(_LibreDWG.Dwg_Entity_ATTRIB_horiz_alignment_get, _LibreDWG.Dwg_Entity_ATTRIB_horiz_alignment_set)
    vert_alignment = property(_LibreDWG.Dwg_Entity_ATTRIB_vert_alignment_get, _LibreDWG.Dwg_Entity_ATTRIB_vert_alignment_set)
    dataflags = property(_LibreDWG.Dwg_Entity_ATTRIB_dataflags_get, _LibreDWG.Dwg_Entity_ATTRIB_dataflags_set)
    is_locked_in_block = property(_LibreDWG.Dwg_Entity_ATTRIB_is_locked_in_block_get, _LibreDWG.Dwg_Entity_ATTRIB_is_locked_in_block_set)
    keep_duplicate_records = property(_LibreDWG.Dwg_Entity_ATTRIB_keep_duplicate_records_get, _LibreDWG.Dwg_Entity_ATTRIB_keep_duplicate_records_set)
    tag = property(_LibreDWG.Dwg_Entity_ATTRIB_tag_get, _LibreDWG.Dwg_Entity_ATTRIB_tag_set)
    field_length = property(_LibreDWG.Dwg_Entity_ATTRIB_field_length_get, _LibreDWG.Dwg_Entity_ATTRIB_field_length_set)
    flags = property(_LibreDWG.Dwg_Entity_ATTRIB_flags_get, _LibreDWG.Dwg_Entity_ATTRIB_flags_set)
    lock_position_flag = property(_LibreDWG.Dwg_Entity_ATTRIB_lock_position_flag_get, _LibreDWG.Dwg_Entity_ATTRIB_lock_position_flag_set)
    style = property(_LibreDWG.Dwg_Entity_ATTRIB_style_get, _LibreDWG.Dwg_Entity_ATTRIB_style_set)
    mtext_type = property(_LibreDWG.Dwg_Entity_ATTRIB_mtext_type_get, _LibreDWG.Dwg_Entity_ATTRIB_mtext_type_set)
    mtext = property(_LibreDWG.Dwg_Entity_ATTRIB_mtext_get, _LibreDWG.Dwg_Entity_ATTRIB_mtext_set)
    is_really_locked = property(_LibreDWG.Dwg_Entity_ATTRIB_is_really_locked_get, _LibreDWG.Dwg_Entity_ATTRIB_is_really_locked_set)
    annotative_data_size = property(_LibreDWG.Dwg_Entity_ATTRIB_annotative_data_size_get, _LibreDWG.Dwg_Entity_ATTRIB_annotative_data_size_set)
    annotative_data = property(_LibreDWG.Dwg_Entity_ATTRIB_annotative_data_get, _LibreDWG.Dwg_Entity_ATTRIB_annotative_data_set)
    annotative_flag = property(_LibreDWG.Dwg_Entity_ATTRIB_annotative_flag_get, _LibreDWG.Dwg_Entity_ATTRIB_annotative_flag_set)
    annotative_style = property(_LibreDWG.Dwg_Entity_ATTRIB_annotative_style_get, _LibreDWG.Dwg_Entity_ATTRIB_annotative_style_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_ATTRIB_swiginit(self, _LibreDWG.new_Dwg_Entity_ATTRIB())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_ATTRIB

# Register Dwg_Entity_ATTRIB in _LibreDWG:
_LibreDWG.Dwg_Entity_ATTRIB_swigregister(Dwg_Entity_ATTRIB)
class Dwg_Entity_ATTDEF(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_ATTDEF_parent_get, _LibreDWG.Dwg_Entity_ATTDEF_parent_set)
    elevation = property(_LibreDWG.Dwg_Entity_ATTDEF_elevation_get, _LibreDWG.Dwg_Entity_ATTDEF_elevation_set)
    ins_pt = property(_LibreDWG.Dwg_Entity_ATTDEF_ins_pt_get, _LibreDWG.Dwg_Entity_ATTDEF_ins_pt_set)
    alignment_pt = property(_LibreDWG.Dwg_Entity_ATTDEF_alignment_pt_get, _LibreDWG.Dwg_Entity_ATTDEF_alignment_pt_set)
    extrusion = property(_LibreDWG.Dwg_Entity_ATTDEF_extrusion_get, _LibreDWG.Dwg_Entity_ATTDEF_extrusion_set)
    thickness = property(_LibreDWG.Dwg_Entity_ATTDEF_thickness_get, _LibreDWG.Dwg_Entity_ATTDEF_thickness_set)
    oblique_angle = property(_LibreDWG.Dwg_Entity_ATTDEF_oblique_angle_get, _LibreDWG.Dwg_Entity_ATTDEF_oblique_angle_set)
    rotation = property(_LibreDWG.Dwg_Entity_ATTDEF_rotation_get, _LibreDWG.Dwg_Entity_ATTDEF_rotation_set)
    height = property(_LibreDWG.Dwg_Entity_ATTDEF_height_get, _LibreDWG.Dwg_Entity_ATTDEF_height_set)
    width_factor = property(_LibreDWG.Dwg_Entity_ATTDEF_width_factor_get, _LibreDWG.Dwg_Entity_ATTDEF_width_factor_set)
    default_value = property(_LibreDWG.Dwg_Entity_ATTDEF_default_value_get, _LibreDWG.Dwg_Entity_ATTDEF_default_value_set)
    generation = property(_LibreDWG.Dwg_Entity_ATTDEF_generation_get, _LibreDWG.Dwg_Entity_ATTDEF_generation_set)
    horiz_alignment = property(_LibreDWG.Dwg_Entity_ATTDEF_horiz_alignment_get, _LibreDWG.Dwg_Entity_ATTDEF_horiz_alignment_set)
    vert_alignment = property(_LibreDWG.Dwg_Entity_ATTDEF_vert_alignment_get, _LibreDWG.Dwg_Entity_ATTDEF_vert_alignment_set)
    dataflags = property(_LibreDWG.Dwg_Entity_ATTDEF_dataflags_get, _LibreDWG.Dwg_Entity_ATTDEF_dataflags_set)
    is_locked_in_block = property(_LibreDWG.Dwg_Entity_ATTDEF_is_locked_in_block_get, _LibreDWG.Dwg_Entity_ATTDEF_is_locked_in_block_set)
    keep_duplicate_records = property(_LibreDWG.Dwg_Entity_ATTDEF_keep_duplicate_records_get, _LibreDWG.Dwg_Entity_ATTDEF_keep_duplicate_records_set)
    tag = property(_LibreDWG.Dwg_Entity_ATTDEF_tag_get, _LibreDWG.Dwg_Entity_ATTDEF_tag_set)
    field_length = property(_LibreDWG.Dwg_Entity_ATTDEF_field_length_get, _LibreDWG.Dwg_Entity_ATTDEF_field_length_set)
    flags = property(_LibreDWG.Dwg_Entity_ATTDEF_flags_get, _LibreDWG.Dwg_Entity_ATTDEF_flags_set)
    lock_position_flag = property(_LibreDWG.Dwg_Entity_ATTDEF_lock_position_flag_get, _LibreDWG.Dwg_Entity_ATTDEF_lock_position_flag_set)
    style = property(_LibreDWG.Dwg_Entity_ATTDEF_style_get, _LibreDWG.Dwg_Entity_ATTDEF_style_set)
    mtext_type = property(_LibreDWG.Dwg_Entity_ATTDEF_mtext_type_get, _LibreDWG.Dwg_Entity_ATTDEF_mtext_type_set)
    mtext = property(_LibreDWG.Dwg_Entity_ATTDEF_mtext_get, _LibreDWG.Dwg_Entity_ATTDEF_mtext_set)
    is_really_locked = property(_LibreDWG.Dwg_Entity_ATTDEF_is_really_locked_get, _LibreDWG.Dwg_Entity_ATTDEF_is_really_locked_set)
    annotative_data_size = property(_LibreDWG.Dwg_Entity_ATTDEF_annotative_data_size_get, _LibreDWG.Dwg_Entity_ATTDEF_annotative_data_size_set)
    annotative_data = property(_LibreDWG.Dwg_Entity_ATTDEF_annotative_data_get, _LibreDWG.Dwg_Entity_ATTDEF_annotative_data_set)
    annotative_flag = property(_LibreDWG.Dwg_Entity_ATTDEF_annotative_flag_get, _LibreDWG.Dwg_Entity_ATTDEF_annotative_flag_set)
    annotative_style = property(_LibreDWG.Dwg_Entity_ATTDEF_annotative_style_get, _LibreDWG.Dwg_Entity_ATTDEF_annotative_style_set)
    prompt = property(_LibreDWG.Dwg_Entity_ATTDEF_prompt_get, _LibreDWG.Dwg_Entity_ATTDEF_prompt_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_ATTDEF_swiginit(self, _LibreDWG.new_Dwg_Entity_ATTDEF())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_ATTDEF

# Register Dwg_Entity_ATTDEF in _LibreDWG:
_LibreDWG.Dwg_Entity_ATTDEF_swigregister(Dwg_Entity_ATTDEF)
class Dwg_Entity_BLOCK(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_BLOCK_parent_get, _LibreDWG.Dwg_Entity_BLOCK_parent_set)
    name = property(_LibreDWG.Dwg_Entity_BLOCK_name_get, _LibreDWG.Dwg_Entity_BLOCK_name_set)
    xref_pname = property(_LibreDWG.Dwg_Entity_BLOCK_xref_pname_get, _LibreDWG.Dwg_Entity_BLOCK_xref_pname_set)
    base_pt = property(_LibreDWG.Dwg_Entity_BLOCK_base_pt_get, _LibreDWG.Dwg_Entity_BLOCK_base_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_BLOCK_swiginit(self, _LibreDWG.new_Dwg_Entity_BLOCK())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_BLOCK

# Register Dwg_Entity_BLOCK in _LibreDWG:
_LibreDWG.Dwg_Entity_BLOCK_swigregister(Dwg_Entity_BLOCK)
class Dwg_Entity_ENDBLK(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_ENDBLK_parent_get, _LibreDWG.Dwg_Entity_ENDBLK_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_ENDBLK_swiginit(self, _LibreDWG.new_Dwg_Entity_ENDBLK())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_ENDBLK

# Register Dwg_Entity_ENDBLK in _LibreDWG:
_LibreDWG.Dwg_Entity_ENDBLK_swigregister(Dwg_Entity_ENDBLK)
class Dwg_Entity_SEQEND(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_SEQEND_parent_get, _LibreDWG.Dwg_Entity_SEQEND_parent_set)
    begin_addr_r11 = property(_LibreDWG.Dwg_Entity_SEQEND_begin_addr_r11_get, _LibreDWG.Dwg_Entity_SEQEND_begin_addr_r11_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_SEQEND_swiginit(self, _LibreDWG.new_Dwg_Entity_SEQEND())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_SEQEND

# Register Dwg_Entity_SEQEND in _LibreDWG:
_LibreDWG.Dwg_Entity_SEQEND_swigregister(Dwg_Entity_SEQEND)
class Dwg_Entity_INSERT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_INSERT_parent_get, _LibreDWG.Dwg_Entity_INSERT_parent_set)
    ins_pt = property(_LibreDWG.Dwg_Entity_INSERT_ins_pt_get, _LibreDWG.Dwg_Entity_INSERT_ins_pt_set)
    scale_flag = property(_LibreDWG.Dwg_Entity_INSERT_scale_flag_get, _LibreDWG.Dwg_Entity_INSERT_scale_flag_set)
    scale = property(_LibreDWG.Dwg_Entity_INSERT_scale_get, _LibreDWG.Dwg_Entity_INSERT_scale_set)
    rotation = property(_LibreDWG.Dwg_Entity_INSERT_rotation_get, _LibreDWG.Dwg_Entity_INSERT_rotation_set)
    extrusion = property(_LibreDWG.Dwg_Entity_INSERT_extrusion_get, _LibreDWG.Dwg_Entity_INSERT_extrusion_set)
    has_attribs = property(_LibreDWG.Dwg_Entity_INSERT_has_attribs_get, _LibreDWG.Dwg_Entity_INSERT_has_attribs_set)
    num_owned = property(_LibreDWG.Dwg_Entity_INSERT_num_owned_get, _LibreDWG.Dwg_Entity_INSERT_num_owned_set)
    block_header = property(_LibreDWG.Dwg_Entity_INSERT_block_header_get, _LibreDWG.Dwg_Entity_INSERT_block_header_set)
    first_attrib = property(_LibreDWG.Dwg_Entity_INSERT_first_attrib_get, _LibreDWG.Dwg_Entity_INSERT_first_attrib_set)
    last_attrib = property(_LibreDWG.Dwg_Entity_INSERT_last_attrib_get, _LibreDWG.Dwg_Entity_INSERT_last_attrib_set)
    attribs = property(_LibreDWG.Dwg_Entity_INSERT_attribs_get, _LibreDWG.Dwg_Entity_INSERT_attribs_set)
    seqend = property(_LibreDWG.Dwg_Entity_INSERT_seqend_get, _LibreDWG.Dwg_Entity_INSERT_seqend_set)
    num_cols = property(_LibreDWG.Dwg_Entity_INSERT_num_cols_get, _LibreDWG.Dwg_Entity_INSERT_num_cols_set)
    num_rows = property(_LibreDWG.Dwg_Entity_INSERT_num_rows_get, _LibreDWG.Dwg_Entity_INSERT_num_rows_set)
    col_spacing = property(_LibreDWG.Dwg_Entity_INSERT_col_spacing_get, _LibreDWG.Dwg_Entity_INSERT_col_spacing_set)
    row_spacing = property(_LibreDWG.Dwg_Entity_INSERT_row_spacing_get, _LibreDWG.Dwg_Entity_INSERT_row_spacing_set)
    block_name = property(_LibreDWG.Dwg_Entity_INSERT_block_name_get, _LibreDWG.Dwg_Entity_INSERT_block_name_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_INSERT_swiginit(self, _LibreDWG.new_Dwg_Entity_INSERT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_INSERT

# Register Dwg_Entity_INSERT in _LibreDWG:
_LibreDWG.Dwg_Entity_INSERT_swigregister(Dwg_Entity_INSERT)
class Dwg_Entity_MINSERT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_MINSERT_parent_get, _LibreDWG.Dwg_Entity_MINSERT_parent_set)
    ins_pt = property(_LibreDWG.Dwg_Entity_MINSERT_ins_pt_get, _LibreDWG.Dwg_Entity_MINSERT_ins_pt_set)
    scale_flag = property(_LibreDWG.Dwg_Entity_MINSERT_scale_flag_get, _LibreDWG.Dwg_Entity_MINSERT_scale_flag_set)
    scale = property(_LibreDWG.Dwg_Entity_MINSERT_scale_get, _LibreDWG.Dwg_Entity_MINSERT_scale_set)
    rotation = property(_LibreDWG.Dwg_Entity_MINSERT_rotation_get, _LibreDWG.Dwg_Entity_MINSERT_rotation_set)
    extrusion = property(_LibreDWG.Dwg_Entity_MINSERT_extrusion_get, _LibreDWG.Dwg_Entity_MINSERT_extrusion_set)
    has_attribs = property(_LibreDWG.Dwg_Entity_MINSERT_has_attribs_get, _LibreDWG.Dwg_Entity_MINSERT_has_attribs_set)
    num_owned = property(_LibreDWG.Dwg_Entity_MINSERT_num_owned_get, _LibreDWG.Dwg_Entity_MINSERT_num_owned_set)
    num_cols = property(_LibreDWG.Dwg_Entity_MINSERT_num_cols_get, _LibreDWG.Dwg_Entity_MINSERT_num_cols_set)
    num_rows = property(_LibreDWG.Dwg_Entity_MINSERT_num_rows_get, _LibreDWG.Dwg_Entity_MINSERT_num_rows_set)
    col_spacing = property(_LibreDWG.Dwg_Entity_MINSERT_col_spacing_get, _LibreDWG.Dwg_Entity_MINSERT_col_spacing_set)
    row_spacing = property(_LibreDWG.Dwg_Entity_MINSERT_row_spacing_get, _LibreDWG.Dwg_Entity_MINSERT_row_spacing_set)
    block_header = property(_LibreDWG.Dwg_Entity_MINSERT_block_header_get, _LibreDWG.Dwg_Entity_MINSERT_block_header_set)
    first_attrib = property(_LibreDWG.Dwg_Entity_MINSERT_first_attrib_get, _LibreDWG.Dwg_Entity_MINSERT_first_attrib_set)
    last_attrib = property(_LibreDWG.Dwg_Entity_MINSERT_last_attrib_get, _LibreDWG.Dwg_Entity_MINSERT_last_attrib_set)
    attribs = property(_LibreDWG.Dwg_Entity_MINSERT_attribs_get, _LibreDWG.Dwg_Entity_MINSERT_attribs_set)
    seqend = property(_LibreDWG.Dwg_Entity_MINSERT_seqend_get, _LibreDWG.Dwg_Entity_MINSERT_seqend_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_MINSERT_swiginit(self, _LibreDWG.new_Dwg_Entity_MINSERT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_MINSERT

# Register Dwg_Entity_MINSERT in _LibreDWG:
_LibreDWG.Dwg_Entity_MINSERT_swigregister(Dwg_Entity_MINSERT)
class Dwg_Entity_VERTEX_2D(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_VERTEX_2D_parent_get, _LibreDWG.Dwg_Entity_VERTEX_2D_parent_set)
    flag = property(_LibreDWG.Dwg_Entity_VERTEX_2D_flag_get, _LibreDWG.Dwg_Entity_VERTEX_2D_flag_set)
    point = property(_LibreDWG.Dwg_Entity_VERTEX_2D_point_get, _LibreDWG.Dwg_Entity_VERTEX_2D_point_set)
    start_width = property(_LibreDWG.Dwg_Entity_VERTEX_2D_start_width_get, _LibreDWG.Dwg_Entity_VERTEX_2D_start_width_set)
    end_width = property(_LibreDWG.Dwg_Entity_VERTEX_2D_end_width_get, _LibreDWG.Dwg_Entity_VERTEX_2D_end_width_set)
    id = property(_LibreDWG.Dwg_Entity_VERTEX_2D_id_get, _LibreDWG.Dwg_Entity_VERTEX_2D_id_set)
    bulge = property(_LibreDWG.Dwg_Entity_VERTEX_2D_bulge_get, _LibreDWG.Dwg_Entity_VERTEX_2D_bulge_set)
    tangent_dir = property(_LibreDWG.Dwg_Entity_VERTEX_2D_tangent_dir_get, _LibreDWG.Dwg_Entity_VERTEX_2D_tangent_dir_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_VERTEX_2D_swiginit(self, _LibreDWG.new_Dwg_Entity_VERTEX_2D())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_VERTEX_2D

# Register Dwg_Entity_VERTEX_2D in _LibreDWG:
_LibreDWG.Dwg_Entity_VERTEX_2D_swigregister(Dwg_Entity_VERTEX_2D)
class Dwg_Entity_VERTEX_3D(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_VERTEX_3D_parent_get, _LibreDWG.Dwg_Entity_VERTEX_3D_parent_set)
    flag = property(_LibreDWG.Dwg_Entity_VERTEX_3D_flag_get, _LibreDWG.Dwg_Entity_VERTEX_3D_flag_set)
    point = property(_LibreDWG.Dwg_Entity_VERTEX_3D_point_get, _LibreDWG.Dwg_Entity_VERTEX_3D_point_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_VERTEX_3D_swiginit(self, _LibreDWG.new_Dwg_Entity_VERTEX_3D())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_VERTEX_3D

# Register Dwg_Entity_VERTEX_3D in _LibreDWG:
_LibreDWG.Dwg_Entity_VERTEX_3D_swigregister(Dwg_Entity_VERTEX_3D)
class Dwg_Entity_VERTEX_PFACE_FACE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_VERTEX_PFACE_FACE_parent_get, _LibreDWG.Dwg_Entity_VERTEX_PFACE_FACE_parent_set)
    flag = property(_LibreDWG.Dwg_Entity_VERTEX_PFACE_FACE_flag_get, _LibreDWG.Dwg_Entity_VERTEX_PFACE_FACE_flag_set)
    vertind = property(_LibreDWG.Dwg_Entity_VERTEX_PFACE_FACE_vertind_get, _LibreDWG.Dwg_Entity_VERTEX_PFACE_FACE_vertind_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_VERTEX_PFACE_FACE_swiginit(self, _LibreDWG.new_Dwg_Entity_VERTEX_PFACE_FACE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_VERTEX_PFACE_FACE

# Register Dwg_Entity_VERTEX_PFACE_FACE in _LibreDWG:
_LibreDWG.Dwg_Entity_VERTEX_PFACE_FACE_swigregister(Dwg_Entity_VERTEX_PFACE_FACE)
class Dwg_Entity_POLYLINE_2D(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_parent_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_parent_set)
    has_vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_has_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_has_vertex_set)
    num_owned = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_num_owned_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_num_owned_set)
    first_vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_first_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_first_vertex_set)
    last_vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_last_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_last_vertex_set)
    vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_vertex_set)
    seqend = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_seqend_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_seqend_set)
    flag = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_flag_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_flag_set)
    curve_type = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_curve_type_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_curve_type_set)
    start_width = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_start_width_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_start_width_set)
    end_width = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_end_width_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_end_width_set)
    thickness = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_thickness_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_thickness_set)
    elevation = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_elevation_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_elevation_set)
    extrusion = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_extrusion_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_extrusion_set)
    extra_r11_size = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_extra_r11_size_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_extra_r11_size_set)
    extra_r11_text = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_extra_r11_text_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_extra_r11_text_set)
    num_m_verts = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_num_m_verts_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_num_m_verts_set)
    num_n_verts = property(_LibreDWG.Dwg_Entity_POLYLINE_2D_num_n_verts_get, _LibreDWG.Dwg_Entity_POLYLINE_2D_num_n_verts_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_POLYLINE_2D_swiginit(self, _LibreDWG.new_Dwg_Entity_POLYLINE_2D())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_POLYLINE_2D

# Register Dwg_Entity_POLYLINE_2D in _LibreDWG:
_LibreDWG.Dwg_Entity_POLYLINE_2D_swigregister(Dwg_Entity_POLYLINE_2D)
class Dwg_Entity_POLYLINE_3D(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_POLYLINE_3D_parent_get, _LibreDWG.Dwg_Entity_POLYLINE_3D_parent_set)
    has_vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_3D_has_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_3D_has_vertex_set)
    num_owned = property(_LibreDWG.Dwg_Entity_POLYLINE_3D_num_owned_get, _LibreDWG.Dwg_Entity_POLYLINE_3D_num_owned_set)
    first_vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_3D_first_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_3D_first_vertex_set)
    last_vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_3D_last_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_3D_last_vertex_set)
    vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_3D_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_3D_vertex_set)
    seqend = property(_LibreDWG.Dwg_Entity_POLYLINE_3D_seqend_get, _LibreDWG.Dwg_Entity_POLYLINE_3D_seqend_set)
    curve_type = property(_LibreDWG.Dwg_Entity_POLYLINE_3D_curve_type_get, _LibreDWG.Dwg_Entity_POLYLINE_3D_curve_type_set)
    start_width = property(_LibreDWG.Dwg_Entity_POLYLINE_3D_start_width_get, _LibreDWG.Dwg_Entity_POLYLINE_3D_start_width_set)
    end_width = property(_LibreDWG.Dwg_Entity_POLYLINE_3D_end_width_get, _LibreDWG.Dwg_Entity_POLYLINE_3D_end_width_set)
    flag = property(_LibreDWG.Dwg_Entity_POLYLINE_3D_flag_get, _LibreDWG.Dwg_Entity_POLYLINE_3D_flag_set)
    extrusion = property(_LibreDWG.Dwg_Entity_POLYLINE_3D_extrusion_get, _LibreDWG.Dwg_Entity_POLYLINE_3D_extrusion_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_POLYLINE_3D_swiginit(self, _LibreDWG.new_Dwg_Entity_POLYLINE_3D())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_POLYLINE_3D

# Register Dwg_Entity_POLYLINE_3D in _LibreDWG:
_LibreDWG.Dwg_Entity_POLYLINE_3D_swigregister(Dwg_Entity_POLYLINE_3D)
class Dwg_Entity_ARC(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_ARC_parent_get, _LibreDWG.Dwg_Entity_ARC_parent_set)
    center = property(_LibreDWG.Dwg_Entity_ARC_center_get, _LibreDWG.Dwg_Entity_ARC_center_set)
    radius = property(_LibreDWG.Dwg_Entity_ARC_radius_get, _LibreDWG.Dwg_Entity_ARC_radius_set)
    thickness = property(_LibreDWG.Dwg_Entity_ARC_thickness_get, _LibreDWG.Dwg_Entity_ARC_thickness_set)
    extrusion = property(_LibreDWG.Dwg_Entity_ARC_extrusion_get, _LibreDWG.Dwg_Entity_ARC_extrusion_set)
    start_angle = property(_LibreDWG.Dwg_Entity_ARC_start_angle_get, _LibreDWG.Dwg_Entity_ARC_start_angle_set)
    end_angle = property(_LibreDWG.Dwg_Entity_ARC_end_angle_get, _LibreDWG.Dwg_Entity_ARC_end_angle_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_ARC_swiginit(self, _LibreDWG.new_Dwg_Entity_ARC())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_ARC

# Register Dwg_Entity_ARC in _LibreDWG:
_LibreDWG.Dwg_Entity_ARC_swigregister(Dwg_Entity_ARC)
class Dwg_Entity_CIRCLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_CIRCLE_parent_get, _LibreDWG.Dwg_Entity_CIRCLE_parent_set)
    center = property(_LibreDWG.Dwg_Entity_CIRCLE_center_get, _LibreDWG.Dwg_Entity_CIRCLE_center_set)
    radius = property(_LibreDWG.Dwg_Entity_CIRCLE_radius_get, _LibreDWG.Dwg_Entity_CIRCLE_radius_set)
    thickness = property(_LibreDWG.Dwg_Entity_CIRCLE_thickness_get, _LibreDWG.Dwg_Entity_CIRCLE_thickness_set)
    extrusion = property(_LibreDWG.Dwg_Entity_CIRCLE_extrusion_get, _LibreDWG.Dwg_Entity_CIRCLE_extrusion_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_CIRCLE_swiginit(self, _LibreDWG.new_Dwg_Entity_CIRCLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_CIRCLE

# Register Dwg_Entity_CIRCLE in _LibreDWG:
_LibreDWG.Dwg_Entity_CIRCLE_swigregister(Dwg_Entity_CIRCLE)
class Dwg_Entity_LINE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_LINE_parent_get, _LibreDWG.Dwg_Entity_LINE_parent_set)
    z_is_zero = property(_LibreDWG.Dwg_Entity_LINE_z_is_zero_get, _LibreDWG.Dwg_Entity_LINE_z_is_zero_set)
    start = property(_LibreDWG.Dwg_Entity_LINE_start_get, _LibreDWG.Dwg_Entity_LINE_start_set)
    end = property(_LibreDWG.Dwg_Entity_LINE_end_get, _LibreDWG.Dwg_Entity_LINE_end_set)
    thickness = property(_LibreDWG.Dwg_Entity_LINE_thickness_get, _LibreDWG.Dwg_Entity_LINE_thickness_set)
    extrusion = property(_LibreDWG.Dwg_Entity_LINE_extrusion_get, _LibreDWG.Dwg_Entity_LINE_extrusion_set)
    unknown_r11 = property(_LibreDWG.Dwg_Entity_LINE_unknown_r11_get, _LibreDWG.Dwg_Entity_LINE_unknown_r11_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_LINE_swiginit(self, _LibreDWG.new_Dwg_Entity_LINE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_LINE

# Register Dwg_Entity_LINE in _LibreDWG:
_LibreDWG.Dwg_Entity_LINE_swigregister(Dwg_Entity_LINE)
class Dwg_DIMENSION_common(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_DIMENSION_common_parent_get, _LibreDWG.Dwg_DIMENSION_common_parent_set)
    class_version = property(_LibreDWG.Dwg_DIMENSION_common_class_version_get, _LibreDWG.Dwg_DIMENSION_common_class_version_set)
    extrusion = property(_LibreDWG.Dwg_DIMENSION_common_extrusion_get, _LibreDWG.Dwg_DIMENSION_common_extrusion_set)
    def_pt = property(_LibreDWG.Dwg_DIMENSION_common_def_pt_get, _LibreDWG.Dwg_DIMENSION_common_def_pt_set)
    text_midpt = property(_LibreDWG.Dwg_DIMENSION_common_text_midpt_get, _LibreDWG.Dwg_DIMENSION_common_text_midpt_set)
    elevation = property(_LibreDWG.Dwg_DIMENSION_common_elevation_get, _LibreDWG.Dwg_DIMENSION_common_elevation_set)
    flag = property(_LibreDWG.Dwg_DIMENSION_common_flag_get, _LibreDWG.Dwg_DIMENSION_common_flag_set)
    flag1 = property(_LibreDWG.Dwg_DIMENSION_common_flag1_get, _LibreDWG.Dwg_DIMENSION_common_flag1_set)
    user_text = property(_LibreDWG.Dwg_DIMENSION_common_user_text_get, _LibreDWG.Dwg_DIMENSION_common_user_text_set)
    text_rotation = property(_LibreDWG.Dwg_DIMENSION_common_text_rotation_get, _LibreDWG.Dwg_DIMENSION_common_text_rotation_set)
    horiz_dir = property(_LibreDWG.Dwg_DIMENSION_common_horiz_dir_get, _LibreDWG.Dwg_DIMENSION_common_horiz_dir_set)
    ins_scale = property(_LibreDWG.Dwg_DIMENSION_common_ins_scale_get, _LibreDWG.Dwg_DIMENSION_common_ins_scale_set)
    ins_rotation = property(_LibreDWG.Dwg_DIMENSION_common_ins_rotation_get, _LibreDWG.Dwg_DIMENSION_common_ins_rotation_set)
    attachment = property(_LibreDWG.Dwg_DIMENSION_common_attachment_get, _LibreDWG.Dwg_DIMENSION_common_attachment_set)
    lspace_style = property(_LibreDWG.Dwg_DIMENSION_common_lspace_style_get, _LibreDWG.Dwg_DIMENSION_common_lspace_style_set)
    lspace_factor = property(_LibreDWG.Dwg_DIMENSION_common_lspace_factor_get, _LibreDWG.Dwg_DIMENSION_common_lspace_factor_set)
    act_measurement = property(_LibreDWG.Dwg_DIMENSION_common_act_measurement_get, _LibreDWG.Dwg_DIMENSION_common_act_measurement_set)
    unknown = property(_LibreDWG.Dwg_DIMENSION_common_unknown_get, _LibreDWG.Dwg_DIMENSION_common_unknown_set)
    flip_arrow1 = property(_LibreDWG.Dwg_DIMENSION_common_flip_arrow1_get, _LibreDWG.Dwg_DIMENSION_common_flip_arrow1_set)
    flip_arrow2 = property(_LibreDWG.Dwg_DIMENSION_common_flip_arrow2_get, _LibreDWG.Dwg_DIMENSION_common_flip_arrow2_set)
    clone_ins_pt = property(_LibreDWG.Dwg_DIMENSION_common_clone_ins_pt_get, _LibreDWG.Dwg_DIMENSION_common_clone_ins_pt_set)
    dimstyle = property(_LibreDWG.Dwg_DIMENSION_common_dimstyle_get, _LibreDWG.Dwg_DIMENSION_common_dimstyle_set)
    block = property(_LibreDWG.Dwg_DIMENSION_common_block_get, _LibreDWG.Dwg_DIMENSION_common_block_set)

    def __init__(self):
        _LibreDWG.Dwg_DIMENSION_common_swiginit(self, _LibreDWG.new_Dwg_DIMENSION_common())
    __swig_destroy__ = _LibreDWG.delete_Dwg_DIMENSION_common

# Register Dwg_DIMENSION_common in _LibreDWG:
_LibreDWG.Dwg_DIMENSION_common_swigregister(Dwg_DIMENSION_common)
class Dwg_Entity_DIMENSION_ORDINATE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_parent_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_class_version_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_class_version_set)
    extrusion = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_extrusion_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_extrusion_set)
    def_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_def_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_def_pt_set)
    text_midpt = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_text_midpt_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_text_midpt_set)
    elevation = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_elevation_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_elevation_set)
    flag = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_flag_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_flag_set)
    flag1 = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_flag1_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_flag1_set)
    user_text = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_user_text_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_user_text_set)
    text_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_text_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_text_rotation_set)
    horiz_dir = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_set)
    ins_scale = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_ins_scale_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_ins_scale_set)
    ins_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_set)
    attachment = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_attachment_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_attachment_set)
    lspace_style = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_lspace_style_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_lspace_style_set)
    lspace_factor = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_set)
    act_measurement = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_act_measurement_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_act_measurement_set)
    unknown = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_unknown_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_unknown_set)
    flip_arrow1 = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_set)
    flip_arrow2 = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_set)
    clone_ins_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_set)
    dimstyle = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_dimstyle_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_dimstyle_set)
    block = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_block_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_block_set)
    feature_location_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_set)
    leader_endpt = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_set)
    flag2 = property(_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_flag2_get, _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_flag2_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_swiginit(self, _LibreDWG.new_Dwg_Entity_DIMENSION_ORDINATE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_DIMENSION_ORDINATE

# Register Dwg_Entity_DIMENSION_ORDINATE in _LibreDWG:
_LibreDWG.Dwg_Entity_DIMENSION_ORDINATE_swigregister(Dwg_Entity_DIMENSION_ORDINATE)
class Dwg_Entity_DIMENSION_LINEAR(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_parent_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_class_version_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_class_version_set)
    extrusion = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_extrusion_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_extrusion_set)
    def_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_def_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_def_pt_set)
    text_midpt = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_text_midpt_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_text_midpt_set)
    elevation = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_elevation_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_elevation_set)
    flag = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_flag_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_flag_set)
    flag1 = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_flag1_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_flag1_set)
    user_text = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_user_text_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_user_text_set)
    text_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_text_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_text_rotation_set)
    horiz_dir = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_horiz_dir_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_horiz_dir_set)
    ins_scale = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_ins_scale_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_ins_scale_set)
    ins_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_ins_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_ins_rotation_set)
    attachment = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_attachment_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_attachment_set)
    lspace_style = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_lspace_style_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_lspace_style_set)
    lspace_factor = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_lspace_factor_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_lspace_factor_set)
    act_measurement = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_act_measurement_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_act_measurement_set)
    unknown = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_unknown_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_unknown_set)
    flip_arrow1 = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_set)
    flip_arrow2 = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_set)
    clone_ins_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_set)
    dimstyle = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_dimstyle_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_dimstyle_set)
    block = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_block_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_block_set)
    xline1_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_xline1_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_xline1_pt_set)
    xline2_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_xline2_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_xline2_pt_set)
    oblique_angle = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_oblique_angle_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_oblique_angle_set)
    dim_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_dim_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_dim_rotation_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_DIMENSION_LINEAR_swiginit(self, _LibreDWG.new_Dwg_Entity_DIMENSION_LINEAR())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_DIMENSION_LINEAR

# Register Dwg_Entity_DIMENSION_LINEAR in _LibreDWG:
_LibreDWG.Dwg_Entity_DIMENSION_LINEAR_swigregister(Dwg_Entity_DIMENSION_LINEAR)
class Dwg_Entity_DIMENSION_ALIGNED(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_parent_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_class_version_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_class_version_set)
    extrusion = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_extrusion_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_extrusion_set)
    def_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_def_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_def_pt_set)
    text_midpt = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_text_midpt_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_text_midpt_set)
    elevation = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_elevation_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_elevation_set)
    flag = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_flag_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_flag_set)
    flag1 = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_flag1_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_flag1_set)
    user_text = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_user_text_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_user_text_set)
    text_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_text_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_text_rotation_set)
    horiz_dir = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_set)
    ins_scale = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_ins_scale_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_ins_scale_set)
    ins_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_set)
    attachment = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_attachment_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_attachment_set)
    lspace_style = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_lspace_style_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_lspace_style_set)
    lspace_factor = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_set)
    act_measurement = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_act_measurement_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_act_measurement_set)
    unknown = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_unknown_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_unknown_set)
    flip_arrow1 = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_set)
    flip_arrow2 = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_set)
    clone_ins_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_set)
    dimstyle = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_dimstyle_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_dimstyle_set)
    block = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_block_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_block_set)
    xline1_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_set)
    xline2_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_set)
    oblique_angle = property(_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_get, _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_swiginit(self, _LibreDWG.new_Dwg_Entity_DIMENSION_ALIGNED())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_DIMENSION_ALIGNED

# Register Dwg_Entity_DIMENSION_ALIGNED in _LibreDWG:
_LibreDWG.Dwg_Entity_DIMENSION_ALIGNED_swigregister(Dwg_Entity_DIMENSION_ALIGNED)
class Dwg_Entity_DIMENSION_ANG3PT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_parent_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_class_version_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_class_version_set)
    extrusion = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_extrusion_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_extrusion_set)
    def_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_def_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_def_pt_set)
    text_midpt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_text_midpt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_text_midpt_set)
    elevation = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_elevation_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_elevation_set)
    flag = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_flag_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_flag_set)
    flag1 = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_flag1_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_flag1_set)
    user_text = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_user_text_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_user_text_set)
    text_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_text_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_text_rotation_set)
    horiz_dir = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_set)
    ins_scale = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_ins_scale_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_ins_scale_set)
    ins_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_set)
    attachment = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_attachment_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_attachment_set)
    lspace_style = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_lspace_style_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_lspace_style_set)
    lspace_factor = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_set)
    act_measurement = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_act_measurement_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_act_measurement_set)
    unknown = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_unknown_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_unknown_set)
    flip_arrow1 = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_set)
    flip_arrow2 = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_set)
    clone_ins_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_set)
    dimstyle = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_dimstyle_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_dimstyle_set)
    block = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_block_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_block_set)
    xline1_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_set)
    xline2_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_set)
    center_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_center_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_center_pt_set)
    xline2end_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_swiginit(self, _LibreDWG.new_Dwg_Entity_DIMENSION_ANG3PT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_DIMENSION_ANG3PT

# Register Dwg_Entity_DIMENSION_ANG3PT in _LibreDWG:
_LibreDWG.Dwg_Entity_DIMENSION_ANG3PT_swigregister(Dwg_Entity_DIMENSION_ANG3PT)
class Dwg_Entity_DIMENSION_ANG2LN(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_parent_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_class_version_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_class_version_set)
    extrusion = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_extrusion_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_extrusion_set)
    def_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_def_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_def_pt_set)
    text_midpt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_text_midpt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_text_midpt_set)
    elevation = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_elevation_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_elevation_set)
    flag = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_flag_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_flag_set)
    flag1 = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_flag1_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_flag1_set)
    user_text = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_user_text_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_user_text_set)
    text_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_text_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_text_rotation_set)
    horiz_dir = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_set)
    ins_scale = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_ins_scale_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_ins_scale_set)
    ins_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_set)
    attachment = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_attachment_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_attachment_set)
    lspace_style = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_lspace_style_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_lspace_style_set)
    lspace_factor = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_set)
    act_measurement = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_act_measurement_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_act_measurement_set)
    unknown = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_unknown_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_unknown_set)
    flip_arrow1 = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_set)
    flip_arrow2 = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_set)
    clone_ins_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_set)
    dimstyle = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_dimstyle_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_dimstyle_set)
    block = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_block_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_block_set)
    xline1start_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_set)
    xline1end_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_set)
    xline2start_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_set)
    xline2end_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_swiginit(self, _LibreDWG.new_Dwg_Entity_DIMENSION_ANG2LN())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_DIMENSION_ANG2LN

# Register Dwg_Entity_DIMENSION_ANG2LN in _LibreDWG:
_LibreDWG.Dwg_Entity_DIMENSION_ANG2LN_swigregister(Dwg_Entity_DIMENSION_ANG2LN)
class Dwg_Entity_DIMENSION_RADIUS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_parent_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_class_version_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_class_version_set)
    extrusion = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_extrusion_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_extrusion_set)
    def_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_def_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_def_pt_set)
    text_midpt = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_text_midpt_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_text_midpt_set)
    elevation = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_elevation_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_elevation_set)
    flag = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_flag_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_flag_set)
    flag1 = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_flag1_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_flag1_set)
    user_text = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_user_text_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_user_text_set)
    text_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_text_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_text_rotation_set)
    horiz_dir = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_horiz_dir_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_horiz_dir_set)
    ins_scale = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_ins_scale_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_ins_scale_set)
    ins_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_ins_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_ins_rotation_set)
    attachment = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_attachment_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_attachment_set)
    lspace_style = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_lspace_style_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_lspace_style_set)
    lspace_factor = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_lspace_factor_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_lspace_factor_set)
    act_measurement = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_act_measurement_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_act_measurement_set)
    unknown = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_unknown_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_unknown_set)
    flip_arrow1 = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_set)
    flip_arrow2 = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_set)
    clone_ins_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_set)
    dimstyle = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_dimstyle_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_dimstyle_set)
    block = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_block_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_block_set)
    first_arc_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_set)
    leader_len = property(_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_leader_len_get, _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_leader_len_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_DIMENSION_RADIUS_swiginit(self, _LibreDWG.new_Dwg_Entity_DIMENSION_RADIUS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_DIMENSION_RADIUS

# Register Dwg_Entity_DIMENSION_RADIUS in _LibreDWG:
_LibreDWG.Dwg_Entity_DIMENSION_RADIUS_swigregister(Dwg_Entity_DIMENSION_RADIUS)
class Dwg_Entity_DIMENSION_DIAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_parent_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_class_version_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_class_version_set)
    extrusion = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_extrusion_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_extrusion_set)
    def_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_def_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_def_pt_set)
    text_midpt = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_text_midpt_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_text_midpt_set)
    elevation = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_elevation_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_elevation_set)
    flag = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_flag_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_flag_set)
    flag1 = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_flag1_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_flag1_set)
    user_text = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_user_text_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_user_text_set)
    text_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_text_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_text_rotation_set)
    horiz_dir = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_set)
    ins_scale = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_ins_scale_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_ins_scale_set)
    ins_rotation = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_set)
    attachment = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_attachment_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_attachment_set)
    lspace_style = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_lspace_style_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_lspace_style_set)
    lspace_factor = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_set)
    act_measurement = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_act_measurement_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_act_measurement_set)
    unknown = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_unknown_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_unknown_set)
    flip_arrow1 = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_set)
    flip_arrow2 = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_set)
    clone_ins_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_set)
    dimstyle = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_dimstyle_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_dimstyle_set)
    block = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_block_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_block_set)
    first_arc_pt = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_set)
    leader_len = property(_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_leader_len_get, _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_leader_len_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_swiginit(self, _LibreDWG.new_Dwg_Entity_DIMENSION_DIAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_DIMENSION_DIAMETER

# Register Dwg_Entity_DIMENSION_DIAMETER in _LibreDWG:
_LibreDWG.Dwg_Entity_DIMENSION_DIAMETER_swigregister(Dwg_Entity_DIMENSION_DIAMETER)
class Dwg_Entity_ARC_DIMENSION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_parent_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_class_version_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_class_version_set)
    extrusion = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_extrusion_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_extrusion_set)
    def_pt = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_def_pt_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_def_pt_set)
    text_midpt = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_text_midpt_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_text_midpt_set)
    elevation = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_elevation_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_elevation_set)
    flag = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_flag_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_flag_set)
    flag1 = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_flag1_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_flag1_set)
    user_text = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_user_text_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_user_text_set)
    text_rotation = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_text_rotation_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_text_rotation_set)
    horiz_dir = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_horiz_dir_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_horiz_dir_set)
    ins_scale = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_ins_scale_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_ins_scale_set)
    ins_rotation = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_ins_rotation_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_ins_rotation_set)
    attachment = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_attachment_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_attachment_set)
    lspace_style = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_lspace_style_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_lspace_style_set)
    lspace_factor = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_lspace_factor_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_lspace_factor_set)
    act_measurement = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_act_measurement_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_act_measurement_set)
    unknown = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_unknown_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_unknown_set)
    flip_arrow1 = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_flip_arrow1_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_flip_arrow1_set)
    flip_arrow2 = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_flip_arrow2_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_flip_arrow2_set)
    clone_ins_pt = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_clone_ins_pt_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_clone_ins_pt_set)
    dimstyle = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_dimstyle_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_dimstyle_set)
    block = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_block_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_block_set)
    xline1_pt = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_xline1_pt_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_xline1_pt_set)
    xline2_pt = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_xline2_pt_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_xline2_pt_set)
    center_pt = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_center_pt_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_center_pt_set)
    is_partial = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_is_partial_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_is_partial_set)
    arc_start_param = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_arc_start_param_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_arc_start_param_set)
    arc_end_param = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_arc_end_param_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_arc_end_param_set)
    has_leader = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_has_leader_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_has_leader_set)
    leader1_pt = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_leader1_pt_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_leader1_pt_set)
    leader2_pt = property(_LibreDWG.Dwg_Entity_ARC_DIMENSION_leader2_pt_get, _LibreDWG.Dwg_Entity_ARC_DIMENSION_leader2_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_ARC_DIMENSION_swiginit(self, _LibreDWG.new_Dwg_Entity_ARC_DIMENSION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_ARC_DIMENSION

# Register Dwg_Entity_ARC_DIMENSION in _LibreDWG:
_LibreDWG.Dwg_Entity_ARC_DIMENSION_swigregister(Dwg_Entity_ARC_DIMENSION)
class Dwg_Entity_LARGE_RADIAL_DIMENSION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_set)
    extrusion = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_set)
    def_pt = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_set)
    text_midpt = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_set)
    elevation = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_set)
    flag = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_set)
    flag1 = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_set)
    user_text = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_set)
    text_rotation = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_set)
    horiz_dir = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_set)
    ins_scale = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_set)
    ins_rotation = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_set)
    attachment = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_set)
    lspace_style = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_set)
    lspace_factor = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_set)
    act_measurement = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_set)
    unknown = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_set)
    flip_arrow1 = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_set)
    flip_arrow2 = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_set)
    clone_ins_pt = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_set)
    dimstyle = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_set)
    block = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_block_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_block_set)
    jog_angle = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_set)
    jog_pt = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_set)
    ovr_center = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_set)
    chord_pt = property(_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_get, _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_swiginit(self, _LibreDWG.new_Dwg_Entity_LARGE_RADIAL_DIMENSION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_LARGE_RADIAL_DIMENSION

# Register Dwg_Entity_LARGE_RADIAL_DIMENSION in _LibreDWG:
_LibreDWG.Dwg_Entity_LARGE_RADIAL_DIMENSION_swigregister(Dwg_Entity_LARGE_RADIAL_DIMENSION)
class Dwg_Entity_POINT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_POINT_parent_get, _LibreDWG.Dwg_Entity_POINT_parent_set)
    x = property(_LibreDWG.Dwg_Entity_POINT_x_get, _LibreDWG.Dwg_Entity_POINT_x_set)
    y = property(_LibreDWG.Dwg_Entity_POINT_y_get, _LibreDWG.Dwg_Entity_POINT_y_set)
    z = property(_LibreDWG.Dwg_Entity_POINT_z_get, _LibreDWG.Dwg_Entity_POINT_z_set)
    thickness = property(_LibreDWG.Dwg_Entity_POINT_thickness_get, _LibreDWG.Dwg_Entity_POINT_thickness_set)
    extrusion = property(_LibreDWG.Dwg_Entity_POINT_extrusion_get, _LibreDWG.Dwg_Entity_POINT_extrusion_set)
    x_ang = property(_LibreDWG.Dwg_Entity_POINT_x_ang_get, _LibreDWG.Dwg_Entity_POINT_x_ang_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_POINT_swiginit(self, _LibreDWG.new_Dwg_Entity_POINT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_POINT

# Register Dwg_Entity_POINT in _LibreDWG:
_LibreDWG.Dwg_Entity_POINT_swigregister(Dwg_Entity_POINT)
class Dwg_Entity__3DFACE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity__3DFACE_parent_get, _LibreDWG.Dwg_Entity__3DFACE_parent_set)
    has_no_flags = property(_LibreDWG.Dwg_Entity__3DFACE_has_no_flags_get, _LibreDWG.Dwg_Entity__3DFACE_has_no_flags_set)
    z_is_zero = property(_LibreDWG.Dwg_Entity__3DFACE_z_is_zero_get, _LibreDWG.Dwg_Entity__3DFACE_z_is_zero_set)
    corner1 = property(_LibreDWG.Dwg_Entity__3DFACE_corner1_get, _LibreDWG.Dwg_Entity__3DFACE_corner1_set)
    corner2 = property(_LibreDWG.Dwg_Entity__3DFACE_corner2_get, _LibreDWG.Dwg_Entity__3DFACE_corner2_set)
    corner3 = property(_LibreDWG.Dwg_Entity__3DFACE_corner3_get, _LibreDWG.Dwg_Entity__3DFACE_corner3_set)
    corner4 = property(_LibreDWG.Dwg_Entity__3DFACE_corner4_get, _LibreDWG.Dwg_Entity__3DFACE_corner4_set)
    invis_flags = property(_LibreDWG.Dwg_Entity__3DFACE_invis_flags_get, _LibreDWG.Dwg_Entity__3DFACE_invis_flags_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity__3DFACE_swiginit(self, _LibreDWG.new_Dwg_Entity__3DFACE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity__3DFACE

# Register Dwg_Entity__3DFACE in _LibreDWG:
_LibreDWG.Dwg_Entity__3DFACE_swigregister(Dwg_Entity__3DFACE)
class Dwg_Entity_POLYLINE_PFACE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_POLYLINE_PFACE_parent_get, _LibreDWG.Dwg_Entity_POLYLINE_PFACE_parent_set)
    has_vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_PFACE_has_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_PFACE_has_vertex_set)
    num_owned = property(_LibreDWG.Dwg_Entity_POLYLINE_PFACE_num_owned_get, _LibreDWG.Dwg_Entity_POLYLINE_PFACE_num_owned_set)
    first_vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_PFACE_first_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_PFACE_first_vertex_set)
    last_vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_PFACE_last_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_PFACE_last_vertex_set)
    vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_PFACE_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_PFACE_vertex_set)
    seqend = property(_LibreDWG.Dwg_Entity_POLYLINE_PFACE_seqend_get, _LibreDWG.Dwg_Entity_POLYLINE_PFACE_seqend_set)
    flag = property(_LibreDWG.Dwg_Entity_POLYLINE_PFACE_flag_get, _LibreDWG.Dwg_Entity_POLYLINE_PFACE_flag_set)
    numverts = property(_LibreDWG.Dwg_Entity_POLYLINE_PFACE_numverts_get, _LibreDWG.Dwg_Entity_POLYLINE_PFACE_numverts_set)
    numfaces = property(_LibreDWG.Dwg_Entity_POLYLINE_PFACE_numfaces_get, _LibreDWG.Dwg_Entity_POLYLINE_PFACE_numfaces_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_POLYLINE_PFACE_swiginit(self, _LibreDWG.new_Dwg_Entity_POLYLINE_PFACE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_POLYLINE_PFACE

# Register Dwg_Entity_POLYLINE_PFACE in _LibreDWG:
_LibreDWG.Dwg_Entity_POLYLINE_PFACE_swigregister(Dwg_Entity_POLYLINE_PFACE)
class Dwg_Entity_POLYLINE_MESH(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_POLYLINE_MESH_parent_get, _LibreDWG.Dwg_Entity_POLYLINE_MESH_parent_set)
    has_vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_MESH_has_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_MESH_has_vertex_set)
    num_owned = property(_LibreDWG.Dwg_Entity_POLYLINE_MESH_num_owned_get, _LibreDWG.Dwg_Entity_POLYLINE_MESH_num_owned_set)
    first_vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_MESH_first_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_MESH_first_vertex_set)
    last_vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_MESH_last_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_MESH_last_vertex_set)
    vertex = property(_LibreDWG.Dwg_Entity_POLYLINE_MESH_vertex_get, _LibreDWG.Dwg_Entity_POLYLINE_MESH_vertex_set)
    seqend = property(_LibreDWG.Dwg_Entity_POLYLINE_MESH_seqend_get, _LibreDWG.Dwg_Entity_POLYLINE_MESH_seqend_set)
    flag = property(_LibreDWG.Dwg_Entity_POLYLINE_MESH_flag_get, _LibreDWG.Dwg_Entity_POLYLINE_MESH_flag_set)
    curve_type = property(_LibreDWG.Dwg_Entity_POLYLINE_MESH_curve_type_get, _LibreDWG.Dwg_Entity_POLYLINE_MESH_curve_type_set)
    num_m_verts = property(_LibreDWG.Dwg_Entity_POLYLINE_MESH_num_m_verts_get, _LibreDWG.Dwg_Entity_POLYLINE_MESH_num_m_verts_set)
    num_n_verts = property(_LibreDWG.Dwg_Entity_POLYLINE_MESH_num_n_verts_get, _LibreDWG.Dwg_Entity_POLYLINE_MESH_num_n_verts_set)
    m_density = property(_LibreDWG.Dwg_Entity_POLYLINE_MESH_m_density_get, _LibreDWG.Dwg_Entity_POLYLINE_MESH_m_density_set)
    n_density = property(_LibreDWG.Dwg_Entity_POLYLINE_MESH_n_density_get, _LibreDWG.Dwg_Entity_POLYLINE_MESH_n_density_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_POLYLINE_MESH_swiginit(self, _LibreDWG.new_Dwg_Entity_POLYLINE_MESH())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_POLYLINE_MESH

# Register Dwg_Entity_POLYLINE_MESH in _LibreDWG:
_LibreDWG.Dwg_Entity_POLYLINE_MESH_swigregister(Dwg_Entity_POLYLINE_MESH)
class Dwg_Entity_SOLID(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_SOLID_parent_get, _LibreDWG.Dwg_Entity_SOLID_parent_set)
    thickness = property(_LibreDWG.Dwg_Entity_SOLID_thickness_get, _LibreDWG.Dwg_Entity_SOLID_thickness_set)
    elevation = property(_LibreDWG.Dwg_Entity_SOLID_elevation_get, _LibreDWG.Dwg_Entity_SOLID_elevation_set)
    corner1 = property(_LibreDWG.Dwg_Entity_SOLID_corner1_get, _LibreDWG.Dwg_Entity_SOLID_corner1_set)
    corner2 = property(_LibreDWG.Dwg_Entity_SOLID_corner2_get, _LibreDWG.Dwg_Entity_SOLID_corner2_set)
    corner3 = property(_LibreDWG.Dwg_Entity_SOLID_corner3_get, _LibreDWG.Dwg_Entity_SOLID_corner3_set)
    corner4 = property(_LibreDWG.Dwg_Entity_SOLID_corner4_get, _LibreDWG.Dwg_Entity_SOLID_corner4_set)
    extrusion = property(_LibreDWG.Dwg_Entity_SOLID_extrusion_get, _LibreDWG.Dwg_Entity_SOLID_extrusion_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_SOLID_swiginit(self, _LibreDWG.new_Dwg_Entity_SOLID())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_SOLID

# Register Dwg_Entity_SOLID in _LibreDWG:
_LibreDWG.Dwg_Entity_SOLID_swigregister(Dwg_Entity_SOLID)
class Dwg_Entity_TRACE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_TRACE_parent_get, _LibreDWG.Dwg_Entity_TRACE_parent_set)
    thickness = property(_LibreDWG.Dwg_Entity_TRACE_thickness_get, _LibreDWG.Dwg_Entity_TRACE_thickness_set)
    elevation = property(_LibreDWG.Dwg_Entity_TRACE_elevation_get, _LibreDWG.Dwg_Entity_TRACE_elevation_set)
    corner1 = property(_LibreDWG.Dwg_Entity_TRACE_corner1_get, _LibreDWG.Dwg_Entity_TRACE_corner1_set)
    corner2 = property(_LibreDWG.Dwg_Entity_TRACE_corner2_get, _LibreDWG.Dwg_Entity_TRACE_corner2_set)
    corner3 = property(_LibreDWG.Dwg_Entity_TRACE_corner3_get, _LibreDWG.Dwg_Entity_TRACE_corner3_set)
    corner4 = property(_LibreDWG.Dwg_Entity_TRACE_corner4_get, _LibreDWG.Dwg_Entity_TRACE_corner4_set)
    extrusion = property(_LibreDWG.Dwg_Entity_TRACE_extrusion_get, _LibreDWG.Dwg_Entity_TRACE_extrusion_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_TRACE_swiginit(self, _LibreDWG.new_Dwg_Entity_TRACE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_TRACE

# Register Dwg_Entity_TRACE in _LibreDWG:
_LibreDWG.Dwg_Entity_TRACE_swigregister(Dwg_Entity_TRACE)
class Dwg_Entity_SHAPE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_SHAPE_parent_get, _LibreDWG.Dwg_Entity_SHAPE_parent_set)
    ins_pt = property(_LibreDWG.Dwg_Entity_SHAPE_ins_pt_get, _LibreDWG.Dwg_Entity_SHAPE_ins_pt_set)
    scale = property(_LibreDWG.Dwg_Entity_SHAPE_scale_get, _LibreDWG.Dwg_Entity_SHAPE_scale_set)
    rotation = property(_LibreDWG.Dwg_Entity_SHAPE_rotation_get, _LibreDWG.Dwg_Entity_SHAPE_rotation_set)
    width_factor = property(_LibreDWG.Dwg_Entity_SHAPE_width_factor_get, _LibreDWG.Dwg_Entity_SHAPE_width_factor_set)
    oblique_angle = property(_LibreDWG.Dwg_Entity_SHAPE_oblique_angle_get, _LibreDWG.Dwg_Entity_SHAPE_oblique_angle_set)
    thickness = property(_LibreDWG.Dwg_Entity_SHAPE_thickness_get, _LibreDWG.Dwg_Entity_SHAPE_thickness_set)
    style_id = property(_LibreDWG.Dwg_Entity_SHAPE_style_id_get, _LibreDWG.Dwg_Entity_SHAPE_style_id_set)
    extrusion = property(_LibreDWG.Dwg_Entity_SHAPE_extrusion_get, _LibreDWG.Dwg_Entity_SHAPE_extrusion_set)
    style = property(_LibreDWG.Dwg_Entity_SHAPE_style_get, _LibreDWG.Dwg_Entity_SHAPE_style_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_SHAPE_swiginit(self, _LibreDWG.new_Dwg_Entity_SHAPE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_SHAPE

# Register Dwg_Entity_SHAPE in _LibreDWG:
_LibreDWG.Dwg_Entity_SHAPE_swigregister(Dwg_Entity_SHAPE)
class Dwg_Entity_VIEWPORT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_VIEWPORT_parent_get, _LibreDWG.Dwg_Entity_VIEWPORT_parent_set)
    center = property(_LibreDWG.Dwg_Entity_VIEWPORT_center_get, _LibreDWG.Dwg_Entity_VIEWPORT_center_set)
    width = property(_LibreDWG.Dwg_Entity_VIEWPORT_width_get, _LibreDWG.Dwg_Entity_VIEWPORT_width_set)
    height = property(_LibreDWG.Dwg_Entity_VIEWPORT_height_get, _LibreDWG.Dwg_Entity_VIEWPORT_height_set)
    on_off = property(_LibreDWG.Dwg_Entity_VIEWPORT_on_off_get, _LibreDWG.Dwg_Entity_VIEWPORT_on_off_set)
    id = property(_LibreDWG.Dwg_Entity_VIEWPORT_id_get, _LibreDWG.Dwg_Entity_VIEWPORT_id_set)
    view_target = property(_LibreDWG.Dwg_Entity_VIEWPORT_view_target_get, _LibreDWG.Dwg_Entity_VIEWPORT_view_target_set)
    VIEWDIR = property(_LibreDWG.Dwg_Entity_VIEWPORT_VIEWDIR_get, _LibreDWG.Dwg_Entity_VIEWPORT_VIEWDIR_set)
    VIEWTWIST = property(_LibreDWG.Dwg_Entity_VIEWPORT_VIEWTWIST_get, _LibreDWG.Dwg_Entity_VIEWPORT_VIEWTWIST_set)
    VIEWSIZE = property(_LibreDWG.Dwg_Entity_VIEWPORT_VIEWSIZE_get, _LibreDWG.Dwg_Entity_VIEWPORT_VIEWSIZE_set)
    LENSLENGTH = property(_LibreDWG.Dwg_Entity_VIEWPORT_LENSLENGTH_get, _LibreDWG.Dwg_Entity_VIEWPORT_LENSLENGTH_set)
    FRONTZ = property(_LibreDWG.Dwg_Entity_VIEWPORT_FRONTZ_get, _LibreDWG.Dwg_Entity_VIEWPORT_FRONTZ_set)
    BACKZ = property(_LibreDWG.Dwg_Entity_VIEWPORT_BACKZ_get, _LibreDWG.Dwg_Entity_VIEWPORT_BACKZ_set)
    SNAPANG = property(_LibreDWG.Dwg_Entity_VIEWPORT_SNAPANG_get, _LibreDWG.Dwg_Entity_VIEWPORT_SNAPANG_set)
    VIEWCTR = property(_LibreDWG.Dwg_Entity_VIEWPORT_VIEWCTR_get, _LibreDWG.Dwg_Entity_VIEWPORT_VIEWCTR_set)
    SNAPBASE = property(_LibreDWG.Dwg_Entity_VIEWPORT_SNAPBASE_get, _LibreDWG.Dwg_Entity_VIEWPORT_SNAPBASE_set)
    SNAPUNIT = property(_LibreDWG.Dwg_Entity_VIEWPORT_SNAPUNIT_get, _LibreDWG.Dwg_Entity_VIEWPORT_SNAPUNIT_set)
    GRIDUNIT = property(_LibreDWG.Dwg_Entity_VIEWPORT_GRIDUNIT_get, _LibreDWG.Dwg_Entity_VIEWPORT_GRIDUNIT_set)
    circle_zoom = property(_LibreDWG.Dwg_Entity_VIEWPORT_circle_zoom_get, _LibreDWG.Dwg_Entity_VIEWPORT_circle_zoom_set)
    grid_major = property(_LibreDWG.Dwg_Entity_VIEWPORT_grid_major_get, _LibreDWG.Dwg_Entity_VIEWPORT_grid_major_set)
    num_frozen_layers = property(_LibreDWG.Dwg_Entity_VIEWPORT_num_frozen_layers_get, _LibreDWG.Dwg_Entity_VIEWPORT_num_frozen_layers_set)
    status_flag = property(_LibreDWG.Dwg_Entity_VIEWPORT_status_flag_get, _LibreDWG.Dwg_Entity_VIEWPORT_status_flag_set)
    style_sheet = property(_LibreDWG.Dwg_Entity_VIEWPORT_style_sheet_get, _LibreDWG.Dwg_Entity_VIEWPORT_style_sheet_set)
    render_mode = property(_LibreDWG.Dwg_Entity_VIEWPORT_render_mode_get, _LibreDWG.Dwg_Entity_VIEWPORT_render_mode_set)
    ucs_at_origin = property(_LibreDWG.Dwg_Entity_VIEWPORT_ucs_at_origin_get, _LibreDWG.Dwg_Entity_VIEWPORT_ucs_at_origin_set)
    UCSVP = property(_LibreDWG.Dwg_Entity_VIEWPORT_UCSVP_get, _LibreDWG.Dwg_Entity_VIEWPORT_UCSVP_set)
    ucsorg = property(_LibreDWG.Dwg_Entity_VIEWPORT_ucsorg_get, _LibreDWG.Dwg_Entity_VIEWPORT_ucsorg_set)
    ucsxdir = property(_LibreDWG.Dwg_Entity_VIEWPORT_ucsxdir_get, _LibreDWG.Dwg_Entity_VIEWPORT_ucsxdir_set)
    ucsydir = property(_LibreDWG.Dwg_Entity_VIEWPORT_ucsydir_get, _LibreDWG.Dwg_Entity_VIEWPORT_ucsydir_set)
    ucs_elevation = property(_LibreDWG.Dwg_Entity_VIEWPORT_ucs_elevation_get, _LibreDWG.Dwg_Entity_VIEWPORT_ucs_elevation_set)
    UCSORTHOVIEW = property(_LibreDWG.Dwg_Entity_VIEWPORT_UCSORTHOVIEW_get, _LibreDWG.Dwg_Entity_VIEWPORT_UCSORTHOVIEW_set)
    shadeplot_mode = property(_LibreDWG.Dwg_Entity_VIEWPORT_shadeplot_mode_get, _LibreDWG.Dwg_Entity_VIEWPORT_shadeplot_mode_set)
    use_default_lights = property(_LibreDWG.Dwg_Entity_VIEWPORT_use_default_lights_get, _LibreDWG.Dwg_Entity_VIEWPORT_use_default_lights_set)
    default_lighting_type = property(_LibreDWG.Dwg_Entity_VIEWPORT_default_lighting_type_get, _LibreDWG.Dwg_Entity_VIEWPORT_default_lighting_type_set)
    brightness = property(_LibreDWG.Dwg_Entity_VIEWPORT_brightness_get, _LibreDWG.Dwg_Entity_VIEWPORT_brightness_set)
    contrast = property(_LibreDWG.Dwg_Entity_VIEWPORT_contrast_get, _LibreDWG.Dwg_Entity_VIEWPORT_contrast_set)
    ambient_color = property(_LibreDWG.Dwg_Entity_VIEWPORT_ambient_color_get, _LibreDWG.Dwg_Entity_VIEWPORT_ambient_color_set)
    vport_entity_header = property(_LibreDWG.Dwg_Entity_VIEWPORT_vport_entity_header_get, _LibreDWG.Dwg_Entity_VIEWPORT_vport_entity_header_set)
    frozen_layers = property(_LibreDWG.Dwg_Entity_VIEWPORT_frozen_layers_get, _LibreDWG.Dwg_Entity_VIEWPORT_frozen_layers_set)
    clip_boundary = property(_LibreDWG.Dwg_Entity_VIEWPORT_clip_boundary_get, _LibreDWG.Dwg_Entity_VIEWPORT_clip_boundary_set)
    named_ucs = property(_LibreDWG.Dwg_Entity_VIEWPORT_named_ucs_get, _LibreDWG.Dwg_Entity_VIEWPORT_named_ucs_set)
    base_ucs = property(_LibreDWG.Dwg_Entity_VIEWPORT_base_ucs_get, _LibreDWG.Dwg_Entity_VIEWPORT_base_ucs_set)
    background = property(_LibreDWG.Dwg_Entity_VIEWPORT_background_get, _LibreDWG.Dwg_Entity_VIEWPORT_background_set)
    visualstyle = property(_LibreDWG.Dwg_Entity_VIEWPORT_visualstyle_get, _LibreDWG.Dwg_Entity_VIEWPORT_visualstyle_set)
    shadeplot = property(_LibreDWG.Dwg_Entity_VIEWPORT_shadeplot_get, _LibreDWG.Dwg_Entity_VIEWPORT_shadeplot_set)
    sun = property(_LibreDWG.Dwg_Entity_VIEWPORT_sun_get, _LibreDWG.Dwg_Entity_VIEWPORT_sun_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_VIEWPORT_swiginit(self, _LibreDWG.new_Dwg_Entity_VIEWPORT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_VIEWPORT

# Register Dwg_Entity_VIEWPORT in _LibreDWG:
_LibreDWG.Dwg_Entity_VIEWPORT_swigregister(Dwg_Entity_VIEWPORT)
class Dwg_Entity_ELLIPSE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_ELLIPSE_parent_get, _LibreDWG.Dwg_Entity_ELLIPSE_parent_set)
    center = property(_LibreDWG.Dwg_Entity_ELLIPSE_center_get, _LibreDWG.Dwg_Entity_ELLIPSE_center_set)
    sm_axis = property(_LibreDWG.Dwg_Entity_ELLIPSE_sm_axis_get, _LibreDWG.Dwg_Entity_ELLIPSE_sm_axis_set)
    extrusion = property(_LibreDWG.Dwg_Entity_ELLIPSE_extrusion_get, _LibreDWG.Dwg_Entity_ELLIPSE_extrusion_set)
    axis_ratio = property(_LibreDWG.Dwg_Entity_ELLIPSE_axis_ratio_get, _LibreDWG.Dwg_Entity_ELLIPSE_axis_ratio_set)
    start_angle = property(_LibreDWG.Dwg_Entity_ELLIPSE_start_angle_get, _LibreDWG.Dwg_Entity_ELLIPSE_start_angle_set)
    end_angle = property(_LibreDWG.Dwg_Entity_ELLIPSE_end_angle_get, _LibreDWG.Dwg_Entity_ELLIPSE_end_angle_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_ELLIPSE_swiginit(self, _LibreDWG.new_Dwg_Entity_ELLIPSE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_ELLIPSE

# Register Dwg_Entity_ELLIPSE in _LibreDWG:
_LibreDWG.Dwg_Entity_ELLIPSE_swigregister(Dwg_Entity_ELLIPSE)
class Dwg_SPLINE_control_point(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_SPLINE_control_point_parent_get, _LibreDWG.Dwg_SPLINE_control_point_parent_set)
    x = property(_LibreDWG.Dwg_SPLINE_control_point_x_get, _LibreDWG.Dwg_SPLINE_control_point_x_set)
    y = property(_LibreDWG.Dwg_SPLINE_control_point_y_get, _LibreDWG.Dwg_SPLINE_control_point_y_set)
    z = property(_LibreDWG.Dwg_SPLINE_control_point_z_get, _LibreDWG.Dwg_SPLINE_control_point_z_set)
    w = property(_LibreDWG.Dwg_SPLINE_control_point_w_get, _LibreDWG.Dwg_SPLINE_control_point_w_set)

    def __init__(self):
        _LibreDWG.Dwg_SPLINE_control_point_swiginit(self, _LibreDWG.new_Dwg_SPLINE_control_point())
    __swig_destroy__ = _LibreDWG.delete_Dwg_SPLINE_control_point

# Register Dwg_SPLINE_control_point in _LibreDWG:
_LibreDWG.Dwg_SPLINE_control_point_swigregister(Dwg_SPLINE_control_point)
SPLINETYPE_QUADRATIC = _LibreDWG.SPLINETYPE_QUADRATIC
SPLINETYPE_CUBIC = _LibreDWG.SPLINETYPE_CUBIC
SPLINE_SCENARIO_SPLINE = _LibreDWG.SPLINE_SCENARIO_SPLINE
SPLINE_SCENARIO_BEZIER = _LibreDWG.SPLINE_SCENARIO_BEZIER
SPLINE_SPLINEFLAGS_METHOD_FIT_POINTS = _LibreDWG.SPLINE_SPLINEFLAGS_METHOD_FIT_POINTS
SPLINE_SPLINEFLAGS_CV_FRAME_SHOW = _LibreDWG.SPLINE_SPLINEFLAGS_CV_FRAME_SHOW
SPLINE_SPLINEFLAGS_CLOSED = _LibreDWG.SPLINE_SPLINEFLAGS_CLOSED
SPLINE_KNOTPARAM_CHORD = _LibreDWG.SPLINE_KNOTPARAM_CHORD
SPLINE_KNOTPARAM_SQUARE_ROOT = _LibreDWG.SPLINE_KNOTPARAM_SQUARE_ROOT
SPLINE_KNOTPARAM_UNIFORM = _LibreDWG.SPLINE_KNOTPARAM_UNIFORM
SPLINE_KNOTPARAM_CUSTOM = _LibreDWG.SPLINE_KNOTPARAM_CUSTOM
class Dwg_Entity_SPLINE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_SPLINE_parent_get, _LibreDWG.Dwg_Entity_SPLINE_parent_set)
    flag = property(_LibreDWG.Dwg_Entity_SPLINE_flag_get, _LibreDWG.Dwg_Entity_SPLINE_flag_set)
    scenario = property(_LibreDWG.Dwg_Entity_SPLINE_scenario_get, _LibreDWG.Dwg_Entity_SPLINE_scenario_set)
    degree = property(_LibreDWG.Dwg_Entity_SPLINE_degree_get, _LibreDWG.Dwg_Entity_SPLINE_degree_set)
    splineflags = property(_LibreDWG.Dwg_Entity_SPLINE_splineflags_get, _LibreDWG.Dwg_Entity_SPLINE_splineflags_set)
    knotparam = property(_LibreDWG.Dwg_Entity_SPLINE_knotparam_get, _LibreDWG.Dwg_Entity_SPLINE_knotparam_set)
    fit_tol = property(_LibreDWG.Dwg_Entity_SPLINE_fit_tol_get, _LibreDWG.Dwg_Entity_SPLINE_fit_tol_set)
    beg_tan_vec = property(_LibreDWG.Dwg_Entity_SPLINE_beg_tan_vec_get, _LibreDWG.Dwg_Entity_SPLINE_beg_tan_vec_set)
    end_tan_vec = property(_LibreDWG.Dwg_Entity_SPLINE_end_tan_vec_get, _LibreDWG.Dwg_Entity_SPLINE_end_tan_vec_set)
    closed_b = property(_LibreDWG.Dwg_Entity_SPLINE_closed_b_get, _LibreDWG.Dwg_Entity_SPLINE_closed_b_set)
    periodic = property(_LibreDWG.Dwg_Entity_SPLINE_periodic_get, _LibreDWG.Dwg_Entity_SPLINE_periodic_set)
    rational = property(_LibreDWG.Dwg_Entity_SPLINE_rational_get, _LibreDWG.Dwg_Entity_SPLINE_rational_set)
    weighted = property(_LibreDWG.Dwg_Entity_SPLINE_weighted_get, _LibreDWG.Dwg_Entity_SPLINE_weighted_set)
    knot_tol = property(_LibreDWG.Dwg_Entity_SPLINE_knot_tol_get, _LibreDWG.Dwg_Entity_SPLINE_knot_tol_set)
    ctrl_tol = property(_LibreDWG.Dwg_Entity_SPLINE_ctrl_tol_get, _LibreDWG.Dwg_Entity_SPLINE_ctrl_tol_set)
    num_fit_pts = property(_LibreDWG.Dwg_Entity_SPLINE_num_fit_pts_get, _LibreDWG.Dwg_Entity_SPLINE_num_fit_pts_set)
    fit_pts = property(_LibreDWG.Dwg_Entity_SPLINE_fit_pts_get, _LibreDWG.Dwg_Entity_SPLINE_fit_pts_set)
    num_knots = property(_LibreDWG.Dwg_Entity_SPLINE_num_knots_get, _LibreDWG.Dwg_Entity_SPLINE_num_knots_set)
    knots = property(_LibreDWG.Dwg_Entity_SPLINE_knots_get, _LibreDWG.Dwg_Entity_SPLINE_knots_set)
    num_ctrl_pts = property(_LibreDWG.Dwg_Entity_SPLINE_num_ctrl_pts_get, _LibreDWG.Dwg_Entity_SPLINE_num_ctrl_pts_set)
    ctrl_pts = property(_LibreDWG.Dwg_Entity_SPLINE_ctrl_pts_get, _LibreDWG.Dwg_Entity_SPLINE_ctrl_pts_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_SPLINE_swiginit(self, _LibreDWG.new_Dwg_Entity_SPLINE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_SPLINE

# Register Dwg_Entity_SPLINE in _LibreDWG:
_LibreDWG.Dwg_Entity_SPLINE_swigregister(Dwg_Entity_SPLINE)
class Dwg_3DSOLID_wire(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_3DSOLID_wire_parent_get, _LibreDWG.Dwg_3DSOLID_wire_parent_set)
    type = property(_LibreDWG.Dwg_3DSOLID_wire_type_get, _LibreDWG.Dwg_3DSOLID_wire_type_set)
    selection_marker = property(_LibreDWG.Dwg_3DSOLID_wire_selection_marker_get, _LibreDWG.Dwg_3DSOLID_wire_selection_marker_set)
    color = property(_LibreDWG.Dwg_3DSOLID_wire_color_get, _LibreDWG.Dwg_3DSOLID_wire_color_set)
    acis_index = property(_LibreDWG.Dwg_3DSOLID_wire_acis_index_get, _LibreDWG.Dwg_3DSOLID_wire_acis_index_set)
    num_points = property(_LibreDWG.Dwg_3DSOLID_wire_num_points_get, _LibreDWG.Dwg_3DSOLID_wire_num_points_set)
    points = property(_LibreDWG.Dwg_3DSOLID_wire_points_get, _LibreDWG.Dwg_3DSOLID_wire_points_set)
    transform_present = property(_LibreDWG.Dwg_3DSOLID_wire_transform_present_get, _LibreDWG.Dwg_3DSOLID_wire_transform_present_set)
    axis_x = property(_LibreDWG.Dwg_3DSOLID_wire_axis_x_get, _LibreDWG.Dwg_3DSOLID_wire_axis_x_set)
    axis_y = property(_LibreDWG.Dwg_3DSOLID_wire_axis_y_get, _LibreDWG.Dwg_3DSOLID_wire_axis_y_set)
    axis_z = property(_LibreDWG.Dwg_3DSOLID_wire_axis_z_get, _LibreDWG.Dwg_3DSOLID_wire_axis_z_set)
    translation = property(_LibreDWG.Dwg_3DSOLID_wire_translation_get, _LibreDWG.Dwg_3DSOLID_wire_translation_set)
    scale = property(_LibreDWG.Dwg_3DSOLID_wire_scale_get, _LibreDWG.Dwg_3DSOLID_wire_scale_set)
    has_rotation = property(_LibreDWG.Dwg_3DSOLID_wire_has_rotation_get, _LibreDWG.Dwg_3DSOLID_wire_has_rotation_set)
    has_reflection = property(_LibreDWG.Dwg_3DSOLID_wire_has_reflection_get, _LibreDWG.Dwg_3DSOLID_wire_has_reflection_set)
    has_shear = property(_LibreDWG.Dwg_3DSOLID_wire_has_shear_get, _LibreDWG.Dwg_3DSOLID_wire_has_shear_set)

    def __init__(self):
        _LibreDWG.Dwg_3DSOLID_wire_swiginit(self, _LibreDWG.new_Dwg_3DSOLID_wire())
    __swig_destroy__ = _LibreDWG.delete_Dwg_3DSOLID_wire

# Register Dwg_3DSOLID_wire in _LibreDWG:
_LibreDWG.Dwg_3DSOLID_wire_swigregister(Dwg_3DSOLID_wire)
class Dwg_3DSOLID_silhouette(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_3DSOLID_silhouette_parent_get, _LibreDWG.Dwg_3DSOLID_silhouette_parent_set)
    vp_id = property(_LibreDWG.Dwg_3DSOLID_silhouette_vp_id_get, _LibreDWG.Dwg_3DSOLID_silhouette_vp_id_set)
    vp_target = property(_LibreDWG.Dwg_3DSOLID_silhouette_vp_target_get, _LibreDWG.Dwg_3DSOLID_silhouette_vp_target_set)
    vp_dir_from_target = property(_LibreDWG.Dwg_3DSOLID_silhouette_vp_dir_from_target_get, _LibreDWG.Dwg_3DSOLID_silhouette_vp_dir_from_target_set)
    vp_up_dir = property(_LibreDWG.Dwg_3DSOLID_silhouette_vp_up_dir_get, _LibreDWG.Dwg_3DSOLID_silhouette_vp_up_dir_set)
    vp_perspective = property(_LibreDWG.Dwg_3DSOLID_silhouette_vp_perspective_get, _LibreDWG.Dwg_3DSOLID_silhouette_vp_perspective_set)
    has_wires = property(_LibreDWG.Dwg_3DSOLID_silhouette_has_wires_get, _LibreDWG.Dwg_3DSOLID_silhouette_has_wires_set)
    num_wires = property(_LibreDWG.Dwg_3DSOLID_silhouette_num_wires_get, _LibreDWG.Dwg_3DSOLID_silhouette_num_wires_set)
    wires = property(_LibreDWG.Dwg_3DSOLID_silhouette_wires_get, _LibreDWG.Dwg_3DSOLID_silhouette_wires_set)

    def __init__(self):
        _LibreDWG.Dwg_3DSOLID_silhouette_swiginit(self, _LibreDWG.new_Dwg_3DSOLID_silhouette())
    __swig_destroy__ = _LibreDWG.delete_Dwg_3DSOLID_silhouette

# Register Dwg_3DSOLID_silhouette in _LibreDWG:
_LibreDWG.Dwg_3DSOLID_silhouette_swigregister(Dwg_3DSOLID_silhouette)
class Dwg_3DSOLID_material(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_3DSOLID_material_parent_get, _LibreDWG.Dwg_3DSOLID_material_parent_set)
    array_index = property(_LibreDWG.Dwg_3DSOLID_material_array_index_get, _LibreDWG.Dwg_3DSOLID_material_array_index_set)
    mat_absref = property(_LibreDWG.Dwg_3DSOLID_material_mat_absref_get, _LibreDWG.Dwg_3DSOLID_material_mat_absref_set)
    material_handle = property(_LibreDWG.Dwg_3DSOLID_material_material_handle_get, _LibreDWG.Dwg_3DSOLID_material_material_handle_set)

    def __init__(self):
        _LibreDWG.Dwg_3DSOLID_material_swiginit(self, _LibreDWG.new_Dwg_3DSOLID_material())
    __swig_destroy__ = _LibreDWG.delete_Dwg_3DSOLID_material

# Register Dwg_3DSOLID_material in _LibreDWG:
_LibreDWG.Dwg_3DSOLID_material_swigregister(Dwg_3DSOLID_material)
class Dwg_Entity__3DSOLID(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity__3DSOLID_parent_get, _LibreDWG.Dwg_Entity__3DSOLID_parent_set)
    acis_empty = property(_LibreDWG.Dwg_Entity__3DSOLID_acis_empty_get, _LibreDWG.Dwg_Entity__3DSOLID_acis_empty_set)
    unknown = property(_LibreDWG.Dwg_Entity__3DSOLID_unknown_get, _LibreDWG.Dwg_Entity__3DSOLID_unknown_set)
    version = property(_LibreDWG.Dwg_Entity__3DSOLID_version_get, _LibreDWG.Dwg_Entity__3DSOLID_version_set)
    num_blocks = property(_LibreDWG.Dwg_Entity__3DSOLID_num_blocks_get, _LibreDWG.Dwg_Entity__3DSOLID_num_blocks_set)
    block_size = property(_LibreDWG.Dwg_Entity__3DSOLID_block_size_get, _LibreDWG.Dwg_Entity__3DSOLID_block_size_set)
    encr_sat_data = property(_LibreDWG.Dwg_Entity__3DSOLID_encr_sat_data_get, _LibreDWG.Dwg_Entity__3DSOLID_encr_sat_data_set)
    sab_size = property(_LibreDWG.Dwg_Entity__3DSOLID_sab_size_get, _LibreDWG.Dwg_Entity__3DSOLID_sab_size_set)
    acis_data = property(_LibreDWG.Dwg_Entity__3DSOLID_acis_data_get, _LibreDWG.Dwg_Entity__3DSOLID_acis_data_set)
    wireframe_data_present = property(_LibreDWG.Dwg_Entity__3DSOLID_wireframe_data_present_get, _LibreDWG.Dwg_Entity__3DSOLID_wireframe_data_present_set)
    point_present = property(_LibreDWG.Dwg_Entity__3DSOLID_point_present_get, _LibreDWG.Dwg_Entity__3DSOLID_point_present_set)
    point = property(_LibreDWG.Dwg_Entity__3DSOLID_point_get, _LibreDWG.Dwg_Entity__3DSOLID_point_set)
    isolines = property(_LibreDWG.Dwg_Entity__3DSOLID_isolines_get, _LibreDWG.Dwg_Entity__3DSOLID_isolines_set)
    isoline_present = property(_LibreDWG.Dwg_Entity__3DSOLID_isoline_present_get, _LibreDWG.Dwg_Entity__3DSOLID_isoline_present_set)
    num_wires = property(_LibreDWG.Dwg_Entity__3DSOLID_num_wires_get, _LibreDWG.Dwg_Entity__3DSOLID_num_wires_set)
    wires = property(_LibreDWG.Dwg_Entity__3DSOLID_wires_get, _LibreDWG.Dwg_Entity__3DSOLID_wires_set)
    num_silhouettes = property(_LibreDWG.Dwg_Entity__3DSOLID_num_silhouettes_get, _LibreDWG.Dwg_Entity__3DSOLID_num_silhouettes_set)
    silhouettes = property(_LibreDWG.Dwg_Entity__3DSOLID_silhouettes_get, _LibreDWG.Dwg_Entity__3DSOLID_silhouettes_set)
    _dxf_sab_converted = property(_LibreDWG.Dwg_Entity__3DSOLID__dxf_sab_converted_get, _LibreDWG.Dwg_Entity__3DSOLID__dxf_sab_converted_set)
    acis_empty2 = property(_LibreDWG.Dwg_Entity__3DSOLID_acis_empty2_get, _LibreDWG.Dwg_Entity__3DSOLID_acis_empty2_set)
    extra_acis_data = property(_LibreDWG.Dwg_Entity__3DSOLID_extra_acis_data_get, _LibreDWG.Dwg_Entity__3DSOLID_extra_acis_data_set)
    num_materials = property(_LibreDWG.Dwg_Entity__3DSOLID_num_materials_get, _LibreDWG.Dwg_Entity__3DSOLID_num_materials_set)
    materials = property(_LibreDWG.Dwg_Entity__3DSOLID_materials_get, _LibreDWG.Dwg_Entity__3DSOLID_materials_set)
    revision_guid = property(_LibreDWG.Dwg_Entity__3DSOLID_revision_guid_get, _LibreDWG.Dwg_Entity__3DSOLID_revision_guid_set)
    revision_major = property(_LibreDWG.Dwg_Entity__3DSOLID_revision_major_get, _LibreDWG.Dwg_Entity__3DSOLID_revision_major_set)
    revision_minor1 = property(_LibreDWG.Dwg_Entity__3DSOLID_revision_minor1_get, _LibreDWG.Dwg_Entity__3DSOLID_revision_minor1_set)
    revision_minor2 = property(_LibreDWG.Dwg_Entity__3DSOLID_revision_minor2_get, _LibreDWG.Dwg_Entity__3DSOLID_revision_minor2_set)
    revision_bytes = property(_LibreDWG.Dwg_Entity__3DSOLID_revision_bytes_get, _LibreDWG.Dwg_Entity__3DSOLID_revision_bytes_set)
    end_marker = property(_LibreDWG.Dwg_Entity__3DSOLID_end_marker_get, _LibreDWG.Dwg_Entity__3DSOLID_end_marker_set)
    history_id = property(_LibreDWG.Dwg_Entity__3DSOLID_history_id_get, _LibreDWG.Dwg_Entity__3DSOLID_history_id_set)
    has_revision_guid = property(_LibreDWG.Dwg_Entity__3DSOLID_has_revision_guid_get, _LibreDWG.Dwg_Entity__3DSOLID_has_revision_guid_set)
    acis_empty_bit = property(_LibreDWG.Dwg_Entity__3DSOLID_acis_empty_bit_get, _LibreDWG.Dwg_Entity__3DSOLID_acis_empty_bit_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity__3DSOLID_swiginit(self, _LibreDWG.new_Dwg_Entity__3DSOLID())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity__3DSOLID

# Register Dwg_Entity__3DSOLID in _LibreDWG:
_LibreDWG.Dwg_Entity__3DSOLID_swigregister(Dwg_Entity__3DSOLID)
class Dwg_Entity_RAY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_RAY_parent_get, _LibreDWG.Dwg_Entity_RAY_parent_set)
    point = property(_LibreDWG.Dwg_Entity_RAY_point_get, _LibreDWG.Dwg_Entity_RAY_point_set)
    vector = property(_LibreDWG.Dwg_Entity_RAY_vector_get, _LibreDWG.Dwg_Entity_RAY_vector_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_RAY_swiginit(self, _LibreDWG.new_Dwg_Entity_RAY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_RAY

# Register Dwg_Entity_RAY in _LibreDWG:
_LibreDWG.Dwg_Entity_RAY_swigregister(Dwg_Entity_RAY)
class Dwg_Object_DICTIONARY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DICTIONARY_parent_get, _LibreDWG.Dwg_Object_DICTIONARY_parent_set)
    numitems = property(_LibreDWG.Dwg_Object_DICTIONARY_numitems_get, _LibreDWG.Dwg_Object_DICTIONARY_numitems_set)
    is_hardowner = property(_LibreDWG.Dwg_Object_DICTIONARY_is_hardowner_get, _LibreDWG.Dwg_Object_DICTIONARY_is_hardowner_set)
    cloning = property(_LibreDWG.Dwg_Object_DICTIONARY_cloning_get, _LibreDWG.Dwg_Object_DICTIONARY_cloning_set)
    texts = property(_LibreDWG.Dwg_Object_DICTIONARY_texts_get, _LibreDWG.Dwg_Object_DICTIONARY_texts_set)
    itemhandles = property(_LibreDWG.Dwg_Object_DICTIONARY_itemhandles_get, _LibreDWG.Dwg_Object_DICTIONARY_itemhandles_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DICTIONARY_swiginit(self, _LibreDWG.new_Dwg_Object_DICTIONARY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DICTIONARY

# Register Dwg_Object_DICTIONARY in _LibreDWG:
_LibreDWG.Dwg_Object_DICTIONARY_swigregister(Dwg_Object_DICTIONARY)
class Dwg_Object_DICTIONARYWDFLT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DICTIONARYWDFLT_parent_get, _LibreDWG.Dwg_Object_DICTIONARYWDFLT_parent_set)
    numitems = property(_LibreDWG.Dwg_Object_DICTIONARYWDFLT_numitems_get, _LibreDWG.Dwg_Object_DICTIONARYWDFLT_numitems_set)
    is_hardowner = property(_LibreDWG.Dwg_Object_DICTIONARYWDFLT_is_hardowner_get, _LibreDWG.Dwg_Object_DICTIONARYWDFLT_is_hardowner_set)
    cloning = property(_LibreDWG.Dwg_Object_DICTIONARYWDFLT_cloning_get, _LibreDWG.Dwg_Object_DICTIONARYWDFLT_cloning_set)
    texts = property(_LibreDWG.Dwg_Object_DICTIONARYWDFLT_texts_get, _LibreDWG.Dwg_Object_DICTIONARYWDFLT_texts_set)
    itemhandles = property(_LibreDWG.Dwg_Object_DICTIONARYWDFLT_itemhandles_get, _LibreDWG.Dwg_Object_DICTIONARYWDFLT_itemhandles_set)
    defaultid = property(_LibreDWG.Dwg_Object_DICTIONARYWDFLT_defaultid_get, _LibreDWG.Dwg_Object_DICTIONARYWDFLT_defaultid_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DICTIONARYWDFLT_swiginit(self, _LibreDWG.new_Dwg_Object_DICTIONARYWDFLT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DICTIONARYWDFLT

# Register Dwg_Object_DICTIONARYWDFLT in _LibreDWG:
_LibreDWG.Dwg_Object_DICTIONARYWDFLT_swigregister(Dwg_Object_DICTIONARYWDFLT)
class Dwg_Entity_OLEFRAME(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_OLEFRAME_parent_get, _LibreDWG.Dwg_Entity_OLEFRAME_parent_set)
    flag = property(_LibreDWG.Dwg_Entity_OLEFRAME_flag_get, _LibreDWG.Dwg_Entity_OLEFRAME_flag_set)
    mode = property(_LibreDWG.Dwg_Entity_OLEFRAME_mode_get, _LibreDWG.Dwg_Entity_OLEFRAME_mode_set)
    data_size = property(_LibreDWG.Dwg_Entity_OLEFRAME_data_size_get, _LibreDWG.Dwg_Entity_OLEFRAME_data_size_set)
    data = property(_LibreDWG.Dwg_Entity_OLEFRAME_data_get, _LibreDWG.Dwg_Entity_OLEFRAME_data_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_OLEFRAME_swiginit(self, _LibreDWG.new_Dwg_Entity_OLEFRAME())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_OLEFRAME

# Register Dwg_Entity_OLEFRAME in _LibreDWG:
_LibreDWG.Dwg_Entity_OLEFRAME_swigregister(Dwg_Entity_OLEFRAME)
class Dwg_Entity_MTEXT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_MTEXT_parent_get, _LibreDWG.Dwg_Entity_MTEXT_parent_set)
    ins_pt = property(_LibreDWG.Dwg_Entity_MTEXT_ins_pt_get, _LibreDWG.Dwg_Entity_MTEXT_ins_pt_set)
    extrusion = property(_LibreDWG.Dwg_Entity_MTEXT_extrusion_get, _LibreDWG.Dwg_Entity_MTEXT_extrusion_set)
    x_axis_dir = property(_LibreDWG.Dwg_Entity_MTEXT_x_axis_dir_get, _LibreDWG.Dwg_Entity_MTEXT_x_axis_dir_set)
    rect_height = property(_LibreDWG.Dwg_Entity_MTEXT_rect_height_get, _LibreDWG.Dwg_Entity_MTEXT_rect_height_set)
    rect_width = property(_LibreDWG.Dwg_Entity_MTEXT_rect_width_get, _LibreDWG.Dwg_Entity_MTEXT_rect_width_set)
    text_height = property(_LibreDWG.Dwg_Entity_MTEXT_text_height_get, _LibreDWG.Dwg_Entity_MTEXT_text_height_set)
    attachment = property(_LibreDWG.Dwg_Entity_MTEXT_attachment_get, _LibreDWG.Dwg_Entity_MTEXT_attachment_set)
    flow_dir = property(_LibreDWG.Dwg_Entity_MTEXT_flow_dir_get, _LibreDWG.Dwg_Entity_MTEXT_flow_dir_set)
    extents_width = property(_LibreDWG.Dwg_Entity_MTEXT_extents_width_get, _LibreDWG.Dwg_Entity_MTEXT_extents_width_set)
    extents_height = property(_LibreDWG.Dwg_Entity_MTEXT_extents_height_get, _LibreDWG.Dwg_Entity_MTEXT_extents_height_set)
    text = property(_LibreDWG.Dwg_Entity_MTEXT_text_get, _LibreDWG.Dwg_Entity_MTEXT_text_set)
    style = property(_LibreDWG.Dwg_Entity_MTEXT_style_get, _LibreDWG.Dwg_Entity_MTEXT_style_set)
    linespace_style = property(_LibreDWG.Dwg_Entity_MTEXT_linespace_style_get, _LibreDWG.Dwg_Entity_MTEXT_linespace_style_set)
    linespace_factor = property(_LibreDWG.Dwg_Entity_MTEXT_linespace_factor_get, _LibreDWG.Dwg_Entity_MTEXT_linespace_factor_set)
    unknown_b0 = property(_LibreDWG.Dwg_Entity_MTEXT_unknown_b0_get, _LibreDWG.Dwg_Entity_MTEXT_unknown_b0_set)
    bg_fill_flag = property(_LibreDWG.Dwg_Entity_MTEXT_bg_fill_flag_get, _LibreDWG.Dwg_Entity_MTEXT_bg_fill_flag_set)
    bg_fill_scale = property(_LibreDWG.Dwg_Entity_MTEXT_bg_fill_scale_get, _LibreDWG.Dwg_Entity_MTEXT_bg_fill_scale_set)
    bg_fill_color = property(_LibreDWG.Dwg_Entity_MTEXT_bg_fill_color_get, _LibreDWG.Dwg_Entity_MTEXT_bg_fill_color_set)
    bg_fill_trans = property(_LibreDWG.Dwg_Entity_MTEXT_bg_fill_trans_get, _LibreDWG.Dwg_Entity_MTEXT_bg_fill_trans_set)
    is_not_annotative = property(_LibreDWG.Dwg_Entity_MTEXT_is_not_annotative_get, _LibreDWG.Dwg_Entity_MTEXT_is_not_annotative_set)
    class_version = property(_LibreDWG.Dwg_Entity_MTEXT_class_version_get, _LibreDWG.Dwg_Entity_MTEXT_class_version_set)
    default_flag = property(_LibreDWG.Dwg_Entity_MTEXT_default_flag_get, _LibreDWG.Dwg_Entity_MTEXT_default_flag_set)
    appid = property(_LibreDWG.Dwg_Entity_MTEXT_appid_get, _LibreDWG.Dwg_Entity_MTEXT_appid_set)
    ignore_attachment = property(_LibreDWG.Dwg_Entity_MTEXT_ignore_attachment_get, _LibreDWG.Dwg_Entity_MTEXT_ignore_attachment_set)
    column_type = property(_LibreDWG.Dwg_Entity_MTEXT_column_type_get, _LibreDWG.Dwg_Entity_MTEXT_column_type_set)
    numfragments = property(_LibreDWG.Dwg_Entity_MTEXT_numfragments_get, _LibreDWG.Dwg_Entity_MTEXT_numfragments_set)
    column_width = property(_LibreDWG.Dwg_Entity_MTEXT_column_width_get, _LibreDWG.Dwg_Entity_MTEXT_column_width_set)
    gutter = property(_LibreDWG.Dwg_Entity_MTEXT_gutter_get, _LibreDWG.Dwg_Entity_MTEXT_gutter_set)
    auto_height = property(_LibreDWG.Dwg_Entity_MTEXT_auto_height_get, _LibreDWG.Dwg_Entity_MTEXT_auto_height_set)
    flow_reversed = property(_LibreDWG.Dwg_Entity_MTEXT_flow_reversed_get, _LibreDWG.Dwg_Entity_MTEXT_flow_reversed_set)
    num_column_heights = property(_LibreDWG.Dwg_Entity_MTEXT_num_column_heights_get, _LibreDWG.Dwg_Entity_MTEXT_num_column_heights_set)
    column_heights = property(_LibreDWG.Dwg_Entity_MTEXT_column_heights_get, _LibreDWG.Dwg_Entity_MTEXT_column_heights_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_MTEXT_swiginit(self, _LibreDWG.new_Dwg_Entity_MTEXT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_MTEXT

# Register Dwg_Entity_MTEXT in _LibreDWG:
_LibreDWG.Dwg_Entity_MTEXT_swigregister(Dwg_Entity_MTEXT)
class Dwg_Entity_LEADER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_LEADER_parent_get, _LibreDWG.Dwg_Entity_LEADER_parent_set)
    unknown_bit_1 = property(_LibreDWG.Dwg_Entity_LEADER_unknown_bit_1_get, _LibreDWG.Dwg_Entity_LEADER_unknown_bit_1_set)
    path_type = property(_LibreDWG.Dwg_Entity_LEADER_path_type_get, _LibreDWG.Dwg_Entity_LEADER_path_type_set)
    annot_type = property(_LibreDWG.Dwg_Entity_LEADER_annot_type_get, _LibreDWG.Dwg_Entity_LEADER_annot_type_set)
    num_points = property(_LibreDWG.Dwg_Entity_LEADER_num_points_get, _LibreDWG.Dwg_Entity_LEADER_num_points_set)
    points = property(_LibreDWG.Dwg_Entity_LEADER_points_get, _LibreDWG.Dwg_Entity_LEADER_points_set)
    origin = property(_LibreDWG.Dwg_Entity_LEADER_origin_get, _LibreDWG.Dwg_Entity_LEADER_origin_set)
    extrusion = property(_LibreDWG.Dwg_Entity_LEADER_extrusion_get, _LibreDWG.Dwg_Entity_LEADER_extrusion_set)
    x_direction = property(_LibreDWG.Dwg_Entity_LEADER_x_direction_get, _LibreDWG.Dwg_Entity_LEADER_x_direction_set)
    inspt_offset = property(_LibreDWG.Dwg_Entity_LEADER_inspt_offset_get, _LibreDWG.Dwg_Entity_LEADER_inspt_offset_set)
    endptproj = property(_LibreDWG.Dwg_Entity_LEADER_endptproj_get, _LibreDWG.Dwg_Entity_LEADER_endptproj_set)
    dimgap = property(_LibreDWG.Dwg_Entity_LEADER_dimgap_get, _LibreDWG.Dwg_Entity_LEADER_dimgap_set)
    box_height = property(_LibreDWG.Dwg_Entity_LEADER_box_height_get, _LibreDWG.Dwg_Entity_LEADER_box_height_set)
    box_width = property(_LibreDWG.Dwg_Entity_LEADER_box_width_get, _LibreDWG.Dwg_Entity_LEADER_box_width_set)
    hookline_dir = property(_LibreDWG.Dwg_Entity_LEADER_hookline_dir_get, _LibreDWG.Dwg_Entity_LEADER_hookline_dir_set)
    hookline_on = property(_LibreDWG.Dwg_Entity_LEADER_hookline_on_get, _LibreDWG.Dwg_Entity_LEADER_hookline_on_set)
    arrowhead_on = property(_LibreDWG.Dwg_Entity_LEADER_arrowhead_on_get, _LibreDWG.Dwg_Entity_LEADER_arrowhead_on_set)
    arrowhead_type = property(_LibreDWG.Dwg_Entity_LEADER_arrowhead_type_get, _LibreDWG.Dwg_Entity_LEADER_arrowhead_type_set)
    dimasz = property(_LibreDWG.Dwg_Entity_LEADER_dimasz_get, _LibreDWG.Dwg_Entity_LEADER_dimasz_set)
    unknown_bit_2 = property(_LibreDWG.Dwg_Entity_LEADER_unknown_bit_2_get, _LibreDWG.Dwg_Entity_LEADER_unknown_bit_2_set)
    unknown_bit_3 = property(_LibreDWG.Dwg_Entity_LEADER_unknown_bit_3_get, _LibreDWG.Dwg_Entity_LEADER_unknown_bit_3_set)
    unknown_short_1 = property(_LibreDWG.Dwg_Entity_LEADER_unknown_short_1_get, _LibreDWG.Dwg_Entity_LEADER_unknown_short_1_set)
    byblock_color = property(_LibreDWG.Dwg_Entity_LEADER_byblock_color_get, _LibreDWG.Dwg_Entity_LEADER_byblock_color_set)
    unknown_bit_4 = property(_LibreDWG.Dwg_Entity_LEADER_unknown_bit_4_get, _LibreDWG.Dwg_Entity_LEADER_unknown_bit_4_set)
    unknown_bit_5 = property(_LibreDWG.Dwg_Entity_LEADER_unknown_bit_5_get, _LibreDWG.Dwg_Entity_LEADER_unknown_bit_5_set)
    associated_annotation = property(_LibreDWG.Dwg_Entity_LEADER_associated_annotation_get, _LibreDWG.Dwg_Entity_LEADER_associated_annotation_set)
    dimstyle = property(_LibreDWG.Dwg_Entity_LEADER_dimstyle_get, _LibreDWG.Dwg_Entity_LEADER_dimstyle_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_LEADER_swiginit(self, _LibreDWG.new_Dwg_Entity_LEADER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_LEADER

# Register Dwg_Entity_LEADER in _LibreDWG:
_LibreDWG.Dwg_Entity_LEADER_swigregister(Dwg_Entity_LEADER)
class Dwg_Entity_TOLERANCE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_TOLERANCE_parent_get, _LibreDWG.Dwg_Entity_TOLERANCE_parent_set)
    unknown_short = property(_LibreDWG.Dwg_Entity_TOLERANCE_unknown_short_get, _LibreDWG.Dwg_Entity_TOLERANCE_unknown_short_set)
    height = property(_LibreDWG.Dwg_Entity_TOLERANCE_height_get, _LibreDWG.Dwg_Entity_TOLERANCE_height_set)
    dimgap = property(_LibreDWG.Dwg_Entity_TOLERANCE_dimgap_get, _LibreDWG.Dwg_Entity_TOLERANCE_dimgap_set)
    ins_pt = property(_LibreDWG.Dwg_Entity_TOLERANCE_ins_pt_get, _LibreDWG.Dwg_Entity_TOLERANCE_ins_pt_set)
    x_direction = property(_LibreDWG.Dwg_Entity_TOLERANCE_x_direction_get, _LibreDWG.Dwg_Entity_TOLERANCE_x_direction_set)
    extrusion = property(_LibreDWG.Dwg_Entity_TOLERANCE_extrusion_get, _LibreDWG.Dwg_Entity_TOLERANCE_extrusion_set)
    text_value = property(_LibreDWG.Dwg_Entity_TOLERANCE_text_value_get, _LibreDWG.Dwg_Entity_TOLERANCE_text_value_set)
    dimstyle = property(_LibreDWG.Dwg_Entity_TOLERANCE_dimstyle_get, _LibreDWG.Dwg_Entity_TOLERANCE_dimstyle_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_TOLERANCE_swiginit(self, _LibreDWG.new_Dwg_Entity_TOLERANCE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_TOLERANCE

# Register Dwg_Entity_TOLERANCE in _LibreDWG:
_LibreDWG.Dwg_Entity_TOLERANCE_swigregister(Dwg_Entity_TOLERANCE)
class Dwg_MLINE_line(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_MLINE_line_parent_get, _LibreDWG.Dwg_MLINE_line_parent_set)
    num_segparms = property(_LibreDWG.Dwg_MLINE_line_num_segparms_get, _LibreDWG.Dwg_MLINE_line_num_segparms_set)
    segparms = property(_LibreDWG.Dwg_MLINE_line_segparms_get, _LibreDWG.Dwg_MLINE_line_segparms_set)
    num_areafillparms = property(_LibreDWG.Dwg_MLINE_line_num_areafillparms_get, _LibreDWG.Dwg_MLINE_line_num_areafillparms_set)
    areafillparms = property(_LibreDWG.Dwg_MLINE_line_areafillparms_get, _LibreDWG.Dwg_MLINE_line_areafillparms_set)

    def __init__(self):
        _LibreDWG.Dwg_MLINE_line_swiginit(self, _LibreDWG.new_Dwg_MLINE_line())
    __swig_destroy__ = _LibreDWG.delete_Dwg_MLINE_line

# Register Dwg_MLINE_line in _LibreDWG:
_LibreDWG.Dwg_MLINE_line_swigregister(Dwg_MLINE_line)
class Dwg_MLINE_vertex(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_MLINE_vertex_parent_get, _LibreDWG.Dwg_MLINE_vertex_parent_set)
    vertex = property(_LibreDWG.Dwg_MLINE_vertex_vertex_get, _LibreDWG.Dwg_MLINE_vertex_vertex_set)
    vertex_direction = property(_LibreDWG.Dwg_MLINE_vertex_vertex_direction_get, _LibreDWG.Dwg_MLINE_vertex_vertex_direction_set)
    miter_direction = property(_LibreDWG.Dwg_MLINE_vertex_miter_direction_get, _LibreDWG.Dwg_MLINE_vertex_miter_direction_set)
    num_lines = property(_LibreDWG.Dwg_MLINE_vertex_num_lines_get, _LibreDWG.Dwg_MLINE_vertex_num_lines_set)
    lines = property(_LibreDWG.Dwg_MLINE_vertex_lines_get, _LibreDWG.Dwg_MLINE_vertex_lines_set)

    def __init__(self):
        _LibreDWG.Dwg_MLINE_vertex_swiginit(self, _LibreDWG.new_Dwg_MLINE_vertex())
    __swig_destroy__ = _LibreDWG.delete_Dwg_MLINE_vertex

# Register Dwg_MLINE_vertex in _LibreDWG:
_LibreDWG.Dwg_MLINE_vertex_swigregister(Dwg_MLINE_vertex)
class Dwg_Entity_MLINE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_MLINE_parent_get, _LibreDWG.Dwg_Entity_MLINE_parent_set)
    scale = property(_LibreDWG.Dwg_Entity_MLINE_scale_get, _LibreDWG.Dwg_Entity_MLINE_scale_set)
    justification = property(_LibreDWG.Dwg_Entity_MLINE_justification_get, _LibreDWG.Dwg_Entity_MLINE_justification_set)
    base_point = property(_LibreDWG.Dwg_Entity_MLINE_base_point_get, _LibreDWG.Dwg_Entity_MLINE_base_point_set)
    extrusion = property(_LibreDWG.Dwg_Entity_MLINE_extrusion_get, _LibreDWG.Dwg_Entity_MLINE_extrusion_set)
    flags = property(_LibreDWG.Dwg_Entity_MLINE_flags_get, _LibreDWG.Dwg_Entity_MLINE_flags_set)
    num_lines = property(_LibreDWG.Dwg_Entity_MLINE_num_lines_get, _LibreDWG.Dwg_Entity_MLINE_num_lines_set)
    num_verts = property(_LibreDWG.Dwg_Entity_MLINE_num_verts_get, _LibreDWG.Dwg_Entity_MLINE_num_verts_set)
    verts = property(_LibreDWG.Dwg_Entity_MLINE_verts_get, _LibreDWG.Dwg_Entity_MLINE_verts_set)
    mlinestyle = property(_LibreDWG.Dwg_Entity_MLINE_mlinestyle_get, _LibreDWG.Dwg_Entity_MLINE_mlinestyle_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_MLINE_swiginit(self, _LibreDWG.new_Dwg_Entity_MLINE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_MLINE

# Register Dwg_Entity_MLINE in _LibreDWG:
_LibreDWG.Dwg_Entity_MLINE_swigregister(Dwg_Entity_MLINE)
class Dwg_Object_BLOCK_CONTROL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCK_CONTROL_parent_get, _LibreDWG.Dwg_Object_BLOCK_CONTROL_parent_set)
    num_entries = property(_LibreDWG.Dwg_Object_BLOCK_CONTROL_num_entries_get, _LibreDWG.Dwg_Object_BLOCK_CONTROL_num_entries_set)
    entries = property(_LibreDWG.Dwg_Object_BLOCK_CONTROL_entries_get, _LibreDWG.Dwg_Object_BLOCK_CONTROL_entries_set)
    flags_r11 = property(_LibreDWG.Dwg_Object_BLOCK_CONTROL_flags_r11_get, _LibreDWG.Dwg_Object_BLOCK_CONTROL_flags_r11_set)
    model_space = property(_LibreDWG.Dwg_Object_BLOCK_CONTROL_model_space_get, _LibreDWG.Dwg_Object_BLOCK_CONTROL_model_space_set)
    paper_space = property(_LibreDWG.Dwg_Object_BLOCK_CONTROL_paper_space_get, _LibreDWG.Dwg_Object_BLOCK_CONTROL_paper_space_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCK_CONTROL_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCK_CONTROL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCK_CONTROL

# Register Dwg_Object_BLOCK_CONTROL in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCK_CONTROL_swigregister(Dwg_Object_BLOCK_CONTROL)
class Dwg_Object_BLOCK_HEADER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_parent_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_parent_set)
    flag = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_flag_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_flag_set)
    name = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_name_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_name_set)
    used = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_used_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_used_set)
    is_xref_ref = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_is_xref_ref_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_is_xref_ref_set)
    is_xref_resolved = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_is_xref_resolved_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_is_xref_resolved_set)
    is_xref_dep = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_is_xref_dep_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_is_xref_dep_set)
    xref = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_xref_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_xref_set)
    __iterator = property(_LibreDWG.Dwg_Object_BLOCK_HEADER___iterator_get, _LibreDWG.Dwg_Object_BLOCK_HEADER___iterator_set)
    anonymous = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_anonymous_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_anonymous_set)
    hasattrs = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_hasattrs_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_hasattrs_set)
    blkisxref = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_blkisxref_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_blkisxref_set)
    xrefoverlaid = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_xrefoverlaid_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_xrefoverlaid_set)
    xref_loaded = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_xref_loaded_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_xref_loaded_set)
    num_owned = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_num_owned_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_num_owned_set)
    base_pt = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_base_pt_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_base_pt_set)
    xref_pname = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_xref_pname_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_xref_pname_set)
    num_inserts = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_num_inserts_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_num_inserts_set)
    description = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_description_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_description_set)
    preview_size = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_preview_size_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_preview_size_set)
    preview = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_preview_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_preview_set)
    insert_units = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_insert_units_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_insert_units_set)
    explodable = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_explodable_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_explodable_set)
    block_scaling = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_block_scaling_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_block_scaling_set)
    block_entity = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_block_entity_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_block_entity_set)
    first_entity = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_first_entity_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_first_entity_set)
    last_entity = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_last_entity_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_last_entity_set)
    entities = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_entities_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_entities_set)
    endblk_entity = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_endblk_entity_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_endblk_entity_set)
    inserts = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_inserts_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_inserts_set)
    layout = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_layout_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_layout_set)
    flag2 = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_flag2_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_flag2_set)
    unknown_r11 = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_unknown_r11_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_unknown_r11_set)
    block_offset_r11 = property(_LibreDWG.Dwg_Object_BLOCK_HEADER_block_offset_r11_get, _LibreDWG.Dwg_Object_BLOCK_HEADER_block_offset_r11_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCK_HEADER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCK_HEADER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCK_HEADER

# Register Dwg_Object_BLOCK_HEADER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCK_HEADER_swigregister(Dwg_Object_BLOCK_HEADER)
class Dwg_Object_LAYER_CONTROL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_LAYER_CONTROL_parent_get, _LibreDWG.Dwg_Object_LAYER_CONTROL_parent_set)
    num_entries = property(_LibreDWG.Dwg_Object_LAYER_CONTROL_num_entries_get, _LibreDWG.Dwg_Object_LAYER_CONTROL_num_entries_set)
    entries = property(_LibreDWG.Dwg_Object_LAYER_CONTROL_entries_get, _LibreDWG.Dwg_Object_LAYER_CONTROL_entries_set)
    flags_r11 = property(_LibreDWG.Dwg_Object_LAYER_CONTROL_flags_r11_get, _LibreDWG.Dwg_Object_LAYER_CONTROL_flags_r11_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_LAYER_CONTROL_swiginit(self, _LibreDWG.new_Dwg_Object_LAYER_CONTROL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_LAYER_CONTROL

# Register Dwg_Object_LAYER_CONTROL in _LibreDWG:
_LibreDWG.Dwg_Object_LAYER_CONTROL_swigregister(Dwg_Object_LAYER_CONTROL)
class Dwg_Object_LAYER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_LAYER_parent_get, _LibreDWG.Dwg_Object_LAYER_parent_set)
    flag = property(_LibreDWG.Dwg_Object_LAYER_flag_get, _LibreDWG.Dwg_Object_LAYER_flag_set)
    name = property(_LibreDWG.Dwg_Object_LAYER_name_get, _LibreDWG.Dwg_Object_LAYER_name_set)
    used = property(_LibreDWG.Dwg_Object_LAYER_used_get, _LibreDWG.Dwg_Object_LAYER_used_set)
    is_xref_ref = property(_LibreDWG.Dwg_Object_LAYER_is_xref_ref_get, _LibreDWG.Dwg_Object_LAYER_is_xref_ref_set)
    is_xref_resolved = property(_LibreDWG.Dwg_Object_LAYER_is_xref_resolved_get, _LibreDWG.Dwg_Object_LAYER_is_xref_resolved_set)
    is_xref_dep = property(_LibreDWG.Dwg_Object_LAYER_is_xref_dep_get, _LibreDWG.Dwg_Object_LAYER_is_xref_dep_set)
    xref = property(_LibreDWG.Dwg_Object_LAYER_xref_get, _LibreDWG.Dwg_Object_LAYER_xref_set)
    flag0 = property(_LibreDWG.Dwg_Object_LAYER_flag0_get, _LibreDWG.Dwg_Object_LAYER_flag0_set)
    frozen = property(_LibreDWG.Dwg_Object_LAYER_frozen_get, _LibreDWG.Dwg_Object_LAYER_frozen_set)
    off = property(_LibreDWG.Dwg_Object_LAYER_off_get, _LibreDWG.Dwg_Object_LAYER_off_set)
    frozen_in_new = property(_LibreDWG.Dwg_Object_LAYER_frozen_in_new_get, _LibreDWG.Dwg_Object_LAYER_frozen_in_new_set)
    locked = property(_LibreDWG.Dwg_Object_LAYER_locked_get, _LibreDWG.Dwg_Object_LAYER_locked_set)
    plotflag = property(_LibreDWG.Dwg_Object_LAYER_plotflag_get, _LibreDWG.Dwg_Object_LAYER_plotflag_set)
    linewt = property(_LibreDWG.Dwg_Object_LAYER_linewt_get, _LibreDWG.Dwg_Object_LAYER_linewt_set)
    color = property(_LibreDWG.Dwg_Object_LAYER_color_get, _LibreDWG.Dwg_Object_LAYER_color_set)
    plotstyle = property(_LibreDWG.Dwg_Object_LAYER_plotstyle_get, _LibreDWG.Dwg_Object_LAYER_plotstyle_set)
    material = property(_LibreDWG.Dwg_Object_LAYER_material_get, _LibreDWG.Dwg_Object_LAYER_material_set)
    ltype = property(_LibreDWG.Dwg_Object_LAYER_ltype_get, _LibreDWG.Dwg_Object_LAYER_ltype_set)
    visualstyle = property(_LibreDWG.Dwg_Object_LAYER_visualstyle_get, _LibreDWG.Dwg_Object_LAYER_visualstyle_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_LAYER_swiginit(self, _LibreDWG.new_Dwg_Object_LAYER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_LAYER

# Register Dwg_Object_LAYER in _LibreDWG:
_LibreDWG.Dwg_Object_LAYER_swigregister(Dwg_Object_LAYER)
class Dwg_Object_STYLE_CONTROL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_STYLE_CONTROL_parent_get, _LibreDWG.Dwg_Object_STYLE_CONTROL_parent_set)
    num_entries = property(_LibreDWG.Dwg_Object_STYLE_CONTROL_num_entries_get, _LibreDWG.Dwg_Object_STYLE_CONTROL_num_entries_set)
    entries = property(_LibreDWG.Dwg_Object_STYLE_CONTROL_entries_get, _LibreDWG.Dwg_Object_STYLE_CONTROL_entries_set)
    flags_r11 = property(_LibreDWG.Dwg_Object_STYLE_CONTROL_flags_r11_get, _LibreDWG.Dwg_Object_STYLE_CONTROL_flags_r11_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_STYLE_CONTROL_swiginit(self, _LibreDWG.new_Dwg_Object_STYLE_CONTROL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_STYLE_CONTROL

# Register Dwg_Object_STYLE_CONTROL in _LibreDWG:
_LibreDWG.Dwg_Object_STYLE_CONTROL_swigregister(Dwg_Object_STYLE_CONTROL)
class Dwg_Object_STYLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_STYLE_parent_get, _LibreDWG.Dwg_Object_STYLE_parent_set)
    flag = property(_LibreDWG.Dwg_Object_STYLE_flag_get, _LibreDWG.Dwg_Object_STYLE_flag_set)
    name = property(_LibreDWG.Dwg_Object_STYLE_name_get, _LibreDWG.Dwg_Object_STYLE_name_set)
    used = property(_LibreDWG.Dwg_Object_STYLE_used_get, _LibreDWG.Dwg_Object_STYLE_used_set)
    is_xref_ref = property(_LibreDWG.Dwg_Object_STYLE_is_xref_ref_get, _LibreDWG.Dwg_Object_STYLE_is_xref_ref_set)
    is_xref_resolved = property(_LibreDWG.Dwg_Object_STYLE_is_xref_resolved_get, _LibreDWG.Dwg_Object_STYLE_is_xref_resolved_set)
    is_xref_dep = property(_LibreDWG.Dwg_Object_STYLE_is_xref_dep_get, _LibreDWG.Dwg_Object_STYLE_is_xref_dep_set)
    xref = property(_LibreDWG.Dwg_Object_STYLE_xref_get, _LibreDWG.Dwg_Object_STYLE_xref_set)
    is_shape = property(_LibreDWG.Dwg_Object_STYLE_is_shape_get, _LibreDWG.Dwg_Object_STYLE_is_shape_set)
    is_vertical = property(_LibreDWG.Dwg_Object_STYLE_is_vertical_get, _LibreDWG.Dwg_Object_STYLE_is_vertical_set)
    text_size = property(_LibreDWG.Dwg_Object_STYLE_text_size_get, _LibreDWG.Dwg_Object_STYLE_text_size_set)
    width_factor = property(_LibreDWG.Dwg_Object_STYLE_width_factor_get, _LibreDWG.Dwg_Object_STYLE_width_factor_set)
    oblique_angle = property(_LibreDWG.Dwg_Object_STYLE_oblique_angle_get, _LibreDWG.Dwg_Object_STYLE_oblique_angle_set)
    generation = property(_LibreDWG.Dwg_Object_STYLE_generation_get, _LibreDWG.Dwg_Object_STYLE_generation_set)
    last_height = property(_LibreDWG.Dwg_Object_STYLE_last_height_get, _LibreDWG.Dwg_Object_STYLE_last_height_set)
    font_file = property(_LibreDWG.Dwg_Object_STYLE_font_file_get, _LibreDWG.Dwg_Object_STYLE_font_file_set)
    bigfont_file = property(_LibreDWG.Dwg_Object_STYLE_bigfont_file_get, _LibreDWG.Dwg_Object_STYLE_bigfont_file_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_STYLE_swiginit(self, _LibreDWG.new_Dwg_Object_STYLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_STYLE

# Register Dwg_Object_STYLE in _LibreDWG:
_LibreDWG.Dwg_Object_STYLE_swigregister(Dwg_Object_STYLE)
class Dwg_Object_LTYPE_CONTROL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_LTYPE_CONTROL_parent_get, _LibreDWG.Dwg_Object_LTYPE_CONTROL_parent_set)
    num_entries = property(_LibreDWG.Dwg_Object_LTYPE_CONTROL_num_entries_get, _LibreDWG.Dwg_Object_LTYPE_CONTROL_num_entries_set)
    entries = property(_LibreDWG.Dwg_Object_LTYPE_CONTROL_entries_get, _LibreDWG.Dwg_Object_LTYPE_CONTROL_entries_set)
    flags_r11 = property(_LibreDWG.Dwg_Object_LTYPE_CONTROL_flags_r11_get, _LibreDWG.Dwg_Object_LTYPE_CONTROL_flags_r11_set)
    bylayer = property(_LibreDWG.Dwg_Object_LTYPE_CONTROL_bylayer_get, _LibreDWG.Dwg_Object_LTYPE_CONTROL_bylayer_set)
    byblock = property(_LibreDWG.Dwg_Object_LTYPE_CONTROL_byblock_get, _LibreDWG.Dwg_Object_LTYPE_CONTROL_byblock_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_LTYPE_CONTROL_swiginit(self, _LibreDWG.new_Dwg_Object_LTYPE_CONTROL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_LTYPE_CONTROL

# Register Dwg_Object_LTYPE_CONTROL in _LibreDWG:
_LibreDWG.Dwg_Object_LTYPE_CONTROL_swigregister(Dwg_Object_LTYPE_CONTROL)
class Dwg_LTYPE_dash(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_LTYPE_dash_parent_get, _LibreDWG.Dwg_LTYPE_dash_parent_set)
    length = property(_LibreDWG.Dwg_LTYPE_dash_length_get, _LibreDWG.Dwg_LTYPE_dash_length_set)
    complex_shapecode = property(_LibreDWG.Dwg_LTYPE_dash_complex_shapecode_get, _LibreDWG.Dwg_LTYPE_dash_complex_shapecode_set)
    style = property(_LibreDWG.Dwg_LTYPE_dash_style_get, _LibreDWG.Dwg_LTYPE_dash_style_set)
    x_offset = property(_LibreDWG.Dwg_LTYPE_dash_x_offset_get, _LibreDWG.Dwg_LTYPE_dash_x_offset_set)
    y_offset = property(_LibreDWG.Dwg_LTYPE_dash_y_offset_get, _LibreDWG.Dwg_LTYPE_dash_y_offset_set)
    scale = property(_LibreDWG.Dwg_LTYPE_dash_scale_get, _LibreDWG.Dwg_LTYPE_dash_scale_set)
    rotation = property(_LibreDWG.Dwg_LTYPE_dash_rotation_get, _LibreDWG.Dwg_LTYPE_dash_rotation_set)
    shape_flag = property(_LibreDWG.Dwg_LTYPE_dash_shape_flag_get, _LibreDWG.Dwg_LTYPE_dash_shape_flag_set)
    text = property(_LibreDWG.Dwg_LTYPE_dash_text_get, _LibreDWG.Dwg_LTYPE_dash_text_set)

    def __init__(self):
        _LibreDWG.Dwg_LTYPE_dash_swiginit(self, _LibreDWG.new_Dwg_LTYPE_dash())
    __swig_destroy__ = _LibreDWG.delete_Dwg_LTYPE_dash

# Register Dwg_LTYPE_dash in _LibreDWG:
_LibreDWG.Dwg_LTYPE_dash_swigregister(Dwg_LTYPE_dash)
class Dwg_Object_LTYPE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_LTYPE_parent_get, _LibreDWG.Dwg_Object_LTYPE_parent_set)
    flag = property(_LibreDWG.Dwg_Object_LTYPE_flag_get, _LibreDWG.Dwg_Object_LTYPE_flag_set)
    name = property(_LibreDWG.Dwg_Object_LTYPE_name_get, _LibreDWG.Dwg_Object_LTYPE_name_set)
    used = property(_LibreDWG.Dwg_Object_LTYPE_used_get, _LibreDWG.Dwg_Object_LTYPE_used_set)
    is_xref_ref = property(_LibreDWG.Dwg_Object_LTYPE_is_xref_ref_get, _LibreDWG.Dwg_Object_LTYPE_is_xref_ref_set)
    is_xref_resolved = property(_LibreDWG.Dwg_Object_LTYPE_is_xref_resolved_get, _LibreDWG.Dwg_Object_LTYPE_is_xref_resolved_set)
    is_xref_dep = property(_LibreDWG.Dwg_Object_LTYPE_is_xref_dep_get, _LibreDWG.Dwg_Object_LTYPE_is_xref_dep_set)
    xref = property(_LibreDWG.Dwg_Object_LTYPE_xref_get, _LibreDWG.Dwg_Object_LTYPE_xref_set)
    description = property(_LibreDWG.Dwg_Object_LTYPE_description_get, _LibreDWG.Dwg_Object_LTYPE_description_set)
    pattern_len = property(_LibreDWG.Dwg_Object_LTYPE_pattern_len_get, _LibreDWG.Dwg_Object_LTYPE_pattern_len_set)
    alignment = property(_LibreDWG.Dwg_Object_LTYPE_alignment_get, _LibreDWG.Dwg_Object_LTYPE_alignment_set)
    numdashes = property(_LibreDWG.Dwg_Object_LTYPE_numdashes_get, _LibreDWG.Dwg_Object_LTYPE_numdashes_set)
    dashes = property(_LibreDWG.Dwg_Object_LTYPE_dashes_get, _LibreDWG.Dwg_Object_LTYPE_dashes_set)
    dashes_r11 = property(_LibreDWG.Dwg_Object_LTYPE_dashes_r11_get, _LibreDWG.Dwg_Object_LTYPE_dashes_r11_set)
    has_strings_area = property(_LibreDWG.Dwg_Object_LTYPE_has_strings_area_get, _LibreDWG.Dwg_Object_LTYPE_has_strings_area_set)
    strings_area = property(_LibreDWG.Dwg_Object_LTYPE_strings_area_get, _LibreDWG.Dwg_Object_LTYPE_strings_area_set)
    unknown_r11 = property(_LibreDWG.Dwg_Object_LTYPE_unknown_r11_get, _LibreDWG.Dwg_Object_LTYPE_unknown_r11_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_LTYPE_swiginit(self, _LibreDWG.new_Dwg_Object_LTYPE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_LTYPE

# Register Dwg_Object_LTYPE in _LibreDWG:
_LibreDWG.Dwg_Object_LTYPE_swigregister(Dwg_Object_LTYPE)
class Dwg_Object_VIEW_CONTROL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_VIEW_CONTROL_parent_get, _LibreDWG.Dwg_Object_VIEW_CONTROL_parent_set)
    num_entries = property(_LibreDWG.Dwg_Object_VIEW_CONTROL_num_entries_get, _LibreDWG.Dwg_Object_VIEW_CONTROL_num_entries_set)
    entries = property(_LibreDWG.Dwg_Object_VIEW_CONTROL_entries_get, _LibreDWG.Dwg_Object_VIEW_CONTROL_entries_set)
    flags_r11 = property(_LibreDWG.Dwg_Object_VIEW_CONTROL_flags_r11_get, _LibreDWG.Dwg_Object_VIEW_CONTROL_flags_r11_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_VIEW_CONTROL_swiginit(self, _LibreDWG.new_Dwg_Object_VIEW_CONTROL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_VIEW_CONTROL

# Register Dwg_Object_VIEW_CONTROL in _LibreDWG:
_LibreDWG.Dwg_Object_VIEW_CONTROL_swigregister(Dwg_Object_VIEW_CONTROL)
class Dwg_Object_VIEW(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_VIEW_parent_get, _LibreDWG.Dwg_Object_VIEW_parent_set)
    flag = property(_LibreDWG.Dwg_Object_VIEW_flag_get, _LibreDWG.Dwg_Object_VIEW_flag_set)
    name = property(_LibreDWG.Dwg_Object_VIEW_name_get, _LibreDWG.Dwg_Object_VIEW_name_set)
    used = property(_LibreDWG.Dwg_Object_VIEW_used_get, _LibreDWG.Dwg_Object_VIEW_used_set)
    is_xref_ref = property(_LibreDWG.Dwg_Object_VIEW_is_xref_ref_get, _LibreDWG.Dwg_Object_VIEW_is_xref_ref_set)
    is_xref_resolved = property(_LibreDWG.Dwg_Object_VIEW_is_xref_resolved_get, _LibreDWG.Dwg_Object_VIEW_is_xref_resolved_set)
    is_xref_dep = property(_LibreDWG.Dwg_Object_VIEW_is_xref_dep_get, _LibreDWG.Dwg_Object_VIEW_is_xref_dep_set)
    xref = property(_LibreDWG.Dwg_Object_VIEW_xref_get, _LibreDWG.Dwg_Object_VIEW_xref_set)
    VIEWSIZE = property(_LibreDWG.Dwg_Object_VIEW_VIEWSIZE_get, _LibreDWG.Dwg_Object_VIEW_VIEWSIZE_set)
    view_width = property(_LibreDWG.Dwg_Object_VIEW_view_width_get, _LibreDWG.Dwg_Object_VIEW_view_width_set)
    aspect_ratio = property(_LibreDWG.Dwg_Object_VIEW_aspect_ratio_get, _LibreDWG.Dwg_Object_VIEW_aspect_ratio_set)
    VIEWCTR = property(_LibreDWG.Dwg_Object_VIEW_VIEWCTR_get, _LibreDWG.Dwg_Object_VIEW_VIEWCTR_set)
    view_target = property(_LibreDWG.Dwg_Object_VIEW_view_target_get, _LibreDWG.Dwg_Object_VIEW_view_target_set)
    VIEWDIR = property(_LibreDWG.Dwg_Object_VIEW_VIEWDIR_get, _LibreDWG.Dwg_Object_VIEW_VIEWDIR_set)
    VIEWTWIST = property(_LibreDWG.Dwg_Object_VIEW_VIEWTWIST_get, _LibreDWG.Dwg_Object_VIEW_VIEWTWIST_set)
    LENSLENGTH = property(_LibreDWG.Dwg_Object_VIEW_LENSLENGTH_get, _LibreDWG.Dwg_Object_VIEW_LENSLENGTH_set)
    FRONTZ = property(_LibreDWG.Dwg_Object_VIEW_FRONTZ_get, _LibreDWG.Dwg_Object_VIEW_FRONTZ_set)
    BACKZ = property(_LibreDWG.Dwg_Object_VIEW_BACKZ_get, _LibreDWG.Dwg_Object_VIEW_BACKZ_set)
    VIEWMODE = property(_LibreDWG.Dwg_Object_VIEW_VIEWMODE_get, _LibreDWG.Dwg_Object_VIEW_VIEWMODE_set)
    render_mode = property(_LibreDWG.Dwg_Object_VIEW_render_mode_get, _LibreDWG.Dwg_Object_VIEW_render_mode_set)
    use_default_lights = property(_LibreDWG.Dwg_Object_VIEW_use_default_lights_get, _LibreDWG.Dwg_Object_VIEW_use_default_lights_set)
    default_lightning_type = property(_LibreDWG.Dwg_Object_VIEW_default_lightning_type_get, _LibreDWG.Dwg_Object_VIEW_default_lightning_type_set)
    brightness = property(_LibreDWG.Dwg_Object_VIEW_brightness_get, _LibreDWG.Dwg_Object_VIEW_brightness_set)
    contrast = property(_LibreDWG.Dwg_Object_VIEW_contrast_get, _LibreDWG.Dwg_Object_VIEW_contrast_set)
    ambient_color = property(_LibreDWG.Dwg_Object_VIEW_ambient_color_get, _LibreDWG.Dwg_Object_VIEW_ambient_color_set)
    is_pspace = property(_LibreDWG.Dwg_Object_VIEW_is_pspace_get, _LibreDWG.Dwg_Object_VIEW_is_pspace_set)
    associated_ucs = property(_LibreDWG.Dwg_Object_VIEW_associated_ucs_get, _LibreDWG.Dwg_Object_VIEW_associated_ucs_set)
    ucsorg = property(_LibreDWG.Dwg_Object_VIEW_ucsorg_get, _LibreDWG.Dwg_Object_VIEW_ucsorg_set)
    ucsxdir = property(_LibreDWG.Dwg_Object_VIEW_ucsxdir_get, _LibreDWG.Dwg_Object_VIEW_ucsxdir_set)
    ucsydir = property(_LibreDWG.Dwg_Object_VIEW_ucsydir_get, _LibreDWG.Dwg_Object_VIEW_ucsydir_set)
    ucs_elevation = property(_LibreDWG.Dwg_Object_VIEW_ucs_elevation_get, _LibreDWG.Dwg_Object_VIEW_ucs_elevation_set)
    UCSORTHOVIEW = property(_LibreDWG.Dwg_Object_VIEW_UCSORTHOVIEW_get, _LibreDWG.Dwg_Object_VIEW_UCSORTHOVIEW_set)
    is_camera_plottable = property(_LibreDWG.Dwg_Object_VIEW_is_camera_plottable_get, _LibreDWG.Dwg_Object_VIEW_is_camera_plottable_set)
    background = property(_LibreDWG.Dwg_Object_VIEW_background_get, _LibreDWG.Dwg_Object_VIEW_background_set)
    visualstyle = property(_LibreDWG.Dwg_Object_VIEW_visualstyle_get, _LibreDWG.Dwg_Object_VIEW_visualstyle_set)
    sun = property(_LibreDWG.Dwg_Object_VIEW_sun_get, _LibreDWG.Dwg_Object_VIEW_sun_set)
    base_ucs = property(_LibreDWG.Dwg_Object_VIEW_base_ucs_get, _LibreDWG.Dwg_Object_VIEW_base_ucs_set)
    named_ucs = property(_LibreDWG.Dwg_Object_VIEW_named_ucs_get, _LibreDWG.Dwg_Object_VIEW_named_ucs_set)
    livesection = property(_LibreDWG.Dwg_Object_VIEW_livesection_get, _LibreDWG.Dwg_Object_VIEW_livesection_set)
    flag_3d = property(_LibreDWG.Dwg_Object_VIEW_flag_3d_get, _LibreDWG.Dwg_Object_VIEW_flag_3d_set)
    unknown_r2 = property(_LibreDWG.Dwg_Object_VIEW_unknown_r2_get, _LibreDWG.Dwg_Object_VIEW_unknown_r2_set)
    unknown_r11 = property(_LibreDWG.Dwg_Object_VIEW_unknown_r11_get, _LibreDWG.Dwg_Object_VIEW_unknown_r11_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_VIEW_swiginit(self, _LibreDWG.new_Dwg_Object_VIEW())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_VIEW

# Register Dwg_Object_VIEW in _LibreDWG:
_LibreDWG.Dwg_Object_VIEW_swigregister(Dwg_Object_VIEW)
class Dwg_Object_UCS_CONTROL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_UCS_CONTROL_parent_get, _LibreDWG.Dwg_Object_UCS_CONTROL_parent_set)
    num_entries = property(_LibreDWG.Dwg_Object_UCS_CONTROL_num_entries_get, _LibreDWG.Dwg_Object_UCS_CONTROL_num_entries_set)
    entries = property(_LibreDWG.Dwg_Object_UCS_CONTROL_entries_get, _LibreDWG.Dwg_Object_UCS_CONTROL_entries_set)
    flags_r11 = property(_LibreDWG.Dwg_Object_UCS_CONTROL_flags_r11_get, _LibreDWG.Dwg_Object_UCS_CONTROL_flags_r11_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_UCS_CONTROL_swiginit(self, _LibreDWG.new_Dwg_Object_UCS_CONTROL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_UCS_CONTROL

# Register Dwg_Object_UCS_CONTROL in _LibreDWG:
_LibreDWG.Dwg_Object_UCS_CONTROL_swigregister(Dwg_Object_UCS_CONTROL)
class Dwg_UCS_orthopts(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_UCS_orthopts_parent_get, _LibreDWG.Dwg_UCS_orthopts_parent_set)
    type = property(_LibreDWG.Dwg_UCS_orthopts_type_get, _LibreDWG.Dwg_UCS_orthopts_type_set)
    pt = property(_LibreDWG.Dwg_UCS_orthopts_pt_get, _LibreDWG.Dwg_UCS_orthopts_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_UCS_orthopts_swiginit(self, _LibreDWG.new_Dwg_UCS_orthopts())
    __swig_destroy__ = _LibreDWG.delete_Dwg_UCS_orthopts

# Register Dwg_UCS_orthopts in _LibreDWG:
_LibreDWG.Dwg_UCS_orthopts_swigregister(Dwg_UCS_orthopts)
class Dwg_Object_UCS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_UCS_parent_get, _LibreDWG.Dwg_Object_UCS_parent_set)
    flag = property(_LibreDWG.Dwg_Object_UCS_flag_get, _LibreDWG.Dwg_Object_UCS_flag_set)
    name = property(_LibreDWG.Dwg_Object_UCS_name_get, _LibreDWG.Dwg_Object_UCS_name_set)
    used = property(_LibreDWG.Dwg_Object_UCS_used_get, _LibreDWG.Dwg_Object_UCS_used_set)
    is_xref_ref = property(_LibreDWG.Dwg_Object_UCS_is_xref_ref_get, _LibreDWG.Dwg_Object_UCS_is_xref_ref_set)
    is_xref_resolved = property(_LibreDWG.Dwg_Object_UCS_is_xref_resolved_get, _LibreDWG.Dwg_Object_UCS_is_xref_resolved_set)
    is_xref_dep = property(_LibreDWG.Dwg_Object_UCS_is_xref_dep_get, _LibreDWG.Dwg_Object_UCS_is_xref_dep_set)
    xref = property(_LibreDWG.Dwg_Object_UCS_xref_get, _LibreDWG.Dwg_Object_UCS_xref_set)
    ucsorg = property(_LibreDWG.Dwg_Object_UCS_ucsorg_get, _LibreDWG.Dwg_Object_UCS_ucsorg_set)
    ucsxdir = property(_LibreDWG.Dwg_Object_UCS_ucsxdir_get, _LibreDWG.Dwg_Object_UCS_ucsxdir_set)
    ucsydir = property(_LibreDWG.Dwg_Object_UCS_ucsydir_get, _LibreDWG.Dwg_Object_UCS_ucsydir_set)
    ucs_elevation = property(_LibreDWG.Dwg_Object_UCS_ucs_elevation_get, _LibreDWG.Dwg_Object_UCS_ucs_elevation_set)
    UCSORTHOVIEW = property(_LibreDWG.Dwg_Object_UCS_UCSORTHOVIEW_get, _LibreDWG.Dwg_Object_UCS_UCSORTHOVIEW_set)
    base_ucs = property(_LibreDWG.Dwg_Object_UCS_base_ucs_get, _LibreDWG.Dwg_Object_UCS_base_ucs_set)
    named_ucs = property(_LibreDWG.Dwg_Object_UCS_named_ucs_get, _LibreDWG.Dwg_Object_UCS_named_ucs_set)
    num_orthopts = property(_LibreDWG.Dwg_Object_UCS_num_orthopts_get, _LibreDWG.Dwg_Object_UCS_num_orthopts_set)
    orthopts = property(_LibreDWG.Dwg_Object_UCS_orthopts_get, _LibreDWG.Dwg_Object_UCS_orthopts_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_UCS_swiginit(self, _LibreDWG.new_Dwg_Object_UCS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_UCS

# Register Dwg_Object_UCS in _LibreDWG:
_LibreDWG.Dwg_Object_UCS_swigregister(Dwg_Object_UCS)
class Dwg_Object_VPORT_CONTROL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_VPORT_CONTROL_parent_get, _LibreDWG.Dwg_Object_VPORT_CONTROL_parent_set)
    num_entries = property(_LibreDWG.Dwg_Object_VPORT_CONTROL_num_entries_get, _LibreDWG.Dwg_Object_VPORT_CONTROL_num_entries_set)
    entries = property(_LibreDWG.Dwg_Object_VPORT_CONTROL_entries_get, _LibreDWG.Dwg_Object_VPORT_CONTROL_entries_set)
    flags_r11 = property(_LibreDWG.Dwg_Object_VPORT_CONTROL_flags_r11_get, _LibreDWG.Dwg_Object_VPORT_CONTROL_flags_r11_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_VPORT_CONTROL_swiginit(self, _LibreDWG.new_Dwg_Object_VPORT_CONTROL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_VPORT_CONTROL

# Register Dwg_Object_VPORT_CONTROL in _LibreDWG:
_LibreDWG.Dwg_Object_VPORT_CONTROL_swigregister(Dwg_Object_VPORT_CONTROL)
class Dwg_Object_VPORT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_VPORT_parent_get, _LibreDWG.Dwg_Object_VPORT_parent_set)
    flag = property(_LibreDWG.Dwg_Object_VPORT_flag_get, _LibreDWG.Dwg_Object_VPORT_flag_set)
    name = property(_LibreDWG.Dwg_Object_VPORT_name_get, _LibreDWG.Dwg_Object_VPORT_name_set)
    used = property(_LibreDWG.Dwg_Object_VPORT_used_get, _LibreDWG.Dwg_Object_VPORT_used_set)
    is_xref_ref = property(_LibreDWG.Dwg_Object_VPORT_is_xref_ref_get, _LibreDWG.Dwg_Object_VPORT_is_xref_ref_set)
    is_xref_resolved = property(_LibreDWG.Dwg_Object_VPORT_is_xref_resolved_get, _LibreDWG.Dwg_Object_VPORT_is_xref_resolved_set)
    is_xref_dep = property(_LibreDWG.Dwg_Object_VPORT_is_xref_dep_get, _LibreDWG.Dwg_Object_VPORT_is_xref_dep_set)
    xref = property(_LibreDWG.Dwg_Object_VPORT_xref_get, _LibreDWG.Dwg_Object_VPORT_xref_set)
    VIEWSIZE = property(_LibreDWG.Dwg_Object_VPORT_VIEWSIZE_get, _LibreDWG.Dwg_Object_VPORT_VIEWSIZE_set)
    view_width = property(_LibreDWG.Dwg_Object_VPORT_view_width_get, _LibreDWG.Dwg_Object_VPORT_view_width_set)
    aspect_ratio = property(_LibreDWG.Dwg_Object_VPORT_aspect_ratio_get, _LibreDWG.Dwg_Object_VPORT_aspect_ratio_set)
    VIEWCTR = property(_LibreDWG.Dwg_Object_VPORT_VIEWCTR_get, _LibreDWG.Dwg_Object_VPORT_VIEWCTR_set)
    view_target = property(_LibreDWG.Dwg_Object_VPORT_view_target_get, _LibreDWG.Dwg_Object_VPORT_view_target_set)
    VIEWDIR = property(_LibreDWG.Dwg_Object_VPORT_VIEWDIR_get, _LibreDWG.Dwg_Object_VPORT_VIEWDIR_set)
    VIEWTWIST = property(_LibreDWG.Dwg_Object_VPORT_VIEWTWIST_get, _LibreDWG.Dwg_Object_VPORT_VIEWTWIST_set)
    LENSLENGTH = property(_LibreDWG.Dwg_Object_VPORT_LENSLENGTH_get, _LibreDWG.Dwg_Object_VPORT_LENSLENGTH_set)
    FRONTZ = property(_LibreDWG.Dwg_Object_VPORT_FRONTZ_get, _LibreDWG.Dwg_Object_VPORT_FRONTZ_set)
    BACKZ = property(_LibreDWG.Dwg_Object_VPORT_BACKZ_get, _LibreDWG.Dwg_Object_VPORT_BACKZ_set)
    VIEWMODE = property(_LibreDWG.Dwg_Object_VPORT_VIEWMODE_get, _LibreDWG.Dwg_Object_VPORT_VIEWMODE_set)
    render_mode = property(_LibreDWG.Dwg_Object_VPORT_render_mode_get, _LibreDWG.Dwg_Object_VPORT_render_mode_set)
    use_default_lights = property(_LibreDWG.Dwg_Object_VPORT_use_default_lights_get, _LibreDWG.Dwg_Object_VPORT_use_default_lights_set)
    default_lightning_type = property(_LibreDWG.Dwg_Object_VPORT_default_lightning_type_get, _LibreDWG.Dwg_Object_VPORT_default_lightning_type_set)
    brightness = property(_LibreDWG.Dwg_Object_VPORT_brightness_get, _LibreDWG.Dwg_Object_VPORT_brightness_set)
    contrast = property(_LibreDWG.Dwg_Object_VPORT_contrast_get, _LibreDWG.Dwg_Object_VPORT_contrast_set)
    ambient_color = property(_LibreDWG.Dwg_Object_VPORT_ambient_color_get, _LibreDWG.Dwg_Object_VPORT_ambient_color_set)
    lower_left = property(_LibreDWG.Dwg_Object_VPORT_lower_left_get, _LibreDWG.Dwg_Object_VPORT_lower_left_set)
    upper_right = property(_LibreDWG.Dwg_Object_VPORT_upper_right_get, _LibreDWG.Dwg_Object_VPORT_upper_right_set)
    UCSFOLLOW = property(_LibreDWG.Dwg_Object_VPORT_UCSFOLLOW_get, _LibreDWG.Dwg_Object_VPORT_UCSFOLLOW_set)
    circle_zoom = property(_LibreDWG.Dwg_Object_VPORT_circle_zoom_get, _LibreDWG.Dwg_Object_VPORT_circle_zoom_set)
    FASTZOOM = property(_LibreDWG.Dwg_Object_VPORT_FASTZOOM_get, _LibreDWG.Dwg_Object_VPORT_FASTZOOM_set)
    UCSICON = property(_LibreDWG.Dwg_Object_VPORT_UCSICON_get, _LibreDWG.Dwg_Object_VPORT_UCSICON_set)
    GRIDMODE = property(_LibreDWG.Dwg_Object_VPORT_GRIDMODE_get, _LibreDWG.Dwg_Object_VPORT_GRIDMODE_set)
    GRIDUNIT = property(_LibreDWG.Dwg_Object_VPORT_GRIDUNIT_get, _LibreDWG.Dwg_Object_VPORT_GRIDUNIT_set)
    SNAPMODE = property(_LibreDWG.Dwg_Object_VPORT_SNAPMODE_get, _LibreDWG.Dwg_Object_VPORT_SNAPMODE_set)
    SNAPSTYLE = property(_LibreDWG.Dwg_Object_VPORT_SNAPSTYLE_get, _LibreDWG.Dwg_Object_VPORT_SNAPSTYLE_set)
    SNAPISOPAIR = property(_LibreDWG.Dwg_Object_VPORT_SNAPISOPAIR_get, _LibreDWG.Dwg_Object_VPORT_SNAPISOPAIR_set)
    SNAPANG = property(_LibreDWG.Dwg_Object_VPORT_SNAPANG_get, _LibreDWG.Dwg_Object_VPORT_SNAPANG_set)
    SNAPBASE = property(_LibreDWG.Dwg_Object_VPORT_SNAPBASE_get, _LibreDWG.Dwg_Object_VPORT_SNAPBASE_set)
    SNAPUNIT = property(_LibreDWG.Dwg_Object_VPORT_SNAPUNIT_get, _LibreDWG.Dwg_Object_VPORT_SNAPUNIT_set)
    ucs_at_origin = property(_LibreDWG.Dwg_Object_VPORT_ucs_at_origin_get, _LibreDWG.Dwg_Object_VPORT_ucs_at_origin_set)
    UCSVP = property(_LibreDWG.Dwg_Object_VPORT_UCSVP_get, _LibreDWG.Dwg_Object_VPORT_UCSVP_set)
    ucsorg = property(_LibreDWG.Dwg_Object_VPORT_ucsorg_get, _LibreDWG.Dwg_Object_VPORT_ucsorg_set)
    ucsxdir = property(_LibreDWG.Dwg_Object_VPORT_ucsxdir_get, _LibreDWG.Dwg_Object_VPORT_ucsxdir_set)
    ucsydir = property(_LibreDWG.Dwg_Object_VPORT_ucsydir_get, _LibreDWG.Dwg_Object_VPORT_ucsydir_set)
    ucs_elevation = property(_LibreDWG.Dwg_Object_VPORT_ucs_elevation_get, _LibreDWG.Dwg_Object_VPORT_ucs_elevation_set)
    UCSORTHOVIEW = property(_LibreDWG.Dwg_Object_VPORT_UCSORTHOVIEW_get, _LibreDWG.Dwg_Object_VPORT_UCSORTHOVIEW_set)
    grid_flags = property(_LibreDWG.Dwg_Object_VPORT_grid_flags_get, _LibreDWG.Dwg_Object_VPORT_grid_flags_set)
    grid_major = property(_LibreDWG.Dwg_Object_VPORT_grid_major_get, _LibreDWG.Dwg_Object_VPORT_grid_major_set)
    background = property(_LibreDWG.Dwg_Object_VPORT_background_get, _LibreDWG.Dwg_Object_VPORT_background_set)
    visualstyle = property(_LibreDWG.Dwg_Object_VPORT_visualstyle_get, _LibreDWG.Dwg_Object_VPORT_visualstyle_set)
    sun = property(_LibreDWG.Dwg_Object_VPORT_sun_get, _LibreDWG.Dwg_Object_VPORT_sun_set)
    named_ucs = property(_LibreDWG.Dwg_Object_VPORT_named_ucs_get, _LibreDWG.Dwg_Object_VPORT_named_ucs_set)
    base_ucs = property(_LibreDWG.Dwg_Object_VPORT_base_ucs_get, _LibreDWG.Dwg_Object_VPORT_base_ucs_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_VPORT_swiginit(self, _LibreDWG.new_Dwg_Object_VPORT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_VPORT

# Register Dwg_Object_VPORT in _LibreDWG:
_LibreDWG.Dwg_Object_VPORT_swigregister(Dwg_Object_VPORT)
class Dwg_Object_APPID_CONTROL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_APPID_CONTROL_parent_get, _LibreDWG.Dwg_Object_APPID_CONTROL_parent_set)
    num_entries = property(_LibreDWG.Dwg_Object_APPID_CONTROL_num_entries_get, _LibreDWG.Dwg_Object_APPID_CONTROL_num_entries_set)
    entries = property(_LibreDWG.Dwg_Object_APPID_CONTROL_entries_get, _LibreDWG.Dwg_Object_APPID_CONTROL_entries_set)
    flags_r11 = property(_LibreDWG.Dwg_Object_APPID_CONTROL_flags_r11_get, _LibreDWG.Dwg_Object_APPID_CONTROL_flags_r11_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_APPID_CONTROL_swiginit(self, _LibreDWG.new_Dwg_Object_APPID_CONTROL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_APPID_CONTROL

# Register Dwg_Object_APPID_CONTROL in _LibreDWG:
_LibreDWG.Dwg_Object_APPID_CONTROL_swigregister(Dwg_Object_APPID_CONTROL)
class Dwg_Object_APPID(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_APPID_parent_get, _LibreDWG.Dwg_Object_APPID_parent_set)
    flag = property(_LibreDWG.Dwg_Object_APPID_flag_get, _LibreDWG.Dwg_Object_APPID_flag_set)
    name = property(_LibreDWG.Dwg_Object_APPID_name_get, _LibreDWG.Dwg_Object_APPID_name_set)
    used = property(_LibreDWG.Dwg_Object_APPID_used_get, _LibreDWG.Dwg_Object_APPID_used_set)
    is_xref_ref = property(_LibreDWG.Dwg_Object_APPID_is_xref_ref_get, _LibreDWG.Dwg_Object_APPID_is_xref_ref_set)
    is_xref_resolved = property(_LibreDWG.Dwg_Object_APPID_is_xref_resolved_get, _LibreDWG.Dwg_Object_APPID_is_xref_resolved_set)
    is_xref_dep = property(_LibreDWG.Dwg_Object_APPID_is_xref_dep_get, _LibreDWG.Dwg_Object_APPID_is_xref_dep_set)
    xref = property(_LibreDWG.Dwg_Object_APPID_xref_get, _LibreDWG.Dwg_Object_APPID_xref_set)
    unknown = property(_LibreDWG.Dwg_Object_APPID_unknown_get, _LibreDWG.Dwg_Object_APPID_unknown_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_APPID_swiginit(self, _LibreDWG.new_Dwg_Object_APPID())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_APPID

# Register Dwg_Object_APPID in _LibreDWG:
_LibreDWG.Dwg_Object_APPID_swigregister(Dwg_Object_APPID)
class Dwg_Object_DIMSTYLE_CONTROL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_parent_get, _LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_parent_set)
    num_entries = property(_LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_num_entries_get, _LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_num_entries_set)
    entries = property(_LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_entries_get, _LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_entries_set)
    flags_r11 = property(_LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_flags_r11_get, _LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_flags_r11_set)
    num_morehandles = property(_LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_get, _LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_set)
    morehandles = property(_LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_morehandles_get, _LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_morehandles_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_swiginit(self, _LibreDWG.new_Dwg_Object_DIMSTYLE_CONTROL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DIMSTYLE_CONTROL

# Register Dwg_Object_DIMSTYLE_CONTROL in _LibreDWG:
_LibreDWG.Dwg_Object_DIMSTYLE_CONTROL_swigregister(Dwg_Object_DIMSTYLE_CONTROL)
class Dwg_Object_DIMSTYLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DIMSTYLE_parent_get, _LibreDWG.Dwg_Object_DIMSTYLE_parent_set)
    flag = property(_LibreDWG.Dwg_Object_DIMSTYLE_flag_get, _LibreDWG.Dwg_Object_DIMSTYLE_flag_set)
    name = property(_LibreDWG.Dwg_Object_DIMSTYLE_name_get, _LibreDWG.Dwg_Object_DIMSTYLE_name_set)
    used = property(_LibreDWG.Dwg_Object_DIMSTYLE_used_get, _LibreDWG.Dwg_Object_DIMSTYLE_used_set)
    is_xref_ref = property(_LibreDWG.Dwg_Object_DIMSTYLE_is_xref_ref_get, _LibreDWG.Dwg_Object_DIMSTYLE_is_xref_ref_set)
    is_xref_resolved = property(_LibreDWG.Dwg_Object_DIMSTYLE_is_xref_resolved_get, _LibreDWG.Dwg_Object_DIMSTYLE_is_xref_resolved_set)
    is_xref_dep = property(_LibreDWG.Dwg_Object_DIMSTYLE_is_xref_dep_get, _LibreDWG.Dwg_Object_DIMSTYLE_is_xref_dep_set)
    xref = property(_LibreDWG.Dwg_Object_DIMSTYLE_xref_get, _LibreDWG.Dwg_Object_DIMSTYLE_xref_set)
    DIMTOL = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTOL_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTOL_set)
    DIMLIM = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMLIM_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMLIM_set)
    DIMTIH = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTIH_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTIH_set)
    DIMTOH = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTOH_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTOH_set)
    DIMSE1 = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMSE1_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMSE1_set)
    DIMSE2 = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMSE2_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMSE2_set)
    DIMALT = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMALT_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMALT_set)
    DIMTOFL = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTOFL_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTOFL_set)
    DIMSAH = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMSAH_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMSAH_set)
    DIMTIX = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTIX_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTIX_set)
    DIMSOXD = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMSOXD_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMSOXD_set)
    DIMALTD = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMALTD_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMALTD_set)
    DIMZIN = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMZIN_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMZIN_set)
    DIMSD1 = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMSD1_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMSD1_set)
    DIMSD2 = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMSD2_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMSD2_set)
    DIMTOLJ = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTOLJ_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTOLJ_set)
    DIMJUST = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMJUST_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMJUST_set)
    DIMFIT = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMFIT_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMFIT_set)
    DIMUPT = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMUPT_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMUPT_set)
    DIMTZIN = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTZIN_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTZIN_set)
    DIMALTZ = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMALTZ_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMALTZ_set)
    DIMALTTZ = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMALTTZ_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMALTTZ_set)
    DIMTAD = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTAD_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTAD_set)
    DIMUNIT = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMUNIT_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMUNIT_set)
    DIMAUNIT = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMAUNIT_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMAUNIT_set)
    DIMDEC = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMDEC_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMDEC_set)
    DIMTDEC = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTDEC_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTDEC_set)
    DIMALTU = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMALTU_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMALTU_set)
    DIMALTTD = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMALTTD_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMALTTD_set)
    DIMSCALE = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMSCALE_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMSCALE_set)
    DIMASZ = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMASZ_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMASZ_set)
    DIMEXO = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMEXO_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMEXO_set)
    DIMDLI = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMDLI_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMDLI_set)
    DIMEXE = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMEXE_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMEXE_set)
    DIMRND = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMRND_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMRND_set)
    DIMDLE = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMDLE_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMDLE_set)
    DIMTP = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTP_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTP_set)
    DIMTM = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTM_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTM_set)
    DIMFXL = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMFXL_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMFXL_set)
    DIMJOGANG = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMJOGANG_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMJOGANG_set)
    DIMTFILL = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTFILL_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTFILL_set)
    DIMTFILLCLR = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTFILLCLR_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTFILLCLR_set)
    DIMAZIN = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMAZIN_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMAZIN_set)
    DIMARCSYM = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMARCSYM_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMARCSYM_set)
    DIMTXT = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTXT_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTXT_set)
    DIMCEN = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMCEN_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMCEN_set)
    DIMTSZ = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTSZ_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTSZ_set)
    DIMALTF = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMALTF_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMALTF_set)
    DIMLFAC = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMLFAC_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMLFAC_set)
    DIMTVP = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTVP_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTVP_set)
    DIMTFAC = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTFAC_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTFAC_set)
    DIMGAP = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMGAP_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMGAP_set)
    DIMPOST = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMPOST_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMPOST_set)
    DIMAPOST = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMAPOST_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMAPOST_set)
    DIMBLK_T = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMBLK_T_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMBLK_T_set)
    DIMBLK1_T = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMBLK1_T_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMBLK1_T_set)
    DIMBLK2_T = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMBLK2_T_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMBLK2_T_set)
    DIMALTRND = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMALTRND_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMALTRND_set)
    DIMCLRD = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMCLRD_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMCLRD_set)
    DIMCLRE = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMCLRE_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMCLRE_set)
    DIMCLRT = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMCLRT_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMCLRT_set)
    DIMADEC = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMADEC_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMADEC_set)
    DIMFRAC = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMFRAC_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMFRAC_set)
    DIMLUNIT = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMLUNIT_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMLUNIT_set)
    DIMDSEP = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMDSEP_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMDSEP_set)
    DIMTMOVE = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTMOVE_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTMOVE_set)
    DIMATFIT = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMATFIT_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMATFIT_set)
    DIMFXLON = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMFXLON_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMFXLON_set)
    DIMTXTDIRECTION = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_set)
    DIMALTMZF = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMALTMZF_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMALTMZF_set)
    DIMALTMZS = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMALTMZS_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMALTMZS_set)
    DIMMZF = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMMZF_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMMZF_set)
    DIMMZS = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMMZS_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMMZS_set)
    DIMLWD = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMLWD_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMLWD_set)
    DIMLWE = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMLWE_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMLWE_set)
    flag0 = property(_LibreDWG.Dwg_Object_DIMSTYLE_flag0_get, _LibreDWG.Dwg_Object_DIMSTYLE_flag0_set)
    DIMTXSTY = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMTXSTY_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMTXSTY_set)
    DIMLDRBLK = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMLDRBLK_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMLDRBLK_set)
    DIMBLK = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMBLK_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMBLK_set)
    DIMBLK1 = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMBLK1_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMBLK1_set)
    DIMBLK2 = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMBLK2_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMBLK2_set)
    DIMLTYPE = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMLTYPE_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMLTYPE_set)
    DIMLTEX1 = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMLTEX1_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMLTEX1_set)
    DIMLTEX2 = property(_LibreDWG.Dwg_Object_DIMSTYLE_DIMLTEX2_get, _LibreDWG.Dwg_Object_DIMSTYLE_DIMLTEX2_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DIMSTYLE_swiginit(self, _LibreDWG.new_Dwg_Object_DIMSTYLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DIMSTYLE

# Register Dwg_Object_DIMSTYLE in _LibreDWG:
_LibreDWG.Dwg_Object_DIMSTYLE_swigregister(Dwg_Object_DIMSTYLE)
class Dwg_Object_VX_CONTROL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_VX_CONTROL_parent_get, _LibreDWG.Dwg_Object_VX_CONTROL_parent_set)
    num_entries = property(_LibreDWG.Dwg_Object_VX_CONTROL_num_entries_get, _LibreDWG.Dwg_Object_VX_CONTROL_num_entries_set)
    entries = property(_LibreDWG.Dwg_Object_VX_CONTROL_entries_get, _LibreDWG.Dwg_Object_VX_CONTROL_entries_set)
    flags_r11 = property(_LibreDWG.Dwg_Object_VX_CONTROL_flags_r11_get, _LibreDWG.Dwg_Object_VX_CONTROL_flags_r11_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_VX_CONTROL_swiginit(self, _LibreDWG.new_Dwg_Object_VX_CONTROL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_VX_CONTROL

# Register Dwg_Object_VX_CONTROL in _LibreDWG:
_LibreDWG.Dwg_Object_VX_CONTROL_swigregister(Dwg_Object_VX_CONTROL)
class Dwg_Object_VX_TABLE_RECORD(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_parent_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_parent_set)
    flag = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_flag_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_flag_set)
    name = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_name_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_name_set)
    used = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_used_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_used_set)
    is_xref_ref = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_is_xref_ref_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_is_xref_ref_set)
    is_xref_resolved = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_set)
    is_xref_dep = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_is_xref_dep_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_is_xref_dep_set)
    xref = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_xref_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_xref_set)
    is_on = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_is_on_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_is_on_set)
    viewport = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_viewport_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_viewport_set)
    prev_entry = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_prev_entry_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_prev_entry_set)
    vport_entity_address = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_vport_entity_address_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_vport_entity_address_set)
    r11_viewport_index = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_set)
    r11_prev_entry_index = property(_LibreDWG.Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_get, _LibreDWG.Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_VX_TABLE_RECORD_swiginit(self, _LibreDWG.new_Dwg_Object_VX_TABLE_RECORD())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_VX_TABLE_RECORD

# Register Dwg_Object_VX_TABLE_RECORD in _LibreDWG:
_LibreDWG.Dwg_Object_VX_TABLE_RECORD_swigregister(Dwg_Object_VX_TABLE_RECORD)
class Dwg_Object_GROUP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_GROUP_parent_get, _LibreDWG.Dwg_Object_GROUP_parent_set)
    name = property(_LibreDWG.Dwg_Object_GROUP_name_get, _LibreDWG.Dwg_Object_GROUP_name_set)
    unnamed = property(_LibreDWG.Dwg_Object_GROUP_unnamed_get, _LibreDWG.Dwg_Object_GROUP_unnamed_set)
    selectable = property(_LibreDWG.Dwg_Object_GROUP_selectable_get, _LibreDWG.Dwg_Object_GROUP_selectable_set)
    num_groups = property(_LibreDWG.Dwg_Object_GROUP_num_groups_get, _LibreDWG.Dwg_Object_GROUP_num_groups_set)
    groups = property(_LibreDWG.Dwg_Object_GROUP_groups_get, _LibreDWG.Dwg_Object_GROUP_groups_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_GROUP_swiginit(self, _LibreDWG.new_Dwg_Object_GROUP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_GROUP

# Register Dwg_Object_GROUP in _LibreDWG:
_LibreDWG.Dwg_Object_GROUP_swigregister(Dwg_Object_GROUP)
class Dwg_MLINESTYLE_line(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_MLINESTYLE_line_parent_get, _LibreDWG.Dwg_MLINESTYLE_line_parent_set)
    offset = property(_LibreDWG.Dwg_MLINESTYLE_line_offset_get, _LibreDWG.Dwg_MLINESTYLE_line_offset_set)
    color = property(_LibreDWG.Dwg_MLINESTYLE_line_color_get, _LibreDWG.Dwg_MLINESTYLE_line_color_set)
    lt_index = property(_LibreDWG.Dwg_MLINESTYLE_line_lt_index_get, _LibreDWG.Dwg_MLINESTYLE_line_lt_index_set)
    lt_ltype = property(_LibreDWG.Dwg_MLINESTYLE_line_lt_ltype_get, _LibreDWG.Dwg_MLINESTYLE_line_lt_ltype_set)

    def __init__(self):
        _LibreDWG.Dwg_MLINESTYLE_line_swiginit(self, _LibreDWG.new_Dwg_MLINESTYLE_line())
    __swig_destroy__ = _LibreDWG.delete_Dwg_MLINESTYLE_line

# Register Dwg_MLINESTYLE_line in _LibreDWG:
_LibreDWG.Dwg_MLINESTYLE_line_swigregister(Dwg_MLINESTYLE_line)
class Dwg_Object_MLINESTYLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_MLINESTYLE_parent_get, _LibreDWG.Dwg_Object_MLINESTYLE_parent_set)
    name = property(_LibreDWG.Dwg_Object_MLINESTYLE_name_get, _LibreDWG.Dwg_Object_MLINESTYLE_name_set)
    description = property(_LibreDWG.Dwg_Object_MLINESTYLE_description_get, _LibreDWG.Dwg_Object_MLINESTYLE_description_set)
    flag = property(_LibreDWG.Dwg_Object_MLINESTYLE_flag_get, _LibreDWG.Dwg_Object_MLINESTYLE_flag_set)
    fill_color = property(_LibreDWG.Dwg_Object_MLINESTYLE_fill_color_get, _LibreDWG.Dwg_Object_MLINESTYLE_fill_color_set)
    start_angle = property(_LibreDWG.Dwg_Object_MLINESTYLE_start_angle_get, _LibreDWG.Dwg_Object_MLINESTYLE_start_angle_set)
    end_angle = property(_LibreDWG.Dwg_Object_MLINESTYLE_end_angle_get, _LibreDWG.Dwg_Object_MLINESTYLE_end_angle_set)
    num_lines = property(_LibreDWG.Dwg_Object_MLINESTYLE_num_lines_get, _LibreDWG.Dwg_Object_MLINESTYLE_num_lines_set)
    lines = property(_LibreDWG.Dwg_Object_MLINESTYLE_lines_get, _LibreDWG.Dwg_Object_MLINESTYLE_lines_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_MLINESTYLE_swiginit(self, _LibreDWG.new_Dwg_Object_MLINESTYLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_MLINESTYLE

# Register Dwg_Object_MLINESTYLE in _LibreDWG:
_LibreDWG.Dwg_Object_MLINESTYLE_swigregister(Dwg_Object_MLINESTYLE)
class Dwg_Entity_OLE2FRAME(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_OLE2FRAME_parent_get, _LibreDWG.Dwg_Entity_OLE2FRAME_parent_set)
    type = property(_LibreDWG.Dwg_Entity_OLE2FRAME_type_get, _LibreDWG.Dwg_Entity_OLE2FRAME_type_set)
    mode = property(_LibreDWG.Dwg_Entity_OLE2FRAME_mode_get, _LibreDWG.Dwg_Entity_OLE2FRAME_mode_set)
    lock_aspect = property(_LibreDWG.Dwg_Entity_OLE2FRAME_lock_aspect_get, _LibreDWG.Dwg_Entity_OLE2FRAME_lock_aspect_set)
    data_size = property(_LibreDWG.Dwg_Entity_OLE2FRAME_data_size_get, _LibreDWG.Dwg_Entity_OLE2FRAME_data_size_set)
    data = property(_LibreDWG.Dwg_Entity_OLE2FRAME_data_get, _LibreDWG.Dwg_Entity_OLE2FRAME_data_set)
    oleversion = property(_LibreDWG.Dwg_Entity_OLE2FRAME_oleversion_get, _LibreDWG.Dwg_Entity_OLE2FRAME_oleversion_set)
    oleclient = property(_LibreDWG.Dwg_Entity_OLE2FRAME_oleclient_get, _LibreDWG.Dwg_Entity_OLE2FRAME_oleclient_set)
    pt1 = property(_LibreDWG.Dwg_Entity_OLE2FRAME_pt1_get, _LibreDWG.Dwg_Entity_OLE2FRAME_pt1_set)
    pt2 = property(_LibreDWG.Dwg_Entity_OLE2FRAME_pt2_get, _LibreDWG.Dwg_Entity_OLE2FRAME_pt2_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_OLE2FRAME_swiginit(self, _LibreDWG.new_Dwg_Entity_OLE2FRAME())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_OLE2FRAME

# Register Dwg_Entity_OLE2FRAME in _LibreDWG:
_LibreDWG.Dwg_Entity_OLE2FRAME_swigregister(Dwg_Entity_OLE2FRAME)
class Dwg_Object_DUMMY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DUMMY_parent_get, _LibreDWG.Dwg_Object_DUMMY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DUMMY_swiginit(self, _LibreDWG.new_Dwg_Object_DUMMY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DUMMY

# Register Dwg_Object_DUMMY in _LibreDWG:
_LibreDWG.Dwg_Object_DUMMY_swigregister(Dwg_Object_DUMMY)
class Dwg_Object_LONG_TRANSACTION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_LONG_TRANSACTION_parent_get, _LibreDWG.Dwg_Object_LONG_TRANSACTION_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_LONG_TRANSACTION_swiginit(self, _LibreDWG.new_Dwg_Object_LONG_TRANSACTION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_LONG_TRANSACTION

# Register Dwg_Object_LONG_TRANSACTION in _LibreDWG:
_LibreDWG.Dwg_Object_LONG_TRANSACTION_swigregister(Dwg_Object_LONG_TRANSACTION)
class Dwg_LWPOLYLINE_width(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    start = property(_LibreDWG.Dwg_LWPOLYLINE_width_start_get, _LibreDWG.Dwg_LWPOLYLINE_width_start_set)
    end = property(_LibreDWG.Dwg_LWPOLYLINE_width_end_get, _LibreDWG.Dwg_LWPOLYLINE_width_end_set)

    def __init__(self):
        _LibreDWG.Dwg_LWPOLYLINE_width_swiginit(self, _LibreDWG.new_Dwg_LWPOLYLINE_width())
    __swig_destroy__ = _LibreDWG.delete_Dwg_LWPOLYLINE_width

# Register Dwg_LWPOLYLINE_width in _LibreDWG:
_LibreDWG.Dwg_LWPOLYLINE_width_swigregister(Dwg_LWPOLYLINE_width)
class Dwg_PROXY_LWPOLYLINE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_parent_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_parent_set)
    data_size = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_data_size_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_data_size_set)
    flags = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_flags_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_flags_set)
    const_width = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_const_width_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_const_width_set)
    elevation = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_elevation_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_elevation_set)
    thickness = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_thickness_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_thickness_set)
    extrusion = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_extrusion_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_extrusion_set)
    num_points = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_num_points_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_num_points_set)
    points = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_points_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_points_set)
    num_bulges = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_num_bulges_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_num_bulges_set)
    bulges = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_bulges_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_bulges_set)
    num_widths = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_num_widths_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_num_widths_set)
    widths = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_widths_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_widths_set)
    unknown_1 = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_unknown_1_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_unknown_1_set)
    unknown_2 = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_unknown_2_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_unknown_2_set)
    unknown_3 = property(_LibreDWG.Dwg_PROXY_LWPOLYLINE_unknown_3_get, _LibreDWG.Dwg_PROXY_LWPOLYLINE_unknown_3_set)

    def __init__(self):
        _LibreDWG.Dwg_PROXY_LWPOLYLINE_swiginit(self, _LibreDWG.new_Dwg_PROXY_LWPOLYLINE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_PROXY_LWPOLYLINE

# Register Dwg_PROXY_LWPOLYLINE in _LibreDWG:
_LibreDWG.Dwg_PROXY_LWPOLYLINE_swigregister(Dwg_PROXY_LWPOLYLINE)
class Dwg_Entity_PROXY_ENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_parent_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_parent_set)
    proxy_id = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_proxy_id_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_proxy_id_set)
    class_id = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_class_id_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_class_id_set)
    version = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_version_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_version_set)
    maint_version = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_maint_version_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_maint_version_set)
    dwg_version = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_dwg_version_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_dwg_version_set)
    from_dxf = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_from_dxf_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_from_dxf_set)
    proxy_data_size = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_proxy_data_size_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_proxy_data_size_set)
    proxy_data = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_proxy_data_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_proxy_data_set)
    data_numbits = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_data_numbits_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_data_numbits_set)
    data_size = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_data_size_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_data_size_set)
    data = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_data_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_data_set)
    num_objids = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_num_objids_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_num_objids_set)
    objids = property(_LibreDWG.Dwg_Entity_PROXY_ENTITY_objids_get, _LibreDWG.Dwg_Entity_PROXY_ENTITY_objids_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_PROXY_ENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_PROXY_ENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_PROXY_ENTITY

# Register Dwg_Entity_PROXY_ENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_PROXY_ENTITY_swigregister(Dwg_Entity_PROXY_ENTITY)
class Dwg_Object_PROXY_OBJECT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_PROXY_OBJECT_parent_get, _LibreDWG.Dwg_Object_PROXY_OBJECT_parent_set)
    proxy_id = property(_LibreDWG.Dwg_Object_PROXY_OBJECT_proxy_id_get, _LibreDWG.Dwg_Object_PROXY_OBJECT_proxy_id_set)
    class_id = property(_LibreDWG.Dwg_Object_PROXY_OBJECT_class_id_get, _LibreDWG.Dwg_Object_PROXY_OBJECT_class_id_set)
    version = property(_LibreDWG.Dwg_Object_PROXY_OBJECT_version_get, _LibreDWG.Dwg_Object_PROXY_OBJECT_version_set)
    maint_version = property(_LibreDWG.Dwg_Object_PROXY_OBJECT_maint_version_get, _LibreDWG.Dwg_Object_PROXY_OBJECT_maint_version_set)
    dwg_version = property(_LibreDWG.Dwg_Object_PROXY_OBJECT_dwg_version_get, _LibreDWG.Dwg_Object_PROXY_OBJECT_dwg_version_set)
    from_dxf = property(_LibreDWG.Dwg_Object_PROXY_OBJECT_from_dxf_get, _LibreDWG.Dwg_Object_PROXY_OBJECT_from_dxf_set)
    data_numbits = property(_LibreDWG.Dwg_Object_PROXY_OBJECT_data_numbits_get, _LibreDWG.Dwg_Object_PROXY_OBJECT_data_numbits_set)
    data_size = property(_LibreDWG.Dwg_Object_PROXY_OBJECT_data_size_get, _LibreDWG.Dwg_Object_PROXY_OBJECT_data_size_set)
    data = property(_LibreDWG.Dwg_Object_PROXY_OBJECT_data_get, _LibreDWG.Dwg_Object_PROXY_OBJECT_data_set)
    num_objids = property(_LibreDWG.Dwg_Object_PROXY_OBJECT_num_objids_get, _LibreDWG.Dwg_Object_PROXY_OBJECT_num_objids_set)
    objids = property(_LibreDWG.Dwg_Object_PROXY_OBJECT_objids_get, _LibreDWG.Dwg_Object_PROXY_OBJECT_objids_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_PROXY_OBJECT_swiginit(self, _LibreDWG.new_Dwg_Object_PROXY_OBJECT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_PROXY_OBJECT

# Register Dwg_Object_PROXY_OBJECT in _LibreDWG:
_LibreDWG.Dwg_Object_PROXY_OBJECT_swigregister(Dwg_Object_PROXY_OBJECT)
class Dwg_HATCH_Color(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_HATCH_Color_parent_get, _LibreDWG.Dwg_HATCH_Color_parent_set)
    shift_value = property(_LibreDWG.Dwg_HATCH_Color_shift_value_get, _LibreDWG.Dwg_HATCH_Color_shift_value_set)
    color = property(_LibreDWG.Dwg_HATCH_Color_color_get, _LibreDWG.Dwg_HATCH_Color_color_set)

    def __init__(self):
        _LibreDWG.Dwg_HATCH_Color_swiginit(self, _LibreDWG.new_Dwg_HATCH_Color())
    __swig_destroy__ = _LibreDWG.delete_Dwg_HATCH_Color

# Register Dwg_HATCH_Color in _LibreDWG:
_LibreDWG.Dwg_HATCH_Color_swigregister(Dwg_HATCH_Color)
class Dwg_HATCH_ControlPoint(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_HATCH_ControlPoint_parent_get, _LibreDWG.Dwg_HATCH_ControlPoint_parent_set)
    point = property(_LibreDWG.Dwg_HATCH_ControlPoint_point_get, _LibreDWG.Dwg_HATCH_ControlPoint_point_set)
    weight = property(_LibreDWG.Dwg_HATCH_ControlPoint_weight_get, _LibreDWG.Dwg_HATCH_ControlPoint_weight_set)

    def __init__(self):
        _LibreDWG.Dwg_HATCH_ControlPoint_swiginit(self, _LibreDWG.new_Dwg_HATCH_ControlPoint())
    __swig_destroy__ = _LibreDWG.delete_Dwg_HATCH_ControlPoint

# Register Dwg_HATCH_ControlPoint in _LibreDWG:
_LibreDWG.Dwg_HATCH_ControlPoint_swigregister(Dwg_HATCH_ControlPoint)
class Dwg_HATCH_PathSeg(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_HATCH_PathSeg_parent_get, _LibreDWG.Dwg_HATCH_PathSeg_parent_set)
    curve_type = property(_LibreDWG.Dwg_HATCH_PathSeg_curve_type_get, _LibreDWG.Dwg_HATCH_PathSeg_curve_type_set)
    first_endpoint = property(_LibreDWG.Dwg_HATCH_PathSeg_first_endpoint_get, _LibreDWG.Dwg_HATCH_PathSeg_first_endpoint_set)
    second_endpoint = property(_LibreDWG.Dwg_HATCH_PathSeg_second_endpoint_get, _LibreDWG.Dwg_HATCH_PathSeg_second_endpoint_set)
    center = property(_LibreDWG.Dwg_HATCH_PathSeg_center_get, _LibreDWG.Dwg_HATCH_PathSeg_center_set)
    radius = property(_LibreDWG.Dwg_HATCH_PathSeg_radius_get, _LibreDWG.Dwg_HATCH_PathSeg_radius_set)
    start_angle = property(_LibreDWG.Dwg_HATCH_PathSeg_start_angle_get, _LibreDWG.Dwg_HATCH_PathSeg_start_angle_set)
    end_angle = property(_LibreDWG.Dwg_HATCH_PathSeg_end_angle_get, _LibreDWG.Dwg_HATCH_PathSeg_end_angle_set)
    is_ccw = property(_LibreDWG.Dwg_HATCH_PathSeg_is_ccw_get, _LibreDWG.Dwg_HATCH_PathSeg_is_ccw_set)
    endpoint = property(_LibreDWG.Dwg_HATCH_PathSeg_endpoint_get, _LibreDWG.Dwg_HATCH_PathSeg_endpoint_set)
    minor_major_ratio = property(_LibreDWG.Dwg_HATCH_PathSeg_minor_major_ratio_get, _LibreDWG.Dwg_HATCH_PathSeg_minor_major_ratio_set)
    degree = property(_LibreDWG.Dwg_HATCH_PathSeg_degree_get, _LibreDWG.Dwg_HATCH_PathSeg_degree_set)
    is_rational = property(_LibreDWG.Dwg_HATCH_PathSeg_is_rational_get, _LibreDWG.Dwg_HATCH_PathSeg_is_rational_set)
    is_periodic = property(_LibreDWG.Dwg_HATCH_PathSeg_is_periodic_get, _LibreDWG.Dwg_HATCH_PathSeg_is_periodic_set)
    num_knots = property(_LibreDWG.Dwg_HATCH_PathSeg_num_knots_get, _LibreDWG.Dwg_HATCH_PathSeg_num_knots_set)
    num_control_points = property(_LibreDWG.Dwg_HATCH_PathSeg_num_control_points_get, _LibreDWG.Dwg_HATCH_PathSeg_num_control_points_set)
    knots = property(_LibreDWG.Dwg_HATCH_PathSeg_knots_get, _LibreDWG.Dwg_HATCH_PathSeg_knots_set)
    control_points = property(_LibreDWG.Dwg_HATCH_PathSeg_control_points_get, _LibreDWG.Dwg_HATCH_PathSeg_control_points_set)
    num_fitpts = property(_LibreDWG.Dwg_HATCH_PathSeg_num_fitpts_get, _LibreDWG.Dwg_HATCH_PathSeg_num_fitpts_set)
    fitpts = property(_LibreDWG.Dwg_HATCH_PathSeg_fitpts_get, _LibreDWG.Dwg_HATCH_PathSeg_fitpts_set)
    start_tangent = property(_LibreDWG.Dwg_HATCH_PathSeg_start_tangent_get, _LibreDWG.Dwg_HATCH_PathSeg_start_tangent_set)
    end_tangent = property(_LibreDWG.Dwg_HATCH_PathSeg_end_tangent_get, _LibreDWG.Dwg_HATCH_PathSeg_end_tangent_set)

    def __init__(self):
        _LibreDWG.Dwg_HATCH_PathSeg_swiginit(self, _LibreDWG.new_Dwg_HATCH_PathSeg())
    __swig_destroy__ = _LibreDWG.delete_Dwg_HATCH_PathSeg

# Register Dwg_HATCH_PathSeg in _LibreDWG:
_LibreDWG.Dwg_HATCH_PathSeg_swigregister(Dwg_HATCH_PathSeg)
class Dwg_HATCH_PolylinePath(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_HATCH_PolylinePath_parent_get, _LibreDWG.Dwg_HATCH_PolylinePath_parent_set)
    point = property(_LibreDWG.Dwg_HATCH_PolylinePath_point_get, _LibreDWG.Dwg_HATCH_PolylinePath_point_set)
    bulge = property(_LibreDWG.Dwg_HATCH_PolylinePath_bulge_get, _LibreDWG.Dwg_HATCH_PolylinePath_bulge_set)

    def __init__(self):
        _LibreDWG.Dwg_HATCH_PolylinePath_swiginit(self, _LibreDWG.new_Dwg_HATCH_PolylinePath())
    __swig_destroy__ = _LibreDWG.delete_Dwg_HATCH_PolylinePath

# Register Dwg_HATCH_PolylinePath in _LibreDWG:
_LibreDWG.Dwg_HATCH_PolylinePath_swigregister(Dwg_HATCH_PolylinePath)
class Dwg_HATCH_Path(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_HATCH_Path_parent_get, _LibreDWG.Dwg_HATCH_Path_parent_set)
    flag = property(_LibreDWG.Dwg_HATCH_Path_flag_get, _LibreDWG.Dwg_HATCH_Path_flag_set)
    num_segs_or_paths = property(_LibreDWG.Dwg_HATCH_Path_num_segs_or_paths_get, _LibreDWG.Dwg_HATCH_Path_num_segs_or_paths_set)
    segs = property(_LibreDWG.Dwg_HATCH_Path_segs_get, _LibreDWG.Dwg_HATCH_Path_segs_set)
    bulges_present = property(_LibreDWG.Dwg_HATCH_Path_bulges_present_get, _LibreDWG.Dwg_HATCH_Path_bulges_present_set)
    closed = property(_LibreDWG.Dwg_HATCH_Path_closed_get, _LibreDWG.Dwg_HATCH_Path_closed_set)
    polyline_paths = property(_LibreDWG.Dwg_HATCH_Path_polyline_paths_get, _LibreDWG.Dwg_HATCH_Path_polyline_paths_set)
    num_boundary_handles = property(_LibreDWG.Dwg_HATCH_Path_num_boundary_handles_get, _LibreDWG.Dwg_HATCH_Path_num_boundary_handles_set)
    boundary_handles = property(_LibreDWG.Dwg_HATCH_Path_boundary_handles_get, _LibreDWG.Dwg_HATCH_Path_boundary_handles_set)

    def __init__(self):
        _LibreDWG.Dwg_HATCH_Path_swiginit(self, _LibreDWG.new_Dwg_HATCH_Path())
    __swig_destroy__ = _LibreDWG.delete_Dwg_HATCH_Path

# Register Dwg_HATCH_Path in _LibreDWG:
_LibreDWG.Dwg_HATCH_Path_swigregister(Dwg_HATCH_Path)
class Dwg_HATCH_DefLine(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_HATCH_DefLine_parent_get, _LibreDWG.Dwg_HATCH_DefLine_parent_set)
    angle = property(_LibreDWG.Dwg_HATCH_DefLine_angle_get, _LibreDWG.Dwg_HATCH_DefLine_angle_set)
    pt0 = property(_LibreDWG.Dwg_HATCH_DefLine_pt0_get, _LibreDWG.Dwg_HATCH_DefLine_pt0_set)
    offset = property(_LibreDWG.Dwg_HATCH_DefLine_offset_get, _LibreDWG.Dwg_HATCH_DefLine_offset_set)
    num_dashes = property(_LibreDWG.Dwg_HATCH_DefLine_num_dashes_get, _LibreDWG.Dwg_HATCH_DefLine_num_dashes_set)
    dashes = property(_LibreDWG.Dwg_HATCH_DefLine_dashes_get, _LibreDWG.Dwg_HATCH_DefLine_dashes_set)

    def __init__(self):
        _LibreDWG.Dwg_HATCH_DefLine_swiginit(self, _LibreDWG.new_Dwg_HATCH_DefLine())
    __swig_destroy__ = _LibreDWG.delete_Dwg_HATCH_DefLine

# Register Dwg_HATCH_DefLine in _LibreDWG:
_LibreDWG.Dwg_HATCH_DefLine_swigregister(Dwg_HATCH_DefLine)
class Dwg_Entity_HATCH(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_HATCH_parent_get, _LibreDWG.Dwg_Entity_HATCH_parent_set)
    is_gradient_fill = property(_LibreDWG.Dwg_Entity_HATCH_is_gradient_fill_get, _LibreDWG.Dwg_Entity_HATCH_is_gradient_fill_set)
    reserved = property(_LibreDWG.Dwg_Entity_HATCH_reserved_get, _LibreDWG.Dwg_Entity_HATCH_reserved_set)
    gradient_angle = property(_LibreDWG.Dwg_Entity_HATCH_gradient_angle_get, _LibreDWG.Dwg_Entity_HATCH_gradient_angle_set)
    gradient_shift = property(_LibreDWG.Dwg_Entity_HATCH_gradient_shift_get, _LibreDWG.Dwg_Entity_HATCH_gradient_shift_set)
    single_color_gradient = property(_LibreDWG.Dwg_Entity_HATCH_single_color_gradient_get, _LibreDWG.Dwg_Entity_HATCH_single_color_gradient_set)
    gradient_tint = property(_LibreDWG.Dwg_Entity_HATCH_gradient_tint_get, _LibreDWG.Dwg_Entity_HATCH_gradient_tint_set)
    num_colors = property(_LibreDWG.Dwg_Entity_HATCH_num_colors_get, _LibreDWG.Dwg_Entity_HATCH_num_colors_set)
    colors = property(_LibreDWG.Dwg_Entity_HATCH_colors_get, _LibreDWG.Dwg_Entity_HATCH_colors_set)
    gradient_name = property(_LibreDWG.Dwg_Entity_HATCH_gradient_name_get, _LibreDWG.Dwg_Entity_HATCH_gradient_name_set)
    elevation = property(_LibreDWG.Dwg_Entity_HATCH_elevation_get, _LibreDWG.Dwg_Entity_HATCH_elevation_set)
    extrusion = property(_LibreDWG.Dwg_Entity_HATCH_extrusion_get, _LibreDWG.Dwg_Entity_HATCH_extrusion_set)
    name = property(_LibreDWG.Dwg_Entity_HATCH_name_get, _LibreDWG.Dwg_Entity_HATCH_name_set)
    is_solid_fill = property(_LibreDWG.Dwg_Entity_HATCH_is_solid_fill_get, _LibreDWG.Dwg_Entity_HATCH_is_solid_fill_set)
    is_associative = property(_LibreDWG.Dwg_Entity_HATCH_is_associative_get, _LibreDWG.Dwg_Entity_HATCH_is_associative_set)
    num_paths = property(_LibreDWG.Dwg_Entity_HATCH_num_paths_get, _LibreDWG.Dwg_Entity_HATCH_num_paths_set)
    paths = property(_LibreDWG.Dwg_Entity_HATCH_paths_get, _LibreDWG.Dwg_Entity_HATCH_paths_set)
    style = property(_LibreDWG.Dwg_Entity_HATCH_style_get, _LibreDWG.Dwg_Entity_HATCH_style_set)
    pattern_type = property(_LibreDWG.Dwg_Entity_HATCH_pattern_type_get, _LibreDWG.Dwg_Entity_HATCH_pattern_type_set)
    angle = property(_LibreDWG.Dwg_Entity_HATCH_angle_get, _LibreDWG.Dwg_Entity_HATCH_angle_set)
    scale_spacing = property(_LibreDWG.Dwg_Entity_HATCH_scale_spacing_get, _LibreDWG.Dwg_Entity_HATCH_scale_spacing_set)
    double_flag = property(_LibreDWG.Dwg_Entity_HATCH_double_flag_get, _LibreDWG.Dwg_Entity_HATCH_double_flag_set)
    num_deflines = property(_LibreDWG.Dwg_Entity_HATCH_num_deflines_get, _LibreDWG.Dwg_Entity_HATCH_num_deflines_set)
    deflines = property(_LibreDWG.Dwg_Entity_HATCH_deflines_get, _LibreDWG.Dwg_Entity_HATCH_deflines_set)
    has_derived = property(_LibreDWG.Dwg_Entity_HATCH_has_derived_get, _LibreDWG.Dwg_Entity_HATCH_has_derived_set)
    pixel_size = property(_LibreDWG.Dwg_Entity_HATCH_pixel_size_get, _LibreDWG.Dwg_Entity_HATCH_pixel_size_set)
    num_seeds = property(_LibreDWG.Dwg_Entity_HATCH_num_seeds_get, _LibreDWG.Dwg_Entity_HATCH_num_seeds_set)
    seeds = property(_LibreDWG.Dwg_Entity_HATCH_seeds_get, _LibreDWG.Dwg_Entity_HATCH_seeds_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_HATCH_swiginit(self, _LibreDWG.new_Dwg_Entity_HATCH())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_HATCH

# Register Dwg_Entity_HATCH in _LibreDWG:
_LibreDWG.Dwg_Entity_HATCH_swigregister(Dwg_Entity_HATCH)
class Dwg_Entity_MPOLYGON(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_MPOLYGON_parent_get, _LibreDWG.Dwg_Entity_MPOLYGON_parent_set)
    is_gradient_fill = property(_LibreDWG.Dwg_Entity_MPOLYGON_is_gradient_fill_get, _LibreDWG.Dwg_Entity_MPOLYGON_is_gradient_fill_set)
    reserved = property(_LibreDWG.Dwg_Entity_MPOLYGON_reserved_get, _LibreDWG.Dwg_Entity_MPOLYGON_reserved_set)
    gradient_angle = property(_LibreDWG.Dwg_Entity_MPOLYGON_gradient_angle_get, _LibreDWG.Dwg_Entity_MPOLYGON_gradient_angle_set)
    gradient_shift = property(_LibreDWG.Dwg_Entity_MPOLYGON_gradient_shift_get, _LibreDWG.Dwg_Entity_MPOLYGON_gradient_shift_set)
    single_color_gradient = property(_LibreDWG.Dwg_Entity_MPOLYGON_single_color_gradient_get, _LibreDWG.Dwg_Entity_MPOLYGON_single_color_gradient_set)
    gradient_tint = property(_LibreDWG.Dwg_Entity_MPOLYGON_gradient_tint_get, _LibreDWG.Dwg_Entity_MPOLYGON_gradient_tint_set)
    num_colors = property(_LibreDWG.Dwg_Entity_MPOLYGON_num_colors_get, _LibreDWG.Dwg_Entity_MPOLYGON_num_colors_set)
    colors = property(_LibreDWG.Dwg_Entity_MPOLYGON_colors_get, _LibreDWG.Dwg_Entity_MPOLYGON_colors_set)
    gradient_name = property(_LibreDWG.Dwg_Entity_MPOLYGON_gradient_name_get, _LibreDWG.Dwg_Entity_MPOLYGON_gradient_name_set)
    elevation = property(_LibreDWG.Dwg_Entity_MPOLYGON_elevation_get, _LibreDWG.Dwg_Entity_MPOLYGON_elevation_set)
    extrusion = property(_LibreDWG.Dwg_Entity_MPOLYGON_extrusion_get, _LibreDWG.Dwg_Entity_MPOLYGON_extrusion_set)
    name = property(_LibreDWG.Dwg_Entity_MPOLYGON_name_get, _LibreDWG.Dwg_Entity_MPOLYGON_name_set)
    is_solid_fill = property(_LibreDWG.Dwg_Entity_MPOLYGON_is_solid_fill_get, _LibreDWG.Dwg_Entity_MPOLYGON_is_solid_fill_set)
    is_associative = property(_LibreDWG.Dwg_Entity_MPOLYGON_is_associative_get, _LibreDWG.Dwg_Entity_MPOLYGON_is_associative_set)
    num_paths = property(_LibreDWG.Dwg_Entity_MPOLYGON_num_paths_get, _LibreDWG.Dwg_Entity_MPOLYGON_num_paths_set)
    paths = property(_LibreDWG.Dwg_Entity_MPOLYGON_paths_get, _LibreDWG.Dwg_Entity_MPOLYGON_paths_set)
    style = property(_LibreDWG.Dwg_Entity_MPOLYGON_style_get, _LibreDWG.Dwg_Entity_MPOLYGON_style_set)
    pattern_type = property(_LibreDWG.Dwg_Entity_MPOLYGON_pattern_type_get, _LibreDWG.Dwg_Entity_MPOLYGON_pattern_type_set)
    angle = property(_LibreDWG.Dwg_Entity_MPOLYGON_angle_get, _LibreDWG.Dwg_Entity_MPOLYGON_angle_set)
    scale_spacing = property(_LibreDWG.Dwg_Entity_MPOLYGON_scale_spacing_get, _LibreDWG.Dwg_Entity_MPOLYGON_scale_spacing_set)
    double_flag = property(_LibreDWG.Dwg_Entity_MPOLYGON_double_flag_get, _LibreDWG.Dwg_Entity_MPOLYGON_double_flag_set)
    num_deflines = property(_LibreDWG.Dwg_Entity_MPOLYGON_num_deflines_get, _LibreDWG.Dwg_Entity_MPOLYGON_num_deflines_set)
    deflines = property(_LibreDWG.Dwg_Entity_MPOLYGON_deflines_get, _LibreDWG.Dwg_Entity_MPOLYGON_deflines_set)
    hatch_color = property(_LibreDWG.Dwg_Entity_MPOLYGON_hatch_color_get, _LibreDWG.Dwg_Entity_MPOLYGON_hatch_color_set)
    x_dir = property(_LibreDWG.Dwg_Entity_MPOLYGON_x_dir_get, _LibreDWG.Dwg_Entity_MPOLYGON_x_dir_set)
    num_boundary_handles = property(_LibreDWG.Dwg_Entity_MPOLYGON_num_boundary_handles_get, _LibreDWG.Dwg_Entity_MPOLYGON_num_boundary_handles_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_MPOLYGON_swiginit(self, _LibreDWG.new_Dwg_Entity_MPOLYGON())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_MPOLYGON

# Register Dwg_Entity_MPOLYGON in _LibreDWG:
_LibreDWG.Dwg_Entity_MPOLYGON_swigregister(Dwg_Entity_MPOLYGON)
class Dwg_Object_XRECORD(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_XRECORD_parent_get, _LibreDWG.Dwg_Object_XRECORD_parent_set)
    cloning = property(_LibreDWG.Dwg_Object_XRECORD_cloning_get, _LibreDWG.Dwg_Object_XRECORD_cloning_set)
    xdata_size = property(_LibreDWG.Dwg_Object_XRECORD_xdata_size_get, _LibreDWG.Dwg_Object_XRECORD_xdata_size_set)
    num_xdata = property(_LibreDWG.Dwg_Object_XRECORD_num_xdata_get, _LibreDWG.Dwg_Object_XRECORD_num_xdata_set)
    xdata = property(_LibreDWG.Dwg_Object_XRECORD_xdata_get, _LibreDWG.Dwg_Object_XRECORD_xdata_set)
    num_objid_handles = property(_LibreDWG.Dwg_Object_XRECORD_num_objid_handles_get, _LibreDWG.Dwg_Object_XRECORD_num_objid_handles_set)
    objid_handles = property(_LibreDWG.Dwg_Object_XRECORD_objid_handles_get, _LibreDWG.Dwg_Object_XRECORD_objid_handles_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_XRECORD_swiginit(self, _LibreDWG.new_Dwg_Object_XRECORD())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_XRECORD

# Register Dwg_Object_XRECORD in _LibreDWG:
_LibreDWG.Dwg_Object_XRECORD_swigregister(Dwg_Object_XRECORD)
class Dwg_Object_PLACEHOLDER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_PLACEHOLDER_parent_get, _LibreDWG.Dwg_Object_PLACEHOLDER_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_PLACEHOLDER_swiginit(self, _LibreDWG.new_Dwg_Object_PLACEHOLDER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_PLACEHOLDER

# Register Dwg_Object_PLACEHOLDER in _LibreDWG:
_LibreDWG.Dwg_Object_PLACEHOLDER_swigregister(Dwg_Object_PLACEHOLDER)
class Dwg_LEADER_Break(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_LEADER_Break_parent_get, _LibreDWG.Dwg_LEADER_Break_parent_set)
    start = property(_LibreDWG.Dwg_LEADER_Break_start_get, _LibreDWG.Dwg_LEADER_Break_start_set)
    end = property(_LibreDWG.Dwg_LEADER_Break_end_get, _LibreDWG.Dwg_LEADER_Break_end_set)

    def __init__(self):
        _LibreDWG.Dwg_LEADER_Break_swiginit(self, _LibreDWG.new_Dwg_LEADER_Break())
    __swig_destroy__ = _LibreDWG.delete_Dwg_LEADER_Break

# Register Dwg_LEADER_Break in _LibreDWG:
_LibreDWG.Dwg_LEADER_Break_swigregister(Dwg_LEADER_Break)
class Dwg_LEADER_Line(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_LEADER_Line_parent_get, _LibreDWG.Dwg_LEADER_Line_parent_set)
    num_points = property(_LibreDWG.Dwg_LEADER_Line_num_points_get, _LibreDWG.Dwg_LEADER_Line_num_points_set)
    points = property(_LibreDWG.Dwg_LEADER_Line_points_get, _LibreDWG.Dwg_LEADER_Line_points_set)
    num_breaks = property(_LibreDWG.Dwg_LEADER_Line_num_breaks_get, _LibreDWG.Dwg_LEADER_Line_num_breaks_set)
    breaks = property(_LibreDWG.Dwg_LEADER_Line_breaks_get, _LibreDWG.Dwg_LEADER_Line_breaks_set)
    line_index = property(_LibreDWG.Dwg_LEADER_Line_line_index_get, _LibreDWG.Dwg_LEADER_Line_line_index_set)
    type = property(_LibreDWG.Dwg_LEADER_Line_type_get, _LibreDWG.Dwg_LEADER_Line_type_set)
    color = property(_LibreDWG.Dwg_LEADER_Line_color_get, _LibreDWG.Dwg_LEADER_Line_color_set)
    ltype = property(_LibreDWG.Dwg_LEADER_Line_ltype_get, _LibreDWG.Dwg_LEADER_Line_ltype_set)
    linewt = property(_LibreDWG.Dwg_LEADER_Line_linewt_get, _LibreDWG.Dwg_LEADER_Line_linewt_set)
    arrow_size = property(_LibreDWG.Dwg_LEADER_Line_arrow_size_get, _LibreDWG.Dwg_LEADER_Line_arrow_size_set)
    arrow_handle = property(_LibreDWG.Dwg_LEADER_Line_arrow_handle_get, _LibreDWG.Dwg_LEADER_Line_arrow_handle_set)
    flags = property(_LibreDWG.Dwg_LEADER_Line_flags_get, _LibreDWG.Dwg_LEADER_Line_flags_set)

    def __init__(self):
        _LibreDWG.Dwg_LEADER_Line_swiginit(self, _LibreDWG.new_Dwg_LEADER_Line())
    __swig_destroy__ = _LibreDWG.delete_Dwg_LEADER_Line

# Register Dwg_LEADER_Line in _LibreDWG:
_LibreDWG.Dwg_LEADER_Line_swigregister(Dwg_LEADER_Line)
class Dwg_LEADER_ArrowHead(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_LEADER_ArrowHead_parent_get, _LibreDWG.Dwg_LEADER_ArrowHead_parent_set)
    is_default = property(_LibreDWG.Dwg_LEADER_ArrowHead_is_default_get, _LibreDWG.Dwg_LEADER_ArrowHead_is_default_set)
    arrowhead = property(_LibreDWG.Dwg_LEADER_ArrowHead_arrowhead_get, _LibreDWG.Dwg_LEADER_ArrowHead_arrowhead_set)

    def __init__(self):
        _LibreDWG.Dwg_LEADER_ArrowHead_swiginit(self, _LibreDWG.new_Dwg_LEADER_ArrowHead())
    __swig_destroy__ = _LibreDWG.delete_Dwg_LEADER_ArrowHead

# Register Dwg_LEADER_ArrowHead in _LibreDWG:
_LibreDWG.Dwg_LEADER_ArrowHead_swigregister(Dwg_LEADER_ArrowHead)
class Dwg_LEADER_BlockLabel(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_LEADER_BlockLabel_parent_get, _LibreDWG.Dwg_LEADER_BlockLabel_parent_set)
    attdef = property(_LibreDWG.Dwg_LEADER_BlockLabel_attdef_get, _LibreDWG.Dwg_LEADER_BlockLabel_attdef_set)
    label_text = property(_LibreDWG.Dwg_LEADER_BlockLabel_label_text_get, _LibreDWG.Dwg_LEADER_BlockLabel_label_text_set)
    ui_index = property(_LibreDWG.Dwg_LEADER_BlockLabel_ui_index_get, _LibreDWG.Dwg_LEADER_BlockLabel_ui_index_set)
    width = property(_LibreDWG.Dwg_LEADER_BlockLabel_width_get, _LibreDWG.Dwg_LEADER_BlockLabel_width_set)

    def __init__(self):
        _LibreDWG.Dwg_LEADER_BlockLabel_swiginit(self, _LibreDWG.new_Dwg_LEADER_BlockLabel())
    __swig_destroy__ = _LibreDWG.delete_Dwg_LEADER_BlockLabel

# Register Dwg_LEADER_BlockLabel in _LibreDWG:
_LibreDWG.Dwg_LEADER_BlockLabel_swigregister(Dwg_LEADER_BlockLabel)
class Dwg_LEADER_Node(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_LEADER_Node_parent_get, _LibreDWG.Dwg_LEADER_Node_parent_set)
    has_lastleaderlinepoint = property(_LibreDWG.Dwg_LEADER_Node_has_lastleaderlinepoint_get, _LibreDWG.Dwg_LEADER_Node_has_lastleaderlinepoint_set)
    has_dogleg = property(_LibreDWG.Dwg_LEADER_Node_has_dogleg_get, _LibreDWG.Dwg_LEADER_Node_has_dogleg_set)
    lastleaderlinepoint = property(_LibreDWG.Dwg_LEADER_Node_lastleaderlinepoint_get, _LibreDWG.Dwg_LEADER_Node_lastleaderlinepoint_set)
    dogleg_vector = property(_LibreDWG.Dwg_LEADER_Node_dogleg_vector_get, _LibreDWG.Dwg_LEADER_Node_dogleg_vector_set)
    branch_index = property(_LibreDWG.Dwg_LEADER_Node_branch_index_get, _LibreDWG.Dwg_LEADER_Node_branch_index_set)
    dogleg_length = property(_LibreDWG.Dwg_LEADER_Node_dogleg_length_get, _LibreDWG.Dwg_LEADER_Node_dogleg_length_set)
    num_lines = property(_LibreDWG.Dwg_LEADER_Node_num_lines_get, _LibreDWG.Dwg_LEADER_Node_num_lines_set)
    lines = property(_LibreDWG.Dwg_LEADER_Node_lines_get, _LibreDWG.Dwg_LEADER_Node_lines_set)
    num_breaks = property(_LibreDWG.Dwg_LEADER_Node_num_breaks_get, _LibreDWG.Dwg_LEADER_Node_num_breaks_set)
    breaks = property(_LibreDWG.Dwg_LEADER_Node_breaks_get, _LibreDWG.Dwg_LEADER_Node_breaks_set)
    attach_dir = property(_LibreDWG.Dwg_LEADER_Node_attach_dir_get, _LibreDWG.Dwg_LEADER_Node_attach_dir_set)

    def __init__(self):
        _LibreDWG.Dwg_LEADER_Node_swiginit(self, _LibreDWG.new_Dwg_LEADER_Node())
    __swig_destroy__ = _LibreDWG.delete_Dwg_LEADER_Node

# Register Dwg_LEADER_Node in _LibreDWG:
_LibreDWG.Dwg_LEADER_Node_swigregister(Dwg_LEADER_Node)
class Dwg_MLEADER_Content_MText(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    type = property(_LibreDWG.Dwg_MLEADER_Content_MText_type_get, _LibreDWG.Dwg_MLEADER_Content_MText_type_set)
    normal = property(_LibreDWG.Dwg_MLEADER_Content_MText_normal_get, _LibreDWG.Dwg_MLEADER_Content_MText_normal_set)
    location = property(_LibreDWG.Dwg_MLEADER_Content_MText_location_get, _LibreDWG.Dwg_MLEADER_Content_MText_location_set)
    rotation = property(_LibreDWG.Dwg_MLEADER_Content_MText_rotation_get, _LibreDWG.Dwg_MLEADER_Content_MText_rotation_set)
    style = property(_LibreDWG.Dwg_MLEADER_Content_MText_style_get, _LibreDWG.Dwg_MLEADER_Content_MText_style_set)
    direction = property(_LibreDWG.Dwg_MLEADER_Content_MText_direction_get, _LibreDWG.Dwg_MLEADER_Content_MText_direction_set)
    color = property(_LibreDWG.Dwg_MLEADER_Content_MText_color_get, _LibreDWG.Dwg_MLEADER_Content_MText_color_set)
    width = property(_LibreDWG.Dwg_MLEADER_Content_MText_width_get, _LibreDWG.Dwg_MLEADER_Content_MText_width_set)
    height = property(_LibreDWG.Dwg_MLEADER_Content_MText_height_get, _LibreDWG.Dwg_MLEADER_Content_MText_height_set)
    line_spacing_factor = property(_LibreDWG.Dwg_MLEADER_Content_MText_line_spacing_factor_get, _LibreDWG.Dwg_MLEADER_Content_MText_line_spacing_factor_set)
    default_text = property(_LibreDWG.Dwg_MLEADER_Content_MText_default_text_get, _LibreDWG.Dwg_MLEADER_Content_MText_default_text_set)
    line_spacing_style = property(_LibreDWG.Dwg_MLEADER_Content_MText_line_spacing_style_get, _LibreDWG.Dwg_MLEADER_Content_MText_line_spacing_style_set)
    alignment = property(_LibreDWG.Dwg_MLEADER_Content_MText_alignment_get, _LibreDWG.Dwg_MLEADER_Content_MText_alignment_set)
    flow = property(_LibreDWG.Dwg_MLEADER_Content_MText_flow_get, _LibreDWG.Dwg_MLEADER_Content_MText_flow_set)
    bg_color = property(_LibreDWG.Dwg_MLEADER_Content_MText_bg_color_get, _LibreDWG.Dwg_MLEADER_Content_MText_bg_color_set)
    bg_scale = property(_LibreDWG.Dwg_MLEADER_Content_MText_bg_scale_get, _LibreDWG.Dwg_MLEADER_Content_MText_bg_scale_set)
    bg_transparency = property(_LibreDWG.Dwg_MLEADER_Content_MText_bg_transparency_get, _LibreDWG.Dwg_MLEADER_Content_MText_bg_transparency_set)
    is_bg_fill = property(_LibreDWG.Dwg_MLEADER_Content_MText_is_bg_fill_get, _LibreDWG.Dwg_MLEADER_Content_MText_is_bg_fill_set)
    is_bg_mask_fill = property(_LibreDWG.Dwg_MLEADER_Content_MText_is_bg_mask_fill_get, _LibreDWG.Dwg_MLEADER_Content_MText_is_bg_mask_fill_set)
    col_type = property(_LibreDWG.Dwg_MLEADER_Content_MText_col_type_get, _LibreDWG.Dwg_MLEADER_Content_MText_col_type_set)
    is_height_auto = property(_LibreDWG.Dwg_MLEADER_Content_MText_is_height_auto_get, _LibreDWG.Dwg_MLEADER_Content_MText_is_height_auto_set)
    col_width = property(_LibreDWG.Dwg_MLEADER_Content_MText_col_width_get, _LibreDWG.Dwg_MLEADER_Content_MText_col_width_set)
    col_gutter = property(_LibreDWG.Dwg_MLEADER_Content_MText_col_gutter_get, _LibreDWG.Dwg_MLEADER_Content_MText_col_gutter_set)
    is_col_flow_reversed = property(_LibreDWG.Dwg_MLEADER_Content_MText_is_col_flow_reversed_get, _LibreDWG.Dwg_MLEADER_Content_MText_is_col_flow_reversed_set)
    num_col_sizes = property(_LibreDWG.Dwg_MLEADER_Content_MText_num_col_sizes_get, _LibreDWG.Dwg_MLEADER_Content_MText_num_col_sizes_set)
    col_sizes = property(_LibreDWG.Dwg_MLEADER_Content_MText_col_sizes_get, _LibreDWG.Dwg_MLEADER_Content_MText_col_sizes_set)
    word_break = property(_LibreDWG.Dwg_MLEADER_Content_MText_word_break_get, _LibreDWG.Dwg_MLEADER_Content_MText_word_break_set)
    unknown = property(_LibreDWG.Dwg_MLEADER_Content_MText_unknown_get, _LibreDWG.Dwg_MLEADER_Content_MText_unknown_set)

    def __init__(self):
        _LibreDWG.Dwg_MLEADER_Content_MText_swiginit(self, _LibreDWG.new_Dwg_MLEADER_Content_MText())
    __swig_destroy__ = _LibreDWG.delete_Dwg_MLEADER_Content_MText

# Register Dwg_MLEADER_Content_MText in _LibreDWG:
_LibreDWG.Dwg_MLEADER_Content_MText_swigregister(Dwg_MLEADER_Content_MText)
class Dwg_MLEADER_Content_Block(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    type = property(_LibreDWG.Dwg_MLEADER_Content_Block_type_get, _LibreDWG.Dwg_MLEADER_Content_Block_type_set)
    normal = property(_LibreDWG.Dwg_MLEADER_Content_Block_normal_get, _LibreDWG.Dwg_MLEADER_Content_Block_normal_set)
    location = property(_LibreDWG.Dwg_MLEADER_Content_Block_location_get, _LibreDWG.Dwg_MLEADER_Content_Block_location_set)
    rotation = property(_LibreDWG.Dwg_MLEADER_Content_Block_rotation_get, _LibreDWG.Dwg_MLEADER_Content_Block_rotation_set)
    block_table = property(_LibreDWG.Dwg_MLEADER_Content_Block_block_table_get, _LibreDWG.Dwg_MLEADER_Content_Block_block_table_set)
    scale = property(_LibreDWG.Dwg_MLEADER_Content_Block_scale_get, _LibreDWG.Dwg_MLEADER_Content_Block_scale_set)
    color = property(_LibreDWG.Dwg_MLEADER_Content_Block_color_get, _LibreDWG.Dwg_MLEADER_Content_Block_color_set)
    transform = property(_LibreDWG.Dwg_MLEADER_Content_Block_transform_get, _LibreDWG.Dwg_MLEADER_Content_Block_transform_set)

    def __init__(self):
        _LibreDWG.Dwg_MLEADER_Content_Block_swiginit(self, _LibreDWG.new_Dwg_MLEADER_Content_Block())
    __swig_destroy__ = _LibreDWG.delete_Dwg_MLEADER_Content_Block

# Register Dwg_MLEADER_Content_Block in _LibreDWG:
_LibreDWG.Dwg_MLEADER_Content_Block_swigregister(Dwg_MLEADER_Content_Block)
class Dwg_MLEADER_Content(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    txt = property(_LibreDWG.Dwg_MLEADER_Content_txt_get, _LibreDWG.Dwg_MLEADER_Content_txt_set)
    blk = property(_LibreDWG.Dwg_MLEADER_Content_blk_get, _LibreDWG.Dwg_MLEADER_Content_blk_set)

    def __init__(self):
        _LibreDWG.Dwg_MLEADER_Content_swiginit(self, _LibreDWG.new_Dwg_MLEADER_Content())
    __swig_destroy__ = _LibreDWG.delete_Dwg_MLEADER_Content

# Register Dwg_MLEADER_Content in _LibreDWG:
_LibreDWG.Dwg_MLEADER_Content_swigregister(Dwg_MLEADER_Content)
class Dwg_MLEADER_AnnotContext(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num_leaders = property(_LibreDWG.Dwg_MLEADER_AnnotContext_num_leaders_get, _LibreDWG.Dwg_MLEADER_AnnotContext_num_leaders_set)
    leaders = property(_LibreDWG.Dwg_MLEADER_AnnotContext_leaders_get, _LibreDWG.Dwg_MLEADER_AnnotContext_leaders_set)
    attach_dir = property(_LibreDWG.Dwg_MLEADER_AnnotContext_attach_dir_get, _LibreDWG.Dwg_MLEADER_AnnotContext_attach_dir_set)
    scale_factor = property(_LibreDWG.Dwg_MLEADER_AnnotContext_scale_factor_get, _LibreDWG.Dwg_MLEADER_AnnotContext_scale_factor_set)
    content_base = property(_LibreDWG.Dwg_MLEADER_AnnotContext_content_base_get, _LibreDWG.Dwg_MLEADER_AnnotContext_content_base_set)
    text_height = property(_LibreDWG.Dwg_MLEADER_AnnotContext_text_height_get, _LibreDWG.Dwg_MLEADER_AnnotContext_text_height_set)
    arrow_size = property(_LibreDWG.Dwg_MLEADER_AnnotContext_arrow_size_get, _LibreDWG.Dwg_MLEADER_AnnotContext_arrow_size_set)
    landing_gap = property(_LibreDWG.Dwg_MLEADER_AnnotContext_landing_gap_get, _LibreDWG.Dwg_MLEADER_AnnotContext_landing_gap_set)
    text_left = property(_LibreDWG.Dwg_MLEADER_AnnotContext_text_left_get, _LibreDWG.Dwg_MLEADER_AnnotContext_text_left_set)
    text_right = property(_LibreDWG.Dwg_MLEADER_AnnotContext_text_right_get, _LibreDWG.Dwg_MLEADER_AnnotContext_text_right_set)
    text_angletype = property(_LibreDWG.Dwg_MLEADER_AnnotContext_text_angletype_get, _LibreDWG.Dwg_MLEADER_AnnotContext_text_angletype_set)
    text_alignment = property(_LibreDWG.Dwg_MLEADER_AnnotContext_text_alignment_get, _LibreDWG.Dwg_MLEADER_AnnotContext_text_alignment_set)
    has_content_txt = property(_LibreDWG.Dwg_MLEADER_AnnotContext_has_content_txt_get, _LibreDWG.Dwg_MLEADER_AnnotContext_has_content_txt_set)
    has_content_blk = property(_LibreDWG.Dwg_MLEADER_AnnotContext_has_content_blk_get, _LibreDWG.Dwg_MLEADER_AnnotContext_has_content_blk_set)
    content = property(_LibreDWG.Dwg_MLEADER_AnnotContext_content_get, _LibreDWG.Dwg_MLEADER_AnnotContext_content_set)
    base = property(_LibreDWG.Dwg_MLEADER_AnnotContext_base_get, _LibreDWG.Dwg_MLEADER_AnnotContext_base_set)
    base_dir = property(_LibreDWG.Dwg_MLEADER_AnnotContext_base_dir_get, _LibreDWG.Dwg_MLEADER_AnnotContext_base_dir_set)
    base_vert = property(_LibreDWG.Dwg_MLEADER_AnnotContext_base_vert_get, _LibreDWG.Dwg_MLEADER_AnnotContext_base_vert_set)
    is_normal_reversed = property(_LibreDWG.Dwg_MLEADER_AnnotContext_is_normal_reversed_get, _LibreDWG.Dwg_MLEADER_AnnotContext_is_normal_reversed_set)
    text_top = property(_LibreDWG.Dwg_MLEADER_AnnotContext_text_top_get, _LibreDWG.Dwg_MLEADER_AnnotContext_text_top_set)
    text_bottom = property(_LibreDWG.Dwg_MLEADER_AnnotContext_text_bottom_get, _LibreDWG.Dwg_MLEADER_AnnotContext_text_bottom_set)

    def __init__(self):
        _LibreDWG.Dwg_MLEADER_AnnotContext_swiginit(self, _LibreDWG.new_Dwg_MLEADER_AnnotContext())
    __swig_destroy__ = _LibreDWG.delete_Dwg_MLEADER_AnnotContext

# Register Dwg_MLEADER_AnnotContext in _LibreDWG:
_LibreDWG.Dwg_MLEADER_AnnotContext_swigregister(Dwg_MLEADER_AnnotContext)
class Dwg_Entity_MULTILEADER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_MULTILEADER_parent_get, _LibreDWG.Dwg_Entity_MULTILEADER_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_MULTILEADER_class_version_get, _LibreDWG.Dwg_Entity_MULTILEADER_class_version_set)
    ctx = property(_LibreDWG.Dwg_Entity_MULTILEADER_ctx_get, _LibreDWG.Dwg_Entity_MULTILEADER_ctx_set)
    mleaderstyle = property(_LibreDWG.Dwg_Entity_MULTILEADER_mleaderstyle_get, _LibreDWG.Dwg_Entity_MULTILEADER_mleaderstyle_set)
    flags = property(_LibreDWG.Dwg_Entity_MULTILEADER_flags_get, _LibreDWG.Dwg_Entity_MULTILEADER_flags_set)
    type = property(_LibreDWG.Dwg_Entity_MULTILEADER_type_get, _LibreDWG.Dwg_Entity_MULTILEADER_type_set)
    line_color = property(_LibreDWG.Dwg_Entity_MULTILEADER_line_color_get, _LibreDWG.Dwg_Entity_MULTILEADER_line_color_set)
    line_ltype = property(_LibreDWG.Dwg_Entity_MULTILEADER_line_ltype_get, _LibreDWG.Dwg_Entity_MULTILEADER_line_ltype_set)
    line_linewt = property(_LibreDWG.Dwg_Entity_MULTILEADER_line_linewt_get, _LibreDWG.Dwg_Entity_MULTILEADER_line_linewt_set)
    has_landing = property(_LibreDWG.Dwg_Entity_MULTILEADER_has_landing_get, _LibreDWG.Dwg_Entity_MULTILEADER_has_landing_set)
    has_dogleg = property(_LibreDWG.Dwg_Entity_MULTILEADER_has_dogleg_get, _LibreDWG.Dwg_Entity_MULTILEADER_has_dogleg_set)
    landing_dist = property(_LibreDWG.Dwg_Entity_MULTILEADER_landing_dist_get, _LibreDWG.Dwg_Entity_MULTILEADER_landing_dist_set)
    arrow_handle = property(_LibreDWG.Dwg_Entity_MULTILEADER_arrow_handle_get, _LibreDWG.Dwg_Entity_MULTILEADER_arrow_handle_set)
    arrow_size = property(_LibreDWG.Dwg_Entity_MULTILEADER_arrow_size_get, _LibreDWG.Dwg_Entity_MULTILEADER_arrow_size_set)
    style_content = property(_LibreDWG.Dwg_Entity_MULTILEADER_style_content_get, _LibreDWG.Dwg_Entity_MULTILEADER_style_content_set)
    text_style = property(_LibreDWG.Dwg_Entity_MULTILEADER_text_style_get, _LibreDWG.Dwg_Entity_MULTILEADER_text_style_set)
    text_left = property(_LibreDWG.Dwg_Entity_MULTILEADER_text_left_get, _LibreDWG.Dwg_Entity_MULTILEADER_text_left_set)
    text_right = property(_LibreDWG.Dwg_Entity_MULTILEADER_text_right_get, _LibreDWG.Dwg_Entity_MULTILEADER_text_right_set)
    text_angletype = property(_LibreDWG.Dwg_Entity_MULTILEADER_text_angletype_get, _LibreDWG.Dwg_Entity_MULTILEADER_text_angletype_set)
    text_alignment = property(_LibreDWG.Dwg_Entity_MULTILEADER_text_alignment_get, _LibreDWG.Dwg_Entity_MULTILEADER_text_alignment_set)
    text_color = property(_LibreDWG.Dwg_Entity_MULTILEADER_text_color_get, _LibreDWG.Dwg_Entity_MULTILEADER_text_color_set)
    has_text_frame = property(_LibreDWG.Dwg_Entity_MULTILEADER_has_text_frame_get, _LibreDWG.Dwg_Entity_MULTILEADER_has_text_frame_set)
    block_style = property(_LibreDWG.Dwg_Entity_MULTILEADER_block_style_get, _LibreDWG.Dwg_Entity_MULTILEADER_block_style_set)
    block_color = property(_LibreDWG.Dwg_Entity_MULTILEADER_block_color_get, _LibreDWG.Dwg_Entity_MULTILEADER_block_color_set)
    block_scale = property(_LibreDWG.Dwg_Entity_MULTILEADER_block_scale_get, _LibreDWG.Dwg_Entity_MULTILEADER_block_scale_set)
    block_rotation = property(_LibreDWG.Dwg_Entity_MULTILEADER_block_rotation_get, _LibreDWG.Dwg_Entity_MULTILEADER_block_rotation_set)
    style_attachment = property(_LibreDWG.Dwg_Entity_MULTILEADER_style_attachment_get, _LibreDWG.Dwg_Entity_MULTILEADER_style_attachment_set)
    is_annotative = property(_LibreDWG.Dwg_Entity_MULTILEADER_is_annotative_get, _LibreDWG.Dwg_Entity_MULTILEADER_is_annotative_set)
    num_arrowheads = property(_LibreDWG.Dwg_Entity_MULTILEADER_num_arrowheads_get, _LibreDWG.Dwg_Entity_MULTILEADER_num_arrowheads_set)
    arrowheads = property(_LibreDWG.Dwg_Entity_MULTILEADER_arrowheads_get, _LibreDWG.Dwg_Entity_MULTILEADER_arrowheads_set)
    num_blocklabels = property(_LibreDWG.Dwg_Entity_MULTILEADER_num_blocklabels_get, _LibreDWG.Dwg_Entity_MULTILEADER_num_blocklabels_set)
    blocklabels = property(_LibreDWG.Dwg_Entity_MULTILEADER_blocklabels_get, _LibreDWG.Dwg_Entity_MULTILEADER_blocklabels_set)
    is_neg_textdir = property(_LibreDWG.Dwg_Entity_MULTILEADER_is_neg_textdir_get, _LibreDWG.Dwg_Entity_MULTILEADER_is_neg_textdir_set)
    ipe_alignment = property(_LibreDWG.Dwg_Entity_MULTILEADER_ipe_alignment_get, _LibreDWG.Dwg_Entity_MULTILEADER_ipe_alignment_set)
    justification = property(_LibreDWG.Dwg_Entity_MULTILEADER_justification_get, _LibreDWG.Dwg_Entity_MULTILEADER_justification_set)
    scale_factor = property(_LibreDWG.Dwg_Entity_MULTILEADER_scale_factor_get, _LibreDWG.Dwg_Entity_MULTILEADER_scale_factor_set)
    attach_dir = property(_LibreDWG.Dwg_Entity_MULTILEADER_attach_dir_get, _LibreDWG.Dwg_Entity_MULTILEADER_attach_dir_set)
    attach_top = property(_LibreDWG.Dwg_Entity_MULTILEADER_attach_top_get, _LibreDWG.Dwg_Entity_MULTILEADER_attach_top_set)
    attach_bottom = property(_LibreDWG.Dwg_Entity_MULTILEADER_attach_bottom_get, _LibreDWG.Dwg_Entity_MULTILEADER_attach_bottom_set)
    is_text_extended = property(_LibreDWG.Dwg_Entity_MULTILEADER_is_text_extended_get, _LibreDWG.Dwg_Entity_MULTILEADER_is_text_extended_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_MULTILEADER_swiginit(self, _LibreDWG.new_Dwg_Entity_MULTILEADER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_MULTILEADER

# Register Dwg_Entity_MULTILEADER in _LibreDWG:
_LibreDWG.Dwg_Entity_MULTILEADER_swigregister(Dwg_Entity_MULTILEADER)
class Dwg_Object_MLEADERSTYLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_parent_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_class_version_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_class_version_set)
    content_type = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_content_type_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_content_type_set)
    mleader_order = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_mleader_order_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_mleader_order_set)
    leader_order = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_leader_order_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_leader_order_set)
    max_points = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_max_points_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_max_points_set)
    first_seg_angle = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_first_seg_angle_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_first_seg_angle_set)
    second_seg_angle = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_second_seg_angle_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_second_seg_angle_set)
    type = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_type_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_type_set)
    line_color = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_line_color_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_line_color_set)
    line_type = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_line_type_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_line_type_set)
    linewt = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_linewt_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_linewt_set)
    has_landing = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_has_landing_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_has_landing_set)
    has_dogleg = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_has_dogleg_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_has_dogleg_set)
    landing_gap = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_landing_gap_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_landing_gap_set)
    landing_dist = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_landing_dist_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_landing_dist_set)
    description = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_description_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_description_set)
    arrow_head = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_arrow_head_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_arrow_head_set)
    arrow_head_size = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_arrow_head_size_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_arrow_head_size_set)
    text_default = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_text_default_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_text_default_set)
    text_style = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_text_style_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_text_style_set)
    attach_left = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_attach_left_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_attach_left_set)
    attach_right = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_attach_right_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_attach_right_set)
    text_angle_type = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_text_angle_type_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_text_angle_type_set)
    text_align_type = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_text_align_type_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_text_align_type_set)
    text_color = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_text_color_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_text_color_set)
    text_height = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_text_height_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_text_height_set)
    has_text_frame = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_has_text_frame_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_has_text_frame_set)
    text_always_left = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_text_always_left_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_text_always_left_set)
    align_space = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_align_space_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_align_space_set)
    block = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_block_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_block_set)
    block_color = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_block_color_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_block_color_set)
    block_scale = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_block_scale_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_block_scale_set)
    use_block_scale = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_use_block_scale_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_use_block_scale_set)
    block_rotation = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_block_rotation_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_block_rotation_set)
    use_block_rotation = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_use_block_rotation_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_use_block_rotation_set)
    block_connection = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_block_connection_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_block_connection_set)
    scale = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_scale_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_scale_set)
    is_changed = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_is_changed_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_is_changed_set)
    is_annotative = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_is_annotative_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_is_annotative_set)
    break_size = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_break_size_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_break_size_set)
    attach_dir = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_attach_dir_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_attach_dir_set)
    attach_top = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_attach_top_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_attach_top_set)
    attach_bottom = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_attach_bottom_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_attach_bottom_set)
    text_extended = property(_LibreDWG.Dwg_Object_MLEADERSTYLE_text_extended_get, _LibreDWG.Dwg_Object_MLEADERSTYLE_text_extended_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_MLEADERSTYLE_swiginit(self, _LibreDWG.new_Dwg_Object_MLEADERSTYLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_MLEADERSTYLE

# Register Dwg_Object_MLEADERSTYLE in _LibreDWG:
_LibreDWG.Dwg_Object_MLEADERSTYLE_swigregister(Dwg_Object_MLEADERSTYLE)
class Dwg_Object_VBA_PROJECT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_VBA_PROJECT_parent_get, _LibreDWG.Dwg_Object_VBA_PROJECT_parent_set)
    data_size = property(_LibreDWG.Dwg_Object_VBA_PROJECT_data_size_get, _LibreDWG.Dwg_Object_VBA_PROJECT_data_size_set)
    data = property(_LibreDWG.Dwg_Object_VBA_PROJECT_data_get, _LibreDWG.Dwg_Object_VBA_PROJECT_data_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_VBA_PROJECT_swiginit(self, _LibreDWG.new_Dwg_Object_VBA_PROJECT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_VBA_PROJECT

# Register Dwg_Object_VBA_PROJECT in _LibreDWG:
_LibreDWG.Dwg_Object_VBA_PROJECT_swigregister(Dwg_Object_VBA_PROJECT)
class Dwg_Object_PLOTSETTINGS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_parent_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_parent_set)
    printer_cfg_file = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_printer_cfg_file_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_printer_cfg_file_set)
    paper_size = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_paper_size_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_paper_size_set)
    canonical_media_name = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_canonical_media_name_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_canonical_media_name_set)
    plot_flags = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_plot_flags_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_plot_flags_set)
    plotview = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_plotview_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_plotview_set)
    plotview_name = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_plotview_name_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_plotview_name_set)
    left_margin = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_left_margin_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_left_margin_set)
    bottom_margin = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_bottom_margin_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_bottom_margin_set)
    right_margin = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_right_margin_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_right_margin_set)
    top_margin = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_top_margin_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_top_margin_set)
    paper_width = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_paper_width_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_paper_width_set)
    paper_height = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_paper_height_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_paper_height_set)
    plot_origin = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_plot_origin_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_plot_origin_set)
    plot_window_ll = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_plot_window_ll_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_plot_window_ll_set)
    plot_window_ur = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_plot_window_ur_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_plot_window_ur_set)
    plot_paper_unit = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_plot_paper_unit_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_plot_paper_unit_set)
    plot_rotation_mode = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_plot_rotation_mode_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_plot_rotation_mode_set)
    plot_type = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_plot_type_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_plot_type_set)
    paper_units = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_paper_units_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_paper_units_set)
    drawing_units = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_drawing_units_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_drawing_units_set)
    stylesheet = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_stylesheet_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_stylesheet_set)
    std_scale_type = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_std_scale_type_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_std_scale_type_set)
    std_scale_factor = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_std_scale_factor_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_std_scale_factor_set)
    paper_image_origin = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_paper_image_origin_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_paper_image_origin_set)
    shadeplot_type = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_shadeplot_type_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_shadeplot_type_set)
    shadeplot_reslevel = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_set)
    shadeplot_customdpi = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_set)
    shadeplot = property(_LibreDWG.Dwg_Object_PLOTSETTINGS_shadeplot_get, _LibreDWG.Dwg_Object_PLOTSETTINGS_shadeplot_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_PLOTSETTINGS_swiginit(self, _LibreDWG.new_Dwg_Object_PLOTSETTINGS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_PLOTSETTINGS

# Register Dwg_Object_PLOTSETTINGS in _LibreDWG:
_LibreDWG.Dwg_Object_PLOTSETTINGS_swigregister(Dwg_Object_PLOTSETTINGS)
class Dwg_Object_LAYOUT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_LAYOUT_parent_get, _LibreDWG.Dwg_Object_LAYOUT_parent_set)
    plotsettings = property(_LibreDWG.Dwg_Object_LAYOUT_plotsettings_get, _LibreDWG.Dwg_Object_LAYOUT_plotsettings_set)
    layout_name = property(_LibreDWG.Dwg_Object_LAYOUT_layout_name_get, _LibreDWG.Dwg_Object_LAYOUT_layout_name_set)
    tab_order = property(_LibreDWG.Dwg_Object_LAYOUT_tab_order_get, _LibreDWG.Dwg_Object_LAYOUT_tab_order_set)
    layout_flags = property(_LibreDWG.Dwg_Object_LAYOUT_layout_flags_get, _LibreDWG.Dwg_Object_LAYOUT_layout_flags_set)
    INSBASE = property(_LibreDWG.Dwg_Object_LAYOUT_INSBASE_get, _LibreDWG.Dwg_Object_LAYOUT_INSBASE_set)
    LIMMIN = property(_LibreDWG.Dwg_Object_LAYOUT_LIMMIN_get, _LibreDWG.Dwg_Object_LAYOUT_LIMMIN_set)
    LIMMAX = property(_LibreDWG.Dwg_Object_LAYOUT_LIMMAX_get, _LibreDWG.Dwg_Object_LAYOUT_LIMMAX_set)
    UCSORG = property(_LibreDWG.Dwg_Object_LAYOUT_UCSORG_get, _LibreDWG.Dwg_Object_LAYOUT_UCSORG_set)
    UCSXDIR = property(_LibreDWG.Dwg_Object_LAYOUT_UCSXDIR_get, _LibreDWG.Dwg_Object_LAYOUT_UCSXDIR_set)
    UCSYDIR = property(_LibreDWG.Dwg_Object_LAYOUT_UCSYDIR_get, _LibreDWG.Dwg_Object_LAYOUT_UCSYDIR_set)
    ucs_elevation = property(_LibreDWG.Dwg_Object_LAYOUT_ucs_elevation_get, _LibreDWG.Dwg_Object_LAYOUT_ucs_elevation_set)
    UCSORTHOVIEW = property(_LibreDWG.Dwg_Object_LAYOUT_UCSORTHOVIEW_get, _LibreDWG.Dwg_Object_LAYOUT_UCSORTHOVIEW_set)
    EXTMIN = property(_LibreDWG.Dwg_Object_LAYOUT_EXTMIN_get, _LibreDWG.Dwg_Object_LAYOUT_EXTMIN_set)
    EXTMAX = property(_LibreDWG.Dwg_Object_LAYOUT_EXTMAX_get, _LibreDWG.Dwg_Object_LAYOUT_EXTMAX_set)
    block_header = property(_LibreDWG.Dwg_Object_LAYOUT_block_header_get, _LibreDWG.Dwg_Object_LAYOUT_block_header_set)
    active_viewport = property(_LibreDWG.Dwg_Object_LAYOUT_active_viewport_get, _LibreDWG.Dwg_Object_LAYOUT_active_viewport_set)
    base_ucs = property(_LibreDWG.Dwg_Object_LAYOUT_base_ucs_get, _LibreDWG.Dwg_Object_LAYOUT_base_ucs_set)
    named_ucs = property(_LibreDWG.Dwg_Object_LAYOUT_named_ucs_get, _LibreDWG.Dwg_Object_LAYOUT_named_ucs_set)
    num_viewports = property(_LibreDWG.Dwg_Object_LAYOUT_num_viewports_get, _LibreDWG.Dwg_Object_LAYOUT_num_viewports_set)
    viewports = property(_LibreDWG.Dwg_Object_LAYOUT_viewports_get, _LibreDWG.Dwg_Object_LAYOUT_viewports_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_LAYOUT_swiginit(self, _LibreDWG.new_Dwg_Object_LAYOUT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_LAYOUT

# Register Dwg_Object_LAYOUT in _LibreDWG:
_LibreDWG.Dwg_Object_LAYOUT_swigregister(Dwg_Object_LAYOUT)
class Dwg_Object_DICTIONARYVAR(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DICTIONARYVAR_parent_get, _LibreDWG.Dwg_Object_DICTIONARYVAR_parent_set)
    schema = property(_LibreDWG.Dwg_Object_DICTIONARYVAR_schema_get, _LibreDWG.Dwg_Object_DICTIONARYVAR_schema_set)
    strvalue = property(_LibreDWG.Dwg_Object_DICTIONARYVAR_strvalue_get, _LibreDWG.Dwg_Object_DICTIONARYVAR_strvalue_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DICTIONARYVAR_swiginit(self, _LibreDWG.new_Dwg_Object_DICTIONARYVAR())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DICTIONARYVAR

# Register Dwg_Object_DICTIONARYVAR in _LibreDWG:
_LibreDWG.Dwg_Object_DICTIONARYVAR_swigregister(Dwg_Object_DICTIONARYVAR)
class Dwg_TABLE_value(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    flags = property(_LibreDWG.Dwg_TABLE_value_flags_get, _LibreDWG.Dwg_TABLE_value_flags_set)
    format_flags = property(_LibreDWG.Dwg_TABLE_value_format_flags_get, _LibreDWG.Dwg_TABLE_value_format_flags_set)
    data_type = property(_LibreDWG.Dwg_TABLE_value_data_type_get, _LibreDWG.Dwg_TABLE_value_data_type_set)
    data_size = property(_LibreDWG.Dwg_TABLE_value_data_size_get, _LibreDWG.Dwg_TABLE_value_data_size_set)
    data_long = property(_LibreDWG.Dwg_TABLE_value_data_long_get, _LibreDWG.Dwg_TABLE_value_data_long_set)
    data_double = property(_LibreDWG.Dwg_TABLE_value_data_double_get, _LibreDWG.Dwg_TABLE_value_data_double_set)
    data_string = property(_LibreDWG.Dwg_TABLE_value_data_string_get, _LibreDWG.Dwg_TABLE_value_data_string_set)
    data_date = property(_LibreDWG.Dwg_TABLE_value_data_date_get, _LibreDWG.Dwg_TABLE_value_data_date_set)
    data_point = property(_LibreDWG.Dwg_TABLE_value_data_point_get, _LibreDWG.Dwg_TABLE_value_data_point_set)
    data_3dpoint = property(_LibreDWG.Dwg_TABLE_value_data_3dpoint_get, _LibreDWG.Dwg_TABLE_value_data_3dpoint_set)
    data_handle = property(_LibreDWG.Dwg_TABLE_value_data_handle_get, _LibreDWG.Dwg_TABLE_value_data_handle_set)
    unit_type = property(_LibreDWG.Dwg_TABLE_value_unit_type_get, _LibreDWG.Dwg_TABLE_value_unit_type_set)
    format_string = property(_LibreDWG.Dwg_TABLE_value_format_string_get, _LibreDWG.Dwg_TABLE_value_format_string_set)
    value_string = property(_LibreDWG.Dwg_TABLE_value_value_string_get, _LibreDWG.Dwg_TABLE_value_value_string_set)

    def __init__(self):
        _LibreDWG.Dwg_TABLE_value_swiginit(self, _LibreDWG.new_Dwg_TABLE_value())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TABLE_value

# Register Dwg_TABLE_value in _LibreDWG:
_LibreDWG.Dwg_TABLE_value_swigregister(Dwg_TABLE_value)
class Dwg_TABLE_CustomDataItem(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    name = property(_LibreDWG.Dwg_TABLE_CustomDataItem_name_get, _LibreDWG.Dwg_TABLE_CustomDataItem_name_set)
    value = property(_LibreDWG.Dwg_TABLE_CustomDataItem_value_get, _LibreDWG.Dwg_TABLE_CustomDataItem_value_set)
    cell_parent = property(_LibreDWG.Dwg_TABLE_CustomDataItem_cell_parent_get, _LibreDWG.Dwg_TABLE_CustomDataItem_cell_parent_set)
    row_parent = property(_LibreDWG.Dwg_TABLE_CustomDataItem_row_parent_get, _LibreDWG.Dwg_TABLE_CustomDataItem_row_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_TABLE_CustomDataItem_swiginit(self, _LibreDWG.new_Dwg_TABLE_CustomDataItem())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TABLE_CustomDataItem

# Register Dwg_TABLE_CustomDataItem in _LibreDWG:
_LibreDWG.Dwg_TABLE_CustomDataItem_swigregister(Dwg_TABLE_CustomDataItem)
class Dwg_TABLE_AttrDef(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_TABLE_AttrDef_parent_get, _LibreDWG.Dwg_TABLE_AttrDef_parent_set)
    attdef = property(_LibreDWG.Dwg_TABLE_AttrDef_attdef_get, _LibreDWG.Dwg_TABLE_AttrDef_attdef_set)
    index = property(_LibreDWG.Dwg_TABLE_AttrDef_index_get, _LibreDWG.Dwg_TABLE_AttrDef_index_set)
    text = property(_LibreDWG.Dwg_TABLE_AttrDef_text_get, _LibreDWG.Dwg_TABLE_AttrDef_text_set)

    def __init__(self):
        _LibreDWG.Dwg_TABLE_AttrDef_swiginit(self, _LibreDWG.new_Dwg_TABLE_AttrDef())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TABLE_AttrDef

# Register Dwg_TABLE_AttrDef in _LibreDWG:
_LibreDWG.Dwg_TABLE_AttrDef_swigregister(Dwg_TABLE_AttrDef)
class Dwg_TABLE_Cell(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_TABLE_Cell_parent_get, _LibreDWG.Dwg_TABLE_Cell_parent_set)
    type = property(_LibreDWG.Dwg_TABLE_Cell_type_get, _LibreDWG.Dwg_TABLE_Cell_type_set)
    flags = property(_LibreDWG.Dwg_TABLE_Cell_flags_get, _LibreDWG.Dwg_TABLE_Cell_flags_set)
    is_merged_value = property(_LibreDWG.Dwg_TABLE_Cell_is_merged_value_get, _LibreDWG.Dwg_TABLE_Cell_is_merged_value_set)
    is_autofit_flag = property(_LibreDWG.Dwg_TABLE_Cell_is_autofit_flag_get, _LibreDWG.Dwg_TABLE_Cell_is_autofit_flag_set)
    merged_width_flag = property(_LibreDWG.Dwg_TABLE_Cell_merged_width_flag_get, _LibreDWG.Dwg_TABLE_Cell_merged_width_flag_set)
    merged_height_flag = property(_LibreDWG.Dwg_TABLE_Cell_merged_height_flag_get, _LibreDWG.Dwg_TABLE_Cell_merged_height_flag_set)
    rotation = property(_LibreDWG.Dwg_TABLE_Cell_rotation_get, _LibreDWG.Dwg_TABLE_Cell_rotation_set)
    text_value = property(_LibreDWG.Dwg_TABLE_Cell_text_value_get, _LibreDWG.Dwg_TABLE_Cell_text_value_set)
    text_style = property(_LibreDWG.Dwg_TABLE_Cell_text_style_get, _LibreDWG.Dwg_TABLE_Cell_text_style_set)
    block_handle = property(_LibreDWG.Dwg_TABLE_Cell_block_handle_get, _LibreDWG.Dwg_TABLE_Cell_block_handle_set)
    block_scale = property(_LibreDWG.Dwg_TABLE_Cell_block_scale_get, _LibreDWG.Dwg_TABLE_Cell_block_scale_set)
    additional_data_flag = property(_LibreDWG.Dwg_TABLE_Cell_additional_data_flag_get, _LibreDWG.Dwg_TABLE_Cell_additional_data_flag_set)
    cell_flag_override = property(_LibreDWG.Dwg_TABLE_Cell_cell_flag_override_get, _LibreDWG.Dwg_TABLE_Cell_cell_flag_override_set)
    virtual_edge_flag = property(_LibreDWG.Dwg_TABLE_Cell_virtual_edge_flag_get, _LibreDWG.Dwg_TABLE_Cell_virtual_edge_flag_set)
    cell_alignment = property(_LibreDWG.Dwg_TABLE_Cell_cell_alignment_get, _LibreDWG.Dwg_TABLE_Cell_cell_alignment_set)
    bg_fill_none = property(_LibreDWG.Dwg_TABLE_Cell_bg_fill_none_get, _LibreDWG.Dwg_TABLE_Cell_bg_fill_none_set)
    bg_color = property(_LibreDWG.Dwg_TABLE_Cell_bg_color_get, _LibreDWG.Dwg_TABLE_Cell_bg_color_set)
    content_color = property(_LibreDWG.Dwg_TABLE_Cell_content_color_get, _LibreDWG.Dwg_TABLE_Cell_content_color_set)
    text_height = property(_LibreDWG.Dwg_TABLE_Cell_text_height_get, _LibreDWG.Dwg_TABLE_Cell_text_height_set)
    top_grid_color = property(_LibreDWG.Dwg_TABLE_Cell_top_grid_color_get, _LibreDWG.Dwg_TABLE_Cell_top_grid_color_set)
    top_grid_linewt = property(_LibreDWG.Dwg_TABLE_Cell_top_grid_linewt_get, _LibreDWG.Dwg_TABLE_Cell_top_grid_linewt_set)
    top_visibility = property(_LibreDWG.Dwg_TABLE_Cell_top_visibility_get, _LibreDWG.Dwg_TABLE_Cell_top_visibility_set)
    right_grid_color = property(_LibreDWG.Dwg_TABLE_Cell_right_grid_color_get, _LibreDWG.Dwg_TABLE_Cell_right_grid_color_set)
    right_grid_linewt = property(_LibreDWG.Dwg_TABLE_Cell_right_grid_linewt_get, _LibreDWG.Dwg_TABLE_Cell_right_grid_linewt_set)
    right_visibility = property(_LibreDWG.Dwg_TABLE_Cell_right_visibility_get, _LibreDWG.Dwg_TABLE_Cell_right_visibility_set)
    bottom_grid_color = property(_LibreDWG.Dwg_TABLE_Cell_bottom_grid_color_get, _LibreDWG.Dwg_TABLE_Cell_bottom_grid_color_set)
    bottom_grid_linewt = property(_LibreDWG.Dwg_TABLE_Cell_bottom_grid_linewt_get, _LibreDWG.Dwg_TABLE_Cell_bottom_grid_linewt_set)
    bottom_visibility = property(_LibreDWG.Dwg_TABLE_Cell_bottom_visibility_get, _LibreDWG.Dwg_TABLE_Cell_bottom_visibility_set)
    left_grid_color = property(_LibreDWG.Dwg_TABLE_Cell_left_grid_color_get, _LibreDWG.Dwg_TABLE_Cell_left_grid_color_set)
    left_grid_linewt = property(_LibreDWG.Dwg_TABLE_Cell_left_grid_linewt_get, _LibreDWG.Dwg_TABLE_Cell_left_grid_linewt_set)
    left_visibility = property(_LibreDWG.Dwg_TABLE_Cell_left_visibility_get, _LibreDWG.Dwg_TABLE_Cell_left_visibility_set)
    unknown = property(_LibreDWG.Dwg_TABLE_Cell_unknown_get, _LibreDWG.Dwg_TABLE_Cell_unknown_set)
    value = property(_LibreDWG.Dwg_TABLE_Cell_value_get, _LibreDWG.Dwg_TABLE_Cell_value_set)
    num_attr_defs = property(_LibreDWG.Dwg_TABLE_Cell_num_attr_defs_get, _LibreDWG.Dwg_TABLE_Cell_num_attr_defs_set)
    attr_defs = property(_LibreDWG.Dwg_TABLE_Cell_attr_defs_get, _LibreDWG.Dwg_TABLE_Cell_attr_defs_set)

    def __init__(self):
        _LibreDWG.Dwg_TABLE_Cell_swiginit(self, _LibreDWG.new_Dwg_TABLE_Cell())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TABLE_Cell

# Register Dwg_TABLE_Cell in _LibreDWG:
_LibreDWG.Dwg_TABLE_Cell_swigregister(Dwg_TABLE_Cell)
class Dwg_TABLE_BreakHeight(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_TABLE_BreakHeight_parent_get, _LibreDWG.Dwg_TABLE_BreakHeight_parent_set)
    position = property(_LibreDWG.Dwg_TABLE_BreakHeight_position_get, _LibreDWG.Dwg_TABLE_BreakHeight_position_set)
    height = property(_LibreDWG.Dwg_TABLE_BreakHeight_height_get, _LibreDWG.Dwg_TABLE_BreakHeight_height_set)
    flag = property(_LibreDWG.Dwg_TABLE_BreakHeight_flag_get, _LibreDWG.Dwg_TABLE_BreakHeight_flag_set)

    def __init__(self):
        _LibreDWG.Dwg_TABLE_BreakHeight_swiginit(self, _LibreDWG.new_Dwg_TABLE_BreakHeight())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TABLE_BreakHeight

# Register Dwg_TABLE_BreakHeight in _LibreDWG:
_LibreDWG.Dwg_TABLE_BreakHeight_swigregister(Dwg_TABLE_BreakHeight)
class Dwg_TABLE_BreakRow(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_TABLE_BreakRow_parent_get, _LibreDWG.Dwg_TABLE_BreakRow_parent_set)
    position = property(_LibreDWG.Dwg_TABLE_BreakRow_position_get, _LibreDWG.Dwg_TABLE_BreakRow_position_set)
    start = property(_LibreDWG.Dwg_TABLE_BreakRow_start_get, _LibreDWG.Dwg_TABLE_BreakRow_start_set)
    end = property(_LibreDWG.Dwg_TABLE_BreakRow_end_get, _LibreDWG.Dwg_TABLE_BreakRow_end_set)

    def __init__(self):
        _LibreDWG.Dwg_TABLE_BreakRow_swiginit(self, _LibreDWG.new_Dwg_TABLE_BreakRow())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TABLE_BreakRow

# Register Dwg_TABLE_BreakRow in _LibreDWG:
_LibreDWG.Dwg_TABLE_BreakRow_swigregister(Dwg_TABLE_BreakRow)
class Dwg_LinkedData(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    name = property(_LibreDWG.Dwg_LinkedData_name_get, _LibreDWG.Dwg_LinkedData_name_set)
    description = property(_LibreDWG.Dwg_LinkedData_description_get, _LibreDWG.Dwg_LinkedData_description_set)

    def __init__(self):
        _LibreDWG.Dwg_LinkedData_swiginit(self, _LibreDWG.new_Dwg_LinkedData())
    __swig_destroy__ = _LibreDWG.delete_Dwg_LinkedData

# Register Dwg_LinkedData in _LibreDWG:
_LibreDWG.Dwg_LinkedData_swigregister(Dwg_LinkedData)
class Dwg_TableCellContent_Attr(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_TableCellContent_Attr_parent_get, _LibreDWG.Dwg_TableCellContent_Attr_parent_set)
    attdef = property(_LibreDWG.Dwg_TableCellContent_Attr_attdef_get, _LibreDWG.Dwg_TableCellContent_Attr_attdef_set)
    value = property(_LibreDWG.Dwg_TableCellContent_Attr_value_get, _LibreDWG.Dwg_TableCellContent_Attr_value_set)
    index = property(_LibreDWG.Dwg_TableCellContent_Attr_index_get, _LibreDWG.Dwg_TableCellContent_Attr_index_set)

    def __init__(self):
        _LibreDWG.Dwg_TableCellContent_Attr_swiginit(self, _LibreDWG.new_Dwg_TableCellContent_Attr())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TableCellContent_Attr

# Register Dwg_TableCellContent_Attr in _LibreDWG:
_LibreDWG.Dwg_TableCellContent_Attr_swigregister(Dwg_TableCellContent_Attr)
class Dwg_ContentFormat(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    property_override_flags = property(_LibreDWG.Dwg_ContentFormat_property_override_flags_get, _LibreDWG.Dwg_ContentFormat_property_override_flags_set)
    property_flags = property(_LibreDWG.Dwg_ContentFormat_property_flags_get, _LibreDWG.Dwg_ContentFormat_property_flags_set)
    value_data_type = property(_LibreDWG.Dwg_ContentFormat_value_data_type_get, _LibreDWG.Dwg_ContentFormat_value_data_type_set)
    value_unit_type = property(_LibreDWG.Dwg_ContentFormat_value_unit_type_get, _LibreDWG.Dwg_ContentFormat_value_unit_type_set)
    value_format_string = property(_LibreDWG.Dwg_ContentFormat_value_format_string_get, _LibreDWG.Dwg_ContentFormat_value_format_string_set)
    rotation = property(_LibreDWG.Dwg_ContentFormat_rotation_get, _LibreDWG.Dwg_ContentFormat_rotation_set)
    block_scale = property(_LibreDWG.Dwg_ContentFormat_block_scale_get, _LibreDWG.Dwg_ContentFormat_block_scale_set)
    cell_alignment = property(_LibreDWG.Dwg_ContentFormat_cell_alignment_get, _LibreDWG.Dwg_ContentFormat_cell_alignment_set)
    content_color = property(_LibreDWG.Dwg_ContentFormat_content_color_get, _LibreDWG.Dwg_ContentFormat_content_color_set)
    text_style = property(_LibreDWG.Dwg_ContentFormat_text_style_get, _LibreDWG.Dwg_ContentFormat_text_style_set)
    text_height = property(_LibreDWG.Dwg_ContentFormat_text_height_get, _LibreDWG.Dwg_ContentFormat_text_height_set)

    def __init__(self):
        _LibreDWG.Dwg_ContentFormat_swiginit(self, _LibreDWG.new_Dwg_ContentFormat())
    __swig_destroy__ = _LibreDWG.delete_Dwg_ContentFormat

# Register Dwg_ContentFormat in _LibreDWG:
_LibreDWG.Dwg_ContentFormat_swigregister(Dwg_ContentFormat)
class Dwg_TableCellContent(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_TableCellContent_parent_get, _LibreDWG.Dwg_TableCellContent_parent_set)
    type = property(_LibreDWG.Dwg_TableCellContent_type_get, _LibreDWG.Dwg_TableCellContent_type_set)
    value = property(_LibreDWG.Dwg_TableCellContent_value_get, _LibreDWG.Dwg_TableCellContent_value_set)
    handle = property(_LibreDWG.Dwg_TableCellContent_handle_get, _LibreDWG.Dwg_TableCellContent_handle_set)
    num_attrs = property(_LibreDWG.Dwg_TableCellContent_num_attrs_get, _LibreDWG.Dwg_TableCellContent_num_attrs_set)
    attrs = property(_LibreDWG.Dwg_TableCellContent_attrs_get, _LibreDWG.Dwg_TableCellContent_attrs_set)
    has_content_format_overrides = property(_LibreDWG.Dwg_TableCellContent_has_content_format_overrides_get, _LibreDWG.Dwg_TableCellContent_has_content_format_overrides_set)
    content_format = property(_LibreDWG.Dwg_TableCellContent_content_format_get, _LibreDWG.Dwg_TableCellContent_content_format_set)

    def __init__(self):
        _LibreDWG.Dwg_TableCellContent_swiginit(self, _LibreDWG.new_Dwg_TableCellContent())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TableCellContent

# Register Dwg_TableCellContent in _LibreDWG:
_LibreDWG.Dwg_TableCellContent_swigregister(Dwg_TableCellContent)
class Dwg_CellContentGeometry(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    dist_top_left = property(_LibreDWG.Dwg_CellContentGeometry_dist_top_left_get, _LibreDWG.Dwg_CellContentGeometry_dist_top_left_set)
    dist_center = property(_LibreDWG.Dwg_CellContentGeometry_dist_center_get, _LibreDWG.Dwg_CellContentGeometry_dist_center_set)
    content_width = property(_LibreDWG.Dwg_CellContentGeometry_content_width_get, _LibreDWG.Dwg_CellContentGeometry_content_width_set)
    content_height = property(_LibreDWG.Dwg_CellContentGeometry_content_height_get, _LibreDWG.Dwg_CellContentGeometry_content_height_set)
    width = property(_LibreDWG.Dwg_CellContentGeometry_width_get, _LibreDWG.Dwg_CellContentGeometry_width_set)
    height = property(_LibreDWG.Dwg_CellContentGeometry_height_get, _LibreDWG.Dwg_CellContentGeometry_height_set)
    unknown = property(_LibreDWG.Dwg_CellContentGeometry_unknown_get, _LibreDWG.Dwg_CellContentGeometry_unknown_set)
    cell_parent = property(_LibreDWG.Dwg_CellContentGeometry_cell_parent_get, _LibreDWG.Dwg_CellContentGeometry_cell_parent_set)
    geom_parent = property(_LibreDWG.Dwg_CellContentGeometry_geom_parent_get, _LibreDWG.Dwg_CellContentGeometry_geom_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_CellContentGeometry_swiginit(self, _LibreDWG.new_Dwg_CellContentGeometry())
    __swig_destroy__ = _LibreDWG.delete_Dwg_CellContentGeometry

# Register Dwg_CellContentGeometry in _LibreDWG:
_LibreDWG.Dwg_CellContentGeometry_swigregister(Dwg_CellContentGeometry)
class Dwg_TableCell(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    flag = property(_LibreDWG.Dwg_TableCell_flag_get, _LibreDWG.Dwg_TableCell_flag_set)
    tooltip = property(_LibreDWG.Dwg_TableCell_tooltip_get, _LibreDWG.Dwg_TableCell_tooltip_set)
    customdata = property(_LibreDWG.Dwg_TableCell_customdata_get, _LibreDWG.Dwg_TableCell_customdata_set)
    num_customdata_items = property(_LibreDWG.Dwg_TableCell_num_customdata_items_get, _LibreDWG.Dwg_TableCell_num_customdata_items_set)
    customdata_items = property(_LibreDWG.Dwg_TableCell_customdata_items_get, _LibreDWG.Dwg_TableCell_customdata_items_set)
    has_linked_data = property(_LibreDWG.Dwg_TableCell_has_linked_data_get, _LibreDWG.Dwg_TableCell_has_linked_data_set)
    data_link = property(_LibreDWG.Dwg_TableCell_data_link_get, _LibreDWG.Dwg_TableCell_data_link_set)
    num_rows = property(_LibreDWG.Dwg_TableCell_num_rows_get, _LibreDWG.Dwg_TableCell_num_rows_set)
    num_cols = property(_LibreDWG.Dwg_TableCell_num_cols_get, _LibreDWG.Dwg_TableCell_num_cols_set)
    unknown = property(_LibreDWG.Dwg_TableCell_unknown_get, _LibreDWG.Dwg_TableCell_unknown_set)
    num_cell_contents = property(_LibreDWG.Dwg_TableCell_num_cell_contents_get, _LibreDWG.Dwg_TableCell_num_cell_contents_set)
    cell_contents = property(_LibreDWG.Dwg_TableCell_cell_contents_get, _LibreDWG.Dwg_TableCell_cell_contents_set)
    style_id = property(_LibreDWG.Dwg_TableCell_style_id_get, _LibreDWG.Dwg_TableCell_style_id_set)
    has_geom_data = property(_LibreDWG.Dwg_TableCell_has_geom_data_get, _LibreDWG.Dwg_TableCell_has_geom_data_set)
    geom_data_flag = property(_LibreDWG.Dwg_TableCell_geom_data_flag_get, _LibreDWG.Dwg_TableCell_geom_data_flag_set)
    width_w_gap = property(_LibreDWG.Dwg_TableCell_width_w_gap_get, _LibreDWG.Dwg_TableCell_width_w_gap_set)
    height_w_gap = property(_LibreDWG.Dwg_TableCell_height_w_gap_get, _LibreDWG.Dwg_TableCell_height_w_gap_set)
    tablegeometry = property(_LibreDWG.Dwg_TableCell_tablegeometry_get, _LibreDWG.Dwg_TableCell_tablegeometry_set)
    num_geometry = property(_LibreDWG.Dwg_TableCell_num_geometry_get, _LibreDWG.Dwg_TableCell_num_geometry_set)
    geometry = property(_LibreDWG.Dwg_TableCell_geometry_get, _LibreDWG.Dwg_TableCell_geometry_set)
    style_parent = property(_LibreDWG.Dwg_TableCell_style_parent_get, _LibreDWG.Dwg_TableCell_style_parent_set)
    row_parent = property(_LibreDWG.Dwg_TableCell_row_parent_get, _LibreDWG.Dwg_TableCell_row_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_TableCell_swiginit(self, _LibreDWG.new_Dwg_TableCell())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TableCell

# Register Dwg_TableCell in _LibreDWG:
_LibreDWG.Dwg_TableCell_swigregister(Dwg_TableCell)
class Dwg_GridFormat(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_GridFormat_parent_get, _LibreDWG.Dwg_GridFormat_parent_set)
    index_mask = property(_LibreDWG.Dwg_GridFormat_index_mask_get, _LibreDWG.Dwg_GridFormat_index_mask_set)
    border_overrides = property(_LibreDWG.Dwg_GridFormat_border_overrides_get, _LibreDWG.Dwg_GridFormat_border_overrides_set)
    border_type = property(_LibreDWG.Dwg_GridFormat_border_type_get, _LibreDWG.Dwg_GridFormat_border_type_set)
    color = property(_LibreDWG.Dwg_GridFormat_color_get, _LibreDWG.Dwg_GridFormat_color_set)
    linewt = property(_LibreDWG.Dwg_GridFormat_linewt_get, _LibreDWG.Dwg_GridFormat_linewt_set)
    ltype = property(_LibreDWG.Dwg_GridFormat_ltype_get, _LibreDWG.Dwg_GridFormat_ltype_set)
    visible = property(_LibreDWG.Dwg_GridFormat_visible_get, _LibreDWG.Dwg_GridFormat_visible_set)
    double_line_spacing = property(_LibreDWG.Dwg_GridFormat_double_line_spacing_get, _LibreDWG.Dwg_GridFormat_double_line_spacing_set)

    def __init__(self):
        _LibreDWG.Dwg_GridFormat_swiginit(self, _LibreDWG.new_Dwg_GridFormat())
    __swig_destroy__ = _LibreDWG.delete_Dwg_GridFormat

# Register Dwg_GridFormat in _LibreDWG:
_LibreDWG.Dwg_GridFormat_swigregister(Dwg_GridFormat)
class Dwg_CellStyle(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    type = property(_LibreDWG.Dwg_CellStyle_type_get, _LibreDWG.Dwg_CellStyle_type_set)
    data_flags = property(_LibreDWG.Dwg_CellStyle_data_flags_get, _LibreDWG.Dwg_CellStyle_data_flags_set)
    property_override_flags = property(_LibreDWG.Dwg_CellStyle_property_override_flags_get, _LibreDWG.Dwg_CellStyle_property_override_flags_set)
    merge_flags = property(_LibreDWG.Dwg_CellStyle_merge_flags_get, _LibreDWG.Dwg_CellStyle_merge_flags_set)
    bg_color = property(_LibreDWG.Dwg_CellStyle_bg_color_get, _LibreDWG.Dwg_CellStyle_bg_color_set)
    content_layout = property(_LibreDWG.Dwg_CellStyle_content_layout_get, _LibreDWG.Dwg_CellStyle_content_layout_set)
    content_format = property(_LibreDWG.Dwg_CellStyle_content_format_get, _LibreDWG.Dwg_CellStyle_content_format_set)
    margin_override_flags = property(_LibreDWG.Dwg_CellStyle_margin_override_flags_get, _LibreDWG.Dwg_CellStyle_margin_override_flags_set)
    vert_margin = property(_LibreDWG.Dwg_CellStyle_vert_margin_get, _LibreDWG.Dwg_CellStyle_vert_margin_set)
    horiz_margin = property(_LibreDWG.Dwg_CellStyle_horiz_margin_get, _LibreDWG.Dwg_CellStyle_horiz_margin_set)
    bottom_margin = property(_LibreDWG.Dwg_CellStyle_bottom_margin_get, _LibreDWG.Dwg_CellStyle_bottom_margin_set)
    right_margin = property(_LibreDWG.Dwg_CellStyle_right_margin_get, _LibreDWG.Dwg_CellStyle_right_margin_set)
    margin_horiz_spacing = property(_LibreDWG.Dwg_CellStyle_margin_horiz_spacing_get, _LibreDWG.Dwg_CellStyle_margin_horiz_spacing_set)
    margin_vert_spacing = property(_LibreDWG.Dwg_CellStyle_margin_vert_spacing_get, _LibreDWG.Dwg_CellStyle_margin_vert_spacing_set)
    num_borders = property(_LibreDWG.Dwg_CellStyle_num_borders_get, _LibreDWG.Dwg_CellStyle_num_borders_set)
    borders = property(_LibreDWG.Dwg_CellStyle_borders_get, _LibreDWG.Dwg_CellStyle_borders_set)
    tablerow_parent = property(_LibreDWG.Dwg_CellStyle_tablerow_parent_get, _LibreDWG.Dwg_CellStyle_tablerow_parent_set)
    tabledatacolumn_parent = property(_LibreDWG.Dwg_CellStyle_tabledatacolumn_parent_get, _LibreDWG.Dwg_CellStyle_tabledatacolumn_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_CellStyle_swiginit(self, _LibreDWG.new_Dwg_CellStyle())
    __swig_destroy__ = _LibreDWG.delete_Dwg_CellStyle

# Register Dwg_CellStyle in _LibreDWG:
_LibreDWG.Dwg_CellStyle_swigregister(Dwg_CellStyle)
class Dwg_TableRow(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_TableRow_parent_get, _LibreDWG.Dwg_TableRow_parent_set)
    num_cells = property(_LibreDWG.Dwg_TableRow_num_cells_get, _LibreDWG.Dwg_TableRow_num_cells_set)
    cells = property(_LibreDWG.Dwg_TableRow_cells_get, _LibreDWG.Dwg_TableRow_cells_set)
    custom_data = property(_LibreDWG.Dwg_TableRow_custom_data_get, _LibreDWG.Dwg_TableRow_custom_data_set)
    num_customdata_items = property(_LibreDWG.Dwg_TableRow_num_customdata_items_get, _LibreDWG.Dwg_TableRow_num_customdata_items_set)
    customdata_items = property(_LibreDWG.Dwg_TableRow_customdata_items_get, _LibreDWG.Dwg_TableRow_customdata_items_set)
    cellstyle = property(_LibreDWG.Dwg_TableRow_cellstyle_get, _LibreDWG.Dwg_TableRow_cellstyle_set)
    style_id = property(_LibreDWG.Dwg_TableRow_style_id_get, _LibreDWG.Dwg_TableRow_style_id_set)
    height = property(_LibreDWG.Dwg_TableRow_height_get, _LibreDWG.Dwg_TableRow_height_set)

    def __init__(self):
        _LibreDWG.Dwg_TableRow_swiginit(self, _LibreDWG.new_Dwg_TableRow())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TableRow

# Register Dwg_TableRow in _LibreDWG:
_LibreDWG.Dwg_TableRow_swigregister(Dwg_TableRow)
class Dwg_TableDataColumn(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_TableDataColumn_parent_get, _LibreDWG.Dwg_TableDataColumn_parent_set)
    name = property(_LibreDWG.Dwg_TableDataColumn_name_get, _LibreDWG.Dwg_TableDataColumn_name_set)
    custom_data = property(_LibreDWG.Dwg_TableDataColumn_custom_data_get, _LibreDWG.Dwg_TableDataColumn_custom_data_set)
    cellstyle = property(_LibreDWG.Dwg_TableDataColumn_cellstyle_get, _LibreDWG.Dwg_TableDataColumn_cellstyle_set)
    cellstyle_id = property(_LibreDWG.Dwg_TableDataColumn_cellstyle_id_get, _LibreDWG.Dwg_TableDataColumn_cellstyle_id_set)
    width = property(_LibreDWG.Dwg_TableDataColumn_width_get, _LibreDWG.Dwg_TableDataColumn_width_set)

    def __init__(self):
        _LibreDWG.Dwg_TableDataColumn_swiginit(self, _LibreDWG.new_Dwg_TableDataColumn())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TableDataColumn

# Register Dwg_TableDataColumn in _LibreDWG:
_LibreDWG.Dwg_TableDataColumn_swigregister(Dwg_TableDataColumn)
class Dwg_LinkedTableData(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num_cols = property(_LibreDWG.Dwg_LinkedTableData_num_cols_get, _LibreDWG.Dwg_LinkedTableData_num_cols_set)
    cols = property(_LibreDWG.Dwg_LinkedTableData_cols_get, _LibreDWG.Dwg_LinkedTableData_cols_set)
    num_rows = property(_LibreDWG.Dwg_LinkedTableData_num_rows_get, _LibreDWG.Dwg_LinkedTableData_num_rows_set)
    rows = property(_LibreDWG.Dwg_LinkedTableData_rows_get, _LibreDWG.Dwg_LinkedTableData_rows_set)
    num_field_refs = property(_LibreDWG.Dwg_LinkedTableData_num_field_refs_get, _LibreDWG.Dwg_LinkedTableData_num_field_refs_set)
    field_refs = property(_LibreDWG.Dwg_LinkedTableData_field_refs_get, _LibreDWG.Dwg_LinkedTableData_field_refs_set)

    def __init__(self):
        _LibreDWG.Dwg_LinkedTableData_swiginit(self, _LibreDWG.new_Dwg_LinkedTableData())
    __swig_destroy__ = _LibreDWG.delete_Dwg_LinkedTableData

# Register Dwg_LinkedTableData in _LibreDWG:
_LibreDWG.Dwg_LinkedTableData_swigregister(Dwg_LinkedTableData)
class Dwg_FormattedTableMerged(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_FormattedTableMerged_parent_get, _LibreDWG.Dwg_FormattedTableMerged_parent_set)
    top_row = property(_LibreDWG.Dwg_FormattedTableMerged_top_row_get, _LibreDWG.Dwg_FormattedTableMerged_top_row_set)
    left_col = property(_LibreDWG.Dwg_FormattedTableMerged_left_col_get, _LibreDWG.Dwg_FormattedTableMerged_left_col_set)
    bottom_row = property(_LibreDWG.Dwg_FormattedTableMerged_bottom_row_get, _LibreDWG.Dwg_FormattedTableMerged_bottom_row_set)
    right_col = property(_LibreDWG.Dwg_FormattedTableMerged_right_col_get, _LibreDWG.Dwg_FormattedTableMerged_right_col_set)

    def __init__(self):
        _LibreDWG.Dwg_FormattedTableMerged_swiginit(self, _LibreDWG.new_Dwg_FormattedTableMerged())
    __swig_destroy__ = _LibreDWG.delete_Dwg_FormattedTableMerged

# Register Dwg_FormattedTableMerged in _LibreDWG:
_LibreDWG.Dwg_FormattedTableMerged_swigregister(Dwg_FormattedTableMerged)
class Dwg_FormattedTableData(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_FormattedTableData_parent_get, _LibreDWG.Dwg_FormattedTableData_parent_set)
    cellstyle = property(_LibreDWG.Dwg_FormattedTableData_cellstyle_get, _LibreDWG.Dwg_FormattedTableData_cellstyle_set)
    num_merged_cells = property(_LibreDWG.Dwg_FormattedTableData_num_merged_cells_get, _LibreDWG.Dwg_FormattedTableData_num_merged_cells_set)
    merged_cells = property(_LibreDWG.Dwg_FormattedTableData_merged_cells_get, _LibreDWG.Dwg_FormattedTableData_merged_cells_set)

    def __init__(self):
        _LibreDWG.Dwg_FormattedTableData_swiginit(self, _LibreDWG.new_Dwg_FormattedTableData())
    __swig_destroy__ = _LibreDWG.delete_Dwg_FormattedTableData

# Register Dwg_FormattedTableData in _LibreDWG:
_LibreDWG.Dwg_FormattedTableData_swigregister(Dwg_FormattedTableData)
class Dwg_Object_TABLECONTENT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_TABLECONTENT_parent_get, _LibreDWG.Dwg_Object_TABLECONTENT_parent_set)
    ldata = property(_LibreDWG.Dwg_Object_TABLECONTENT_ldata_get, _LibreDWG.Dwg_Object_TABLECONTENT_ldata_set)
    tdata = property(_LibreDWG.Dwg_Object_TABLECONTENT_tdata_get, _LibreDWG.Dwg_Object_TABLECONTENT_tdata_set)
    fdata = property(_LibreDWG.Dwg_Object_TABLECONTENT_fdata_get, _LibreDWG.Dwg_Object_TABLECONTENT_fdata_set)
    tablestyle = property(_LibreDWG.Dwg_Object_TABLECONTENT_tablestyle_get, _LibreDWG.Dwg_Object_TABLECONTENT_tablestyle_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_TABLECONTENT_swiginit(self, _LibreDWG.new_Dwg_Object_TABLECONTENT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_TABLECONTENT

# Register Dwg_Object_TABLECONTENT in _LibreDWG:
_LibreDWG.Dwg_Object_TABLECONTENT_swigregister(Dwg_Object_TABLECONTENT)
class Dwg_Entity_TABLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_TABLE_parent_get, _LibreDWG.Dwg_Entity_TABLE_parent_set)
    ldata = property(_LibreDWG.Dwg_Entity_TABLE_ldata_get, _LibreDWG.Dwg_Entity_TABLE_ldata_set)
    tdata = property(_LibreDWG.Dwg_Entity_TABLE_tdata_get, _LibreDWG.Dwg_Entity_TABLE_tdata_set)
    fdata = property(_LibreDWG.Dwg_Entity_TABLE_fdata_get, _LibreDWG.Dwg_Entity_TABLE_fdata_set)
    tablestyle = property(_LibreDWG.Dwg_Entity_TABLE_tablestyle_get, _LibreDWG.Dwg_Entity_TABLE_tablestyle_set)
    unknown_rc = property(_LibreDWG.Dwg_Entity_TABLE_unknown_rc_get, _LibreDWG.Dwg_Entity_TABLE_unknown_rc_set)
    unknown_h = property(_LibreDWG.Dwg_Entity_TABLE_unknown_h_get, _LibreDWG.Dwg_Entity_TABLE_unknown_h_set)
    unknown_bl = property(_LibreDWG.Dwg_Entity_TABLE_unknown_bl_get, _LibreDWG.Dwg_Entity_TABLE_unknown_bl_set)
    unknown_b = property(_LibreDWG.Dwg_Entity_TABLE_unknown_b_get, _LibreDWG.Dwg_Entity_TABLE_unknown_b_set)
    unknown_bl1 = property(_LibreDWG.Dwg_Entity_TABLE_unknown_bl1_get, _LibreDWG.Dwg_Entity_TABLE_unknown_bl1_set)
    ins_pt = property(_LibreDWG.Dwg_Entity_TABLE_ins_pt_get, _LibreDWG.Dwg_Entity_TABLE_ins_pt_set)
    scale = property(_LibreDWG.Dwg_Entity_TABLE_scale_get, _LibreDWG.Dwg_Entity_TABLE_scale_set)
    scale_flag = property(_LibreDWG.Dwg_Entity_TABLE_scale_flag_get, _LibreDWG.Dwg_Entity_TABLE_scale_flag_set)
    rotation = property(_LibreDWG.Dwg_Entity_TABLE_rotation_get, _LibreDWG.Dwg_Entity_TABLE_rotation_set)
    extrusion = property(_LibreDWG.Dwg_Entity_TABLE_extrusion_get, _LibreDWG.Dwg_Entity_TABLE_extrusion_set)
    has_attribs = property(_LibreDWG.Dwg_Entity_TABLE_has_attribs_get, _LibreDWG.Dwg_Entity_TABLE_has_attribs_set)
    num_owned = property(_LibreDWG.Dwg_Entity_TABLE_num_owned_get, _LibreDWG.Dwg_Entity_TABLE_num_owned_set)
    flag_for_table_value = property(_LibreDWG.Dwg_Entity_TABLE_flag_for_table_value_get, _LibreDWG.Dwg_Entity_TABLE_flag_for_table_value_set)
    horiz_direction = property(_LibreDWG.Dwg_Entity_TABLE_horiz_direction_get, _LibreDWG.Dwg_Entity_TABLE_horiz_direction_set)
    num_cols = property(_LibreDWG.Dwg_Entity_TABLE_num_cols_get, _LibreDWG.Dwg_Entity_TABLE_num_cols_set)
    num_rows = property(_LibreDWG.Dwg_Entity_TABLE_num_rows_get, _LibreDWG.Dwg_Entity_TABLE_num_rows_set)
    num_cells = property(_LibreDWG.Dwg_Entity_TABLE_num_cells_get, _LibreDWG.Dwg_Entity_TABLE_num_cells_set)
    col_widths = property(_LibreDWG.Dwg_Entity_TABLE_col_widths_get, _LibreDWG.Dwg_Entity_TABLE_col_widths_set)
    row_heights = property(_LibreDWG.Dwg_Entity_TABLE_row_heights_get, _LibreDWG.Dwg_Entity_TABLE_row_heights_set)
    cells = property(_LibreDWG.Dwg_Entity_TABLE_cells_get, _LibreDWG.Dwg_Entity_TABLE_cells_set)
    has_table_overrides = property(_LibreDWG.Dwg_Entity_TABLE_has_table_overrides_get, _LibreDWG.Dwg_Entity_TABLE_has_table_overrides_set)
    table_flag_override = property(_LibreDWG.Dwg_Entity_TABLE_table_flag_override_get, _LibreDWG.Dwg_Entity_TABLE_table_flag_override_set)
    title_suppressed = property(_LibreDWG.Dwg_Entity_TABLE_title_suppressed_get, _LibreDWG.Dwg_Entity_TABLE_title_suppressed_set)
    header_suppressed = property(_LibreDWG.Dwg_Entity_TABLE_header_suppressed_get, _LibreDWG.Dwg_Entity_TABLE_header_suppressed_set)
    flow_direction = property(_LibreDWG.Dwg_Entity_TABLE_flow_direction_get, _LibreDWG.Dwg_Entity_TABLE_flow_direction_set)
    horiz_cell_margin = property(_LibreDWG.Dwg_Entity_TABLE_horiz_cell_margin_get, _LibreDWG.Dwg_Entity_TABLE_horiz_cell_margin_set)
    vert_cell_margin = property(_LibreDWG.Dwg_Entity_TABLE_vert_cell_margin_get, _LibreDWG.Dwg_Entity_TABLE_vert_cell_margin_set)
    title_row_color = property(_LibreDWG.Dwg_Entity_TABLE_title_row_color_get, _LibreDWG.Dwg_Entity_TABLE_title_row_color_set)
    header_row_color = property(_LibreDWG.Dwg_Entity_TABLE_header_row_color_get, _LibreDWG.Dwg_Entity_TABLE_header_row_color_set)
    data_row_color = property(_LibreDWG.Dwg_Entity_TABLE_data_row_color_get, _LibreDWG.Dwg_Entity_TABLE_data_row_color_set)
    title_row_fill_none = property(_LibreDWG.Dwg_Entity_TABLE_title_row_fill_none_get, _LibreDWG.Dwg_Entity_TABLE_title_row_fill_none_set)
    header_row_fill_none = property(_LibreDWG.Dwg_Entity_TABLE_header_row_fill_none_get, _LibreDWG.Dwg_Entity_TABLE_header_row_fill_none_set)
    data_row_fill_none = property(_LibreDWG.Dwg_Entity_TABLE_data_row_fill_none_get, _LibreDWG.Dwg_Entity_TABLE_data_row_fill_none_set)
    title_row_fill_color = property(_LibreDWG.Dwg_Entity_TABLE_title_row_fill_color_get, _LibreDWG.Dwg_Entity_TABLE_title_row_fill_color_set)
    header_row_fill_color = property(_LibreDWG.Dwg_Entity_TABLE_header_row_fill_color_get, _LibreDWG.Dwg_Entity_TABLE_header_row_fill_color_set)
    data_row_fill_color = property(_LibreDWG.Dwg_Entity_TABLE_data_row_fill_color_get, _LibreDWG.Dwg_Entity_TABLE_data_row_fill_color_set)
    title_row_alignment = property(_LibreDWG.Dwg_Entity_TABLE_title_row_alignment_get, _LibreDWG.Dwg_Entity_TABLE_title_row_alignment_set)
    header_row_alignment = property(_LibreDWG.Dwg_Entity_TABLE_header_row_alignment_get, _LibreDWG.Dwg_Entity_TABLE_header_row_alignment_set)
    data_row_alignment = property(_LibreDWG.Dwg_Entity_TABLE_data_row_alignment_get, _LibreDWG.Dwg_Entity_TABLE_data_row_alignment_set)
    title_text_style = property(_LibreDWG.Dwg_Entity_TABLE_title_text_style_get, _LibreDWG.Dwg_Entity_TABLE_title_text_style_set)
    header_text_style = property(_LibreDWG.Dwg_Entity_TABLE_header_text_style_get, _LibreDWG.Dwg_Entity_TABLE_header_text_style_set)
    data_text_style = property(_LibreDWG.Dwg_Entity_TABLE_data_text_style_get, _LibreDWG.Dwg_Entity_TABLE_data_text_style_set)
    title_row_height = property(_LibreDWG.Dwg_Entity_TABLE_title_row_height_get, _LibreDWG.Dwg_Entity_TABLE_title_row_height_set)
    header_row_height = property(_LibreDWG.Dwg_Entity_TABLE_header_row_height_get, _LibreDWG.Dwg_Entity_TABLE_header_row_height_set)
    data_row_height = property(_LibreDWG.Dwg_Entity_TABLE_data_row_height_get, _LibreDWG.Dwg_Entity_TABLE_data_row_height_set)
    has_border_color_overrides = property(_LibreDWG.Dwg_Entity_TABLE_has_border_color_overrides_get, _LibreDWG.Dwg_Entity_TABLE_has_border_color_overrides_set)
    border_color_overrides_flag = property(_LibreDWG.Dwg_Entity_TABLE_border_color_overrides_flag_get, _LibreDWG.Dwg_Entity_TABLE_border_color_overrides_flag_set)
    title_horiz_top_color = property(_LibreDWG.Dwg_Entity_TABLE_title_horiz_top_color_get, _LibreDWG.Dwg_Entity_TABLE_title_horiz_top_color_set)
    title_horiz_ins_color = property(_LibreDWG.Dwg_Entity_TABLE_title_horiz_ins_color_get, _LibreDWG.Dwg_Entity_TABLE_title_horiz_ins_color_set)
    title_horiz_bottom_color = property(_LibreDWG.Dwg_Entity_TABLE_title_horiz_bottom_color_get, _LibreDWG.Dwg_Entity_TABLE_title_horiz_bottom_color_set)
    title_vert_left_color = property(_LibreDWG.Dwg_Entity_TABLE_title_vert_left_color_get, _LibreDWG.Dwg_Entity_TABLE_title_vert_left_color_set)
    title_vert_ins_color = property(_LibreDWG.Dwg_Entity_TABLE_title_vert_ins_color_get, _LibreDWG.Dwg_Entity_TABLE_title_vert_ins_color_set)
    title_vert_right_color = property(_LibreDWG.Dwg_Entity_TABLE_title_vert_right_color_get, _LibreDWG.Dwg_Entity_TABLE_title_vert_right_color_set)
    header_horiz_top_color = property(_LibreDWG.Dwg_Entity_TABLE_header_horiz_top_color_get, _LibreDWG.Dwg_Entity_TABLE_header_horiz_top_color_set)
    header_horiz_ins_color = property(_LibreDWG.Dwg_Entity_TABLE_header_horiz_ins_color_get, _LibreDWG.Dwg_Entity_TABLE_header_horiz_ins_color_set)
    header_horiz_bottom_color = property(_LibreDWG.Dwg_Entity_TABLE_header_horiz_bottom_color_get, _LibreDWG.Dwg_Entity_TABLE_header_horiz_bottom_color_set)
    header_vert_left_color = property(_LibreDWG.Dwg_Entity_TABLE_header_vert_left_color_get, _LibreDWG.Dwg_Entity_TABLE_header_vert_left_color_set)
    header_vert_ins_color = property(_LibreDWG.Dwg_Entity_TABLE_header_vert_ins_color_get, _LibreDWG.Dwg_Entity_TABLE_header_vert_ins_color_set)
    header_vert_right_color = property(_LibreDWG.Dwg_Entity_TABLE_header_vert_right_color_get, _LibreDWG.Dwg_Entity_TABLE_header_vert_right_color_set)
    data_horiz_top_color = property(_LibreDWG.Dwg_Entity_TABLE_data_horiz_top_color_get, _LibreDWG.Dwg_Entity_TABLE_data_horiz_top_color_set)
    data_horiz_ins_color = property(_LibreDWG.Dwg_Entity_TABLE_data_horiz_ins_color_get, _LibreDWG.Dwg_Entity_TABLE_data_horiz_ins_color_set)
    data_horiz_bottom_color = property(_LibreDWG.Dwg_Entity_TABLE_data_horiz_bottom_color_get, _LibreDWG.Dwg_Entity_TABLE_data_horiz_bottom_color_set)
    data_vert_left_color = property(_LibreDWG.Dwg_Entity_TABLE_data_vert_left_color_get, _LibreDWG.Dwg_Entity_TABLE_data_vert_left_color_set)
    data_vert_ins_color = property(_LibreDWG.Dwg_Entity_TABLE_data_vert_ins_color_get, _LibreDWG.Dwg_Entity_TABLE_data_vert_ins_color_set)
    data_vert_right_color = property(_LibreDWG.Dwg_Entity_TABLE_data_vert_right_color_get, _LibreDWG.Dwg_Entity_TABLE_data_vert_right_color_set)
    has_border_lineweight_overrides = property(_LibreDWG.Dwg_Entity_TABLE_has_border_lineweight_overrides_get, _LibreDWG.Dwg_Entity_TABLE_has_border_lineweight_overrides_set)
    border_lineweight_overrides_flag = property(_LibreDWG.Dwg_Entity_TABLE_border_lineweight_overrides_flag_get, _LibreDWG.Dwg_Entity_TABLE_border_lineweight_overrides_flag_set)
    title_horiz_top_linewt = property(_LibreDWG.Dwg_Entity_TABLE_title_horiz_top_linewt_get, _LibreDWG.Dwg_Entity_TABLE_title_horiz_top_linewt_set)
    title_horiz_ins_linewt = property(_LibreDWG.Dwg_Entity_TABLE_title_horiz_ins_linewt_get, _LibreDWG.Dwg_Entity_TABLE_title_horiz_ins_linewt_set)
    title_horiz_bottom_linewt = property(_LibreDWG.Dwg_Entity_TABLE_title_horiz_bottom_linewt_get, _LibreDWG.Dwg_Entity_TABLE_title_horiz_bottom_linewt_set)
    title_vert_left_linewt = property(_LibreDWG.Dwg_Entity_TABLE_title_vert_left_linewt_get, _LibreDWG.Dwg_Entity_TABLE_title_vert_left_linewt_set)
    title_vert_ins_linewt = property(_LibreDWG.Dwg_Entity_TABLE_title_vert_ins_linewt_get, _LibreDWG.Dwg_Entity_TABLE_title_vert_ins_linewt_set)
    title_vert_right_linewt = property(_LibreDWG.Dwg_Entity_TABLE_title_vert_right_linewt_get, _LibreDWG.Dwg_Entity_TABLE_title_vert_right_linewt_set)
    header_horiz_top_linewt = property(_LibreDWG.Dwg_Entity_TABLE_header_horiz_top_linewt_get, _LibreDWG.Dwg_Entity_TABLE_header_horiz_top_linewt_set)
    header_horiz_ins_linewt = property(_LibreDWG.Dwg_Entity_TABLE_header_horiz_ins_linewt_get, _LibreDWG.Dwg_Entity_TABLE_header_horiz_ins_linewt_set)
    header_horiz_bottom_linewt = property(_LibreDWG.Dwg_Entity_TABLE_header_horiz_bottom_linewt_get, _LibreDWG.Dwg_Entity_TABLE_header_horiz_bottom_linewt_set)
    header_vert_left_linewt = property(_LibreDWG.Dwg_Entity_TABLE_header_vert_left_linewt_get, _LibreDWG.Dwg_Entity_TABLE_header_vert_left_linewt_set)
    header_vert_ins_linewt = property(_LibreDWG.Dwg_Entity_TABLE_header_vert_ins_linewt_get, _LibreDWG.Dwg_Entity_TABLE_header_vert_ins_linewt_set)
    header_vert_right_linewt = property(_LibreDWG.Dwg_Entity_TABLE_header_vert_right_linewt_get, _LibreDWG.Dwg_Entity_TABLE_header_vert_right_linewt_set)
    data_horiz_top_linewt = property(_LibreDWG.Dwg_Entity_TABLE_data_horiz_top_linewt_get, _LibreDWG.Dwg_Entity_TABLE_data_horiz_top_linewt_set)
    data_horiz_ins_linewt = property(_LibreDWG.Dwg_Entity_TABLE_data_horiz_ins_linewt_get, _LibreDWG.Dwg_Entity_TABLE_data_horiz_ins_linewt_set)
    data_horiz_bottom_linewt = property(_LibreDWG.Dwg_Entity_TABLE_data_horiz_bottom_linewt_get, _LibreDWG.Dwg_Entity_TABLE_data_horiz_bottom_linewt_set)
    data_vert_left_linewt = property(_LibreDWG.Dwg_Entity_TABLE_data_vert_left_linewt_get, _LibreDWG.Dwg_Entity_TABLE_data_vert_left_linewt_set)
    data_vert_ins_linewt = property(_LibreDWG.Dwg_Entity_TABLE_data_vert_ins_linewt_get, _LibreDWG.Dwg_Entity_TABLE_data_vert_ins_linewt_set)
    data_vert_right_linewt = property(_LibreDWG.Dwg_Entity_TABLE_data_vert_right_linewt_get, _LibreDWG.Dwg_Entity_TABLE_data_vert_right_linewt_set)
    has_border_visibility_overrides = property(_LibreDWG.Dwg_Entity_TABLE_has_border_visibility_overrides_get, _LibreDWG.Dwg_Entity_TABLE_has_border_visibility_overrides_set)
    border_visibility_overrides_flag = property(_LibreDWG.Dwg_Entity_TABLE_border_visibility_overrides_flag_get, _LibreDWG.Dwg_Entity_TABLE_border_visibility_overrides_flag_set)
    title_horiz_top_visibility = property(_LibreDWG.Dwg_Entity_TABLE_title_horiz_top_visibility_get, _LibreDWG.Dwg_Entity_TABLE_title_horiz_top_visibility_set)
    title_horiz_ins_visibility = property(_LibreDWG.Dwg_Entity_TABLE_title_horiz_ins_visibility_get, _LibreDWG.Dwg_Entity_TABLE_title_horiz_ins_visibility_set)
    title_horiz_bottom_visibility = property(_LibreDWG.Dwg_Entity_TABLE_title_horiz_bottom_visibility_get, _LibreDWG.Dwg_Entity_TABLE_title_horiz_bottom_visibility_set)
    title_vert_left_visibility = property(_LibreDWG.Dwg_Entity_TABLE_title_vert_left_visibility_get, _LibreDWG.Dwg_Entity_TABLE_title_vert_left_visibility_set)
    title_vert_ins_visibility = property(_LibreDWG.Dwg_Entity_TABLE_title_vert_ins_visibility_get, _LibreDWG.Dwg_Entity_TABLE_title_vert_ins_visibility_set)
    title_vert_right_visibility = property(_LibreDWG.Dwg_Entity_TABLE_title_vert_right_visibility_get, _LibreDWG.Dwg_Entity_TABLE_title_vert_right_visibility_set)
    header_horiz_top_visibility = property(_LibreDWG.Dwg_Entity_TABLE_header_horiz_top_visibility_get, _LibreDWG.Dwg_Entity_TABLE_header_horiz_top_visibility_set)
    header_horiz_ins_visibility = property(_LibreDWG.Dwg_Entity_TABLE_header_horiz_ins_visibility_get, _LibreDWG.Dwg_Entity_TABLE_header_horiz_ins_visibility_set)
    header_horiz_bottom_visibility = property(_LibreDWG.Dwg_Entity_TABLE_header_horiz_bottom_visibility_get, _LibreDWG.Dwg_Entity_TABLE_header_horiz_bottom_visibility_set)
    header_vert_left_visibility = property(_LibreDWG.Dwg_Entity_TABLE_header_vert_left_visibility_get, _LibreDWG.Dwg_Entity_TABLE_header_vert_left_visibility_set)
    header_vert_ins_visibility = property(_LibreDWG.Dwg_Entity_TABLE_header_vert_ins_visibility_get, _LibreDWG.Dwg_Entity_TABLE_header_vert_ins_visibility_set)
    header_vert_right_visibility = property(_LibreDWG.Dwg_Entity_TABLE_header_vert_right_visibility_get, _LibreDWG.Dwg_Entity_TABLE_header_vert_right_visibility_set)
    data_horiz_top_visibility = property(_LibreDWG.Dwg_Entity_TABLE_data_horiz_top_visibility_get, _LibreDWG.Dwg_Entity_TABLE_data_horiz_top_visibility_set)
    data_horiz_ins_visibility = property(_LibreDWG.Dwg_Entity_TABLE_data_horiz_ins_visibility_get, _LibreDWG.Dwg_Entity_TABLE_data_horiz_ins_visibility_set)
    data_horiz_bottom_visibility = property(_LibreDWG.Dwg_Entity_TABLE_data_horiz_bottom_visibility_get, _LibreDWG.Dwg_Entity_TABLE_data_horiz_bottom_visibility_set)
    data_vert_left_visibility = property(_LibreDWG.Dwg_Entity_TABLE_data_vert_left_visibility_get, _LibreDWG.Dwg_Entity_TABLE_data_vert_left_visibility_set)
    data_vert_ins_visibility = property(_LibreDWG.Dwg_Entity_TABLE_data_vert_ins_visibility_get, _LibreDWG.Dwg_Entity_TABLE_data_vert_ins_visibility_set)
    data_vert_right_visibility = property(_LibreDWG.Dwg_Entity_TABLE_data_vert_right_visibility_get, _LibreDWG.Dwg_Entity_TABLE_data_vert_right_visibility_set)
    block_header = property(_LibreDWG.Dwg_Entity_TABLE_block_header_get, _LibreDWG.Dwg_Entity_TABLE_block_header_set)
    first_attrib = property(_LibreDWG.Dwg_Entity_TABLE_first_attrib_get, _LibreDWG.Dwg_Entity_TABLE_first_attrib_set)
    last_attrib = property(_LibreDWG.Dwg_Entity_TABLE_last_attrib_get, _LibreDWG.Dwg_Entity_TABLE_last_attrib_set)
    attribs = property(_LibreDWG.Dwg_Entity_TABLE_attribs_get, _LibreDWG.Dwg_Entity_TABLE_attribs_set)
    seqend = property(_LibreDWG.Dwg_Entity_TABLE_seqend_get, _LibreDWG.Dwg_Entity_TABLE_seqend_set)
    title_row_style_override = property(_LibreDWG.Dwg_Entity_TABLE_title_row_style_override_get, _LibreDWG.Dwg_Entity_TABLE_title_row_style_override_set)
    header_row_style_override = property(_LibreDWG.Dwg_Entity_TABLE_header_row_style_override_get, _LibreDWG.Dwg_Entity_TABLE_header_row_style_override_set)
    data_row_style_override = property(_LibreDWG.Dwg_Entity_TABLE_data_row_style_override_get, _LibreDWG.Dwg_Entity_TABLE_data_row_style_override_set)
    unknown_bs = property(_LibreDWG.Dwg_Entity_TABLE_unknown_bs_get, _LibreDWG.Dwg_Entity_TABLE_unknown_bs_set)
    hor_dir = property(_LibreDWG.Dwg_Entity_TABLE_hor_dir_get, _LibreDWG.Dwg_Entity_TABLE_hor_dir_set)
    has_break_data = property(_LibreDWG.Dwg_Entity_TABLE_has_break_data_get, _LibreDWG.Dwg_Entity_TABLE_has_break_data_set)
    break_flag = property(_LibreDWG.Dwg_Entity_TABLE_break_flag_get, _LibreDWG.Dwg_Entity_TABLE_break_flag_set)
    break_flow_direction = property(_LibreDWG.Dwg_Entity_TABLE_break_flow_direction_get, _LibreDWG.Dwg_Entity_TABLE_break_flow_direction_set)
    break_spacing = property(_LibreDWG.Dwg_Entity_TABLE_break_spacing_get, _LibreDWG.Dwg_Entity_TABLE_break_spacing_set)
    break_unknown1 = property(_LibreDWG.Dwg_Entity_TABLE_break_unknown1_get, _LibreDWG.Dwg_Entity_TABLE_break_unknown1_set)
    break_unknown2 = property(_LibreDWG.Dwg_Entity_TABLE_break_unknown2_get, _LibreDWG.Dwg_Entity_TABLE_break_unknown2_set)
    num_break_heights = property(_LibreDWG.Dwg_Entity_TABLE_num_break_heights_get, _LibreDWG.Dwg_Entity_TABLE_num_break_heights_set)
    break_heights = property(_LibreDWG.Dwg_Entity_TABLE_break_heights_get, _LibreDWG.Dwg_Entity_TABLE_break_heights_set)
    num_break_rows = property(_LibreDWG.Dwg_Entity_TABLE_num_break_rows_get, _LibreDWG.Dwg_Entity_TABLE_num_break_rows_set)
    break_rows = property(_LibreDWG.Dwg_Entity_TABLE_break_rows_get, _LibreDWG.Dwg_Entity_TABLE_break_rows_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_TABLE_swiginit(self, _LibreDWG.new_Dwg_Entity_TABLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_TABLE

# Register Dwg_Entity_TABLE in _LibreDWG:
_LibreDWG.Dwg_Entity_TABLE_swigregister(Dwg_Entity_TABLE)
class Dwg_TABLESTYLE_CellStyle(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_TABLESTYLE_CellStyle_parent_get, _LibreDWG.Dwg_TABLESTYLE_CellStyle_parent_set)
    id = property(_LibreDWG.Dwg_TABLESTYLE_CellStyle_id_get, _LibreDWG.Dwg_TABLESTYLE_CellStyle_id_set)
    type = property(_LibreDWG.Dwg_TABLESTYLE_CellStyle_type_get, _LibreDWG.Dwg_TABLESTYLE_CellStyle_type_set)
    name = property(_LibreDWG.Dwg_TABLESTYLE_CellStyle_name_get, _LibreDWG.Dwg_TABLESTYLE_CellStyle_name_set)
    cellstyle = property(_LibreDWG.Dwg_TABLESTYLE_CellStyle_cellstyle_get, _LibreDWG.Dwg_TABLESTYLE_CellStyle_cellstyle_set)

    def __init__(self):
        _LibreDWG.Dwg_TABLESTYLE_CellStyle_swiginit(self, _LibreDWG.new_Dwg_TABLESTYLE_CellStyle())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TABLESTYLE_CellStyle

# Register Dwg_TABLESTYLE_CellStyle in _LibreDWG:
_LibreDWG.Dwg_TABLESTYLE_CellStyle_swigregister(Dwg_TABLESTYLE_CellStyle)
class Dwg_TABLESTYLE_border(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    linewt = property(_LibreDWG.Dwg_TABLESTYLE_border_linewt_get, _LibreDWG.Dwg_TABLESTYLE_border_linewt_set)
    visible = property(_LibreDWG.Dwg_TABLESTYLE_border_visible_get, _LibreDWG.Dwg_TABLESTYLE_border_visible_set)
    color = property(_LibreDWG.Dwg_TABLESTYLE_border_color_get, _LibreDWG.Dwg_TABLESTYLE_border_color_set)

    def __init__(self):
        _LibreDWG.Dwg_TABLESTYLE_border_swiginit(self, _LibreDWG.new_Dwg_TABLESTYLE_border())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TABLESTYLE_border

# Register Dwg_TABLESTYLE_border in _LibreDWG:
_LibreDWG.Dwg_TABLESTYLE_border_swigregister(Dwg_TABLESTYLE_border)
class Dwg_TABLESTYLE_rowstyles(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_TABLESTYLE_rowstyles_parent_get, _LibreDWG.Dwg_TABLESTYLE_rowstyles_parent_set)
    text_style = property(_LibreDWG.Dwg_TABLESTYLE_rowstyles_text_style_get, _LibreDWG.Dwg_TABLESTYLE_rowstyles_text_style_set)
    text_height = property(_LibreDWG.Dwg_TABLESTYLE_rowstyles_text_height_get, _LibreDWG.Dwg_TABLESTYLE_rowstyles_text_height_set)
    text_alignment = property(_LibreDWG.Dwg_TABLESTYLE_rowstyles_text_alignment_get, _LibreDWG.Dwg_TABLESTYLE_rowstyles_text_alignment_set)
    text_color = property(_LibreDWG.Dwg_TABLESTYLE_rowstyles_text_color_get, _LibreDWG.Dwg_TABLESTYLE_rowstyles_text_color_set)
    fill_color = property(_LibreDWG.Dwg_TABLESTYLE_rowstyles_fill_color_get, _LibreDWG.Dwg_TABLESTYLE_rowstyles_fill_color_set)
    has_bgcolor = property(_LibreDWG.Dwg_TABLESTYLE_rowstyles_has_bgcolor_get, _LibreDWG.Dwg_TABLESTYLE_rowstyles_has_bgcolor_set)
    num_borders = property(_LibreDWG.Dwg_TABLESTYLE_rowstyles_num_borders_get, _LibreDWG.Dwg_TABLESTYLE_rowstyles_num_borders_set)
    borders = property(_LibreDWG.Dwg_TABLESTYLE_rowstyles_borders_get, _LibreDWG.Dwg_TABLESTYLE_rowstyles_borders_set)
    data_type = property(_LibreDWG.Dwg_TABLESTYLE_rowstyles_data_type_get, _LibreDWG.Dwg_TABLESTYLE_rowstyles_data_type_set)
    unit_type = property(_LibreDWG.Dwg_TABLESTYLE_rowstyles_unit_type_get, _LibreDWG.Dwg_TABLESTYLE_rowstyles_unit_type_set)
    format_string = property(_LibreDWG.Dwg_TABLESTYLE_rowstyles_format_string_get, _LibreDWG.Dwg_TABLESTYLE_rowstyles_format_string_set)

    def __init__(self):
        _LibreDWG.Dwg_TABLESTYLE_rowstyles_swiginit(self, _LibreDWG.new_Dwg_TABLESTYLE_rowstyles())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TABLESTYLE_rowstyles

# Register Dwg_TABLESTYLE_rowstyles in _LibreDWG:
_LibreDWG.Dwg_TABLESTYLE_rowstyles_swigregister(Dwg_TABLESTYLE_rowstyles)
class Dwg_Object_TABLESTYLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_TABLESTYLE_parent_get, _LibreDWG.Dwg_Object_TABLESTYLE_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_TABLESTYLE_class_version_get, _LibreDWG.Dwg_Object_TABLESTYLE_class_version_set)
    name = property(_LibreDWG.Dwg_Object_TABLESTYLE_name_get, _LibreDWG.Dwg_Object_TABLESTYLE_name_set)
    flags = property(_LibreDWG.Dwg_Object_TABLESTYLE_flags_get, _LibreDWG.Dwg_Object_TABLESTYLE_flags_set)
    flow_direction = property(_LibreDWG.Dwg_Object_TABLESTYLE_flow_direction_get, _LibreDWG.Dwg_Object_TABLESTYLE_flow_direction_set)
    horiz_cell_margin = property(_LibreDWG.Dwg_Object_TABLESTYLE_horiz_cell_margin_get, _LibreDWG.Dwg_Object_TABLESTYLE_horiz_cell_margin_set)
    vert_cell_margin = property(_LibreDWG.Dwg_Object_TABLESTYLE_vert_cell_margin_get, _LibreDWG.Dwg_Object_TABLESTYLE_vert_cell_margin_set)
    is_title_suppressed = property(_LibreDWG.Dwg_Object_TABLESTYLE_is_title_suppressed_get, _LibreDWG.Dwg_Object_TABLESTYLE_is_title_suppressed_set)
    is_header_suppressed = property(_LibreDWG.Dwg_Object_TABLESTYLE_is_header_suppressed_get, _LibreDWG.Dwg_Object_TABLESTYLE_is_header_suppressed_set)
    unknown_rc = property(_LibreDWG.Dwg_Object_TABLESTYLE_unknown_rc_get, _LibreDWG.Dwg_Object_TABLESTYLE_unknown_rc_set)
    unknown_bl1 = property(_LibreDWG.Dwg_Object_TABLESTYLE_unknown_bl1_get, _LibreDWG.Dwg_Object_TABLESTYLE_unknown_bl1_set)
    unknown_bl2 = property(_LibreDWG.Dwg_Object_TABLESTYLE_unknown_bl2_get, _LibreDWG.Dwg_Object_TABLESTYLE_unknown_bl2_set)
    cellstyle = property(_LibreDWG.Dwg_Object_TABLESTYLE_cellstyle_get, _LibreDWG.Dwg_Object_TABLESTYLE_cellstyle_set)
    sty = property(_LibreDWG.Dwg_Object_TABLESTYLE_sty_get, _LibreDWG.Dwg_Object_TABLESTYLE_sty_set)
    numoverrides = property(_LibreDWG.Dwg_Object_TABLESTYLE_numoverrides_get, _LibreDWG.Dwg_Object_TABLESTYLE_numoverrides_set)
    unknown_bl3 = property(_LibreDWG.Dwg_Object_TABLESTYLE_unknown_bl3_get, _LibreDWG.Dwg_Object_TABLESTYLE_unknown_bl3_set)
    ovr = property(_LibreDWG.Dwg_Object_TABLESTYLE_ovr_get, _LibreDWG.Dwg_Object_TABLESTYLE_ovr_set)
    num_rowstyles = property(_LibreDWG.Dwg_Object_TABLESTYLE_num_rowstyles_get, _LibreDWG.Dwg_Object_TABLESTYLE_num_rowstyles_set)
    rowstyles = property(_LibreDWG.Dwg_Object_TABLESTYLE_rowstyles_get, _LibreDWG.Dwg_Object_TABLESTYLE_rowstyles_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_TABLESTYLE_swiginit(self, _LibreDWG.new_Dwg_Object_TABLESTYLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_TABLESTYLE

# Register Dwg_Object_TABLESTYLE in _LibreDWG:
_LibreDWG.Dwg_Object_TABLESTYLE_swigregister(Dwg_Object_TABLESTYLE)
class Dwg_Object_CELLSTYLEMAP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_CELLSTYLEMAP_parent_get, _LibreDWG.Dwg_Object_CELLSTYLEMAP_parent_set)
    num_cells = property(_LibreDWG.Dwg_Object_CELLSTYLEMAP_num_cells_get, _LibreDWG.Dwg_Object_CELLSTYLEMAP_num_cells_set)
    cells = property(_LibreDWG.Dwg_Object_CELLSTYLEMAP_cells_get, _LibreDWG.Dwg_Object_CELLSTYLEMAP_cells_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_CELLSTYLEMAP_swiginit(self, _LibreDWG.new_Dwg_Object_CELLSTYLEMAP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_CELLSTYLEMAP

# Register Dwg_Object_CELLSTYLEMAP in _LibreDWG:
_LibreDWG.Dwg_Object_CELLSTYLEMAP_swigregister(Dwg_Object_CELLSTYLEMAP)
class Dwg_TABLEGEOMETRY_Cell(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_TABLEGEOMETRY_Cell_parent_get, _LibreDWG.Dwg_TABLEGEOMETRY_Cell_parent_set)
    geom_data_flag = property(_LibreDWG.Dwg_TABLEGEOMETRY_Cell_geom_data_flag_get, _LibreDWG.Dwg_TABLEGEOMETRY_Cell_geom_data_flag_set)
    width_w_gap = property(_LibreDWG.Dwg_TABLEGEOMETRY_Cell_width_w_gap_get, _LibreDWG.Dwg_TABLEGEOMETRY_Cell_width_w_gap_set)
    height_w_gap = property(_LibreDWG.Dwg_TABLEGEOMETRY_Cell_height_w_gap_get, _LibreDWG.Dwg_TABLEGEOMETRY_Cell_height_w_gap_set)
    tablegeometry = property(_LibreDWG.Dwg_TABLEGEOMETRY_Cell_tablegeometry_get, _LibreDWG.Dwg_TABLEGEOMETRY_Cell_tablegeometry_set)
    num_geometry = property(_LibreDWG.Dwg_TABLEGEOMETRY_Cell_num_geometry_get, _LibreDWG.Dwg_TABLEGEOMETRY_Cell_num_geometry_set)
    geometry = property(_LibreDWG.Dwg_TABLEGEOMETRY_Cell_geometry_get, _LibreDWG.Dwg_TABLEGEOMETRY_Cell_geometry_set)

    def __init__(self):
        _LibreDWG.Dwg_TABLEGEOMETRY_Cell_swiginit(self, _LibreDWG.new_Dwg_TABLEGEOMETRY_Cell())
    __swig_destroy__ = _LibreDWG.delete_Dwg_TABLEGEOMETRY_Cell

# Register Dwg_TABLEGEOMETRY_Cell in _LibreDWG:
_LibreDWG.Dwg_TABLEGEOMETRY_Cell_swigregister(Dwg_TABLEGEOMETRY_Cell)
class Dwg_Object_TABLEGEOMETRY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_TABLEGEOMETRY_parent_get, _LibreDWG.Dwg_Object_TABLEGEOMETRY_parent_set)
    numrows = property(_LibreDWG.Dwg_Object_TABLEGEOMETRY_numrows_get, _LibreDWG.Dwg_Object_TABLEGEOMETRY_numrows_set)
    numcols = property(_LibreDWG.Dwg_Object_TABLEGEOMETRY_numcols_get, _LibreDWG.Dwg_Object_TABLEGEOMETRY_numcols_set)
    num_cells = property(_LibreDWG.Dwg_Object_TABLEGEOMETRY_num_cells_get, _LibreDWG.Dwg_Object_TABLEGEOMETRY_num_cells_set)
    cells = property(_LibreDWG.Dwg_Object_TABLEGEOMETRY_cells_get, _LibreDWG.Dwg_Object_TABLEGEOMETRY_cells_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_TABLEGEOMETRY_swiginit(self, _LibreDWG.new_Dwg_Object_TABLEGEOMETRY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_TABLEGEOMETRY

# Register Dwg_Object_TABLEGEOMETRY in _LibreDWG:
_LibreDWG.Dwg_Object_TABLEGEOMETRY_swigregister(Dwg_Object_TABLEGEOMETRY)
class Dwg_Object_UNDERLAYDEFINITION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_UNDERLAYDEFINITION_parent_get, _LibreDWG.Dwg_Object_UNDERLAYDEFINITION_parent_set)
    filename = property(_LibreDWG.Dwg_Object_UNDERLAYDEFINITION_filename_get, _LibreDWG.Dwg_Object_UNDERLAYDEFINITION_filename_set)
    name = property(_LibreDWG.Dwg_Object_UNDERLAYDEFINITION_name_get, _LibreDWG.Dwg_Object_UNDERLAYDEFINITION_name_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_UNDERLAYDEFINITION_swiginit(self, _LibreDWG.new_Dwg_Object_UNDERLAYDEFINITION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_UNDERLAYDEFINITION

# Register Dwg_Object_UNDERLAYDEFINITION in _LibreDWG:
_LibreDWG.Dwg_Object_UNDERLAYDEFINITION_swigregister(Dwg_Object_UNDERLAYDEFINITION)
class Dwg_Entity_UNDERLAY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_UNDERLAY_parent_get, _LibreDWG.Dwg_Entity_UNDERLAY_parent_set)
    extrusion = property(_LibreDWG.Dwg_Entity_UNDERLAY_extrusion_get, _LibreDWG.Dwg_Entity_UNDERLAY_extrusion_set)
    ins_pt = property(_LibreDWG.Dwg_Entity_UNDERLAY_ins_pt_get, _LibreDWG.Dwg_Entity_UNDERLAY_ins_pt_set)
    scale = property(_LibreDWG.Dwg_Entity_UNDERLAY_scale_get, _LibreDWG.Dwg_Entity_UNDERLAY_scale_set)
    angle = property(_LibreDWG.Dwg_Entity_UNDERLAY_angle_get, _LibreDWG.Dwg_Entity_UNDERLAY_angle_set)
    flag = property(_LibreDWG.Dwg_Entity_UNDERLAY_flag_get, _LibreDWG.Dwg_Entity_UNDERLAY_flag_set)
    contrast = property(_LibreDWG.Dwg_Entity_UNDERLAY_contrast_get, _LibreDWG.Dwg_Entity_UNDERLAY_contrast_set)
    fade = property(_LibreDWG.Dwg_Entity_UNDERLAY_fade_get, _LibreDWG.Dwg_Entity_UNDERLAY_fade_set)
    num_clip_verts = property(_LibreDWG.Dwg_Entity_UNDERLAY_num_clip_verts_get, _LibreDWG.Dwg_Entity_UNDERLAY_num_clip_verts_set)
    clip_verts = property(_LibreDWG.Dwg_Entity_UNDERLAY_clip_verts_get, _LibreDWG.Dwg_Entity_UNDERLAY_clip_verts_set)
    num_clip_inverts = property(_LibreDWG.Dwg_Entity_UNDERLAY_num_clip_inverts_get, _LibreDWG.Dwg_Entity_UNDERLAY_num_clip_inverts_set)
    clip_inverts = property(_LibreDWG.Dwg_Entity_UNDERLAY_clip_inverts_get, _LibreDWG.Dwg_Entity_UNDERLAY_clip_inverts_set)
    definition_id = property(_LibreDWG.Dwg_Entity_UNDERLAY_definition_id_get, _LibreDWG.Dwg_Entity_UNDERLAY_definition_id_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_UNDERLAY_swiginit(self, _LibreDWG.new_Dwg_Entity_UNDERLAY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_UNDERLAY

# Register Dwg_Entity_UNDERLAY in _LibreDWG:
_LibreDWG.Dwg_Entity_UNDERLAY_swigregister(Dwg_Entity_UNDERLAY)
class Dwg_Object_DBCOLOR(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DBCOLOR_parent_get, _LibreDWG.Dwg_Object_DBCOLOR_parent_set)
    color = property(_LibreDWG.Dwg_Object_DBCOLOR_color_get, _LibreDWG.Dwg_Object_DBCOLOR_color_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DBCOLOR_swiginit(self, _LibreDWG.new_Dwg_Object_DBCOLOR())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DBCOLOR

# Register Dwg_Object_DBCOLOR in _LibreDWG:
_LibreDWG.Dwg_Object_DBCOLOR_swigregister(Dwg_Object_DBCOLOR)
class Dwg_FIELD_ChildValue(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_FIELD_ChildValue_parent_get, _LibreDWG.Dwg_FIELD_ChildValue_parent_set)
    key = property(_LibreDWG.Dwg_FIELD_ChildValue_key_get, _LibreDWG.Dwg_FIELD_ChildValue_key_set)
    value = property(_LibreDWG.Dwg_FIELD_ChildValue_value_get, _LibreDWG.Dwg_FIELD_ChildValue_value_set)

    def __init__(self):
        _LibreDWG.Dwg_FIELD_ChildValue_swiginit(self, _LibreDWG.new_Dwg_FIELD_ChildValue())
    __swig_destroy__ = _LibreDWG.delete_Dwg_FIELD_ChildValue

# Register Dwg_FIELD_ChildValue in _LibreDWG:
_LibreDWG.Dwg_FIELD_ChildValue_swigregister(Dwg_FIELD_ChildValue)
class Dwg_Object_FIELD(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_FIELD_parent_get, _LibreDWG.Dwg_Object_FIELD_parent_set)
    id = property(_LibreDWG.Dwg_Object_FIELD_id_get, _LibreDWG.Dwg_Object_FIELD_id_set)
    code = property(_LibreDWG.Dwg_Object_FIELD_code_get, _LibreDWG.Dwg_Object_FIELD_code_set)
    num_childs = property(_LibreDWG.Dwg_Object_FIELD_num_childs_get, _LibreDWG.Dwg_Object_FIELD_num_childs_set)
    childs = property(_LibreDWG.Dwg_Object_FIELD_childs_get, _LibreDWG.Dwg_Object_FIELD_childs_set)
    num_objects = property(_LibreDWG.Dwg_Object_FIELD_num_objects_get, _LibreDWG.Dwg_Object_FIELD_num_objects_set)
    objects = property(_LibreDWG.Dwg_Object_FIELD_objects_get, _LibreDWG.Dwg_Object_FIELD_objects_set)
    format = property(_LibreDWG.Dwg_Object_FIELD_format_get, _LibreDWG.Dwg_Object_FIELD_format_set)
    evaluation_option = property(_LibreDWG.Dwg_Object_FIELD_evaluation_option_get, _LibreDWG.Dwg_Object_FIELD_evaluation_option_set)
    filing_option = property(_LibreDWG.Dwg_Object_FIELD_filing_option_get, _LibreDWG.Dwg_Object_FIELD_filing_option_set)
    field_state = property(_LibreDWG.Dwg_Object_FIELD_field_state_get, _LibreDWG.Dwg_Object_FIELD_field_state_set)
    evaluation_status = property(_LibreDWG.Dwg_Object_FIELD_evaluation_status_get, _LibreDWG.Dwg_Object_FIELD_evaluation_status_set)
    evaluation_error_code = property(_LibreDWG.Dwg_Object_FIELD_evaluation_error_code_get, _LibreDWG.Dwg_Object_FIELD_evaluation_error_code_set)
    evaluation_error_msg = property(_LibreDWG.Dwg_Object_FIELD_evaluation_error_msg_get, _LibreDWG.Dwg_Object_FIELD_evaluation_error_msg_set)
    value = property(_LibreDWG.Dwg_Object_FIELD_value_get, _LibreDWG.Dwg_Object_FIELD_value_set)
    value_string = property(_LibreDWG.Dwg_Object_FIELD_value_string_get, _LibreDWG.Dwg_Object_FIELD_value_string_set)
    value_string_length = property(_LibreDWG.Dwg_Object_FIELD_value_string_length_get, _LibreDWG.Dwg_Object_FIELD_value_string_length_set)
    num_childval = property(_LibreDWG.Dwg_Object_FIELD_num_childval_get, _LibreDWG.Dwg_Object_FIELD_num_childval_set)
    childval = property(_LibreDWG.Dwg_Object_FIELD_childval_get, _LibreDWG.Dwg_Object_FIELD_childval_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_FIELD_swiginit(self, _LibreDWG.new_Dwg_Object_FIELD())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_FIELD

# Register Dwg_Object_FIELD in _LibreDWG:
_LibreDWG.Dwg_Object_FIELD_swigregister(Dwg_Object_FIELD)
class Dwg_Object_FIELDLIST(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_FIELDLIST_parent_get, _LibreDWG.Dwg_Object_FIELDLIST_parent_set)
    num_fields = property(_LibreDWG.Dwg_Object_FIELDLIST_num_fields_get, _LibreDWG.Dwg_Object_FIELDLIST_num_fields_set)
    unknown = property(_LibreDWG.Dwg_Object_FIELDLIST_unknown_get, _LibreDWG.Dwg_Object_FIELDLIST_unknown_set)
    fields = property(_LibreDWG.Dwg_Object_FIELDLIST_fields_get, _LibreDWG.Dwg_Object_FIELDLIST_fields_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_FIELDLIST_swiginit(self, _LibreDWG.new_Dwg_Object_FIELDLIST())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_FIELDLIST

# Register Dwg_Object_FIELDLIST in _LibreDWG:
_LibreDWG.Dwg_Object_FIELDLIST_swigregister(Dwg_Object_FIELDLIST)
class Dwg_GEODATA_meshpt(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    source_pt = property(_LibreDWG.Dwg_GEODATA_meshpt_source_pt_get, _LibreDWG.Dwg_GEODATA_meshpt_source_pt_set)
    dest_pt = property(_LibreDWG.Dwg_GEODATA_meshpt_dest_pt_get, _LibreDWG.Dwg_GEODATA_meshpt_dest_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_GEODATA_meshpt_swiginit(self, _LibreDWG.new_Dwg_GEODATA_meshpt())
    __swig_destroy__ = _LibreDWG.delete_Dwg_GEODATA_meshpt

# Register Dwg_GEODATA_meshpt in _LibreDWG:
_LibreDWG.Dwg_GEODATA_meshpt_swigregister(Dwg_GEODATA_meshpt)
class Dwg_GEODATA_meshface(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    face1 = property(_LibreDWG.Dwg_GEODATA_meshface_face1_get, _LibreDWG.Dwg_GEODATA_meshface_face1_set)
    face2 = property(_LibreDWG.Dwg_GEODATA_meshface_face2_get, _LibreDWG.Dwg_GEODATA_meshface_face2_set)
    face3 = property(_LibreDWG.Dwg_GEODATA_meshface_face3_get, _LibreDWG.Dwg_GEODATA_meshface_face3_set)

    def __init__(self):
        _LibreDWG.Dwg_GEODATA_meshface_swiginit(self, _LibreDWG.new_Dwg_GEODATA_meshface())
    __swig_destroy__ = _LibreDWG.delete_Dwg_GEODATA_meshface

# Register Dwg_GEODATA_meshface in _LibreDWG:
_LibreDWG.Dwg_GEODATA_meshface_swigregister(Dwg_GEODATA_meshface)
class Dwg_Object_GEODATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_GEODATA_parent_get, _LibreDWG.Dwg_Object_GEODATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_GEODATA_class_version_get, _LibreDWG.Dwg_Object_GEODATA_class_version_set)
    host_block = property(_LibreDWG.Dwg_Object_GEODATA_host_block_get, _LibreDWG.Dwg_Object_GEODATA_host_block_set)
    coord_type = property(_LibreDWG.Dwg_Object_GEODATA_coord_type_get, _LibreDWG.Dwg_Object_GEODATA_coord_type_set)
    design_pt = property(_LibreDWG.Dwg_Object_GEODATA_design_pt_get, _LibreDWG.Dwg_Object_GEODATA_design_pt_set)
    ref_pt = property(_LibreDWG.Dwg_Object_GEODATA_ref_pt_get, _LibreDWG.Dwg_Object_GEODATA_ref_pt_set)
    obs_pt = property(_LibreDWG.Dwg_Object_GEODATA_obs_pt_get, _LibreDWG.Dwg_Object_GEODATA_obs_pt_set)
    scale_vec = property(_LibreDWG.Dwg_Object_GEODATA_scale_vec_get, _LibreDWG.Dwg_Object_GEODATA_scale_vec_set)
    unit_scale_horiz = property(_LibreDWG.Dwg_Object_GEODATA_unit_scale_horiz_get, _LibreDWG.Dwg_Object_GEODATA_unit_scale_horiz_set)
    units_value_horiz = property(_LibreDWG.Dwg_Object_GEODATA_units_value_horiz_get, _LibreDWG.Dwg_Object_GEODATA_units_value_horiz_set)
    unit_scale_vert = property(_LibreDWG.Dwg_Object_GEODATA_unit_scale_vert_get, _LibreDWG.Dwg_Object_GEODATA_unit_scale_vert_set)
    units_value_vert = property(_LibreDWG.Dwg_Object_GEODATA_units_value_vert_get, _LibreDWG.Dwg_Object_GEODATA_units_value_vert_set)
    up_dir = property(_LibreDWG.Dwg_Object_GEODATA_up_dir_get, _LibreDWG.Dwg_Object_GEODATA_up_dir_set)
    north_dir = property(_LibreDWG.Dwg_Object_GEODATA_north_dir_get, _LibreDWG.Dwg_Object_GEODATA_north_dir_set)
    scale_est = property(_LibreDWG.Dwg_Object_GEODATA_scale_est_get, _LibreDWG.Dwg_Object_GEODATA_scale_est_set)
    user_scale_factor = property(_LibreDWG.Dwg_Object_GEODATA_user_scale_factor_get, _LibreDWG.Dwg_Object_GEODATA_user_scale_factor_set)
    do_sea_level_corr = property(_LibreDWG.Dwg_Object_GEODATA_do_sea_level_corr_get, _LibreDWG.Dwg_Object_GEODATA_do_sea_level_corr_set)
    sea_level_elev = property(_LibreDWG.Dwg_Object_GEODATA_sea_level_elev_get, _LibreDWG.Dwg_Object_GEODATA_sea_level_elev_set)
    coord_proj_radius = property(_LibreDWG.Dwg_Object_GEODATA_coord_proj_radius_get, _LibreDWG.Dwg_Object_GEODATA_coord_proj_radius_set)
    coord_system_def = property(_LibreDWG.Dwg_Object_GEODATA_coord_system_def_get, _LibreDWG.Dwg_Object_GEODATA_coord_system_def_set)
    geo_rss_tag = property(_LibreDWG.Dwg_Object_GEODATA_geo_rss_tag_get, _LibreDWG.Dwg_Object_GEODATA_geo_rss_tag_set)
    coord_system_datum = property(_LibreDWG.Dwg_Object_GEODATA_coord_system_datum_get, _LibreDWG.Dwg_Object_GEODATA_coord_system_datum_set)
    coord_system_wkt = property(_LibreDWG.Dwg_Object_GEODATA_coord_system_wkt_get, _LibreDWG.Dwg_Object_GEODATA_coord_system_wkt_set)
    observation_from_tag = property(_LibreDWG.Dwg_Object_GEODATA_observation_from_tag_get, _LibreDWG.Dwg_Object_GEODATA_observation_from_tag_set)
    observation_to_tag = property(_LibreDWG.Dwg_Object_GEODATA_observation_to_tag_get, _LibreDWG.Dwg_Object_GEODATA_observation_to_tag_set)
    observation_coverage_tag = property(_LibreDWG.Dwg_Object_GEODATA_observation_coverage_tag_get, _LibreDWG.Dwg_Object_GEODATA_observation_coverage_tag_set)
    num_geomesh_pts = property(_LibreDWG.Dwg_Object_GEODATA_num_geomesh_pts_get, _LibreDWG.Dwg_Object_GEODATA_num_geomesh_pts_set)
    geomesh_pts = property(_LibreDWG.Dwg_Object_GEODATA_geomesh_pts_get, _LibreDWG.Dwg_Object_GEODATA_geomesh_pts_set)
    num_geomesh_faces = property(_LibreDWG.Dwg_Object_GEODATA_num_geomesh_faces_get, _LibreDWG.Dwg_Object_GEODATA_num_geomesh_faces_set)
    geomesh_faces = property(_LibreDWG.Dwg_Object_GEODATA_geomesh_faces_get, _LibreDWG.Dwg_Object_GEODATA_geomesh_faces_set)
    has_civil_data = property(_LibreDWG.Dwg_Object_GEODATA_has_civil_data_get, _LibreDWG.Dwg_Object_GEODATA_has_civil_data_set)
    obsolete_false = property(_LibreDWG.Dwg_Object_GEODATA_obsolete_false_get, _LibreDWG.Dwg_Object_GEODATA_obsolete_false_set)
    ref_pt2d = property(_LibreDWG.Dwg_Object_GEODATA_ref_pt2d_get, _LibreDWG.Dwg_Object_GEODATA_ref_pt2d_set)
    zero1 = property(_LibreDWG.Dwg_Object_GEODATA_zero1_get, _LibreDWG.Dwg_Object_GEODATA_zero1_set)
    zero2 = property(_LibreDWG.Dwg_Object_GEODATA_zero2_get, _LibreDWG.Dwg_Object_GEODATA_zero2_set)
    unknown1 = property(_LibreDWG.Dwg_Object_GEODATA_unknown1_get, _LibreDWG.Dwg_Object_GEODATA_unknown1_set)
    unknown2 = property(_LibreDWG.Dwg_Object_GEODATA_unknown2_get, _LibreDWG.Dwg_Object_GEODATA_unknown2_set)
    unknown_b = property(_LibreDWG.Dwg_Object_GEODATA_unknown_b_get, _LibreDWG.Dwg_Object_GEODATA_unknown_b_set)
    north_dir_angle_deg = property(_LibreDWG.Dwg_Object_GEODATA_north_dir_angle_deg_get, _LibreDWG.Dwg_Object_GEODATA_north_dir_angle_deg_set)
    north_dir_angle_rad = property(_LibreDWG.Dwg_Object_GEODATA_north_dir_angle_rad_get, _LibreDWG.Dwg_Object_GEODATA_north_dir_angle_rad_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_GEODATA_swiginit(self, _LibreDWG.new_Dwg_Object_GEODATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_GEODATA

# Register Dwg_Object_GEODATA in _LibreDWG:
_LibreDWG.Dwg_Object_GEODATA_swigregister(Dwg_Object_GEODATA)
class Dwg_Object_IDBUFFER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_IDBUFFER_parent_get, _LibreDWG.Dwg_Object_IDBUFFER_parent_set)
    unknown = property(_LibreDWG.Dwg_Object_IDBUFFER_unknown_get, _LibreDWG.Dwg_Object_IDBUFFER_unknown_set)
    num_obj_ids = property(_LibreDWG.Dwg_Object_IDBUFFER_num_obj_ids_get, _LibreDWG.Dwg_Object_IDBUFFER_num_obj_ids_set)
    obj_ids = property(_LibreDWG.Dwg_Object_IDBUFFER_obj_ids_get, _LibreDWG.Dwg_Object_IDBUFFER_obj_ids_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_IDBUFFER_swiginit(self, _LibreDWG.new_Dwg_Object_IDBUFFER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_IDBUFFER

# Register Dwg_Object_IDBUFFER in _LibreDWG:
_LibreDWG.Dwg_Object_IDBUFFER_swigregister(Dwg_Object_IDBUFFER)
class Dwg_Entity_IMAGE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_IMAGE_parent_get, _LibreDWG.Dwg_Entity_IMAGE_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_IMAGE_class_version_get, _LibreDWG.Dwg_Entity_IMAGE_class_version_set)
    pt0 = property(_LibreDWG.Dwg_Entity_IMAGE_pt0_get, _LibreDWG.Dwg_Entity_IMAGE_pt0_set)
    uvec = property(_LibreDWG.Dwg_Entity_IMAGE_uvec_get, _LibreDWG.Dwg_Entity_IMAGE_uvec_set)
    vvec = property(_LibreDWG.Dwg_Entity_IMAGE_vvec_get, _LibreDWG.Dwg_Entity_IMAGE_vvec_set)
    image_size = property(_LibreDWG.Dwg_Entity_IMAGE_image_size_get, _LibreDWG.Dwg_Entity_IMAGE_image_size_set)
    display_props = property(_LibreDWG.Dwg_Entity_IMAGE_display_props_get, _LibreDWG.Dwg_Entity_IMAGE_display_props_set)
    clipping = property(_LibreDWG.Dwg_Entity_IMAGE_clipping_get, _LibreDWG.Dwg_Entity_IMAGE_clipping_set)
    brightness = property(_LibreDWG.Dwg_Entity_IMAGE_brightness_get, _LibreDWG.Dwg_Entity_IMAGE_brightness_set)
    contrast = property(_LibreDWG.Dwg_Entity_IMAGE_contrast_get, _LibreDWG.Dwg_Entity_IMAGE_contrast_set)
    fade = property(_LibreDWG.Dwg_Entity_IMAGE_fade_get, _LibreDWG.Dwg_Entity_IMAGE_fade_set)
    clip_mode = property(_LibreDWG.Dwg_Entity_IMAGE_clip_mode_get, _LibreDWG.Dwg_Entity_IMAGE_clip_mode_set)
    clip_boundary_type = property(_LibreDWG.Dwg_Entity_IMAGE_clip_boundary_type_get, _LibreDWG.Dwg_Entity_IMAGE_clip_boundary_type_set)
    num_clip_verts = property(_LibreDWG.Dwg_Entity_IMAGE_num_clip_verts_get, _LibreDWG.Dwg_Entity_IMAGE_num_clip_verts_set)
    clip_verts = property(_LibreDWG.Dwg_Entity_IMAGE_clip_verts_get, _LibreDWG.Dwg_Entity_IMAGE_clip_verts_set)
    imagedef = property(_LibreDWG.Dwg_Entity_IMAGE_imagedef_get, _LibreDWG.Dwg_Entity_IMAGE_imagedef_set)
    imagedefreactor = property(_LibreDWG.Dwg_Entity_IMAGE_imagedefreactor_get, _LibreDWG.Dwg_Entity_IMAGE_imagedefreactor_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_IMAGE_swiginit(self, _LibreDWG.new_Dwg_Entity_IMAGE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_IMAGE

# Register Dwg_Entity_IMAGE in _LibreDWG:
_LibreDWG.Dwg_Entity_IMAGE_swigregister(Dwg_Entity_IMAGE)
class Dwg_Object_IMAGEDEF(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_IMAGEDEF_parent_get, _LibreDWG.Dwg_Object_IMAGEDEF_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_IMAGEDEF_class_version_get, _LibreDWG.Dwg_Object_IMAGEDEF_class_version_set)
    image_size = property(_LibreDWG.Dwg_Object_IMAGEDEF_image_size_get, _LibreDWG.Dwg_Object_IMAGEDEF_image_size_set)
    file_path = property(_LibreDWG.Dwg_Object_IMAGEDEF_file_path_get, _LibreDWG.Dwg_Object_IMAGEDEF_file_path_set)
    is_loaded = property(_LibreDWG.Dwg_Object_IMAGEDEF_is_loaded_get, _LibreDWG.Dwg_Object_IMAGEDEF_is_loaded_set)
    resunits = property(_LibreDWG.Dwg_Object_IMAGEDEF_resunits_get, _LibreDWG.Dwg_Object_IMAGEDEF_resunits_set)
    pixel_size = property(_LibreDWG.Dwg_Object_IMAGEDEF_pixel_size_get, _LibreDWG.Dwg_Object_IMAGEDEF_pixel_size_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_IMAGEDEF_swiginit(self, _LibreDWG.new_Dwg_Object_IMAGEDEF())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_IMAGEDEF

# Register Dwg_Object_IMAGEDEF in _LibreDWG:
_LibreDWG.Dwg_Object_IMAGEDEF_swigregister(Dwg_Object_IMAGEDEF)
class Dwg_Object_IMAGEDEF_REACTOR(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_IMAGEDEF_REACTOR_parent_get, _LibreDWG.Dwg_Object_IMAGEDEF_REACTOR_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_IMAGEDEF_REACTOR_class_version_get, _LibreDWG.Dwg_Object_IMAGEDEF_REACTOR_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_IMAGEDEF_REACTOR_swiginit(self, _LibreDWG.new_Dwg_Object_IMAGEDEF_REACTOR())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_IMAGEDEF_REACTOR

# Register Dwg_Object_IMAGEDEF_REACTOR in _LibreDWG:
_LibreDWG.Dwg_Object_IMAGEDEF_REACTOR_swigregister(Dwg_Object_IMAGEDEF_REACTOR)
class Dwg_Object_INDEX(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_INDEX_parent_get, _LibreDWG.Dwg_Object_INDEX_parent_set)
    last_updated = property(_LibreDWG.Dwg_Object_INDEX_last_updated_get, _LibreDWG.Dwg_Object_INDEX_last_updated_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_INDEX_swiginit(self, _LibreDWG.new_Dwg_Object_INDEX())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_INDEX

# Register Dwg_Object_INDEX in _LibreDWG:
_LibreDWG.Dwg_Object_INDEX_swigregister(Dwg_Object_INDEX)
class Dwg_LAYER_entry(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_LAYER_entry_parent_get, _LibreDWG.Dwg_LAYER_entry_parent_set)
    numlayers = property(_LibreDWG.Dwg_LAYER_entry_numlayers_get, _LibreDWG.Dwg_LAYER_entry_numlayers_set)
    name = property(_LibreDWG.Dwg_LAYER_entry_name_get, _LibreDWG.Dwg_LAYER_entry_name_set)
    handle = property(_LibreDWG.Dwg_LAYER_entry_handle_get, _LibreDWG.Dwg_LAYER_entry_handle_set)

    def __init__(self):
        _LibreDWG.Dwg_LAYER_entry_swiginit(self, _LibreDWG.new_Dwg_LAYER_entry())
    __swig_destroy__ = _LibreDWG.delete_Dwg_LAYER_entry

# Register Dwg_LAYER_entry in _LibreDWG:
_LibreDWG.Dwg_LAYER_entry_swigregister(Dwg_LAYER_entry)
class Dwg_Object_LAYER_INDEX(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_LAYER_INDEX_parent_get, _LibreDWG.Dwg_Object_LAYER_INDEX_parent_set)
    last_updated = property(_LibreDWG.Dwg_Object_LAYER_INDEX_last_updated_get, _LibreDWG.Dwg_Object_LAYER_INDEX_last_updated_set)
    num_entries = property(_LibreDWG.Dwg_Object_LAYER_INDEX_num_entries_get, _LibreDWG.Dwg_Object_LAYER_INDEX_num_entries_set)
    entries = property(_LibreDWG.Dwg_Object_LAYER_INDEX_entries_get, _LibreDWG.Dwg_Object_LAYER_INDEX_entries_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_LAYER_INDEX_swiginit(self, _LibreDWG.new_Dwg_Object_LAYER_INDEX())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_LAYER_INDEX

# Register Dwg_Object_LAYER_INDEX in _LibreDWG:
_LibreDWG.Dwg_Object_LAYER_INDEX_swigregister(Dwg_Object_LAYER_INDEX)
class Dwg_Entity_LWPOLYLINE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_parent_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_parent_set)
    flag = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_flag_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_flag_set)
    const_width = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_const_width_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_const_width_set)
    elevation = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_elevation_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_elevation_set)
    thickness = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_thickness_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_thickness_set)
    extrusion = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_extrusion_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_extrusion_set)
    num_points = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_num_points_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_num_points_set)
    points = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_points_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_points_set)
    num_bulges = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_num_bulges_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_num_bulges_set)
    bulges = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_bulges_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_bulges_set)
    num_vertexids = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_num_vertexids_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_num_vertexids_set)
    vertexids = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_vertexids_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_vertexids_set)
    num_widths = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_num_widths_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_num_widths_set)
    widths = property(_LibreDWG.Dwg_Entity_LWPOLYLINE_widths_get, _LibreDWG.Dwg_Entity_LWPOLYLINE_widths_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_LWPOLYLINE_swiginit(self, _LibreDWG.new_Dwg_Entity_LWPOLYLINE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_LWPOLYLINE

# Register Dwg_Entity_LWPOLYLINE in _LibreDWG:
_LibreDWG.Dwg_Entity_LWPOLYLINE_swigregister(Dwg_Entity_LWPOLYLINE)
class Dwg_Object_RASTERVARIABLES(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_RASTERVARIABLES_parent_get, _LibreDWG.Dwg_Object_RASTERVARIABLES_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_RASTERVARIABLES_class_version_get, _LibreDWG.Dwg_Object_RASTERVARIABLES_class_version_set)
    image_frame = property(_LibreDWG.Dwg_Object_RASTERVARIABLES_image_frame_get, _LibreDWG.Dwg_Object_RASTERVARIABLES_image_frame_set)
    image_quality = property(_LibreDWG.Dwg_Object_RASTERVARIABLES_image_quality_get, _LibreDWG.Dwg_Object_RASTERVARIABLES_image_quality_set)
    units = property(_LibreDWG.Dwg_Object_RASTERVARIABLES_units_get, _LibreDWG.Dwg_Object_RASTERVARIABLES_units_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_RASTERVARIABLES_swiginit(self, _LibreDWG.new_Dwg_Object_RASTERVARIABLES())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_RASTERVARIABLES

# Register Dwg_Object_RASTERVARIABLES in _LibreDWG:
_LibreDWG.Dwg_Object_RASTERVARIABLES_swigregister(Dwg_Object_RASTERVARIABLES)
class Dwg_Object_SCALE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_SCALE_parent_get, _LibreDWG.Dwg_Object_SCALE_parent_set)
    flag = property(_LibreDWG.Dwg_Object_SCALE_flag_get, _LibreDWG.Dwg_Object_SCALE_flag_set)
    name = property(_LibreDWG.Dwg_Object_SCALE_name_get, _LibreDWG.Dwg_Object_SCALE_name_set)
    paper_units = property(_LibreDWG.Dwg_Object_SCALE_paper_units_get, _LibreDWG.Dwg_Object_SCALE_paper_units_set)
    drawing_units = property(_LibreDWG.Dwg_Object_SCALE_drawing_units_get, _LibreDWG.Dwg_Object_SCALE_drawing_units_set)
    is_unit_scale = property(_LibreDWG.Dwg_Object_SCALE_is_unit_scale_get, _LibreDWG.Dwg_Object_SCALE_is_unit_scale_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_SCALE_swiginit(self, _LibreDWG.new_Dwg_Object_SCALE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_SCALE

# Register Dwg_Object_SCALE in _LibreDWG:
_LibreDWG.Dwg_Object_SCALE_swigregister(Dwg_Object_SCALE)
class Dwg_Object_SORTENTSTABLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_SORTENTSTABLE_parent_get, _LibreDWG.Dwg_Object_SORTENTSTABLE_parent_set)
    num_ents = property(_LibreDWG.Dwg_Object_SORTENTSTABLE_num_ents_get, _LibreDWG.Dwg_Object_SORTENTSTABLE_num_ents_set)
    sort_ents = property(_LibreDWG.Dwg_Object_SORTENTSTABLE_sort_ents_get, _LibreDWG.Dwg_Object_SORTENTSTABLE_sort_ents_set)
    block_owner = property(_LibreDWG.Dwg_Object_SORTENTSTABLE_block_owner_get, _LibreDWG.Dwg_Object_SORTENTSTABLE_block_owner_set)
    ents = property(_LibreDWG.Dwg_Object_SORTENTSTABLE_ents_get, _LibreDWG.Dwg_Object_SORTENTSTABLE_ents_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_SORTENTSTABLE_swiginit(self, _LibreDWG.new_Dwg_Object_SORTENTSTABLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_SORTENTSTABLE

# Register Dwg_Object_SORTENTSTABLE in _LibreDWG:
_LibreDWG.Dwg_Object_SORTENTSTABLE_swigregister(Dwg_Object_SORTENTSTABLE)
class Dwg_Object_SPATIAL_FILTER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_SPATIAL_FILTER_parent_get, _LibreDWG.Dwg_Object_SPATIAL_FILTER_parent_set)
    num_clip_verts = property(_LibreDWG.Dwg_Object_SPATIAL_FILTER_num_clip_verts_get, _LibreDWG.Dwg_Object_SPATIAL_FILTER_num_clip_verts_set)
    clip_verts = property(_LibreDWG.Dwg_Object_SPATIAL_FILTER_clip_verts_get, _LibreDWG.Dwg_Object_SPATIAL_FILTER_clip_verts_set)
    extrusion = property(_LibreDWG.Dwg_Object_SPATIAL_FILTER_extrusion_get, _LibreDWG.Dwg_Object_SPATIAL_FILTER_extrusion_set)
    origin = property(_LibreDWG.Dwg_Object_SPATIAL_FILTER_origin_get, _LibreDWG.Dwg_Object_SPATIAL_FILTER_origin_set)
    display_boundary_on = property(_LibreDWG.Dwg_Object_SPATIAL_FILTER_display_boundary_on_get, _LibreDWG.Dwg_Object_SPATIAL_FILTER_display_boundary_on_set)
    front_clip_on = property(_LibreDWG.Dwg_Object_SPATIAL_FILTER_front_clip_on_get, _LibreDWG.Dwg_Object_SPATIAL_FILTER_front_clip_on_set)
    front_clip_z = property(_LibreDWG.Dwg_Object_SPATIAL_FILTER_front_clip_z_get, _LibreDWG.Dwg_Object_SPATIAL_FILTER_front_clip_z_set)
    back_clip_on = property(_LibreDWG.Dwg_Object_SPATIAL_FILTER_back_clip_on_get, _LibreDWG.Dwg_Object_SPATIAL_FILTER_back_clip_on_set)
    back_clip_z = property(_LibreDWG.Dwg_Object_SPATIAL_FILTER_back_clip_z_get, _LibreDWG.Dwg_Object_SPATIAL_FILTER_back_clip_z_set)
    inverse_transform = property(_LibreDWG.Dwg_Object_SPATIAL_FILTER_inverse_transform_get, _LibreDWG.Dwg_Object_SPATIAL_FILTER_inverse_transform_set)
    transform = property(_LibreDWG.Dwg_Object_SPATIAL_FILTER_transform_get, _LibreDWG.Dwg_Object_SPATIAL_FILTER_transform_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_SPATIAL_FILTER_swiginit(self, _LibreDWG.new_Dwg_Object_SPATIAL_FILTER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_SPATIAL_FILTER

# Register Dwg_Object_SPATIAL_FILTER in _LibreDWG:
_LibreDWG.Dwg_Object_SPATIAL_FILTER_swigregister(Dwg_Object_SPATIAL_FILTER)
class Dwg_Object_SPATIAL_INDEX(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_SPATIAL_INDEX_parent_get, _LibreDWG.Dwg_Object_SPATIAL_INDEX_parent_set)
    last_updated = property(_LibreDWG.Dwg_Object_SPATIAL_INDEX_last_updated_get, _LibreDWG.Dwg_Object_SPATIAL_INDEX_last_updated_set)
    num1 = property(_LibreDWG.Dwg_Object_SPATIAL_INDEX_num1_get, _LibreDWG.Dwg_Object_SPATIAL_INDEX_num1_set)
    num2 = property(_LibreDWG.Dwg_Object_SPATIAL_INDEX_num2_get, _LibreDWG.Dwg_Object_SPATIAL_INDEX_num2_set)
    num3 = property(_LibreDWG.Dwg_Object_SPATIAL_INDEX_num3_get, _LibreDWG.Dwg_Object_SPATIAL_INDEX_num3_set)
    num4 = property(_LibreDWG.Dwg_Object_SPATIAL_INDEX_num4_get, _LibreDWG.Dwg_Object_SPATIAL_INDEX_num4_set)
    num5 = property(_LibreDWG.Dwg_Object_SPATIAL_INDEX_num5_get, _LibreDWG.Dwg_Object_SPATIAL_INDEX_num5_set)
    num6 = property(_LibreDWG.Dwg_Object_SPATIAL_INDEX_num6_get, _LibreDWG.Dwg_Object_SPATIAL_INDEX_num6_set)
    num_hdls = property(_LibreDWG.Dwg_Object_SPATIAL_INDEX_num_hdls_get, _LibreDWG.Dwg_Object_SPATIAL_INDEX_num_hdls_set)
    hdls = property(_LibreDWG.Dwg_Object_SPATIAL_INDEX_hdls_get, _LibreDWG.Dwg_Object_SPATIAL_INDEX_hdls_set)
    bindata_size = property(_LibreDWG.Dwg_Object_SPATIAL_INDEX_bindata_size_get, _LibreDWG.Dwg_Object_SPATIAL_INDEX_bindata_size_set)
    bindata = property(_LibreDWG.Dwg_Object_SPATIAL_INDEX_bindata_get, _LibreDWG.Dwg_Object_SPATIAL_INDEX_bindata_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_SPATIAL_INDEX_swiginit(self, _LibreDWG.new_Dwg_Object_SPATIAL_INDEX())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_SPATIAL_INDEX

# Register Dwg_Object_SPATIAL_INDEX in _LibreDWG:
_LibreDWG.Dwg_Object_SPATIAL_INDEX_swigregister(Dwg_Object_SPATIAL_INDEX)
class Dwg_Entity_WIPEOUT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_WIPEOUT_parent_get, _LibreDWG.Dwg_Entity_WIPEOUT_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_WIPEOUT_class_version_get, _LibreDWG.Dwg_Entity_WIPEOUT_class_version_set)
    pt0 = property(_LibreDWG.Dwg_Entity_WIPEOUT_pt0_get, _LibreDWG.Dwg_Entity_WIPEOUT_pt0_set)
    uvec = property(_LibreDWG.Dwg_Entity_WIPEOUT_uvec_get, _LibreDWG.Dwg_Entity_WIPEOUT_uvec_set)
    vvec = property(_LibreDWG.Dwg_Entity_WIPEOUT_vvec_get, _LibreDWG.Dwg_Entity_WIPEOUT_vvec_set)
    image_size = property(_LibreDWG.Dwg_Entity_WIPEOUT_image_size_get, _LibreDWG.Dwg_Entity_WIPEOUT_image_size_set)
    display_props = property(_LibreDWG.Dwg_Entity_WIPEOUT_display_props_get, _LibreDWG.Dwg_Entity_WIPEOUT_display_props_set)
    clipping = property(_LibreDWG.Dwg_Entity_WIPEOUT_clipping_get, _LibreDWG.Dwg_Entity_WIPEOUT_clipping_set)
    brightness = property(_LibreDWG.Dwg_Entity_WIPEOUT_brightness_get, _LibreDWG.Dwg_Entity_WIPEOUT_brightness_set)
    contrast = property(_LibreDWG.Dwg_Entity_WIPEOUT_contrast_get, _LibreDWG.Dwg_Entity_WIPEOUT_contrast_set)
    fade = property(_LibreDWG.Dwg_Entity_WIPEOUT_fade_get, _LibreDWG.Dwg_Entity_WIPEOUT_fade_set)
    clip_mode = property(_LibreDWG.Dwg_Entity_WIPEOUT_clip_mode_get, _LibreDWG.Dwg_Entity_WIPEOUT_clip_mode_set)
    clip_boundary_type = property(_LibreDWG.Dwg_Entity_WIPEOUT_clip_boundary_type_get, _LibreDWG.Dwg_Entity_WIPEOUT_clip_boundary_type_set)
    num_clip_verts = property(_LibreDWG.Dwg_Entity_WIPEOUT_num_clip_verts_get, _LibreDWG.Dwg_Entity_WIPEOUT_num_clip_verts_set)
    clip_verts = property(_LibreDWG.Dwg_Entity_WIPEOUT_clip_verts_get, _LibreDWG.Dwg_Entity_WIPEOUT_clip_verts_set)
    imagedef = property(_LibreDWG.Dwg_Entity_WIPEOUT_imagedef_get, _LibreDWG.Dwg_Entity_WIPEOUT_imagedef_set)
    imagedefreactor = property(_LibreDWG.Dwg_Entity_WIPEOUT_imagedefreactor_get, _LibreDWG.Dwg_Entity_WIPEOUT_imagedefreactor_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_WIPEOUT_swiginit(self, _LibreDWG.new_Dwg_Entity_WIPEOUT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_WIPEOUT

# Register Dwg_Entity_WIPEOUT in _LibreDWG:
_LibreDWG.Dwg_Entity_WIPEOUT_swigregister(Dwg_Entity_WIPEOUT)
class Dwg_Object_WIPEOUTVARIABLES(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_WIPEOUTVARIABLES_parent_get, _LibreDWG.Dwg_Object_WIPEOUTVARIABLES_parent_set)
    display_frame = property(_LibreDWG.Dwg_Object_WIPEOUTVARIABLES_display_frame_get, _LibreDWG.Dwg_Object_WIPEOUTVARIABLES_display_frame_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_WIPEOUTVARIABLES_swiginit(self, _LibreDWG.new_Dwg_Object_WIPEOUTVARIABLES())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_WIPEOUTVARIABLES

# Register Dwg_Object_WIPEOUTVARIABLES in _LibreDWG:
_LibreDWG.Dwg_Object_WIPEOUTVARIABLES_swigregister(Dwg_Object_WIPEOUTVARIABLES)
class Dwg_Entity_SECTIONOBJECT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_parent_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_parent_set)
    state = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_state_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_state_set)
    flags = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_flags_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_flags_set)
    name = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_name_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_name_set)
    vert_dir = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_vert_dir_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_vert_dir_set)
    top_height = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_top_height_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_top_height_set)
    bottom_height = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_bottom_height_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_bottom_height_set)
    indicator_alpha = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_indicator_alpha_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_indicator_alpha_set)
    indicator_color = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_indicator_color_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_indicator_color_set)
    num_verts = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_num_verts_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_num_verts_set)
    verts = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_verts_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_verts_set)
    num_blverts = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_num_blverts_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_num_blverts_set)
    blverts = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_blverts_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_blverts_set)
    section_settings = property(_LibreDWG.Dwg_Entity_SECTIONOBJECT_section_settings_get, _LibreDWG.Dwg_Entity_SECTIONOBJECT_section_settings_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_SECTIONOBJECT_swiginit(self, _LibreDWG.new_Dwg_Entity_SECTIONOBJECT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_SECTIONOBJECT

# Register Dwg_Entity_SECTIONOBJECT in _LibreDWG:
_LibreDWG.Dwg_Entity_SECTIONOBJECT_swigregister(Dwg_Entity_SECTIONOBJECT)
class Dwg_Object_VISUALSTYLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_VISUALSTYLE_parent_get, _LibreDWG.Dwg_Object_VISUALSTYLE_parent_set)
    description = property(_LibreDWG.Dwg_Object_VISUALSTYLE_description_get, _LibreDWG.Dwg_Object_VISUALSTYLE_description_set)
    style_type = property(_LibreDWG.Dwg_Object_VISUALSTYLE_style_type_get, _LibreDWG.Dwg_Object_VISUALSTYLE_style_type_set)
    ext_lighting_model = property(_LibreDWG.Dwg_Object_VISUALSTYLE_ext_lighting_model_get, _LibreDWG.Dwg_Object_VISUALSTYLE_ext_lighting_model_set)
    internal_only = property(_LibreDWG.Dwg_Object_VISUALSTYLE_internal_only_get, _LibreDWG.Dwg_Object_VISUALSTYLE_internal_only_set)
    face_lighting_model = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_lighting_model_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_lighting_model_set)
    face_lighting_model_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_lighting_model_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_lighting_model_int_set)
    face_lighting_quality = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_lighting_quality_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_lighting_quality_set)
    face_lighting_quality_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_lighting_quality_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_lighting_quality_int_set)
    face_color_mode = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_color_mode_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_color_mode_set)
    face_color_mode_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_color_mode_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_color_mode_int_set)
    face_opacity = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_opacity_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_opacity_set)
    face_opacity_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_opacity_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_opacity_int_set)
    face_specular = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_specular_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_specular_set)
    face_specular_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_specular_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_specular_int_set)
    face_modifier = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_modifier_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_modifier_set)
    face_modifier_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_modifier_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_modifier_int_set)
    face_mono_color = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_mono_color_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_mono_color_set)
    face_mono_color_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_face_mono_color_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_face_mono_color_int_set)
    edge_model = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_model_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_model_set)
    edge_model_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_model_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_model_int_set)
    edge_style = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_style_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_style_set)
    edge_style_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_style_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_style_int_set)
    edge_intersection_color = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_intersection_color_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_intersection_color_set)
    edge_intersection_color_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_intersection_color_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_intersection_color_int_set)
    edge_obscured_color = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_obscured_color_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_obscured_color_set)
    edge_obscured_color_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_obscured_color_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_obscured_color_int_set)
    edge_obscured_ltype = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_obscured_ltype_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_obscured_ltype_set)
    edge_obscured_ltype_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_set)
    edge_intersection_ltype = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_intersection_ltype_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_intersection_ltype_set)
    edge_intersection_ltype_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_set)
    edge_crease_angle = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_crease_angle_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_crease_angle_set)
    edge_crease_angle_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_crease_angle_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_crease_angle_int_set)
    edge_modifier = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_modifier_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_modifier_set)
    edge_modifier_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_modifier_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_modifier_int_set)
    edge_color = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_color_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_color_set)
    edge_color_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_color_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_color_int_set)
    edge_opacity = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_opacity_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_opacity_set)
    edge_opacity_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_opacity_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_opacity_int_set)
    edge_width = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_width_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_width_set)
    edge_width_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_width_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_width_int_set)
    edge_overhang = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_overhang_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_overhang_set)
    edge_overhang_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_overhang_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_overhang_int_set)
    edge_jitter = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_jitter_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_jitter_set)
    edge_jitter_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_jitter_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_jitter_int_set)
    edge_silhouette_color = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_silhouette_color_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_silhouette_color_set)
    edge_silhouette_color_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_set)
    edge_silhouette_width = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_silhouette_width_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_silhouette_width_set)
    edge_silhouette_width_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_set)
    edge_halo_gap = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_halo_gap_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_halo_gap_set)
    edge_halo_gap_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_halo_gap_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_halo_gap_int_set)
    edge_isolines = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_isolines_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_isolines_set)
    edge_isolines_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_isolines_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_isolines_int_set)
    edge_do_hide_precision = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_do_hide_precision_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_do_hide_precision_set)
    edge_do_hide_precision_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_set)
    edge_style_apply = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_style_apply_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_style_apply_set)
    edge_style_apply_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_style_apply_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_style_apply_int_set)
    display_settings = property(_LibreDWG.Dwg_Object_VISUALSTYLE_display_settings_get, _LibreDWG.Dwg_Object_VISUALSTYLE_display_settings_set)
    display_settings_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_display_settings_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_display_settings_int_set)
    display_brightness_bl = property(_LibreDWG.Dwg_Object_VISUALSTYLE_display_brightness_bl_get, _LibreDWG.Dwg_Object_VISUALSTYLE_display_brightness_bl_set)
    display_brightness = property(_LibreDWG.Dwg_Object_VISUALSTYLE_display_brightness_get, _LibreDWG.Dwg_Object_VISUALSTYLE_display_brightness_set)
    display_brightness_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_display_brightness_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_display_brightness_int_set)
    display_shadow_type = property(_LibreDWG.Dwg_Object_VISUALSTYLE_display_shadow_type_get, _LibreDWG.Dwg_Object_VISUALSTYLE_display_shadow_type_set)
    display_shadow_type_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_display_shadow_type_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_display_shadow_type_int_set)
    bd2007_45 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bd2007_45_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bd2007_45_set)
    num_props = property(_LibreDWG.Dwg_Object_VISUALSTYLE_num_props_get, _LibreDWG.Dwg_Object_VISUALSTYLE_num_props_set)
    b_prop1c = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1c_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1c_set)
    b_prop1c_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1c_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1c_int_set)
    b_prop1d = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1d_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1d_set)
    b_prop1d_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1d_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1d_int_set)
    b_prop1e = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1e_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1e_set)
    b_prop1e_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1e_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1e_int_set)
    b_prop1f = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1f_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1f_set)
    b_prop1f_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1f_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop1f_int_set)
    b_prop20 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop20_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop20_set)
    b_prop20_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop20_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop20_int_set)
    b_prop21 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop21_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop21_set)
    b_prop21_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop21_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop21_int_set)
    b_prop22 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop22_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop22_set)
    b_prop22_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop22_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop22_int_set)
    b_prop23 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop23_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop23_set)
    b_prop23_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop23_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop23_int_set)
    b_prop24 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop24_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop24_set)
    b_prop24_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop24_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop24_int_set)
    bl_prop25 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop25_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop25_set)
    bl_prop25_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop25_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop25_int_set)
    bd_prop26 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop26_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop26_set)
    bd_prop26_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop26_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop26_int_set)
    bd_prop27 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop27_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop27_set)
    bd_prop27_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop27_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop27_int_set)
    bl_prop28 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop28_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop28_set)
    bl_prop28_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop28_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop28_int_set)
    c_prop29 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_c_prop29_get, _LibreDWG.Dwg_Object_VISUALSTYLE_c_prop29_set)
    c_prop29_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_c_prop29_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_c_prop29_int_set)
    bl_prop2a = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2a_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2a_set)
    bl_prop2a_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2a_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2a_int_set)
    bl_prop2b = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2b_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2b_set)
    bl_prop2b_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2b_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2b_int_set)
    c_prop2c = property(_LibreDWG.Dwg_Object_VISUALSTYLE_c_prop2c_get, _LibreDWG.Dwg_Object_VISUALSTYLE_c_prop2c_set)
    c_prop2c_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_c_prop2c_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_c_prop2c_int_set)
    b_prop2d = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop2d_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop2d_set)
    b_prop2d_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop2d_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop2d_int_set)
    bl_prop2e = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2e_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2e_set)
    bl_prop2e_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2e_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2e_int_set)
    bl_prop2f = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2f_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2f_set)
    bl_prop2f_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2f_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop2f_int_set)
    bl_prop30 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop30_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop30_set)
    bl_prop30_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop30_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop30_int_set)
    b_prop31 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop31_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop31_set)
    b_prop31_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop31_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop31_int_set)
    bl_prop32 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop32_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop32_set)
    bl_prop32_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop32_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bl_prop32_int_set)
    c_prop33 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_c_prop33_get, _LibreDWG.Dwg_Object_VISUALSTYLE_c_prop33_set)
    c_prop33_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_c_prop33_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_c_prop33_int_set)
    bd_prop34 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop34_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop34_set)
    bd_prop34_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop34_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop34_int_set)
    edge_wiggle = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_wiggle_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_wiggle_set)
    edge_wiggle_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_edge_wiggle_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_edge_wiggle_int_set)
    strokes = property(_LibreDWG.Dwg_Object_VISUALSTYLE_strokes_get, _LibreDWG.Dwg_Object_VISUALSTYLE_strokes_set)
    strokes_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_strokes_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_strokes_int_set)
    b_prop37 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop37_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop37_set)
    b_prop37_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_b_prop37_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_b_prop37_int_set)
    bd_prop38 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop38_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop38_set)
    bd_prop38_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop38_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop38_int_set)
    bd_prop39 = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop39_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop39_set)
    bd_prop39_int = property(_LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop39_int_get, _LibreDWG.Dwg_Object_VISUALSTYLE_bd_prop39_int_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_VISUALSTYLE_swiginit(self, _LibreDWG.new_Dwg_Object_VISUALSTYLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_VISUALSTYLE

# Register Dwg_Object_VISUALSTYLE in _LibreDWG:
_LibreDWG.Dwg_Object_VISUALSTYLE_swigregister(Dwg_Object_VISUALSTYLE)
class Dwg_LIGHTLIST_light(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_LIGHTLIST_light_parent_get, _LibreDWG.Dwg_LIGHTLIST_light_parent_set)
    name = property(_LibreDWG.Dwg_LIGHTLIST_light_name_get, _LibreDWG.Dwg_LIGHTLIST_light_name_set)
    handle = property(_LibreDWG.Dwg_LIGHTLIST_light_handle_get, _LibreDWG.Dwg_LIGHTLIST_light_handle_set)

    def __init__(self):
        _LibreDWG.Dwg_LIGHTLIST_light_swiginit(self, _LibreDWG.new_Dwg_LIGHTLIST_light())
    __swig_destroy__ = _LibreDWG.delete_Dwg_LIGHTLIST_light

# Register Dwg_LIGHTLIST_light in _LibreDWG:
_LibreDWG.Dwg_LIGHTLIST_light_swigregister(Dwg_LIGHTLIST_light)
class Dwg_Object_LIGHTLIST(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_LIGHTLIST_parent_get, _LibreDWG.Dwg_Object_LIGHTLIST_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_LIGHTLIST_class_version_get, _LibreDWG.Dwg_Object_LIGHTLIST_class_version_set)
    num_lights = property(_LibreDWG.Dwg_Object_LIGHTLIST_num_lights_get, _LibreDWG.Dwg_Object_LIGHTLIST_num_lights_set)
    lights = property(_LibreDWG.Dwg_Object_LIGHTLIST_lights_get, _LibreDWG.Dwg_Object_LIGHTLIST_lights_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_LIGHTLIST_swiginit(self, _LibreDWG.new_Dwg_Object_LIGHTLIST())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_LIGHTLIST

# Register Dwg_Object_LIGHTLIST in _LibreDWG:
_LibreDWG.Dwg_Object_LIGHTLIST_swigregister(Dwg_Object_LIGHTLIST)
class Dwg_MATERIAL_color(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_MATERIAL_color_parent_get, _LibreDWG.Dwg_MATERIAL_color_parent_set)
    flag = property(_LibreDWG.Dwg_MATERIAL_color_flag_get, _LibreDWG.Dwg_MATERIAL_color_flag_set)
    factor = property(_LibreDWG.Dwg_MATERIAL_color_factor_get, _LibreDWG.Dwg_MATERIAL_color_factor_set)
    rgb = property(_LibreDWG.Dwg_MATERIAL_color_rgb_get, _LibreDWG.Dwg_MATERIAL_color_rgb_set)

    def __init__(self):
        _LibreDWG.Dwg_MATERIAL_color_swiginit(self, _LibreDWG.new_Dwg_MATERIAL_color())
    __swig_destroy__ = _LibreDWG.delete_Dwg_MATERIAL_color

# Register Dwg_MATERIAL_color in _LibreDWG:
_LibreDWG.Dwg_MATERIAL_color_swigregister(Dwg_MATERIAL_color)
class Dwg_MATERIAL_mapper(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_MATERIAL_mapper_parent_get, _LibreDWG.Dwg_MATERIAL_mapper_parent_set)
    blendfactor = property(_LibreDWG.Dwg_MATERIAL_mapper_blendfactor_get, _LibreDWG.Dwg_MATERIAL_mapper_blendfactor_set)
    transmatrix = property(_LibreDWG.Dwg_MATERIAL_mapper_transmatrix_get, _LibreDWG.Dwg_MATERIAL_mapper_transmatrix_set)
    filename = property(_LibreDWG.Dwg_MATERIAL_mapper_filename_get, _LibreDWG.Dwg_MATERIAL_mapper_filename_set)
    color1 = property(_LibreDWG.Dwg_MATERIAL_mapper_color1_get, _LibreDWG.Dwg_MATERIAL_mapper_color1_set)
    color2 = property(_LibreDWG.Dwg_MATERIAL_mapper_color2_get, _LibreDWG.Dwg_MATERIAL_mapper_color2_set)
    source = property(_LibreDWG.Dwg_MATERIAL_mapper_source_get, _LibreDWG.Dwg_MATERIAL_mapper_source_set)
    projection = property(_LibreDWG.Dwg_MATERIAL_mapper_projection_get, _LibreDWG.Dwg_MATERIAL_mapper_projection_set)
    tiling = property(_LibreDWG.Dwg_MATERIAL_mapper_tiling_get, _LibreDWG.Dwg_MATERIAL_mapper_tiling_set)
    autotransform = property(_LibreDWG.Dwg_MATERIAL_mapper_autotransform_get, _LibreDWG.Dwg_MATERIAL_mapper_autotransform_set)
    texturemode = property(_LibreDWG.Dwg_MATERIAL_mapper_texturemode_get, _LibreDWG.Dwg_MATERIAL_mapper_texturemode_set)

    def __init__(self):
        _LibreDWG.Dwg_MATERIAL_mapper_swiginit(self, _LibreDWG.new_Dwg_MATERIAL_mapper())
    __swig_destroy__ = _LibreDWG.delete_Dwg_MATERIAL_mapper

# Register Dwg_MATERIAL_mapper in _LibreDWG:
_LibreDWG.Dwg_MATERIAL_mapper_swigregister(Dwg_MATERIAL_mapper)
class Dwg_MATERIAL_gentexture(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_MATERIAL_gentexture_parent_get, _LibreDWG.Dwg_MATERIAL_gentexture_parent_set)
    genprocname = property(_LibreDWG.Dwg_MATERIAL_gentexture_genprocname_get, _LibreDWG.Dwg_MATERIAL_gentexture_genprocname_set)
    material = property(_LibreDWG.Dwg_MATERIAL_gentexture_material_get, _LibreDWG.Dwg_MATERIAL_gentexture_material_set)

    def __init__(self):
        _LibreDWG.Dwg_MATERIAL_gentexture_swiginit(self, _LibreDWG.new_Dwg_MATERIAL_gentexture())
    __swig_destroy__ = _LibreDWG.delete_Dwg_MATERIAL_gentexture

# Register Dwg_MATERIAL_gentexture in _LibreDWG:
_LibreDWG.Dwg_MATERIAL_gentexture_swigregister(Dwg_MATERIAL_gentexture)
class Dwg_Object_MATERIAL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_MATERIAL_parent_get, _LibreDWG.Dwg_Object_MATERIAL_parent_set)
    name = property(_LibreDWG.Dwg_Object_MATERIAL_name_get, _LibreDWG.Dwg_Object_MATERIAL_name_set)
    description = property(_LibreDWG.Dwg_Object_MATERIAL_description_get, _LibreDWG.Dwg_Object_MATERIAL_description_set)
    ambient_color = property(_LibreDWG.Dwg_Object_MATERIAL_ambient_color_get, _LibreDWG.Dwg_Object_MATERIAL_ambient_color_set)
    diffuse_color = property(_LibreDWG.Dwg_Object_MATERIAL_diffuse_color_get, _LibreDWG.Dwg_Object_MATERIAL_diffuse_color_set)
    diffusemap = property(_LibreDWG.Dwg_Object_MATERIAL_diffusemap_get, _LibreDWG.Dwg_Object_MATERIAL_diffusemap_set)
    specular_gloss_factor = property(_LibreDWG.Dwg_Object_MATERIAL_specular_gloss_factor_get, _LibreDWG.Dwg_Object_MATERIAL_specular_gloss_factor_set)
    specular_color = property(_LibreDWG.Dwg_Object_MATERIAL_specular_color_get, _LibreDWG.Dwg_Object_MATERIAL_specular_color_set)
    specularmap = property(_LibreDWG.Dwg_Object_MATERIAL_specularmap_get, _LibreDWG.Dwg_Object_MATERIAL_specularmap_set)
    reflectionmap = property(_LibreDWG.Dwg_Object_MATERIAL_reflectionmap_get, _LibreDWG.Dwg_Object_MATERIAL_reflectionmap_set)
    opacity_percent = property(_LibreDWG.Dwg_Object_MATERIAL_opacity_percent_get, _LibreDWG.Dwg_Object_MATERIAL_opacity_percent_set)
    opacitymap = property(_LibreDWG.Dwg_Object_MATERIAL_opacitymap_get, _LibreDWG.Dwg_Object_MATERIAL_opacitymap_set)
    bumpmap = property(_LibreDWG.Dwg_Object_MATERIAL_bumpmap_get, _LibreDWG.Dwg_Object_MATERIAL_bumpmap_set)
    refraction_index = property(_LibreDWG.Dwg_Object_MATERIAL_refraction_index_get, _LibreDWG.Dwg_Object_MATERIAL_refraction_index_set)
    refractionmap = property(_LibreDWG.Dwg_Object_MATERIAL_refractionmap_get, _LibreDWG.Dwg_Object_MATERIAL_refractionmap_set)
    color_bleed_scale = property(_LibreDWG.Dwg_Object_MATERIAL_color_bleed_scale_get, _LibreDWG.Dwg_Object_MATERIAL_color_bleed_scale_set)
    indirect_bump_scale = property(_LibreDWG.Dwg_Object_MATERIAL_indirect_bump_scale_get, _LibreDWG.Dwg_Object_MATERIAL_indirect_bump_scale_set)
    reflectance_scale = property(_LibreDWG.Dwg_Object_MATERIAL_reflectance_scale_get, _LibreDWG.Dwg_Object_MATERIAL_reflectance_scale_set)
    transmittance_scale = property(_LibreDWG.Dwg_Object_MATERIAL_transmittance_scale_get, _LibreDWG.Dwg_Object_MATERIAL_transmittance_scale_set)
    two_sided_material = property(_LibreDWG.Dwg_Object_MATERIAL_two_sided_material_get, _LibreDWG.Dwg_Object_MATERIAL_two_sided_material_set)
    luminance = property(_LibreDWG.Dwg_Object_MATERIAL_luminance_get, _LibreDWG.Dwg_Object_MATERIAL_luminance_set)
    luminance_mode = property(_LibreDWG.Dwg_Object_MATERIAL_luminance_mode_get, _LibreDWG.Dwg_Object_MATERIAL_luminance_mode_set)
    translucence = property(_LibreDWG.Dwg_Object_MATERIAL_translucence_get, _LibreDWG.Dwg_Object_MATERIAL_translucence_set)
    self_illumination = property(_LibreDWG.Dwg_Object_MATERIAL_self_illumination_get, _LibreDWG.Dwg_Object_MATERIAL_self_illumination_set)
    reflectivity = property(_LibreDWG.Dwg_Object_MATERIAL_reflectivity_get, _LibreDWG.Dwg_Object_MATERIAL_reflectivity_set)
    illumination_model = property(_LibreDWG.Dwg_Object_MATERIAL_illumination_model_get, _LibreDWG.Dwg_Object_MATERIAL_illumination_model_set)
    channel_flags = property(_LibreDWG.Dwg_Object_MATERIAL_channel_flags_get, _LibreDWG.Dwg_Object_MATERIAL_channel_flags_set)
    mode = property(_LibreDWG.Dwg_Object_MATERIAL_mode_get, _LibreDWG.Dwg_Object_MATERIAL_mode_set)
    genprocname = property(_LibreDWG.Dwg_Object_MATERIAL_genprocname_get, _LibreDWG.Dwg_Object_MATERIAL_genprocname_set)
    genproctype = property(_LibreDWG.Dwg_Object_MATERIAL_genproctype_get, _LibreDWG.Dwg_Object_MATERIAL_genproctype_set)
    genprocvalbool = property(_LibreDWG.Dwg_Object_MATERIAL_genprocvalbool_get, _LibreDWG.Dwg_Object_MATERIAL_genprocvalbool_set)
    genprocvalint = property(_LibreDWG.Dwg_Object_MATERIAL_genprocvalint_get, _LibreDWG.Dwg_Object_MATERIAL_genprocvalint_set)
    genprocvalreal = property(_LibreDWG.Dwg_Object_MATERIAL_genprocvalreal_get, _LibreDWG.Dwg_Object_MATERIAL_genprocvalreal_set)
    genprocvaltext = property(_LibreDWG.Dwg_Object_MATERIAL_genprocvaltext_get, _LibreDWG.Dwg_Object_MATERIAL_genprocvaltext_set)
    genprocvalcolor = property(_LibreDWG.Dwg_Object_MATERIAL_genprocvalcolor_get, _LibreDWG.Dwg_Object_MATERIAL_genprocvalcolor_set)
    genproctableend = property(_LibreDWG.Dwg_Object_MATERIAL_genproctableend_get, _LibreDWG.Dwg_Object_MATERIAL_genproctableend_set)
    num_gentextures = property(_LibreDWG.Dwg_Object_MATERIAL_num_gentextures_get, _LibreDWG.Dwg_Object_MATERIAL_num_gentextures_set)
    gentextures = property(_LibreDWG.Dwg_Object_MATERIAL_gentextures_get, _LibreDWG.Dwg_Object_MATERIAL_gentextures_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_MATERIAL_swiginit(self, _LibreDWG.new_Dwg_Object_MATERIAL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_MATERIAL

# Register Dwg_Object_MATERIAL in _LibreDWG:
_LibreDWG.Dwg_Object_MATERIAL_swigregister(Dwg_Object_MATERIAL)
class Dwg_Object_OBJECT_PTR(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_OBJECT_PTR_parent_get, _LibreDWG.Dwg_Object_OBJECT_PTR_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_OBJECT_PTR_swiginit(self, _LibreDWG.new_Dwg_Object_OBJECT_PTR())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_OBJECT_PTR

# Register Dwg_Object_OBJECT_PTR in _LibreDWG:
_LibreDWG.Dwg_Object_OBJECT_PTR_swigregister(Dwg_Object_OBJECT_PTR)
class Dwg_Entity_LIGHT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_LIGHT_parent_get, _LibreDWG.Dwg_Entity_LIGHT_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_LIGHT_class_version_get, _LibreDWG.Dwg_Entity_LIGHT_class_version_set)
    name = property(_LibreDWG.Dwg_Entity_LIGHT_name_get, _LibreDWG.Dwg_Entity_LIGHT_name_set)
    type = property(_LibreDWG.Dwg_Entity_LIGHT_type_get, _LibreDWG.Dwg_Entity_LIGHT_type_set)
    status = property(_LibreDWG.Dwg_Entity_LIGHT_status_get, _LibreDWG.Dwg_Entity_LIGHT_status_set)
    light_color = property(_LibreDWG.Dwg_Entity_LIGHT_light_color_get, _LibreDWG.Dwg_Entity_LIGHT_light_color_set)
    plot_glyph = property(_LibreDWG.Dwg_Entity_LIGHT_plot_glyph_get, _LibreDWG.Dwg_Entity_LIGHT_plot_glyph_set)
    intensity = property(_LibreDWG.Dwg_Entity_LIGHT_intensity_get, _LibreDWG.Dwg_Entity_LIGHT_intensity_set)
    position = property(_LibreDWG.Dwg_Entity_LIGHT_position_get, _LibreDWG.Dwg_Entity_LIGHT_position_set)
    target = property(_LibreDWG.Dwg_Entity_LIGHT_target_get, _LibreDWG.Dwg_Entity_LIGHT_target_set)
    attenuation_type = property(_LibreDWG.Dwg_Entity_LIGHT_attenuation_type_get, _LibreDWG.Dwg_Entity_LIGHT_attenuation_type_set)
    use_attenuation_limits = property(_LibreDWG.Dwg_Entity_LIGHT_use_attenuation_limits_get, _LibreDWG.Dwg_Entity_LIGHT_use_attenuation_limits_set)
    attenuation_start_limit = property(_LibreDWG.Dwg_Entity_LIGHT_attenuation_start_limit_get, _LibreDWG.Dwg_Entity_LIGHT_attenuation_start_limit_set)
    attenuation_end_limit = property(_LibreDWG.Dwg_Entity_LIGHT_attenuation_end_limit_get, _LibreDWG.Dwg_Entity_LIGHT_attenuation_end_limit_set)
    hotspot_angle = property(_LibreDWG.Dwg_Entity_LIGHT_hotspot_angle_get, _LibreDWG.Dwg_Entity_LIGHT_hotspot_angle_set)
    falloff_angle = property(_LibreDWG.Dwg_Entity_LIGHT_falloff_angle_get, _LibreDWG.Dwg_Entity_LIGHT_falloff_angle_set)
    cast_shadows = property(_LibreDWG.Dwg_Entity_LIGHT_cast_shadows_get, _LibreDWG.Dwg_Entity_LIGHT_cast_shadows_set)
    shadow_type = property(_LibreDWG.Dwg_Entity_LIGHT_shadow_type_get, _LibreDWG.Dwg_Entity_LIGHT_shadow_type_set)
    shadow_map_size = property(_LibreDWG.Dwg_Entity_LIGHT_shadow_map_size_get, _LibreDWG.Dwg_Entity_LIGHT_shadow_map_size_set)
    shadow_map_softness = property(_LibreDWG.Dwg_Entity_LIGHT_shadow_map_softness_get, _LibreDWG.Dwg_Entity_LIGHT_shadow_map_softness_set)
    is_photometric = property(_LibreDWG.Dwg_Entity_LIGHT_is_photometric_get, _LibreDWG.Dwg_Entity_LIGHT_is_photometric_set)
    has_photometric_data = property(_LibreDWG.Dwg_Entity_LIGHT_has_photometric_data_get, _LibreDWG.Dwg_Entity_LIGHT_has_photometric_data_set)
    has_webfile = property(_LibreDWG.Dwg_Entity_LIGHT_has_webfile_get, _LibreDWG.Dwg_Entity_LIGHT_has_webfile_set)
    webfile = property(_LibreDWG.Dwg_Entity_LIGHT_webfile_get, _LibreDWG.Dwg_Entity_LIGHT_webfile_set)
    physical_intensity_method = property(_LibreDWG.Dwg_Entity_LIGHT_physical_intensity_method_get, _LibreDWG.Dwg_Entity_LIGHT_physical_intensity_method_set)
    physical_intensity = property(_LibreDWG.Dwg_Entity_LIGHT_physical_intensity_get, _LibreDWG.Dwg_Entity_LIGHT_physical_intensity_set)
    illuminance_dist = property(_LibreDWG.Dwg_Entity_LIGHT_illuminance_dist_get, _LibreDWG.Dwg_Entity_LIGHT_illuminance_dist_set)
    lamp_color_type = property(_LibreDWG.Dwg_Entity_LIGHT_lamp_color_type_get, _LibreDWG.Dwg_Entity_LIGHT_lamp_color_type_set)
    lamp_color_temp = property(_LibreDWG.Dwg_Entity_LIGHT_lamp_color_temp_get, _LibreDWG.Dwg_Entity_LIGHT_lamp_color_temp_set)
    lamp_color_preset = property(_LibreDWG.Dwg_Entity_LIGHT_lamp_color_preset_get, _LibreDWG.Dwg_Entity_LIGHT_lamp_color_preset_set)
    lamp_color_rgb = property(_LibreDWG.Dwg_Entity_LIGHT_lamp_color_rgb_get, _LibreDWG.Dwg_Entity_LIGHT_lamp_color_rgb_set)
    web_rotation = property(_LibreDWG.Dwg_Entity_LIGHT_web_rotation_get, _LibreDWG.Dwg_Entity_LIGHT_web_rotation_set)
    extlight_shape = property(_LibreDWG.Dwg_Entity_LIGHT_extlight_shape_get, _LibreDWG.Dwg_Entity_LIGHT_extlight_shape_set)
    extlight_length = property(_LibreDWG.Dwg_Entity_LIGHT_extlight_length_get, _LibreDWG.Dwg_Entity_LIGHT_extlight_length_set)
    extlight_width = property(_LibreDWG.Dwg_Entity_LIGHT_extlight_width_get, _LibreDWG.Dwg_Entity_LIGHT_extlight_width_set)
    extlight_radius = property(_LibreDWG.Dwg_Entity_LIGHT_extlight_radius_get, _LibreDWG.Dwg_Entity_LIGHT_extlight_radius_set)
    webfile_type = property(_LibreDWG.Dwg_Entity_LIGHT_webfile_type_get, _LibreDWG.Dwg_Entity_LIGHT_webfile_type_set)
    web_symetry = property(_LibreDWG.Dwg_Entity_LIGHT_web_symetry_get, _LibreDWG.Dwg_Entity_LIGHT_web_symetry_set)
    has_target_grip = property(_LibreDWG.Dwg_Entity_LIGHT_has_target_grip_get, _LibreDWG.Dwg_Entity_LIGHT_has_target_grip_set)
    web_flux = property(_LibreDWG.Dwg_Entity_LIGHT_web_flux_get, _LibreDWG.Dwg_Entity_LIGHT_web_flux_set)
    web_angle1 = property(_LibreDWG.Dwg_Entity_LIGHT_web_angle1_get, _LibreDWG.Dwg_Entity_LIGHT_web_angle1_set)
    web_angle2 = property(_LibreDWG.Dwg_Entity_LIGHT_web_angle2_get, _LibreDWG.Dwg_Entity_LIGHT_web_angle2_set)
    web_angle3 = property(_LibreDWG.Dwg_Entity_LIGHT_web_angle3_get, _LibreDWG.Dwg_Entity_LIGHT_web_angle3_set)
    web_angle4 = property(_LibreDWG.Dwg_Entity_LIGHT_web_angle4_get, _LibreDWG.Dwg_Entity_LIGHT_web_angle4_set)
    web_angle5 = property(_LibreDWG.Dwg_Entity_LIGHT_web_angle5_get, _LibreDWG.Dwg_Entity_LIGHT_web_angle5_set)
    glyph_display_type = property(_LibreDWG.Dwg_Entity_LIGHT_glyph_display_type_get, _LibreDWG.Dwg_Entity_LIGHT_glyph_display_type_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_LIGHT_swiginit(self, _LibreDWG.new_Dwg_Entity_LIGHT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_LIGHT

# Register Dwg_Entity_LIGHT in _LibreDWG:
_LibreDWG.Dwg_Entity_LIGHT_swigregister(Dwg_Entity_LIGHT)
class Dwg_Entity_CAMERA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_CAMERA_parent_get, _LibreDWG.Dwg_Entity_CAMERA_parent_set)
    view = property(_LibreDWG.Dwg_Entity_CAMERA_view_get, _LibreDWG.Dwg_Entity_CAMERA_view_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_CAMERA_swiginit(self, _LibreDWG.new_Dwg_Entity_CAMERA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_CAMERA

# Register Dwg_Entity_CAMERA in _LibreDWG:
_LibreDWG.Dwg_Entity_CAMERA_swigregister(Dwg_Entity_CAMERA)
class Dwg_Entity_GEOPOSITIONMARKER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_parent_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_class_version_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_class_version_set)
    position = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_position_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_position_set)
    radius = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_radius_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_radius_set)
    landing_gap = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_landing_gap_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_landing_gap_set)
    notes = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_notes_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_notes_set)
    text_alignment = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_text_alignment_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_text_alignment_set)
    mtext_visible = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_set)
    enable_frame_text = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_set)
    ins_pt = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_ins_pt_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_ins_pt_set)
    alignment_pt = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_set)
    mtext = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_mtext_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_mtext_set)
    is_really_locked = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_set)
    annotative_data_size = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_set)
    annotative_data = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_annotative_data_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_annotative_data_set)
    annotative_flag = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_set)
    annotative_style = property(_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_annotative_style_get, _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_annotative_style_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_swiginit(self, _LibreDWG.new_Dwg_Entity_GEOPOSITIONMARKER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_GEOPOSITIONMARKER

# Register Dwg_Entity_GEOPOSITIONMARKER in _LibreDWG:
_LibreDWG.Dwg_Entity_GEOPOSITIONMARKER_swigregister(Dwg_Entity_GEOPOSITIONMARKER)
class Dwg_Object_GEOMAPIMAGE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_parent_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_class_version_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_class_version_set)
    pt0 = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_pt0_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_pt0_set)
    image_size = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_image_size_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_image_size_set)
    display_props = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_display_props_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_display_props_set)
    clipping = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_clipping_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_clipping_set)
    brightness = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_brightness_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_brightness_set)
    contrast = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_contrast_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_contrast_set)
    fade = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_fade_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_fade_set)
    rotation = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_rotation_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_rotation_set)
    image_width = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_image_width_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_image_width_set)
    image_height = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_image_height_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_image_height_set)
    name = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_name_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_name_set)
    image_file = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_image_file_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_image_file_set)
    image_visibility = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_image_visibility_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_image_visibility_set)
    transparency = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_transparency_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_transparency_set)
    height = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_height_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_height_set)
    width = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_width_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_width_set)
    show_rotation = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_show_rotation_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_show_rotation_set)
    scale_factor = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_scale_factor_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_scale_factor_set)
    geoimage_brightness = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_geoimage_brightness_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_geoimage_brightness_set)
    geoimage_contrast = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_geoimage_contrast_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_geoimage_contrast_set)
    geoimage_fade = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_geoimage_fade_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_geoimage_fade_set)
    geoimage_position = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_geoimage_position_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_geoimage_position_set)
    geoimage_width = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_geoimage_width_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_geoimage_width_set)
    geoimage_height = property(_LibreDWG.Dwg_Object_GEOMAPIMAGE_geoimage_height_get, _LibreDWG.Dwg_Object_GEOMAPIMAGE_geoimage_height_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_GEOMAPIMAGE_swiginit(self, _LibreDWG.new_Dwg_Object_GEOMAPIMAGE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_GEOMAPIMAGE

# Register Dwg_Object_GEOMAPIMAGE in _LibreDWG:
_LibreDWG.Dwg_Object_GEOMAPIMAGE_swigregister(Dwg_Object_GEOMAPIMAGE)
class Dwg_Entity_HELIX(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_HELIX_parent_get, _LibreDWG.Dwg_Entity_HELIX_parent_set)
    flag = property(_LibreDWG.Dwg_Entity_HELIX_flag_get, _LibreDWG.Dwg_Entity_HELIX_flag_set)
    scenario = property(_LibreDWG.Dwg_Entity_HELIX_scenario_get, _LibreDWG.Dwg_Entity_HELIX_scenario_set)
    degree = property(_LibreDWG.Dwg_Entity_HELIX_degree_get, _LibreDWG.Dwg_Entity_HELIX_degree_set)
    splineflags = property(_LibreDWG.Dwg_Entity_HELIX_splineflags_get, _LibreDWG.Dwg_Entity_HELIX_splineflags_set)
    knotparam = property(_LibreDWG.Dwg_Entity_HELIX_knotparam_get, _LibreDWG.Dwg_Entity_HELIX_knotparam_set)
    fit_tol = property(_LibreDWG.Dwg_Entity_HELIX_fit_tol_get, _LibreDWG.Dwg_Entity_HELIX_fit_tol_set)
    beg_tan_vec = property(_LibreDWG.Dwg_Entity_HELIX_beg_tan_vec_get, _LibreDWG.Dwg_Entity_HELIX_beg_tan_vec_set)
    end_tan_vec = property(_LibreDWG.Dwg_Entity_HELIX_end_tan_vec_get, _LibreDWG.Dwg_Entity_HELIX_end_tan_vec_set)
    rational = property(_LibreDWG.Dwg_Entity_HELIX_rational_get, _LibreDWG.Dwg_Entity_HELIX_rational_set)
    closed_b = property(_LibreDWG.Dwg_Entity_HELIX_closed_b_get, _LibreDWG.Dwg_Entity_HELIX_closed_b_set)
    periodic = property(_LibreDWG.Dwg_Entity_HELIX_periodic_get, _LibreDWG.Dwg_Entity_HELIX_periodic_set)
    weighted = property(_LibreDWG.Dwg_Entity_HELIX_weighted_get, _LibreDWG.Dwg_Entity_HELIX_weighted_set)
    knot_tol = property(_LibreDWG.Dwg_Entity_HELIX_knot_tol_get, _LibreDWG.Dwg_Entity_HELIX_knot_tol_set)
    ctrl_tol = property(_LibreDWG.Dwg_Entity_HELIX_ctrl_tol_get, _LibreDWG.Dwg_Entity_HELIX_ctrl_tol_set)
    num_fit_pts = property(_LibreDWG.Dwg_Entity_HELIX_num_fit_pts_get, _LibreDWG.Dwg_Entity_HELIX_num_fit_pts_set)
    fit_pts = property(_LibreDWG.Dwg_Entity_HELIX_fit_pts_get, _LibreDWG.Dwg_Entity_HELIX_fit_pts_set)
    num_knots = property(_LibreDWG.Dwg_Entity_HELIX_num_knots_get, _LibreDWG.Dwg_Entity_HELIX_num_knots_set)
    knots = property(_LibreDWG.Dwg_Entity_HELIX_knots_get, _LibreDWG.Dwg_Entity_HELIX_knots_set)
    num_ctrl_pts = property(_LibreDWG.Dwg_Entity_HELIX_num_ctrl_pts_get, _LibreDWG.Dwg_Entity_HELIX_num_ctrl_pts_set)
    ctrl_pts = property(_LibreDWG.Dwg_Entity_HELIX_ctrl_pts_get, _LibreDWG.Dwg_Entity_HELIX_ctrl_pts_set)
    major_version = property(_LibreDWG.Dwg_Entity_HELIX_major_version_get, _LibreDWG.Dwg_Entity_HELIX_major_version_set)
    maint_version = property(_LibreDWG.Dwg_Entity_HELIX_maint_version_get, _LibreDWG.Dwg_Entity_HELIX_maint_version_set)
    axis_base_pt = property(_LibreDWG.Dwg_Entity_HELIX_axis_base_pt_get, _LibreDWG.Dwg_Entity_HELIX_axis_base_pt_set)
    start_pt = property(_LibreDWG.Dwg_Entity_HELIX_start_pt_get, _LibreDWG.Dwg_Entity_HELIX_start_pt_set)
    axis_vector = property(_LibreDWG.Dwg_Entity_HELIX_axis_vector_get, _LibreDWG.Dwg_Entity_HELIX_axis_vector_set)
    radius = property(_LibreDWG.Dwg_Entity_HELIX_radius_get, _LibreDWG.Dwg_Entity_HELIX_radius_set)
    turns = property(_LibreDWG.Dwg_Entity_HELIX_turns_get, _LibreDWG.Dwg_Entity_HELIX_turns_set)
    turn_height = property(_LibreDWG.Dwg_Entity_HELIX_turn_height_get, _LibreDWG.Dwg_Entity_HELIX_turn_height_set)
    handedness = property(_LibreDWG.Dwg_Entity_HELIX_handedness_get, _LibreDWG.Dwg_Entity_HELIX_handedness_set)
    constraint_type = property(_LibreDWG.Dwg_Entity_HELIX_constraint_type_get, _LibreDWG.Dwg_Entity_HELIX_constraint_type_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_HELIX_swiginit(self, _LibreDWG.new_Dwg_Entity_HELIX())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_HELIX

# Register Dwg_Entity_HELIX in _LibreDWG:
_LibreDWG.Dwg_Entity_HELIX_swigregister(Dwg_Entity_HELIX)
class Dwg_Entity_EXTRUDEDSURFACE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_parent_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_parent_set)
    acis_empty = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_acis_empty_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_acis_empty_set)
    unknown = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_unknown_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_unknown_set)
    version = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_version_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_version_set)
    num_blocks = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_num_blocks_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_num_blocks_set)
    block_size = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_block_size_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_block_size_set)
    encr_sat_data = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_set)
    sab_size = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sab_size_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sab_size_set)
    acis_data = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_acis_data_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_acis_data_set)
    wireframe_data_present = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_set)
    point_present = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_point_present_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_point_present_set)
    point = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_point_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_point_set)
    isolines = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_isolines_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_isolines_set)
    isoline_present = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_isoline_present_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_isoline_present_set)
    num_wires = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_num_wires_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_num_wires_set)
    wires = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_wires_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_wires_set)
    num_silhouettes = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_set)
    silhouettes = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_silhouettes_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_silhouettes_set)
    _dxf_sab_converted = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_set)
    acis_empty2 = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_set)
    extra_acis_data = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_set)
    num_materials = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_num_materials_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_num_materials_set)
    materials = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_materials_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_materials_set)
    revision_guid = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_revision_guid_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_revision_guid_set)
    revision_major = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_revision_major_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_revision_major_set)
    revision_minor1 = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_set)
    revision_minor2 = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_set)
    revision_bytes = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_set)
    end_marker = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_end_marker_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_end_marker_set)
    history_id = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_history_id_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_history_id_set)
    has_revision_guid = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_set)
    acis_empty_bit = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_set)
    modeler_format_version = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_set)
    bindata_size = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_bindata_size_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_bindata_size_set)
    bindata = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_bindata_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_bindata_set)
    u_isolines = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_u_isolines_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_u_isolines_set)
    v_isolines = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_v_isolines_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_v_isolines_set)
    class_version = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_class_version_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_class_version_set)
    draft_angle = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_draft_angle_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_draft_angle_set)
    draft_start_distance = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_set)
    draft_end_distance = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_set)
    twist_angle = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_twist_angle_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_twist_angle_set)
    scale_factor = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_scale_factor_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_scale_factor_set)
    align_angle = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_align_angle_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_align_angle_set)
    sweep_entity_transmatrix = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_set)
    path_entity_transmatrix = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_set)
    is_solid = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_is_solid_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_is_solid_set)
    sweep_alignment_flags = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_set)
    path_flags = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_path_flags_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_path_flags_set)
    align_start = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_align_start_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_align_start_set)
    bank = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_bank_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_bank_set)
    base_point_set = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_base_point_set_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_base_point_set_set)
    sweep_entity_transform_computed = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_set)
    path_entity_transform_computed = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_set)
    reference_vector_for_controlling_twist = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_set)
    sweep_entity = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_set)
    path_entity = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_path_entity_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_path_entity_set)
    sweep_vector = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_set)
    sweep_transmatrix = property(_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_get, _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_swiginit(self, _LibreDWG.new_Dwg_Entity_EXTRUDEDSURFACE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_EXTRUDEDSURFACE

# Register Dwg_Entity_EXTRUDEDSURFACE in _LibreDWG:
_LibreDWG.Dwg_Entity_EXTRUDEDSURFACE_swigregister(Dwg_Entity_EXTRUDEDSURFACE)
class Dwg_Entity_SWEPTSURFACE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_parent_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_parent_set)
    acis_empty = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_acis_empty_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_acis_empty_set)
    unknown = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_unknown_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_unknown_set)
    version = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_version_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_version_set)
    num_blocks = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_num_blocks_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_num_blocks_set)
    block_size = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_block_size_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_block_size_set)
    encr_sat_data = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_encr_sat_data_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_encr_sat_data_set)
    sab_size = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_sab_size_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_sab_size_set)
    acis_data = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_acis_data_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_acis_data_set)
    wireframe_data_present = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_wireframe_data_present_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_wireframe_data_present_set)
    point_present = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_point_present_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_point_present_set)
    point = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_point_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_point_set)
    isolines = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_isolines_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_isolines_set)
    isoline_present = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_isoline_present_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_isoline_present_set)
    num_wires = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_num_wires_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_num_wires_set)
    wires = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_wires_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_wires_set)
    num_silhouettes = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_num_silhouettes_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_num_silhouettes_set)
    silhouettes = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_silhouettes_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_silhouettes_set)
    _dxf_sab_converted = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_set)
    acis_empty2 = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_acis_empty2_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_acis_empty2_set)
    extra_acis_data = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_extra_acis_data_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_extra_acis_data_set)
    num_materials = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_num_materials_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_num_materials_set)
    materials = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_materials_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_materials_set)
    revision_guid = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_revision_guid_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_revision_guid_set)
    revision_major = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_revision_major_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_revision_major_set)
    revision_minor1 = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_revision_minor1_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_revision_minor1_set)
    revision_minor2 = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_revision_minor2_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_revision_minor2_set)
    revision_bytes = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_revision_bytes_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_revision_bytes_set)
    end_marker = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_end_marker_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_end_marker_set)
    history_id = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_history_id_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_history_id_set)
    has_revision_guid = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_has_revision_guid_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_has_revision_guid_set)
    acis_empty_bit = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_acis_empty_bit_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_acis_empty_bit_set)
    modeler_format_version = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_modeler_format_version_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_modeler_format_version_set)
    u_isolines = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_u_isolines_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_u_isolines_set)
    v_isolines = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_v_isolines_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_v_isolines_set)
    class_version = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_class_version_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_class_version_set)
    sweep_entity_id = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_sweep_entity_id_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_sweep_entity_id_set)
    sweepdata_size = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_sweepdata_size_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_sweepdata_size_set)
    sweepdata = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_sweepdata_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_sweepdata_set)
    path_entity_id = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_path_entity_id_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_path_entity_id_set)
    pathdata_size = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_pathdata_size_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_pathdata_size_set)
    pathdata = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_pathdata_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_pathdata_set)
    draft_angle = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_draft_angle_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_draft_angle_set)
    draft_start_distance = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_draft_start_distance_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_draft_start_distance_set)
    draft_end_distance = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_draft_end_distance_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_draft_end_distance_set)
    twist_angle = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_twist_angle_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_twist_angle_set)
    scale_factor = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_scale_factor_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_scale_factor_set)
    align_angle = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_align_angle_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_align_angle_set)
    sweep_entity_transmatrix = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_set)
    path_entity_transmatrix = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_set)
    is_solid = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_is_solid_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_is_solid_set)
    sweep_alignment_flags = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_set)
    path_flags = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_path_flags_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_path_flags_set)
    align_start = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_align_start_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_align_start_set)
    bank = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_bank_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_bank_set)
    base_point_set = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_base_point_set_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_base_point_set_set)
    sweep_entity_transform_computed = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_set)
    path_entity_transform_computed = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_set)
    reference_vector_for_controlling_twist = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_set)
    sweep_entity = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_sweep_entity_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_sweep_entity_set)
    path_entity = property(_LibreDWG.Dwg_Entity_SWEPTSURFACE_path_entity_get, _LibreDWG.Dwg_Entity_SWEPTSURFACE_path_entity_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_SWEPTSURFACE_swiginit(self, _LibreDWG.new_Dwg_Entity_SWEPTSURFACE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_SWEPTSURFACE

# Register Dwg_Entity_SWEPTSURFACE in _LibreDWG:
_LibreDWG.Dwg_Entity_SWEPTSURFACE_swigregister(Dwg_Entity_SWEPTSURFACE)
class Dwg_Entity_LOFTEDSURFACE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_parent_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_parent_set)
    acis_empty = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_acis_empty_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_acis_empty_set)
    unknown = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_unknown_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_unknown_set)
    version = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_version_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_version_set)
    num_blocks = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_num_blocks_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_num_blocks_set)
    block_size = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_block_size_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_block_size_set)
    encr_sat_data = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_encr_sat_data_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_encr_sat_data_set)
    sab_size = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_sab_size_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_sab_size_set)
    acis_data = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_acis_data_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_acis_data_set)
    wireframe_data_present = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_set)
    point_present = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_point_present_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_point_present_set)
    point = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_point_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_point_set)
    isolines = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_isolines_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_isolines_set)
    isoline_present = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_isoline_present_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_isoline_present_set)
    num_wires = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_num_wires_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_num_wires_set)
    wires = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_wires_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_wires_set)
    num_silhouettes = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_num_silhouettes_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_num_silhouettes_set)
    silhouettes = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_silhouettes_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_silhouettes_set)
    _dxf_sab_converted = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_set)
    acis_empty2 = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_acis_empty2_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_acis_empty2_set)
    extra_acis_data = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_extra_acis_data_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_extra_acis_data_set)
    num_materials = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_num_materials_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_num_materials_set)
    materials = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_materials_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_materials_set)
    revision_guid = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_revision_guid_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_revision_guid_set)
    revision_major = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_revision_major_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_revision_major_set)
    revision_minor1 = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_revision_minor1_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_revision_minor1_set)
    revision_minor2 = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_revision_minor2_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_revision_minor2_set)
    revision_bytes = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_revision_bytes_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_revision_bytes_set)
    end_marker = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_end_marker_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_end_marker_set)
    history_id = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_history_id_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_history_id_set)
    has_revision_guid = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_has_revision_guid_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_has_revision_guid_set)
    acis_empty_bit = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_set)
    modeler_format_version = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_modeler_format_version_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_modeler_format_version_set)
    u_isolines = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_u_isolines_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_u_isolines_set)
    v_isolines = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_v_isolines_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_v_isolines_set)
    loft_entity_transmatrix = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_set)
    plane_normal_lofting_type = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_set)
    start_draft_angle = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_start_draft_angle_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_start_draft_angle_set)
    end_draft_angle = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_end_draft_angle_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_end_draft_angle_set)
    start_draft_magnitude = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_set)
    end_draft_magnitude = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_set)
    arc_length_parameterization = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_set)
    no_twist = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_no_twist_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_no_twist_set)
    align_direction = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_align_direction_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_align_direction_set)
    simple_surfaces = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_simple_surfaces_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_simple_surfaces_set)
    closed_surfaces = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_closed_surfaces_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_closed_surfaces_set)
    solid = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_solid_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_solid_set)
    ruled_surface = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_ruled_surface_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_ruled_surface_set)
    virtual_guide = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_virtual_guide_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_virtual_guide_set)
    num_cross_sections = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_num_cross_sections_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_num_cross_sections_set)
    num_guide_curves = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_num_guide_curves_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_num_guide_curves_set)
    cross_sections = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_cross_sections_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_cross_sections_set)
    guide_curves = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_guide_curves_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_guide_curves_set)
    path_curve = property(_LibreDWG.Dwg_Entity_LOFTEDSURFACE_path_curve_get, _LibreDWG.Dwg_Entity_LOFTEDSURFACE_path_curve_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_LOFTEDSURFACE_swiginit(self, _LibreDWG.new_Dwg_Entity_LOFTEDSURFACE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_LOFTEDSURFACE

# Register Dwg_Entity_LOFTEDSURFACE in _LibreDWG:
_LibreDWG.Dwg_Entity_LOFTEDSURFACE_swigregister(Dwg_Entity_LOFTEDSURFACE)
class Dwg_Entity_NURBSURFACE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_NURBSURFACE_parent_get, _LibreDWG.Dwg_Entity_NURBSURFACE_parent_set)
    acis_empty = property(_LibreDWG.Dwg_Entity_NURBSURFACE_acis_empty_get, _LibreDWG.Dwg_Entity_NURBSURFACE_acis_empty_set)
    unknown = property(_LibreDWG.Dwg_Entity_NURBSURFACE_unknown_get, _LibreDWG.Dwg_Entity_NURBSURFACE_unknown_set)
    version = property(_LibreDWG.Dwg_Entity_NURBSURFACE_version_get, _LibreDWG.Dwg_Entity_NURBSURFACE_version_set)
    num_blocks = property(_LibreDWG.Dwg_Entity_NURBSURFACE_num_blocks_get, _LibreDWG.Dwg_Entity_NURBSURFACE_num_blocks_set)
    block_size = property(_LibreDWG.Dwg_Entity_NURBSURFACE_block_size_get, _LibreDWG.Dwg_Entity_NURBSURFACE_block_size_set)
    encr_sat_data = property(_LibreDWG.Dwg_Entity_NURBSURFACE_encr_sat_data_get, _LibreDWG.Dwg_Entity_NURBSURFACE_encr_sat_data_set)
    sab_size = property(_LibreDWG.Dwg_Entity_NURBSURFACE_sab_size_get, _LibreDWG.Dwg_Entity_NURBSURFACE_sab_size_set)
    acis_data = property(_LibreDWG.Dwg_Entity_NURBSURFACE_acis_data_get, _LibreDWG.Dwg_Entity_NURBSURFACE_acis_data_set)
    wireframe_data_present = property(_LibreDWG.Dwg_Entity_NURBSURFACE_wireframe_data_present_get, _LibreDWG.Dwg_Entity_NURBSURFACE_wireframe_data_present_set)
    point_present = property(_LibreDWG.Dwg_Entity_NURBSURFACE_point_present_get, _LibreDWG.Dwg_Entity_NURBSURFACE_point_present_set)
    point = property(_LibreDWG.Dwg_Entity_NURBSURFACE_point_get, _LibreDWG.Dwg_Entity_NURBSURFACE_point_set)
    isolines = property(_LibreDWG.Dwg_Entity_NURBSURFACE_isolines_get, _LibreDWG.Dwg_Entity_NURBSURFACE_isolines_set)
    isoline_present = property(_LibreDWG.Dwg_Entity_NURBSURFACE_isoline_present_get, _LibreDWG.Dwg_Entity_NURBSURFACE_isoline_present_set)
    num_wires = property(_LibreDWG.Dwg_Entity_NURBSURFACE_num_wires_get, _LibreDWG.Dwg_Entity_NURBSURFACE_num_wires_set)
    wires = property(_LibreDWG.Dwg_Entity_NURBSURFACE_wires_get, _LibreDWG.Dwg_Entity_NURBSURFACE_wires_set)
    num_silhouettes = property(_LibreDWG.Dwg_Entity_NURBSURFACE_num_silhouettes_get, _LibreDWG.Dwg_Entity_NURBSURFACE_num_silhouettes_set)
    silhouettes = property(_LibreDWG.Dwg_Entity_NURBSURFACE_silhouettes_get, _LibreDWG.Dwg_Entity_NURBSURFACE_silhouettes_set)
    _dxf_sab_converted = property(_LibreDWG.Dwg_Entity_NURBSURFACE__dxf_sab_converted_get, _LibreDWG.Dwg_Entity_NURBSURFACE__dxf_sab_converted_set)
    acis_empty2 = property(_LibreDWG.Dwg_Entity_NURBSURFACE_acis_empty2_get, _LibreDWG.Dwg_Entity_NURBSURFACE_acis_empty2_set)
    extra_acis_data = property(_LibreDWG.Dwg_Entity_NURBSURFACE_extra_acis_data_get, _LibreDWG.Dwg_Entity_NURBSURFACE_extra_acis_data_set)
    num_materials = property(_LibreDWG.Dwg_Entity_NURBSURFACE_num_materials_get, _LibreDWG.Dwg_Entity_NURBSURFACE_num_materials_set)
    materials = property(_LibreDWG.Dwg_Entity_NURBSURFACE_materials_get, _LibreDWG.Dwg_Entity_NURBSURFACE_materials_set)
    revision_guid = property(_LibreDWG.Dwg_Entity_NURBSURFACE_revision_guid_get, _LibreDWG.Dwg_Entity_NURBSURFACE_revision_guid_set)
    revision_major = property(_LibreDWG.Dwg_Entity_NURBSURFACE_revision_major_get, _LibreDWG.Dwg_Entity_NURBSURFACE_revision_major_set)
    revision_minor1 = property(_LibreDWG.Dwg_Entity_NURBSURFACE_revision_minor1_get, _LibreDWG.Dwg_Entity_NURBSURFACE_revision_minor1_set)
    revision_minor2 = property(_LibreDWG.Dwg_Entity_NURBSURFACE_revision_minor2_get, _LibreDWG.Dwg_Entity_NURBSURFACE_revision_minor2_set)
    revision_bytes = property(_LibreDWG.Dwg_Entity_NURBSURFACE_revision_bytes_get, _LibreDWG.Dwg_Entity_NURBSURFACE_revision_bytes_set)
    end_marker = property(_LibreDWG.Dwg_Entity_NURBSURFACE_end_marker_get, _LibreDWG.Dwg_Entity_NURBSURFACE_end_marker_set)
    history_id = property(_LibreDWG.Dwg_Entity_NURBSURFACE_history_id_get, _LibreDWG.Dwg_Entity_NURBSURFACE_history_id_set)
    has_revision_guid = property(_LibreDWG.Dwg_Entity_NURBSURFACE_has_revision_guid_get, _LibreDWG.Dwg_Entity_NURBSURFACE_has_revision_guid_set)
    acis_empty_bit = property(_LibreDWG.Dwg_Entity_NURBSURFACE_acis_empty_bit_get, _LibreDWG.Dwg_Entity_NURBSURFACE_acis_empty_bit_set)
    u_isolines = property(_LibreDWG.Dwg_Entity_NURBSURFACE_u_isolines_get, _LibreDWG.Dwg_Entity_NURBSURFACE_u_isolines_set)
    v_isolines = property(_LibreDWG.Dwg_Entity_NURBSURFACE_v_isolines_get, _LibreDWG.Dwg_Entity_NURBSURFACE_v_isolines_set)
    short170 = property(_LibreDWG.Dwg_Entity_NURBSURFACE_short170_get, _LibreDWG.Dwg_Entity_NURBSURFACE_short170_set)
    cv_hull_display = property(_LibreDWG.Dwg_Entity_NURBSURFACE_cv_hull_display_get, _LibreDWG.Dwg_Entity_NURBSURFACE_cv_hull_display_set)
    uvec1 = property(_LibreDWG.Dwg_Entity_NURBSURFACE_uvec1_get, _LibreDWG.Dwg_Entity_NURBSURFACE_uvec1_set)
    vvec1 = property(_LibreDWG.Dwg_Entity_NURBSURFACE_vvec1_get, _LibreDWG.Dwg_Entity_NURBSURFACE_vvec1_set)
    uvec2 = property(_LibreDWG.Dwg_Entity_NURBSURFACE_uvec2_get, _LibreDWG.Dwg_Entity_NURBSURFACE_uvec2_set)
    vvec2 = property(_LibreDWG.Dwg_Entity_NURBSURFACE_vvec2_get, _LibreDWG.Dwg_Entity_NURBSURFACE_vvec2_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_NURBSURFACE_swiginit(self, _LibreDWG.new_Dwg_Entity_NURBSURFACE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_NURBSURFACE

# Register Dwg_Entity_NURBSURFACE in _LibreDWG:
_LibreDWG.Dwg_Entity_NURBSURFACE_swigregister(Dwg_Entity_NURBSURFACE)
class Dwg_Entity_PLANESURFACE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_PLANESURFACE_parent_get, _LibreDWG.Dwg_Entity_PLANESURFACE_parent_set)
    acis_empty = property(_LibreDWG.Dwg_Entity_PLANESURFACE_acis_empty_get, _LibreDWG.Dwg_Entity_PLANESURFACE_acis_empty_set)
    unknown = property(_LibreDWG.Dwg_Entity_PLANESURFACE_unknown_get, _LibreDWG.Dwg_Entity_PLANESURFACE_unknown_set)
    version = property(_LibreDWG.Dwg_Entity_PLANESURFACE_version_get, _LibreDWG.Dwg_Entity_PLANESURFACE_version_set)
    num_blocks = property(_LibreDWG.Dwg_Entity_PLANESURFACE_num_blocks_get, _LibreDWG.Dwg_Entity_PLANESURFACE_num_blocks_set)
    block_size = property(_LibreDWG.Dwg_Entity_PLANESURFACE_block_size_get, _LibreDWG.Dwg_Entity_PLANESURFACE_block_size_set)
    encr_sat_data = property(_LibreDWG.Dwg_Entity_PLANESURFACE_encr_sat_data_get, _LibreDWG.Dwg_Entity_PLANESURFACE_encr_sat_data_set)
    sab_size = property(_LibreDWG.Dwg_Entity_PLANESURFACE_sab_size_get, _LibreDWG.Dwg_Entity_PLANESURFACE_sab_size_set)
    acis_data = property(_LibreDWG.Dwg_Entity_PLANESURFACE_acis_data_get, _LibreDWG.Dwg_Entity_PLANESURFACE_acis_data_set)
    wireframe_data_present = property(_LibreDWG.Dwg_Entity_PLANESURFACE_wireframe_data_present_get, _LibreDWG.Dwg_Entity_PLANESURFACE_wireframe_data_present_set)
    point_present = property(_LibreDWG.Dwg_Entity_PLANESURFACE_point_present_get, _LibreDWG.Dwg_Entity_PLANESURFACE_point_present_set)
    point = property(_LibreDWG.Dwg_Entity_PLANESURFACE_point_get, _LibreDWG.Dwg_Entity_PLANESURFACE_point_set)
    isolines = property(_LibreDWG.Dwg_Entity_PLANESURFACE_isolines_get, _LibreDWG.Dwg_Entity_PLANESURFACE_isolines_set)
    isoline_present = property(_LibreDWG.Dwg_Entity_PLANESURFACE_isoline_present_get, _LibreDWG.Dwg_Entity_PLANESURFACE_isoline_present_set)
    num_wires = property(_LibreDWG.Dwg_Entity_PLANESURFACE_num_wires_get, _LibreDWG.Dwg_Entity_PLANESURFACE_num_wires_set)
    wires = property(_LibreDWG.Dwg_Entity_PLANESURFACE_wires_get, _LibreDWG.Dwg_Entity_PLANESURFACE_wires_set)
    num_silhouettes = property(_LibreDWG.Dwg_Entity_PLANESURFACE_num_silhouettes_get, _LibreDWG.Dwg_Entity_PLANESURFACE_num_silhouettes_set)
    silhouettes = property(_LibreDWG.Dwg_Entity_PLANESURFACE_silhouettes_get, _LibreDWG.Dwg_Entity_PLANESURFACE_silhouettes_set)
    _dxf_sab_converted = property(_LibreDWG.Dwg_Entity_PLANESURFACE__dxf_sab_converted_get, _LibreDWG.Dwg_Entity_PLANESURFACE__dxf_sab_converted_set)
    acis_empty2 = property(_LibreDWG.Dwg_Entity_PLANESURFACE_acis_empty2_get, _LibreDWG.Dwg_Entity_PLANESURFACE_acis_empty2_set)
    extra_acis_data = property(_LibreDWG.Dwg_Entity_PLANESURFACE_extra_acis_data_get, _LibreDWG.Dwg_Entity_PLANESURFACE_extra_acis_data_set)
    num_materials = property(_LibreDWG.Dwg_Entity_PLANESURFACE_num_materials_get, _LibreDWG.Dwg_Entity_PLANESURFACE_num_materials_set)
    materials = property(_LibreDWG.Dwg_Entity_PLANESURFACE_materials_get, _LibreDWG.Dwg_Entity_PLANESURFACE_materials_set)
    revision_guid = property(_LibreDWG.Dwg_Entity_PLANESURFACE_revision_guid_get, _LibreDWG.Dwg_Entity_PLANESURFACE_revision_guid_set)
    revision_major = property(_LibreDWG.Dwg_Entity_PLANESURFACE_revision_major_get, _LibreDWG.Dwg_Entity_PLANESURFACE_revision_major_set)
    revision_minor1 = property(_LibreDWG.Dwg_Entity_PLANESURFACE_revision_minor1_get, _LibreDWG.Dwg_Entity_PLANESURFACE_revision_minor1_set)
    revision_minor2 = property(_LibreDWG.Dwg_Entity_PLANESURFACE_revision_minor2_get, _LibreDWG.Dwg_Entity_PLANESURFACE_revision_minor2_set)
    revision_bytes = property(_LibreDWG.Dwg_Entity_PLANESURFACE_revision_bytes_get, _LibreDWG.Dwg_Entity_PLANESURFACE_revision_bytes_set)
    end_marker = property(_LibreDWG.Dwg_Entity_PLANESURFACE_end_marker_get, _LibreDWG.Dwg_Entity_PLANESURFACE_end_marker_set)
    history_id = property(_LibreDWG.Dwg_Entity_PLANESURFACE_history_id_get, _LibreDWG.Dwg_Entity_PLANESURFACE_history_id_set)
    has_revision_guid = property(_LibreDWG.Dwg_Entity_PLANESURFACE_has_revision_guid_get, _LibreDWG.Dwg_Entity_PLANESURFACE_has_revision_guid_set)
    acis_empty_bit = property(_LibreDWG.Dwg_Entity_PLANESURFACE_acis_empty_bit_get, _LibreDWG.Dwg_Entity_PLANESURFACE_acis_empty_bit_set)
    modeler_format_version = property(_LibreDWG.Dwg_Entity_PLANESURFACE_modeler_format_version_get, _LibreDWG.Dwg_Entity_PLANESURFACE_modeler_format_version_set)
    u_isolines = property(_LibreDWG.Dwg_Entity_PLANESURFACE_u_isolines_get, _LibreDWG.Dwg_Entity_PLANESURFACE_u_isolines_set)
    v_isolines = property(_LibreDWG.Dwg_Entity_PLANESURFACE_v_isolines_get, _LibreDWG.Dwg_Entity_PLANESURFACE_v_isolines_set)
    class_version = property(_LibreDWG.Dwg_Entity_PLANESURFACE_class_version_get, _LibreDWG.Dwg_Entity_PLANESURFACE_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_PLANESURFACE_swiginit(self, _LibreDWG.new_Dwg_Entity_PLANESURFACE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_PLANESURFACE

# Register Dwg_Entity_PLANESURFACE in _LibreDWG:
_LibreDWG.Dwg_Entity_PLANESURFACE_swigregister(Dwg_Entity_PLANESURFACE)
class Dwg_Entity_REVOLVEDSURFACE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_parent_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_parent_set)
    acis_empty = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_acis_empty_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_acis_empty_set)
    unknown = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_unknown_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_unknown_set)
    version = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_version_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_version_set)
    num_blocks = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_num_blocks_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_num_blocks_set)
    block_size = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_block_size_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_block_size_set)
    encr_sat_data = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_set)
    sab_size = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_sab_size_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_sab_size_set)
    acis_data = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_acis_data_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_acis_data_set)
    wireframe_data_present = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_set)
    point_present = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_point_present_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_point_present_set)
    point = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_point_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_point_set)
    isolines = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_isolines_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_isolines_set)
    isoline_present = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_isoline_present_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_isoline_present_set)
    num_wires = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_num_wires_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_num_wires_set)
    wires = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_wires_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_wires_set)
    num_silhouettes = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_set)
    silhouettes = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_silhouettes_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_silhouettes_set)
    _dxf_sab_converted = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_set)
    acis_empty2 = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_acis_empty2_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_acis_empty2_set)
    extra_acis_data = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_set)
    num_materials = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_num_materials_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_num_materials_set)
    materials = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_materials_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_materials_set)
    revision_guid = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revision_guid_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revision_guid_set)
    revision_major = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revision_major_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revision_major_set)
    revision_minor1 = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revision_minor1_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revision_minor1_set)
    revision_minor2 = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revision_minor2_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revision_minor2_set)
    revision_bytes = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revision_bytes_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revision_bytes_set)
    end_marker = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_end_marker_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_end_marker_set)
    history_id = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_history_id_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_history_id_set)
    has_revision_guid = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_set)
    acis_empty_bit = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_set)
    modeler_format_version = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_set)
    u_isolines = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_u_isolines_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_u_isolines_set)
    v_isolines = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_v_isolines_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_v_isolines_set)
    class_version = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_class_version_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_class_version_set)
    id = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_id_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_id_set)
    axis_point = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_axis_point_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_axis_point_set)
    axis_vector = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_axis_vector_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_axis_vector_set)
    revolve_angle = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revolve_angle_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revolve_angle_set)
    start_angle = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_start_angle_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_start_angle_set)
    revolved_entity_transmatrix = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_set)
    draft_angle = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_draft_angle_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_draft_angle_set)
    draft_start_distance = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_set)
    draft_end_distance = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_set)
    twist_angle = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_twist_angle_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_twist_angle_set)
    solid = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_solid_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_solid_set)
    close_to_axis = property(_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_close_to_axis_get, _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_close_to_axis_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_REVOLVEDSURFACE_swiginit(self, _LibreDWG.new_Dwg_Entity_REVOLVEDSURFACE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_REVOLVEDSURFACE

# Register Dwg_Entity_REVOLVEDSURFACE in _LibreDWG:
_LibreDWG.Dwg_Entity_REVOLVEDSURFACE_swigregister(Dwg_Entity_REVOLVEDSURFACE)
class Dwg_MESH_edge(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_MESH_edge_parent_get, _LibreDWG.Dwg_MESH_edge_parent_set)
    idxfrom = property(_LibreDWG.Dwg_MESH_edge_idxfrom_get, _LibreDWG.Dwg_MESH_edge_idxfrom_set)
    idxto = property(_LibreDWG.Dwg_MESH_edge_idxto_get, _LibreDWG.Dwg_MESH_edge_idxto_set)

    def __init__(self):
        _LibreDWG.Dwg_MESH_edge_swiginit(self, _LibreDWG.new_Dwg_MESH_edge())
    __swig_destroy__ = _LibreDWG.delete_Dwg_MESH_edge

# Register Dwg_MESH_edge in _LibreDWG:
_LibreDWG.Dwg_MESH_edge_swigregister(Dwg_MESH_edge)
class Dwg_Entity_MESH(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_MESH_parent_get, _LibreDWG.Dwg_Entity_MESH_parent_set)
    dlevel = property(_LibreDWG.Dwg_Entity_MESH_dlevel_get, _LibreDWG.Dwg_Entity_MESH_dlevel_set)
    is_watertight = property(_LibreDWG.Dwg_Entity_MESH_is_watertight_get, _LibreDWG.Dwg_Entity_MESH_is_watertight_set)
    num_subdiv_vertex = property(_LibreDWG.Dwg_Entity_MESH_num_subdiv_vertex_get, _LibreDWG.Dwg_Entity_MESH_num_subdiv_vertex_set)
    subdiv_vertex = property(_LibreDWG.Dwg_Entity_MESH_subdiv_vertex_get, _LibreDWG.Dwg_Entity_MESH_subdiv_vertex_set)
    num_vertex = property(_LibreDWG.Dwg_Entity_MESH_num_vertex_get, _LibreDWG.Dwg_Entity_MESH_num_vertex_set)
    vertex = property(_LibreDWG.Dwg_Entity_MESH_vertex_get, _LibreDWG.Dwg_Entity_MESH_vertex_set)
    num_faces = property(_LibreDWG.Dwg_Entity_MESH_num_faces_get, _LibreDWG.Dwg_Entity_MESH_num_faces_set)
    faces = property(_LibreDWG.Dwg_Entity_MESH_faces_get, _LibreDWG.Dwg_Entity_MESH_faces_set)
    num_edges = property(_LibreDWG.Dwg_Entity_MESH_num_edges_get, _LibreDWG.Dwg_Entity_MESH_num_edges_set)
    edges = property(_LibreDWG.Dwg_Entity_MESH_edges_get, _LibreDWG.Dwg_Entity_MESH_edges_set)
    num_crease = property(_LibreDWG.Dwg_Entity_MESH_num_crease_get, _LibreDWG.Dwg_Entity_MESH_num_crease_set)
    crease = property(_LibreDWG.Dwg_Entity_MESH_crease_get, _LibreDWG.Dwg_Entity_MESH_crease_set)
    unknown_b1 = property(_LibreDWG.Dwg_Entity_MESH_unknown_b1_get, _LibreDWG.Dwg_Entity_MESH_unknown_b1_set)
    unknown_b2 = property(_LibreDWG.Dwg_Entity_MESH_unknown_b2_get, _LibreDWG.Dwg_Entity_MESH_unknown_b2_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_MESH_swiginit(self, _LibreDWG.new_Dwg_Entity_MESH())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_MESH

# Register Dwg_Entity_MESH in _LibreDWG:
_LibreDWG.Dwg_Entity_MESH_swigregister(Dwg_Entity_MESH)
class Dwg_Object_SUN(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_SUN_parent_get, _LibreDWG.Dwg_Object_SUN_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_SUN_class_version_get, _LibreDWG.Dwg_Object_SUN_class_version_set)
    is_on = property(_LibreDWG.Dwg_Object_SUN_is_on_get, _LibreDWG.Dwg_Object_SUN_is_on_set)
    color = property(_LibreDWG.Dwg_Object_SUN_color_get, _LibreDWG.Dwg_Object_SUN_color_set)
    intensity = property(_LibreDWG.Dwg_Object_SUN_intensity_get, _LibreDWG.Dwg_Object_SUN_intensity_set)
    has_shadow = property(_LibreDWG.Dwg_Object_SUN_has_shadow_get, _LibreDWG.Dwg_Object_SUN_has_shadow_set)
    julian_day = property(_LibreDWG.Dwg_Object_SUN_julian_day_get, _LibreDWG.Dwg_Object_SUN_julian_day_set)
    msecs = property(_LibreDWG.Dwg_Object_SUN_msecs_get, _LibreDWG.Dwg_Object_SUN_msecs_set)
    is_dst = property(_LibreDWG.Dwg_Object_SUN_is_dst_get, _LibreDWG.Dwg_Object_SUN_is_dst_set)
    shadow_type = property(_LibreDWG.Dwg_Object_SUN_shadow_type_get, _LibreDWG.Dwg_Object_SUN_shadow_type_set)
    shadow_mapsize = property(_LibreDWG.Dwg_Object_SUN_shadow_mapsize_get, _LibreDWG.Dwg_Object_SUN_shadow_mapsize_set)
    shadow_softness = property(_LibreDWG.Dwg_Object_SUN_shadow_softness_get, _LibreDWG.Dwg_Object_SUN_shadow_softness_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_SUN_swiginit(self, _LibreDWG.new_Dwg_Object_SUN())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_SUN

# Register Dwg_Object_SUN in _LibreDWG:
_LibreDWG.Dwg_Object_SUN_swigregister(Dwg_Object_SUN)
class Dwg_SUNSTUDY_Dates(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    julian_day = property(_LibreDWG.Dwg_SUNSTUDY_Dates_julian_day_get, _LibreDWG.Dwg_SUNSTUDY_Dates_julian_day_set)
    msecs = property(_LibreDWG.Dwg_SUNSTUDY_Dates_msecs_get, _LibreDWG.Dwg_SUNSTUDY_Dates_msecs_set)

    def __init__(self):
        _LibreDWG.Dwg_SUNSTUDY_Dates_swiginit(self, _LibreDWG.new_Dwg_SUNSTUDY_Dates())
    __swig_destroy__ = _LibreDWG.delete_Dwg_SUNSTUDY_Dates

# Register Dwg_SUNSTUDY_Dates in _LibreDWG:
_LibreDWG.Dwg_SUNSTUDY_Dates_swigregister(Dwg_SUNSTUDY_Dates)
class Dwg_Object_SUNSTUDY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_SUNSTUDY_parent_get, _LibreDWG.Dwg_Object_SUNSTUDY_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_SUNSTUDY_class_version_get, _LibreDWG.Dwg_Object_SUNSTUDY_class_version_set)
    setup_name = property(_LibreDWG.Dwg_Object_SUNSTUDY_setup_name_get, _LibreDWG.Dwg_Object_SUNSTUDY_setup_name_set)
    description = property(_LibreDWG.Dwg_Object_SUNSTUDY_description_get, _LibreDWG.Dwg_Object_SUNSTUDY_description_set)
    output_type = property(_LibreDWG.Dwg_Object_SUNSTUDY_output_type_get, _LibreDWG.Dwg_Object_SUNSTUDY_output_type_set)
    sheet_set_name = property(_LibreDWG.Dwg_Object_SUNSTUDY_sheet_set_name_get, _LibreDWG.Dwg_Object_SUNSTUDY_sheet_set_name_set)
    use_subset = property(_LibreDWG.Dwg_Object_SUNSTUDY_use_subset_get, _LibreDWG.Dwg_Object_SUNSTUDY_use_subset_set)
    sheet_subset_name = property(_LibreDWG.Dwg_Object_SUNSTUDY_sheet_subset_name_get, _LibreDWG.Dwg_Object_SUNSTUDY_sheet_subset_name_set)
    select_dates_from_calendar = property(_LibreDWG.Dwg_Object_SUNSTUDY_select_dates_from_calendar_get, _LibreDWG.Dwg_Object_SUNSTUDY_select_dates_from_calendar_set)
    num_dates = property(_LibreDWG.Dwg_Object_SUNSTUDY_num_dates_get, _LibreDWG.Dwg_Object_SUNSTUDY_num_dates_set)
    dates = property(_LibreDWG.Dwg_Object_SUNSTUDY_dates_get, _LibreDWG.Dwg_Object_SUNSTUDY_dates_set)
    select_range_of_dates = property(_LibreDWG.Dwg_Object_SUNSTUDY_select_range_of_dates_get, _LibreDWG.Dwg_Object_SUNSTUDY_select_range_of_dates_set)
    start_time = property(_LibreDWG.Dwg_Object_SUNSTUDY_start_time_get, _LibreDWG.Dwg_Object_SUNSTUDY_start_time_set)
    end_time = property(_LibreDWG.Dwg_Object_SUNSTUDY_end_time_get, _LibreDWG.Dwg_Object_SUNSTUDY_end_time_set)
    interval = property(_LibreDWG.Dwg_Object_SUNSTUDY_interval_get, _LibreDWG.Dwg_Object_SUNSTUDY_interval_set)
    num_hours = property(_LibreDWG.Dwg_Object_SUNSTUDY_num_hours_get, _LibreDWG.Dwg_Object_SUNSTUDY_num_hours_set)
    hours = property(_LibreDWG.Dwg_Object_SUNSTUDY_hours_get, _LibreDWG.Dwg_Object_SUNSTUDY_hours_set)
    shade_plot_type = property(_LibreDWG.Dwg_Object_SUNSTUDY_shade_plot_type_get, _LibreDWG.Dwg_Object_SUNSTUDY_shade_plot_type_set)
    numvports = property(_LibreDWG.Dwg_Object_SUNSTUDY_numvports_get, _LibreDWG.Dwg_Object_SUNSTUDY_numvports_set)
    numrows = property(_LibreDWG.Dwg_Object_SUNSTUDY_numrows_get, _LibreDWG.Dwg_Object_SUNSTUDY_numrows_set)
    numcols = property(_LibreDWG.Dwg_Object_SUNSTUDY_numcols_get, _LibreDWG.Dwg_Object_SUNSTUDY_numcols_set)
    spacing = property(_LibreDWG.Dwg_Object_SUNSTUDY_spacing_get, _LibreDWG.Dwg_Object_SUNSTUDY_spacing_set)
    lock_viewports = property(_LibreDWG.Dwg_Object_SUNSTUDY_lock_viewports_get, _LibreDWG.Dwg_Object_SUNSTUDY_lock_viewports_set)
    label_viewports = property(_LibreDWG.Dwg_Object_SUNSTUDY_label_viewports_get, _LibreDWG.Dwg_Object_SUNSTUDY_label_viewports_set)
    page_setup_wizard = property(_LibreDWG.Dwg_Object_SUNSTUDY_page_setup_wizard_get, _LibreDWG.Dwg_Object_SUNSTUDY_page_setup_wizard_set)
    view = property(_LibreDWG.Dwg_Object_SUNSTUDY_view_get, _LibreDWG.Dwg_Object_SUNSTUDY_view_set)
    visualstyle = property(_LibreDWG.Dwg_Object_SUNSTUDY_visualstyle_get, _LibreDWG.Dwg_Object_SUNSTUDY_visualstyle_set)
    text_style = property(_LibreDWG.Dwg_Object_SUNSTUDY_text_style_get, _LibreDWG.Dwg_Object_SUNSTUDY_text_style_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_SUNSTUDY_swiginit(self, _LibreDWG.new_Dwg_Object_SUNSTUDY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_SUNSTUDY

# Register Dwg_Object_SUNSTUDY in _LibreDWG:
_LibreDWG.Dwg_Object_SUNSTUDY_swigregister(Dwg_Object_SUNSTUDY)
class Dwg_DATATABLE_row(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_DATATABLE_row_parent_get, _LibreDWG.Dwg_DATATABLE_row_parent_set)
    value = property(_LibreDWG.Dwg_DATATABLE_row_value_get, _LibreDWG.Dwg_DATATABLE_row_value_set)

    def __init__(self):
        _LibreDWG.Dwg_DATATABLE_row_swiginit(self, _LibreDWG.new_Dwg_DATATABLE_row())
    __swig_destroy__ = _LibreDWG.delete_Dwg_DATATABLE_row

# Register Dwg_DATATABLE_row in _LibreDWG:
_LibreDWG.Dwg_DATATABLE_row_swigregister(Dwg_DATATABLE_row)
class Dwg_DATATABLE_column(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_DATATABLE_column_parent_get, _LibreDWG.Dwg_DATATABLE_column_parent_set)
    type = property(_LibreDWG.Dwg_DATATABLE_column_type_get, _LibreDWG.Dwg_DATATABLE_column_type_set)
    text = property(_LibreDWG.Dwg_DATATABLE_column_text_get, _LibreDWG.Dwg_DATATABLE_column_text_set)
    rows = property(_LibreDWG.Dwg_DATATABLE_column_rows_get, _LibreDWG.Dwg_DATATABLE_column_rows_set)

    def __init__(self):
        _LibreDWG.Dwg_DATATABLE_column_swiginit(self, _LibreDWG.new_Dwg_DATATABLE_column())
    __swig_destroy__ = _LibreDWG.delete_Dwg_DATATABLE_column

# Register Dwg_DATATABLE_column in _LibreDWG:
_LibreDWG.Dwg_DATATABLE_column_swigregister(Dwg_DATATABLE_column)
class Dwg_Object_DATATABLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DATATABLE_parent_get, _LibreDWG.Dwg_Object_DATATABLE_parent_set)
    flags = property(_LibreDWG.Dwg_Object_DATATABLE_flags_get, _LibreDWG.Dwg_Object_DATATABLE_flags_set)
    num_cols = property(_LibreDWG.Dwg_Object_DATATABLE_num_cols_get, _LibreDWG.Dwg_Object_DATATABLE_num_cols_set)
    num_rows = property(_LibreDWG.Dwg_Object_DATATABLE_num_rows_get, _LibreDWG.Dwg_Object_DATATABLE_num_rows_set)
    table_name = property(_LibreDWG.Dwg_Object_DATATABLE_table_name_get, _LibreDWG.Dwg_Object_DATATABLE_table_name_set)
    cols = property(_LibreDWG.Dwg_Object_DATATABLE_cols_get, _LibreDWG.Dwg_Object_DATATABLE_cols_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DATATABLE_swiginit(self, _LibreDWG.new_Dwg_Object_DATATABLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DATATABLE

# Register Dwg_Object_DATATABLE in _LibreDWG:
_LibreDWG.Dwg_Object_DATATABLE_swigregister(Dwg_Object_DATATABLE)
class Dwg_DATALINK_customdata(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_DATALINK_customdata_parent_get, _LibreDWG.Dwg_DATALINK_customdata_parent_set)
    target = property(_LibreDWG.Dwg_DATALINK_customdata_target_get, _LibreDWG.Dwg_DATALINK_customdata_target_set)
    text = property(_LibreDWG.Dwg_DATALINK_customdata_text_get, _LibreDWG.Dwg_DATALINK_customdata_text_set)

    def __init__(self):
        _LibreDWG.Dwg_DATALINK_customdata_swiginit(self, _LibreDWG.new_Dwg_DATALINK_customdata())
    __swig_destroy__ = _LibreDWG.delete_Dwg_DATALINK_customdata

# Register Dwg_DATALINK_customdata in _LibreDWG:
_LibreDWG.Dwg_DATALINK_customdata_swigregister(Dwg_DATALINK_customdata)
class Dwg_Object_DATALINK(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DATALINK_parent_get, _LibreDWG.Dwg_Object_DATALINK_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_DATALINK_class_version_get, _LibreDWG.Dwg_Object_DATALINK_class_version_set)
    data_adapter = property(_LibreDWG.Dwg_Object_DATALINK_data_adapter_get, _LibreDWG.Dwg_Object_DATALINK_data_adapter_set)
    description = property(_LibreDWG.Dwg_Object_DATALINK_description_get, _LibreDWG.Dwg_Object_DATALINK_description_set)
    tooltip = property(_LibreDWG.Dwg_Object_DATALINK_tooltip_get, _LibreDWG.Dwg_Object_DATALINK_tooltip_set)
    connection_string = property(_LibreDWG.Dwg_Object_DATALINK_connection_string_get, _LibreDWG.Dwg_Object_DATALINK_connection_string_set)
    option = property(_LibreDWG.Dwg_Object_DATALINK_option_get, _LibreDWG.Dwg_Object_DATALINK_option_set)
    update_option = property(_LibreDWG.Dwg_Object_DATALINK_update_option_get, _LibreDWG.Dwg_Object_DATALINK_update_option_set)
    bl92 = property(_LibreDWG.Dwg_Object_DATALINK_bl92_get, _LibreDWG.Dwg_Object_DATALINK_bl92_set)
    year = property(_LibreDWG.Dwg_Object_DATALINK_year_get, _LibreDWG.Dwg_Object_DATALINK_year_set)
    month = property(_LibreDWG.Dwg_Object_DATALINK_month_get, _LibreDWG.Dwg_Object_DATALINK_month_set)
    day = property(_LibreDWG.Dwg_Object_DATALINK_day_get, _LibreDWG.Dwg_Object_DATALINK_day_set)
    hour = property(_LibreDWG.Dwg_Object_DATALINK_hour_get, _LibreDWG.Dwg_Object_DATALINK_hour_set)
    minute = property(_LibreDWG.Dwg_Object_DATALINK_minute_get, _LibreDWG.Dwg_Object_DATALINK_minute_set)
    seconds = property(_LibreDWG.Dwg_Object_DATALINK_seconds_get, _LibreDWG.Dwg_Object_DATALINK_seconds_set)
    msec = property(_LibreDWG.Dwg_Object_DATALINK_msec_get, _LibreDWG.Dwg_Object_DATALINK_msec_set)
    path_option = property(_LibreDWG.Dwg_Object_DATALINK_path_option_get, _LibreDWG.Dwg_Object_DATALINK_path_option_set)
    bl93 = property(_LibreDWG.Dwg_Object_DATALINK_bl93_get, _LibreDWG.Dwg_Object_DATALINK_bl93_set)
    update_status = property(_LibreDWG.Dwg_Object_DATALINK_update_status_get, _LibreDWG.Dwg_Object_DATALINK_update_status_set)
    num_customdata = property(_LibreDWG.Dwg_Object_DATALINK_num_customdata_get, _LibreDWG.Dwg_Object_DATALINK_num_customdata_set)
    customdata = property(_LibreDWG.Dwg_Object_DATALINK_customdata_get, _LibreDWG.Dwg_Object_DATALINK_customdata_set)
    hardowner = property(_LibreDWG.Dwg_Object_DATALINK_hardowner_get, _LibreDWG.Dwg_Object_DATALINK_hardowner_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DATALINK_swiginit(self, _LibreDWG.new_Dwg_Object_DATALINK())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DATALINK

# Register Dwg_Object_DATALINK in _LibreDWG:
_LibreDWG.Dwg_Object_DATALINK_swigregister(Dwg_Object_DATALINK)
class Dwg_DIMASSOC_Ref(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_DIMASSOC_Ref_parent_get, _LibreDWG.Dwg_DIMASSOC_Ref_parent_set)
    classname = property(_LibreDWG.Dwg_DIMASSOC_Ref_classname_get, _LibreDWG.Dwg_DIMASSOC_Ref_classname_set)
    osnap_type = property(_LibreDWG.Dwg_DIMASSOC_Ref_osnap_type_get, _LibreDWG.Dwg_DIMASSOC_Ref_osnap_type_set)
    osnap_dist = property(_LibreDWG.Dwg_DIMASSOC_Ref_osnap_dist_get, _LibreDWG.Dwg_DIMASSOC_Ref_osnap_dist_set)
    osnap_pt = property(_LibreDWG.Dwg_DIMASSOC_Ref_osnap_pt_get, _LibreDWG.Dwg_DIMASSOC_Ref_osnap_pt_set)
    num_xrefs = property(_LibreDWG.Dwg_DIMASSOC_Ref_num_xrefs_get, _LibreDWG.Dwg_DIMASSOC_Ref_num_xrefs_set)
    xrefs = property(_LibreDWG.Dwg_DIMASSOC_Ref_xrefs_get, _LibreDWG.Dwg_DIMASSOC_Ref_xrefs_set)
    main_subent_type = property(_LibreDWG.Dwg_DIMASSOC_Ref_main_subent_type_get, _LibreDWG.Dwg_DIMASSOC_Ref_main_subent_type_set)
    main_gsmarker = property(_LibreDWG.Dwg_DIMASSOC_Ref_main_gsmarker_get, _LibreDWG.Dwg_DIMASSOC_Ref_main_gsmarker_set)
    num_xrefpaths = property(_LibreDWG.Dwg_DIMASSOC_Ref_num_xrefpaths_get, _LibreDWG.Dwg_DIMASSOC_Ref_num_xrefpaths_set)
    xrefpaths = property(_LibreDWG.Dwg_DIMASSOC_Ref_xrefpaths_get, _LibreDWG.Dwg_DIMASSOC_Ref_xrefpaths_set)
    num_intersec_xrefpaths = property(_LibreDWG.Dwg_DIMASSOC_Ref_num_intersec_xrefpaths_get, _LibreDWG.Dwg_DIMASSOC_Ref_num_intersec_xrefpaths_set)
    intersec_xrefpaths = property(_LibreDWG.Dwg_DIMASSOC_Ref_intersec_xrefpaths_get, _LibreDWG.Dwg_DIMASSOC_Ref_intersec_xrefpaths_set)
    has_lastpt_ref = property(_LibreDWG.Dwg_DIMASSOC_Ref_has_lastpt_ref_get, _LibreDWG.Dwg_DIMASSOC_Ref_has_lastpt_ref_set)
    intersec_subent_type = property(_LibreDWG.Dwg_DIMASSOC_Ref_intersec_subent_type_get, _LibreDWG.Dwg_DIMASSOC_Ref_intersec_subent_type_set)
    intersec_gsmarker = property(_LibreDWG.Dwg_DIMASSOC_Ref_intersec_gsmarker_get, _LibreDWG.Dwg_DIMASSOC_Ref_intersec_gsmarker_set)
    num_intsectobj = property(_LibreDWG.Dwg_DIMASSOC_Ref_num_intsectobj_get, _LibreDWG.Dwg_DIMASSOC_Ref_num_intsectobj_set)
    intsectobj = property(_LibreDWG.Dwg_DIMASSOC_Ref_intsectobj_get, _LibreDWG.Dwg_DIMASSOC_Ref_intsectobj_set)

    def __init__(self):
        _LibreDWG.Dwg_DIMASSOC_Ref_swiginit(self, _LibreDWG.new_Dwg_DIMASSOC_Ref())
    __swig_destroy__ = _LibreDWG.delete_Dwg_DIMASSOC_Ref

# Register Dwg_DIMASSOC_Ref in _LibreDWG:
_LibreDWG.Dwg_DIMASSOC_Ref_swigregister(Dwg_DIMASSOC_Ref)
class Dwg_Object_DIMASSOC(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DIMASSOC_parent_get, _LibreDWG.Dwg_Object_DIMASSOC_parent_set)
    dimensionobj = property(_LibreDWG.Dwg_Object_DIMASSOC_dimensionobj_get, _LibreDWG.Dwg_Object_DIMASSOC_dimensionobj_set)
    associativity = property(_LibreDWG.Dwg_Object_DIMASSOC_associativity_get, _LibreDWG.Dwg_Object_DIMASSOC_associativity_set)
    trans_space_flag = property(_LibreDWG.Dwg_Object_DIMASSOC_trans_space_flag_get, _LibreDWG.Dwg_Object_DIMASSOC_trans_space_flag_set)
    rotated_type = property(_LibreDWG.Dwg_Object_DIMASSOC_rotated_type_get, _LibreDWG.Dwg_Object_DIMASSOC_rotated_type_set)
    has_lastpt_ref = property(_LibreDWG.Dwg_Object_DIMASSOC_has_lastpt_ref_get, _LibreDWG.Dwg_Object_DIMASSOC_has_lastpt_ref_set)
    ref = property(_LibreDWG.Dwg_Object_DIMASSOC_ref_get, _LibreDWG.Dwg_Object_DIMASSOC_ref_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DIMASSOC_swiginit(self, _LibreDWG.new_Dwg_Object_DIMASSOC())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DIMASSOC

# Register Dwg_Object_DIMASSOC in _LibreDWG:
_LibreDWG.Dwg_Object_DIMASSOC_swigregister(Dwg_Object_DIMASSOC)
class Dwg_ACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_ACTIONBODY_parent_get, _LibreDWG.Dwg_ACTIONBODY_parent_set)
    evaluatorid = property(_LibreDWG.Dwg_ACTIONBODY_evaluatorid_get, _LibreDWG.Dwg_ACTIONBODY_evaluatorid_set)
    expression = property(_LibreDWG.Dwg_ACTIONBODY_expression_get, _LibreDWG.Dwg_ACTIONBODY_expression_set)
    value = property(_LibreDWG.Dwg_ACTIONBODY_value_get, _LibreDWG.Dwg_ACTIONBODY_value_set)

    def __init__(self):
        _LibreDWG.Dwg_ACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_ACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_ACTIONBODY

# Register Dwg_ACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_ACTIONBODY_swigregister(Dwg_ACTIONBODY)
class Dwg_EvalVariant(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    code = property(_LibreDWG.Dwg_EvalVariant_code_get, _LibreDWG.Dwg_EvalVariant_code_set)
    u = property(_LibreDWG.Dwg_EvalVariant_u_get)

    def __init__(self):
        _LibreDWG.Dwg_EvalVariant_swiginit(self, _LibreDWG.new_Dwg_EvalVariant())
    __swig_destroy__ = _LibreDWG.delete_Dwg_EvalVariant

# Register Dwg_EvalVariant in _LibreDWG:
_LibreDWG.Dwg_EvalVariant_swigregister(Dwg_EvalVariant)
class _dwg_EvalVariant_u(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    bd = property(_LibreDWG._dwg_EvalVariant_u_bd_get, _LibreDWG._dwg_EvalVariant_u_bd_set)
    bl = property(_LibreDWG._dwg_EvalVariant_u_bl_get, _LibreDWG._dwg_EvalVariant_u_bl_set)
    bs = property(_LibreDWG._dwg_EvalVariant_u_bs_get, _LibreDWG._dwg_EvalVariant_u_bs_set)
    rc = property(_LibreDWG._dwg_EvalVariant_u_rc_get, _LibreDWG._dwg_EvalVariant_u_rc_set)
    text = property(_LibreDWG._dwg_EvalVariant_u_text_get, _LibreDWG._dwg_EvalVariant_u_text_set)
    handle = property(_LibreDWG._dwg_EvalVariant_u_handle_get, _LibreDWG._dwg_EvalVariant_u_handle_set)

    def __init__(self):
        _LibreDWG._dwg_EvalVariant_u_swiginit(self, _LibreDWG.new__dwg_EvalVariant_u())
    __swig_destroy__ = _LibreDWG.delete__dwg_EvalVariant_u

# Register _dwg_EvalVariant_u in _LibreDWG:
_LibreDWG._dwg_EvalVariant_u_swigregister(_dwg_EvalVariant_u)
class Dwg_VALUEPARAM_vars(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    value = property(_LibreDWG.Dwg_VALUEPARAM_vars_value_get, _LibreDWG.Dwg_VALUEPARAM_vars_value_set)
    handle = property(_LibreDWG.Dwg_VALUEPARAM_vars_handle_get, _LibreDWG.Dwg_VALUEPARAM_vars_handle_set)

    def __init__(self):
        _LibreDWG.Dwg_VALUEPARAM_vars_swiginit(self, _LibreDWG.new_Dwg_VALUEPARAM_vars())
    __swig_destroy__ = _LibreDWG.delete_Dwg_VALUEPARAM_vars

# Register Dwg_VALUEPARAM_vars in _LibreDWG:
_LibreDWG.Dwg_VALUEPARAM_vars_swigregister(Dwg_VALUEPARAM_vars)
class Dwg_VALUEPARAM(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_VALUEPARAM_parent_get, _LibreDWG.Dwg_VALUEPARAM_parent_set)
    class_version = property(_LibreDWG.Dwg_VALUEPARAM_class_version_get, _LibreDWG.Dwg_VALUEPARAM_class_version_set)
    name = property(_LibreDWG.Dwg_VALUEPARAM_name_get, _LibreDWG.Dwg_VALUEPARAM_name_set)
    unit_type = property(_LibreDWG.Dwg_VALUEPARAM_unit_type_get, _LibreDWG.Dwg_VALUEPARAM_unit_type_set)
    num_vars = property(_LibreDWG.Dwg_VALUEPARAM_num_vars_get, _LibreDWG.Dwg_VALUEPARAM_num_vars_set)
    vars = property(_LibreDWG.Dwg_VALUEPARAM_vars_get, _LibreDWG.Dwg_VALUEPARAM_vars_set)
    controlled_objdep = property(_LibreDWG.Dwg_VALUEPARAM_controlled_objdep_get, _LibreDWG.Dwg_VALUEPARAM_controlled_objdep_set)

    def __init__(self):
        _LibreDWG.Dwg_VALUEPARAM_swiginit(self, _LibreDWG.new_Dwg_VALUEPARAM())
    __swig_destroy__ = _LibreDWG.delete_Dwg_VALUEPARAM

# Register Dwg_VALUEPARAM in _LibreDWG:
_LibreDWG.Dwg_VALUEPARAM_swigregister(Dwg_VALUEPARAM)
class Dwg_ASSOCPARAMBASEDACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_parent_get, _LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_parent_set)
    version = property(_LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_version_get, _LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_version_set)
    minor = property(_LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_minor_get, _LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_minor_set)
    num_deps = property(_LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_get, _LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_set)
    deps = property(_LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_deps_get, _LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_deps_set)
    l4 = property(_LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_l4_get, _LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_l4_set)
    l5 = property(_LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_l5_get, _LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_l5_set)
    assocdep = property(_LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_get, _LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_set)
    num_values = property(_LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_get, _LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_set)
    values = property(_LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_values_get, _LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_values_set)

    def __init__(self):
        _LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_ASSOCPARAMBASEDACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_ASSOCPARAMBASEDACTIONBODY

# Register Dwg_ASSOCPARAMBASEDACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_ASSOCPARAMBASEDACTIONBODY_swigregister(Dwg_ASSOCPARAMBASEDACTIONBODY)
class Dwg_ASSOCACTION_Deps(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_ASSOCACTION_Deps_parent_get, _LibreDWG.Dwg_ASSOCACTION_Deps_parent_set)
    is_owned = property(_LibreDWG.Dwg_ASSOCACTION_Deps_is_owned_get, _LibreDWG.Dwg_ASSOCACTION_Deps_is_owned_set)
    dep = property(_LibreDWG.Dwg_ASSOCACTION_Deps_dep_get, _LibreDWG.Dwg_ASSOCACTION_Deps_dep_set)

    def __init__(self):
        _LibreDWG.Dwg_ASSOCACTION_Deps_swiginit(self, _LibreDWG.new_Dwg_ASSOCACTION_Deps())
    __swig_destroy__ = _LibreDWG.delete_Dwg_ASSOCACTION_Deps

# Register Dwg_ASSOCACTION_Deps in _LibreDWG:
_LibreDWG.Dwg_ASSOCACTION_Deps_swigregister(Dwg_ASSOCACTION_Deps)
class Dwg_Object_ASSOCDEPENDENCY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_parent_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_class_version_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_class_version_set)
    status = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_status_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_status_set)
    is_read_dep = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_is_read_dep_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_is_read_dep_set)
    is_write_dep = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_is_write_dep_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_is_write_dep_set)
    is_attached_to_object = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_set)
    is_delegating_to_owning_action = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_set)
    order = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_order_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_order_set)
    dep_on = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_dep_on_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_dep_on_set)
    has_name = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_has_name_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_has_name_set)
    name = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_name_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_name_set)
    depbodyid = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_depbodyid_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_depbodyid_set)
    readdep = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_readdep_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_readdep_set)
    dep_body = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_dep_body_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_dep_body_set)
    node = property(_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_node_get, _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_node_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCDEPENDENCY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCDEPENDENCY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCDEPENDENCY

# Register Dwg_Object_ASSOCDEPENDENCY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCDEPENDENCY_swigregister(Dwg_Object_ASSOCDEPENDENCY)
class Dwg_Object_ASSOCVALUEDEPENDENCY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCVALUEDEPENDENCY_parent_get, _LibreDWG.Dwg_Object_ASSOCVALUEDEPENDENCY_parent_set)
    assocdep = property(_LibreDWG.Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_get, _LibreDWG.Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCVALUEDEPENDENCY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCVALUEDEPENDENCY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCVALUEDEPENDENCY

# Register Dwg_Object_ASSOCVALUEDEPENDENCY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCVALUEDEPENDENCY_swigregister(Dwg_Object_ASSOCVALUEDEPENDENCY)
class Dwg_Object_ASSOCGEOMDEPENDENCY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_parent_get, _LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_parent_set)
    assocdep = property(_LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_get, _LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_get, _LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_set)
    enabled = property(_LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_get, _LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_set)
    classname = property(_LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_classname_get, _LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_classname_set)
    dependent_on_compound_object = property(_LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_get, _LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCGEOMDEPENDENCY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCGEOMDEPENDENCY

# Register Dwg_Object_ASSOCGEOMDEPENDENCY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCGEOMDEPENDENCY_swigregister(Dwg_Object_ASSOCGEOMDEPENDENCY)
class Dwg_Object_ASSOCACTION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCACTION_parent_get, _LibreDWG.Dwg_Object_ASSOCACTION_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCACTION_class_version_get, _LibreDWG.Dwg_Object_ASSOCACTION_class_version_set)
    geometry_status = property(_LibreDWG.Dwg_Object_ASSOCACTION_geometry_status_get, _LibreDWG.Dwg_Object_ASSOCACTION_geometry_status_set)
    owningnetwork = property(_LibreDWG.Dwg_Object_ASSOCACTION_owningnetwork_get, _LibreDWG.Dwg_Object_ASSOCACTION_owningnetwork_set)
    actionbody = property(_LibreDWG.Dwg_Object_ASSOCACTION_actionbody_get, _LibreDWG.Dwg_Object_ASSOCACTION_actionbody_set)
    action_index = property(_LibreDWG.Dwg_Object_ASSOCACTION_action_index_get, _LibreDWG.Dwg_Object_ASSOCACTION_action_index_set)
    max_assoc_dep_index = property(_LibreDWG.Dwg_Object_ASSOCACTION_max_assoc_dep_index_get, _LibreDWG.Dwg_Object_ASSOCACTION_max_assoc_dep_index_set)
    num_deps = property(_LibreDWG.Dwg_Object_ASSOCACTION_num_deps_get, _LibreDWG.Dwg_Object_ASSOCACTION_num_deps_set)
    deps = property(_LibreDWG.Dwg_Object_ASSOCACTION_deps_get, _LibreDWG.Dwg_Object_ASSOCACTION_deps_set)
    num_owned_params = property(_LibreDWG.Dwg_Object_ASSOCACTION_num_owned_params_get, _LibreDWG.Dwg_Object_ASSOCACTION_num_owned_params_set)
    owned_params = property(_LibreDWG.Dwg_Object_ASSOCACTION_owned_params_get, _LibreDWG.Dwg_Object_ASSOCACTION_owned_params_set)
    num_values = property(_LibreDWG.Dwg_Object_ASSOCACTION_num_values_get, _LibreDWG.Dwg_Object_ASSOCACTION_num_values_set)
    values = property(_LibreDWG.Dwg_Object_ASSOCACTION_values_get, _LibreDWG.Dwg_Object_ASSOCACTION_values_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCACTION_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCACTION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCACTION

# Register Dwg_Object_ASSOCACTION in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCACTION_swigregister(Dwg_Object_ASSOCACTION)
class Dwg_Object_ASSOCNETWORK(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_parent_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_class_version_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_class_version_set)
    geometry_status = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_geometry_status_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_geometry_status_set)
    owningnetwork = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_owningnetwork_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_owningnetwork_set)
    actionbody = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_actionbody_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_actionbody_set)
    action_index = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_action_index_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_action_index_set)
    max_assoc_dep_index = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_set)
    num_deps = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_num_deps_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_num_deps_set)
    deps = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_deps_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_deps_set)
    num_owned_params = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_num_owned_params_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_num_owned_params_set)
    owned_params = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_owned_params_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_owned_params_set)
    num_values = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_num_values_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_num_values_set)
    values = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_values_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_values_set)
    network_version = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_network_version_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_network_version_set)
    network_action_index = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_network_action_index_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_network_action_index_set)
    num_actions = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_num_actions_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_num_actions_set)
    actions = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_actions_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_actions_set)
    num_owned_actions = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_num_owned_actions_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_num_owned_actions_set)
    owned_actions = property(_LibreDWG.Dwg_Object_ASSOCNETWORK_owned_actions_get, _LibreDWG.Dwg_Object_ASSOCNETWORK_owned_actions_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCNETWORK_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCNETWORK())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCNETWORK

# Register Dwg_Object_ASSOCNETWORK in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCNETWORK_swigregister(Dwg_Object_ASSOCNETWORK)
class Dwg_CONSTRAINTGROUPNODE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_CONSTRAINTGROUPNODE_parent_get, _LibreDWG.Dwg_CONSTRAINTGROUPNODE_parent_set)
    nodeid = property(_LibreDWG.Dwg_CONSTRAINTGROUPNODE_nodeid_get, _LibreDWG.Dwg_CONSTRAINTGROUPNODE_nodeid_set)
    status = property(_LibreDWG.Dwg_CONSTRAINTGROUPNODE_status_get, _LibreDWG.Dwg_CONSTRAINTGROUPNODE_status_set)
    num_connections = property(_LibreDWG.Dwg_CONSTRAINTGROUPNODE_num_connections_get, _LibreDWG.Dwg_CONSTRAINTGROUPNODE_num_connections_set)
    connections = property(_LibreDWG.Dwg_CONSTRAINTGROUPNODE_connections_get, _LibreDWG.Dwg_CONSTRAINTGROUPNODE_connections_set)

    def __init__(self):
        _LibreDWG.Dwg_CONSTRAINTGROUPNODE_swiginit(self, _LibreDWG.new_Dwg_CONSTRAINTGROUPNODE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_CONSTRAINTGROUPNODE

# Register Dwg_CONSTRAINTGROUPNODE in _LibreDWG:
_LibreDWG.Dwg_CONSTRAINTGROUPNODE_swigregister(Dwg_CONSTRAINTGROUPNODE)
class Dwg_Object_ASSOC2DCONSTRAINTGROUP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_set)
    geometry_status = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_set)
    owningnetwork = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_set)
    actionbody = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_set)
    action_index = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_set)
    max_assoc_dep_index = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_set)
    num_deps = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_set)
    deps = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_set)
    num_owned_params = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_set)
    owned_params = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_set)
    num_values = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_set)
    values = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_set)
    version = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_set)
    b1 = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_set)
    workplane = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_set)
    h1 = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_set)
    num_actions = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_set)
    actions = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_set)
    num_nodes = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_set)
    nodes = property(_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_get, _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOC2DCONSTRAINTGROUP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOC2DCONSTRAINTGROUP

# Register Dwg_Object_ASSOC2DCONSTRAINTGROUP in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOC2DCONSTRAINTGROUP_swigregister(Dwg_Object_ASSOC2DCONSTRAINTGROUP)
class Dwg_Object_ASSOCVARIABLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_parent_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_parent_set)
    av_class_version = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_av_class_version_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_av_class_version_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_class_version_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_class_version_set)
    geometry_status = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_geometry_status_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_geometry_status_set)
    owningnetwork = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_owningnetwork_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_owningnetwork_set)
    actionbody = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_actionbody_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_actionbody_set)
    action_index = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_action_index_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_action_index_set)
    max_assoc_dep_index = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_set)
    num_deps = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_num_deps_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_num_deps_set)
    deps = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_deps_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_deps_set)
    num_owned_params = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_num_owned_params_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_num_owned_params_set)
    owned_params = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_owned_params_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_owned_params_set)
    num_values = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_num_values_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_num_values_set)
    values = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_values_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_values_set)
    name = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_name_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_name_set)
    t58 = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_t58_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_t58_set)
    evaluator = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_evaluator_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_evaluator_set)
    desc = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_desc_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_desc_set)
    value = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_value_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_value_set)
    has_t78 = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_has_t78_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_has_t78_set)
    t78 = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_t78_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_t78_set)
    b290 = property(_LibreDWG.Dwg_Object_ASSOCVARIABLE_b290_get, _LibreDWG.Dwg_Object_ASSOCVARIABLE_b290_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCVARIABLE_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCVARIABLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCVARIABLE

# Register Dwg_Object_ASSOCVARIABLE in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCVARIABLE_swigregister(Dwg_Object_ASSOCVARIABLE)
class Dwg_EVAL_Node(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_EVAL_Node_parent_get, _LibreDWG.Dwg_EVAL_Node_parent_set)
    id = property(_LibreDWG.Dwg_EVAL_Node_id_get, _LibreDWG.Dwg_EVAL_Node_id_set)
    edge_flags = property(_LibreDWG.Dwg_EVAL_Node_edge_flags_get, _LibreDWG.Dwg_EVAL_Node_edge_flags_set)
    nextid = property(_LibreDWG.Dwg_EVAL_Node_nextid_get, _LibreDWG.Dwg_EVAL_Node_nextid_set)
    evalexpr = property(_LibreDWG.Dwg_EVAL_Node_evalexpr_get, _LibreDWG.Dwg_EVAL_Node_evalexpr_set)
    node = property(_LibreDWG.Dwg_EVAL_Node_node_get, _LibreDWG.Dwg_EVAL_Node_node_set)
    active_cycles = property(_LibreDWG.Dwg_EVAL_Node_active_cycles_get, _LibreDWG.Dwg_EVAL_Node_active_cycles_set)

    def __init__(self):
        _LibreDWG.Dwg_EVAL_Node_swiginit(self, _LibreDWG.new_Dwg_EVAL_Node())
    __swig_destroy__ = _LibreDWG.delete_Dwg_EVAL_Node

# Register Dwg_EVAL_Node in _LibreDWG:
_LibreDWG.Dwg_EVAL_Node_swigregister(Dwg_EVAL_Node)
class Dwg_EVAL_Edge(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_EVAL_Edge_parent_get, _LibreDWG.Dwg_EVAL_Edge_parent_set)
    id = property(_LibreDWG.Dwg_EVAL_Edge_id_get, _LibreDWG.Dwg_EVAL_Edge_id_set)
    nextid = property(_LibreDWG.Dwg_EVAL_Edge_nextid_get, _LibreDWG.Dwg_EVAL_Edge_nextid_set)
    e1 = property(_LibreDWG.Dwg_EVAL_Edge_e1_get, _LibreDWG.Dwg_EVAL_Edge_e1_set)
    e2 = property(_LibreDWG.Dwg_EVAL_Edge_e2_get, _LibreDWG.Dwg_EVAL_Edge_e2_set)
    e3 = property(_LibreDWG.Dwg_EVAL_Edge_e3_get, _LibreDWG.Dwg_EVAL_Edge_e3_set)
    out_edge = property(_LibreDWG.Dwg_EVAL_Edge_out_edge_get, _LibreDWG.Dwg_EVAL_Edge_out_edge_set)

    def __init__(self):
        _LibreDWG.Dwg_EVAL_Edge_swiginit(self, _LibreDWG.new_Dwg_EVAL_Edge())
    __swig_destroy__ = _LibreDWG.delete_Dwg_EVAL_Edge

# Register Dwg_EVAL_Edge in _LibreDWG:
_LibreDWG.Dwg_EVAL_Edge_swigregister(Dwg_EVAL_Edge)
class Dwg_Object_EVALUATION_GRAPH(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_EVALUATION_GRAPH_parent_get, _LibreDWG.Dwg_Object_EVALUATION_GRAPH_parent_set)
    major = property(_LibreDWG.Dwg_Object_EVALUATION_GRAPH_major_get, _LibreDWG.Dwg_Object_EVALUATION_GRAPH_major_set)
    minor = property(_LibreDWG.Dwg_Object_EVALUATION_GRAPH_minor_get, _LibreDWG.Dwg_Object_EVALUATION_GRAPH_minor_set)
    first_nodeid = property(_LibreDWG.Dwg_Object_EVALUATION_GRAPH_first_nodeid_get, _LibreDWG.Dwg_Object_EVALUATION_GRAPH_first_nodeid_set)
    first_nodeid_copy = property(_LibreDWG.Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_get, _LibreDWG.Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_set)
    num_nodes = property(_LibreDWG.Dwg_Object_EVALUATION_GRAPH_num_nodes_get, _LibreDWG.Dwg_Object_EVALUATION_GRAPH_num_nodes_set)
    nodes = property(_LibreDWG.Dwg_Object_EVALUATION_GRAPH_nodes_get, _LibreDWG.Dwg_Object_EVALUATION_GRAPH_nodes_set)
    has_graph = property(_LibreDWG.Dwg_Object_EVALUATION_GRAPH_has_graph_get, _LibreDWG.Dwg_Object_EVALUATION_GRAPH_has_graph_set)
    num_edges = property(_LibreDWG.Dwg_Object_EVALUATION_GRAPH_num_edges_get, _LibreDWG.Dwg_Object_EVALUATION_GRAPH_num_edges_set)
    edges = property(_LibreDWG.Dwg_Object_EVALUATION_GRAPH_edges_get, _LibreDWG.Dwg_Object_EVALUATION_GRAPH_edges_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_EVALUATION_GRAPH_swiginit(self, _LibreDWG.new_Dwg_Object_EVALUATION_GRAPH())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_EVALUATION_GRAPH

# Register Dwg_Object_EVALUATION_GRAPH in _LibreDWG:
_LibreDWG.Dwg_Object_EVALUATION_GRAPH_swigregister(Dwg_Object_EVALUATION_GRAPH)
class Dwg_Object_DYNAMICBLOCKPURGEPREVENTER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_get, _LibreDWG.Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_set)
    flag = property(_LibreDWG.Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_get, _LibreDWG.Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_set)
    block = property(_LibreDWG.Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_get, _LibreDWG.Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_swiginit(self, _LibreDWG.new_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER

# Register Dwg_Object_DYNAMICBLOCKPURGEPREVENTER in _LibreDWG:
_LibreDWG.Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_swigregister(Dwg_Object_DYNAMICBLOCKPURGEPREVENTER)
class Dwg_Object_PERSUBENTMGR(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_PERSUBENTMGR_parent_get, _LibreDWG.Dwg_Object_PERSUBENTMGR_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_PERSUBENTMGR_class_version_get, _LibreDWG.Dwg_Object_PERSUBENTMGR_class_version_set)
    unknown_0 = property(_LibreDWG.Dwg_Object_PERSUBENTMGR_unknown_0_get, _LibreDWG.Dwg_Object_PERSUBENTMGR_unknown_0_set)
    unknown_2 = property(_LibreDWG.Dwg_Object_PERSUBENTMGR_unknown_2_get, _LibreDWG.Dwg_Object_PERSUBENTMGR_unknown_2_set)
    numassocsteps = property(_LibreDWG.Dwg_Object_PERSUBENTMGR_numassocsteps_get, _LibreDWG.Dwg_Object_PERSUBENTMGR_numassocsteps_set)
    numassocsubents = property(_LibreDWG.Dwg_Object_PERSUBENTMGR_numassocsubents_get, _LibreDWG.Dwg_Object_PERSUBENTMGR_numassocsubents_set)
    num_steps = property(_LibreDWG.Dwg_Object_PERSUBENTMGR_num_steps_get, _LibreDWG.Dwg_Object_PERSUBENTMGR_num_steps_set)
    steps = property(_LibreDWG.Dwg_Object_PERSUBENTMGR_steps_get, _LibreDWG.Dwg_Object_PERSUBENTMGR_steps_set)
    num_subents = property(_LibreDWG.Dwg_Object_PERSUBENTMGR_num_subents_get, _LibreDWG.Dwg_Object_PERSUBENTMGR_num_subents_set)
    subents = property(_LibreDWG.Dwg_Object_PERSUBENTMGR_subents_get, _LibreDWG.Dwg_Object_PERSUBENTMGR_subents_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_PERSUBENTMGR_swiginit(self, _LibreDWG.new_Dwg_Object_PERSUBENTMGR())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_PERSUBENTMGR

# Register Dwg_Object_PERSUBENTMGR in _LibreDWG:
_LibreDWG.Dwg_Object_PERSUBENTMGR_swigregister(Dwg_Object_PERSUBENTMGR)
class Dwg_Object_ASSOCPERSSUBENTMANAGER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_set)
    unknown_3 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_set)
    unknown_0 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_set)
    unknown_2 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_set)
    num_steps = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_set)
    num_subents = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_set)
    steps = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_set)
    subents = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_set)
    unknown_bl6 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_set)
    unknown_bl6a = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_set)
    unknown_bl7a = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_set)
    unknown_bl7 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_set)
    unknown_bl8 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_set)
    unknown_bl9 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_set)
    unknown_bl10 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_set)
    unknown_bl11 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_set)
    unknown_bl12 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_set)
    unknown_bl13 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_set)
    unknown_bl14 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_set)
    unknown_bl15 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_set)
    unknown_bl16 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_set)
    unknown_bl17 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_set)
    unknown_bl18 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_set)
    unknown_bl19 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_set)
    unknown_bl20 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_set)
    unknown_bl21 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_set)
    unknown_bl22 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_set)
    unknown_bl23 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_set)
    unknown_bl24 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_set)
    unknown_bl25 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_set)
    unknown_bl26 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_set)
    unknown_bl27 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_set)
    unknown_bl28 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_set)
    unknown_bl29 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_set)
    unknown_bl30 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_set)
    unknown_bl31 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_set)
    unknown_bl32 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_set)
    unknown_bl33 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_set)
    unknown_bl34 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_set)
    unknown_bl35 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_set)
    unknown_bl36 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_set)
    unknown_b37 = property(_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_get, _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCPERSSUBENTMANAGER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCPERSSUBENTMANAGER

# Register Dwg_Object_ASSOCPERSSUBENTMANAGER in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCPERSSUBENTMANAGER_swigregister(Dwg_Object_ASSOCPERSSUBENTMANAGER)
class Dwg_Object_ASSOCACTIONPARAM(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCACTIONPARAM_parent_get, _LibreDWG.Dwg_Object_ASSOCACTIONPARAM_parent_set)
    is_r2013 = property(_LibreDWG.Dwg_Object_ASSOCACTIONPARAM_is_r2013_get, _LibreDWG.Dwg_Object_ASSOCACTIONPARAM_is_r2013_set)
    aap_version = property(_LibreDWG.Dwg_Object_ASSOCACTIONPARAM_aap_version_get, _LibreDWG.Dwg_Object_ASSOCACTIONPARAM_aap_version_set)
    name = property(_LibreDWG.Dwg_Object_ASSOCACTIONPARAM_name_get, _LibreDWG.Dwg_Object_ASSOCACTIONPARAM_name_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCACTIONPARAM_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCACTIONPARAM())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCACTIONPARAM

# Register Dwg_Object_ASSOCACTIONPARAM in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCACTIONPARAM_swigregister(Dwg_Object_ASSOCACTIONPARAM)
class Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_set)
    is_r2013 = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_set)
    aap_version = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_set)
    name = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_set)
    bs1 = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_set)
    num_params = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_set)
    params = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_set)
    has_child_param = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_set)
    child_status = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_set)
    child_id = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_set)
    child_param = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_set)
    h330_2 = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_set)
    bl2 = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_set)
    h330_3 = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_set)
    status = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_set)
    osnap_mode = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_set)
    param = property(_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_get, _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM

# Register Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_swigregister(Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM)
class Dwg_Object_ASSOCPOINTREFACTIONPARAM(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_set)
    is_r2013 = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_set)
    aap_version = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_set)
    name = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_set)
    bs1 = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_set)
    num_params = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_set)
    params = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_set)
    has_child_param = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_set)
    child_status = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_set)
    child_id = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_set)
    child_param = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_set)
    h330_2 = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_set)
    bl2 = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_set)
    h330_3 = property(_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_get, _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCPOINTREFACTIONPARAM())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCPOINTREFACTIONPARAM

# Register Dwg_Object_ASSOCPOINTREFACTIONPARAM in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCPOINTREFACTIONPARAM_swigregister(Dwg_Object_ASSOCPOINTREFACTIONPARAM)
class Dwg_Object_ASSOCASMBODYACTIONPARAM(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_set)
    is_r2013 = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_set)
    aap_version = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_set)
    name = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_name_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_name_set)
    asdap_class_version = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_set)
    dep = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_set)
    acis_empty = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_set)
    unknown = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_set)
    version = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_version_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_version_set)
    num_blocks = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_set)
    block_size = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_set)
    encr_sat_data = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_set)
    sab_size = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_set)
    acis_data = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_set)
    wireframe_data_present = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_set)
    point_present = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_set)
    point = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_point_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_point_set)
    isolines = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_set)
    isoline_present = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_set)
    num_wires = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_set)
    wires = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_set)
    num_silhouettes = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_set)
    silhouettes = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_set)
    _dxf_sab_converted = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_set)
    acis_empty2 = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_set)
    extra_acis_data = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_set)
    num_materials = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_set)
    materials = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_set)
    revision_guid = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_set)
    revision_major = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_set)
    revision_minor1 = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_set)
    revision_minor2 = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_set)
    revision_bytes = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_set)
    end_marker = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_set)
    history_id = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_set)
    has_revision_guid = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_set)
    acis_empty_bit = property(_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_get, _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCASMBODYACTIONPARAM())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCASMBODYACTIONPARAM

# Register Dwg_Object_ASSOCASMBODYACTIONPARAM in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCASMBODYACTIONPARAM_swigregister(Dwg_Object_ASSOCASMBODYACTIONPARAM)
class Dwg_Object_ASSOCCOMPOUNDACTIONPARAM(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_set)
    is_r2013 = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_set)
    aap_version = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_set)
    name = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_set)
    bs1 = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_set)
    num_params = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_set)
    params = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_set)
    has_child_param = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_set)
    child_status = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_set)
    child_id = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_set)
    child_param = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_set)
    h330_2 = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_set)
    bl2 = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_set)
    h330_3 = property(_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_get, _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM

# Register Dwg_Object_ASSOCCOMPOUNDACTIONPARAM in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_swigregister(Dwg_Object_ASSOCCOMPOUNDACTIONPARAM)
class Dwg_Object_ASSOCOBJECTACTIONPARAM(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_get, _LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_set)
    is_r2013 = property(_LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_get, _LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_set)
    aap_version = property(_LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_get, _LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_set)
    name = property(_LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_name_get, _LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_name_set)
    asdap_class_version = property(_LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_get, _LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_set)
    dep = property(_LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_get, _LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_get, _LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCOBJECTACTIONPARAM())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCOBJECTACTIONPARAM

# Register Dwg_Object_ASSOCOBJECTACTIONPARAM in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCOBJECTACTIONPARAM_swigregister(Dwg_Object_ASSOCOBJECTACTIONPARAM)
class Dwg_Object_ASSOCEDGEACTIONPARAM(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_parent_get, _LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_parent_set)
    is_r2013 = property(_LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_get, _LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_set)
    aap_version = property(_LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_get, _LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_set)
    name = property(_LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_name_get, _LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_name_set)
    asdap_class_version = property(_LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_get, _LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_set)
    dep = property(_LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_dep_get, _LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_dep_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_get, _LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_set)
    param = property(_LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_param_get, _LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_param_set)
    has_action = property(_LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_get, _LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_set)
    action_type = property(_LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_get, _LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_set)
    subent = property(_LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_subent_get, _LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_subent_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCEDGEACTIONPARAM())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCEDGEACTIONPARAM

# Register Dwg_Object_ASSOCEDGEACTIONPARAM in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCEDGEACTIONPARAM_swigregister(Dwg_Object_ASSOCEDGEACTIONPARAM)
class Dwg_Object_ASSOCFACEACTIONPARAM(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_parent_get, _LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_parent_set)
    is_r2013 = property(_LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_get, _LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_set)
    aap_version = property(_LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_get, _LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_set)
    name = property(_LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_name_get, _LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_name_set)
    asdap_class_version = property(_LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_get, _LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_set)
    dep = property(_LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_dep_get, _LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_dep_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_class_version_get, _LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_class_version_set)
    index = property(_LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_index_get, _LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_index_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCFACEACTIONPARAM())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCFACEACTIONPARAM

# Register Dwg_Object_ASSOCFACEACTIONPARAM in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCFACEACTIONPARAM_swigregister(Dwg_Object_ASSOCFACEACTIONPARAM)
class Dwg_Object_ASSOCPATHACTIONPARAM(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_parent_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_parent_set)
    is_r2013 = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_set)
    aap_version = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_set)
    name = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_name_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_name_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_class_version_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_class_version_set)
    bs1 = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_bs1_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_bs1_set)
    num_params = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_num_params_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_num_params_set)
    params = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_params_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_params_set)
    has_child_param = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_set)
    child_status = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_child_status_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_child_status_set)
    child_id = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_child_id_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_child_id_set)
    child_param = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_child_param_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_child_param_set)
    h330_2 = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_set)
    bl2 = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_bl2_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_bl2_set)
    h330_3 = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_set)
    version = property(_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_version_get, _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCPATHACTIONPARAM())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCPATHACTIONPARAM

# Register Dwg_Object_ASSOCPATHACTIONPARAM in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCPATHACTIONPARAM_swigregister(Dwg_Object_ASSOCPATHACTIONPARAM)
class Dwg_Object_ASSOCVERTEXACTIONPARAM(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_get, _LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_set)
    is_r2013 = property(_LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_get, _LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_set)
    aap_version = property(_LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_get, _LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_set)
    name = property(_LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_name_get, _LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_name_set)
    asdap_class_version = property(_LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_get, _LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_set)
    dep = property(_LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_get, _LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_get, _LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_set)
    pt = property(_LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_get, _LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCVERTEXACTIONPARAM())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCVERTEXACTIONPARAM

# Register Dwg_Object_ASSOCVERTEXACTIONPARAM in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCVERTEXACTIONPARAM_swigregister(Dwg_Object_ASSOCVERTEXACTIONPARAM)
class Dwg_ASSOCARRAYITEM(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_ASSOCARRAYITEM_parent_get, _LibreDWG.Dwg_ASSOCARRAYITEM_parent_set)
    class_version = property(_LibreDWG.Dwg_ASSOCARRAYITEM_class_version_get, _LibreDWG.Dwg_ASSOCARRAYITEM_class_version_set)
    itemloc = property(_LibreDWG.Dwg_ASSOCARRAYITEM_itemloc_get, _LibreDWG.Dwg_ASSOCARRAYITEM_itemloc_set)
    flags = property(_LibreDWG.Dwg_ASSOCARRAYITEM_flags_get, _LibreDWG.Dwg_ASSOCARRAYITEM_flags_set)
    is_default_transmatrix = property(_LibreDWG.Dwg_ASSOCARRAYITEM_is_default_transmatrix_get, _LibreDWG.Dwg_ASSOCARRAYITEM_is_default_transmatrix_set)
    x_dir = property(_LibreDWG.Dwg_ASSOCARRAYITEM_x_dir_get, _LibreDWG.Dwg_ASSOCARRAYITEM_x_dir_set)
    transmatrix = property(_LibreDWG.Dwg_ASSOCARRAYITEM_transmatrix_get, _LibreDWG.Dwg_ASSOCARRAYITEM_transmatrix_set)
    rel_transform = property(_LibreDWG.Dwg_ASSOCARRAYITEM_rel_transform_get, _LibreDWG.Dwg_ASSOCARRAYITEM_rel_transform_set)
    has_h1 = property(_LibreDWG.Dwg_ASSOCARRAYITEM_has_h1_get, _LibreDWG.Dwg_ASSOCARRAYITEM_has_h1_set)
    h1 = property(_LibreDWG.Dwg_ASSOCARRAYITEM_h1_get, _LibreDWG.Dwg_ASSOCARRAYITEM_h1_set)
    h2 = property(_LibreDWG.Dwg_ASSOCARRAYITEM_h2_get, _LibreDWG.Dwg_ASSOCARRAYITEM_h2_set)

    def __init__(self):
        _LibreDWG.Dwg_ASSOCARRAYITEM_swiginit(self, _LibreDWG.new_Dwg_ASSOCARRAYITEM())
    __swig_destroy__ = _LibreDWG.delete_Dwg_ASSOCARRAYITEM

# Register Dwg_ASSOCARRAYITEM in _LibreDWG:
_LibreDWG.Dwg_ASSOCARRAYITEM_swigregister(Dwg_ASSOCARRAYITEM)
class Dwg_Object_ASSOCARRAYPARAMETERS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_parent_get, _LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_parent_set)
    aap_version = property(_LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_get, _LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_set)
    num_items = property(_LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_num_items_get, _LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_num_items_set)
    classname = property(_LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_classname_get, _LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_classname_set)
    items = property(_LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_items_get, _LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_items_set)
    numitems = property(_LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_numitems_get, _LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_numitems_set)
    numrows = property(_LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_numrows_get, _LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_numrows_set)
    numlevels = property(_LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_get, _LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCARRAYPARAMETERS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCARRAYPARAMETERS

# Register Dwg_Object_ASSOCARRAYPARAMETERS in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCARRAYPARAMETERS_swigregister(Dwg_Object_ASSOCARRAYPARAMETERS)
class Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_set)
    entity = property(_LibreDWG.Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_get, _LibreDWG.Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY

# Register Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_swigregister(Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY)
class Dwg_ASSOCSURFACEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_parent_get, _LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_parent_set)
    version = property(_LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_version_get, _LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_version_set)
    is_semi_assoc = property(_LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_get, _LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_set)
    l2 = property(_LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_l2_get, _LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_l2_set)
    is_semi_ovr = property(_LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_get, _LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_set)
    grip_status = property(_LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_grip_status_get, _LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_grip_status_set)
    assocdep = property(_LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_assocdep_get, _LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_assocdep_set)

    def __init__(self):
        _LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_ASSOCSURFACEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_ASSOCSURFACEACTIONBODY

# Register Dwg_ASSOCSURFACEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_ASSOCSURFACEACTIONBODY_swigregister(Dwg_ASSOCSURFACEACTIONBODY)
class Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_set)
    option = property(_LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_get, _LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY

# Register Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_swigregister(Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY)
class Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY

# Register Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_swigregister(Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY)
class Dwg_Object_ASSOCPLANESURFACEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCPLANESURFACEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCPLANESURFACEACTIONBODY

# Register Dwg_Object_ASSOCPLANESURFACEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCPLANESURFACEACTIONBODY_swigregister(Dwg_Object_ASSOCPLANESURFACEACTIONBODY)
class Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY

# Register Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_swigregister(Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY)
class Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY

# Register Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_swigregister(Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY)
class Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_set)
    b1 = property(_LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_get, _LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY

# Register Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_swigregister(Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY)
class Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY

# Register Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_swigregister(Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY)
class Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY

# Register Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_swigregister(Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY)
class Dwg_Object_ASSOCEDGECHAMFERACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY

# Register Dwg_Object_ASSOCEDGECHAMFERACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_swigregister(Dwg_Object_ASSOCEDGECHAMFERACTIONBODY)
class Dwg_Object_ASSOCEDGEFILLETACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCEDGEFILLETACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCEDGEFILLETACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCEDGEFILLETACTIONBODY

# Register Dwg_Object_ASSOCEDGEFILLETACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCEDGEFILLETACTIONBODY_swigregister(Dwg_Object_ASSOCEDGEFILLETACTIONBODY)
class Dwg_Object_ASSOCTRIMSURFACEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_set)
    b1 = property(_LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_get, _LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_set)
    b2 = property(_LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_get, _LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_set)
    distance = property(_LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_get, _LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY

# Register Dwg_Object_ASSOCTRIMSURFACEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_swigregister(Dwg_Object_ASSOCTRIMSURFACEACTIONBODY)
class Dwg_Object_ASSOCBLENDSURFACEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_set)
    b1 = property(_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_get, _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_set)
    b2 = property(_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_get, _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_set)
    b3 = property(_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_get, _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_set)
    b4 = property(_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_get, _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_set)
    b5 = property(_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_get, _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_set)
    blend_options = property(_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_get, _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_set)
    bs2 = property(_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_get, _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY

# Register Dwg_Object_ASSOCBLENDSURFACEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_swigregister(Dwg_Object_ASSOCBLENDSURFACEACTIONBODY)
class Dwg_Object_ASSOCFILLETSURFACEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_set)
    status = property(_LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_get, _LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_set)
    pt1 = property(_LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_get, _LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_set)
    pt2 = property(_LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_get, _LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY

# Register Dwg_Object_ASSOCFILLETSURFACEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_swigregister(Dwg_Object_ASSOCFILLETSURFACEACTIONBODY)
class Dwg_Object_ASSOCPATCHSURFACEACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_set)
    sab = property(_LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_get, _LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_set)
    pbsab_status = property(_LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_get, _LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY

# Register Dwg_Object_ASSOCPATCHSURFACEACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_swigregister(Dwg_Object_ASSOCPATCHSURFACEACTIONBODY)
class Dwg_ASSOCACTIONBODY_action(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_ASSOCACTIONBODY_action_parent_get, _LibreDWG.Dwg_ASSOCACTIONBODY_action_parent_set)
    depid = property(_LibreDWG.Dwg_ASSOCACTIONBODY_action_depid_get, _LibreDWG.Dwg_ASSOCACTIONBODY_action_depid_set)
    dep = property(_LibreDWG.Dwg_ASSOCACTIONBODY_action_dep_get, _LibreDWG.Dwg_ASSOCACTIONBODY_action_dep_set)

    def __init__(self):
        _LibreDWG.Dwg_ASSOCACTIONBODY_action_swiginit(self, _LibreDWG.new_Dwg_ASSOCACTIONBODY_action())
    __swig_destroy__ = _LibreDWG.delete_Dwg_ASSOCACTIONBODY_action

# Register Dwg_ASSOCACTIONBODY_action in _LibreDWG:
_LibreDWG.Dwg_ASSOCACTIONBODY_action_swigregister(Dwg_ASSOCACTIONBODY_action)
class Dwg_Object_ASSOCMLEADERACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_parent_set)
    aaab_version = property(_LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_get, _LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_set)
    assoc_dep = property(_LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_get, _LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_set)
    actionbody = property(_LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_get, _LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_pab_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_set)
    num_actions = property(_LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_get, _LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_set)
    actions = property(_LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_actions_get, _LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_actions_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCMLEADERACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCMLEADERACTIONBODY

# Register Dwg_Object_ASSOCMLEADERACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCMLEADERACTIONBODY_swigregister(Dwg_Object_ASSOCMLEADERACTIONBODY)
class Dwg_Object_ASSOCALIGNEDDIMACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_set)
    aaab_version = property(_LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_get, _LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_set)
    assoc_dep = property(_LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_get, _LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_set)
    actionbody = property(_LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_get, _LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_set)
    r_node = property(_LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_get, _LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_set)
    d_node = property(_LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_get, _LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY

# Register Dwg_Object_ASSOCALIGNEDDIMACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_swigregister(Dwg_Object_ASSOCALIGNEDDIMACTIONBODY)
class Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_set)
    aaab_version = property(_LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_get, _LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_set)
    assoc_dep = property(_LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_get, _LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_set)
    actionbody = property(_LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_get, _LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_set)
    r_node = property(_LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_get, _LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_set)
    d_node = property(_LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_get, _LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_set)
    assocdep = property(_LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_get, _LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY

# Register Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_swigregister(Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY)
class Dwg_Object_ASSOCORDINATEDIMACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_set)
    aaab_version = property(_LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_get, _LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_set)
    assoc_dep = property(_LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_get, _LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_set)
    actionbody = property(_LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_get, _LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_set)
    r_node = property(_LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_get, _LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_set)
    d_node = property(_LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_get, _LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCORDINATEDIMACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCORDINATEDIMACTIONBODY

# Register Dwg_Object_ASSOCORDINATEDIMACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCORDINATEDIMACTIONBODY_swigregister(Dwg_Object_ASSOCORDINATEDIMACTIONBODY)
class Dwg_Object_ASSOCROTATEDDIMACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_set)
    aaab_version = property(_LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_get, _LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_set)
    assoc_dep = property(_LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_get, _LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_set)
    actionbody = property(_LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_get, _LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_set)
    r_node = property(_LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_get, _LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_set)
    d_node = property(_LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_get, _LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCROTATEDDIMACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCROTATEDDIMACTIONBODY

# Register Dwg_Object_ASSOCROTATEDDIMACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCROTATEDDIMACTIONBODY_swigregister(Dwg_Object_ASSOCROTATEDDIMACTIONBODY)
class Dwg_Object_ASSOCDIMDEPENDENCYBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_set)
    adb_version = property(_LibreDWG.Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_get, _LibreDWG.Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_set)
    dimbase_version = property(_LibreDWG.Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_get, _LibreDWG.Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_set)
    name = property(_LibreDWG.Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_get, _LibreDWG.Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_set)
    class_version = property(_LibreDWG.Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_get, _LibreDWG.Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCDIMDEPENDENCYBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCDIMDEPENDENCYBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCDIMDEPENDENCYBODY

# Register Dwg_Object_ASSOCDIMDEPENDENCYBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCDIMDEPENDENCYBODY_swigregister(Dwg_Object_ASSOCDIMDEPENDENCYBODY)
class Dwg_Object_BLOCKPARAMDEPENDENCYBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_get, _LibreDWG.Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_set)
    adb_version = property(_LibreDWG.Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_get, _LibreDWG.Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_set)
    dimbase_version = property(_LibreDWG.Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_get, _LibreDWG.Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_get, _LibreDWG.Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_set)
    class_version = property(_LibreDWG.Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_get, _LibreDWG.Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKPARAMDEPENDENCYBODY_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKPARAMDEPENDENCYBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKPARAMDEPENDENCYBODY

# Register Dwg_Object_BLOCKPARAMDEPENDENCYBODY in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKPARAMDEPENDENCYBODY_swigregister(Dwg_Object_BLOCKPARAMDEPENDENCYBODY)
class Dwg_ARRAYITEMLOCATOR(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_ARRAYITEMLOCATOR_parent_get, _LibreDWG.Dwg_ARRAYITEMLOCATOR_parent_set)
    itemloc = property(_LibreDWG.Dwg_ARRAYITEMLOCATOR_itemloc_get, _LibreDWG.Dwg_ARRAYITEMLOCATOR_itemloc_set)

    def __init__(self):
        _LibreDWG.Dwg_ARRAYITEMLOCATOR_swiginit(self, _LibreDWG.new_Dwg_ARRAYITEMLOCATOR())
    __swig_destroy__ = _LibreDWG.delete_Dwg_ARRAYITEMLOCATOR

# Register Dwg_ARRAYITEMLOCATOR in _LibreDWG:
_LibreDWG.Dwg_ARRAYITEMLOCATOR_swigregister(Dwg_ARRAYITEMLOCATOR)
class Dwg_Object_ASSOCARRAYACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_pab_set)
    aaab_version = property(_LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_get, _LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_set)
    paramblock = property(_LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_get, _LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_set)
    transmatrix = property(_LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_get, _LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCARRAYACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCARRAYACTIONBODY

# Register Dwg_Object_ASSOCARRAYACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCARRAYACTIONBODY_swigregister(Dwg_Object_ASSOCARRAYACTIONBODY)
class Dwg_Object_ASSOCARRAYMODIFYACTIONBODY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_get, _LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_set)
    aab_version = property(_LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_get, _LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_set)
    pab = property(_LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_get, _LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_set)
    aaab_version = property(_LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_get, _LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_set)
    paramblock = property(_LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_get, _LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_set)
    transmatrix = property(_LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_get, _LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_set)
    status = property(_LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_get, _LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_set)
    num_items = property(_LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_get, _LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_set)
    items = property(_LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_get, _LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_swiginit(self, _LibreDWG.new_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY

# Register Dwg_Object_ASSOCARRAYMODIFYACTIONBODY in _LibreDWG:
_LibreDWG.Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_swigregister(Dwg_Object_ASSOCARRAYMODIFYACTIONBODY)
class Dwg_EvalExpr(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parentid = property(_LibreDWG.Dwg_EvalExpr_parentid_get, _LibreDWG.Dwg_EvalExpr_parentid_set)
    major = property(_LibreDWG.Dwg_EvalExpr_major_get, _LibreDWG.Dwg_EvalExpr_major_set)
    minor = property(_LibreDWG.Dwg_EvalExpr_minor_get, _LibreDWG.Dwg_EvalExpr_minor_set)
    value_code = property(_LibreDWG.Dwg_EvalExpr_value_code_get, _LibreDWG.Dwg_EvalExpr_value_code_set)
    value = property(_LibreDWG.Dwg_EvalExpr_value_get)
    nodeid = property(_LibreDWG.Dwg_EvalExpr_nodeid_get, _LibreDWG.Dwg_EvalExpr_nodeid_set)

    def __init__(self):
        _LibreDWG.Dwg_EvalExpr_swiginit(self, _LibreDWG.new_Dwg_EvalExpr())
    __swig_destroy__ = _LibreDWG.delete_Dwg_EvalExpr

# Register Dwg_EvalExpr in _LibreDWG:
_LibreDWG.Dwg_EvalExpr_swigregister(Dwg_EvalExpr)
class _dwg_EvalExpr_value(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num40 = property(_LibreDWG._dwg_EvalExpr_value_num40_get, _LibreDWG._dwg_EvalExpr_value_num40_set)
    pt2d = property(_LibreDWG._dwg_EvalExpr_value_pt2d_get, _LibreDWG._dwg_EvalExpr_value_pt2d_set)
    pt3d = property(_LibreDWG._dwg_EvalExpr_value_pt3d_get, _LibreDWG._dwg_EvalExpr_value_pt3d_set)
    text1 = property(_LibreDWG._dwg_EvalExpr_value_text1_get, _LibreDWG._dwg_EvalExpr_value_text1_set)
    long90 = property(_LibreDWG._dwg_EvalExpr_value_long90_get, _LibreDWG._dwg_EvalExpr_value_long90_set)
    handle91 = property(_LibreDWG._dwg_EvalExpr_value_handle91_get, _LibreDWG._dwg_EvalExpr_value_handle91_set)
    short70 = property(_LibreDWG._dwg_EvalExpr_value_short70_get, _LibreDWG._dwg_EvalExpr_value_short70_set)

    def __init__(self):
        _LibreDWG._dwg_EvalExpr_value_swiginit(self, _LibreDWG.new__dwg_EvalExpr_value())
    __swig_destroy__ = _LibreDWG.delete__dwg_EvalExpr_value

# Register _dwg_EvalExpr_value in _LibreDWG:
_LibreDWG._dwg_EvalExpr_value_swigregister(_dwg_EvalExpr_value)
class Dwg_ACSH_SubentMaterial(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    major = property(_LibreDWG.Dwg_ACSH_SubentMaterial_major_get, _LibreDWG.Dwg_ACSH_SubentMaterial_major_set)
    minor = property(_LibreDWG.Dwg_ACSH_SubentMaterial_minor_get, _LibreDWG.Dwg_ACSH_SubentMaterial_minor_set)
    reflectance = property(_LibreDWG.Dwg_ACSH_SubentMaterial_reflectance_get, _LibreDWG.Dwg_ACSH_SubentMaterial_reflectance_set)
    displacement = property(_LibreDWG.Dwg_ACSH_SubentMaterial_displacement_get, _LibreDWG.Dwg_ACSH_SubentMaterial_displacement_set)

    def __init__(self):
        _LibreDWG.Dwg_ACSH_SubentMaterial_swiginit(self, _LibreDWG.new_Dwg_ACSH_SubentMaterial())
    __swig_destroy__ = _LibreDWG.delete_Dwg_ACSH_SubentMaterial

# Register Dwg_ACSH_SubentMaterial in _LibreDWG:
_LibreDWG.Dwg_ACSH_SubentMaterial_swigregister(Dwg_ACSH_SubentMaterial)
class Dwg_ACSH_SubentColor(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    major = property(_LibreDWG.Dwg_ACSH_SubentColor_major_get, _LibreDWG.Dwg_ACSH_SubentColor_major_set)
    minor = property(_LibreDWG.Dwg_ACSH_SubentColor_minor_get, _LibreDWG.Dwg_ACSH_SubentColor_minor_set)
    transparency = property(_LibreDWG.Dwg_ACSH_SubentColor_transparency_get, _LibreDWG.Dwg_ACSH_SubentColor_transparency_set)
    bl93 = property(_LibreDWG.Dwg_ACSH_SubentColor_bl93_get, _LibreDWG.Dwg_ACSH_SubentColor_bl93_set)
    is_face_variable = property(_LibreDWG.Dwg_ACSH_SubentColor_is_face_variable_get, _LibreDWG.Dwg_ACSH_SubentColor_is_face_variable_set)

    def __init__(self):
        _LibreDWG.Dwg_ACSH_SubentColor_swiginit(self, _LibreDWG.new_Dwg_ACSH_SubentColor())
    __swig_destroy__ = _LibreDWG.delete_Dwg_ACSH_SubentColor

# Register Dwg_ACSH_SubentColor in _LibreDWG:
_LibreDWG.Dwg_ACSH_SubentColor_swigregister(Dwg_ACSH_SubentColor)
class Dwg_ACSH_HistoryNode(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    major = property(_LibreDWG.Dwg_ACSH_HistoryNode_major_get, _LibreDWG.Dwg_ACSH_HistoryNode_major_set)
    minor = property(_LibreDWG.Dwg_ACSH_HistoryNode_minor_get, _LibreDWG.Dwg_ACSH_HistoryNode_minor_set)
    trans = property(_LibreDWG.Dwg_ACSH_HistoryNode_trans_get, _LibreDWG.Dwg_ACSH_HistoryNode_trans_set)
    color = property(_LibreDWG.Dwg_ACSH_HistoryNode_color_get, _LibreDWG.Dwg_ACSH_HistoryNode_color_set)
    step_id = property(_LibreDWG.Dwg_ACSH_HistoryNode_step_id_get, _LibreDWG.Dwg_ACSH_HistoryNode_step_id_set)
    material = property(_LibreDWG.Dwg_ACSH_HistoryNode_material_get, _LibreDWG.Dwg_ACSH_HistoryNode_material_set)

    def __init__(self):
        _LibreDWG.Dwg_ACSH_HistoryNode_swiginit(self, _LibreDWG.new_Dwg_ACSH_HistoryNode())
    __swig_destroy__ = _LibreDWG.delete_Dwg_ACSH_HistoryNode

# Register Dwg_ACSH_HistoryNode in _LibreDWG:
_LibreDWG.Dwg_ACSH_HistoryNode_swigregister(Dwg_ACSH_HistoryNode)
class Dwg_Object_ACSH_HISTORY_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_parent_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_minor_set)
    owner = property(_LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_owner_get, _LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_owner_set)
    h_nodeid = property(_LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_get, _LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_set)
    show_history = property(_LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_show_history_get, _LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_show_history_set)
    record_history = property(_LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_record_history_get, _LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_record_history_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_HISTORY_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_HISTORY_CLASS

# Register Dwg_Object_ACSH_HISTORY_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_HISTORY_CLASS_swigregister(Dwg_Object_ACSH_HISTORY_CLASS)
class Dwg_Object_ACSH_BOX_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_BOX_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_BOX_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_BOX_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_BOX_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_BOX_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_BOX_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_BOX_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_BOX_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_BOX_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_BOX_CLASS_minor_set)
    length = property(_LibreDWG.Dwg_Object_ACSH_BOX_CLASS_length_get, _LibreDWG.Dwg_Object_ACSH_BOX_CLASS_length_set)
    width = property(_LibreDWG.Dwg_Object_ACSH_BOX_CLASS_width_get, _LibreDWG.Dwg_Object_ACSH_BOX_CLASS_width_set)
    height = property(_LibreDWG.Dwg_Object_ACSH_BOX_CLASS_height_get, _LibreDWG.Dwg_Object_ACSH_BOX_CLASS_height_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_BOX_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_BOX_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_BOX_CLASS

# Register Dwg_Object_ACSH_BOX_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_BOX_CLASS_swigregister(Dwg_Object_ACSH_BOX_CLASS)
class Dwg_Object_ACSH_WEDGE_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_minor_set)
    length = property(_LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_length_get, _LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_length_set)
    width = property(_LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_width_get, _LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_width_set)
    height = property(_LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_height_get, _LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_height_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_WEDGE_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_WEDGE_CLASS

# Register Dwg_Object_ACSH_WEDGE_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_WEDGE_CLASS_swigregister(Dwg_Object_ACSH_WEDGE_CLASS)
class Dwg_Object_ACSH_BOOLEAN_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_minor_set)
    operation = property(_LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_operation_get, _LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_operation_set)
    operand1 = property(_LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_get, _LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_set)
    operand2 = property(_LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_get, _LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_BOOLEAN_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_BOOLEAN_CLASS

# Register Dwg_Object_ACSH_BOOLEAN_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_BOOLEAN_CLASS_swigregister(Dwg_Object_ACSH_BOOLEAN_CLASS)
class Dwg_Object_ACSH_BREP_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_parent_set)
    acis_empty = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_acis_empty_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_acis_empty_set)
    unknown = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_unknown_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_unknown_set)
    version = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_version_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_version_set)
    num_blocks = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_num_blocks_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_num_blocks_set)
    block_size = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_block_size_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_block_size_set)
    encr_sat_data = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_set)
    sab_size = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_sab_size_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_sab_size_set)
    acis_data = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_acis_data_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_acis_data_set)
    wireframe_data_present = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_set)
    point_present = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_point_present_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_point_present_set)
    point = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_point_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_point_set)
    isolines = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_isolines_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_isolines_set)
    isoline_present = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_isoline_present_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_isoline_present_set)
    num_wires = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_num_wires_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_num_wires_set)
    wires = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_wires_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_wires_set)
    num_silhouettes = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_set)
    silhouettes = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_silhouettes_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_silhouettes_set)
    _dxf_sab_converted = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_set)
    acis_empty2 = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_acis_empty2_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_acis_empty2_set)
    extra_acis_data = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_set)
    num_materials = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_num_materials_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_num_materials_set)
    materials = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_materials_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_materials_set)
    revision_guid = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_revision_guid_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_revision_guid_set)
    revision_major = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_revision_major_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_revision_major_set)
    revision_minor1 = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_revision_minor1_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_revision_minor1_set)
    revision_minor2 = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_revision_minor2_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_revision_minor2_set)
    revision_bytes = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_revision_bytes_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_revision_bytes_set)
    end_marker = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_end_marker_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_end_marker_set)
    history_id = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_history_id_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_history_id_set)
    has_revision_guid = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_set)
    acis_empty_bit = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_minor_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_BREP_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_BREP_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_BREP_CLASS

# Register Dwg_Object_ACSH_BREP_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_BREP_CLASS_swigregister(Dwg_Object_ACSH_BREP_CLASS)
class Dwg_Object_ACSH_SWEEP_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_minor_set)
    direction = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_direction_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_direction_set)
    bl92 = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_bl92_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_bl92_set)
    shsw_text_size = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_set)
    shsw_text = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_set)
    shsw_bl93 = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_set)
    shsw_text2_size = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_set)
    shsw_text2 = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_set)
    draft_angle = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_set)
    start_draft_dist = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_set)
    end_draft_dist = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_set)
    scale_factor = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_set)
    twist_angle = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_set)
    align_angle = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_align_angle_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_align_angle_set)
    sweepentity_transform = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_set)
    pathentity_transform = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_set)
    align_option = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_align_option_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_align_option_set)
    miter_option = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_miter_option_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_miter_option_set)
    has_align_start = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_set)
    bank = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_bank_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_bank_set)
    check_intersections = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_set)
    shsw_b294 = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_set)
    shsw_b295 = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_set)
    shsw_b296 = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_set)
    pt2 = property(_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_pt2_get, _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_pt2_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_SWEEP_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_SWEEP_CLASS

# Register Dwg_Object_ACSH_SWEEP_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_SWEEP_CLASS_swigregister(Dwg_Object_ACSH_SWEEP_CLASS)
class Dwg_Object_ACSH_EXTRUSION_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_minor_set)
    direction = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_direction_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_direction_set)
    bl92 = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_set)
    shsw_text_size = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_set)
    shsw_text = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_set)
    shsw_bl93 = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_set)
    shsw_text2_size = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_set)
    shsw_text2 = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_set)
    draft_angle = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_set)
    start_draft_dist = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_set)
    end_draft_dist = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_set)
    scale_factor = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_set)
    twist_angle = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_set)
    align_angle = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_set)
    sweepentity_transform = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_set)
    pathentity_transform = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_set)
    align_option = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_set)
    miter_option = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_set)
    has_align_start = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_set)
    bank = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_bank_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_bank_set)
    check_intersections = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_set)
    shsw_b294 = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_set)
    shsw_b295 = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_set)
    shsw_b296 = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_set)
    pt2 = property(_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_get, _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_EXTRUSION_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_EXTRUSION_CLASS

# Register Dwg_Object_ACSH_EXTRUSION_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_EXTRUSION_CLASS_swigregister(Dwg_Object_ACSH_EXTRUSION_CLASS)
class Dwg_Object_ACSH_LOFT_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_minor_set)
    num_crosssects = property(_LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_get, _LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_set)
    crosssects = property(_LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_crosssects_get, _LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_crosssects_set)
    num_guides = property(_LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_num_guides_get, _LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_num_guides_set)
    guides = property(_LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_guides_get, _LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_guides_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_LOFT_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_LOFT_CLASS

# Register Dwg_Object_ACSH_LOFT_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_LOFT_CLASS_swigregister(Dwg_Object_ACSH_LOFT_CLASS)
class Dwg_Object_ACSH_FILLET_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_minor_set)
    bl92 = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_bl92_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_bl92_set)
    num_edges = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_num_edges_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_num_edges_set)
    edges = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_edges_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_edges_set)
    num_radiuses = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_set)
    num_startsetbacks = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_set)
    num_endsetbacks = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_set)
    radiuses = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_radiuses_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_radiuses_set)
    startsetbacks = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_set)
    endsetbacks = property(_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_get, _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_FILLET_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_FILLET_CLASS

# Register Dwg_Object_ACSH_FILLET_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_FILLET_CLASS_swigregister(Dwg_Object_ACSH_FILLET_CLASS)
class Dwg_Object_ACSH_CHAMFER_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_minor_set)
    bl92 = property(_LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_bl92_get, _LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_bl92_set)
    base_dist = property(_LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_get, _LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_set)
    other_dist = property(_LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_get, _LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_set)
    num_edges = property(_LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_get, _LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_set)
    edges = property(_LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_edges_get, _LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_edges_set)
    bl95 = property(_LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_bl95_get, _LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_bl95_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_CHAMFER_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_CHAMFER_CLASS

# Register Dwg_Object_ACSH_CHAMFER_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_CHAMFER_CLASS_swigregister(Dwg_Object_ACSH_CHAMFER_CLASS)
class Dwg_Object_ACSH_CYLINDER_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_minor_set)
    height = property(_LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_height_get, _LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_height_set)
    major_radius = property(_LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_get, _LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_set)
    minor_radius = property(_LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_get, _LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_set)
    x_radius = property(_LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_get, _LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_CYLINDER_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_CYLINDER_CLASS

# Register Dwg_Object_ACSH_CYLINDER_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_CYLINDER_CLASS_swigregister(Dwg_Object_ACSH_CYLINDER_CLASS)
class Dwg_Object_ACSH_CONE_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_CONE_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_CONE_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_CONE_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_CONE_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_CONE_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_CONE_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_CONE_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_CONE_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_CONE_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_CONE_CLASS_minor_set)
    height = property(_LibreDWG.Dwg_Object_ACSH_CONE_CLASS_height_get, _LibreDWG.Dwg_Object_ACSH_CONE_CLASS_height_set)
    major_radius = property(_LibreDWG.Dwg_Object_ACSH_CONE_CLASS_major_radius_get, _LibreDWG.Dwg_Object_ACSH_CONE_CLASS_major_radius_set)
    minor_radius = property(_LibreDWG.Dwg_Object_ACSH_CONE_CLASS_minor_radius_get, _LibreDWG.Dwg_Object_ACSH_CONE_CLASS_minor_radius_set)
    x_radius = property(_LibreDWG.Dwg_Object_ACSH_CONE_CLASS_x_radius_get, _LibreDWG.Dwg_Object_ACSH_CONE_CLASS_x_radius_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_CONE_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_CONE_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_CONE_CLASS

# Register Dwg_Object_ACSH_CONE_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_CONE_CLASS_swigregister(Dwg_Object_ACSH_CONE_CLASS)
class Dwg_Object_ACSH_PYRAMID_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_minor_set)
    height = property(_LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_height_get, _LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_height_set)
    sides = property(_LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_sides_get, _LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_sides_set)
    radius = property(_LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_radius_get, _LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_radius_set)
    topradius = property(_LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_topradius_get, _LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_topradius_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_PYRAMID_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_PYRAMID_CLASS

# Register Dwg_Object_ACSH_PYRAMID_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_PYRAMID_CLASS_swigregister(Dwg_Object_ACSH_PYRAMID_CLASS)
class Dwg_Object_ACSH_SPHERE_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_minor_set)
    radius = property(_LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_radius_get, _LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_radius_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_SPHERE_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_SPHERE_CLASS

# Register Dwg_Object_ACSH_SPHERE_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_SPHERE_CLASS_swigregister(Dwg_Object_ACSH_SPHERE_CLASS)
class Dwg_Object_ACSH_TORUS_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_minor_set)
    major_radius = property(_LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_major_radius_get, _LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_major_radius_set)
    minor_radius = property(_LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_minor_radius_get, _LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_minor_radius_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_TORUS_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_TORUS_CLASS

# Register Dwg_Object_ACSH_TORUS_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_TORUS_CLASS_swigregister(Dwg_Object_ACSH_TORUS_CLASS)
class Dwg_Object_ACSH_REVOLVE_CLASS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_parent_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_set)
    history_node = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_history_node_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_history_node_set)
    major = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_major_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_major_set)
    minor = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_minor_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_minor_set)
    axis_pt = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_set)
    direction = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_direction_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_direction_set)
    revolve_angle = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_set)
    start_angle = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_set)
    draft_angle = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_set)
    bd44 = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_bd44_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_bd44_set)
    bd45 = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_bd45_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_bd45_set)
    twist_angle = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_set)
    b290 = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_b290_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_b290_set)
    is_close_to_axis = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_set)
    sweep_entity = property(_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_get, _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_swiginit(self, _LibreDWG.new_Dwg_Object_ACSH_REVOLVE_CLASS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACSH_REVOLVE_CLASS

# Register Dwg_Object_ACSH_REVOLVE_CLASS in _LibreDWG:
_LibreDWG.Dwg_Object_ACSH_REVOLVE_CLASS_swigregister(Dwg_Object_ACSH_REVOLVE_CLASS)
class Dwg_Entity_NAVISWORKSMODEL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_NAVISWORKSMODEL_parent_get, _LibreDWG.Dwg_Entity_NAVISWORKSMODEL_parent_set)
    flags = property(_LibreDWG.Dwg_Entity_NAVISWORKSMODEL_flags_get, _LibreDWG.Dwg_Entity_NAVISWORKSMODEL_flags_set)
    definition = property(_LibreDWG.Dwg_Entity_NAVISWORKSMODEL_definition_get, _LibreDWG.Dwg_Entity_NAVISWORKSMODEL_definition_set)
    transmatrix = property(_LibreDWG.Dwg_Entity_NAVISWORKSMODEL_transmatrix_get, _LibreDWG.Dwg_Entity_NAVISWORKSMODEL_transmatrix_set)
    unitfactor = property(_LibreDWG.Dwg_Entity_NAVISWORKSMODEL_unitfactor_get, _LibreDWG.Dwg_Entity_NAVISWORKSMODEL_unitfactor_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_NAVISWORKSMODEL_swiginit(self, _LibreDWG.new_Dwg_Entity_NAVISWORKSMODEL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_NAVISWORKSMODEL

# Register Dwg_Entity_NAVISWORKSMODEL in _LibreDWG:
_LibreDWG.Dwg_Entity_NAVISWORKSMODEL_swigregister(Dwg_Entity_NAVISWORKSMODEL)
class Dwg_Object_NAVISWORKSMODELDEF(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_parent_get, _LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_parent_set)
    flags = property(_LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_flags_get, _LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_flags_set)
    path = property(_LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_path_get, _LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_path_set)
    status = property(_LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_status_get, _LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_status_set)
    min_extent = property(_LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_min_extent_get, _LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_min_extent_set)
    max_extent = property(_LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_max_extent_get, _LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_max_extent_set)
    host_drawing_visibility = property(_LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_get, _LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_swiginit(self, _LibreDWG.new_Dwg_Object_NAVISWORKSMODELDEF())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_NAVISWORKSMODELDEF

# Register Dwg_Object_NAVISWORKSMODELDEF in _LibreDWG:
_LibreDWG.Dwg_Object_NAVISWORKSMODELDEF_swigregister(Dwg_Object_NAVISWORKSMODELDEF)
class Dwg_Object_RENDERSETTINGS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_RENDERSETTINGS_parent_get, _LibreDWG.Dwg_Object_RENDERSETTINGS_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_RENDERSETTINGS_class_version_get, _LibreDWG.Dwg_Object_RENDERSETTINGS_class_version_set)
    name = property(_LibreDWG.Dwg_Object_RENDERSETTINGS_name_get, _LibreDWG.Dwg_Object_RENDERSETTINGS_name_set)
    fog_enabled = property(_LibreDWG.Dwg_Object_RENDERSETTINGS_fog_enabled_get, _LibreDWG.Dwg_Object_RENDERSETTINGS_fog_enabled_set)
    fog_background_enabled = property(_LibreDWG.Dwg_Object_RENDERSETTINGS_fog_background_enabled_get, _LibreDWG.Dwg_Object_RENDERSETTINGS_fog_background_enabled_set)
    backfaces_enabled = property(_LibreDWG.Dwg_Object_RENDERSETTINGS_backfaces_enabled_get, _LibreDWG.Dwg_Object_RENDERSETTINGS_backfaces_enabled_set)
    environ_image_enabled = property(_LibreDWG.Dwg_Object_RENDERSETTINGS_environ_image_enabled_get, _LibreDWG.Dwg_Object_RENDERSETTINGS_environ_image_enabled_set)
    environ_image_filename = property(_LibreDWG.Dwg_Object_RENDERSETTINGS_environ_image_filename_get, _LibreDWG.Dwg_Object_RENDERSETTINGS_environ_image_filename_set)
    description = property(_LibreDWG.Dwg_Object_RENDERSETTINGS_description_get, _LibreDWG.Dwg_Object_RENDERSETTINGS_description_set)
    display_index = property(_LibreDWG.Dwg_Object_RENDERSETTINGS_display_index_get, _LibreDWG.Dwg_Object_RENDERSETTINGS_display_index_set)
    has_predefined = property(_LibreDWG.Dwg_Object_RENDERSETTINGS_has_predefined_get, _LibreDWG.Dwg_Object_RENDERSETTINGS_has_predefined_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_RENDERSETTINGS_swiginit(self, _LibreDWG.new_Dwg_Object_RENDERSETTINGS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_RENDERSETTINGS

# Register Dwg_Object_RENDERSETTINGS in _LibreDWG:
_LibreDWG.Dwg_Object_RENDERSETTINGS_swigregister(Dwg_Object_RENDERSETTINGS)
class Dwg_Object_MENTALRAYRENDERSETTINGS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_parent_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_set)
    name = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_name_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_name_set)
    fog_enabled = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_set)
    fog_background_enabled = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_set)
    backfaces_enabled = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_set)
    environ_image_enabled = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_set)
    environ_image_filename = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_set)
    description = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_description_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_description_set)
    display_index = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_set)
    has_predefined = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_set)
    mr_version = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_set)
    sampling1 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_set)
    sampling2 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_set)
    sampling_mr_filter = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_set)
    sampling_filter1 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_set)
    sampling_filter2 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_set)
    sampling_contrast_color1 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_set)
    sampling_contrast_color2 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_set)
    sampling_contrast_color3 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_set)
    sampling_contrast_color4 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_set)
    shadow_mode = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_set)
    shadow_maps_enabled = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_set)
    ray_tracing_enabled = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_set)
    ray_trace_depth1 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_set)
    ray_trace_depth2 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_set)
    ray_trace_depth3 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_set)
    global_illumination_enabled = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_set)
    gi_sample_count = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_set)
    gi_sample_radius_enabled = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_set)
    gi_sample_radius = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_set)
    gi_photons_per_light = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_set)
    photon_trace_depth1 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_set)
    photon_trace_depth2 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_set)
    photon_trace_depth3 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_set)
    final_gathering_enabled = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_set)
    fg_ray_count = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_set)
    fg_sample_radius_state1 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_set)
    fg_sample_radius_state2 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_set)
    fg_sample_radius_state3 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_set)
    fg_sample_radius1 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_set)
    fg_sample_radius2 = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_set)
    light_luminance_scale = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_set)
    diagnostics_mode = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_set)
    diagnostics_grid_mode = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_set)
    diagnostics_grid_float = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_set)
    diagnostics_photon_mode = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_set)
    diagnostics_bsp_mode = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_set)
    export_mi_enabled = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_set)
    mr_description = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_set)
    tile_size = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_set)
    tile_order = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_set)
    memory_limit = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_set)
    diagnostics_samples_mode = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_set)
    energy_multiplier = property(_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_get, _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_swiginit(self, _LibreDWG.new_Dwg_Object_MENTALRAYRENDERSETTINGS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_MENTALRAYRENDERSETTINGS

# Register Dwg_Object_MENTALRAYRENDERSETTINGS in _LibreDWG:
_LibreDWG.Dwg_Object_MENTALRAYRENDERSETTINGS_swigregister(Dwg_Object_MENTALRAYRENDERSETTINGS)
class Dwg_Object_RAPIDRTRENDERSETTINGS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_parent_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_set)
    name = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_name_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_name_set)
    fog_enabled = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_set)
    fog_background_enabled = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_set)
    backfaces_enabled = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_set)
    environ_image_enabled = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_set)
    environ_image_filename = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_set)
    description = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_description_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_description_set)
    display_index = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_set)
    has_predefined = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_set)
    rapidrt_version = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_set)
    render_target = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_set)
    render_level = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_set)
    render_time = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_set)
    lighting_model = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_set)
    filter_type = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_set)
    filter_width = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_set)
    filter_height = property(_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_get, _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_swiginit(self, _LibreDWG.new_Dwg_Object_RAPIDRTRENDERSETTINGS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_RAPIDRTRENDERSETTINGS

# Register Dwg_Object_RAPIDRTRENDERSETTINGS in _LibreDWG:
_LibreDWG.Dwg_Object_RAPIDRTRENDERSETTINGS_swigregister(Dwg_Object_RAPIDRTRENDERSETTINGS)
class Dwg_Object_RENDERENVIRONMENT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_RENDERENVIRONMENT_parent_get, _LibreDWG.Dwg_Object_RENDERENVIRONMENT_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_RENDERENVIRONMENT_class_version_get, _LibreDWG.Dwg_Object_RENDERENVIRONMENT_class_version_set)
    fog_enabled = property(_LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_enabled_get, _LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_enabled_set)
    fog_background_enabled = property(_LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_get, _LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_set)
    fog_color_r = property(_LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_color_r_get, _LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_color_r_set)
    fog_color_g = property(_LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_color_g_get, _LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_color_g_set)
    fog_color_b = property(_LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_color_b_get, _LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_color_b_set)
    fog_density_near = property(_LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_density_near_get, _LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_density_near_set)
    fog_density_far = property(_LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_density_far_get, _LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_density_far_set)
    fog_distance_near = property(_LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_distance_near_get, _LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_distance_near_set)
    fog_distance_far = property(_LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_distance_far_get, _LibreDWG.Dwg_Object_RENDERENVIRONMENT_fog_distance_far_set)
    environ_image_enabled = property(_LibreDWG.Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_get, _LibreDWG.Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_set)
    environ_image_filename = property(_LibreDWG.Dwg_Object_RENDERENVIRONMENT_environ_image_filename_get, _LibreDWG.Dwg_Object_RENDERENVIRONMENT_environ_image_filename_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_RENDERENVIRONMENT_swiginit(self, _LibreDWG.new_Dwg_Object_RENDERENVIRONMENT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_RENDERENVIRONMENT

# Register Dwg_Object_RENDERENVIRONMENT in _LibreDWG:
_LibreDWG.Dwg_Object_RENDERENVIRONMENT_swigregister(Dwg_Object_RENDERENVIRONMENT)
class Dwg_Object_RENDERGLOBAL(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_RENDERGLOBAL_parent_get, _LibreDWG.Dwg_Object_RENDERGLOBAL_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_RENDERGLOBAL_class_version_get, _LibreDWG.Dwg_Object_RENDERGLOBAL_class_version_set)
    procedure = property(_LibreDWG.Dwg_Object_RENDERGLOBAL_procedure_get, _LibreDWG.Dwg_Object_RENDERGLOBAL_procedure_set)
    destination = property(_LibreDWG.Dwg_Object_RENDERGLOBAL_destination_get, _LibreDWG.Dwg_Object_RENDERGLOBAL_destination_set)
    save_enabled = property(_LibreDWG.Dwg_Object_RENDERGLOBAL_save_enabled_get, _LibreDWG.Dwg_Object_RENDERGLOBAL_save_enabled_set)
    save_filename = property(_LibreDWG.Dwg_Object_RENDERGLOBAL_save_filename_get, _LibreDWG.Dwg_Object_RENDERGLOBAL_save_filename_set)
    image_width = property(_LibreDWG.Dwg_Object_RENDERGLOBAL_image_width_get, _LibreDWG.Dwg_Object_RENDERGLOBAL_image_width_set)
    image_height = property(_LibreDWG.Dwg_Object_RENDERGLOBAL_image_height_get, _LibreDWG.Dwg_Object_RENDERGLOBAL_image_height_set)
    predef_presets_first = property(_LibreDWG.Dwg_Object_RENDERGLOBAL_predef_presets_first_get, _LibreDWG.Dwg_Object_RENDERGLOBAL_predef_presets_first_set)
    highlevel_info = property(_LibreDWG.Dwg_Object_RENDERGLOBAL_highlevel_info_get, _LibreDWG.Dwg_Object_RENDERGLOBAL_highlevel_info_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_RENDERGLOBAL_swiginit(self, _LibreDWG.new_Dwg_Object_RENDERGLOBAL())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_RENDERGLOBAL

# Register Dwg_Object_RENDERGLOBAL in _LibreDWG:
_LibreDWG.Dwg_Object_RENDERGLOBAL_swigregister(Dwg_Object_RENDERGLOBAL)
class Dwg_Object_RENDERENTRY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_RENDERENTRY_parent_get, _LibreDWG.Dwg_Object_RENDERENTRY_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_RENDERENTRY_class_version_get, _LibreDWG.Dwg_Object_RENDERENTRY_class_version_set)
    image_file_name = property(_LibreDWG.Dwg_Object_RENDERENTRY_image_file_name_get, _LibreDWG.Dwg_Object_RENDERENTRY_image_file_name_set)
    preset_name = property(_LibreDWG.Dwg_Object_RENDERENTRY_preset_name_get, _LibreDWG.Dwg_Object_RENDERENTRY_preset_name_set)
    view_name = property(_LibreDWG.Dwg_Object_RENDERENTRY_view_name_get, _LibreDWG.Dwg_Object_RENDERENTRY_view_name_set)
    dimension_x = property(_LibreDWG.Dwg_Object_RENDERENTRY_dimension_x_get, _LibreDWG.Dwg_Object_RENDERENTRY_dimension_x_set)
    dimension_y = property(_LibreDWG.Dwg_Object_RENDERENTRY_dimension_y_get, _LibreDWG.Dwg_Object_RENDERENTRY_dimension_y_set)
    start_year = property(_LibreDWG.Dwg_Object_RENDERENTRY_start_year_get, _LibreDWG.Dwg_Object_RENDERENTRY_start_year_set)
    start_month = property(_LibreDWG.Dwg_Object_RENDERENTRY_start_month_get, _LibreDWG.Dwg_Object_RENDERENTRY_start_month_set)
    start_day = property(_LibreDWG.Dwg_Object_RENDERENTRY_start_day_get, _LibreDWG.Dwg_Object_RENDERENTRY_start_day_set)
    start_minute = property(_LibreDWG.Dwg_Object_RENDERENTRY_start_minute_get, _LibreDWG.Dwg_Object_RENDERENTRY_start_minute_set)
    start_second = property(_LibreDWG.Dwg_Object_RENDERENTRY_start_second_get, _LibreDWG.Dwg_Object_RENDERENTRY_start_second_set)
    start_msec = property(_LibreDWG.Dwg_Object_RENDERENTRY_start_msec_get, _LibreDWG.Dwg_Object_RENDERENTRY_start_msec_set)
    render_time = property(_LibreDWG.Dwg_Object_RENDERENTRY_render_time_get, _LibreDWG.Dwg_Object_RENDERENTRY_render_time_set)
    memory_amount = property(_LibreDWG.Dwg_Object_RENDERENTRY_memory_amount_get, _LibreDWG.Dwg_Object_RENDERENTRY_memory_amount_set)
    material_count = property(_LibreDWG.Dwg_Object_RENDERENTRY_material_count_get, _LibreDWG.Dwg_Object_RENDERENTRY_material_count_set)
    light_count = property(_LibreDWG.Dwg_Object_RENDERENTRY_light_count_get, _LibreDWG.Dwg_Object_RENDERENTRY_light_count_set)
    triangle_count = property(_LibreDWG.Dwg_Object_RENDERENTRY_triangle_count_get, _LibreDWG.Dwg_Object_RENDERENTRY_triangle_count_set)
    display_index = property(_LibreDWG.Dwg_Object_RENDERENTRY_display_index_get, _LibreDWG.Dwg_Object_RENDERENTRY_display_index_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_RENDERENTRY_swiginit(self, _LibreDWG.new_Dwg_Object_RENDERENTRY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_RENDERENTRY

# Register Dwg_Object_RENDERENTRY in _LibreDWG:
_LibreDWG.Dwg_Object_RENDERENTRY_swigregister(Dwg_Object_RENDERENTRY)
class Dwg_Object_MOTIONPATH(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_MOTIONPATH_parent_get, _LibreDWG.Dwg_Object_MOTIONPATH_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_MOTIONPATH_class_version_get, _LibreDWG.Dwg_Object_MOTIONPATH_class_version_set)
    camera_path = property(_LibreDWG.Dwg_Object_MOTIONPATH_camera_path_get, _LibreDWG.Dwg_Object_MOTIONPATH_camera_path_set)
    target_path = property(_LibreDWG.Dwg_Object_MOTIONPATH_target_path_get, _LibreDWG.Dwg_Object_MOTIONPATH_target_path_set)
    viewtable = property(_LibreDWG.Dwg_Object_MOTIONPATH_viewtable_get, _LibreDWG.Dwg_Object_MOTIONPATH_viewtable_set)
    frames = property(_LibreDWG.Dwg_Object_MOTIONPATH_frames_get, _LibreDWG.Dwg_Object_MOTIONPATH_frames_set)
    frame_rate = property(_LibreDWG.Dwg_Object_MOTIONPATH_frame_rate_get, _LibreDWG.Dwg_Object_MOTIONPATH_frame_rate_set)
    corner_decel = property(_LibreDWG.Dwg_Object_MOTIONPATH_corner_decel_get, _LibreDWG.Dwg_Object_MOTIONPATH_corner_decel_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_MOTIONPATH_swiginit(self, _LibreDWG.new_Dwg_Object_MOTIONPATH())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_MOTIONPATH

# Register Dwg_Object_MOTIONPATH in _LibreDWG:
_LibreDWG.Dwg_Object_MOTIONPATH_swigregister(Dwg_Object_MOTIONPATH)
class Dwg_Object_CURVEPATH(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_CURVEPATH_parent_get, _LibreDWG.Dwg_Object_CURVEPATH_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_CURVEPATH_class_version_get, _LibreDWG.Dwg_Object_CURVEPATH_class_version_set)
    entity = property(_LibreDWG.Dwg_Object_CURVEPATH_entity_get, _LibreDWG.Dwg_Object_CURVEPATH_entity_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_CURVEPATH_swiginit(self, _LibreDWG.new_Dwg_Object_CURVEPATH())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_CURVEPATH

# Register Dwg_Object_CURVEPATH in _LibreDWG:
_LibreDWG.Dwg_Object_CURVEPATH_swigregister(Dwg_Object_CURVEPATH)
class Dwg_Object_POINTPATH(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_POINTPATH_parent_get, _LibreDWG.Dwg_Object_POINTPATH_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_POINTPATH_class_version_get, _LibreDWG.Dwg_Object_POINTPATH_class_version_set)
    point = property(_LibreDWG.Dwg_Object_POINTPATH_point_get, _LibreDWG.Dwg_Object_POINTPATH_point_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_POINTPATH_swiginit(self, _LibreDWG.new_Dwg_Object_POINTPATH())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_POINTPATH

# Register Dwg_Object_POINTPATH in _LibreDWG:
_LibreDWG.Dwg_Object_POINTPATH_swigregister(Dwg_Object_POINTPATH)
class Dwg_Object_TVDEVICEPROPERTIES(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_parent_get, _LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_parent_set)
    flags = property(_LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_flags_get, _LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_flags_set)
    max_regen_threads = property(_LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_get, _LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_set)
    use_lut_palette = property(_LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_get, _LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_set)
    alt_hlt = property(_LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_get, _LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_set)
    alt_hltcolor = property(_LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_get, _LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_set)
    geom_shader_usage = property(_LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_get, _LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_set)
    blending_mode = property(_LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_blending_mode_get, _LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_blending_mode_set)
    antialiasing_level = property(_LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_get, _LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_set)
    bd2 = property(_LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_bd2_get, _LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_bd2_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_swiginit(self, _LibreDWG.new_Dwg_Object_TVDEVICEPROPERTIES())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_TVDEVICEPROPERTIES

# Register Dwg_Object_TVDEVICEPROPERTIES in _LibreDWG:
_LibreDWG.Dwg_Object_TVDEVICEPROPERTIES_swigregister(Dwg_Object_TVDEVICEPROPERTIES)
class Dwg_Object_SKYLIGHT_BACKGROUND(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_SKYLIGHT_BACKGROUND_parent_get, _LibreDWG.Dwg_Object_SKYLIGHT_BACKGROUND_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_SKYLIGHT_BACKGROUND_class_version_get, _LibreDWG.Dwg_Object_SKYLIGHT_BACKGROUND_class_version_set)
    sunid = property(_LibreDWG.Dwg_Object_SKYLIGHT_BACKGROUND_sunid_get, _LibreDWG.Dwg_Object_SKYLIGHT_BACKGROUND_sunid_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_SKYLIGHT_BACKGROUND_swiginit(self, _LibreDWG.new_Dwg_Object_SKYLIGHT_BACKGROUND())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_SKYLIGHT_BACKGROUND

# Register Dwg_Object_SKYLIGHT_BACKGROUND in _LibreDWG:
_LibreDWG.Dwg_Object_SKYLIGHT_BACKGROUND_swigregister(Dwg_Object_SKYLIGHT_BACKGROUND)
class Dwg_Object_SOLID_BACKGROUND(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_SOLID_BACKGROUND_parent_get, _LibreDWG.Dwg_Object_SOLID_BACKGROUND_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_SOLID_BACKGROUND_class_version_get, _LibreDWG.Dwg_Object_SOLID_BACKGROUND_class_version_set)
    color = property(_LibreDWG.Dwg_Object_SOLID_BACKGROUND_color_get, _LibreDWG.Dwg_Object_SOLID_BACKGROUND_color_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_SOLID_BACKGROUND_swiginit(self, _LibreDWG.new_Dwg_Object_SOLID_BACKGROUND())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_SOLID_BACKGROUND

# Register Dwg_Object_SOLID_BACKGROUND in _LibreDWG:
_LibreDWG.Dwg_Object_SOLID_BACKGROUND_swigregister(Dwg_Object_SOLID_BACKGROUND)
class Dwg_Object_IMAGE_BACKGROUND(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_IMAGE_BACKGROUND_parent_get, _LibreDWG.Dwg_Object_IMAGE_BACKGROUND_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_IMAGE_BACKGROUND_class_version_get, _LibreDWG.Dwg_Object_IMAGE_BACKGROUND_class_version_set)
    filename = property(_LibreDWG.Dwg_Object_IMAGE_BACKGROUND_filename_get, _LibreDWG.Dwg_Object_IMAGE_BACKGROUND_filename_set)
    fit_to_screen = property(_LibreDWG.Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_get, _LibreDWG.Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_set)
    maintain_aspect_ratio = property(_LibreDWG.Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_get, _LibreDWG.Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_set)
    use_tiling = property(_LibreDWG.Dwg_Object_IMAGE_BACKGROUND_use_tiling_get, _LibreDWG.Dwg_Object_IMAGE_BACKGROUND_use_tiling_set)
    offset = property(_LibreDWG.Dwg_Object_IMAGE_BACKGROUND_offset_get, _LibreDWG.Dwg_Object_IMAGE_BACKGROUND_offset_set)
    scale = property(_LibreDWG.Dwg_Object_IMAGE_BACKGROUND_scale_get, _LibreDWG.Dwg_Object_IMAGE_BACKGROUND_scale_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_IMAGE_BACKGROUND_swiginit(self, _LibreDWG.new_Dwg_Object_IMAGE_BACKGROUND())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_IMAGE_BACKGROUND

# Register Dwg_Object_IMAGE_BACKGROUND in _LibreDWG:
_LibreDWG.Dwg_Object_IMAGE_BACKGROUND_swigregister(Dwg_Object_IMAGE_BACKGROUND)
class Dwg_Object_IBL_BACKGROUND(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_IBL_BACKGROUND_parent_get, _LibreDWG.Dwg_Object_IBL_BACKGROUND_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_IBL_BACKGROUND_class_version_get, _LibreDWG.Dwg_Object_IBL_BACKGROUND_class_version_set)
    enable = property(_LibreDWG.Dwg_Object_IBL_BACKGROUND_enable_get, _LibreDWG.Dwg_Object_IBL_BACKGROUND_enable_set)
    name = property(_LibreDWG.Dwg_Object_IBL_BACKGROUND_name_get, _LibreDWG.Dwg_Object_IBL_BACKGROUND_name_set)
    rotation = property(_LibreDWG.Dwg_Object_IBL_BACKGROUND_rotation_get, _LibreDWG.Dwg_Object_IBL_BACKGROUND_rotation_set)
    display_image = property(_LibreDWG.Dwg_Object_IBL_BACKGROUND_display_image_get, _LibreDWG.Dwg_Object_IBL_BACKGROUND_display_image_set)
    secondary_background = property(_LibreDWG.Dwg_Object_IBL_BACKGROUND_secondary_background_get, _LibreDWG.Dwg_Object_IBL_BACKGROUND_secondary_background_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_IBL_BACKGROUND_swiginit(self, _LibreDWG.new_Dwg_Object_IBL_BACKGROUND())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_IBL_BACKGROUND

# Register Dwg_Object_IBL_BACKGROUND in _LibreDWG:
_LibreDWG.Dwg_Object_IBL_BACKGROUND_swigregister(Dwg_Object_IBL_BACKGROUND)
class Dwg_Object_GRADIENT_BACKGROUND(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_parent_get, _LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_class_version_get, _LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_class_version_set)
    color_top = property(_LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_color_top_get, _LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_color_top_set)
    color_middle = property(_LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_color_middle_get, _LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_color_middle_set)
    color_bottom = property(_LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_color_bottom_get, _LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_color_bottom_set)
    horizon = property(_LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_horizon_get, _LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_horizon_set)
    height = property(_LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_height_get, _LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_height_set)
    rotation = property(_LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_rotation_get, _LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_rotation_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_swiginit(self, _LibreDWG.new_Dwg_Object_GRADIENT_BACKGROUND())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_GRADIENT_BACKGROUND

# Register Dwg_Object_GRADIENT_BACKGROUND in _LibreDWG:
_LibreDWG.Dwg_Object_GRADIENT_BACKGROUND_swigregister(Dwg_Object_GRADIENT_BACKGROUND)
class Dwg_Object_GROUND_PLANE_BACKGROUND(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_parent_get, _LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_get, _LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_set)
    color_sky_zenith = property(_LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_get, _LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_set)
    color_sky_horizon = property(_LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_get, _LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_set)
    color_underground_horizon = property(_LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_get, _LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_set)
    color_underground_azimuth = property(_LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_get, _LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_set)
    color_near = property(_LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_get, _LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_set)
    color_far = property(_LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_get, _LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_swiginit(self, _LibreDWG.new_Dwg_Object_GROUND_PLANE_BACKGROUND())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_GROUND_PLANE_BACKGROUND

# Register Dwg_Object_GROUND_PLANE_BACKGROUND in _LibreDWG:
_LibreDWG.Dwg_Object_GROUND_PLANE_BACKGROUND_swigregister(Dwg_Object_GROUND_PLANE_BACKGROUND)
class Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA

# Register Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_swigregister(Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA)
class Dwg_CONTEXTDATA_dict(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_CONTEXTDATA_dict_parent_get, _LibreDWG.Dwg_CONTEXTDATA_dict_parent_set)
    text = property(_LibreDWG.Dwg_CONTEXTDATA_dict_text_get, _LibreDWG.Dwg_CONTEXTDATA_dict_text_set)
    itemhandle = property(_LibreDWG.Dwg_CONTEXTDATA_dict_itemhandle_get, _LibreDWG.Dwg_CONTEXTDATA_dict_itemhandle_set)

    def __init__(self):
        _LibreDWG.Dwg_CONTEXTDATA_dict_swiginit(self, _LibreDWG.new_Dwg_CONTEXTDATA_dict())
    __swig_destroy__ = _LibreDWG.delete_Dwg_CONTEXTDATA_dict

# Register Dwg_CONTEXTDATA_dict in _LibreDWG:
_LibreDWG.Dwg_CONTEXTDATA_dict_swigregister(Dwg_CONTEXTDATA_dict)
class Dwg_CONTEXTDATA_submgr(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_CONTEXTDATA_submgr_parent_get, _LibreDWG.Dwg_CONTEXTDATA_submgr_parent_set)
    handle = property(_LibreDWG.Dwg_CONTEXTDATA_submgr_handle_get, _LibreDWG.Dwg_CONTEXTDATA_submgr_handle_set)
    num_entries = property(_LibreDWG.Dwg_CONTEXTDATA_submgr_num_entries_get, _LibreDWG.Dwg_CONTEXTDATA_submgr_num_entries_set)
    entries = property(_LibreDWG.Dwg_CONTEXTDATA_submgr_entries_get, _LibreDWG.Dwg_CONTEXTDATA_submgr_entries_set)

    def __init__(self):
        _LibreDWG.Dwg_CONTEXTDATA_submgr_swiginit(self, _LibreDWG.new_Dwg_CONTEXTDATA_submgr())
    __swig_destroy__ = _LibreDWG.delete_Dwg_CONTEXTDATA_submgr

# Register Dwg_CONTEXTDATA_submgr in _LibreDWG:
_LibreDWG.Dwg_CONTEXTDATA_submgr_swigregister(Dwg_CONTEXTDATA_submgr)
class Dwg_Object_CONTEXTDATAMANAGER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_CONTEXTDATAMANAGER_parent_get, _LibreDWG.Dwg_Object_CONTEXTDATAMANAGER_parent_set)
    objectcontext = property(_LibreDWG.Dwg_Object_CONTEXTDATAMANAGER_objectcontext_get, _LibreDWG.Dwg_Object_CONTEXTDATAMANAGER_objectcontext_set)
    num_submgrs = property(_LibreDWG.Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_get, _LibreDWG.Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_set)
    submgrs = property(_LibreDWG.Dwg_Object_CONTEXTDATAMANAGER_submgrs_get, _LibreDWG.Dwg_Object_CONTEXTDATAMANAGER_submgrs_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_CONTEXTDATAMANAGER_swiginit(self, _LibreDWG.new_Dwg_Object_CONTEXTDATAMANAGER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_CONTEXTDATAMANAGER

# Register Dwg_Object_CONTEXTDATAMANAGER in _LibreDWG:
_LibreDWG.Dwg_Object_CONTEXTDATAMANAGER_swigregister(Dwg_Object_CONTEXTDATAMANAGER)
class Dwg_Object_TEXTOBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_set)
    horizontal_mode = property(_LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_get, _LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_set)
    rotation = property(_LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_get, _LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_set)
    ins_pt = property(_LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_get, _LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_set)
    alignment_pt = property(_LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_get, _LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_TEXTOBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_TEXTOBJECTCONTEXTDATA

# Register Dwg_Object_TEXTOBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_TEXTOBJECTCONTEXTDATA_swigregister(Dwg_Object_TEXTOBJECTCONTEXTDATA)
class Dwg_Object_MTEXTOBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_set)
    attachment = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_set)
    ins_pt = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_set)
    x_axis_dir = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_set)
    rect_height = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_set)
    rect_width = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_set)
    extents_width = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_set)
    extents_height = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_set)
    column_type = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_set)
    column_width = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_set)
    gutter = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_set)
    auto_height = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_set)
    flow_reversed = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_set)
    num_column_heights = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_set)
    column_heights = property(_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_get, _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_MTEXTOBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_MTEXTOBJECTCONTEXTDATA

# Register Dwg_Object_MTEXTOBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_MTEXTOBJECTCONTEXTDATA_swigregister(Dwg_Object_MTEXTOBJECTCONTEXTDATA)
class Dwg_OCD_Dimension(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    b293 = property(_LibreDWG.Dwg_OCD_Dimension_b293_get, _LibreDWG.Dwg_OCD_Dimension_b293_set)
    def_pt = property(_LibreDWG.Dwg_OCD_Dimension_def_pt_get, _LibreDWG.Dwg_OCD_Dimension_def_pt_set)
    is_def_textloc = property(_LibreDWG.Dwg_OCD_Dimension_is_def_textloc_get, _LibreDWG.Dwg_OCD_Dimension_is_def_textloc_set)
    text_rotation = property(_LibreDWG.Dwg_OCD_Dimension_text_rotation_get, _LibreDWG.Dwg_OCD_Dimension_text_rotation_set)
    block = property(_LibreDWG.Dwg_OCD_Dimension_block_get, _LibreDWG.Dwg_OCD_Dimension_block_set)
    dimtofl = property(_LibreDWG.Dwg_OCD_Dimension_dimtofl_get, _LibreDWG.Dwg_OCD_Dimension_dimtofl_set)
    dimosxd = property(_LibreDWG.Dwg_OCD_Dimension_dimosxd_get, _LibreDWG.Dwg_OCD_Dimension_dimosxd_set)
    dimatfit = property(_LibreDWG.Dwg_OCD_Dimension_dimatfit_get, _LibreDWG.Dwg_OCD_Dimension_dimatfit_set)
    dimtix = property(_LibreDWG.Dwg_OCD_Dimension_dimtix_get, _LibreDWG.Dwg_OCD_Dimension_dimtix_set)
    dimtmove = property(_LibreDWG.Dwg_OCD_Dimension_dimtmove_get, _LibreDWG.Dwg_OCD_Dimension_dimtmove_set)
    override_code = property(_LibreDWG.Dwg_OCD_Dimension_override_code_get, _LibreDWG.Dwg_OCD_Dimension_override_code_set)
    has_arrow2 = property(_LibreDWG.Dwg_OCD_Dimension_has_arrow2_get, _LibreDWG.Dwg_OCD_Dimension_has_arrow2_set)
    flip_arrow2 = property(_LibreDWG.Dwg_OCD_Dimension_flip_arrow2_get, _LibreDWG.Dwg_OCD_Dimension_flip_arrow2_set)
    flip_arrow1 = property(_LibreDWG.Dwg_OCD_Dimension_flip_arrow1_get, _LibreDWG.Dwg_OCD_Dimension_flip_arrow1_set)

    def __init__(self):
        _LibreDWG.Dwg_OCD_Dimension_swiginit(self, _LibreDWG.new_Dwg_OCD_Dimension())
    __swig_destroy__ = _LibreDWG.delete_Dwg_OCD_Dimension

# Register Dwg_OCD_Dimension in _LibreDWG:
_LibreDWG.Dwg_OCD_Dimension_swigregister(Dwg_OCD_Dimension)
class Dwg_Object_ALDIMOBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_set)
    dimension = property(_LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_get, _LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_set)
    dimline_pt = property(_LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_get, _LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_ALDIMOBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ALDIMOBJECTCONTEXTDATA

# Register Dwg_Object_ALDIMOBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_ALDIMOBJECTCONTEXTDATA_swigregister(Dwg_Object_ALDIMOBJECTCONTEXTDATA)
class Dwg_Object_ANGDIMOBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_set)
    dimension = property(_LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_get, _LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_set)
    arc_pt = property(_LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_get, _LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_ANGDIMOBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ANGDIMOBJECTCONTEXTDATA

# Register Dwg_Object_ANGDIMOBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_ANGDIMOBJECTCONTEXTDATA_swigregister(Dwg_Object_ANGDIMOBJECTCONTEXTDATA)
class Dwg_Object_DMDIMOBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_set)
    dimension = property(_LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_get, _LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_set)
    first_arc_pt = property(_LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_get, _LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_set)
    def_pt = property(_LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_get, _LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_DMDIMOBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DMDIMOBJECTCONTEXTDATA

# Register Dwg_Object_DMDIMOBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_DMDIMOBJECTCONTEXTDATA_swigregister(Dwg_Object_DMDIMOBJECTCONTEXTDATA)
class Dwg_Object_ORDDIMOBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_set)
    dimension = property(_LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_get, _LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_set)
    feature_location_pt = property(_LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_get, _LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_set)
    leader_endpt = property(_LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_get, _LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_ORDDIMOBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ORDDIMOBJECTCONTEXTDATA

# Register Dwg_Object_ORDDIMOBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_ORDDIMOBJECTCONTEXTDATA_swigregister(Dwg_Object_ORDDIMOBJECTCONTEXTDATA)
class Dwg_Object_RADIMOBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_set)
    dimension = property(_LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_get, _LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_set)
    first_arc_pt = property(_LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_get, _LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_RADIMOBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_RADIMOBJECTCONTEXTDATA

# Register Dwg_Object_RADIMOBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_RADIMOBJECTCONTEXTDATA_swigregister(Dwg_Object_RADIMOBJECTCONTEXTDATA)
class Dwg_Object_RADIMLGOBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_set)
    dimension = property(_LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_get, _LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_set)
    ovr_center = property(_LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_get, _LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_set)
    jog_point = property(_LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_get, _LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_RADIMLGOBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_RADIMLGOBJECTCONTEXTDATA

# Register Dwg_Object_RADIMLGOBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_RADIMLGOBJECTCONTEXTDATA_swigregister(Dwg_Object_RADIMLGOBJECTCONTEXTDATA)
class Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_set)
    horizontal_mode = property(_LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_get, _LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_set)
    rotation = property(_LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_get, _LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_set)
    ins_pt = property(_LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_get, _LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_set)
    alignment_pt = property(_LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_get, _LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_set)
    enable_context = property(_LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_get, _LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_set)
    context = property(_LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_get, _LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA

# Register Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_swigregister(Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA)
class Dwg_Object_MLEADEROBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_MLEADEROBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_MLEADEROBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_MLEADEROBJECTCONTEXTDATA

# Register Dwg_Object_MLEADEROBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_MLEADEROBJECTCONTEXTDATA_swigregister(Dwg_Object_MLEADEROBJECTCONTEXTDATA)
class Dwg_Object_LEADEROBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_set)
    num_points = property(_LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_get, _LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_set)
    points = property(_LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_points_get, _LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_points_set)
    b290 = property(_LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_get, _LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_set)
    x_direction = property(_LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_get, _LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_set)
    inspt_offset = property(_LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_get, _LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_set)
    endptproj = property(_LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_get, _LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_LEADEROBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_LEADEROBJECTCONTEXTDATA

# Register Dwg_Object_LEADEROBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_LEADEROBJECTCONTEXTDATA_swigregister(Dwg_Object_LEADEROBJECTCONTEXTDATA)
class Dwg_Object_BLKREFOBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_set)
    rotation = property(_LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_get, _LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_set)
    ins_pt = property(_LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_get, _LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_set)
    scale_factor = property(_LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_get, _LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_BLKREFOBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLKREFOBJECTCONTEXTDATA

# Register Dwg_Object_BLKREFOBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_BLKREFOBJECTCONTEXTDATA_swigregister(Dwg_Object_BLKREFOBJECTCONTEXTDATA)
class Dwg_Object_FCFOBJECTCONTEXTDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_parent_get, _LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_get, _LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_set)
    is_default = property(_LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_get, _LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_set)
    scale = property(_LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_scale_get, _LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_scale_set)
    location = property(_LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_location_get, _LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_location_set)
    horiz_dir = property(_LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_get, _LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_swiginit(self, _LibreDWG.new_Dwg_Object_FCFOBJECTCONTEXTDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_FCFOBJECTCONTEXTDATA

# Register Dwg_Object_FCFOBJECTCONTEXTDATA in _LibreDWG:
_LibreDWG.Dwg_Object_FCFOBJECTCONTEXTDATA_swigregister(Dwg_Object_FCFOBJECTCONTEXTDATA)
class Dwg_Object_DETAILVIEWSTYLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_parent_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_parent_set)
    mdoc_class_version = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_set)
    desc = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_desc_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_desc_set)
    is_modified_for_recompute = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_set)
    display_name = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_display_name_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_display_name_set)
    viewstyle_flags = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_set)
    class_version = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_class_version_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_class_version_set)
    flags = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_flags_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_flags_set)
    identifier_style = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_identifier_style_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_identifier_style_set)
    identifier_color = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_identifier_color_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_identifier_color_set)
    identifier_height = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_identifier_height_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_identifier_height_set)
    identifier_exclude_characters = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_set)
    identifier_offset = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_identifier_offset_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_identifier_offset_set)
    identifier_placement = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_identifier_placement_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_identifier_placement_set)
    arrow_symbol = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_set)
    arrow_symbol_color = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_set)
    arrow_symbol_size = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_set)
    boundary_ltype = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_set)
    boundary_linewt = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_set)
    boundary_line_color = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_set)
    viewlabel_text_style = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_set)
    viewlabel_text_color = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_set)
    viewlabel_text_height = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_set)
    viewlabel_attachment = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_set)
    viewlabel_offset = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_set)
    viewlabel_alignment = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_set)
    viewlabel_pattern = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_set)
    connection_ltype = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_connection_ltype_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_connection_ltype_set)
    connection_linewt = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_connection_linewt_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_connection_linewt_set)
    connection_line_color = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_connection_line_color_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_connection_line_color_set)
    borderline_ltype = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_set)
    borderline_linewt = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_set)
    borderline_color = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_borderline_color_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_borderline_color_set)
    model_edge = property(_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_model_edge_get, _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_model_edge_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DETAILVIEWSTYLE_swiginit(self, _LibreDWG.new_Dwg_Object_DETAILVIEWSTYLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DETAILVIEWSTYLE

# Register Dwg_Object_DETAILVIEWSTYLE in _LibreDWG:
_LibreDWG.Dwg_Object_DETAILVIEWSTYLE_swigregister(Dwg_Object_DETAILVIEWSTYLE)
class Dwg_Object_SECTIONVIEWSTYLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_parent_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_parent_set)
    mdoc_class_version = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_set)
    desc = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_desc_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_desc_set)
    is_modified_for_recompute = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_set)
    display_name = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_display_name_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_display_name_set)
    viewstyle_flags = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_set)
    class_version = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_class_version_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_class_version_set)
    flags = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_flags_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_flags_set)
    identifier_style = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_identifier_style_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_identifier_style_set)
    identifier_color = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_identifier_color_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_identifier_color_set)
    identifier_height = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_identifier_height_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_identifier_height_set)
    arrow_start_symbol = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_set)
    arrow_end_symbol = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_set)
    arrow_symbol_color = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_set)
    arrow_symbol_size = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_set)
    identifier_exclude_characters = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_set)
    identifier_position = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_identifier_position_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_identifier_position_set)
    identifier_offset = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_set)
    arrow_position = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_arrow_position_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_arrow_position_set)
    arrow_symbol_extension_length = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_set)
    plane_ltype = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_set)
    plane_linewt = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_set)
    plane_line_color = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_set)
    bend_ltype = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_set)
    bend_linewt = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_set)
    bend_line_color = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_set)
    bend_line_length = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_set)
    end_line_overshoot = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_set)
    end_line_length = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_end_line_length_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_end_line_length_set)
    viewlabel_text_style = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_set)
    viewlabel_text_color = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_set)
    viewlabel_text_height = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_set)
    viewlabel_attachment = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_set)
    viewlabel_offset = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_set)
    viewlabel_alignment = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_set)
    viewlabel_pattern = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_set)
    hatch_color = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_hatch_color_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_hatch_color_set)
    hatch_bg_color = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_set)
    hatch_pattern = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_set)
    hatch_scale = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_set)
    hatch_transparency = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_set)
    unknown_b1 = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_set)
    unknown_b2 = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_set)
    num_hatch_angles = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_set)
    hatch_angles = property(_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_get, _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_swiginit(self, _LibreDWG.new_Dwg_Object_SECTIONVIEWSTYLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_SECTIONVIEWSTYLE

# Register Dwg_Object_SECTIONVIEWSTYLE in _LibreDWG:
_LibreDWG.Dwg_Object_SECTIONVIEWSTYLE_swigregister(Dwg_Object_SECTIONVIEWSTYLE)
class Dwg_Object_SECTION_MANAGER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_SECTION_MANAGER_parent_get, _LibreDWG.Dwg_Object_SECTION_MANAGER_parent_set)
    is_live = property(_LibreDWG.Dwg_Object_SECTION_MANAGER_is_live_get, _LibreDWG.Dwg_Object_SECTION_MANAGER_is_live_set)
    num_sections = property(_LibreDWG.Dwg_Object_SECTION_MANAGER_num_sections_get, _LibreDWG.Dwg_Object_SECTION_MANAGER_num_sections_set)
    sections = property(_LibreDWG.Dwg_Object_SECTION_MANAGER_sections_get, _LibreDWG.Dwg_Object_SECTION_MANAGER_sections_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_SECTION_MANAGER_swiginit(self, _LibreDWG.new_Dwg_Object_SECTION_MANAGER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_SECTION_MANAGER

# Register Dwg_Object_SECTION_MANAGER in _LibreDWG:
_LibreDWG.Dwg_Object_SECTION_MANAGER_swigregister(Dwg_Object_SECTION_MANAGER)
class Dwg_SECTION_geometrysettings(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_SECTION_geometrysettings_parent_get, _LibreDWG.Dwg_SECTION_geometrysettings_parent_set)
    num_geoms = property(_LibreDWG.Dwg_SECTION_geometrysettings_num_geoms_get, _LibreDWG.Dwg_SECTION_geometrysettings_num_geoms_set)
    hexindex = property(_LibreDWG.Dwg_SECTION_geometrysettings_hexindex_get, _LibreDWG.Dwg_SECTION_geometrysettings_hexindex_set)
    flags = property(_LibreDWG.Dwg_SECTION_geometrysettings_flags_get, _LibreDWG.Dwg_SECTION_geometrysettings_flags_set)
    color = property(_LibreDWG.Dwg_SECTION_geometrysettings_color_get, _LibreDWG.Dwg_SECTION_geometrysettings_color_set)
    layer = property(_LibreDWG.Dwg_SECTION_geometrysettings_layer_get, _LibreDWG.Dwg_SECTION_geometrysettings_layer_set)
    ltype = property(_LibreDWG.Dwg_SECTION_geometrysettings_ltype_get, _LibreDWG.Dwg_SECTION_geometrysettings_ltype_set)
    ltype_scale = property(_LibreDWG.Dwg_SECTION_geometrysettings_ltype_scale_get, _LibreDWG.Dwg_SECTION_geometrysettings_ltype_scale_set)
    plotstyle = property(_LibreDWG.Dwg_SECTION_geometrysettings_plotstyle_get, _LibreDWG.Dwg_SECTION_geometrysettings_plotstyle_set)
    linewt = property(_LibreDWG.Dwg_SECTION_geometrysettings_linewt_get, _LibreDWG.Dwg_SECTION_geometrysettings_linewt_set)
    face_transparency = property(_LibreDWG.Dwg_SECTION_geometrysettings_face_transparency_get, _LibreDWG.Dwg_SECTION_geometrysettings_face_transparency_set)
    edge_transparency = property(_LibreDWG.Dwg_SECTION_geometrysettings_edge_transparency_get, _LibreDWG.Dwg_SECTION_geometrysettings_edge_transparency_set)
    hatch_type = property(_LibreDWG.Dwg_SECTION_geometrysettings_hatch_type_get, _LibreDWG.Dwg_SECTION_geometrysettings_hatch_type_set)
    hatch_pattern = property(_LibreDWG.Dwg_SECTION_geometrysettings_hatch_pattern_get, _LibreDWG.Dwg_SECTION_geometrysettings_hatch_pattern_set)
    hatch_angle = property(_LibreDWG.Dwg_SECTION_geometrysettings_hatch_angle_get, _LibreDWG.Dwg_SECTION_geometrysettings_hatch_angle_set)
    hatch_spacing = property(_LibreDWG.Dwg_SECTION_geometrysettings_hatch_spacing_get, _LibreDWG.Dwg_SECTION_geometrysettings_hatch_spacing_set)
    hatch_scale = property(_LibreDWG.Dwg_SECTION_geometrysettings_hatch_scale_get, _LibreDWG.Dwg_SECTION_geometrysettings_hatch_scale_set)

    def __init__(self):
        _LibreDWG.Dwg_SECTION_geometrysettings_swiginit(self, _LibreDWG.new_Dwg_SECTION_geometrysettings())
    __swig_destroy__ = _LibreDWG.delete_Dwg_SECTION_geometrysettings

# Register Dwg_SECTION_geometrysettings in _LibreDWG:
_LibreDWG.Dwg_SECTION_geometrysettings_swigregister(Dwg_SECTION_geometrysettings)
class Dwg_SECTION_typesettings(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_SECTION_typesettings_parent_get, _LibreDWG.Dwg_SECTION_typesettings_parent_set)
    type = property(_LibreDWG.Dwg_SECTION_typesettings_type_get, _LibreDWG.Dwg_SECTION_typesettings_type_set)
    generation = property(_LibreDWG.Dwg_SECTION_typesettings_generation_get, _LibreDWG.Dwg_SECTION_typesettings_generation_set)
    num_sources = property(_LibreDWG.Dwg_SECTION_typesettings_num_sources_get, _LibreDWG.Dwg_SECTION_typesettings_num_sources_set)
    sources = property(_LibreDWG.Dwg_SECTION_typesettings_sources_get, _LibreDWG.Dwg_SECTION_typesettings_sources_set)
    destblock = property(_LibreDWG.Dwg_SECTION_typesettings_destblock_get, _LibreDWG.Dwg_SECTION_typesettings_destblock_set)
    destfile = property(_LibreDWG.Dwg_SECTION_typesettings_destfile_get, _LibreDWG.Dwg_SECTION_typesettings_destfile_set)
    num_geom = property(_LibreDWG.Dwg_SECTION_typesettings_num_geom_get, _LibreDWG.Dwg_SECTION_typesettings_num_geom_set)
    geom = property(_LibreDWG.Dwg_SECTION_typesettings_geom_get, _LibreDWG.Dwg_SECTION_typesettings_geom_set)

    def __init__(self):
        _LibreDWG.Dwg_SECTION_typesettings_swiginit(self, _LibreDWG.new_Dwg_SECTION_typesettings())
    __swig_destroy__ = _LibreDWG.delete_Dwg_SECTION_typesettings

# Register Dwg_SECTION_typesettings in _LibreDWG:
_LibreDWG.Dwg_SECTION_typesettings_swigregister(Dwg_SECTION_typesettings)
class Dwg_Object_SECTION_SETTINGS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_SECTION_SETTINGS_parent_get, _LibreDWG.Dwg_Object_SECTION_SETTINGS_parent_set)
    curr_type = property(_LibreDWG.Dwg_Object_SECTION_SETTINGS_curr_type_get, _LibreDWG.Dwg_Object_SECTION_SETTINGS_curr_type_set)
    num_types = property(_LibreDWG.Dwg_Object_SECTION_SETTINGS_num_types_get, _LibreDWG.Dwg_Object_SECTION_SETTINGS_num_types_set)
    types = property(_LibreDWG.Dwg_Object_SECTION_SETTINGS_types_get, _LibreDWG.Dwg_Object_SECTION_SETTINGS_types_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_SECTION_SETTINGS_swiginit(self, _LibreDWG.new_Dwg_Object_SECTION_SETTINGS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_SECTION_SETTINGS

# Register Dwg_Object_SECTION_SETTINGS in _LibreDWG:
_LibreDWG.Dwg_Object_SECTION_SETTINGS_swigregister(Dwg_Object_SECTION_SETTINGS)
class Dwg_Object_LAYERFILTER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_LAYERFILTER_parent_get, _LibreDWG.Dwg_Object_LAYERFILTER_parent_set)
    num_names = property(_LibreDWG.Dwg_Object_LAYERFILTER_num_names_get, _LibreDWG.Dwg_Object_LAYERFILTER_num_names_set)
    names = property(_LibreDWG.Dwg_Object_LAYERFILTER_names_get, _LibreDWG.Dwg_Object_LAYERFILTER_names_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_LAYERFILTER_swiginit(self, _LibreDWG.new_Dwg_Object_LAYERFILTER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_LAYERFILTER

# Register Dwg_Object_LAYERFILTER in _LibreDWG:
_LibreDWG.Dwg_Object_LAYERFILTER_swigregister(Dwg_Object_LAYERFILTER)
class Dwg_Entity_ARCALIGNEDTEXT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_parent_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_parent_set)
    text_size = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_text_size_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_text_size_set)
    xscale = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_xscale_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_xscale_set)
    char_spacing = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_char_spacing_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_char_spacing_set)
    style = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_style_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_style_set)
    text_value = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_text_value_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_text_value_set)
    font_name = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_font_name_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_font_name_set)
    bigfont_name = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_set)
    offset_from_arc = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_set)
    right_offset = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_right_offset_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_right_offset_set)
    left_offset = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_left_offset_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_left_offset_set)
    center = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_center_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_center_set)
    radius = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_radius_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_radius_set)
    start_angle = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_start_angle_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_start_angle_set)
    end_angle = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_end_angle_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_end_angle_set)
    extrusion = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_extrusion_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_extrusion_set)
    color = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_color_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_color_set)
    is_reverse = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_is_reverse_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_is_reverse_set)
    text_direction = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_text_direction_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_text_direction_set)
    alignment = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_alignment_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_alignment_set)
    text_position = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_text_position_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_text_position_set)
    is_bold = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_is_bold_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_is_bold_set)
    is_italic = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_is_italic_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_is_italic_set)
    is_underlined = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_is_underlined_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_is_underlined_set)
    character_set = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_character_set_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_character_set_set)
    pitch_and_family = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_set)
    is_shx = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_is_shx_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_is_shx_set)
    wizard_flag = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_set)
    arc_handle = property(_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_arc_handle_get, _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_arc_handle_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_swiginit(self, _LibreDWG.new_Dwg_Entity_ARCALIGNEDTEXT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_ARCALIGNEDTEXT

# Register Dwg_Entity_ARCALIGNEDTEXT in _LibreDWG:
_LibreDWG.Dwg_Entity_ARCALIGNEDTEXT_swigregister(Dwg_Entity_ARCALIGNEDTEXT)
class Dwg_Entity_RTEXT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_RTEXT_parent_get, _LibreDWG.Dwg_Entity_RTEXT_parent_set)
    pt = property(_LibreDWG.Dwg_Entity_RTEXT_pt_get, _LibreDWG.Dwg_Entity_RTEXT_pt_set)
    extrusion = property(_LibreDWG.Dwg_Entity_RTEXT_extrusion_get, _LibreDWG.Dwg_Entity_RTEXT_extrusion_set)
    rotation = property(_LibreDWG.Dwg_Entity_RTEXT_rotation_get, _LibreDWG.Dwg_Entity_RTEXT_rotation_set)
    height = property(_LibreDWG.Dwg_Entity_RTEXT_height_get, _LibreDWG.Dwg_Entity_RTEXT_height_set)
    flags = property(_LibreDWG.Dwg_Entity_RTEXT_flags_get, _LibreDWG.Dwg_Entity_RTEXT_flags_set)
    text_value = property(_LibreDWG.Dwg_Entity_RTEXT_text_value_get, _LibreDWG.Dwg_Entity_RTEXT_text_value_set)
    style = property(_LibreDWG.Dwg_Entity_RTEXT_style_get, _LibreDWG.Dwg_Entity_RTEXT_style_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_RTEXT_swiginit(self, _LibreDWG.new_Dwg_Entity_RTEXT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_RTEXT

# Register Dwg_Entity_RTEXT in _LibreDWG:
_LibreDWG.Dwg_Entity_RTEXT_swigregister(Dwg_Entity_RTEXT)
class Dwg_Entity_LAYOUTPRINTCONFIG(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_LAYOUTPRINTCONFIG_parent_get, _LibreDWG.Dwg_Entity_LAYOUTPRINTCONFIG_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_LAYOUTPRINTCONFIG_class_version_get, _LibreDWG.Dwg_Entity_LAYOUTPRINTCONFIG_class_version_set)
    flag = property(_LibreDWG.Dwg_Entity_LAYOUTPRINTCONFIG_flag_get, _LibreDWG.Dwg_Entity_LAYOUTPRINTCONFIG_flag_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_LAYOUTPRINTCONFIG_swiginit(self, _LibreDWG.new_Dwg_Entity_LAYOUTPRINTCONFIG())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_LAYOUTPRINTCONFIG

# Register Dwg_Entity_LAYOUTPRINTCONFIG in _LibreDWG:
_LibreDWG.Dwg_Entity_LAYOUTPRINTCONFIG_swigregister(Dwg_Entity_LAYOUTPRINTCONFIG)
class Dwg_Object_ACMECOMMANDHISTORY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACMECOMMANDHISTORY_parent_get, _LibreDWG.Dwg_Object_ACMECOMMANDHISTORY_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_ACMECOMMANDHISTORY_class_version_get, _LibreDWG.Dwg_Object_ACMECOMMANDHISTORY_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACMECOMMANDHISTORY_swiginit(self, _LibreDWG.new_Dwg_Object_ACMECOMMANDHISTORY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACMECOMMANDHISTORY

# Register Dwg_Object_ACMECOMMANDHISTORY in _LibreDWG:
_LibreDWG.Dwg_Object_ACMECOMMANDHISTORY_swigregister(Dwg_Object_ACMECOMMANDHISTORY)
class Dwg_Object_ACMESCOPE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACMESCOPE_parent_get, _LibreDWG.Dwg_Object_ACMESCOPE_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_ACMESCOPE_class_version_get, _LibreDWG.Dwg_Object_ACMESCOPE_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACMESCOPE_swiginit(self, _LibreDWG.new_Dwg_Object_ACMESCOPE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACMESCOPE

# Register Dwg_Object_ACMESCOPE in _LibreDWG:
_LibreDWG.Dwg_Object_ACMESCOPE_swigregister(Dwg_Object_ACMESCOPE)
class Dwg_Object_ACMESTATEMGR(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_ACMESTATEMGR_parent_get, _LibreDWG.Dwg_Object_ACMESTATEMGR_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_ACMESTATEMGR_class_version_get, _LibreDWG.Dwg_Object_ACMESTATEMGR_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_ACMESTATEMGR_swiginit(self, _LibreDWG.new_Dwg_Object_ACMESTATEMGR())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_ACMESTATEMGR

# Register Dwg_Object_ACMESTATEMGR in _LibreDWG:
_LibreDWG.Dwg_Object_ACMESTATEMGR_swigregister(Dwg_Object_ACMESTATEMGR)
class Dwg_Object_CSACDOCUMENTOPTIONS(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_CSACDOCUMENTOPTIONS_parent_get, _LibreDWG.Dwg_Object_CSACDOCUMENTOPTIONS_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_CSACDOCUMENTOPTIONS_class_version_get, _LibreDWG.Dwg_Object_CSACDOCUMENTOPTIONS_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_CSACDOCUMENTOPTIONS_swiginit(self, _LibreDWG.new_Dwg_Object_CSACDOCUMENTOPTIONS())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_CSACDOCUMENTOPTIONS

# Register Dwg_Object_CSACDOCUMENTOPTIONS in _LibreDWG:
_LibreDWG.Dwg_Object_CSACDOCUMENTOPTIONS_swigregister(Dwg_Object_CSACDOCUMENTOPTIONS)
class Dwg_BLOCKPARAMETER_connection(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    code = property(_LibreDWG.Dwg_BLOCKPARAMETER_connection_code_get, _LibreDWG.Dwg_BLOCKPARAMETER_connection_code_set)
    name = property(_LibreDWG.Dwg_BLOCKPARAMETER_connection_name_get, _LibreDWG.Dwg_BLOCKPARAMETER_connection_name_set)

    def __init__(self):
        _LibreDWG.Dwg_BLOCKPARAMETER_connection_swiginit(self, _LibreDWG.new_Dwg_BLOCKPARAMETER_connection())
    __swig_destroy__ = _LibreDWG.delete_Dwg_BLOCKPARAMETER_connection

# Register Dwg_BLOCKPARAMETER_connection in _LibreDWG:
_LibreDWG.Dwg_BLOCKPARAMETER_connection_swigregister(Dwg_BLOCKPARAMETER_connection)
class Dwg_BLOCKPARAMETER_PropInfo(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num_connections = property(_LibreDWG.Dwg_BLOCKPARAMETER_PropInfo_num_connections_get, _LibreDWG.Dwg_BLOCKPARAMETER_PropInfo_num_connections_set)
    connections = property(_LibreDWG.Dwg_BLOCKPARAMETER_PropInfo_connections_get, _LibreDWG.Dwg_BLOCKPARAMETER_PropInfo_connections_set)

    def __init__(self):
        _LibreDWG.Dwg_BLOCKPARAMETER_PropInfo_swiginit(self, _LibreDWG.new_Dwg_BLOCKPARAMETER_PropInfo())
    __swig_destroy__ = _LibreDWG.delete_Dwg_BLOCKPARAMETER_PropInfo

# Register Dwg_BLOCKPARAMETER_PropInfo in _LibreDWG:
_LibreDWG.Dwg_BLOCKPARAMETER_PropInfo_swigregister(Dwg_BLOCKPARAMETER_PropInfo)
class Dwg_BLOCKPARAMVALUESET(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    desc = property(_LibreDWG.Dwg_BLOCKPARAMVALUESET_desc_get, _LibreDWG.Dwg_BLOCKPARAMVALUESET_desc_set)
    flags = property(_LibreDWG.Dwg_BLOCKPARAMVALUESET_flags_get, _LibreDWG.Dwg_BLOCKPARAMVALUESET_flags_set)
    minimum = property(_LibreDWG.Dwg_BLOCKPARAMVALUESET_minimum_get, _LibreDWG.Dwg_BLOCKPARAMVALUESET_minimum_set)
    maximum = property(_LibreDWG.Dwg_BLOCKPARAMVALUESET_maximum_get, _LibreDWG.Dwg_BLOCKPARAMVALUESET_maximum_set)
    increment = property(_LibreDWG.Dwg_BLOCKPARAMVALUESET_increment_get, _LibreDWG.Dwg_BLOCKPARAMVALUESET_increment_set)
    num_valuelist = property(_LibreDWG.Dwg_BLOCKPARAMVALUESET_num_valuelist_get, _LibreDWG.Dwg_BLOCKPARAMVALUESET_num_valuelist_set)
    valuelist = property(_LibreDWG.Dwg_BLOCKPARAMVALUESET_valuelist_get, _LibreDWG.Dwg_BLOCKPARAMVALUESET_valuelist_set)

    def __init__(self):
        _LibreDWG.Dwg_BLOCKPARAMVALUESET_swiginit(self, _LibreDWG.new_Dwg_BLOCKPARAMVALUESET())
    __swig_destroy__ = _LibreDWG.delete_Dwg_BLOCKPARAMVALUESET

# Register Dwg_BLOCKPARAMVALUESET in _LibreDWG:
_LibreDWG.Dwg_BLOCKPARAMVALUESET_swigregister(Dwg_BLOCKPARAMVALUESET)
class Dwg_BLOCKACTION_connectionpts(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    code = property(_LibreDWG.Dwg_BLOCKACTION_connectionpts_code_get, _LibreDWG.Dwg_BLOCKACTION_connectionpts_code_set)
    name = property(_LibreDWG.Dwg_BLOCKACTION_connectionpts_name_get, _LibreDWG.Dwg_BLOCKACTION_connectionpts_name_set)

    def __init__(self):
        _LibreDWG.Dwg_BLOCKACTION_connectionpts_swiginit(self, _LibreDWG.new_Dwg_BLOCKACTION_connectionpts())
    __swig_destroy__ = _LibreDWG.delete_Dwg_BLOCKACTION_connectionpts

# Register Dwg_BLOCKACTION_connectionpts in _LibreDWG:
_LibreDWG.Dwg_BLOCKACTION_connectionpts_swigregister(Dwg_BLOCKACTION_connectionpts)
class Dwg_BLOCKVISIBILITYPARAMETER_state(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_parent_get, _LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_parent_set)
    name = property(_LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_name_get, _LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_name_set)
    num_blocks = property(_LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_get, _LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_set)
    blocks = property(_LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_get, _LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_set)
    num_params = property(_LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_get, _LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_set)
    params = property(_LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_params_get, _LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_params_set)

    def __init__(self):
        _LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_swiginit(self, _LibreDWG.new_Dwg_BLOCKVISIBILITYPARAMETER_state())
    __swig_destroy__ = _LibreDWG.delete_Dwg_BLOCKVISIBILITYPARAMETER_state

# Register Dwg_BLOCKVISIBILITYPARAMETER_state in _LibreDWG:
_LibreDWG.Dwg_BLOCKVISIBILITYPARAMETER_state_swigregister(Dwg_BLOCKVISIBILITYPARAMETER_state)
class Dwg_Object_BLOCKVISIBILITYPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_set)
    def_pt = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_set)
    num_propinfos = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_set)
    is_initialized = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_set)
    unknown_bool = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_set)
    blockvisi_name = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_set)
    blockvisi_desc = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_set)
    num_blocks = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_set)
    blocks = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_set)
    num_states = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_set)
    states = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_states_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_states_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKVISIBILITYPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKVISIBILITYPARAMETER

# Register Dwg_Object_BLOCKVISIBILITYPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKVISIBILITYPARAMETER_swigregister(Dwg_Object_BLOCKVISIBILITYPARAMETER)
class Dwg_Object_BLOCKVISIBILITYGRIP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_parent_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_name_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_be_major_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_set)
    bg_bl91 = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_set)
    bg_bl92 = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_set)
    bg_location = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_set)
    bg_insert_cycling = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_set)
    bg_insert_cycling_weight = property(_LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_get, _LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKVISIBILITYGRIP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKVISIBILITYGRIP

# Register Dwg_Object_BLOCKVISIBILITYGRIP in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKVISIBILITYGRIP_swigregister(Dwg_Object_BLOCKVISIBILITYGRIP)
class Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_get, _LibreDWG.Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_set)
    grip_type = property(_LibreDWG.Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_get, _LibreDWG.Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_set)
    grip_expr = property(_LibreDWG.Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_get, _LibreDWG.Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT

# Register Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_swigregister(Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT)
class Dwg_Object_BREAKDATA(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BREAKDATA_parent_get, _LibreDWG.Dwg_Object_BREAKDATA_parent_set)
    num_pointrefs = property(_LibreDWG.Dwg_Object_BREAKDATA_num_pointrefs_get, _LibreDWG.Dwg_Object_BREAKDATA_num_pointrefs_set)
    pointrefs = property(_LibreDWG.Dwg_Object_BREAKDATA_pointrefs_get, _LibreDWG.Dwg_Object_BREAKDATA_pointrefs_set)
    dimref = property(_LibreDWG.Dwg_Object_BREAKDATA_dimref_get, _LibreDWG.Dwg_Object_BREAKDATA_dimref_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BREAKDATA_swiginit(self, _LibreDWG.new_Dwg_Object_BREAKDATA())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BREAKDATA

# Register Dwg_Object_BREAKDATA in _LibreDWG:
_LibreDWG.Dwg_Object_BREAKDATA_swigregister(Dwg_Object_BREAKDATA)
class Dwg_Object_BREAKPOINTREF(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BREAKPOINTREF_parent_get, _LibreDWG.Dwg_Object_BREAKPOINTREF_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BREAKPOINTREF_swiginit(self, _LibreDWG.new_Dwg_Object_BREAKPOINTREF())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BREAKPOINTREF

# Register Dwg_Object_BREAKPOINTREF in _LibreDWG:
_LibreDWG.Dwg_Object_BREAKPOINTREF_swigregister(Dwg_Object_BREAKPOINTREF)
class Dwg_Entity_FLIPGRIPENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_FLIPGRIPENTITY_parent_get, _LibreDWG.Dwg_Entity_FLIPGRIPENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_FLIPGRIPENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_FLIPGRIPENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_FLIPGRIPENTITY

# Register Dwg_Entity_FLIPGRIPENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_FLIPGRIPENTITY_swigregister(Dwg_Entity_FLIPGRIPENTITY)
class Dwg_Entity_LINEARGRIPENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_LINEARGRIPENTITY_parent_get, _LibreDWG.Dwg_Entity_LINEARGRIPENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_LINEARGRIPENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_LINEARGRIPENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_LINEARGRIPENTITY

# Register Dwg_Entity_LINEARGRIPENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_LINEARGRIPENTITY_swigregister(Dwg_Entity_LINEARGRIPENTITY)
class Dwg_Entity_POLARGRIPENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_POLARGRIPENTITY_parent_get, _LibreDWG.Dwg_Entity_POLARGRIPENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_POLARGRIPENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_POLARGRIPENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_POLARGRIPENTITY

# Register Dwg_Entity_POLARGRIPENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_POLARGRIPENTITY_swigregister(Dwg_Entity_POLARGRIPENTITY)
class Dwg_Entity_ROTATIONGRIPENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_ROTATIONGRIPENTITY_parent_get, _LibreDWG.Dwg_Entity_ROTATIONGRIPENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_ROTATIONGRIPENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_ROTATIONGRIPENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_ROTATIONGRIPENTITY

# Register Dwg_Entity_ROTATIONGRIPENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_ROTATIONGRIPENTITY_swigregister(Dwg_Entity_ROTATIONGRIPENTITY)
class Dwg_Entity_VISIBILITYGRIPENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_VISIBILITYGRIPENTITY_parent_get, _LibreDWG.Dwg_Entity_VISIBILITYGRIPENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_VISIBILITYGRIPENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_VISIBILITYGRIPENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_VISIBILITYGRIPENTITY

# Register Dwg_Entity_VISIBILITYGRIPENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_VISIBILITYGRIPENTITY_swigregister(Dwg_Entity_VISIBILITYGRIPENTITY)
class Dwg_Entity_XYGRIPENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_XYGRIPENTITY_parent_get, _LibreDWG.Dwg_Entity_XYGRIPENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_XYGRIPENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_XYGRIPENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_XYGRIPENTITY

# Register Dwg_Entity_XYGRIPENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_XYGRIPENTITY_swigregister(Dwg_Entity_XYGRIPENTITY)
class Dwg_Entity_ALIGNMENTPARAMETERENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_get, _LibreDWG.Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_ALIGNMENTPARAMETERENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_ALIGNMENTPARAMETERENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_ALIGNMENTPARAMETERENTITY

# Register Dwg_Entity_ALIGNMENTPARAMETERENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_ALIGNMENTPARAMETERENTITY_swigregister(Dwg_Entity_ALIGNMENTPARAMETERENTITY)
class Dwg_Entity_BASEPOINTPARAMETERENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_get, _LibreDWG.Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_BASEPOINTPARAMETERENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_BASEPOINTPARAMETERENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_BASEPOINTPARAMETERENTITY

# Register Dwg_Entity_BASEPOINTPARAMETERENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_BASEPOINTPARAMETERENTITY_swigregister(Dwg_Entity_BASEPOINTPARAMETERENTITY)
class Dwg_Entity_FLIPPARAMETERENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_FLIPPARAMETERENTITY_parent_get, _LibreDWG.Dwg_Entity_FLIPPARAMETERENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_FLIPPARAMETERENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_FLIPPARAMETERENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_FLIPPARAMETERENTITY

# Register Dwg_Entity_FLIPPARAMETERENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_FLIPPARAMETERENTITY_swigregister(Dwg_Entity_FLIPPARAMETERENTITY)
class Dwg_Entity_LINEARPARAMETERENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_LINEARPARAMETERENTITY_parent_get, _LibreDWG.Dwg_Entity_LINEARPARAMETERENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_LINEARPARAMETERENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_LINEARPARAMETERENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_LINEARPARAMETERENTITY

# Register Dwg_Entity_LINEARPARAMETERENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_LINEARPARAMETERENTITY_swigregister(Dwg_Entity_LINEARPARAMETERENTITY)
class Dwg_Entity_POINTPARAMETERENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_POINTPARAMETERENTITY_parent_get, _LibreDWG.Dwg_Entity_POINTPARAMETERENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_POINTPARAMETERENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_POINTPARAMETERENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_POINTPARAMETERENTITY

# Register Dwg_Entity_POINTPARAMETERENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_POINTPARAMETERENTITY_swigregister(Dwg_Entity_POINTPARAMETERENTITY)
class Dwg_Entity_ROTATIONPARAMETERENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_ROTATIONPARAMETERENTITY_parent_get, _LibreDWG.Dwg_Entity_ROTATIONPARAMETERENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_ROTATIONPARAMETERENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_ROTATIONPARAMETERENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_ROTATIONPARAMETERENTITY

# Register Dwg_Entity_ROTATIONPARAMETERENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_ROTATIONPARAMETERENTITY_swigregister(Dwg_Entity_ROTATIONPARAMETERENTITY)
class Dwg_Entity_VISIBILITYPARAMETERENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_get, _LibreDWG.Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_VISIBILITYPARAMETERENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_VISIBILITYPARAMETERENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_VISIBILITYPARAMETERENTITY

# Register Dwg_Entity_VISIBILITYPARAMETERENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_VISIBILITYPARAMETERENTITY_swigregister(Dwg_Entity_VISIBILITYPARAMETERENTITY)
class Dwg_Entity_XYPARAMETERENTITY(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_XYPARAMETERENTITY_parent_get, _LibreDWG.Dwg_Entity_XYPARAMETERENTITY_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_XYPARAMETERENTITY_swiginit(self, _LibreDWG.new_Dwg_Entity_XYPARAMETERENTITY())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_XYPARAMETERENTITY

# Register Dwg_Entity_XYPARAMETERENTITY in _LibreDWG:
_LibreDWG.Dwg_Entity_XYPARAMETERENTITY_swigregister(Dwg_Entity_XYPARAMETERENTITY)
class Dwg_Object_BLOCKALIGNMENTGRIP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_parent_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_name_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_be_major_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_set)
    bg_bl91 = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_set)
    bg_bl92 = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_set)
    bg_location = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_set)
    bg_insert_cycling = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_set)
    bg_insert_cycling_weight = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_set)
    orientation = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_orientation_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_orientation_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKALIGNMENTGRIP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKALIGNMENTGRIP

# Register Dwg_Object_BLOCKALIGNMENTGRIP in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKALIGNMENTGRIP_swigregister(Dwg_Object_BLOCKALIGNMENTGRIP)
class Dwg_Object_BLOCKALIGNMENTPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_set)
    def_basept = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_set)
    def_endpt = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_set)
    prop3 = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_set)
    prop4 = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_set)
    prop_states = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_set)
    parameter_base_location = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_set)
    upd_basept = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_set)
    basept = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_set)
    upd_endpt = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_set)
    endpt = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_set)
    align_perpendicular = property(_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_get, _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKALIGNMENTPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKALIGNMENTPARAMETER

# Register Dwg_Object_BLOCKALIGNMENTPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKALIGNMENTPARAMETER_swigregister(Dwg_Object_BLOCKALIGNMENTPARAMETER)
class Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_set)
    def_basept = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_set)
    def_endpt = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_set)
    prop3 = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_set)
    prop4 = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_set)
    prop_states = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_set)
    parameter_base_location = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_set)
    upd_basept = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_set)
    basept = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_set)
    upd_endpt = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_set)
    endpt = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_set)
    dependency = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_set)
    center_pt = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_set)
    end_pt = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_set)
    expr_name = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_set)
    expr_description = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_set)
    angle = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_set)
    orientation_on_both_grips = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_set)
    value_set = property(_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_get, _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER

# Register Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_swigregister(Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER)
class Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_set)
    def_basept = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_set)
    def_endpt = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_set)
    prop3 = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_set)
    prop4 = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_set)
    prop_states = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_set)
    parameter_base_location = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_set)
    upd_basept = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_set)
    basept = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_set)
    upd_endpt = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_set)
    endpt = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_set)
    dependency = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_set)
    expr_name = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_set)
    expr_description = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_set)
    distance = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_set)
    orientation_on_both_grips = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_set)
    value_set = property(_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_get, _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER

# Register Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_swigregister(Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER)
class Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_set)
    def_basept = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_set)
    def_endpt = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_set)
    prop3 = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_set)
    prop4 = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_set)
    prop_states = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_set)
    parameter_base_location = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_set)
    upd_basept = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_set)
    basept = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_set)
    upd_endpt = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_set)
    endpt = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_set)
    dependency = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_set)
    expr_name = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_set)
    expr_description = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_set)
    distance = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_set)
    value_set = property(_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_get, _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER

# Register Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_swigregister(Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER)
class Dwg_Object_BLOCKARRAYACTION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_parent_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_name_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_be_major_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_be_minor_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_eed1071_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_eed1071_set)
    display_location = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_display_location_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_display_location_set)
    num_actions = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_num_actions_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_num_actions_set)
    actions = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_actions_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_actions_set)
    num_deps = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_num_deps_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_num_deps_set)
    deps = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_deps_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_deps_set)
    conn_pts = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_conn_pts_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_conn_pts_set)
    column_offset = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_column_offset_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_column_offset_set)
    row_offset = property(_LibreDWG.Dwg_Object_BLOCKARRAYACTION_row_offset_get, _LibreDWG.Dwg_Object_BLOCKARRAYACTION_row_offset_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKARRAYACTION_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKARRAYACTION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKARRAYACTION

# Register Dwg_Object_BLOCKARRAYACTION in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKARRAYACTION_swigregister(Dwg_Object_BLOCKARRAYACTION)
class Dwg_Object_BLOCKBASEPOINTPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_set)
    def_pt = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_set)
    num_propinfos = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_set)
    pt = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_set)
    base_pt = property(_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_get, _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKBASEPOINTPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKBASEPOINTPARAMETER

# Register Dwg_Object_BLOCKBASEPOINTPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKBASEPOINTPARAMETER_swigregister(Dwg_Object_BLOCKBASEPOINTPARAMETER)
class Dwg_Object_BLOCKFLIPACTION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_parent_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_name_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_be_major_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_be_minor_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_eed1071_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_eed1071_set)
    display_location = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_display_location_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_display_location_set)
    num_actions = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_num_actions_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_num_actions_set)
    actions = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_actions_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_actions_set)
    num_deps = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_num_deps_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_num_deps_set)
    deps = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_deps_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_deps_set)
    conn_pts = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_conn_pts_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_conn_pts_set)
    action_offset_x = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_action_offset_x_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_action_offset_x_set)
    action_offset_y = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_action_offset_y_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_action_offset_y_set)
    angle_offset = property(_LibreDWG.Dwg_Object_BLOCKFLIPACTION_angle_offset_get, _LibreDWG.Dwg_Object_BLOCKFLIPACTION_angle_offset_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKFLIPACTION_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKFLIPACTION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKFLIPACTION

# Register Dwg_Object_BLOCKFLIPACTION in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKFLIPACTION_swigregister(Dwg_Object_BLOCKFLIPACTION)
class Dwg_Object_BLOCKFLIPGRIP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_parent_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_name_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_be_major_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_be_minor_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_eed1071_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_eed1071_set)
    bg_bl91 = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_bg_bl91_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_bg_bl91_set)
    bg_bl92 = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_bg_bl92_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_bg_bl92_set)
    bg_location = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_bg_location_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_bg_location_set)
    bg_insert_cycling = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_set)
    bg_insert_cycling_weight = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_set)
    combined_state = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_combined_state_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_combined_state_set)
    orientation = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_orientation_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_orientation_set)
    upd_state = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_upd_state_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_upd_state_set)
    state = property(_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_state_get, _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_state_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKFLIPGRIP_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKFLIPGRIP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKFLIPGRIP

# Register Dwg_Object_BLOCKFLIPGRIP in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKFLIPGRIP_swigregister(Dwg_Object_BLOCKFLIPGRIP)
class Dwg_Object_BLOCKFLIPPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_set)
    def_basept = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_def_basept_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_def_basept_set)
    def_endpt = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_prop2_set)
    prop3 = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_prop3_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_prop3_set)
    prop4 = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_prop4_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_prop4_set)
    prop_states = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_prop_states_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_prop_states_set)
    parameter_base_location = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_set)
    upd_basept = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_set)
    basept = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_basept_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_basept_set)
    upd_endpt = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_set)
    endpt = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_endpt_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_endpt_set)
    flip_label = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_flip_label_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_flip_label_set)
    flip_label_desc = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_set)
    base_state_label = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_set)
    flipped_state_label = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_set)
    def_label_pt = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_set)
    bl96 = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_bl96_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_bl96_set)
    tooltip = property(_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_tooltip_get, _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_tooltip_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKFLIPPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKFLIPPARAMETER

# Register Dwg_Object_BLOCKFLIPPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKFLIPPARAMETER_swigregister(Dwg_Object_BLOCKFLIPPARAMETER)
class Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_set)
    def_basept = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_set)
    def_endpt = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_set)
    prop3 = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_set)
    prop4 = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_set)
    prop_states = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_set)
    parameter_base_location = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_set)
    upd_basept = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_set)
    basept = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_set)
    upd_endpt = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_set)
    endpt = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_set)
    dependency = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_set)
    expr_name = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_set)
    expr_description = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_set)
    value = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set)
    value_set = property(_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_get, _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER

# Register Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_swigregister(Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER)
class Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_set)
    def_basept = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_set)
    def_endpt = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_set)
    prop3 = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_set)
    prop4 = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_set)
    prop_states = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_set)
    parameter_base_location = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_set)
    upd_basept = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_set)
    basept = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_set)
    upd_endpt = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_set)
    endpt = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_set)
    dependency = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_set)
    expr_name = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_set)
    expr_description = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_set)
    value = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set)
    value_set = property(_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_get, _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER

# Register Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_swigregister(Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER)
class Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_set)
    def_basept = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_set)
    def_endpt = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_set)
    prop3 = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_set)
    prop4 = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_set)
    prop_states = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_set)
    parameter_base_location = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_set)
    upd_basept = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_set)
    basept = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_set)
    upd_endpt = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_set)
    endpt = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_set)
    dependency = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_set)
    expr_name = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_set)
    expr_description = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_set)
    value = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set)
    value_set = property(_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_get, _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER

# Register Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_swigregister(Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER)
class Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_set)
    def_basept = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_set)
    def_endpt = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_set)
    prop3 = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_set)
    prop4 = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_set)
    prop_states = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_set)
    parameter_base_location = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_set)
    upd_basept = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_set)
    basept = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_set)
    upd_endpt = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_set)
    endpt = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_set)
    dependency = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_set)
    expr_name = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_set)
    expr_description = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_set)
    value = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set)
    value_set = property(_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_get, _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER

# Register Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_swigregister(Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER)
class Dwg_Object_BLOCKLINEARGRIP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKLINEARGRIP_parent_get, _LibreDWG.Dwg_Object_BLOCKLINEARGRIP_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKLINEARGRIP_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKLINEARGRIP_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKLINEARGRIP_name_get, _LibreDWG.Dwg_Object_BLOCKLINEARGRIP_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKLINEARGRIP_be_major_get, _LibreDWG.Dwg_Object_BLOCKLINEARGRIP_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKLINEARGRIP_be_minor_get, _LibreDWG.Dwg_Object_BLOCKLINEARGRIP_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKLINEARGRIP_eed1071_get, _LibreDWG.Dwg_Object_BLOCKLINEARGRIP_eed1071_set)
    bg_bl91 = property(_LibreDWG.Dwg_Object_BLOCKLINEARGRIP_bg_bl91_get, _LibreDWG.Dwg_Object_BLOCKLINEARGRIP_bg_bl91_set)
    bg_bl92 = property(_LibreDWG.Dwg_Object_BLOCKLINEARGRIP_bg_bl92_get, _LibreDWG.Dwg_Object_BLOCKLINEARGRIP_bg_bl92_set)
    bg_location = property(_LibreDWG.Dwg_Object_BLOCKLINEARGRIP_bg_location_get, _LibreDWG.Dwg_Object_BLOCKLINEARGRIP_bg_location_set)
    bg_insert_cycling = property(_LibreDWG.Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_get, _LibreDWG.Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_set)
    bg_insert_cycling_weight = property(_LibreDWG.Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_get, _LibreDWG.Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_set)
    orientation = property(_LibreDWG.Dwg_Object_BLOCKLINEARGRIP_orientation_get, _LibreDWG.Dwg_Object_BLOCKLINEARGRIP_orientation_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKLINEARGRIP_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKLINEARGRIP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKLINEARGRIP

# Register Dwg_Object_BLOCKLINEARGRIP in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKLINEARGRIP_swigregister(Dwg_Object_BLOCKLINEARGRIP)
class Dwg_Object_BLOCKLINEARPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_set)
    def_basept = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_def_basept_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_def_basept_set)
    def_endpt = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_prop2_set)
    prop3 = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_prop3_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_prop3_set)
    prop4 = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_prop4_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_prop4_set)
    prop_states = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_prop_states_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_prop_states_set)
    parameter_base_location = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_set)
    upd_basept = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_set)
    basept = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_basept_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_basept_set)
    upd_endpt = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_set)
    endpt = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_endpt_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_endpt_set)
    distance_name = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_distance_name_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_distance_name_set)
    distance_desc = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_set)
    distance = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_distance_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_distance_set)
    value_set = property(_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_value_set_get, _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_value_set_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKLINEARPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKLINEARPARAMETER

# Register Dwg_Object_BLOCKLINEARPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKLINEARPARAMETER_swigregister(Dwg_Object_BLOCKLINEARPARAMETER)
class Dwg_BLOCKLOOKUPACTION_lut(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_BLOCKLOOKUPACTION_lut_parent_get, _LibreDWG.Dwg_BLOCKLOOKUPACTION_lut_parent_set)
    conn_pts = property(_LibreDWG.Dwg_BLOCKLOOKUPACTION_lut_conn_pts_get, _LibreDWG.Dwg_BLOCKLOOKUPACTION_lut_conn_pts_set)
    b282 = property(_LibreDWG.Dwg_BLOCKLOOKUPACTION_lut_b282_get, _LibreDWG.Dwg_BLOCKLOOKUPACTION_lut_b282_set)
    b281 = property(_LibreDWG.Dwg_BLOCKLOOKUPACTION_lut_b281_get, _LibreDWG.Dwg_BLOCKLOOKUPACTION_lut_b281_set)

    def __init__(self):
        _LibreDWG.Dwg_BLOCKLOOKUPACTION_lut_swiginit(self, _LibreDWG.new_Dwg_BLOCKLOOKUPACTION_lut())
    __swig_destroy__ = _LibreDWG.delete_Dwg_BLOCKLOOKUPACTION_lut

# Register Dwg_BLOCKLOOKUPACTION_lut in _LibreDWG:
_LibreDWG.Dwg_BLOCKLOOKUPACTION_lut_swigregister(Dwg_BLOCKLOOKUPACTION_lut)
class Dwg_Object_BLOCKLOOKUPACTION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_parent_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_name_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_be_major_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_be_minor_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_eed1071_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_eed1071_set)
    display_location = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_display_location_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_display_location_set)
    num_actions = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_num_actions_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_num_actions_set)
    actions = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_actions_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_actions_set)
    num_deps = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_num_deps_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_num_deps_set)
    deps = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_deps_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_deps_set)
    numelems = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_numelems_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_numelems_set)
    numrows = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_numrows_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_numrows_set)
    numcols = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_numcols_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_numcols_set)
    lut = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_lut_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_lut_set)
    exprs = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_exprs_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_exprs_set)
    b280 = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_b280_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_b280_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKLOOKUPACTION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKLOOKUPACTION

# Register Dwg_Object_BLOCKLOOKUPACTION in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKLOOKUPACTION_swigregister(Dwg_Object_BLOCKLOOKUPACTION)
class Dwg_Object_BLOCKLOOKUPGRIP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_parent_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_name_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_be_major_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_be_minor_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_eed1071_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_eed1071_set)
    bg_bl91 = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_set)
    bg_bl92 = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_set)
    bg_location = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_bg_location_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_bg_location_set)
    bg_insert_cycling = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_set)
    bg_insert_cycling_weight = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKLOOKUPGRIP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKLOOKUPGRIP

# Register Dwg_Object_BLOCKLOOKUPGRIP in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKLOOKUPGRIP_swigregister(Dwg_Object_BLOCKLOOKUPGRIP)
class Dwg_Object_BLOCKLOOKUPPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_set)
    def_pt = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_set)
    num_propinfos = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_set)
    lookup_name = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_set)
    lookup_desc = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_set)
    index = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_index_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_index_set)
    unknown_t = property(_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_get, _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKLOOKUPPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKLOOKUPPARAMETER

# Register Dwg_Object_BLOCKLOOKUPPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKLOOKUPPARAMETER_swigregister(Dwg_Object_BLOCKLOOKUPPARAMETER)
class Dwg_Object_BLOCKMOVEACTION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_parent_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_name_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_be_major_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_be_minor_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_eed1071_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_eed1071_set)
    display_location = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_display_location_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_display_location_set)
    num_actions = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_num_actions_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_num_actions_set)
    actions = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_actions_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_actions_set)
    num_deps = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_num_deps_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_num_deps_set)
    deps = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_deps_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_deps_set)
    conn_pts = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_conn_pts_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_conn_pts_set)
    action_offset_x = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_action_offset_x_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_action_offset_x_set)
    action_offset_y = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_action_offset_y_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_action_offset_y_set)
    angle_offset = property(_LibreDWG.Dwg_Object_BLOCKMOVEACTION_angle_offset_get, _LibreDWG.Dwg_Object_BLOCKMOVEACTION_angle_offset_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKMOVEACTION_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKMOVEACTION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKMOVEACTION

# Register Dwg_Object_BLOCKMOVEACTION in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKMOVEACTION_swigregister(Dwg_Object_BLOCKMOVEACTION)
class Dwg_Object_BLOCKPOINTPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_set)
    def_pt = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_def_pt_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_def_pt_set)
    num_propinfos = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_prop2_set)
    position_name = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_position_name_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_position_name_set)
    position_desc = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_position_desc_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_position_desc_set)
    def_label_pt = property(_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_get, _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKPOINTPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKPOINTPARAMETER

# Register Dwg_Object_BLOCKPOINTPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKPOINTPARAMETER_swigregister(Dwg_Object_BLOCKPOINTPARAMETER)
class Dwg_Object_BLOCKPOLARGRIP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKPOLARGRIP_parent_get, _LibreDWG.Dwg_Object_BLOCKPOLARGRIP_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKPOLARGRIP_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKPOLARGRIP_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKPOLARGRIP_name_get, _LibreDWG.Dwg_Object_BLOCKPOLARGRIP_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKPOLARGRIP_be_major_get, _LibreDWG.Dwg_Object_BLOCKPOLARGRIP_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKPOLARGRIP_be_minor_get, _LibreDWG.Dwg_Object_BLOCKPOLARGRIP_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKPOLARGRIP_eed1071_get, _LibreDWG.Dwg_Object_BLOCKPOLARGRIP_eed1071_set)
    bg_bl91 = property(_LibreDWG.Dwg_Object_BLOCKPOLARGRIP_bg_bl91_get, _LibreDWG.Dwg_Object_BLOCKPOLARGRIP_bg_bl91_set)
    bg_bl92 = property(_LibreDWG.Dwg_Object_BLOCKPOLARGRIP_bg_bl92_get, _LibreDWG.Dwg_Object_BLOCKPOLARGRIP_bg_bl92_set)
    bg_location = property(_LibreDWG.Dwg_Object_BLOCKPOLARGRIP_bg_location_get, _LibreDWG.Dwg_Object_BLOCKPOLARGRIP_bg_location_set)
    bg_insert_cycling = property(_LibreDWG.Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_get, _LibreDWG.Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_set)
    bg_insert_cycling_weight = property(_LibreDWG.Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_get, _LibreDWG.Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKPOLARGRIP_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKPOLARGRIP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKPOLARGRIP

# Register Dwg_Object_BLOCKPOLARGRIP in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKPOLARGRIP_swigregister(Dwg_Object_BLOCKPOLARGRIP)
class Dwg_Object_BLOCKPOLARPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_set)
    def_basept = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_def_basept_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_def_basept_set)
    def_endpt = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_prop2_set)
    prop3 = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_prop3_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_prop3_set)
    prop4 = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_prop4_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_prop4_set)
    prop_states = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_prop_states_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_prop_states_set)
    parameter_base_location = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_set)
    upd_basept = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_set)
    basept = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_basept_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_basept_set)
    upd_endpt = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_set)
    endpt = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_endpt_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_endpt_set)
    angle_name = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_angle_name_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_angle_name_set)
    angle_desc = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_set)
    distance_name = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_distance_name_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_distance_name_set)
    distance_desc = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_set)
    offset = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_offset_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_offset_set)
    angle_value_set = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_set)
    distance_value_set = property(_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_get, _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKPOLARPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKPOLARPARAMETER

# Register Dwg_Object_BLOCKPOLARPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKPOLARPARAMETER_swigregister(Dwg_Object_BLOCKPOLARPARAMETER)
class Dwg_Object_BLOCKPOLARSTRETCHACTION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_name_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_set)
    display_location = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_set)
    num_actions = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_set)
    actions = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_set)
    num_deps = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_set)
    deps = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_set)
    conn_pts = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_set)
    num_pts = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_set)
    pts = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_set)
    num_hdls = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_set)
    hdls = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_set)
    shorts = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_set)
    num_codes = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_set)
    codes = property(_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_get, _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKPOLARSTRETCHACTION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKPOLARSTRETCHACTION

# Register Dwg_Object_BLOCKPOLARSTRETCHACTION in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKPOLARSTRETCHACTION_swigregister(Dwg_Object_BLOCKPOLARSTRETCHACTION)
class Dwg_Object_BLOCKPROPERTIESTABLE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLE_parent_get, _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLE_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLE_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKPROPERTIESTABLE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKPROPERTIESTABLE

# Register Dwg_Object_BLOCKPROPERTIESTABLE in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLE_swigregister(Dwg_Object_BLOCKPROPERTIESTABLE)
class Dwg_Object_BLOCKPROPERTIESTABLEGRIP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_get, _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_get, _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_get, _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_get, _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_get, _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_set)
    bg_bl91 = property(_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_get, _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_set)
    bg_bl92 = property(_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_get, _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_set)
    bg_location = property(_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_get, _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_set)
    bg_insert_cycling = property(_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_get, _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_set)
    bg_insert_cycling_weight = property(_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_get, _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKPROPERTIESTABLEGRIP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKPROPERTIESTABLEGRIP

# Register Dwg_Object_BLOCKPROPERTIESTABLEGRIP in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKPROPERTIESTABLEGRIP_swigregister(Dwg_Object_BLOCKPROPERTIESTABLEGRIP)
class Dwg_Object_BLOCKREPRESENTATION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKREPRESENTATION_parent_get, _LibreDWG.Dwg_Object_BLOCKREPRESENTATION_parent_set)
    flag = property(_LibreDWG.Dwg_Object_BLOCKREPRESENTATION_flag_get, _LibreDWG.Dwg_Object_BLOCKREPRESENTATION_flag_set)
    block = property(_LibreDWG.Dwg_Object_BLOCKREPRESENTATION_block_get, _LibreDWG.Dwg_Object_BLOCKREPRESENTATION_block_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKREPRESENTATION_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKREPRESENTATION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKREPRESENTATION

# Register Dwg_Object_BLOCKREPRESENTATION in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKREPRESENTATION_swigregister(Dwg_Object_BLOCKREPRESENTATION)
class Dwg_Object_BLOCKROTATEACTION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_parent_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_name_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_be_major_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_be_minor_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_eed1071_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_eed1071_set)
    display_location = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_display_location_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_display_location_set)
    num_actions = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_num_actions_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_num_actions_set)
    actions = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_actions_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_actions_set)
    num_deps = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_num_deps_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_num_deps_set)
    deps = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_deps_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_deps_set)
    offset = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_offset_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_offset_set)
    conn_pts = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_conn_pts_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_conn_pts_set)
    dependent = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_dependent_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_dependent_set)
    base_pt = property(_LibreDWG.Dwg_Object_BLOCKROTATEACTION_base_pt_get, _LibreDWG.Dwg_Object_BLOCKROTATEACTION_base_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKROTATEACTION_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKROTATEACTION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKROTATEACTION

# Register Dwg_Object_BLOCKROTATEACTION in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKROTATEACTION_swigregister(Dwg_Object_BLOCKROTATEACTION)
class Dwg_Object_BLOCKROTATIONGRIP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_parent_get, _LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_name_get, _LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_be_major_get, _LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_be_minor_get, _LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_eed1071_get, _LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_eed1071_set)
    bg_bl91 = property(_LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_get, _LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_set)
    bg_bl92 = property(_LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_get, _LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_set)
    bg_location = property(_LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_bg_location_get, _LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_bg_location_set)
    bg_insert_cycling = property(_LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_get, _LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_set)
    bg_insert_cycling_weight = property(_LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_get, _LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKROTATIONGRIP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKROTATIONGRIP

# Register Dwg_Object_BLOCKROTATIONGRIP in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKROTATIONGRIP_swigregister(Dwg_Object_BLOCKROTATIONGRIP)
class Dwg_Object_BLOCKROTATIONPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_set)
    def_basept = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_set)
    def_endpt = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_prop2_set)
    prop3 = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_prop3_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_prop3_set)
    prop4 = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_prop4_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_prop4_set)
    prop_states = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_set)
    parameter_base_location = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_set)
    upd_basept = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_set)
    basept = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_basept_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_basept_set)
    upd_endpt = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_set)
    endpt = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_endpt_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_endpt_set)
    def_base_angle_pt = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_set)
    angle_name = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_set)
    angle_desc = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_set)
    angle = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_angle_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_angle_set)
    angle_value_set = property(_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_get, _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKROTATIONPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKROTATIONPARAMETER

# Register Dwg_Object_BLOCKROTATIONPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKROTATIONPARAMETER_swigregister(Dwg_Object_BLOCKROTATIONPARAMETER)
class Dwg_Object_BLOCKSCALEACTION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_parent_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_name_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_be_major_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_be_minor_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_eed1071_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_eed1071_set)
    display_location = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_display_location_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_display_location_set)
    num_actions = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_num_actions_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_num_actions_set)
    actions = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_actions_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_actions_set)
    num_deps = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_num_deps_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_num_deps_set)
    deps = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_deps_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_deps_set)
    offset = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_offset_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_offset_set)
    conn_pts = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_conn_pts_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_conn_pts_set)
    dependent = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_dependent_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_dependent_set)
    base_pt = property(_LibreDWG.Dwg_Object_BLOCKSCALEACTION_base_pt_get, _LibreDWG.Dwg_Object_BLOCKSCALEACTION_base_pt_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKSCALEACTION_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKSCALEACTION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKSCALEACTION

# Register Dwg_Object_BLOCKSCALEACTION in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKSCALEACTION_swigregister(Dwg_Object_BLOCKSCALEACTION)
class Dwg_BLOCKSTRETCHACTION_handles(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_BLOCKSTRETCHACTION_handles_parent_get, _LibreDWG.Dwg_BLOCKSTRETCHACTION_handles_parent_set)
    hdl = property(_LibreDWG.Dwg_BLOCKSTRETCHACTION_handles_hdl_get, _LibreDWG.Dwg_BLOCKSTRETCHACTION_handles_hdl_set)
    num_indexes = property(_LibreDWG.Dwg_BLOCKSTRETCHACTION_handles_num_indexes_get, _LibreDWG.Dwg_BLOCKSTRETCHACTION_handles_num_indexes_set)
    indexes = property(_LibreDWG.Dwg_BLOCKSTRETCHACTION_handles_indexes_get, _LibreDWG.Dwg_BLOCKSTRETCHACTION_handles_indexes_set)

    def __init__(self):
        _LibreDWG.Dwg_BLOCKSTRETCHACTION_handles_swiginit(self, _LibreDWG.new_Dwg_BLOCKSTRETCHACTION_handles())
    __swig_destroy__ = _LibreDWG.delete_Dwg_BLOCKSTRETCHACTION_handles

# Register Dwg_BLOCKSTRETCHACTION_handles in _LibreDWG:
_LibreDWG.Dwg_BLOCKSTRETCHACTION_handles_swigregister(Dwg_BLOCKSTRETCHACTION_handles)
class Dwg_BLOCKSTRETCHACTION_codes(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_BLOCKSTRETCHACTION_codes_parent_get, _LibreDWG.Dwg_BLOCKSTRETCHACTION_codes_parent_set)
    bl95 = property(_LibreDWG.Dwg_BLOCKSTRETCHACTION_codes_bl95_get, _LibreDWG.Dwg_BLOCKSTRETCHACTION_codes_bl95_set)
    num_indexes = property(_LibreDWG.Dwg_BLOCKSTRETCHACTION_codes_num_indexes_get, _LibreDWG.Dwg_BLOCKSTRETCHACTION_codes_num_indexes_set)
    indexes = property(_LibreDWG.Dwg_BLOCKSTRETCHACTION_codes_indexes_get, _LibreDWG.Dwg_BLOCKSTRETCHACTION_codes_indexes_set)

    def __init__(self):
        _LibreDWG.Dwg_BLOCKSTRETCHACTION_codes_swiginit(self, _LibreDWG.new_Dwg_BLOCKSTRETCHACTION_codes())
    __swig_destroy__ = _LibreDWG.delete_Dwg_BLOCKSTRETCHACTION_codes

# Register Dwg_BLOCKSTRETCHACTION_codes in _LibreDWG:
_LibreDWG.Dwg_BLOCKSTRETCHACTION_codes_swigregister(Dwg_BLOCKSTRETCHACTION_codes)
class Dwg_Object_BLOCKSTRETCHACTION(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_parent_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_name_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_be_major_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_be_minor_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_eed1071_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_eed1071_set)
    display_location = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_display_location_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_display_location_set)
    num_actions = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_num_actions_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_num_actions_set)
    actions = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_actions_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_actions_set)
    num_deps = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_num_deps_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_num_deps_set)
    deps = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_deps_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_deps_set)
    conn_pts = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_conn_pts_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_conn_pts_set)
    num_pts = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_num_pts_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_num_pts_set)
    pts = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_pts_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_pts_set)
    num_hdls = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_num_hdls_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_num_hdls_set)
    hdls = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_hdls_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_hdls_set)
    num_codes = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_num_codes_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_num_codes_set)
    codes = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_codes_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_codes_set)
    action_offset_x = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_set)
    action_offset_y = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_set)
    angle_offset = property(_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_angle_offset_get, _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_angle_offset_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKSTRETCHACTION())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKSTRETCHACTION

# Register Dwg_Object_BLOCKSTRETCHACTION in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKSTRETCHACTION_swigregister(Dwg_Object_BLOCKSTRETCHACTION)
class Dwg_Object_BLOCKUSERPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_chain_actions_set)
    def_pt = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_def_pt_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_def_pt_set)
    num_propinfos = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_prop2_set)
    flag = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_flag_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_flag_set)
    assocvariable = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_assocvariable_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_assocvariable_set)
    expr = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_expr_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_expr_set)
    value = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_value_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_value_set)
    type = property(_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_type_get, _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_type_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKUSERPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKUSERPARAMETER

# Register Dwg_Object_BLOCKUSERPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKUSERPARAMETER_swigregister(Dwg_Object_BLOCKUSERPARAMETER)
class Dwg_Object_BLOCKXYGRIP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKXYGRIP_parent_get, _LibreDWG.Dwg_Object_BLOCKXYGRIP_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKXYGRIP_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKXYGRIP_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKXYGRIP_name_get, _LibreDWG.Dwg_Object_BLOCKXYGRIP_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKXYGRIP_be_major_get, _LibreDWG.Dwg_Object_BLOCKXYGRIP_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKXYGRIP_be_minor_get, _LibreDWG.Dwg_Object_BLOCKXYGRIP_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKXYGRIP_eed1071_get, _LibreDWG.Dwg_Object_BLOCKXYGRIP_eed1071_set)
    bg_bl91 = property(_LibreDWG.Dwg_Object_BLOCKXYGRIP_bg_bl91_get, _LibreDWG.Dwg_Object_BLOCKXYGRIP_bg_bl91_set)
    bg_bl92 = property(_LibreDWG.Dwg_Object_BLOCKXYGRIP_bg_bl92_get, _LibreDWG.Dwg_Object_BLOCKXYGRIP_bg_bl92_set)
    bg_location = property(_LibreDWG.Dwg_Object_BLOCKXYGRIP_bg_location_get, _LibreDWG.Dwg_Object_BLOCKXYGRIP_bg_location_set)
    bg_insert_cycling = property(_LibreDWG.Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_get, _LibreDWG.Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_set)
    bg_insert_cycling_weight = property(_LibreDWG.Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_get, _LibreDWG.Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKXYGRIP_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKXYGRIP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKXYGRIP

# Register Dwg_Object_BLOCKXYGRIP in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKXYGRIP_swigregister(Dwg_Object_BLOCKXYGRIP)
class Dwg_Object_BLOCKXYPARAMETER(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_parent_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_evalexpr_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_evalexpr_set)
    name = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_name_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_name_set)
    be_major = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_be_major_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_be_major_set)
    be_minor = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_be_minor_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_be_minor_set)
    eed1071 = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_eed1071_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_eed1071_set)
    show_properties = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_show_properties_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_show_properties_set)
    chain_actions = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_chain_actions_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_chain_actions_set)
    def_basept = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_def_basept_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_def_basept_set)
    def_endpt = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_def_endpt_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_def_endpt_set)
    prop1 = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_prop1_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_prop1_set)
    prop2 = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_prop2_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_prop2_set)
    prop3 = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_prop3_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_prop3_set)
    prop4 = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_prop4_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_prop4_set)
    prop_states = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_prop_states_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_prop_states_set)
    parameter_base_location = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_set)
    upd_basept = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_upd_basept_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_upd_basept_set)
    basept = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_basept_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_basept_set)
    upd_endpt = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_upd_endpt_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_upd_endpt_set)
    endpt = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_endpt_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_endpt_set)
    x_label = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_x_label_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_x_label_set)
    x_label_desc = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_x_label_desc_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_x_label_desc_set)
    y_label = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_y_label_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_y_label_set)
    y_label_desc = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_y_label_desc_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_y_label_desc_set)
    x_value = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_x_value_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_x_value_set)
    y_value = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_y_value_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_y_value_set)
    x_value_set = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_x_value_set_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_x_value_set_set)
    y_value_set = property(_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_y_value_set_get, _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_y_value_set_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_BLOCKXYPARAMETER_swiginit(self, _LibreDWG.new_Dwg_Object_BLOCKXYPARAMETER())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_BLOCKXYPARAMETER

# Register Dwg_Object_BLOCKXYPARAMETER in _LibreDWG:
_LibreDWG.Dwg_Object_BLOCKXYPARAMETER_swigregister(Dwg_Object_BLOCKXYPARAMETER)
class Dwg_Object_DYNAMICBLOCKPROXYNODE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_get, _LibreDWG.Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_set)
    evalexpr = property(_LibreDWG.Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_get, _LibreDWG.Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_DYNAMICBLOCKPROXYNODE_swiginit(self, _LibreDWG.new_Dwg_Object_DYNAMICBLOCKPROXYNODE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_DYNAMICBLOCKPROXYNODE

# Register Dwg_Object_DYNAMICBLOCKPROXYNODE in _LibreDWG:
_LibreDWG.Dwg_Object_DYNAMICBLOCKPROXYNODE_swigregister(Dwg_Object_DYNAMICBLOCKPROXYNODE)
class Dwg_POINTCLOUD_IntensityStyle(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_POINTCLOUD_IntensityStyle_parent_get, _LibreDWG.Dwg_POINTCLOUD_IntensityStyle_parent_set)
    min_intensity = property(_LibreDWG.Dwg_POINTCLOUD_IntensityStyle_min_intensity_get, _LibreDWG.Dwg_POINTCLOUD_IntensityStyle_min_intensity_set)
    max_intensity = property(_LibreDWG.Dwg_POINTCLOUD_IntensityStyle_max_intensity_get, _LibreDWG.Dwg_POINTCLOUD_IntensityStyle_max_intensity_set)
    intensity_low_treshold = property(_LibreDWG.Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_get, _LibreDWG.Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_set)
    intensity_high_treshold = property(_LibreDWG.Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_get, _LibreDWG.Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_set)

    def __init__(self):
        _LibreDWG.Dwg_POINTCLOUD_IntensityStyle_swiginit(self, _LibreDWG.new_Dwg_POINTCLOUD_IntensityStyle())
    __swig_destroy__ = _LibreDWG.delete_Dwg_POINTCLOUD_IntensityStyle

# Register Dwg_POINTCLOUD_IntensityStyle in _LibreDWG:
_LibreDWG.Dwg_POINTCLOUD_IntensityStyle_swigregister(Dwg_POINTCLOUD_IntensityStyle)
class Dwg_POINTCLOUD_Clippings(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_POINTCLOUD_Clippings_parent_get, _LibreDWG.Dwg_POINTCLOUD_Clippings_parent_set)
    is_inverted = property(_LibreDWG.Dwg_POINTCLOUD_Clippings_is_inverted_get, _LibreDWG.Dwg_POINTCLOUD_Clippings_is_inverted_set)
    type = property(_LibreDWG.Dwg_POINTCLOUD_Clippings_type_get, _LibreDWG.Dwg_POINTCLOUD_Clippings_type_set)
    num_vertices = property(_LibreDWG.Dwg_POINTCLOUD_Clippings_num_vertices_get, _LibreDWG.Dwg_POINTCLOUD_Clippings_num_vertices_set)
    vertices = property(_LibreDWG.Dwg_POINTCLOUD_Clippings_vertices_get, _LibreDWG.Dwg_POINTCLOUD_Clippings_vertices_set)
    z_min = property(_LibreDWG.Dwg_POINTCLOUD_Clippings_z_min_get, _LibreDWG.Dwg_POINTCLOUD_Clippings_z_min_set)
    z_max = property(_LibreDWG.Dwg_POINTCLOUD_Clippings_z_max_get, _LibreDWG.Dwg_POINTCLOUD_Clippings_z_max_set)

    def __init__(self):
        _LibreDWG.Dwg_POINTCLOUD_Clippings_swiginit(self, _LibreDWG.new_Dwg_POINTCLOUD_Clippings())
    __swig_destroy__ = _LibreDWG.delete_Dwg_POINTCLOUD_Clippings

# Register Dwg_POINTCLOUD_Clippings in _LibreDWG:
_LibreDWG.Dwg_POINTCLOUD_Clippings_swigregister(Dwg_POINTCLOUD_Clippings)
class Dwg_POINTCLOUDEX_Croppings(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_POINTCLOUDEX_Croppings_parent_get, _LibreDWG.Dwg_POINTCLOUDEX_Croppings_parent_set)
    type = property(_LibreDWG.Dwg_POINTCLOUDEX_Croppings_type_get, _LibreDWG.Dwg_POINTCLOUDEX_Croppings_type_set)
    is_inside = property(_LibreDWG.Dwg_POINTCLOUDEX_Croppings_is_inside_get, _LibreDWG.Dwg_POINTCLOUDEX_Croppings_is_inside_set)
    is_inverted = property(_LibreDWG.Dwg_POINTCLOUDEX_Croppings_is_inverted_get, _LibreDWG.Dwg_POINTCLOUDEX_Croppings_is_inverted_set)
    crop_plane = property(_LibreDWG.Dwg_POINTCLOUDEX_Croppings_crop_plane_get, _LibreDWG.Dwg_POINTCLOUDEX_Croppings_crop_plane_set)
    crop_x_dir = property(_LibreDWG.Dwg_POINTCLOUDEX_Croppings_crop_x_dir_get, _LibreDWG.Dwg_POINTCLOUDEX_Croppings_crop_x_dir_set)
    crop_y_dir = property(_LibreDWG.Dwg_POINTCLOUDEX_Croppings_crop_y_dir_get, _LibreDWG.Dwg_POINTCLOUDEX_Croppings_crop_y_dir_set)
    num_pts = property(_LibreDWG.Dwg_POINTCLOUDEX_Croppings_num_pts_get, _LibreDWG.Dwg_POINTCLOUDEX_Croppings_num_pts_set)
    pts = property(_LibreDWG.Dwg_POINTCLOUDEX_Croppings_pts_get, _LibreDWG.Dwg_POINTCLOUDEX_Croppings_pts_set)

    def __init__(self):
        _LibreDWG.Dwg_POINTCLOUDEX_Croppings_swiginit(self, _LibreDWG.new_Dwg_POINTCLOUDEX_Croppings())
    __swig_destroy__ = _LibreDWG.delete_Dwg_POINTCLOUDEX_Croppings

# Register Dwg_POINTCLOUDEX_Croppings in _LibreDWG:
_LibreDWG.Dwg_POINTCLOUDEX_Croppings_swigregister(Dwg_POINTCLOUDEX_Croppings)
class Dwg_Entity_POINTCLOUD(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_POINTCLOUD_parent_get, _LibreDWG.Dwg_Entity_POINTCLOUD_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_POINTCLOUD_class_version_get, _LibreDWG.Dwg_Entity_POINTCLOUD_class_version_set)
    origin = property(_LibreDWG.Dwg_Entity_POINTCLOUD_origin_get, _LibreDWG.Dwg_Entity_POINTCLOUD_origin_set)
    saved_filename = property(_LibreDWG.Dwg_Entity_POINTCLOUD_saved_filename_get, _LibreDWG.Dwg_Entity_POINTCLOUD_saved_filename_set)
    num_source_files = property(_LibreDWG.Dwg_Entity_POINTCLOUD_num_source_files_get, _LibreDWG.Dwg_Entity_POINTCLOUD_num_source_files_set)
    source_files = property(_LibreDWG.Dwg_Entity_POINTCLOUD_source_files_get, _LibreDWG.Dwg_Entity_POINTCLOUD_source_files_set)
    extents_min = property(_LibreDWG.Dwg_Entity_POINTCLOUD_extents_min_get, _LibreDWG.Dwg_Entity_POINTCLOUD_extents_min_set)
    extents_max = property(_LibreDWG.Dwg_Entity_POINTCLOUD_extents_max_get, _LibreDWG.Dwg_Entity_POINTCLOUD_extents_max_set)
    numpoints = property(_LibreDWG.Dwg_Entity_POINTCLOUD_numpoints_get, _LibreDWG.Dwg_Entity_POINTCLOUD_numpoints_set)
    ucs_name = property(_LibreDWG.Dwg_Entity_POINTCLOUD_ucs_name_get, _LibreDWG.Dwg_Entity_POINTCLOUD_ucs_name_set)
    ucs_origin = property(_LibreDWG.Dwg_Entity_POINTCLOUD_ucs_origin_get, _LibreDWG.Dwg_Entity_POINTCLOUD_ucs_origin_set)
    ucs_x_dir = property(_LibreDWG.Dwg_Entity_POINTCLOUD_ucs_x_dir_get, _LibreDWG.Dwg_Entity_POINTCLOUD_ucs_x_dir_set)
    ucs_y_dir = property(_LibreDWG.Dwg_Entity_POINTCLOUD_ucs_y_dir_get, _LibreDWG.Dwg_Entity_POINTCLOUD_ucs_y_dir_set)
    ucs_z_dir = property(_LibreDWG.Dwg_Entity_POINTCLOUD_ucs_z_dir_get, _LibreDWG.Dwg_Entity_POINTCLOUD_ucs_z_dir_set)
    pointclouddef = property(_LibreDWG.Dwg_Entity_POINTCLOUD_pointclouddef_get, _LibreDWG.Dwg_Entity_POINTCLOUD_pointclouddef_set)
    reactor = property(_LibreDWG.Dwg_Entity_POINTCLOUD_reactor_get, _LibreDWG.Dwg_Entity_POINTCLOUD_reactor_set)
    show_intensity = property(_LibreDWG.Dwg_Entity_POINTCLOUD_show_intensity_get, _LibreDWG.Dwg_Entity_POINTCLOUD_show_intensity_set)
    intensity_scheme = property(_LibreDWG.Dwg_Entity_POINTCLOUD_intensity_scheme_get, _LibreDWG.Dwg_Entity_POINTCLOUD_intensity_scheme_set)
    intensity_style = property(_LibreDWG.Dwg_Entity_POINTCLOUD_intensity_style_get, _LibreDWG.Dwg_Entity_POINTCLOUD_intensity_style_set)
    show_clipping = property(_LibreDWG.Dwg_Entity_POINTCLOUD_show_clipping_get, _LibreDWG.Dwg_Entity_POINTCLOUD_show_clipping_set)
    num_clippings = property(_LibreDWG.Dwg_Entity_POINTCLOUD_num_clippings_get, _LibreDWG.Dwg_Entity_POINTCLOUD_num_clippings_set)
    clippings = property(_LibreDWG.Dwg_Entity_POINTCLOUD_clippings_get, _LibreDWG.Dwg_Entity_POINTCLOUD_clippings_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_POINTCLOUD_swiginit(self, _LibreDWG.new_Dwg_Entity_POINTCLOUD())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_POINTCLOUD

# Register Dwg_Entity_POINTCLOUD in _LibreDWG:
_LibreDWG.Dwg_Entity_POINTCLOUD_swigregister(Dwg_Entity_POINTCLOUD)
class Dwg_Entity_POINTCLOUDEX(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_parent_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_parent_set)
    class_version = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_class_version_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_class_version_set)
    extents_min = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_extents_min_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_extents_min_set)
    extents_max = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_extents_max_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_extents_max_set)
    ucs_origin = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_ucs_origin_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_ucs_origin_set)
    ucs_x_dir = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_ucs_x_dir_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_ucs_x_dir_set)
    ucs_y_dir = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_ucs_y_dir_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_ucs_y_dir_set)
    ucs_z_dir = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_ucs_z_dir_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_ucs_z_dir_set)
    is_locked = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_is_locked_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_is_locked_set)
    pointclouddefex = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_pointclouddefex_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_pointclouddefex_set)
    reactor = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_reactor_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_reactor_set)
    name = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_name_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_name_set)
    show_intensity = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_show_intensity_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_show_intensity_set)
    stylization_type = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_stylization_type_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_stylization_type_set)
    intensity_colorscheme = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_set)
    cur_colorscheme = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_cur_colorscheme_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_cur_colorscheme_set)
    classification_colorscheme = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_classification_colorscheme_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_classification_colorscheme_set)
    elevation_min = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_elevation_min_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_elevation_min_set)
    elevation_max = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_elevation_max_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_elevation_max_set)
    intensity_min = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_intensity_min_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_intensity_min_set)
    intensity_max = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_intensity_max_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_intensity_max_set)
    intensity_out_of_range_behavior = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_set)
    elevation_out_of_range_behavior = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_set)
    elevation_apply_to_fixed_range = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_set)
    intensity_as_gradient = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_set)
    elevation_as_gradient = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_set)
    show_cropping = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_show_cropping_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_show_cropping_set)
    unknown_bl0 = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_unknown_bl0_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_unknown_bl0_set)
    unknown_bl1 = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_unknown_bl1_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_unknown_bl1_set)
    num_croppings = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_num_croppings_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_num_croppings_set)
    croppings = property(_LibreDWG.Dwg_Entity_POINTCLOUDEX_croppings_get, _LibreDWG.Dwg_Entity_POINTCLOUDEX_croppings_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_POINTCLOUDEX_swiginit(self, _LibreDWG.new_Dwg_Entity_POINTCLOUDEX())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_POINTCLOUDEX

# Register Dwg_Entity_POINTCLOUDEX in _LibreDWG:
_LibreDWG.Dwg_Entity_POINTCLOUDEX_swigregister(Dwg_Entity_POINTCLOUDEX)
class Dwg_Object_POINTCLOUDDEF(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_POINTCLOUDDEF_parent_get, _LibreDWG.Dwg_Object_POINTCLOUDDEF_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_POINTCLOUDDEF_class_version_get, _LibreDWG.Dwg_Object_POINTCLOUDDEF_class_version_set)
    source_filename = property(_LibreDWG.Dwg_Object_POINTCLOUDDEF_source_filename_get, _LibreDWG.Dwg_Object_POINTCLOUDDEF_source_filename_set)
    is_loaded = property(_LibreDWG.Dwg_Object_POINTCLOUDDEF_is_loaded_get, _LibreDWG.Dwg_Object_POINTCLOUDDEF_is_loaded_set)
    numpoints = property(_LibreDWG.Dwg_Object_POINTCLOUDDEF_numpoints_get, _LibreDWG.Dwg_Object_POINTCLOUDDEF_numpoints_set)
    extents_min = property(_LibreDWG.Dwg_Object_POINTCLOUDDEF_extents_min_get, _LibreDWG.Dwg_Object_POINTCLOUDDEF_extents_min_set)
    extents_max = property(_LibreDWG.Dwg_Object_POINTCLOUDDEF_extents_max_get, _LibreDWG.Dwg_Object_POINTCLOUDDEF_extents_max_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_POINTCLOUDDEF_swiginit(self, _LibreDWG.new_Dwg_Object_POINTCLOUDDEF())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_POINTCLOUDDEF

# Register Dwg_Object_POINTCLOUDDEF in _LibreDWG:
_LibreDWG.Dwg_Object_POINTCLOUDDEF_swigregister(Dwg_Object_POINTCLOUDDEF)
class Dwg_Object_POINTCLOUDDEFEX(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_POINTCLOUDDEFEX_parent_get, _LibreDWG.Dwg_Object_POINTCLOUDDEFEX_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_POINTCLOUDDEFEX_class_version_get, _LibreDWG.Dwg_Object_POINTCLOUDDEFEX_class_version_set)
    source_filename = property(_LibreDWG.Dwg_Object_POINTCLOUDDEFEX_source_filename_get, _LibreDWG.Dwg_Object_POINTCLOUDDEFEX_source_filename_set)
    is_loaded = property(_LibreDWG.Dwg_Object_POINTCLOUDDEFEX_is_loaded_get, _LibreDWG.Dwg_Object_POINTCLOUDDEFEX_is_loaded_set)
    numpoints = property(_LibreDWG.Dwg_Object_POINTCLOUDDEFEX_numpoints_get, _LibreDWG.Dwg_Object_POINTCLOUDDEFEX_numpoints_set)
    extents_min = property(_LibreDWG.Dwg_Object_POINTCLOUDDEFEX_extents_min_get, _LibreDWG.Dwg_Object_POINTCLOUDDEFEX_extents_min_set)
    extents_max = property(_LibreDWG.Dwg_Object_POINTCLOUDDEFEX_extents_max_get, _LibreDWG.Dwg_Object_POINTCLOUDDEFEX_extents_max_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_POINTCLOUDDEFEX_swiginit(self, _LibreDWG.new_Dwg_Object_POINTCLOUDDEFEX())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_POINTCLOUDDEFEX

# Register Dwg_Object_POINTCLOUDDEFEX in _LibreDWG:
_LibreDWG.Dwg_Object_POINTCLOUDDEFEX_swigregister(Dwg_Object_POINTCLOUDDEFEX)
class Dwg_Object_POINTCLOUDDEF_REACTOR(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_POINTCLOUDDEF_REACTOR_parent_get, _LibreDWG.Dwg_Object_POINTCLOUDDEF_REACTOR_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_get, _LibreDWG.Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_POINTCLOUDDEF_REACTOR_swiginit(self, _LibreDWG.new_Dwg_Object_POINTCLOUDDEF_REACTOR())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_POINTCLOUDDEF_REACTOR

# Register Dwg_Object_POINTCLOUDDEF_REACTOR in _LibreDWG:
_LibreDWG.Dwg_Object_POINTCLOUDDEF_REACTOR_swigregister(Dwg_Object_POINTCLOUDDEF_REACTOR)
class Dwg_Object_POINTCLOUDDEF_REACTOR_EX(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_get, _LibreDWG.Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_get, _LibreDWG.Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_POINTCLOUDDEF_REACTOR_EX_swiginit(self, _LibreDWG.new_Dwg_Object_POINTCLOUDDEF_REACTOR_EX())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_POINTCLOUDDEF_REACTOR_EX

# Register Dwg_Object_POINTCLOUDDEF_REACTOR_EX in _LibreDWG:
_LibreDWG.Dwg_Object_POINTCLOUDDEF_REACTOR_EX_swigregister(Dwg_Object_POINTCLOUDDEF_REACTOR_EX)
class Dwg_ColorRamp(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_ColorRamp_parent_get, _LibreDWG.Dwg_ColorRamp_parent_set)
    colorscheme = property(_LibreDWG.Dwg_ColorRamp_colorscheme_get, _LibreDWG.Dwg_ColorRamp_colorscheme_set)
    unknown_bl = property(_LibreDWG.Dwg_ColorRamp_unknown_bl_get, _LibreDWG.Dwg_ColorRamp_unknown_bl_set)
    unknown_b = property(_LibreDWG.Dwg_ColorRamp_unknown_b_get, _LibreDWG.Dwg_ColorRamp_unknown_b_set)

    def __init__(self):
        _LibreDWG.Dwg_ColorRamp_swiginit(self, _LibreDWG.new_Dwg_ColorRamp())
    __swig_destroy__ = _LibreDWG.delete_Dwg_ColorRamp

# Register Dwg_ColorRamp in _LibreDWG:
_LibreDWG.Dwg_ColorRamp_swigregister(Dwg_ColorRamp)
class Dwg_POINTCLOUDCOLORMAP_Ramp(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_POINTCLOUDCOLORMAP_Ramp_parent_get, _LibreDWG.Dwg_POINTCLOUDCOLORMAP_Ramp_parent_set)
    class_version = property(_LibreDWG.Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_get, _LibreDWG.Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_set)
    num_ramps = property(_LibreDWG.Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_get, _LibreDWG.Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_set)
    ramps = property(_LibreDWG.Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_get, _LibreDWG.Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_set)

    def __init__(self):
        _LibreDWG.Dwg_POINTCLOUDCOLORMAP_Ramp_swiginit(self, _LibreDWG.new_Dwg_POINTCLOUDCOLORMAP_Ramp())
    __swig_destroy__ = _LibreDWG.delete_Dwg_POINTCLOUDCOLORMAP_Ramp

# Register Dwg_POINTCLOUDCOLORMAP_Ramp in _LibreDWG:
_LibreDWG.Dwg_POINTCLOUDCOLORMAP_Ramp_swigregister(Dwg_POINTCLOUDCOLORMAP_Ramp)
class Dwg_Object_POINTCLOUDCOLORMAP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_parent_get, _LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_parent_set)
    class_version = property(_LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_class_version_get, _LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_class_version_set)
    def_intensity_colorscheme = property(_LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_get, _LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_set)
    def_elevation_colorscheme = property(_LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_get, _LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_set)
    def_classification_colorscheme = property(_LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_get, _LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_set)
    num_colorramps = property(_LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_get, _LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_set)
    colorramps = property(_LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_colorramps_get, _LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_colorramps_set)
    num_classification_colorramps = property(_LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_get, _LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_set)
    classification_colorramps = property(_LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_get, _LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_swiginit(self, _LibreDWG.new_Dwg_Object_POINTCLOUDCOLORMAP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_POINTCLOUDCOLORMAP

# Register Dwg_Object_POINTCLOUDCOLORMAP in _LibreDWG:
_LibreDWG.Dwg_Object_POINTCLOUDCOLORMAP_swigregister(Dwg_Object_POINTCLOUDCOLORMAP)
class Dwg_COMPOUNDOBJECTID(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_COMPOUNDOBJECTID_parent_get, _LibreDWG.Dwg_COMPOUNDOBJECTID_parent_set)
    has_object = property(_LibreDWG.Dwg_COMPOUNDOBJECTID_has_object_get, _LibreDWG.Dwg_COMPOUNDOBJECTID_has_object_set)
    name = property(_LibreDWG.Dwg_COMPOUNDOBJECTID_name_get, _LibreDWG.Dwg_COMPOUNDOBJECTID_name_set)
    object = property(_LibreDWG.Dwg_COMPOUNDOBJECTID_object_get, _LibreDWG.Dwg_COMPOUNDOBJECTID_object_set)

    def __init__(self):
        _LibreDWG.Dwg_COMPOUNDOBJECTID_swiginit(self, _LibreDWG.new_Dwg_COMPOUNDOBJECTID())
    __swig_destroy__ = _LibreDWG.delete_Dwg_COMPOUNDOBJECTID

# Register Dwg_COMPOUNDOBJECTID in _LibreDWG:
_LibreDWG.Dwg_COMPOUNDOBJECTID_swigregister(Dwg_COMPOUNDOBJECTID)
class Dwg_PARTIAL_VIEWING_INDEX_Entry(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_get, _LibreDWG.Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_set)
    extents_min = property(_LibreDWG.Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_get, _LibreDWG.Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_set)
    extents_max = property(_LibreDWG.Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_get, _LibreDWG.Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_set)
    object = property(_LibreDWG.Dwg_PARTIAL_VIEWING_INDEX_Entry_object_get, _LibreDWG.Dwg_PARTIAL_VIEWING_INDEX_Entry_object_set)

    def __init__(self):
        _LibreDWG.Dwg_PARTIAL_VIEWING_INDEX_Entry_swiginit(self, _LibreDWG.new_Dwg_PARTIAL_VIEWING_INDEX_Entry())
    __swig_destroy__ = _LibreDWG.delete_Dwg_PARTIAL_VIEWING_INDEX_Entry

# Register Dwg_PARTIAL_VIEWING_INDEX_Entry in _LibreDWG:
_LibreDWG.Dwg_PARTIAL_VIEWING_INDEX_Entry_swigregister(Dwg_PARTIAL_VIEWING_INDEX_Entry)
class Dwg_Object_PARTIAL_VIEWING_INDEX(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_PARTIAL_VIEWING_INDEX_parent_get, _LibreDWG.Dwg_Object_PARTIAL_VIEWING_INDEX_parent_set)
    num_entries = property(_LibreDWG.Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_get, _LibreDWG.Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_set)
    has_entries = property(_LibreDWG.Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_get, _LibreDWG.Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_set)
    entries = property(_LibreDWG.Dwg_Object_PARTIAL_VIEWING_INDEX_entries_get, _LibreDWG.Dwg_Object_PARTIAL_VIEWING_INDEX_entries_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_PARTIAL_VIEWING_INDEX_swiginit(self, _LibreDWG.new_Dwg_Object_PARTIAL_VIEWING_INDEX())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_PARTIAL_VIEWING_INDEX

# Register Dwg_Object_PARTIAL_VIEWING_INDEX in _LibreDWG:
_LibreDWG.Dwg_Object_PARTIAL_VIEWING_INDEX_swigregister(Dwg_Object_PARTIAL_VIEWING_INDEX)
class Dwg_Entity_UNKNOWN_ENT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_UNKNOWN_ENT_parent_get, _LibreDWG.Dwg_Entity_UNKNOWN_ENT_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_UNKNOWN_ENT_swiginit(self, _LibreDWG.new_Dwg_Entity_UNKNOWN_ENT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_UNKNOWN_ENT

# Register Dwg_Entity_UNKNOWN_ENT in _LibreDWG:
_LibreDWG.Dwg_Entity_UNKNOWN_ENT_swigregister(Dwg_Entity_UNKNOWN_ENT)
class Dwg_Object_UNKNOWN_OBJ(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Object_UNKNOWN_OBJ_parent_get, _LibreDWG.Dwg_Object_UNKNOWN_OBJ_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_UNKNOWN_OBJ_swiginit(self, _LibreDWG.new_Dwg_Object_UNKNOWN_OBJ())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_UNKNOWN_OBJ

# Register Dwg_Object_UNKNOWN_OBJ in _LibreDWG:
_LibreDWG.Dwg_Object_UNKNOWN_OBJ_swigregister(Dwg_Object_UNKNOWN_OBJ)
class Dwg_Entity_REPEAT(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_REPEAT_parent_get, _LibreDWG.Dwg_Entity_REPEAT_parent_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_REPEAT_swiginit(self, _LibreDWG.new_Dwg_Entity_REPEAT())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_REPEAT

# Register Dwg_Entity_REPEAT in _LibreDWG:
_LibreDWG.Dwg_Entity_REPEAT_swigregister(Dwg_Entity_REPEAT)
class Dwg_Entity_ENDREP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_ENDREP_parent_get, _LibreDWG.Dwg_Entity_ENDREP_parent_set)
    numcols = property(_LibreDWG.Dwg_Entity_ENDREP_numcols_get, _LibreDWG.Dwg_Entity_ENDREP_numcols_set)
    numrows = property(_LibreDWG.Dwg_Entity_ENDREP_numrows_get, _LibreDWG.Dwg_Entity_ENDREP_numrows_set)
    colspacing = property(_LibreDWG.Dwg_Entity_ENDREP_colspacing_get, _LibreDWG.Dwg_Entity_ENDREP_colspacing_set)
    rowspacing = property(_LibreDWG.Dwg_Entity_ENDREP_rowspacing_get, _LibreDWG.Dwg_Entity_ENDREP_rowspacing_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_ENDREP_swiginit(self, _LibreDWG.new_Dwg_Entity_ENDREP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_ENDREP

# Register Dwg_Entity_ENDREP in _LibreDWG:
_LibreDWG.Dwg_Entity_ENDREP_swigregister(Dwg_Entity_ENDREP)
class Dwg_Entity_LOAD(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_LOAD_parent_get, _LibreDWG.Dwg_Entity_LOAD_parent_set)
    file_name = property(_LibreDWG.Dwg_Entity_LOAD_file_name_get, _LibreDWG.Dwg_Entity_LOAD_file_name_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_LOAD_swiginit(self, _LibreDWG.new_Dwg_Entity_LOAD())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_LOAD

# Register Dwg_Entity_LOAD in _LibreDWG:
_LibreDWG.Dwg_Entity_LOAD_swigregister(Dwg_Entity_LOAD)
class Dwg_Entity_JUMP(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity_JUMP_parent_get, _LibreDWG.Dwg_Entity_JUMP_parent_set)
    jump_address_raw = property(_LibreDWG.Dwg_Entity_JUMP_jump_address_raw_get, _LibreDWG.Dwg_Entity_JUMP_jump_address_raw_set)
    jump_address = property(_LibreDWG.Dwg_Entity_JUMP_jump_address_get, _LibreDWG.Dwg_Entity_JUMP_jump_address_set)
    jump_entity_section = property(_LibreDWG.Dwg_Entity_JUMP_jump_entity_section_get, _LibreDWG.Dwg_Entity_JUMP_jump_entity_section_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity_JUMP_swiginit(self, _LibreDWG.new_Dwg_Entity_JUMP())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity_JUMP

# Register Dwg_Entity_JUMP in _LibreDWG:
_LibreDWG.Dwg_Entity_JUMP_swigregister(Dwg_Entity_JUMP)
class Dwg_Entity__3DLINE(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_LibreDWG.Dwg_Entity__3DLINE_parent_get, _LibreDWG.Dwg_Entity__3DLINE_parent_set)
    start = property(_LibreDWG.Dwg_Entity__3DLINE_start_get, _LibreDWG.Dwg_Entity__3DLINE_start_set)
    end = property(_LibreDWG.Dwg_Entity__3DLINE_end_get, _LibreDWG.Dwg_Entity__3DLINE_end_set)
    extrusion = property(_LibreDWG.Dwg_Entity__3DLINE_extrusion_get, _LibreDWG.Dwg_Entity__3DLINE_extrusion_set)
    thickness = property(_LibreDWG.Dwg_Entity__3DLINE_thickness_get, _LibreDWG.Dwg_Entity__3DLINE_thickness_set)

    def __init__(self):
        _LibreDWG.Dwg_Entity__3DLINE_swiginit(self, _LibreDWG.new_Dwg_Entity__3DLINE())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Entity__3DLINE

# Register Dwg_Entity__3DLINE in _LibreDWG:
_LibreDWG.Dwg_Entity__3DLINE_swigregister(Dwg_Entity__3DLINE)
class Dwg_Eed_Data(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    u = property(_LibreDWG.Dwg_Eed_Data_u_get, _LibreDWG.Dwg_Eed_Data_u_set)
    code = property(_LibreDWG.Dwg_Eed_Data_code_get, _LibreDWG.Dwg_Eed_Data_code_set)

    def __init__(self):
        _LibreDWG.Dwg_Eed_Data_swiginit(self, _LibreDWG.new_Dwg_Eed_Data())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Eed_Data

# Register Dwg_Eed_Data in _LibreDWG:
_LibreDWG.Dwg_Eed_Data_swigregister(Dwg_Eed_Data)
class eed_data_t(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    eed_0 = property(_LibreDWG.eed_data_t_eed_0_get)
    eed_0_r2007 = property(_LibreDWG.eed_data_t_eed_0_r2007_get)
    eed_1 = property(_LibreDWG.eed_data_t_eed_1_get)
    eed_2 = property(_LibreDWG.eed_data_t_eed_2_get)
    eed_3 = property(_LibreDWG.eed_data_t_eed_3_get)
    eed_4 = property(_LibreDWG.eed_data_t_eed_4_get)
    eed_5 = property(_LibreDWG.eed_data_t_eed_5_get)
    eed_10 = property(_LibreDWG.eed_data_t_eed_10_get)
    eed_40 = property(_LibreDWG.eed_data_t_eed_40_get)
    eed_70 = property(_LibreDWG.eed_data_t_eed_70_get)
    eed_71 = property(_LibreDWG.eed_data_t_eed_71_get)

    def __init__(self):
        _LibreDWG.eed_data_t_swiginit(self, _LibreDWG.new_eed_data_t())
    __swig_destroy__ = _LibreDWG.delete_eed_data_t

# Register eed_data_t in _LibreDWG:
_LibreDWG.eed_data_t_swigregister(eed_data_t)
class eed_data_t_eed_71(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    rl = property(_LibreDWG.eed_data_t_eed_71_rl_get, _LibreDWG.eed_data_t_eed_71_rl_set)

    def __init__(self):
        _LibreDWG.eed_data_t_eed_71_swiginit(self, _LibreDWG.new_eed_data_t_eed_71())
    __swig_destroy__ = _LibreDWG.delete_eed_data_t_eed_71

# Register eed_data_t_eed_71 in _LibreDWG:
_LibreDWG.eed_data_t_eed_71_swigregister(eed_data_t_eed_71)
class eed_data_t_eed_70(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    rs = property(_LibreDWG.eed_data_t_eed_70_rs_get, _LibreDWG.eed_data_t_eed_70_rs_set)

    def __init__(self):
        _LibreDWG.eed_data_t_eed_70_swiginit(self, _LibreDWG.new_eed_data_t_eed_70())
    __swig_destroy__ = _LibreDWG.delete_eed_data_t_eed_70

# Register eed_data_t_eed_70 in _LibreDWG:
_LibreDWG.eed_data_t_eed_70_swigregister(eed_data_t_eed_70)
class eed_data_t_eed_40(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    real = property(_LibreDWG.eed_data_t_eed_40_real_get, _LibreDWG.eed_data_t_eed_40_real_set)

    def __init__(self):
        _LibreDWG.eed_data_t_eed_40_swiginit(self, _LibreDWG.new_eed_data_t_eed_40())
    __swig_destroy__ = _LibreDWG.delete_eed_data_t_eed_40

# Register eed_data_t_eed_40 in _LibreDWG:
_LibreDWG.eed_data_t_eed_40_swigregister(eed_data_t_eed_40)
class eed_data_t_eed_10(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    point = property(_LibreDWG.eed_data_t_eed_10_point_get, _LibreDWG.eed_data_t_eed_10_point_set)

    def __init__(self):
        _LibreDWG.eed_data_t_eed_10_swiginit(self, _LibreDWG.new_eed_data_t_eed_10())
    __swig_destroy__ = _LibreDWG.delete_eed_data_t_eed_10

# Register eed_data_t_eed_10 in _LibreDWG:
_LibreDWG.eed_data_t_eed_10_swigregister(eed_data_t_eed_10)
class eed_data_t_eed_5(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    entity = property(_LibreDWG.eed_data_t_eed_5_entity_get, _LibreDWG.eed_data_t_eed_5_entity_set)

    def __init__(self):
        _LibreDWG.eed_data_t_eed_5_swiginit(self, _LibreDWG.new_eed_data_t_eed_5())
    __swig_destroy__ = _LibreDWG.delete_eed_data_t_eed_5

# Register eed_data_t_eed_5 in _LibreDWG:
_LibreDWG.eed_data_t_eed_5_swigregister(eed_data_t_eed_5)
class eed_data_t_eed_4(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    length = property(_LibreDWG.eed_data_t_eed_4_length_get, _LibreDWG.eed_data_t_eed_4_length_set)
    data = property(_LibreDWG.eed_data_t_eed_4_data_get)

    def __init__(self):
        _LibreDWG.eed_data_t_eed_4_swiginit(self, _LibreDWG.new_eed_data_t_eed_4())
    __swig_destroy__ = _LibreDWG.delete_eed_data_t_eed_4

# Register eed_data_t_eed_4 in _LibreDWG:
_LibreDWG.eed_data_t_eed_4_swigregister(eed_data_t_eed_4)
class eed_data_t_eed_3(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    layer = property(_LibreDWG.eed_data_t_eed_3_layer_get, _LibreDWG.eed_data_t_eed_3_layer_set)

    def __init__(self):
        _LibreDWG.eed_data_t_eed_3_swiginit(self, _LibreDWG.new_eed_data_t_eed_3())
    __swig_destroy__ = _LibreDWG.delete_eed_data_t_eed_3

# Register eed_data_t_eed_3 in _LibreDWG:
_LibreDWG.eed_data_t_eed_3_swigregister(eed_data_t_eed_3)
class eed_data_t_eed_2(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    close = property(_LibreDWG.eed_data_t_eed_2_close_get, _LibreDWG.eed_data_t_eed_2_close_set)

    def __init__(self):
        _LibreDWG.eed_data_t_eed_2_swiginit(self, _LibreDWG.new_eed_data_t_eed_2())
    __swig_destroy__ = _LibreDWG.delete_eed_data_t_eed_2

# Register eed_data_t_eed_2 in _LibreDWG:
_LibreDWG.eed_data_t_eed_2_swigregister(eed_data_t_eed_2)
class eed_data_t_eed_1(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    appid_index = property(_LibreDWG.eed_data_t_eed_1_appid_index_get, _LibreDWG.eed_data_t_eed_1_appid_index_set)
    invalid = property(_LibreDWG.eed_data_t_eed_1_invalid_get, _LibreDWG.eed_data_t_eed_1_invalid_set)

    def __init__(self):
        _LibreDWG.eed_data_t_eed_1_swiginit(self, _LibreDWG.new_eed_data_t_eed_1())
    __swig_destroy__ = _LibreDWG.delete_eed_data_t_eed_1

# Register eed_data_t_eed_1 in _LibreDWG:
_LibreDWG.eed_data_t_eed_1_swigregister(eed_data_t_eed_1)
class eed_data_t_eed_0_r2007(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    length = property(_LibreDWG.eed_data_t_eed_0_r2007_length_get, _LibreDWG.eed_data_t_eed_0_r2007_length_set)
    _padding = property(_LibreDWG.eed_data_t_eed_0_r2007__padding_get, _LibreDWG.eed_data_t_eed_0_r2007__padding_set)
    is_tu = property(_LibreDWG.eed_data_t_eed_0_r2007_is_tu_get, _LibreDWG.eed_data_t_eed_0_r2007_is_tu_set)
    string = property(_LibreDWG.eed_data_t_eed_0_r2007_string_get, _LibreDWG.eed_data_t_eed_0_r2007_string_set)

    def __init__(self):
        _LibreDWG.eed_data_t_eed_0_r2007_swiginit(self, _LibreDWG.new_eed_data_t_eed_0_r2007())
    __swig_destroy__ = _LibreDWG.delete_eed_data_t_eed_0_r2007

# Register eed_data_t_eed_0_r2007 in _LibreDWG:
_LibreDWG.eed_data_t_eed_0_r2007_swigregister(eed_data_t_eed_0_r2007)
class eed_data_t_eed_0(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    length = property(_LibreDWG.eed_data_t_eed_0_length_get, _LibreDWG.eed_data_t_eed_0_length_set)
    codepage = property(_LibreDWG.eed_data_t_eed_0_codepage_get, _LibreDWG.eed_data_t_eed_0_codepage_set)
    is_tu = property(_LibreDWG.eed_data_t_eed_0_is_tu_get, _LibreDWG.eed_data_t_eed_0_is_tu_set)
    string = property(_LibreDWG.eed_data_t_eed_0_string_get, _LibreDWG.eed_data_t_eed_0_string_set)

    def __init__(self):
        _LibreDWG.eed_data_t_eed_0_swiginit(self, _LibreDWG.new_eed_data_t_eed_0())
    __swig_destroy__ = _LibreDWG.delete_eed_data_t_eed_0

# Register eed_data_t_eed_0 in _LibreDWG:
_LibreDWG.eed_data_t_eed_0_swigregister(eed_data_t_eed_0)
class Dwg_Eed(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    size = property(_LibreDWG.Dwg_Eed_size_get, _LibreDWG.Dwg_Eed_size_set)
    handle = property(_LibreDWG.Dwg_Eed_handle_get, _LibreDWG.Dwg_Eed_handle_set)
    data = property(_LibreDWG.Dwg_Eed_data_get, _LibreDWG.Dwg_Eed_data_set)
    raw = property(_LibreDWG.Dwg_Eed_raw_get, _LibreDWG.Dwg_Eed_raw_set)

    def __init__(self):
        _LibreDWG.Dwg_Eed_swiginit(self, _LibreDWG.new_Dwg_Eed())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Eed

# Register Dwg_Eed in _LibreDWG:
_LibreDWG.Dwg_Eed_swigregister(Dwg_Eed)
EXTRA_R11_UNKNOWN_1 = _LibreDWG.EXTRA_R11_UNKNOWN_1
EXTRA_R11_HAS_EED = _LibreDWG.EXTRA_R11_HAS_EED
EXTRA_R11_HAS_VIEWPORT = _LibreDWG.EXTRA_R11_HAS_VIEWPORT
EXTRA_R11_UNKNOWN_8 = _LibreDWG.EXTRA_R11_UNKNOWN_8
EXTRA_R11_UNKNOWN_16 = _LibreDWG.EXTRA_R11_UNKNOWN_16
EXTRA_R11_UNKNOWN_32 = _LibreDWG.EXTRA_R11_UNKNOWN_32
EXTRA_R11_UNKNOWN_64 = _LibreDWG.EXTRA_R11_UNKNOWN_64
EXTRA_R11_UNKNOWN_128 = _LibreDWG.EXTRA_R11_UNKNOWN_128
FLAG_R11_HAS_COLOR = _LibreDWG.FLAG_R11_HAS_COLOR
FLAG_R11_HAS_LTYPE = _LibreDWG.FLAG_R11_HAS_LTYPE
FLAG_R11_HAS_ELEVATION = _LibreDWG.FLAG_R11_HAS_ELEVATION
FLAG_R11_HAS_THICKNESS = _LibreDWG.FLAG_R11_HAS_THICKNESS
FLAG_R11_HAS_UNKNOWN_16 = _LibreDWG.FLAG_R11_HAS_UNKNOWN_16
FLAG_R11_HAS_HANDLING = _LibreDWG.FLAG_R11_HAS_HANDLING
FLAG_R11_HAS_PSPACE = _LibreDWG.FLAG_R11_HAS_PSPACE
FLAG_R11_HAS_ATTRIBS = _LibreDWG.FLAG_R11_HAS_ATTRIBS
FLAG_R11_DIMENSION_LINEAR = _LibreDWG.FLAG_R11_DIMENSION_LINEAR
FLAG_R11_DIMENSION_ALIGNED = _LibreDWG.FLAG_R11_DIMENSION_ALIGNED
FLAG_R11_DIMENSION_ANG2LN = _LibreDWG.FLAG_R11_DIMENSION_ANG2LN
FLAG_R11_DIMENSION_DIAMETER = _LibreDWG.FLAG_R11_DIMENSION_DIAMETER
FLAG_R11_DIMENSION_RADIUS = _LibreDWG.FLAG_R11_DIMENSION_RADIUS
FLAG_R11_DIMENSION_ANG3PT = _LibreDWG.FLAG_R11_DIMENSION_ANG3PT
FLAG_R11_DIMENSION_ORDINATE = _LibreDWG.FLAG_R11_DIMENSION_ORDINATE
FLAG_R11_DIMENSION_ORDINATE_TYPE = _LibreDWG.FLAG_R11_DIMENSION_ORDINATE_TYPE
FLAG_R11_DIMENSION_TEXT_POSITIONED = _LibreDWG.FLAG_R11_DIMENSION_TEXT_POSITIONED
OPTS_R11_XREF_DEP = _LibreDWG.OPTS_R11_XREF_DEP
OPTS_R11_XREF_REF = _LibreDWG.OPTS_R11_XREF_REF
OPTS_R11_XREF_RESOLVED = _LibreDWG.OPTS_R11_XREF_RESOLVED
OPTS_R11_XDATA = _LibreDWG.OPTS_R11_XDATA
OPTS_R11_3DFACE_HAS_Z_FIRST = _LibreDWG.OPTS_R11_3DFACE_HAS_Z_FIRST
OPTS_R11_3DFACE_HAS_Z_SECOND = _LibreDWG.OPTS_R11_3DFACE_HAS_Z_SECOND
OPTS_R11_3DFACE_HAS_Z_THIRD = _LibreDWG.OPTS_R11_3DFACE_HAS_Z_THIRD
OPTS_R11_3DFACE_HAS_Z_FOURTH = _LibreDWG.OPTS_R11_3DFACE_HAS_Z_FOURTH
OPTS_R11_3DLINE_HAS_Z_FIRST = _LibreDWG.OPTS_R11_3DLINE_HAS_Z_FIRST
OPTS_R11_3DLINE_HAS_Z_SECOND = _LibreDWG.OPTS_R11_3DLINE_HAS_Z_SECOND
OPTS_R11_ARC_HAS_EXTRUSION = _LibreDWG.OPTS_R11_ARC_HAS_EXTRUSION
OPTS_R11_ATTDEF_UNKNOWN_1 = _LibreDWG.OPTS_R11_ATTDEF_UNKNOWN_1
OPTS_R11_ATTDEF_HAS_ROTATION = _LibreDWG.OPTS_R11_ATTDEF_HAS_ROTATION
OPTS_R11_ATTDEF_HAS_WIDTH_FACTOR = _LibreDWG.OPTS_R11_ATTDEF_HAS_WIDTH_FACTOR
OPTS_R11_ATTDEF_HAS_OBLIQUE_ANGLE = _LibreDWG.OPTS_R11_ATTDEF_HAS_OBLIQUE_ANGLE
OPTS_R11_ATTDEF_HAS_STYLE = _LibreDWG.OPTS_R11_ATTDEF_HAS_STYLE
OPTS_R11_ATTDEF_HAS_GENERATION = _LibreDWG.OPTS_R11_ATTDEF_HAS_GENERATION
OPTS_R11_ATTDEF_HAS_HORIZ_ALIGNMENT = _LibreDWG.OPTS_R11_ATTDEF_HAS_HORIZ_ALIGNMENT
OPTS_R11_ATTDEF_HAS_ALIGNMENT_POINT = _LibreDWG.OPTS_R11_ATTDEF_HAS_ALIGNMENT_POINT
OPTS_R11_ATTDEF_HAS_EXTRUSION = _LibreDWG.OPTS_R11_ATTDEF_HAS_EXTRUSION
OPTS_R11_ATTDEF_HAS_VERT_ALIGNMENT = _LibreDWG.OPTS_R11_ATTDEF_HAS_VERT_ALIGNMENT
OPTS_R11_ATTRIB_UNKNOWN_1 = _LibreDWG.OPTS_R11_ATTRIB_UNKNOWN_1
OPTS_R11_ATTRIB_HAS_ROTATION = _LibreDWG.OPTS_R11_ATTRIB_HAS_ROTATION
OPTS_R11_ATTRIB_HAS_WIDTH_FACTOR = _LibreDWG.OPTS_R11_ATTRIB_HAS_WIDTH_FACTOR
OPTS_R11_ATTRIB_HAS_OBLIQUE_ANGLE = _LibreDWG.OPTS_R11_ATTRIB_HAS_OBLIQUE_ANGLE
OPTS_R11_ATTRIB_HAS_STYLE = _LibreDWG.OPTS_R11_ATTRIB_HAS_STYLE
OPTS_R11_ATTRIB_HAS_GENERATION = _LibreDWG.OPTS_R11_ATTRIB_HAS_GENERATION
OPTS_R11_ATTRIB_HAS_HORIZ_ALIGNMENT = _LibreDWG.OPTS_R11_ATTRIB_HAS_HORIZ_ALIGNMENT
OPTS_R11_ATTRIB_HAS_ALIGNMENT_POINT = _LibreDWG.OPTS_R11_ATTRIB_HAS_ALIGNMENT_POINT
OPTS_R11_ATTRIB_HAS_EXTRUSION = _LibreDWG.OPTS_R11_ATTRIB_HAS_EXTRUSION
OPTS_R11_ATTRIB_HAS_VERT_ALIGNMENT = _LibreDWG.OPTS_R11_ATTRIB_HAS_VERT_ALIGNMENT
OPTS_R11_BLOCK_UNKNOWN_1 = _LibreDWG.OPTS_R11_BLOCK_UNKNOWN_1
OPTS_R11_BLOCK_HAS_XREF_PNAME = _LibreDWG.OPTS_R11_BLOCK_HAS_XREF_PNAME
OPTS_R11_BLOCK_HAS_BLOCK_NAME = _LibreDWG.OPTS_R11_BLOCK_HAS_BLOCK_NAME
OPTS_R11_CIRCLE_HAS_EXTRUSION = _LibreDWG.OPTS_R11_CIRCLE_HAS_EXTRUSION
OPTS_R11_DIMENSION_HAS_DXF12 = _LibreDWG.OPTS_R11_DIMENSION_HAS_DXF12
OPTS_R11_DIMENSION_HAS_FLAG = _LibreDWG.OPTS_R11_DIMENSION_HAS_FLAG
OPTS_R11_DIMENSION_HAS_TEXT = _LibreDWG.OPTS_R11_DIMENSION_HAS_TEXT
OPTS_R11_DIMENSION_HAS_DXF13 = _LibreDWG.OPTS_R11_DIMENSION_HAS_DXF13
OPTS_R11_DIMENSION_HAS_DXF14 = _LibreDWG.OPTS_R11_DIMENSION_HAS_DXF14
OPTS_R11_DIMENSION_HAS_DXF15 = _LibreDWG.OPTS_R11_DIMENSION_HAS_DXF15
OPTS_R11_DIMENSION_HAS_ANGLES = _LibreDWG.OPTS_R11_DIMENSION_HAS_ANGLES
OPTS_R11_DIMENSION_HAS_DXF40 = _LibreDWG.OPTS_R11_DIMENSION_HAS_DXF40
OPTS_R11_DIMENSION_HAS_ROTATION = _LibreDWG.OPTS_R11_DIMENSION_HAS_ROTATION
OPTS_R11_DIMENSION_UNKNOWN_512 = _LibreDWG.OPTS_R11_DIMENSION_UNKNOWN_512
OPTS_R11_DIMENSION_HAS_DXF53 = _LibreDWG.OPTS_R11_DIMENSION_HAS_DXF53
OPTS_R11_DIMENSION_UNKNOWN_2048 = _LibreDWG.OPTS_R11_DIMENSION_UNKNOWN_2048
OPTS_R11_DIMENSION_UNKNOWN_4096 = _LibreDWG.OPTS_R11_DIMENSION_UNKNOWN_4096
OPTS_R11_DIMENSION_UNKNOWN_8192 = _LibreDWG.OPTS_R11_DIMENSION_UNKNOWN_8192
OPTS_R11_DIMENSION_HAS_EXTRUSION = _LibreDWG.OPTS_R11_DIMENSION_HAS_EXTRUSION
OPTS_R11_DIMENSION_HAS_DIMSTYLE = _LibreDWG.OPTS_R11_DIMENSION_HAS_DIMSTYLE
OPTS_R11_INSERT_HAS_SCALE_X = _LibreDWG.OPTS_R11_INSERT_HAS_SCALE_X
OPTS_R11_INSERT_HAS_SCALE_Y = _LibreDWG.OPTS_R11_INSERT_HAS_SCALE_Y
OPTS_R11_INSERT_HAS_ROTATION = _LibreDWG.OPTS_R11_INSERT_HAS_ROTATION
OPTS_R11_INSERT_HAS_SCALE_Z = _LibreDWG.OPTS_R11_INSERT_HAS_SCALE_Z
OPTS_R11_INSERT_HAS_NUM_COLS = _LibreDWG.OPTS_R11_INSERT_HAS_NUM_COLS
OPTS_R11_INSERT_HAS_NUM_ROWS = _LibreDWG.OPTS_R11_INSERT_HAS_NUM_ROWS
OPTS_R11_INSERT_HAS_COL_SPACING = _LibreDWG.OPTS_R11_INSERT_HAS_COL_SPACING
OPTS_R11_INSERT_HAS_ROW_SPACING = _LibreDWG.OPTS_R11_INSERT_HAS_ROW_SPACING
OPTS_R11_INSERT_HAS_EXTRUSION = _LibreDWG.OPTS_R11_INSERT_HAS_EXTRUSION
OPTS_R11_LINE_HAS_EXTRUSION = _LibreDWG.OPTS_R11_LINE_HAS_EXTRUSION
OPTS_R11_POINT_HAS_EXTRUSION = _LibreDWG.OPTS_R11_POINT_HAS_EXTRUSION
OPTS_R11_POINT_HAS_X_ANG = _LibreDWG.OPTS_R11_POINT_HAS_X_ANG
OPTS_R11_POLYLINE_HAS_FLAG = _LibreDWG.OPTS_R11_POLYLINE_HAS_FLAG
OPTS_R11_POLYLINE_HAS_START_WIDTH = _LibreDWG.OPTS_R11_POLYLINE_HAS_START_WIDTH
OPTS_R11_POLYLINE_HAS_END_WIDTH = _LibreDWG.OPTS_R11_POLYLINE_HAS_END_WIDTH
OPTS_R11_POLYLINE_HAS_EXTRUSION = _LibreDWG.OPTS_R11_POLYLINE_HAS_EXTRUSION
OPTS_R11_POLYLINE_HAS_M_VERTS = _LibreDWG.OPTS_R11_POLYLINE_HAS_M_VERTS
OPTS_R11_POLYLINE_HAS_N_VERTS = _LibreDWG.OPTS_R11_POLYLINE_HAS_N_VERTS
OPTS_R11_POLYLINE_HAS_M_DENSITY = _LibreDWG.OPTS_R11_POLYLINE_HAS_M_DENSITY
OPTS_R11_POLYLINE_HAS_N_DENSITY = _LibreDWG.OPTS_R11_POLYLINE_HAS_N_DENSITY
OPTS_R11_POLYLINE_HAS_CURVETYPE = _LibreDWG.OPTS_R11_POLYLINE_HAS_CURVETYPE
OPTS_R11_POLYLINE_IN_EXTRA = _LibreDWG.OPTS_R11_POLYLINE_IN_EXTRA
OPTS_R11_SHAPE_HAS_ROTATION = _LibreDWG.OPTS_R11_SHAPE_HAS_ROTATION
OPTS_R11_SHAPE_HAS_LOAD_NUM = _LibreDWG.OPTS_R11_SHAPE_HAS_LOAD_NUM
OPTS_R11_SHAPE_HAS_WIDTH_FACTOR = _LibreDWG.OPTS_R11_SHAPE_HAS_WIDTH_FACTOR
OPTS_R11_SHAPE_HAS_OBLIQUE_ANGLE = _LibreDWG.OPTS_R11_SHAPE_HAS_OBLIQUE_ANGLE
OPTS_R11_SOLID_HAS_EXTRUSION = _LibreDWG.OPTS_R11_SOLID_HAS_EXTRUSION
OPTS_R11_TEXT_HAS_ROTATION = _LibreDWG.OPTS_R11_TEXT_HAS_ROTATION
OPTS_R11_TEXT_HAS_WIDTH_FACTOR = _LibreDWG.OPTS_R11_TEXT_HAS_WIDTH_FACTOR
OPTS_R11_TEXT_HAS_OBLIQUE_ANGLE = _LibreDWG.OPTS_R11_TEXT_HAS_OBLIQUE_ANGLE
OPTS_R11_TEXT_HAS_STYLE = _LibreDWG.OPTS_R11_TEXT_HAS_STYLE
OPTS_R11_TEXT_HAS_GENERATION = _LibreDWG.OPTS_R11_TEXT_HAS_GENERATION
OPTS_R11_TEXT_HAS_HORIZ_ALIGNMENT = _LibreDWG.OPTS_R11_TEXT_HAS_HORIZ_ALIGNMENT
OPTS_R11_TEXT_HAS_ALIGNMENT_POINT = _LibreDWG.OPTS_R11_TEXT_HAS_ALIGNMENT_POINT
OPTS_R11_TEXT_HAS_EXTRUSION = _LibreDWG.OPTS_R11_TEXT_HAS_EXTRUSION
OPTS_R11_TEXT_ALIGNED_VERT_TO = _LibreDWG.OPTS_R11_TEXT_ALIGNED_VERT_TO
OPTS_R11_VERTEX_HAS_START_WIDTH = _LibreDWG.OPTS_R11_VERTEX_HAS_START_WIDTH
OPTS_R11_VERTEX_HAS_END_WIDTH = _LibreDWG.OPTS_R11_VERTEX_HAS_END_WIDTH
OPTS_R11_VERTEX_HAS_BULGE = _LibreDWG.OPTS_R11_VERTEX_HAS_BULGE
OPTS_R11_VERTEX_HAS_FLAG = _LibreDWG.OPTS_R11_VERTEX_HAS_FLAG
OPTS_R11_VERTEX_HAS_TANGENT_DIR = _LibreDWG.OPTS_R11_VERTEX_HAS_TANGENT_DIR
OPTS_R11_VERTEX_HAS_INDEX1 = _LibreDWG.OPTS_R11_VERTEX_HAS_INDEX1
OPTS_R11_VERTEX_HAS_INDEX2 = _LibreDWG.OPTS_R11_VERTEX_HAS_INDEX2
OPTS_R11_VERTEX_HAS_INDEX3 = _LibreDWG.OPTS_R11_VERTEX_HAS_INDEX3
OPTS_R11_VERTEX_HAS_INDEX4 = _LibreDWG.OPTS_R11_VERTEX_HAS_INDEX4
OPTS_R11_VERTEX_UNKNOWN_512 = _LibreDWG.OPTS_R11_VERTEX_UNKNOWN_512
OPTS_R11_VERTEX_UNKNOWN_1024 = _LibreDWG.OPTS_R11_VERTEX_UNKNOWN_1024
OPTS_R11_VERTEX_UNKNOWN_2048 = _LibreDWG.OPTS_R11_VERTEX_UNKNOWN_2048
OPTS_R11_VERTEX_UNKNOWN_4096 = _LibreDWG.OPTS_R11_VERTEX_UNKNOWN_4096
OPTS_R11_VERTEX_UNKNOWN_8192 = _LibreDWG.OPTS_R11_VERTEX_UNKNOWN_8192
OPTS_R11_VERTEX_HAS_NOT_X_Y = _LibreDWG.OPTS_R11_VERTEX_HAS_NOT_X_Y
FLAG_TABLE_IS_XREF_REF = _LibreDWG.FLAG_TABLE_IS_XREF_REF
FLAG_TABLE_IS_XREF_RESOLVED = _LibreDWG.FLAG_TABLE_IS_XREF_RESOLVED
FLAG_TABLE_IS_XREF_DEP = _LibreDWG.FLAG_TABLE_IS_XREF_DEP
FLAG_TABLE_IS_REMOVED = _LibreDWG.FLAG_TABLE_IS_REMOVED
FLAG_BLOCK_ANONYMOUS = _LibreDWG.FLAG_BLOCK_ANONYMOUS
FLAG_BLOCK_HAS_ATTRIBS = _LibreDWG.FLAG_BLOCK_HAS_ATTRIBS
FLAG_BLOCK_IS_EXT_REF = _LibreDWG.FLAG_BLOCK_IS_EXT_REF
FLAG_BLOCK_IS_XREF_OVERLAY = _LibreDWG.FLAG_BLOCK_IS_XREF_OVERLAY
FLAG_ATTDEF_INVISIBLE = _LibreDWG.FLAG_ATTDEF_INVISIBLE
FLAG_ATTDEF_CONSTANT = _LibreDWG.FLAG_ATTDEF_CONSTANT
FLAG_ATTDEF_VERIFY = _LibreDWG.FLAG_ATTDEF_VERIFY
FLAG_ATTDEF_PRESET = _LibreDWG.FLAG_ATTDEF_PRESET
FLAG_ATTRIB_INVISIBLE = _LibreDWG.FLAG_ATTRIB_INVISIBLE
FLAG_ATTRIB_CONSTANT = _LibreDWG.FLAG_ATTRIB_CONSTANT
FLAG_ATTRIB_VERIFY = _LibreDWG.FLAG_ATTRIB_VERIFY
FLAG_ATTRIB_PRESET = _LibreDWG.FLAG_ATTRIB_PRESET
FLAG_POLYLINE_CLOSED = _LibreDWG.FLAG_POLYLINE_CLOSED
FLAG_POLYLINE_CURVE_FIT = _LibreDWG.FLAG_POLYLINE_CURVE_FIT
FLAG_POLYLINE_SPLINE_FIT = _LibreDWG.FLAG_POLYLINE_SPLINE_FIT
FLAG_POLYLINE_3D = _LibreDWG.FLAG_POLYLINE_3D
FLAG_POLYLINE_MESH = _LibreDWG.FLAG_POLYLINE_MESH
FLAG_POLYLINE_MESH_CLOSED = _LibreDWG.FLAG_POLYLINE_MESH_CLOSED
FLAG_POLYLINE_PFACE_MESH = _LibreDWG.FLAG_POLYLINE_PFACE_MESH
FLAG_POLYLINE_LT_PATTERN_CONTINUES = _LibreDWG.FLAG_POLYLINE_LT_PATTERN_CONTINUES
LTYPE_SHAPE_FLAG_ABS_ROTATION = _LibreDWG.LTYPE_SHAPE_FLAG_ABS_ROTATION
LTYPE_SHAPE_FLAG_IS_TEXT = _LibreDWG.LTYPE_SHAPE_FLAG_IS_TEXT
LTYPE_SHAPE_FLAG_IS_SHAPE = _LibreDWG.LTYPE_SHAPE_FLAG_IS_SHAPE
FLAG_LAYER_FROZEN = _LibreDWG.FLAG_LAYER_FROZEN
FLAG_LAYER_FROZEN_IN_NEW = _LibreDWG.FLAG_LAYER_FROZEN_IN_NEW
FLAG_LAYER_LOCKED = _LibreDWG.FLAG_LAYER_LOCKED
FLAG_LAYER_PLOTFLAG = _LibreDWG.FLAG_LAYER_PLOTFLAG
FLAG_STYLE_SHAPE = _LibreDWG.FLAG_STYLE_SHAPE
FLAG_STYLE_VERTICAL_TEXT = _LibreDWG.FLAG_STYLE_VERTICAL_TEXT
FLAG_VIEW_PSPACE = _LibreDWG.FLAG_VIEW_PSPACE
FLAG_VX_IS_ON = _LibreDWG.FLAG_VX_IS_ON
FLAG_LWPOLYLINE_HAS_EXTRUSION = _LibreDWG.FLAG_LWPOLYLINE_HAS_EXTRUSION
FLAG_LWPOLYLINE_HAS_THICKNESS = _LibreDWG.FLAG_LWPOLYLINE_HAS_THICKNESS
FLAG_LWPOLYLINE_HAS_CONSTWIDTH = _LibreDWG.FLAG_LWPOLYLINE_HAS_CONSTWIDTH
FLAG_LWPOLYLINE_HAS_ELEVATION = _LibreDWG.FLAG_LWPOLYLINE_HAS_ELEVATION
FLAG_LWPOLYLINE_HAS_NUM_BULGES = _LibreDWG.FLAG_LWPOLYLINE_HAS_NUM_BULGES
FLAG_LWPOLYLINE_HAS_NUM_WIDTHS = _LibreDWG.FLAG_LWPOLYLINE_HAS_NUM_WIDTHS
FLAG_LWPOLYLINE_UNKNOWN_64 = _LibreDWG.FLAG_LWPOLYLINE_UNKNOWN_64
FLAG_LWPOLYLINE_UNKNOWN_128 = _LibreDWG.FLAG_LWPOLYLINE_UNKNOWN_128
FLAG_LWPOLYLINE_PLINEGEN = _LibreDWG.FLAG_LWPOLYLINE_PLINEGEN
FLAG_LWPOLYLINE_CLOSED = _LibreDWG.FLAG_LWPOLYLINE_CLOSED
FLAG_LWPOLYLINE_VERTEXIDCOUNT = _LibreDWG.FLAG_LWPOLYLINE_VERTEXIDCOUNT
FLAG_VERTEX_EXTRA_VERTEX = _LibreDWG.FLAG_VERTEX_EXTRA_VERTEX
FLAG_VERTEX_CURVE_FIT = _LibreDWG.FLAG_VERTEX_CURVE_FIT
FLAG_VERTEX_UNUSED_4 = _LibreDWG.FLAG_VERTEX_UNUSED_4
FLAG_VERTEX_SPLINE_FIT = _LibreDWG.FLAG_VERTEX_SPLINE_FIT
FLAG_VERTEX_SPLINE_FRAME_CONTROL_POINT = _LibreDWG.FLAG_VERTEX_SPLINE_FRAME_CONTROL_POINT
FLAG_VERTEX_3D = _LibreDWG.FLAG_VERTEX_3D
FLAG_VERTEX_MESH = _LibreDWG.FLAG_VERTEX_MESH
FLAG_VERTEX_PFACE_MESH = _LibreDWG.FLAG_VERTEX_PFACE_MESH
TEXT_GENERATION_BACKWARDS = _LibreDWG.TEXT_GENERATION_BACKWARDS
TEXT_GENERATION_UPSIDE_DOWN = _LibreDWG.TEXT_GENERATION_UPSIDE_DOWN
HORIZ_ALIGNMENT_LEFT = _LibreDWG.HORIZ_ALIGNMENT_LEFT
HORIZ_ALIGNMENT_CENTER = _LibreDWG.HORIZ_ALIGNMENT_CENTER
HORIZ_ALIGNMENT_RIGHT = _LibreDWG.HORIZ_ALIGNMENT_RIGHT
HORIZ_ALIGNMENT_ALIGNED = _LibreDWG.HORIZ_ALIGNMENT_ALIGNED
HORIZ_ALIGNMENT_MIDDLE = _LibreDWG.HORIZ_ALIGNMENT_MIDDLE
HORIZ_ALIGNMENT_FIT = _LibreDWG.HORIZ_ALIGNMENT_FIT
VERT_ALIGNMENT_BASELINE = _LibreDWG.VERT_ALIGNMENT_BASELINE
VERT_ALIGNMENT_BOTTOM = _LibreDWG.VERT_ALIGNMENT_BOTTOM
VERT_ALIGNMENT_MIDDLE = _LibreDWG.VERT_ALIGNMENT_MIDDLE
VERT_ALIGNMENT_TOP = _LibreDWG.VERT_ALIGNMENT_TOP
MLINE_JUSTIFICATION_TOP = _LibreDWG.MLINE_JUSTIFICATION_TOP
MLINE_JUSTIFICATION_MIDDLE = _LibreDWG.MLINE_JUSTIFICATION_MIDDLE
MLINE_JUSTIFICATION_BOTTOM = _LibreDWG.MLINE_JUSTIFICATION_BOTTOM
POLYLINE_CURVETYPE_DEFAULT = _LibreDWG.POLYLINE_CURVETYPE_DEFAULT
POLYLINE_CURVETYPE_QUADR_BSPLINE = _LibreDWG.POLYLINE_CURVETYPE_QUADR_BSPLINE
POLYLINE_CURVETYPE_CUBIC_BSPLINE = _LibreDWG.POLYLINE_CURVETYPE_CUBIC_BSPLINE
POLYLINE_CURVETYPE_BEZIER_SURFACE = _LibreDWG.POLYLINE_CURVETYPE_BEZIER_SURFACE
LEADER_PATHTYPE_STRAIGHT = _LibreDWG.LEADER_PATHTYPE_STRAIGHT
LEADER_PATHTYPE_SPLINE = _LibreDWG.LEADER_PATHTYPE_SPLINE
LEADER_ANNOTTYPE_MTEXT = _LibreDWG.LEADER_ANNOTTYPE_MTEXT
LEADER_ANNOTTYPE_TOLERANCE = _LibreDWG.LEADER_ANNOTTYPE_TOLERANCE
LEADER_ANNOTTYPE_INSERT = _LibreDWG.LEADER_ANNOTTYPE_INSERT
LEADER_ANNOTTYPE_NO_ANNOT = _LibreDWG.LEADER_ANNOTTYPE_NO_ANNOT
MLINE_FLAGS_HAS_VERTEX = _LibreDWG.MLINE_FLAGS_HAS_VERTEX
MLINE_FLAGS_CLOSED = _LibreDWG.MLINE_FLAGS_CLOSED
MLINE_FLAGS_SUPPRESS_START_CAPS = _LibreDWG.MLINE_FLAGS_SUPPRESS_START_CAPS
MLINE_FLAGS_SUPPRESS_END_CAPS = _LibreDWG.MLINE_FLAGS_SUPPRESS_END_CAPS
MLINESTYLE_FLAG_FILL = _LibreDWG.MLINESTYLE_FLAG_FILL
MLINESTYLE_FLAG_MITERS = _LibreDWG.MLINESTYLE_FLAG_MITERS
MLINESTYLE_FLAG_UNKNOWN_4 = _LibreDWG.MLINESTYLE_FLAG_UNKNOWN_4
MLINESTYLE_FLAG_UNKNOWN_8 = _LibreDWG.MLINESTYLE_FLAG_UNKNOWN_8
MLINESTYLE_FLAG_START_SQUARE_END = _LibreDWG.MLINESTYLE_FLAG_START_SQUARE_END
MLINESTYLE_FLAG_START_INNER_ARC = _LibreDWG.MLINESTYLE_FLAG_START_INNER_ARC
MLINESTYLE_FLAG_START_ROUND = _LibreDWG.MLINESTYLE_FLAG_START_ROUND
MLINESTYLE_FLAG_UNKNOWN_128 = _LibreDWG.MLINESTYLE_FLAG_UNKNOWN_128
MLINESTYLE_FLAG_END_SQUARE = _LibreDWG.MLINESTYLE_FLAG_END_SQUARE
MLINESTYLE_FLAG_END_INNER_ARC = _LibreDWG.MLINESTYLE_FLAG_END_INNER_ARC
MLINESTYLE_FLAG_END_ROUND = _LibreDWG.MLINESTYLE_FLAG_END_ROUND
LIGHT_TYPE_DISTANT = _LibreDWG.LIGHT_TYPE_DISTANT
LIGHT_TYPE_POINT = _LibreDWG.LIGHT_TYPE_POINT
LIGHT_TYPE_SPOT = _LibreDWG.LIGHT_TYPE_SPOT
LIGHT_ATTENUATION_TYPE_NONE = _LibreDWG.LIGHT_ATTENUATION_TYPE_NONE
LIGHT_ATTENUATION_TYPE_INV_LINEAR = _LibreDWG.LIGHT_ATTENUATION_TYPE_INV_LINEAR
LIGHT_ATTENUATION_TYPE_INV_SQUARE = _LibreDWG.LIGHT_ATTENUATION_TYPE_INV_SQUARE
LIGHT_EXTLIGHT_SHAPE_LINEAR = _LibreDWG.LIGHT_EXTLIGHT_SHAPE_LINEAR
LIGHT_EXTLIGHT_SHAPE_RECT = _LibreDWG.LIGHT_EXTLIGHT_SHAPE_RECT
LIGHT_EXTLIGHT_SHAPE_DISK = _LibreDWG.LIGHT_EXTLIGHT_SHAPE_DISK
LIGHT_EXTLIGHT_SHAPE_CYLINDER = _LibreDWG.LIGHT_EXTLIGHT_SHAPE_CYLINDER
LIGHT_EXTLIGHT_SHAPE_SPHERE = _LibreDWG.LIGHT_EXTLIGHT_SHAPE_SPHERE
_3DFACE_INVISIBLE_EDGE1 = _LibreDWG._3DFACE_INVISIBLE_EDGE1
_3DFACE_INVISIBLE_EDGE2 = _LibreDWG._3DFACE_INVISIBLE_EDGE2
_3DFACE_INVISIBLE_EDGE3 = _LibreDWG._3DFACE_INVISIBLE_EDGE3
_3DFACE_INVISIBLE_EDGE4 = _LibreDWG._3DFACE_INVISIBLE_EDGE4
class Dwg_Object_Entity(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    objid = property(_LibreDWG.Dwg_Object_Entity_objid_get, _LibreDWG.Dwg_Object_Entity_objid_set)
    tio = property(_LibreDWG.Dwg_Object_Entity_tio_get)
    dwg = property(_LibreDWG.Dwg_Object_Entity_dwg_get, _LibreDWG.Dwg_Object_Entity_dwg_set)
    num_eed = property(_LibreDWG.Dwg_Object_Entity_num_eed_get, _LibreDWG.Dwg_Object_Entity_num_eed_set)
    eed = property(_LibreDWG.Dwg_Object_Entity_eed_get, _LibreDWG.Dwg_Object_Entity_eed_set)
    preview_exists = property(_LibreDWG.Dwg_Object_Entity_preview_exists_get, _LibreDWG.Dwg_Object_Entity_preview_exists_set)
    preview_is_proxy = property(_LibreDWG.Dwg_Object_Entity_preview_is_proxy_get, _LibreDWG.Dwg_Object_Entity_preview_is_proxy_set)
    preview_size = property(_LibreDWG.Dwg_Object_Entity_preview_size_get, _LibreDWG.Dwg_Object_Entity_preview_size_set)
    preview = property(_LibreDWG.Dwg_Object_Entity_preview_get, _LibreDWG.Dwg_Object_Entity_preview_set)
    entmode = property(_LibreDWG.Dwg_Object_Entity_entmode_get, _LibreDWG.Dwg_Object_Entity_entmode_set)
    num_reactors = property(_LibreDWG.Dwg_Object_Entity_num_reactors_get, _LibreDWG.Dwg_Object_Entity_num_reactors_set)
    is_xdic_missing = property(_LibreDWG.Dwg_Object_Entity_is_xdic_missing_get, _LibreDWG.Dwg_Object_Entity_is_xdic_missing_set)
    isbylayerlt = property(_LibreDWG.Dwg_Object_Entity_isbylayerlt_get, _LibreDWG.Dwg_Object_Entity_isbylayerlt_set)
    nolinks = property(_LibreDWG.Dwg_Object_Entity_nolinks_get, _LibreDWG.Dwg_Object_Entity_nolinks_set)
    has_ds_data = property(_LibreDWG.Dwg_Object_Entity_has_ds_data_get, _LibreDWG.Dwg_Object_Entity_has_ds_data_set)
    color = property(_LibreDWG.Dwg_Object_Entity_color_get, _LibreDWG.Dwg_Object_Entity_color_set)
    ltype_scale = property(_LibreDWG.Dwg_Object_Entity_ltype_scale_get, _LibreDWG.Dwg_Object_Entity_ltype_scale_set)
    ltype_flags = property(_LibreDWG.Dwg_Object_Entity_ltype_flags_get, _LibreDWG.Dwg_Object_Entity_ltype_flags_set)
    plotstyle_flags = property(_LibreDWG.Dwg_Object_Entity_plotstyle_flags_get, _LibreDWG.Dwg_Object_Entity_plotstyle_flags_set)
    material_flags = property(_LibreDWG.Dwg_Object_Entity_material_flags_get, _LibreDWG.Dwg_Object_Entity_material_flags_set)
    shadow_flags = property(_LibreDWG.Dwg_Object_Entity_shadow_flags_get, _LibreDWG.Dwg_Object_Entity_shadow_flags_set)
    has_full_visualstyle = property(_LibreDWG.Dwg_Object_Entity_has_full_visualstyle_get, _LibreDWG.Dwg_Object_Entity_has_full_visualstyle_set)
    has_face_visualstyle = property(_LibreDWG.Dwg_Object_Entity_has_face_visualstyle_get, _LibreDWG.Dwg_Object_Entity_has_face_visualstyle_set)
    has_edge_visualstyle = property(_LibreDWG.Dwg_Object_Entity_has_edge_visualstyle_get, _LibreDWG.Dwg_Object_Entity_has_edge_visualstyle_set)
    invisible = property(_LibreDWG.Dwg_Object_Entity_invisible_get, _LibreDWG.Dwg_Object_Entity_invisible_set)
    linewt = property(_LibreDWG.Dwg_Object_Entity_linewt_get, _LibreDWG.Dwg_Object_Entity_linewt_set)
    flag_r11 = property(_LibreDWG.Dwg_Object_Entity_flag_r11_get, _LibreDWG.Dwg_Object_Entity_flag_r11_set)
    opts_r11 = property(_LibreDWG.Dwg_Object_Entity_opts_r11_get, _LibreDWG.Dwg_Object_Entity_opts_r11_set)
    extra_r11 = property(_LibreDWG.Dwg_Object_Entity_extra_r11_get, _LibreDWG.Dwg_Object_Entity_extra_r11_set)
    color_r11 = property(_LibreDWG.Dwg_Object_Entity_color_r11_get, _LibreDWG.Dwg_Object_Entity_color_r11_set)
    elevation_r11 = property(_LibreDWG.Dwg_Object_Entity_elevation_r11_get, _LibreDWG.Dwg_Object_Entity_elevation_r11_set)
    thickness_r11 = property(_LibreDWG.Dwg_Object_Entity_thickness_r11_get, _LibreDWG.Dwg_Object_Entity_thickness_r11_set)
    viewport = property(_LibreDWG.Dwg_Object_Entity_viewport_get, _LibreDWG.Dwg_Object_Entity_viewport_set)
    __iterator = property(_LibreDWG.Dwg_Object_Entity___iterator_get, _LibreDWG.Dwg_Object_Entity___iterator_set)
    ownerhandle = property(_LibreDWG.Dwg_Object_Entity_ownerhandle_get, _LibreDWG.Dwg_Object_Entity_ownerhandle_set)
    reactors = property(_LibreDWG.Dwg_Object_Entity_reactors_get, _LibreDWG.Dwg_Object_Entity_reactors_set)
    xdicobjhandle = property(_LibreDWG.Dwg_Object_Entity_xdicobjhandle_get, _LibreDWG.Dwg_Object_Entity_xdicobjhandle_set)
    prev_entity = property(_LibreDWG.Dwg_Object_Entity_prev_entity_get, _LibreDWG.Dwg_Object_Entity_prev_entity_set)
    next_entity = property(_LibreDWG.Dwg_Object_Entity_next_entity_get, _LibreDWG.Dwg_Object_Entity_next_entity_set)
    layer = property(_LibreDWG.Dwg_Object_Entity_layer_get, _LibreDWG.Dwg_Object_Entity_layer_set)
    ltype = property(_LibreDWG.Dwg_Object_Entity_ltype_get, _LibreDWG.Dwg_Object_Entity_ltype_set)
    material = property(_LibreDWG.Dwg_Object_Entity_material_get, _LibreDWG.Dwg_Object_Entity_material_set)
    shadow = property(_LibreDWG.Dwg_Object_Entity_shadow_get, _LibreDWG.Dwg_Object_Entity_shadow_set)
    plotstyle = property(_LibreDWG.Dwg_Object_Entity_plotstyle_get, _LibreDWG.Dwg_Object_Entity_plotstyle_set)
    full_visualstyle = property(_LibreDWG.Dwg_Object_Entity_full_visualstyle_get, _LibreDWG.Dwg_Object_Entity_full_visualstyle_set)
    face_visualstyle = property(_LibreDWG.Dwg_Object_Entity_face_visualstyle_get, _LibreDWG.Dwg_Object_Entity_face_visualstyle_set)
    edge_visualstyle = property(_LibreDWG.Dwg_Object_Entity_edge_visualstyle_get, _LibreDWG.Dwg_Object_Entity_edge_visualstyle_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_Entity_swiginit(self, _LibreDWG.new_Dwg_Object_Entity())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_Entity

# Register Dwg_Object_Entity in _LibreDWG:
_LibreDWG.Dwg_Object_Entity_swigregister(Dwg_Object_Entity)
class _dwg_object_entity_tio(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    UNUSED = property(_LibreDWG._dwg_object_entity_tio_UNUSED_get, _LibreDWG._dwg_object_entity_tio_UNUSED_set)
    DIMENSION_common = property(_LibreDWG._dwg_object_entity_tio_DIMENSION_common_get, _LibreDWG._dwg_object_entity_tio_DIMENSION_common_set)
    _3DFACE = property(_LibreDWG._dwg_object_entity_tio__3DFACE_get, _LibreDWG._dwg_object_entity_tio__3DFACE_set)
    _3DSOLID = property(_LibreDWG._dwg_object_entity_tio__3DSOLID_get, _LibreDWG._dwg_object_entity_tio__3DSOLID_set)
    ARC = property(_LibreDWG._dwg_object_entity_tio_ARC_get, _LibreDWG._dwg_object_entity_tio_ARC_set)
    ATTDEF = property(_LibreDWG._dwg_object_entity_tio_ATTDEF_get, _LibreDWG._dwg_object_entity_tio_ATTDEF_set)
    ATTRIB = property(_LibreDWG._dwg_object_entity_tio_ATTRIB_get, _LibreDWG._dwg_object_entity_tio_ATTRIB_set)
    BLOCK = property(_LibreDWG._dwg_object_entity_tio_BLOCK_get, _LibreDWG._dwg_object_entity_tio_BLOCK_set)
    BODY = property(_LibreDWG._dwg_object_entity_tio_BODY_get, _LibreDWG._dwg_object_entity_tio_BODY_set)
    CIRCLE = property(_LibreDWG._dwg_object_entity_tio_CIRCLE_get, _LibreDWG._dwg_object_entity_tio_CIRCLE_set)
    DIMENSION_ALIGNED = property(_LibreDWG._dwg_object_entity_tio_DIMENSION_ALIGNED_get, _LibreDWG._dwg_object_entity_tio_DIMENSION_ALIGNED_set)
    DIMENSION_ANG2LN = property(_LibreDWG._dwg_object_entity_tio_DIMENSION_ANG2LN_get, _LibreDWG._dwg_object_entity_tio_DIMENSION_ANG2LN_set)
    DIMENSION_ANG3PT = property(_LibreDWG._dwg_object_entity_tio_DIMENSION_ANG3PT_get, _LibreDWG._dwg_object_entity_tio_DIMENSION_ANG3PT_set)
    DIMENSION_DIAMETER = property(_LibreDWG._dwg_object_entity_tio_DIMENSION_DIAMETER_get, _LibreDWG._dwg_object_entity_tio_DIMENSION_DIAMETER_set)
    DIMENSION_LINEAR = property(_LibreDWG._dwg_object_entity_tio_DIMENSION_LINEAR_get, _LibreDWG._dwg_object_entity_tio_DIMENSION_LINEAR_set)
    DIMENSION_ORDINATE = property(_LibreDWG._dwg_object_entity_tio_DIMENSION_ORDINATE_get, _LibreDWG._dwg_object_entity_tio_DIMENSION_ORDINATE_set)
    DIMENSION_RADIUS = property(_LibreDWG._dwg_object_entity_tio_DIMENSION_RADIUS_get, _LibreDWG._dwg_object_entity_tio_DIMENSION_RADIUS_set)
    ELLIPSE = property(_LibreDWG._dwg_object_entity_tio_ELLIPSE_get, _LibreDWG._dwg_object_entity_tio_ELLIPSE_set)
    ENDBLK = property(_LibreDWG._dwg_object_entity_tio_ENDBLK_get, _LibreDWG._dwg_object_entity_tio_ENDBLK_set)
    INSERT = property(_LibreDWG._dwg_object_entity_tio_INSERT_get, _LibreDWG._dwg_object_entity_tio_INSERT_set)
    LEADER = property(_LibreDWG._dwg_object_entity_tio_LEADER_get, _LibreDWG._dwg_object_entity_tio_LEADER_set)
    LINE = property(_LibreDWG._dwg_object_entity_tio_LINE_get, _LibreDWG._dwg_object_entity_tio_LINE_set)
    LOAD = property(_LibreDWG._dwg_object_entity_tio_LOAD_get, _LibreDWG._dwg_object_entity_tio_LOAD_set)
    MINSERT = property(_LibreDWG._dwg_object_entity_tio_MINSERT_get, _LibreDWG._dwg_object_entity_tio_MINSERT_set)
    MLINE = property(_LibreDWG._dwg_object_entity_tio_MLINE_get, _LibreDWG._dwg_object_entity_tio_MLINE_set)
    MTEXT = property(_LibreDWG._dwg_object_entity_tio_MTEXT_get, _LibreDWG._dwg_object_entity_tio_MTEXT_set)
    OLEFRAME = property(_LibreDWG._dwg_object_entity_tio_OLEFRAME_get, _LibreDWG._dwg_object_entity_tio_OLEFRAME_set)
    POINT = property(_LibreDWG._dwg_object_entity_tio_POINT_get, _LibreDWG._dwg_object_entity_tio_POINT_set)
    POLYLINE_2D = property(_LibreDWG._dwg_object_entity_tio_POLYLINE_2D_get, _LibreDWG._dwg_object_entity_tio_POLYLINE_2D_set)
    POLYLINE_3D = property(_LibreDWG._dwg_object_entity_tio_POLYLINE_3D_get, _LibreDWG._dwg_object_entity_tio_POLYLINE_3D_set)
    POLYLINE_MESH = property(_LibreDWG._dwg_object_entity_tio_POLYLINE_MESH_get, _LibreDWG._dwg_object_entity_tio_POLYLINE_MESH_set)
    POLYLINE_PFACE = property(_LibreDWG._dwg_object_entity_tio_POLYLINE_PFACE_get, _LibreDWG._dwg_object_entity_tio_POLYLINE_PFACE_set)
    PROXY_ENTITY = property(_LibreDWG._dwg_object_entity_tio_PROXY_ENTITY_get, _LibreDWG._dwg_object_entity_tio_PROXY_ENTITY_set)
    RAY = property(_LibreDWG._dwg_object_entity_tio_RAY_get, _LibreDWG._dwg_object_entity_tio_RAY_set)
    REGION = property(_LibreDWG._dwg_object_entity_tio_REGION_get, _LibreDWG._dwg_object_entity_tio_REGION_set)
    SEQEND = property(_LibreDWG._dwg_object_entity_tio_SEQEND_get, _LibreDWG._dwg_object_entity_tio_SEQEND_set)
    SHAPE = property(_LibreDWG._dwg_object_entity_tio_SHAPE_get, _LibreDWG._dwg_object_entity_tio_SHAPE_set)
    SOLID = property(_LibreDWG._dwg_object_entity_tio_SOLID_get, _LibreDWG._dwg_object_entity_tio_SOLID_set)
    SPLINE = property(_LibreDWG._dwg_object_entity_tio_SPLINE_get, _LibreDWG._dwg_object_entity_tio_SPLINE_set)
    TEXT = property(_LibreDWG._dwg_object_entity_tio_TEXT_get, _LibreDWG._dwg_object_entity_tio_TEXT_set)
    TOLERANCE = property(_LibreDWG._dwg_object_entity_tio_TOLERANCE_get, _LibreDWG._dwg_object_entity_tio_TOLERANCE_set)
    TRACE = property(_LibreDWG._dwg_object_entity_tio_TRACE_get, _LibreDWG._dwg_object_entity_tio_TRACE_set)
    UNKNOWN_ENT = property(_LibreDWG._dwg_object_entity_tio_UNKNOWN_ENT_get, _LibreDWG._dwg_object_entity_tio_UNKNOWN_ENT_set)
    VERTEX_2D = property(_LibreDWG._dwg_object_entity_tio_VERTEX_2D_get, _LibreDWG._dwg_object_entity_tio_VERTEX_2D_set)
    VERTEX_3D = property(_LibreDWG._dwg_object_entity_tio_VERTEX_3D_get, _LibreDWG._dwg_object_entity_tio_VERTEX_3D_set)
    VERTEX_MESH = property(_LibreDWG._dwg_object_entity_tio_VERTEX_MESH_get, _LibreDWG._dwg_object_entity_tio_VERTEX_MESH_set)
    VERTEX_PFACE = property(_LibreDWG._dwg_object_entity_tio_VERTEX_PFACE_get, _LibreDWG._dwg_object_entity_tio_VERTEX_PFACE_set)
    VERTEX_PFACE_FACE = property(_LibreDWG._dwg_object_entity_tio_VERTEX_PFACE_FACE_get, _LibreDWG._dwg_object_entity_tio_VERTEX_PFACE_FACE_set)
    VIEWPORT = property(_LibreDWG._dwg_object_entity_tio_VIEWPORT_get, _LibreDWG._dwg_object_entity_tio_VIEWPORT_set)
    XLINE = property(_LibreDWG._dwg_object_entity_tio_XLINE_get, _LibreDWG._dwg_object_entity_tio_XLINE_set)
    _3DLINE = property(_LibreDWG._dwg_object_entity_tio__3DLINE_get, _LibreDWG._dwg_object_entity_tio__3DLINE_set)
    CAMERA = property(_LibreDWG._dwg_object_entity_tio_CAMERA_get, _LibreDWG._dwg_object_entity_tio_CAMERA_set)
    DGNUNDERLAY = property(_LibreDWG._dwg_object_entity_tio_DGNUNDERLAY_get, _LibreDWG._dwg_object_entity_tio_DGNUNDERLAY_set)
    DWFUNDERLAY = property(_LibreDWG._dwg_object_entity_tio_DWFUNDERLAY_get, _LibreDWG._dwg_object_entity_tio_DWFUNDERLAY_set)
    ENDREP = property(_LibreDWG._dwg_object_entity_tio_ENDREP_get, _LibreDWG._dwg_object_entity_tio_ENDREP_set)
    HATCH = property(_LibreDWG._dwg_object_entity_tio_HATCH_get, _LibreDWG._dwg_object_entity_tio_HATCH_set)
    IMAGE = property(_LibreDWG._dwg_object_entity_tio_IMAGE_get, _LibreDWG._dwg_object_entity_tio_IMAGE_set)
    JUMP = property(_LibreDWG._dwg_object_entity_tio_JUMP_get, _LibreDWG._dwg_object_entity_tio_JUMP_set)
    LIGHT = property(_LibreDWG._dwg_object_entity_tio_LIGHT_get, _LibreDWG._dwg_object_entity_tio_LIGHT_set)
    LWPOLYLINE = property(_LibreDWG._dwg_object_entity_tio_LWPOLYLINE_get, _LibreDWG._dwg_object_entity_tio_LWPOLYLINE_set)
    MESH = property(_LibreDWG._dwg_object_entity_tio_MESH_get, _LibreDWG._dwg_object_entity_tio_MESH_set)
    MULTILEADER = property(_LibreDWG._dwg_object_entity_tio_MULTILEADER_get, _LibreDWG._dwg_object_entity_tio_MULTILEADER_set)
    OLE2FRAME = property(_LibreDWG._dwg_object_entity_tio_OLE2FRAME_get, _LibreDWG._dwg_object_entity_tio_OLE2FRAME_set)
    PDFUNDERLAY = property(_LibreDWG._dwg_object_entity_tio_PDFUNDERLAY_get, _LibreDWG._dwg_object_entity_tio_PDFUNDERLAY_set)
    REPEAT = property(_LibreDWG._dwg_object_entity_tio_REPEAT_get, _LibreDWG._dwg_object_entity_tio_REPEAT_set)
    SECTIONOBJECT = property(_LibreDWG._dwg_object_entity_tio_SECTIONOBJECT_get, _LibreDWG._dwg_object_entity_tio_SECTIONOBJECT_set)
    WIPEOUT = property(_LibreDWG._dwg_object_entity_tio_WIPEOUT_get, _LibreDWG._dwg_object_entity_tio_WIPEOUT_set)
    ARC_DIMENSION = property(_LibreDWG._dwg_object_entity_tio_ARC_DIMENSION_get, _LibreDWG._dwg_object_entity_tio_ARC_DIMENSION_set)
    HELIX = property(_LibreDWG._dwg_object_entity_tio_HELIX_get, _LibreDWG._dwg_object_entity_tio_HELIX_set)
    LARGE_RADIAL_DIMENSION = property(_LibreDWG._dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_get, _LibreDWG._dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_set)
    LAYOUTPRINTCONFIG = property(_LibreDWG._dwg_object_entity_tio_LAYOUTPRINTCONFIG_get, _LibreDWG._dwg_object_entity_tio_LAYOUTPRINTCONFIG_set)
    PLANESURFACE = property(_LibreDWG._dwg_object_entity_tio_PLANESURFACE_get, _LibreDWG._dwg_object_entity_tio_PLANESURFACE_set)
    POINTCLOUD = property(_LibreDWG._dwg_object_entity_tio_POINTCLOUD_get, _LibreDWG._dwg_object_entity_tio_POINTCLOUD_set)
    POINTCLOUDEX = property(_LibreDWG._dwg_object_entity_tio_POINTCLOUDEX_get, _LibreDWG._dwg_object_entity_tio_POINTCLOUDEX_set)
    ALIGNMENTPARAMETERENTITY = property(_LibreDWG._dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_get, _LibreDWG._dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_set)
    ARCALIGNEDTEXT = property(_LibreDWG._dwg_object_entity_tio_ARCALIGNEDTEXT_get, _LibreDWG._dwg_object_entity_tio_ARCALIGNEDTEXT_set)
    BASEPOINTPARAMETERENTITY = property(_LibreDWG._dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_get, _LibreDWG._dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_set)
    EXTRUDEDSURFACE = property(_LibreDWG._dwg_object_entity_tio_EXTRUDEDSURFACE_get, _LibreDWG._dwg_object_entity_tio_EXTRUDEDSURFACE_set)
    FLIPGRIPENTITY = property(_LibreDWG._dwg_object_entity_tio_FLIPGRIPENTITY_get, _LibreDWG._dwg_object_entity_tio_FLIPGRIPENTITY_set)
    FLIPPARAMETERENTITY = property(_LibreDWG._dwg_object_entity_tio_FLIPPARAMETERENTITY_get, _LibreDWG._dwg_object_entity_tio_FLIPPARAMETERENTITY_set)
    GEOPOSITIONMARKER = property(_LibreDWG._dwg_object_entity_tio_GEOPOSITIONMARKER_get, _LibreDWG._dwg_object_entity_tio_GEOPOSITIONMARKER_set)
    LINEARGRIPENTITY = property(_LibreDWG._dwg_object_entity_tio_LINEARGRIPENTITY_get, _LibreDWG._dwg_object_entity_tio_LINEARGRIPENTITY_set)
    LINEARPARAMETERENTITY = property(_LibreDWG._dwg_object_entity_tio_LINEARPARAMETERENTITY_get, _LibreDWG._dwg_object_entity_tio_LINEARPARAMETERENTITY_set)
    LOFTEDSURFACE = property(_LibreDWG._dwg_object_entity_tio_LOFTEDSURFACE_get, _LibreDWG._dwg_object_entity_tio_LOFTEDSURFACE_set)
    MPOLYGON = property(_LibreDWG._dwg_object_entity_tio_MPOLYGON_get, _LibreDWG._dwg_object_entity_tio_MPOLYGON_set)
    NAVISWORKSMODEL = property(_LibreDWG._dwg_object_entity_tio_NAVISWORKSMODEL_get, _LibreDWG._dwg_object_entity_tio_NAVISWORKSMODEL_set)
    NURBSURFACE = property(_LibreDWG._dwg_object_entity_tio_NURBSURFACE_get, _LibreDWG._dwg_object_entity_tio_NURBSURFACE_set)
    POINTPARAMETERENTITY = property(_LibreDWG._dwg_object_entity_tio_POINTPARAMETERENTITY_get, _LibreDWG._dwg_object_entity_tio_POINTPARAMETERENTITY_set)
    POLARGRIPENTITY = property(_LibreDWG._dwg_object_entity_tio_POLARGRIPENTITY_get, _LibreDWG._dwg_object_entity_tio_POLARGRIPENTITY_set)
    REVOLVEDSURFACE = property(_LibreDWG._dwg_object_entity_tio_REVOLVEDSURFACE_get, _LibreDWG._dwg_object_entity_tio_REVOLVEDSURFACE_set)
    ROTATIONGRIPENTITY = property(_LibreDWG._dwg_object_entity_tio_ROTATIONGRIPENTITY_get, _LibreDWG._dwg_object_entity_tio_ROTATIONGRIPENTITY_set)
    ROTATIONPARAMETERENTITY = property(_LibreDWG._dwg_object_entity_tio_ROTATIONPARAMETERENTITY_get, _LibreDWG._dwg_object_entity_tio_ROTATIONPARAMETERENTITY_set)
    RTEXT = property(_LibreDWG._dwg_object_entity_tio_RTEXT_get, _LibreDWG._dwg_object_entity_tio_RTEXT_set)
    SWEPTSURFACE = property(_LibreDWG._dwg_object_entity_tio_SWEPTSURFACE_get, _LibreDWG._dwg_object_entity_tio_SWEPTSURFACE_set)
    TABLE = property(_LibreDWG._dwg_object_entity_tio_TABLE_get, _LibreDWG._dwg_object_entity_tio_TABLE_set)
    VISIBILITYGRIPENTITY = property(_LibreDWG._dwg_object_entity_tio_VISIBILITYGRIPENTITY_get, _LibreDWG._dwg_object_entity_tio_VISIBILITYGRIPENTITY_set)
    VISIBILITYPARAMETERENTITY = property(_LibreDWG._dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_get, _LibreDWG._dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_set)
    XYGRIPENTITY = property(_LibreDWG._dwg_object_entity_tio_XYGRIPENTITY_get, _LibreDWG._dwg_object_entity_tio_XYGRIPENTITY_set)
    XYPARAMETERENTITY = property(_LibreDWG._dwg_object_entity_tio_XYPARAMETERENTITY_get, _LibreDWG._dwg_object_entity_tio_XYPARAMETERENTITY_set)

    def __init__(self):
        _LibreDWG._dwg_object_entity_tio_swiginit(self, _LibreDWG.new__dwg_object_entity_tio())
    __swig_destroy__ = _LibreDWG.delete__dwg_object_entity_tio

# Register _dwg_object_entity_tio in _LibreDWG:
_LibreDWG._dwg_object_entity_tio_swigregister(_dwg_object_entity_tio)
class Dwg_Object_Object(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    objid = property(_LibreDWG.Dwg_Object_Object_objid_get, _LibreDWG.Dwg_Object_Object_objid_set)
    tio = property(_LibreDWG.Dwg_Object_Object_tio_get)
    dwg = property(_LibreDWG.Dwg_Object_Object_dwg_get, _LibreDWG.Dwg_Object_Object_dwg_set)
    num_eed = property(_LibreDWG.Dwg_Object_Object_num_eed_get, _LibreDWG.Dwg_Object_Object_num_eed_set)
    eed = property(_LibreDWG.Dwg_Object_Object_eed_get, _LibreDWG.Dwg_Object_Object_eed_set)
    ownerhandle = property(_LibreDWG.Dwg_Object_Object_ownerhandle_get, _LibreDWG.Dwg_Object_Object_ownerhandle_set)
    num_reactors = property(_LibreDWG.Dwg_Object_Object_num_reactors_get, _LibreDWG.Dwg_Object_Object_num_reactors_set)
    reactors = property(_LibreDWG.Dwg_Object_Object_reactors_get, _LibreDWG.Dwg_Object_Object_reactors_set)
    xdicobjhandle = property(_LibreDWG.Dwg_Object_Object_xdicobjhandle_get, _LibreDWG.Dwg_Object_Object_xdicobjhandle_set)
    is_xdic_missing = property(_LibreDWG.Dwg_Object_Object_is_xdic_missing_get, _LibreDWG.Dwg_Object_Object_is_xdic_missing_set)
    has_ds_data = property(_LibreDWG.Dwg_Object_Object_has_ds_data_get, _LibreDWG.Dwg_Object_Object_has_ds_data_set)
    handleref = property(_LibreDWG.Dwg_Object_Object_handleref_get, _LibreDWG.Dwg_Object_Object_handleref_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_Object_swiginit(self, _LibreDWG.new_Dwg_Object_Object())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object_Object

# Register Dwg_Object_Object in _LibreDWG:
_LibreDWG.Dwg_Object_Object_swigregister(Dwg_Object_Object)
class _dwg_object_object_tio(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    APPID = property(_LibreDWG._dwg_object_object_tio_APPID_get, _LibreDWG._dwg_object_object_tio_APPID_set)
    APPID_CONTROL = property(_LibreDWG._dwg_object_object_tio_APPID_CONTROL_get, _LibreDWG._dwg_object_object_tio_APPID_CONTROL_set)
    BLOCK_CONTROL = property(_LibreDWG._dwg_object_object_tio_BLOCK_CONTROL_get, _LibreDWG._dwg_object_object_tio_BLOCK_CONTROL_set)
    BLOCK_HEADER = property(_LibreDWG._dwg_object_object_tio_BLOCK_HEADER_get, _LibreDWG._dwg_object_object_tio_BLOCK_HEADER_set)
    DICTIONARY = property(_LibreDWG._dwg_object_object_tio_DICTIONARY_get, _LibreDWG._dwg_object_object_tio_DICTIONARY_set)
    DIMSTYLE = property(_LibreDWG._dwg_object_object_tio_DIMSTYLE_get, _LibreDWG._dwg_object_object_tio_DIMSTYLE_set)
    DIMSTYLE_CONTROL = property(_LibreDWG._dwg_object_object_tio_DIMSTYLE_CONTROL_get, _LibreDWG._dwg_object_object_tio_DIMSTYLE_CONTROL_set)
    DUMMY = property(_LibreDWG._dwg_object_object_tio_DUMMY_get, _LibreDWG._dwg_object_object_tio_DUMMY_set)
    LAYER = property(_LibreDWG._dwg_object_object_tio_LAYER_get, _LibreDWG._dwg_object_object_tio_LAYER_set)
    LAYER_CONTROL = property(_LibreDWG._dwg_object_object_tio_LAYER_CONTROL_get, _LibreDWG._dwg_object_object_tio_LAYER_CONTROL_set)
    LONG_TRANSACTION = property(_LibreDWG._dwg_object_object_tio_LONG_TRANSACTION_get, _LibreDWG._dwg_object_object_tio_LONG_TRANSACTION_set)
    LTYPE = property(_LibreDWG._dwg_object_object_tio_LTYPE_get, _LibreDWG._dwg_object_object_tio_LTYPE_set)
    LTYPE_CONTROL = property(_LibreDWG._dwg_object_object_tio_LTYPE_CONTROL_get, _LibreDWG._dwg_object_object_tio_LTYPE_CONTROL_set)
    MLINESTYLE = property(_LibreDWG._dwg_object_object_tio_MLINESTYLE_get, _LibreDWG._dwg_object_object_tio_MLINESTYLE_set)
    STYLE = property(_LibreDWG._dwg_object_object_tio_STYLE_get, _LibreDWG._dwg_object_object_tio_STYLE_set)
    STYLE_CONTROL = property(_LibreDWG._dwg_object_object_tio_STYLE_CONTROL_get, _LibreDWG._dwg_object_object_tio_STYLE_CONTROL_set)
    UCS = property(_LibreDWG._dwg_object_object_tio_UCS_get, _LibreDWG._dwg_object_object_tio_UCS_set)
    UCS_CONTROL = property(_LibreDWG._dwg_object_object_tio_UCS_CONTROL_get, _LibreDWG._dwg_object_object_tio_UCS_CONTROL_set)
    UNKNOWN_OBJ = property(_LibreDWG._dwg_object_object_tio_UNKNOWN_OBJ_get, _LibreDWG._dwg_object_object_tio_UNKNOWN_OBJ_set)
    VIEW = property(_LibreDWG._dwg_object_object_tio_VIEW_get, _LibreDWG._dwg_object_object_tio_VIEW_set)
    VIEW_CONTROL = property(_LibreDWG._dwg_object_object_tio_VIEW_CONTROL_get, _LibreDWG._dwg_object_object_tio_VIEW_CONTROL_set)
    VPORT = property(_LibreDWG._dwg_object_object_tio_VPORT_get, _LibreDWG._dwg_object_object_tio_VPORT_set)
    VPORT_CONTROL = property(_LibreDWG._dwg_object_object_tio_VPORT_CONTROL_get, _LibreDWG._dwg_object_object_tio_VPORT_CONTROL_set)
    VX_CONTROL = property(_LibreDWG._dwg_object_object_tio_VX_CONTROL_get, _LibreDWG._dwg_object_object_tio_VX_CONTROL_set)
    VX_TABLE_RECORD = property(_LibreDWG._dwg_object_object_tio_VX_TABLE_RECORD_get, _LibreDWG._dwg_object_object_tio_VX_TABLE_RECORD_set)
    ACSH_BOOLEAN_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_BOOLEAN_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_BOOLEAN_CLASS_set)
    ACSH_BOX_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_BOX_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_BOX_CLASS_set)
    ACSH_CONE_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_CONE_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_CONE_CLASS_set)
    ACSH_CYLINDER_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_CYLINDER_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_CYLINDER_CLASS_set)
    ACSH_FILLET_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_FILLET_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_FILLET_CLASS_set)
    ACSH_HISTORY_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_HISTORY_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_HISTORY_CLASS_set)
    ACSH_SPHERE_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_SPHERE_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_SPHERE_CLASS_set)
    ACSH_TORUS_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_TORUS_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_TORUS_CLASS_set)
    ACSH_WEDGE_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_WEDGE_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_WEDGE_CLASS_set)
    ASSOCGEOMDEPENDENCY = property(_LibreDWG._dwg_object_object_tio_ASSOCGEOMDEPENDENCY_get, _LibreDWG._dwg_object_object_tio_ASSOCGEOMDEPENDENCY_set)
    ASSOCNETWORK = property(_LibreDWG._dwg_object_object_tio_ASSOCNETWORK_get, _LibreDWG._dwg_object_object_tio_ASSOCNETWORK_set)
    BLOCKALIGNMENTGRIP = property(_LibreDWG._dwg_object_object_tio_BLOCKALIGNMENTGRIP_get, _LibreDWG._dwg_object_object_tio_BLOCKALIGNMENTGRIP_set)
    BLOCKALIGNMENTPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_set)
    BLOCKBASEPOINTPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_set)
    BLOCKFLIPACTION = property(_LibreDWG._dwg_object_object_tio_BLOCKFLIPACTION_get, _LibreDWG._dwg_object_object_tio_BLOCKFLIPACTION_set)
    BLOCKFLIPGRIP = property(_LibreDWG._dwg_object_object_tio_BLOCKFLIPGRIP_get, _LibreDWG._dwg_object_object_tio_BLOCKFLIPGRIP_set)
    BLOCKFLIPPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKFLIPPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKFLIPPARAMETER_set)
    BLOCKGRIPLOCATIONCOMPONENT = property(_LibreDWG._dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_get, _LibreDWG._dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_set)
    BLOCKLINEARGRIP = property(_LibreDWG._dwg_object_object_tio_BLOCKLINEARGRIP_get, _LibreDWG._dwg_object_object_tio_BLOCKLINEARGRIP_set)
    BLOCKLOOKUPGRIP = property(_LibreDWG._dwg_object_object_tio_BLOCKLOOKUPGRIP_get, _LibreDWG._dwg_object_object_tio_BLOCKLOOKUPGRIP_set)
    BLOCKMOVEACTION = property(_LibreDWG._dwg_object_object_tio_BLOCKMOVEACTION_get, _LibreDWG._dwg_object_object_tio_BLOCKMOVEACTION_set)
    BLOCKROTATEACTION = property(_LibreDWG._dwg_object_object_tio_BLOCKROTATEACTION_get, _LibreDWG._dwg_object_object_tio_BLOCKROTATEACTION_set)
    BLOCKROTATIONGRIP = property(_LibreDWG._dwg_object_object_tio_BLOCKROTATIONGRIP_get, _LibreDWG._dwg_object_object_tio_BLOCKROTATIONGRIP_set)
    BLOCKSCALEACTION = property(_LibreDWG._dwg_object_object_tio_BLOCKSCALEACTION_get, _LibreDWG._dwg_object_object_tio_BLOCKSCALEACTION_set)
    BLOCKVISIBILITYGRIP = property(_LibreDWG._dwg_object_object_tio_BLOCKVISIBILITYGRIP_get, _LibreDWG._dwg_object_object_tio_BLOCKVISIBILITYGRIP_set)
    CELLSTYLEMAP = property(_LibreDWG._dwg_object_object_tio_CELLSTYLEMAP_get, _LibreDWG._dwg_object_object_tio_CELLSTYLEMAP_set)
    DETAILVIEWSTYLE = property(_LibreDWG._dwg_object_object_tio_DETAILVIEWSTYLE_get, _LibreDWG._dwg_object_object_tio_DETAILVIEWSTYLE_set)
    DICTIONARYVAR = property(_LibreDWG._dwg_object_object_tio_DICTIONARYVAR_get, _LibreDWG._dwg_object_object_tio_DICTIONARYVAR_set)
    DICTIONARYWDFLT = property(_LibreDWG._dwg_object_object_tio_DICTIONARYWDFLT_get, _LibreDWG._dwg_object_object_tio_DICTIONARYWDFLT_set)
    DYNAMICBLOCKPURGEPREVENTER = property(_LibreDWG._dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_get, _LibreDWG._dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_set)
    FIELD = property(_LibreDWG._dwg_object_object_tio_FIELD_get, _LibreDWG._dwg_object_object_tio_FIELD_set)
    FIELDLIST = property(_LibreDWG._dwg_object_object_tio_FIELDLIST_get, _LibreDWG._dwg_object_object_tio_FIELDLIST_set)
    GEODATA = property(_LibreDWG._dwg_object_object_tio_GEODATA_get, _LibreDWG._dwg_object_object_tio_GEODATA_set)
    GROUP = property(_LibreDWG._dwg_object_object_tio_GROUP_get, _LibreDWG._dwg_object_object_tio_GROUP_set)
    IDBUFFER = property(_LibreDWG._dwg_object_object_tio_IDBUFFER_get, _LibreDWG._dwg_object_object_tio_IDBUFFER_set)
    IMAGEDEF = property(_LibreDWG._dwg_object_object_tio_IMAGEDEF_get, _LibreDWG._dwg_object_object_tio_IMAGEDEF_set)
    IMAGEDEF_REACTOR = property(_LibreDWG._dwg_object_object_tio_IMAGEDEF_REACTOR_get, _LibreDWG._dwg_object_object_tio_IMAGEDEF_REACTOR_set)
    INDEX = property(_LibreDWG._dwg_object_object_tio_INDEX_get, _LibreDWG._dwg_object_object_tio_INDEX_set)
    LAYERFILTER = property(_LibreDWG._dwg_object_object_tio_LAYERFILTER_get, _LibreDWG._dwg_object_object_tio_LAYERFILTER_set)
    LAYER_INDEX = property(_LibreDWG._dwg_object_object_tio_LAYER_INDEX_get, _LibreDWG._dwg_object_object_tio_LAYER_INDEX_set)
    LAYOUT = property(_LibreDWG._dwg_object_object_tio_LAYOUT_get, _LibreDWG._dwg_object_object_tio_LAYOUT_set)
    PLACEHOLDER = property(_LibreDWG._dwg_object_object_tio_PLACEHOLDER_get, _LibreDWG._dwg_object_object_tio_PLACEHOLDER_set)
    PLOTSETTINGS = property(_LibreDWG._dwg_object_object_tio_PLOTSETTINGS_get, _LibreDWG._dwg_object_object_tio_PLOTSETTINGS_set)
    RASTERVARIABLES = property(_LibreDWG._dwg_object_object_tio_RASTERVARIABLES_get, _LibreDWG._dwg_object_object_tio_RASTERVARIABLES_set)
    RENDERENVIRONMENT = property(_LibreDWG._dwg_object_object_tio_RENDERENVIRONMENT_get, _LibreDWG._dwg_object_object_tio_RENDERENVIRONMENT_set)
    SCALE = property(_LibreDWG._dwg_object_object_tio_SCALE_get, _LibreDWG._dwg_object_object_tio_SCALE_set)
    SECTIONVIEWSTYLE = property(_LibreDWG._dwg_object_object_tio_SECTIONVIEWSTYLE_get, _LibreDWG._dwg_object_object_tio_SECTIONVIEWSTYLE_set)
    SECTION_MANAGER = property(_LibreDWG._dwg_object_object_tio_SECTION_MANAGER_get, _LibreDWG._dwg_object_object_tio_SECTION_MANAGER_set)
    SORTENTSTABLE = property(_LibreDWG._dwg_object_object_tio_SORTENTSTABLE_get, _LibreDWG._dwg_object_object_tio_SORTENTSTABLE_set)
    SPATIAL_FILTER = property(_LibreDWG._dwg_object_object_tio_SPATIAL_FILTER_get, _LibreDWG._dwg_object_object_tio_SPATIAL_FILTER_set)
    SUN = property(_LibreDWG._dwg_object_object_tio_SUN_get, _LibreDWG._dwg_object_object_tio_SUN_set)
    TABLEGEOMETRY = property(_LibreDWG._dwg_object_object_tio_TABLEGEOMETRY_get, _LibreDWG._dwg_object_object_tio_TABLEGEOMETRY_set)
    VBA_PROJECT = property(_LibreDWG._dwg_object_object_tio_VBA_PROJECT_get, _LibreDWG._dwg_object_object_tio_VBA_PROJECT_set)
    VISUALSTYLE = property(_LibreDWG._dwg_object_object_tio_VISUALSTYLE_get, _LibreDWG._dwg_object_object_tio_VISUALSTYLE_set)
    WIPEOUTVARIABLES = property(_LibreDWG._dwg_object_object_tio_WIPEOUTVARIABLES_get, _LibreDWG._dwg_object_object_tio_WIPEOUTVARIABLES_set)
    XRECORD = property(_LibreDWG._dwg_object_object_tio_XRECORD_get, _LibreDWG._dwg_object_object_tio_XRECORD_set)
    PDFDEFINITION = property(_LibreDWG._dwg_object_object_tio_PDFDEFINITION_get, _LibreDWG._dwg_object_object_tio_PDFDEFINITION_set)
    DGNDEFINITION = property(_LibreDWG._dwg_object_object_tio_DGNDEFINITION_get, _LibreDWG._dwg_object_object_tio_DGNDEFINITION_set)
    DWFDEFINITION = property(_LibreDWG._dwg_object_object_tio_DWFDEFINITION_get, _LibreDWG._dwg_object_object_tio_DWFDEFINITION_set)
    ACSH_BREP_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_BREP_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_BREP_CLASS_set)
    ACSH_CHAMFER_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_CHAMFER_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_CHAMFER_CLASS_set)
    ACSH_PYRAMID_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_PYRAMID_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_PYRAMID_CLASS_set)
    ALDIMOBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_set)
    ASSOC2DCONSTRAINTGROUP = property(_LibreDWG._dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_get, _LibreDWG._dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_set)
    ASSOCACTION = property(_LibreDWG._dwg_object_object_tio_ASSOCACTION_get, _LibreDWG._dwg_object_object_tio_ASSOCACTION_set)
    ASSOCACTIONPARAM = property(_LibreDWG._dwg_object_object_tio_ASSOCACTIONPARAM_get, _LibreDWG._dwg_object_object_tio_ASSOCACTIONPARAM_set)
    ASSOCARRAYACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCARRAYACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCARRAYACTIONBODY_set)
    ASSOCASMBODYACTIONPARAM = property(_LibreDWG._dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_get, _LibreDWG._dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_set)
    ASSOCBLENDSURFACEACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_set)
    ASSOCCOMPOUNDACTIONPARAM = property(_LibreDWG._dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_get, _LibreDWG._dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_set)
    ASSOCDEPENDENCY = property(_LibreDWG._dwg_object_object_tio_ASSOCDEPENDENCY_get, _LibreDWG._dwg_object_object_tio_ASSOCDEPENDENCY_set)
    ASSOCDIMDEPENDENCYBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_set)
    ASSOCEXTENDSURFACEACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_set)
    ASSOCEXTRUDEDSURFACEACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_set)
    ASSOCFACEACTIONPARAM = property(_LibreDWG._dwg_object_object_tio_ASSOCFACEACTIONPARAM_get, _LibreDWG._dwg_object_object_tio_ASSOCFACEACTIONPARAM_set)
    ASSOCFILLETSURFACEACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_set)
    ASSOCLOFTEDSURFACEACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_set)
    ASSOCNETWORKSURFACEACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_set)
    ASSOCOBJECTACTIONPARAM = property(_LibreDWG._dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_get, _LibreDWG._dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_set)
    ASSOCOFFSETSURFACEACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_set)
    ASSOCOSNAPPOINTREFACTIONPARAM = property(_LibreDWG._dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_get, _LibreDWG._dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_set)
    ASSOCPATCHSURFACEACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_set)
    ASSOCPATHACTIONPARAM = property(_LibreDWG._dwg_object_object_tio_ASSOCPATHACTIONPARAM_get, _LibreDWG._dwg_object_object_tio_ASSOCPATHACTIONPARAM_set)
    ASSOCPLANESURFACEACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_set)
    ASSOCPOINTREFACTIONPARAM = property(_LibreDWG._dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_get, _LibreDWG._dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_set)
    ASSOCREVOLVEDSURFACEACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_set)
    ASSOCTRIMSURFACEACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_set)
    ASSOCVALUEDEPENDENCY = property(_LibreDWG._dwg_object_object_tio_ASSOCVALUEDEPENDENCY_get, _LibreDWG._dwg_object_object_tio_ASSOCVALUEDEPENDENCY_set)
    ASSOCVARIABLE = property(_LibreDWG._dwg_object_object_tio_ASSOCVARIABLE_get, _LibreDWG._dwg_object_object_tio_ASSOCVARIABLE_set)
    ASSOCVERTEXACTIONPARAM = property(_LibreDWG._dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_get, _LibreDWG._dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_set)
    BLKREFOBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_set)
    BLOCKALIGNEDCONSTRAINTPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_set)
    BLOCKANGULARCONSTRAINTPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_set)
    BLOCKARRAYACTION = property(_LibreDWG._dwg_object_object_tio_BLOCKARRAYACTION_get, _LibreDWG._dwg_object_object_tio_BLOCKARRAYACTION_set)
    BLOCKDIAMETRICCONSTRAINTPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_set)
    BLOCKHORIZONTALCONSTRAINTPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_set)
    BLOCKLINEARCONSTRAINTPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_set)
    BLOCKLINEARPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKLINEARPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKLINEARPARAMETER_set)
    BLOCKLOOKUPACTION = property(_LibreDWG._dwg_object_object_tio_BLOCKLOOKUPACTION_get, _LibreDWG._dwg_object_object_tio_BLOCKLOOKUPACTION_set)
    BLOCKLOOKUPPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKLOOKUPPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKLOOKUPPARAMETER_set)
    BLOCKPARAMDEPENDENCYBODY = property(_LibreDWG._dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_get, _LibreDWG._dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_set)
    BLOCKPOINTPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKPOINTPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKPOINTPARAMETER_set)
    BLOCKPOLARGRIP = property(_LibreDWG._dwg_object_object_tio_BLOCKPOLARGRIP_get, _LibreDWG._dwg_object_object_tio_BLOCKPOLARGRIP_set)
    BLOCKPOLARPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKPOLARPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKPOLARPARAMETER_set)
    BLOCKPOLARSTRETCHACTION = property(_LibreDWG._dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_get, _LibreDWG._dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_set)
    BLOCKRADIALCONSTRAINTPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_set)
    BLOCKREPRESENTATION = property(_LibreDWG._dwg_object_object_tio_BLOCKREPRESENTATION_get, _LibreDWG._dwg_object_object_tio_BLOCKREPRESENTATION_set)
    BLOCKROTATIONPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKROTATIONPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKROTATIONPARAMETER_set)
    BLOCKSTRETCHACTION = property(_LibreDWG._dwg_object_object_tio_BLOCKSTRETCHACTION_get, _LibreDWG._dwg_object_object_tio_BLOCKSTRETCHACTION_set)
    BLOCKUSERPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKUSERPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKUSERPARAMETER_set)
    BLOCKVERTICALCONSTRAINTPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_set)
    BLOCKVISIBILITYPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_set)
    BLOCKXYGRIP = property(_LibreDWG._dwg_object_object_tio_BLOCKXYGRIP_get, _LibreDWG._dwg_object_object_tio_BLOCKXYGRIP_set)
    BLOCKXYPARAMETER = property(_LibreDWG._dwg_object_object_tio_BLOCKXYPARAMETER_get, _LibreDWG._dwg_object_object_tio_BLOCKXYPARAMETER_set)
    DATALINK = property(_LibreDWG._dwg_object_object_tio_DATALINK_get, _LibreDWG._dwg_object_object_tio_DATALINK_set)
    DBCOLOR = property(_LibreDWG._dwg_object_object_tio_DBCOLOR_get, _LibreDWG._dwg_object_object_tio_DBCOLOR_set)
    DIMASSOC = property(_LibreDWG._dwg_object_object_tio_DIMASSOC_get, _LibreDWG._dwg_object_object_tio_DIMASSOC_set)
    EVALUATION_GRAPH = property(_LibreDWG._dwg_object_object_tio_EVALUATION_GRAPH_get, _LibreDWG._dwg_object_object_tio_EVALUATION_GRAPH_set)
    FCFOBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_FCFOBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_FCFOBJECTCONTEXTDATA_set)
    GRADIENT_BACKGROUND = property(_LibreDWG._dwg_object_object_tio_GRADIENT_BACKGROUND_get, _LibreDWG._dwg_object_object_tio_GRADIENT_BACKGROUND_set)
    GROUND_PLANE_BACKGROUND = property(_LibreDWG._dwg_object_object_tio_GROUND_PLANE_BACKGROUND_get, _LibreDWG._dwg_object_object_tio_GROUND_PLANE_BACKGROUND_set)
    IBL_BACKGROUND = property(_LibreDWG._dwg_object_object_tio_IBL_BACKGROUND_get, _LibreDWG._dwg_object_object_tio_IBL_BACKGROUND_set)
    IMAGE_BACKGROUND = property(_LibreDWG._dwg_object_object_tio_IMAGE_BACKGROUND_get, _LibreDWG._dwg_object_object_tio_IMAGE_BACKGROUND_set)
    LEADEROBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_set)
    LIGHTLIST = property(_LibreDWG._dwg_object_object_tio_LIGHTLIST_get, _LibreDWG._dwg_object_object_tio_LIGHTLIST_set)
    MATERIAL = property(_LibreDWG._dwg_object_object_tio_MATERIAL_get, _LibreDWG._dwg_object_object_tio_MATERIAL_set)
    MENTALRAYRENDERSETTINGS = property(_LibreDWG._dwg_object_object_tio_MENTALRAYRENDERSETTINGS_get, _LibreDWG._dwg_object_object_tio_MENTALRAYRENDERSETTINGS_set)
    MLEADERSTYLE = property(_LibreDWG._dwg_object_object_tio_MLEADERSTYLE_get, _LibreDWG._dwg_object_object_tio_MLEADERSTYLE_set)
    MTEXTOBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_set)
    OBJECT_PTR = property(_LibreDWG._dwg_object_object_tio_OBJECT_PTR_get, _LibreDWG._dwg_object_object_tio_OBJECT_PTR_set)
    PARTIAL_VIEWING_INDEX = property(_LibreDWG._dwg_object_object_tio_PARTIAL_VIEWING_INDEX_get, _LibreDWG._dwg_object_object_tio_PARTIAL_VIEWING_INDEX_set)
    POINTCLOUDCOLORMAP = property(_LibreDWG._dwg_object_object_tio_POINTCLOUDCOLORMAP_get, _LibreDWG._dwg_object_object_tio_POINTCLOUDCOLORMAP_set)
    POINTCLOUDDEF = property(_LibreDWG._dwg_object_object_tio_POINTCLOUDDEF_get, _LibreDWG._dwg_object_object_tio_POINTCLOUDDEF_set)
    POINTCLOUDDEFEX = property(_LibreDWG._dwg_object_object_tio_POINTCLOUDDEFEX_get, _LibreDWG._dwg_object_object_tio_POINTCLOUDDEFEX_set)
    POINTCLOUDDEF_REACTOR = property(_LibreDWG._dwg_object_object_tio_POINTCLOUDDEF_REACTOR_get, _LibreDWG._dwg_object_object_tio_POINTCLOUDDEF_REACTOR_set)
    POINTCLOUDDEF_REACTOR_EX = property(_LibreDWG._dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_get, _LibreDWG._dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_set)
    PROXY_OBJECT = property(_LibreDWG._dwg_object_object_tio_PROXY_OBJECT_get, _LibreDWG._dwg_object_object_tio_PROXY_OBJECT_set)
    RAPIDRTRENDERSETTINGS = property(_LibreDWG._dwg_object_object_tio_RAPIDRTRENDERSETTINGS_get, _LibreDWG._dwg_object_object_tio_RAPIDRTRENDERSETTINGS_set)
    RENDERENTRY = property(_LibreDWG._dwg_object_object_tio_RENDERENTRY_get, _LibreDWG._dwg_object_object_tio_RENDERENTRY_set)
    RENDERGLOBAL = property(_LibreDWG._dwg_object_object_tio_RENDERGLOBAL_get, _LibreDWG._dwg_object_object_tio_RENDERGLOBAL_set)
    RENDERSETTINGS = property(_LibreDWG._dwg_object_object_tio_RENDERSETTINGS_get, _LibreDWG._dwg_object_object_tio_RENDERSETTINGS_set)
    SECTION_SETTINGS = property(_LibreDWG._dwg_object_object_tio_SECTION_SETTINGS_get, _LibreDWG._dwg_object_object_tio_SECTION_SETTINGS_set)
    SKYLIGHT_BACKGROUND = property(_LibreDWG._dwg_object_object_tio_SKYLIGHT_BACKGROUND_get, _LibreDWG._dwg_object_object_tio_SKYLIGHT_BACKGROUND_set)
    SOLID_BACKGROUND = property(_LibreDWG._dwg_object_object_tio_SOLID_BACKGROUND_get, _LibreDWG._dwg_object_object_tio_SOLID_BACKGROUND_set)
    SPATIAL_INDEX = property(_LibreDWG._dwg_object_object_tio_SPATIAL_INDEX_get, _LibreDWG._dwg_object_object_tio_SPATIAL_INDEX_set)
    TABLESTYLE = property(_LibreDWG._dwg_object_object_tio_TABLESTYLE_get, _LibreDWG._dwg_object_object_tio_TABLESTYLE_set)
    TEXTOBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_set)
    ASSOCARRAYMODIFYPARAMETERS = property(_LibreDWG._dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_get, _LibreDWG._dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_set)
    ASSOCARRAYPATHPARAMETERS = property(_LibreDWG._dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_get, _LibreDWG._dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_set)
    ASSOCARRAYPOLARPARAMETERS = property(_LibreDWG._dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_get, _LibreDWG._dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_set)
    ASSOCARRAYRECTANGULARPARAMETERS = property(_LibreDWG._dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_get, _LibreDWG._dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_set)
    ACMECOMMANDHISTORY = property(_LibreDWG._dwg_object_object_tio_ACMECOMMANDHISTORY_get, _LibreDWG._dwg_object_object_tio_ACMECOMMANDHISTORY_set)
    ACMESCOPE = property(_LibreDWG._dwg_object_object_tio_ACMESCOPE_get, _LibreDWG._dwg_object_object_tio_ACMESCOPE_set)
    ACMESTATEMGR = property(_LibreDWG._dwg_object_object_tio_ACMESTATEMGR_get, _LibreDWG._dwg_object_object_tio_ACMESTATEMGR_set)
    ACSH_EXTRUSION_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_EXTRUSION_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_EXTRUSION_CLASS_set)
    ACSH_LOFT_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_LOFT_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_LOFT_CLASS_set)
    ACSH_REVOLVE_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_REVOLVE_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_REVOLVE_CLASS_set)
    ACSH_SWEEP_CLASS = property(_LibreDWG._dwg_object_object_tio_ACSH_SWEEP_CLASS_get, _LibreDWG._dwg_object_object_tio_ACSH_SWEEP_CLASS_set)
    ANGDIMOBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_set)
    ANNOTSCALEOBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_set)
    ASSOC3POINTANGULARDIMACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_set)
    ASSOCALIGNEDDIMACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_set)
    ASSOCARRAYMODIFYACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_set)
    ASSOCEDGEACTIONPARAM = property(_LibreDWG._dwg_object_object_tio_ASSOCEDGEACTIONPARAM_get, _LibreDWG._dwg_object_object_tio_ASSOCEDGEACTIONPARAM_set)
    ASSOCEDGECHAMFERACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_set)
    ASSOCEDGEFILLETACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_set)
    ASSOCMLEADERACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCMLEADERACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCMLEADERACTIONBODY_set)
    ASSOCORDINATEDIMACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_set)
    ASSOCPERSSUBENTMANAGER = property(_LibreDWG._dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_get, _LibreDWG._dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_set)
    ASSOCRESTOREENTITYSTATEACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_set)
    ASSOCROTATEDDIMACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_set)
    ASSOCSWEPTSURFACEACTIONBODY = property(_LibreDWG._dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_get, _LibreDWG._dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_set)
    BLOCKPROPERTIESTABLE = property(_LibreDWG._dwg_object_object_tio_BLOCKPROPERTIESTABLE_get, _LibreDWG._dwg_object_object_tio_BLOCKPROPERTIESTABLE_set)
    BLOCKPROPERTIESTABLEGRIP = property(_LibreDWG._dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_get, _LibreDWG._dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_set)
    BREAKDATA = property(_LibreDWG._dwg_object_object_tio_BREAKDATA_get, _LibreDWG._dwg_object_object_tio_BREAKDATA_set)
    BREAKPOINTREF = property(_LibreDWG._dwg_object_object_tio_BREAKPOINTREF_get, _LibreDWG._dwg_object_object_tio_BREAKPOINTREF_set)
    CONTEXTDATAMANAGER = property(_LibreDWG._dwg_object_object_tio_CONTEXTDATAMANAGER_get, _LibreDWG._dwg_object_object_tio_CONTEXTDATAMANAGER_set)
    CSACDOCUMENTOPTIONS = property(_LibreDWG._dwg_object_object_tio_CSACDOCUMENTOPTIONS_get, _LibreDWG._dwg_object_object_tio_CSACDOCUMENTOPTIONS_set)
    CURVEPATH = property(_LibreDWG._dwg_object_object_tio_CURVEPATH_get, _LibreDWG._dwg_object_object_tio_CURVEPATH_set)
    DATATABLE = property(_LibreDWG._dwg_object_object_tio_DATATABLE_get, _LibreDWG._dwg_object_object_tio_DATATABLE_set)
    DMDIMOBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_set)
    DYNAMICBLOCKPROXYNODE = property(_LibreDWG._dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_get, _LibreDWG._dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_set)
    GEOMAPIMAGE = property(_LibreDWG._dwg_object_object_tio_GEOMAPIMAGE_get, _LibreDWG._dwg_object_object_tio_GEOMAPIMAGE_set)
    MLEADEROBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_set)
    MOTIONPATH = property(_LibreDWG._dwg_object_object_tio_MOTIONPATH_get, _LibreDWG._dwg_object_object_tio_MOTIONPATH_set)
    MTEXTATTRIBUTEOBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_set)
    NAVISWORKSMODELDEF = property(_LibreDWG._dwg_object_object_tio_NAVISWORKSMODELDEF_get, _LibreDWG._dwg_object_object_tio_NAVISWORKSMODELDEF_set)
    ORDDIMOBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_set)
    PERSUBENTMGR = property(_LibreDWG._dwg_object_object_tio_PERSUBENTMGR_get, _LibreDWG._dwg_object_object_tio_PERSUBENTMGR_set)
    POINTPATH = property(_LibreDWG._dwg_object_object_tio_POINTPATH_get, _LibreDWG._dwg_object_object_tio_POINTPATH_set)
    RADIMLGOBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_set)
    RADIMOBJECTCONTEXTDATA = property(_LibreDWG._dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_get, _LibreDWG._dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_set)
    SUNSTUDY = property(_LibreDWG._dwg_object_object_tio_SUNSTUDY_get, _LibreDWG._dwg_object_object_tio_SUNSTUDY_set)
    TABLECONTENT = property(_LibreDWG._dwg_object_object_tio_TABLECONTENT_get, _LibreDWG._dwg_object_object_tio_TABLECONTENT_set)
    TVDEVICEPROPERTIES = property(_LibreDWG._dwg_object_object_tio_TVDEVICEPROPERTIES_get, _LibreDWG._dwg_object_object_tio_TVDEVICEPROPERTIES_set)

    def __init__(self):
        _LibreDWG._dwg_object_object_tio_swiginit(self, _LibreDWG.new__dwg_object_object_tio())
    __swig_destroy__ = _LibreDWG.delete__dwg_object_object_tio

# Register _dwg_object_object_tio in _LibreDWG:
_LibreDWG._dwg_object_object_tio_swigregister(_dwg_object_object_tio)
class Dwg_Class(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    number = property(_LibreDWG.Dwg_Class_number_get, _LibreDWG.Dwg_Class_number_set)
    proxyflag = property(_LibreDWG.Dwg_Class_proxyflag_get, _LibreDWG.Dwg_Class_proxyflag_set)
    appname = property(_LibreDWG.Dwg_Class_appname_get, _LibreDWG.Dwg_Class_appname_set)
    cppname = property(_LibreDWG.Dwg_Class_cppname_get, _LibreDWG.Dwg_Class_cppname_set)
    dxfname = property(_LibreDWG.Dwg_Class_dxfname_get, _LibreDWG.Dwg_Class_dxfname_set)
    dxfname_u = property(_LibreDWG.Dwg_Class_dxfname_u_get, _LibreDWG.Dwg_Class_dxfname_u_set)
    is_zombie = property(_LibreDWG.Dwg_Class_is_zombie_get, _LibreDWG.Dwg_Class_is_zombie_set)
    item_class_id = property(_LibreDWG.Dwg_Class_item_class_id_get, _LibreDWG.Dwg_Class_item_class_id_set)
    num_instances = property(_LibreDWG.Dwg_Class_num_instances_get, _LibreDWG.Dwg_Class_num_instances_set)
    dwg_version = property(_LibreDWG.Dwg_Class_dwg_version_get, _LibreDWG.Dwg_Class_dwg_version_set)
    maint_version = property(_LibreDWG.Dwg_Class_maint_version_get, _LibreDWG.Dwg_Class_maint_version_set)
    unknown_1 = property(_LibreDWG.Dwg_Class_unknown_1_get, _LibreDWG.Dwg_Class_unknown_1_set)
    unknown_2 = property(_LibreDWG.Dwg_Class_unknown_2_get, _LibreDWG.Dwg_Class_unknown_2_set)

    def __init__(self):
        _LibreDWG.Dwg_Class_swiginit(self, _LibreDWG.new_Dwg_Class())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Class

# Register Dwg_Class in _LibreDWG:
_LibreDWG.Dwg_Class_swigregister(Dwg_Class)
class Dwg_Object(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    size = property(_LibreDWG.Dwg_Object_size_get, _LibreDWG.Dwg_Object_size_set)
    address = property(_LibreDWG.Dwg_Object_address_get, _LibreDWG.Dwg_Object_address_set)
    type = property(_LibreDWG.Dwg_Object_type_get, _LibreDWG.Dwg_Object_type_set)
    index = property(_LibreDWG.Dwg_Object_index_get, _LibreDWG.Dwg_Object_index_set)
    fixedtype = property(_LibreDWG.Dwg_Object_fixedtype_get, _LibreDWG.Dwg_Object_fixedtype_set)
    name = property(_LibreDWG.Dwg_Object_name_get, _LibreDWG.Dwg_Object_name_set)
    dxfname = property(_LibreDWG.Dwg_Object_dxfname_get, _LibreDWG.Dwg_Object_dxfname_set)
    supertype = property(_LibreDWG.Dwg_Object_supertype_get, _LibreDWG.Dwg_Object_supertype_set)
    tio = property(_LibreDWG.Dwg_Object_tio_get)
    handle = property(_LibreDWG.Dwg_Object_handle_get, _LibreDWG.Dwg_Object_handle_set)
    parent = property(_LibreDWG.Dwg_Object_parent_get, _LibreDWG.Dwg_Object_parent_set)
    klass = property(_LibreDWG.Dwg_Object_klass_get, _LibreDWG.Dwg_Object_klass_set)
    bitsize = property(_LibreDWG.Dwg_Object_bitsize_get, _LibreDWG.Dwg_Object_bitsize_set)
    bitsize_pos = property(_LibreDWG.Dwg_Object_bitsize_pos_get, _LibreDWG.Dwg_Object_bitsize_pos_set)
    hdlpos = property(_LibreDWG.Dwg_Object_hdlpos_get, _LibreDWG.Dwg_Object_hdlpos_set)
    was_bitsize_set = property(_LibreDWG.Dwg_Object_was_bitsize_set_get, _LibreDWG.Dwg_Object_was_bitsize_set_set)
    has_strings = property(_LibreDWG.Dwg_Object_has_strings_get, _LibreDWG.Dwg_Object_has_strings_set)
    stringstream_size = property(_LibreDWG.Dwg_Object_stringstream_size_get, _LibreDWG.Dwg_Object_stringstream_size_set)
    handlestream_size = property(_LibreDWG.Dwg_Object_handlestream_size_get, _LibreDWG.Dwg_Object_handlestream_size_set)
    common_size = property(_LibreDWG.Dwg_Object_common_size_get, _LibreDWG.Dwg_Object_common_size_set)
    num_unknown_bits = property(_LibreDWG.Dwg_Object_num_unknown_bits_get, _LibreDWG.Dwg_Object_num_unknown_bits_set)
    unknown_bits = property(_LibreDWG.Dwg_Object_unknown_bits_get, _LibreDWG.Dwg_Object_unknown_bits_set)
    num_unknown_rest = property(_LibreDWG.Dwg_Object_num_unknown_rest_get, _LibreDWG.Dwg_Object_num_unknown_rest_set)
    unknown_rest = property(_LibreDWG.Dwg_Object_unknown_rest_get, _LibreDWG.Dwg_Object_unknown_rest_set)

    def __init__(self):
        _LibreDWG.Dwg_Object_swiginit(self, _LibreDWG.new_Dwg_Object())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Object

# Register Dwg_Object in _LibreDWG:
_LibreDWG.Dwg_Object_swigregister(Dwg_Object)
class _dwg_object_tio(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    entity = property(_LibreDWG._dwg_object_tio_entity_get, _LibreDWG._dwg_object_tio_entity_set)
    object = property(_LibreDWG._dwg_object_tio_object_get, _LibreDWG._dwg_object_tio_object_set)

    def __init__(self):
        _LibreDWG._dwg_object_tio_swiginit(self, _LibreDWG.new__dwg_object_tio())
    __swig_destroy__ = _LibreDWG.delete__dwg_object_tio

# Register _dwg_object_tio in _LibreDWG:
_LibreDWG._dwg_object_tio_swigregister(_dwg_object_tio)
class Dwg_Chain(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    chain = property(_LibreDWG.Dwg_Chain_chain_get, _LibreDWG.Dwg_Chain_chain_set)
    size = property(_LibreDWG.Dwg_Chain_size_get, _LibreDWG.Dwg_Chain_size_set)
    byte = property(_LibreDWG.Dwg_Chain_byte_get, _LibreDWG.Dwg_Chain_byte_set)
    bit = property(_LibreDWG.Dwg_Chain_bit_get, _LibreDWG.Dwg_Chain_bit_set)

    def __init__(self):
        _LibreDWG.Dwg_Chain_swiginit(self, _LibreDWG.new_Dwg_Chain())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Chain

# Register Dwg_Chain in _LibreDWG:
_LibreDWG.Dwg_Chain_swigregister(Dwg_Chain)
SECTION_UNKNOWN = _LibreDWG.SECTION_UNKNOWN
SECTION_HEADER = _LibreDWG.SECTION_HEADER
SECTION_AUXHEADER = _LibreDWG.SECTION_AUXHEADER
SECTION_CLASSES = _LibreDWG.SECTION_CLASSES
SECTION_HANDLES = _LibreDWG.SECTION_HANDLES
SECTION_TEMPLATE = _LibreDWG.SECTION_TEMPLATE
SECTION_OBJFREESPACE = _LibreDWG.SECTION_OBJFREESPACE
SECTION_OBJECTS = _LibreDWG.SECTION_OBJECTS
SECTION_REVHISTORY = _LibreDWG.SECTION_REVHISTORY
SECTION_SUMMARYINFO = _LibreDWG.SECTION_SUMMARYINFO
SECTION_PREVIEW = _LibreDWG.SECTION_PREVIEW
SECTION_APPINFO = _LibreDWG.SECTION_APPINFO
SECTION_APPINFOHISTORY = _LibreDWG.SECTION_APPINFOHISTORY
SECTION_FILEDEPLIST = _LibreDWG.SECTION_FILEDEPLIST
SECTION_SECURITY = _LibreDWG.SECTION_SECURITY
SECTION_VBAPROJECT = _LibreDWG.SECTION_VBAPROJECT
SECTION_SIGNATURE = _LibreDWG.SECTION_SIGNATURE
SECTION_ACDS = _LibreDWG.SECTION_ACDS
SECTION_INFO = _LibreDWG.SECTION_INFO
SECTION_SYSTEM_MAP = _LibreDWG.SECTION_SYSTEM_MAP
SECTION_HEADER_R13 = _LibreDWG.SECTION_HEADER_R13
SECTION_CLASSES_R13 = _LibreDWG.SECTION_CLASSES_R13
SECTION_HANDLES_R13 = _LibreDWG.SECTION_HANDLES_R13
SECTION_OBJFREESPACE_R13 = _LibreDWG.SECTION_OBJFREESPACE_R13
SECTION_TEMPLATE_R13 = _LibreDWG.SECTION_TEMPLATE_R13
SECTION_AUXHEADER_R2000 = _LibreDWG.SECTION_AUXHEADER_R2000
SECTION_THUMBNAIL_R13 = _LibreDWG.SECTION_THUMBNAIL_R13
SECTION_HEADER_R11 = _LibreDWG.SECTION_HEADER_R11
SECTION_BLOCK = _LibreDWG.SECTION_BLOCK
SECTION_LAYER = _LibreDWG.SECTION_LAYER
SECTION_STYLE = _LibreDWG.SECTION_STYLE
SECTION_LTYPE = _LibreDWG.SECTION_LTYPE
SECTION_VIEW = _LibreDWG.SECTION_VIEW
SECTION_UCS = _LibreDWG.SECTION_UCS
SECTION_VPORT = _LibreDWG.SECTION_VPORT
SECTION_APPID = _LibreDWG.SECTION_APPID
SECTION_DIMSTYLE = _LibreDWG.SECTION_DIMSTYLE
SECTION_VX = _LibreDWG.SECTION_VX
class Dwg_Section(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    number = property(_LibreDWG.Dwg_Section_number_get, _LibreDWG.Dwg_Section_number_set)
    size = property(_LibreDWG.Dwg_Section_size_get, _LibreDWG.Dwg_Section_size_set)
    address = property(_LibreDWG.Dwg_Section_address_get, _LibreDWG.Dwg_Section_address_set)
    objid_r11 = property(_LibreDWG.Dwg_Section_objid_r11_get, _LibreDWG.Dwg_Section_objid_r11_set)
    parent = property(_LibreDWG.Dwg_Section_parent_get, _LibreDWG.Dwg_Section_parent_set)
    left = property(_LibreDWG.Dwg_Section_left_get, _LibreDWG.Dwg_Section_left_set)
    right = property(_LibreDWG.Dwg_Section_right_get, _LibreDWG.Dwg_Section_right_set)
    x00 = property(_LibreDWG.Dwg_Section_x00_get, _LibreDWG.Dwg_Section_x00_set)
    type = property(_LibreDWG.Dwg_Section_type_get, _LibreDWG.Dwg_Section_type_set)
    name = property(_LibreDWG.Dwg_Section_name_get, _LibreDWG.Dwg_Section_name_set)
    section_type = property(_LibreDWG.Dwg_Section_section_type_get, _LibreDWG.Dwg_Section_section_type_set)
    decomp_data_size = property(_LibreDWG.Dwg_Section_decomp_data_size_get, _LibreDWG.Dwg_Section_decomp_data_size_set)
    comp_data_size = property(_LibreDWG.Dwg_Section_comp_data_size_get, _LibreDWG.Dwg_Section_comp_data_size_set)
    compression_type = property(_LibreDWG.Dwg_Section_compression_type_get, _LibreDWG.Dwg_Section_compression_type_set)
    checksum = property(_LibreDWG.Dwg_Section_checksum_get, _LibreDWG.Dwg_Section_checksum_set)
    flags_r11 = property(_LibreDWG.Dwg_Section_flags_r11_get, _LibreDWG.Dwg_Section_flags_r11_set)

    def __init__(self):
        _LibreDWG.Dwg_Section_swiginit(self, _LibreDWG.new_Dwg_Section())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Section

# Register Dwg_Section in _LibreDWG:
_LibreDWG.Dwg_Section_swigregister(Dwg_Section)
class Dwg_Section_InfoHdr(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num_desc = property(_LibreDWG.Dwg_Section_InfoHdr_num_desc_get, _LibreDWG.Dwg_Section_InfoHdr_num_desc_set)
    compressed = property(_LibreDWG.Dwg_Section_InfoHdr_compressed_get, _LibreDWG.Dwg_Section_InfoHdr_compressed_set)
    max_size = property(_LibreDWG.Dwg_Section_InfoHdr_max_size_get, _LibreDWG.Dwg_Section_InfoHdr_max_size_set)
    encrypted = property(_LibreDWG.Dwg_Section_InfoHdr_encrypted_get, _LibreDWG.Dwg_Section_InfoHdr_encrypted_set)
    num_desc2 = property(_LibreDWG.Dwg_Section_InfoHdr_num_desc2_get, _LibreDWG.Dwg_Section_InfoHdr_num_desc2_set)

    def __init__(self):
        _LibreDWG.Dwg_Section_InfoHdr_swiginit(self, _LibreDWG.new_Dwg_Section_InfoHdr())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Section_InfoHdr

# Register Dwg_Section_InfoHdr in _LibreDWG:
_LibreDWG.Dwg_Section_InfoHdr_swigregister(Dwg_Section_InfoHdr)
class Dwg_Section_Info(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    size = property(_LibreDWG.Dwg_Section_Info_size_get, _LibreDWG.Dwg_Section_Info_size_set)
    num_sections = property(_LibreDWG.Dwg_Section_Info_num_sections_get, _LibreDWG.Dwg_Section_Info_num_sections_set)
    max_decomp_size = property(_LibreDWG.Dwg_Section_Info_max_decomp_size_get, _LibreDWG.Dwg_Section_Info_max_decomp_size_set)
    unknown = property(_LibreDWG.Dwg_Section_Info_unknown_get, _LibreDWG.Dwg_Section_Info_unknown_set)
    compressed = property(_LibreDWG.Dwg_Section_Info_compressed_get, _LibreDWG.Dwg_Section_Info_compressed_set)
    type = property(_LibreDWG.Dwg_Section_Info_type_get, _LibreDWG.Dwg_Section_Info_type_set)
    encrypted = property(_LibreDWG.Dwg_Section_Info_encrypted_get, _LibreDWG.Dwg_Section_Info_encrypted_set)
    name = property(_LibreDWG.Dwg_Section_Info_name_get, _LibreDWG.Dwg_Section_Info_name_set)
    fixedtype = property(_LibreDWG.Dwg_Section_Info_fixedtype_get, _LibreDWG.Dwg_Section_Info_fixedtype_set)
    sections = property(_LibreDWG.Dwg_Section_Info_sections_get, _LibreDWG.Dwg_Section_Info_sections_set)

    def __init__(self):
        _LibreDWG.Dwg_Section_Info_swiginit(self, _LibreDWG.new_Dwg_Section_Info())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Section_Info

# Register Dwg_Section_Info in _LibreDWG:
_LibreDWG.Dwg_Section_Info_swigregister(Dwg_Section_Info)
class Dwg_SummaryInfo_Property(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    tag = property(_LibreDWG.Dwg_SummaryInfo_Property_tag_get, _LibreDWG.Dwg_SummaryInfo_Property_tag_set)
    value = property(_LibreDWG.Dwg_SummaryInfo_Property_value_get, _LibreDWG.Dwg_SummaryInfo_Property_value_set)

    def __init__(self):
        _LibreDWG.Dwg_SummaryInfo_Property_swiginit(self, _LibreDWG.new_Dwg_SummaryInfo_Property())
    __swig_destroy__ = _LibreDWG.delete_Dwg_SummaryInfo_Property

# Register Dwg_SummaryInfo_Property in _LibreDWG:
_LibreDWG.Dwg_SummaryInfo_Property_swigregister(Dwg_SummaryInfo_Property)
class Dwg_FileDepList_Files(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    filename = property(_LibreDWG.Dwg_FileDepList_Files_filename_get, _LibreDWG.Dwg_FileDepList_Files_filename_set)
    filepath = property(_LibreDWG.Dwg_FileDepList_Files_filepath_get, _LibreDWG.Dwg_FileDepList_Files_filepath_set)
    fingerprint = property(_LibreDWG.Dwg_FileDepList_Files_fingerprint_get, _LibreDWG.Dwg_FileDepList_Files_fingerprint_set)
    version = property(_LibreDWG.Dwg_FileDepList_Files_version_get, _LibreDWG.Dwg_FileDepList_Files_version_set)
    feature_index = property(_LibreDWG.Dwg_FileDepList_Files_feature_index_get, _LibreDWG.Dwg_FileDepList_Files_feature_index_set)
    timestamp = property(_LibreDWG.Dwg_FileDepList_Files_timestamp_get, _LibreDWG.Dwg_FileDepList_Files_timestamp_set)
    filesize = property(_LibreDWG.Dwg_FileDepList_Files_filesize_get, _LibreDWG.Dwg_FileDepList_Files_filesize_set)
    affects_graphics = property(_LibreDWG.Dwg_FileDepList_Files_affects_graphics_get, _LibreDWG.Dwg_FileDepList_Files_affects_graphics_set)
    refcount = property(_LibreDWG.Dwg_FileDepList_Files_refcount_get, _LibreDWG.Dwg_FileDepList_Files_refcount_set)

    def __init__(self):
        _LibreDWG.Dwg_FileDepList_Files_swiginit(self, _LibreDWG.new_Dwg_FileDepList_Files())
    __swig_destroy__ = _LibreDWG.delete_Dwg_FileDepList_Files

# Register Dwg_FileDepList_Files in _LibreDWG:
_LibreDWG.Dwg_FileDepList_Files_swigregister(Dwg_FileDepList_Files)
class Dwg_AcDs_SegmentIndex(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    offset = property(_LibreDWG.Dwg_AcDs_SegmentIndex_offset_get, _LibreDWG.Dwg_AcDs_SegmentIndex_offset_set)
    size = property(_LibreDWG.Dwg_AcDs_SegmentIndex_size_get, _LibreDWG.Dwg_AcDs_SegmentIndex_size_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_SegmentIndex_swiginit(self, _LibreDWG.new_Dwg_AcDs_SegmentIndex())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_SegmentIndex

# Register Dwg_AcDs_SegmentIndex in _LibreDWG:
_LibreDWG.Dwg_AcDs_SegmentIndex_swigregister(Dwg_AcDs_SegmentIndex)
class Dwg_AcDs_DataIndex_Entry(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    segidx = property(_LibreDWG.Dwg_AcDs_DataIndex_Entry_segidx_get, _LibreDWG.Dwg_AcDs_DataIndex_Entry_segidx_set)
    offset = property(_LibreDWG.Dwg_AcDs_DataIndex_Entry_offset_get, _LibreDWG.Dwg_AcDs_DataIndex_Entry_offset_set)
    schidx = property(_LibreDWG.Dwg_AcDs_DataIndex_Entry_schidx_get, _LibreDWG.Dwg_AcDs_DataIndex_Entry_schidx_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_DataIndex_Entry_swiginit(self, _LibreDWG.new_Dwg_AcDs_DataIndex_Entry())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_DataIndex_Entry

# Register Dwg_AcDs_DataIndex_Entry in _LibreDWG:
_LibreDWG.Dwg_AcDs_DataIndex_Entry_swigregister(Dwg_AcDs_DataIndex_Entry)
class Dwg_AcDs_DataIndex(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num_entries = property(_LibreDWG.Dwg_AcDs_DataIndex_num_entries_get, _LibreDWG.Dwg_AcDs_DataIndex_num_entries_set)
    di_unknown = property(_LibreDWG.Dwg_AcDs_DataIndex_di_unknown_get, _LibreDWG.Dwg_AcDs_DataIndex_di_unknown_set)
    entries = property(_LibreDWG.Dwg_AcDs_DataIndex_entries_get, _LibreDWG.Dwg_AcDs_DataIndex_entries_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_DataIndex_swiginit(self, _LibreDWG.new_Dwg_AcDs_DataIndex())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_DataIndex

# Register Dwg_AcDs_DataIndex in _LibreDWG:
_LibreDWG.Dwg_AcDs_DataIndex_swigregister(Dwg_AcDs_DataIndex)
class Dwg_AcDs_Data_RecordHdr(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    entry_size = property(_LibreDWG.Dwg_AcDs_Data_RecordHdr_entry_size_get, _LibreDWG.Dwg_AcDs_Data_RecordHdr_entry_size_set)
    unknown = property(_LibreDWG.Dwg_AcDs_Data_RecordHdr_unknown_get, _LibreDWG.Dwg_AcDs_Data_RecordHdr_unknown_set)
    handle = property(_LibreDWG.Dwg_AcDs_Data_RecordHdr_handle_get, _LibreDWG.Dwg_AcDs_Data_RecordHdr_handle_set)
    offset = property(_LibreDWG.Dwg_AcDs_Data_RecordHdr_offset_get, _LibreDWG.Dwg_AcDs_Data_RecordHdr_offset_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_Data_RecordHdr_swiginit(self, _LibreDWG.new_Dwg_AcDs_Data_RecordHdr())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_Data_RecordHdr

# Register Dwg_AcDs_Data_RecordHdr in _LibreDWG:
_LibreDWG.Dwg_AcDs_Data_RecordHdr_swigregister(Dwg_AcDs_Data_RecordHdr)
class Dwg_AcDs_Data_Record(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    data_size = property(_LibreDWG.Dwg_AcDs_Data_Record_data_size_get, _LibreDWG.Dwg_AcDs_Data_Record_data_size_set)
    blob = property(_LibreDWG.Dwg_AcDs_Data_Record_blob_get, _LibreDWG.Dwg_AcDs_Data_Record_blob_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_Data_Record_swiginit(self, _LibreDWG.new_Dwg_AcDs_Data_Record())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_Data_Record

# Register Dwg_AcDs_Data_Record in _LibreDWG:
_LibreDWG.Dwg_AcDs_Data_Record_swigregister(Dwg_AcDs_Data_Record)
class Dwg_AcDs_Data(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    record_hdrs = property(_LibreDWG.Dwg_AcDs_Data_record_hdrs_get, _LibreDWG.Dwg_AcDs_Data_record_hdrs_set)
    records = property(_LibreDWG.Dwg_AcDs_Data_records_get, _LibreDWG.Dwg_AcDs_Data_records_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_Data_swiginit(self, _LibreDWG.new_Dwg_AcDs_Data())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_Data

# Register Dwg_AcDs_Data in _LibreDWG:
_LibreDWG.Dwg_AcDs_Data_swigregister(Dwg_AcDs_Data)
class Dwg_AcDs_DataBlobRef_Page(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    segidx = property(_LibreDWG.Dwg_AcDs_DataBlobRef_Page_segidx_get, _LibreDWG.Dwg_AcDs_DataBlobRef_Page_segidx_set)
    size = property(_LibreDWG.Dwg_AcDs_DataBlobRef_Page_size_get, _LibreDWG.Dwg_AcDs_DataBlobRef_Page_size_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_DataBlobRef_Page_swiginit(self, _LibreDWG.new_Dwg_AcDs_DataBlobRef_Page())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_DataBlobRef_Page

# Register Dwg_AcDs_DataBlobRef_Page in _LibreDWG:
_LibreDWG.Dwg_AcDs_DataBlobRef_Page_swigregister(Dwg_AcDs_DataBlobRef_Page)
class Dwg_AcDs_DataBlobRef(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    total_data_size = property(_LibreDWG.Dwg_AcDs_DataBlobRef_total_data_size_get, _LibreDWG.Dwg_AcDs_DataBlobRef_total_data_size_set)
    num_pages = property(_LibreDWG.Dwg_AcDs_DataBlobRef_num_pages_get, _LibreDWG.Dwg_AcDs_DataBlobRef_num_pages_set)
    record_size = property(_LibreDWG.Dwg_AcDs_DataBlobRef_record_size_get, _LibreDWG.Dwg_AcDs_DataBlobRef_record_size_set)
    page_size = property(_LibreDWG.Dwg_AcDs_DataBlobRef_page_size_get, _LibreDWG.Dwg_AcDs_DataBlobRef_page_size_set)
    unknown_1 = property(_LibreDWG.Dwg_AcDs_DataBlobRef_unknown_1_get, _LibreDWG.Dwg_AcDs_DataBlobRef_unknown_1_set)
    unknown_2 = property(_LibreDWG.Dwg_AcDs_DataBlobRef_unknown_2_get, _LibreDWG.Dwg_AcDs_DataBlobRef_unknown_2_set)
    pages = property(_LibreDWG.Dwg_AcDs_DataBlobRef_pages_get, _LibreDWG.Dwg_AcDs_DataBlobRef_pages_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_DataBlobRef_swiginit(self, _LibreDWG.new_Dwg_AcDs_DataBlobRef())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_DataBlobRef

# Register Dwg_AcDs_DataBlobRef in _LibreDWG:
_LibreDWG.Dwg_AcDs_DataBlobRef_swigregister(Dwg_AcDs_DataBlobRef)
class Dwg_AcDs_DataBlob(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    data_size = property(_LibreDWG.Dwg_AcDs_DataBlob_data_size_get, _LibreDWG.Dwg_AcDs_DataBlob_data_size_set)
    page_count = property(_LibreDWG.Dwg_AcDs_DataBlob_page_count_get, _LibreDWG.Dwg_AcDs_DataBlob_page_count_set)
    record_size = property(_LibreDWG.Dwg_AcDs_DataBlob_record_size_get, _LibreDWG.Dwg_AcDs_DataBlob_record_size_set)
    page_size = property(_LibreDWG.Dwg_AcDs_DataBlob_page_size_get, _LibreDWG.Dwg_AcDs_DataBlob_page_size_set)
    unknown_1 = property(_LibreDWG.Dwg_AcDs_DataBlob_unknown_1_get, _LibreDWG.Dwg_AcDs_DataBlob_unknown_1_set)
    unknown_2 = property(_LibreDWG.Dwg_AcDs_DataBlob_unknown_2_get, _LibreDWG.Dwg_AcDs_DataBlob_unknown_2_set)
    ref = property(_LibreDWG.Dwg_AcDs_DataBlob_ref_get, _LibreDWG.Dwg_AcDs_DataBlob_ref_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_DataBlob_swiginit(self, _LibreDWG.new_Dwg_AcDs_DataBlob())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_DataBlob

# Register Dwg_AcDs_DataBlob in _LibreDWG:
_LibreDWG.Dwg_AcDs_DataBlob_swigregister(Dwg_AcDs_DataBlob)
class Dwg_AcDs_DataBlob01(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    total_data_size = property(_LibreDWG.Dwg_AcDs_DataBlob01_total_data_size_get, _LibreDWG.Dwg_AcDs_DataBlob01_total_data_size_set)
    page_start_offset = property(_LibreDWG.Dwg_AcDs_DataBlob01_page_start_offset_get, _LibreDWG.Dwg_AcDs_DataBlob01_page_start_offset_set)
    page_index = property(_LibreDWG.Dwg_AcDs_DataBlob01_page_index_get, _LibreDWG.Dwg_AcDs_DataBlob01_page_index_set)
    page_count = property(_LibreDWG.Dwg_AcDs_DataBlob01_page_count_get, _LibreDWG.Dwg_AcDs_DataBlob01_page_count_set)
    page_data_size = property(_LibreDWG.Dwg_AcDs_DataBlob01_page_data_size_get, _LibreDWG.Dwg_AcDs_DataBlob01_page_data_size_set)
    page_data = property(_LibreDWG.Dwg_AcDs_DataBlob01_page_data_get, _LibreDWG.Dwg_AcDs_DataBlob01_page_data_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_DataBlob01_swiginit(self, _LibreDWG.new_Dwg_AcDs_DataBlob01())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_DataBlob01

# Register Dwg_AcDs_DataBlob01 in _LibreDWG:
_LibreDWG.Dwg_AcDs_DataBlob01_swigregister(Dwg_AcDs_DataBlob01)
class Dwg_AcDs_SchemaIndex_Prop(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    index = property(_LibreDWG.Dwg_AcDs_SchemaIndex_Prop_index_get, _LibreDWG.Dwg_AcDs_SchemaIndex_Prop_index_set)
    segidx = property(_LibreDWG.Dwg_AcDs_SchemaIndex_Prop_segidx_get, _LibreDWG.Dwg_AcDs_SchemaIndex_Prop_segidx_set)
    offset = property(_LibreDWG.Dwg_AcDs_SchemaIndex_Prop_offset_get, _LibreDWG.Dwg_AcDs_SchemaIndex_Prop_offset_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_SchemaIndex_Prop_swiginit(self, _LibreDWG.new_Dwg_AcDs_SchemaIndex_Prop())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_SchemaIndex_Prop

# Register Dwg_AcDs_SchemaIndex_Prop in _LibreDWG:
_LibreDWG.Dwg_AcDs_SchemaIndex_Prop_swigregister(Dwg_AcDs_SchemaIndex_Prop)
class Dwg_AcDs_SchemaIndex(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num_props = property(_LibreDWG.Dwg_AcDs_SchemaIndex_num_props_get, _LibreDWG.Dwg_AcDs_SchemaIndex_num_props_set)
    si_unknown_1 = property(_LibreDWG.Dwg_AcDs_SchemaIndex_si_unknown_1_get, _LibreDWG.Dwg_AcDs_SchemaIndex_si_unknown_1_set)
    props = property(_LibreDWG.Dwg_AcDs_SchemaIndex_props_get, _LibreDWG.Dwg_AcDs_SchemaIndex_props_set)
    si_tag = property(_LibreDWG.Dwg_AcDs_SchemaIndex_si_tag_get, _LibreDWG.Dwg_AcDs_SchemaIndex_si_tag_set)
    num_prop_entries = property(_LibreDWG.Dwg_AcDs_SchemaIndex_num_prop_entries_get, _LibreDWG.Dwg_AcDs_SchemaIndex_num_prop_entries_set)
    si_unknown_2 = property(_LibreDWG.Dwg_AcDs_SchemaIndex_si_unknown_2_get, _LibreDWG.Dwg_AcDs_SchemaIndex_si_unknown_2_set)
    prop_entries = property(_LibreDWG.Dwg_AcDs_SchemaIndex_prop_entries_get, _LibreDWG.Dwg_AcDs_SchemaIndex_prop_entries_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_SchemaIndex_swiginit(self, _LibreDWG.new_Dwg_AcDs_SchemaIndex())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_SchemaIndex

# Register Dwg_AcDs_SchemaIndex in _LibreDWG:
_LibreDWG.Dwg_AcDs_SchemaIndex_swigregister(Dwg_AcDs_SchemaIndex)
class Dwg_AcDs_Schema_Prop(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    flags = property(_LibreDWG.Dwg_AcDs_Schema_Prop_flags_get, _LibreDWG.Dwg_AcDs_Schema_Prop_flags_set)
    namidx = property(_LibreDWG.Dwg_AcDs_Schema_Prop_namidx_get, _LibreDWG.Dwg_AcDs_Schema_Prop_namidx_set)
    type = property(_LibreDWG.Dwg_AcDs_Schema_Prop_type_get, _LibreDWG.Dwg_AcDs_Schema_Prop_type_set)
    type_size = property(_LibreDWG.Dwg_AcDs_Schema_Prop_type_size_get, _LibreDWG.Dwg_AcDs_Schema_Prop_type_size_set)
    unknown_1 = property(_LibreDWG.Dwg_AcDs_Schema_Prop_unknown_1_get, _LibreDWG.Dwg_AcDs_Schema_Prop_unknown_1_set)
    unknown_2 = property(_LibreDWG.Dwg_AcDs_Schema_Prop_unknown_2_get, _LibreDWG.Dwg_AcDs_Schema_Prop_unknown_2_set)
    num_values = property(_LibreDWG.Dwg_AcDs_Schema_Prop_num_values_get, _LibreDWG.Dwg_AcDs_Schema_Prop_num_values_set)
    values = property(_LibreDWG.Dwg_AcDs_Schema_Prop_values_get, _LibreDWG.Dwg_AcDs_Schema_Prop_values_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_Schema_Prop_swiginit(self, _LibreDWG.new_Dwg_AcDs_Schema_Prop())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_Schema_Prop

# Register Dwg_AcDs_Schema_Prop in _LibreDWG:
_LibreDWG.Dwg_AcDs_Schema_Prop_swigregister(Dwg_AcDs_Schema_Prop)
class Dwg_AcDs_Schema(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num_index = property(_LibreDWG.Dwg_AcDs_Schema_num_index_get, _LibreDWG.Dwg_AcDs_Schema_num_index_set)
    index = property(_LibreDWG.Dwg_AcDs_Schema_index_get, _LibreDWG.Dwg_AcDs_Schema_index_set)
    num_props = property(_LibreDWG.Dwg_AcDs_Schema_num_props_get, _LibreDWG.Dwg_AcDs_Schema_num_props_set)
    props = property(_LibreDWG.Dwg_AcDs_Schema_props_get, _LibreDWG.Dwg_AcDs_Schema_props_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_Schema_swiginit(self, _LibreDWG.new_Dwg_AcDs_Schema())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_Schema

# Register Dwg_AcDs_Schema in _LibreDWG:
_LibreDWG.Dwg_AcDs_Schema_swigregister(Dwg_AcDs_Schema)
class Dwg_AcDs_SchemaData_UProp(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    size = property(_LibreDWG.Dwg_AcDs_SchemaData_UProp_size_get, _LibreDWG.Dwg_AcDs_SchemaData_UProp_size_set)
    flags = property(_LibreDWG.Dwg_AcDs_SchemaData_UProp_flags_get, _LibreDWG.Dwg_AcDs_SchemaData_UProp_flags_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_SchemaData_UProp_swiginit(self, _LibreDWG.new_Dwg_AcDs_SchemaData_UProp())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_SchemaData_UProp

# Register Dwg_AcDs_SchemaData_UProp in _LibreDWG:
_LibreDWG.Dwg_AcDs_SchemaData_UProp_swigregister(Dwg_AcDs_SchemaData_UProp)
class Dwg_AcDs_SchemaData(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num_uprops = property(_LibreDWG.Dwg_AcDs_SchemaData_num_uprops_get, _LibreDWG.Dwg_AcDs_SchemaData_num_uprops_set)
    uprops = property(_LibreDWG.Dwg_AcDs_SchemaData_uprops_get, _LibreDWG.Dwg_AcDs_SchemaData_uprops_set)
    num_schemas = property(_LibreDWG.Dwg_AcDs_SchemaData_num_schemas_get, _LibreDWG.Dwg_AcDs_SchemaData_num_schemas_set)
    schemas = property(_LibreDWG.Dwg_AcDs_SchemaData_schemas_get, _LibreDWG.Dwg_AcDs_SchemaData_schemas_set)
    num_propnames = property(_LibreDWG.Dwg_AcDs_SchemaData_num_propnames_get, _LibreDWG.Dwg_AcDs_SchemaData_num_propnames_set)
    propnames = property(_LibreDWG.Dwg_AcDs_SchemaData_propnames_get, _LibreDWG.Dwg_AcDs_SchemaData_propnames_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_SchemaData_swiginit(self, _LibreDWG.new_Dwg_AcDs_SchemaData())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_SchemaData

# Register Dwg_AcDs_SchemaData in _LibreDWG:
_LibreDWG.Dwg_AcDs_SchemaData_swigregister(Dwg_AcDs_SchemaData)
class Dwg_AcDs_Search_IdIdx(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    handle = property(_LibreDWG.Dwg_AcDs_Search_IdIdx_handle_get, _LibreDWG.Dwg_AcDs_Search_IdIdx_handle_set)
    num_ididx = property(_LibreDWG.Dwg_AcDs_Search_IdIdx_num_ididx_get, _LibreDWG.Dwg_AcDs_Search_IdIdx_num_ididx_set)
    ididx = property(_LibreDWG.Dwg_AcDs_Search_IdIdx_ididx_get, _LibreDWG.Dwg_AcDs_Search_IdIdx_ididx_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_Search_IdIdx_swiginit(self, _LibreDWG.new_Dwg_AcDs_Search_IdIdx())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_Search_IdIdx

# Register Dwg_AcDs_Search_IdIdx in _LibreDWG:
_LibreDWG.Dwg_AcDs_Search_IdIdx_swigregister(Dwg_AcDs_Search_IdIdx)
class Dwg_AcDs_Search_IdIdxs(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num_ididx = property(_LibreDWG.Dwg_AcDs_Search_IdIdxs_num_ididx_get, _LibreDWG.Dwg_AcDs_Search_IdIdxs_num_ididx_set)
    ididx = property(_LibreDWG.Dwg_AcDs_Search_IdIdxs_ididx_get, _LibreDWG.Dwg_AcDs_Search_IdIdxs_ididx_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_Search_IdIdxs_swiginit(self, _LibreDWG.new_Dwg_AcDs_Search_IdIdxs())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_Search_IdIdxs

# Register Dwg_AcDs_Search_IdIdxs in _LibreDWG:
_LibreDWG.Dwg_AcDs_Search_IdIdxs_swigregister(Dwg_AcDs_Search_IdIdxs)
class Dwg_AcDs_Search_Data(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    schema_namidx = property(_LibreDWG.Dwg_AcDs_Search_Data_schema_namidx_get, _LibreDWG.Dwg_AcDs_Search_Data_schema_namidx_set)
    num_sortedidx = property(_LibreDWG.Dwg_AcDs_Search_Data_num_sortedidx_get, _LibreDWG.Dwg_AcDs_Search_Data_num_sortedidx_set)
    sortedidx = property(_LibreDWG.Dwg_AcDs_Search_Data_sortedidx_get, _LibreDWG.Dwg_AcDs_Search_Data_sortedidx_set)
    num_ididxs = property(_LibreDWG.Dwg_AcDs_Search_Data_num_ididxs_get, _LibreDWG.Dwg_AcDs_Search_Data_num_ididxs_set)
    unknown = property(_LibreDWG.Dwg_AcDs_Search_Data_unknown_get, _LibreDWG.Dwg_AcDs_Search_Data_unknown_set)
    ididxs = property(_LibreDWG.Dwg_AcDs_Search_Data_ididxs_get, _LibreDWG.Dwg_AcDs_Search_Data_ididxs_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_Search_Data_swiginit(self, _LibreDWG.new_Dwg_AcDs_Search_Data())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_Search_Data

# Register Dwg_AcDs_Search_Data in _LibreDWG:
_LibreDWG.Dwg_AcDs_Search_Data_swigregister(Dwg_AcDs_Search_Data)
class Dwg_AcDs_Search(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num_search = property(_LibreDWG.Dwg_AcDs_Search_num_search_get, _LibreDWG.Dwg_AcDs_Search_num_search_set)
    search = property(_LibreDWG.Dwg_AcDs_Search_search_get, _LibreDWG.Dwg_AcDs_Search_search_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_Search_swiginit(self, _LibreDWG.new_Dwg_AcDs_Search())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_Search

# Register Dwg_AcDs_Search in _LibreDWG:
_LibreDWG.Dwg_AcDs_Search_swigregister(Dwg_AcDs_Search)
class Dwg_AcDs_Segment(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    signature = property(_LibreDWG.Dwg_AcDs_Segment_signature_get, _LibreDWG.Dwg_AcDs_Segment_signature_set)
    name = property(_LibreDWG.Dwg_AcDs_Segment_name_get, _LibreDWG.Dwg_AcDs_Segment_name_set)
    type = property(_LibreDWG.Dwg_AcDs_Segment_type_get, _LibreDWG.Dwg_AcDs_Segment_type_set)
    segment_idx = property(_LibreDWG.Dwg_AcDs_Segment_segment_idx_get, _LibreDWG.Dwg_AcDs_Segment_segment_idx_set)
    is_blob01 = property(_LibreDWG.Dwg_AcDs_Segment_is_blob01_get, _LibreDWG.Dwg_AcDs_Segment_is_blob01_set)
    segsize = property(_LibreDWG.Dwg_AcDs_Segment_segsize_get, _LibreDWG.Dwg_AcDs_Segment_segsize_set)
    unknown_2 = property(_LibreDWG.Dwg_AcDs_Segment_unknown_2_get, _LibreDWG.Dwg_AcDs_Segment_unknown_2_set)
    ds_version = property(_LibreDWG.Dwg_AcDs_Segment_ds_version_get, _LibreDWG.Dwg_AcDs_Segment_ds_version_set)
    unknown_3 = property(_LibreDWG.Dwg_AcDs_Segment_unknown_3_get, _LibreDWG.Dwg_AcDs_Segment_unknown_3_set)
    data_algn_offset = property(_LibreDWG.Dwg_AcDs_Segment_data_algn_offset_get, _LibreDWG.Dwg_AcDs_Segment_data_algn_offset_set)
    objdata_algn_offset = property(_LibreDWG.Dwg_AcDs_Segment_objdata_algn_offset_get, _LibreDWG.Dwg_AcDs_Segment_objdata_algn_offset_set)
    padding = property(_LibreDWG.Dwg_AcDs_Segment_padding_get, _LibreDWG.Dwg_AcDs_Segment_padding_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_Segment_swiginit(self, _LibreDWG.new_Dwg_AcDs_Segment())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs_Segment

# Register Dwg_AcDs_Segment in _LibreDWG:
_LibreDWG.Dwg_AcDs_Segment_swigregister(Dwg_AcDs_Segment)
class Dwg_AcDs(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    file_signature = property(_LibreDWG.Dwg_AcDs_file_signature_get, _LibreDWG.Dwg_AcDs_file_signature_set)
    file_header_size = property(_LibreDWG.Dwg_AcDs_file_header_size_get, _LibreDWG.Dwg_AcDs_file_header_size_set)
    unknown_1 = property(_LibreDWG.Dwg_AcDs_unknown_1_get, _LibreDWG.Dwg_AcDs_unknown_1_set)
    version = property(_LibreDWG.Dwg_AcDs_version_get, _LibreDWG.Dwg_AcDs_version_set)
    unknown_2 = property(_LibreDWG.Dwg_AcDs_unknown_2_get, _LibreDWG.Dwg_AcDs_unknown_2_set)
    ds_version = property(_LibreDWG.Dwg_AcDs_ds_version_get, _LibreDWG.Dwg_AcDs_ds_version_set)
    segidx_offset = property(_LibreDWG.Dwg_AcDs_segidx_offset_get, _LibreDWG.Dwg_AcDs_segidx_offset_set)
    segidx_unknown = property(_LibreDWG.Dwg_AcDs_segidx_unknown_get, _LibreDWG.Dwg_AcDs_segidx_unknown_set)
    num_segidx = property(_LibreDWG.Dwg_AcDs_num_segidx_get, _LibreDWG.Dwg_AcDs_num_segidx_set)
    schidx_segidx = property(_LibreDWG.Dwg_AcDs_schidx_segidx_get, _LibreDWG.Dwg_AcDs_schidx_segidx_set)
    datidx_segidx = property(_LibreDWG.Dwg_AcDs_datidx_segidx_get, _LibreDWG.Dwg_AcDs_datidx_segidx_set)
    search_segidx = property(_LibreDWG.Dwg_AcDs_search_segidx_get, _LibreDWG.Dwg_AcDs_search_segidx_set)
    prvsav_segidx = property(_LibreDWG.Dwg_AcDs_prvsav_segidx_get, _LibreDWG.Dwg_AcDs_prvsav_segidx_set)
    file_size = property(_LibreDWG.Dwg_AcDs_file_size_get, _LibreDWG.Dwg_AcDs_file_size_set)
    total_segments = property(_LibreDWG.Dwg_AcDs_total_segments_get, _LibreDWG.Dwg_AcDs_total_segments_set)
    segidx = property(_LibreDWG.Dwg_AcDs_segidx_get, _LibreDWG.Dwg_AcDs_segidx_set)
    datidx = property(_LibreDWG.Dwg_AcDs_datidx_get, _LibreDWG.Dwg_AcDs_datidx_set)
    data = property(_LibreDWG.Dwg_AcDs_data_get, _LibreDWG.Dwg_AcDs_data_set)
    blob01 = property(_LibreDWG.Dwg_AcDs_blob01_get, _LibreDWG.Dwg_AcDs_blob01_set)
    schidx = property(_LibreDWG.Dwg_AcDs_schidx_get, _LibreDWG.Dwg_AcDs_schidx_set)
    schdat = property(_LibreDWG.Dwg_AcDs_schdat_get, _LibreDWG.Dwg_AcDs_schdat_set)
    search = property(_LibreDWG.Dwg_AcDs_search_get, _LibreDWG.Dwg_AcDs_search_set)
    segments = property(_LibreDWG.Dwg_AcDs_segments_get, _LibreDWG.Dwg_AcDs_segments_set)

    def __init__(self):
        _LibreDWG.Dwg_AcDs_swiginit(self, _LibreDWG.new_Dwg_AcDs())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AcDs

# Register Dwg_AcDs in _LibreDWG:
_LibreDWG.Dwg_AcDs_swigregister(Dwg_AcDs)
class Dwg_Header(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    version = property(_LibreDWG.Dwg_Header_version_get, _LibreDWG.Dwg_Header_version_set)
    from_version = property(_LibreDWG.Dwg_Header_from_version_get, _LibreDWG.Dwg_Header_from_version_set)
    maint_rel_version = property(_LibreDWG.Dwg_Header_maint_rel_version_get, _LibreDWG.Dwg_Header_maint_rel_version_set)
    zero_one_or_three = property(_LibreDWG.Dwg_Header_zero_one_or_three_get, _LibreDWG.Dwg_Header_zero_one_or_three_set)
    numentity_sections = property(_LibreDWG.Dwg_Header_numentity_sections_get, _LibreDWG.Dwg_Header_numentity_sections_set)
    numheader_vars = property(_LibreDWG.Dwg_Header_numheader_vars_get, _LibreDWG.Dwg_Header_numheader_vars_set)
    thumbnail_address = property(_LibreDWG.Dwg_Header_thumbnail_address_get, _LibreDWG.Dwg_Header_thumbnail_address_set)
    dwg_version = property(_LibreDWG.Dwg_Header_dwg_version_get, _LibreDWG.Dwg_Header_dwg_version_set)
    maint_version = property(_LibreDWG.Dwg_Header_maint_version_get, _LibreDWG.Dwg_Header_maint_version_set)
    entities_start = property(_LibreDWG.Dwg_Header_entities_start_get, _LibreDWG.Dwg_Header_entities_start_set)
    entities_end = property(_LibreDWG.Dwg_Header_entities_end_get, _LibreDWG.Dwg_Header_entities_end_set)
    blocks_start = property(_LibreDWG.Dwg_Header_blocks_start_get, _LibreDWG.Dwg_Header_blocks_start_set)
    blocks_size = property(_LibreDWG.Dwg_Header_blocks_size_get, _LibreDWG.Dwg_Header_blocks_size_set)
    extras_start = property(_LibreDWG.Dwg_Header_extras_start_get, _LibreDWG.Dwg_Header_extras_start_set)
    extras_size = property(_LibreDWG.Dwg_Header_extras_size_get, _LibreDWG.Dwg_Header_extras_size_set)
    codepage = property(_LibreDWG.Dwg_Header_codepage_get, _LibreDWG.Dwg_Header_codepage_set)
    unknown_0 = property(_LibreDWG.Dwg_Header_unknown_0_get, _LibreDWG.Dwg_Header_unknown_0_set)
    app_dwg_version = property(_LibreDWG.Dwg_Header_app_dwg_version_get, _LibreDWG.Dwg_Header_app_dwg_version_set)
    app_maint_version = property(_LibreDWG.Dwg_Header_app_maint_version_get, _LibreDWG.Dwg_Header_app_maint_version_set)
    security_type = property(_LibreDWG.Dwg_Header_security_type_get, _LibreDWG.Dwg_Header_security_type_set)
    rl_1c_address = property(_LibreDWG.Dwg_Header_rl_1c_address_get, _LibreDWG.Dwg_Header_rl_1c_address_set)
    summaryinfo_address = property(_LibreDWG.Dwg_Header_summaryinfo_address_get, _LibreDWG.Dwg_Header_summaryinfo_address_set)
    vbaproj_address = property(_LibreDWG.Dwg_Header_vbaproj_address_get, _LibreDWG.Dwg_Header_vbaproj_address_set)
    r2004_header_address = property(_LibreDWG.Dwg_Header_r2004_header_address_get, _LibreDWG.Dwg_Header_r2004_header_address_set)
    sections = property(_LibreDWG.Dwg_Header_sections_get, _LibreDWG.Dwg_Header_sections_set)
    num_sections = property(_LibreDWG.Dwg_Header_num_sections_get, _LibreDWG.Dwg_Header_num_sections_set)
    section = property(_LibreDWG.Dwg_Header_section_get, _LibreDWG.Dwg_Header_section_set)
    section_infohdr = property(_LibreDWG.Dwg_Header_section_infohdr_get, _LibreDWG.Dwg_Header_section_infohdr_set)
    section_info = property(_LibreDWG.Dwg_Header_section_info_get, _LibreDWG.Dwg_Header_section_info_set)

    def __init__(self):
        _LibreDWG.Dwg_Header_swiginit(self, _LibreDWG.new_Dwg_Header())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Header

# Register Dwg_Header in _LibreDWG:
_LibreDWG.Dwg_Header_swigregister(Dwg_Header)
class Dwg_R2007_Header(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    header_size = property(_LibreDWG.Dwg_R2007_Header_header_size_get, _LibreDWG.Dwg_R2007_Header_header_size_set)
    file_size = property(_LibreDWG.Dwg_R2007_Header_file_size_get, _LibreDWG.Dwg_R2007_Header_file_size_set)
    pages_map_crc_compressed = property(_LibreDWG.Dwg_R2007_Header_pages_map_crc_compressed_get, _LibreDWG.Dwg_R2007_Header_pages_map_crc_compressed_set)
    pages_map_correction = property(_LibreDWG.Dwg_R2007_Header_pages_map_correction_get, _LibreDWG.Dwg_R2007_Header_pages_map_correction_set)
    pages_map_crc_seed = property(_LibreDWG.Dwg_R2007_Header_pages_map_crc_seed_get, _LibreDWG.Dwg_R2007_Header_pages_map_crc_seed_set)
    pages_map2_offset = property(_LibreDWG.Dwg_R2007_Header_pages_map2_offset_get, _LibreDWG.Dwg_R2007_Header_pages_map2_offset_set)
    pages_map2_id = property(_LibreDWG.Dwg_R2007_Header_pages_map2_id_get, _LibreDWG.Dwg_R2007_Header_pages_map2_id_set)
    pages_map_offset = property(_LibreDWG.Dwg_R2007_Header_pages_map_offset_get, _LibreDWG.Dwg_R2007_Header_pages_map_offset_set)
    pages_map_id = property(_LibreDWG.Dwg_R2007_Header_pages_map_id_get, _LibreDWG.Dwg_R2007_Header_pages_map_id_set)
    header2_offset = property(_LibreDWG.Dwg_R2007_Header_header2_offset_get, _LibreDWG.Dwg_R2007_Header_header2_offset_set)
    pages_map_size_comp = property(_LibreDWG.Dwg_R2007_Header_pages_map_size_comp_get, _LibreDWG.Dwg_R2007_Header_pages_map_size_comp_set)
    pages_map_size_uncomp = property(_LibreDWG.Dwg_R2007_Header_pages_map_size_uncomp_get, _LibreDWG.Dwg_R2007_Header_pages_map_size_uncomp_set)
    pages_amount = property(_LibreDWG.Dwg_R2007_Header_pages_amount_get, _LibreDWG.Dwg_R2007_Header_pages_amount_set)
    pages_maxid = property(_LibreDWG.Dwg_R2007_Header_pages_maxid_get, _LibreDWG.Dwg_R2007_Header_pages_maxid_set)
    unknown1 = property(_LibreDWG.Dwg_R2007_Header_unknown1_get, _LibreDWG.Dwg_R2007_Header_unknown1_set)
    unknown2 = property(_LibreDWG.Dwg_R2007_Header_unknown2_get, _LibreDWG.Dwg_R2007_Header_unknown2_set)
    pages_map_crc_uncomp = property(_LibreDWG.Dwg_R2007_Header_pages_map_crc_uncomp_get, _LibreDWG.Dwg_R2007_Header_pages_map_crc_uncomp_set)
    unknown3 = property(_LibreDWG.Dwg_R2007_Header_unknown3_get, _LibreDWG.Dwg_R2007_Header_unknown3_set)
    unknown4 = property(_LibreDWG.Dwg_R2007_Header_unknown4_get, _LibreDWG.Dwg_R2007_Header_unknown4_set)
    unknown5 = property(_LibreDWG.Dwg_R2007_Header_unknown5_get, _LibreDWG.Dwg_R2007_Header_unknown5_set)
    num_sections = property(_LibreDWG.Dwg_R2007_Header_num_sections_get, _LibreDWG.Dwg_R2007_Header_num_sections_set)
    sections_map_crc_uncomp = property(_LibreDWG.Dwg_R2007_Header_sections_map_crc_uncomp_get, _LibreDWG.Dwg_R2007_Header_sections_map_crc_uncomp_set)
    sections_map_size_comp = property(_LibreDWG.Dwg_R2007_Header_sections_map_size_comp_get, _LibreDWG.Dwg_R2007_Header_sections_map_size_comp_set)
    sections_map2_id = property(_LibreDWG.Dwg_R2007_Header_sections_map2_id_get, _LibreDWG.Dwg_R2007_Header_sections_map2_id_set)
    sections_map_id = property(_LibreDWG.Dwg_R2007_Header_sections_map_id_get, _LibreDWG.Dwg_R2007_Header_sections_map_id_set)
    sections_map_size_uncomp = property(_LibreDWG.Dwg_R2007_Header_sections_map_size_uncomp_get, _LibreDWG.Dwg_R2007_Header_sections_map_size_uncomp_set)
    sections_map_crc_comp = property(_LibreDWG.Dwg_R2007_Header_sections_map_crc_comp_get, _LibreDWG.Dwg_R2007_Header_sections_map_crc_comp_set)
    sections_map_correction = property(_LibreDWG.Dwg_R2007_Header_sections_map_correction_get, _LibreDWG.Dwg_R2007_Header_sections_map_correction_set)
    sections_map_crc_seed = property(_LibreDWG.Dwg_R2007_Header_sections_map_crc_seed_get, _LibreDWG.Dwg_R2007_Header_sections_map_crc_seed_set)
    stream_version = property(_LibreDWG.Dwg_R2007_Header_stream_version_get, _LibreDWG.Dwg_R2007_Header_stream_version_set)
    crc_seed = property(_LibreDWG.Dwg_R2007_Header_crc_seed_get, _LibreDWG.Dwg_R2007_Header_crc_seed_set)
    crc_seed_encoded = property(_LibreDWG.Dwg_R2007_Header_crc_seed_encoded_get, _LibreDWG.Dwg_R2007_Header_crc_seed_encoded_set)
    random_seed = property(_LibreDWG.Dwg_R2007_Header_random_seed_get, _LibreDWG.Dwg_R2007_Header_random_seed_set)
    header_crc = property(_LibreDWG.Dwg_R2007_Header_header_crc_get, _LibreDWG.Dwg_R2007_Header_header_crc_set)

    def __init__(self):
        _LibreDWG.Dwg_R2007_Header_swiginit(self, _LibreDWG.new_Dwg_R2007_Header())
    __swig_destroy__ = _LibreDWG.delete_Dwg_R2007_Header

# Register Dwg_R2007_Header in _LibreDWG:
_LibreDWG.Dwg_R2007_Header_swigregister(Dwg_R2007_Header)
class Dwg_R2004_Header(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    file_ID_string = property(_LibreDWG.Dwg_R2004_Header_file_ID_string_get, _LibreDWG.Dwg_R2004_Header_file_ID_string_set)
    header_address = property(_LibreDWG.Dwg_R2004_Header_header_address_get, _LibreDWG.Dwg_R2004_Header_header_address_set)
    header_size = property(_LibreDWG.Dwg_R2004_Header_header_size_get, _LibreDWG.Dwg_R2004_Header_header_size_set)
    x04 = property(_LibreDWG.Dwg_R2004_Header_x04_get, _LibreDWG.Dwg_R2004_Header_x04_set)
    root_tree_node_gap = property(_LibreDWG.Dwg_R2004_Header_root_tree_node_gap_get, _LibreDWG.Dwg_R2004_Header_root_tree_node_gap_set)
    lowermost_left_tree_node_gap = property(_LibreDWG.Dwg_R2004_Header_lowermost_left_tree_node_gap_get, _LibreDWG.Dwg_R2004_Header_lowermost_left_tree_node_gap_set)
    lowermost_right_tree_node_gap = property(_LibreDWG.Dwg_R2004_Header_lowermost_right_tree_node_gap_get, _LibreDWG.Dwg_R2004_Header_lowermost_right_tree_node_gap_set)
    unknown_long = property(_LibreDWG.Dwg_R2004_Header_unknown_long_get, _LibreDWG.Dwg_R2004_Header_unknown_long_set)
    last_section_id = property(_LibreDWG.Dwg_R2004_Header_last_section_id_get, _LibreDWG.Dwg_R2004_Header_last_section_id_set)
    last_section_address = property(_LibreDWG.Dwg_R2004_Header_last_section_address_get, _LibreDWG.Dwg_R2004_Header_last_section_address_set)
    secondheader_address = property(_LibreDWG.Dwg_R2004_Header_secondheader_address_get, _LibreDWG.Dwg_R2004_Header_secondheader_address_set)
    numgaps = property(_LibreDWG.Dwg_R2004_Header_numgaps_get, _LibreDWG.Dwg_R2004_Header_numgaps_set)
    numsections = property(_LibreDWG.Dwg_R2004_Header_numsections_get, _LibreDWG.Dwg_R2004_Header_numsections_set)
    x20 = property(_LibreDWG.Dwg_R2004_Header_x20_get, _LibreDWG.Dwg_R2004_Header_x20_set)
    x80 = property(_LibreDWG.Dwg_R2004_Header_x80_get, _LibreDWG.Dwg_R2004_Header_x80_set)
    x40 = property(_LibreDWG.Dwg_R2004_Header_x40_get, _LibreDWG.Dwg_R2004_Header_x40_set)
    section_map_id = property(_LibreDWG.Dwg_R2004_Header_section_map_id_get, _LibreDWG.Dwg_R2004_Header_section_map_id_set)
    section_map_address = property(_LibreDWG.Dwg_R2004_Header_section_map_address_get, _LibreDWG.Dwg_R2004_Header_section_map_address_set)
    section_info_id = property(_LibreDWG.Dwg_R2004_Header_section_info_id_get, _LibreDWG.Dwg_R2004_Header_section_info_id_set)
    section_array_size = property(_LibreDWG.Dwg_R2004_Header_section_array_size_get, _LibreDWG.Dwg_R2004_Header_section_array_size_set)
    gap_array_size = property(_LibreDWG.Dwg_R2004_Header_gap_array_size_get, _LibreDWG.Dwg_R2004_Header_gap_array_size_set)
    crc32 = property(_LibreDWG.Dwg_R2004_Header_crc32_get, _LibreDWG.Dwg_R2004_Header_crc32_set)
    padding = property(_LibreDWG.Dwg_R2004_Header_padding_get, _LibreDWG.Dwg_R2004_Header_padding_set)
    section_type = property(_LibreDWG.Dwg_R2004_Header_section_type_get, _LibreDWG.Dwg_R2004_Header_section_type_set)
    decomp_data_size = property(_LibreDWG.Dwg_R2004_Header_decomp_data_size_get, _LibreDWG.Dwg_R2004_Header_decomp_data_size_set)
    comp_data_size = property(_LibreDWG.Dwg_R2004_Header_comp_data_size_get, _LibreDWG.Dwg_R2004_Header_comp_data_size_set)
    compression_type = property(_LibreDWG.Dwg_R2004_Header_compression_type_get, _LibreDWG.Dwg_R2004_Header_compression_type_set)
    checksum = property(_LibreDWG.Dwg_R2004_Header_checksum_get, _LibreDWG.Dwg_R2004_Header_checksum_set)

    def __init__(self):
        _LibreDWG.Dwg_R2004_Header_swiginit(self, _LibreDWG.new_Dwg_R2004_Header())
    __swig_destroy__ = _LibreDWG.delete_Dwg_R2004_Header

# Register Dwg_R2004_Header in _LibreDWG:
_LibreDWG.Dwg_R2004_Header_swigregister(Dwg_R2004_Header)
class Dwg_AuxHeader(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    aux_intro = property(_LibreDWG.Dwg_AuxHeader_aux_intro_get, _LibreDWG.Dwg_AuxHeader_aux_intro_set)
    dwg_version = property(_LibreDWG.Dwg_AuxHeader_dwg_version_get, _LibreDWG.Dwg_AuxHeader_dwg_version_set)
    maint_version = property(_LibreDWG.Dwg_AuxHeader_maint_version_get, _LibreDWG.Dwg_AuxHeader_maint_version_set)
    numsaves = property(_LibreDWG.Dwg_AuxHeader_numsaves_get, _LibreDWG.Dwg_AuxHeader_numsaves_set)
    minus_1 = property(_LibreDWG.Dwg_AuxHeader_minus_1_get, _LibreDWG.Dwg_AuxHeader_minus_1_set)
    numsaves_1 = property(_LibreDWG.Dwg_AuxHeader_numsaves_1_get, _LibreDWG.Dwg_AuxHeader_numsaves_1_set)
    numsaves_2 = property(_LibreDWG.Dwg_AuxHeader_numsaves_2_get, _LibreDWG.Dwg_AuxHeader_numsaves_2_set)
    zero = property(_LibreDWG.Dwg_AuxHeader_zero_get, _LibreDWG.Dwg_AuxHeader_zero_set)
    dwg_version_1 = property(_LibreDWG.Dwg_AuxHeader_dwg_version_1_get, _LibreDWG.Dwg_AuxHeader_dwg_version_1_set)
    maint_version_1 = property(_LibreDWG.Dwg_AuxHeader_maint_version_1_get, _LibreDWG.Dwg_AuxHeader_maint_version_1_set)
    dwg_version_2 = property(_LibreDWG.Dwg_AuxHeader_dwg_version_2_get, _LibreDWG.Dwg_AuxHeader_dwg_version_2_set)
    maint_version_2 = property(_LibreDWG.Dwg_AuxHeader_maint_version_2_get, _LibreDWG.Dwg_AuxHeader_maint_version_2_set)
    unknown_6rs = property(_LibreDWG.Dwg_AuxHeader_unknown_6rs_get, _LibreDWG.Dwg_AuxHeader_unknown_6rs_set)
    unknown_5rl = property(_LibreDWG.Dwg_AuxHeader_unknown_5rl_get, _LibreDWG.Dwg_AuxHeader_unknown_5rl_set)
    TDCREATE = property(_LibreDWG.Dwg_AuxHeader_TDCREATE_get, _LibreDWG.Dwg_AuxHeader_TDCREATE_set)
    TDUPDATE = property(_LibreDWG.Dwg_AuxHeader_TDUPDATE_get, _LibreDWG.Dwg_AuxHeader_TDUPDATE_set)
    HANDSEED = property(_LibreDWG.Dwg_AuxHeader_HANDSEED_get, _LibreDWG.Dwg_AuxHeader_HANDSEED_set)
    plot_stamp = property(_LibreDWG.Dwg_AuxHeader_plot_stamp_get, _LibreDWG.Dwg_AuxHeader_plot_stamp_set)
    zero_1 = property(_LibreDWG.Dwg_AuxHeader_zero_1_get, _LibreDWG.Dwg_AuxHeader_zero_1_set)
    numsaves_3 = property(_LibreDWG.Dwg_AuxHeader_numsaves_3_get, _LibreDWG.Dwg_AuxHeader_numsaves_3_set)
    zero_2 = property(_LibreDWG.Dwg_AuxHeader_zero_2_get, _LibreDWG.Dwg_AuxHeader_zero_2_set)
    zero_3 = property(_LibreDWG.Dwg_AuxHeader_zero_3_get, _LibreDWG.Dwg_AuxHeader_zero_3_set)
    zero_4 = property(_LibreDWG.Dwg_AuxHeader_zero_4_get, _LibreDWG.Dwg_AuxHeader_zero_4_set)
    numsaves_4 = property(_LibreDWG.Dwg_AuxHeader_numsaves_4_get, _LibreDWG.Dwg_AuxHeader_numsaves_4_set)
    zero_5 = property(_LibreDWG.Dwg_AuxHeader_zero_5_get, _LibreDWG.Dwg_AuxHeader_zero_5_set)
    zero_6 = property(_LibreDWG.Dwg_AuxHeader_zero_6_get, _LibreDWG.Dwg_AuxHeader_zero_6_set)
    zero_7 = property(_LibreDWG.Dwg_AuxHeader_zero_7_get, _LibreDWG.Dwg_AuxHeader_zero_7_set)
    zero_8 = property(_LibreDWG.Dwg_AuxHeader_zero_8_get, _LibreDWG.Dwg_AuxHeader_zero_8_set)
    zero_18 = property(_LibreDWG.Dwg_AuxHeader_zero_18_get, _LibreDWG.Dwg_AuxHeader_zero_18_set)
    num_auxheader_variables = property(_LibreDWG.Dwg_AuxHeader_num_auxheader_variables_get, _LibreDWG.Dwg_AuxHeader_num_auxheader_variables_set)
    auxheader_size = property(_LibreDWG.Dwg_AuxHeader_auxheader_size_get, _LibreDWG.Dwg_AuxHeader_auxheader_size_set)
    entities_start = property(_LibreDWG.Dwg_AuxHeader_entities_start_get, _LibreDWG.Dwg_AuxHeader_entities_start_set)
    entities_end = property(_LibreDWG.Dwg_AuxHeader_entities_end_get, _LibreDWG.Dwg_AuxHeader_entities_end_set)
    blocks_start = property(_LibreDWG.Dwg_AuxHeader_blocks_start_get, _LibreDWG.Dwg_AuxHeader_blocks_start_set)
    extras_start = property(_LibreDWG.Dwg_AuxHeader_extras_start_get, _LibreDWG.Dwg_AuxHeader_extras_start_set)
    auxheader_address = property(_LibreDWG.Dwg_AuxHeader_auxheader_address_get, _LibreDWG.Dwg_AuxHeader_auxheader_address_set)
    num_aux_tables = property(_LibreDWG.Dwg_AuxHeader_num_aux_tables_get, _LibreDWG.Dwg_AuxHeader_num_aux_tables_set)
    R11_HANDLING = property(_LibreDWG.Dwg_AuxHeader_R11_HANDLING_get, _LibreDWG.Dwg_AuxHeader_R11_HANDLING_set)

    def __init__(self):
        _LibreDWG.Dwg_AuxHeader_swiginit(self, _LibreDWG.new_Dwg_AuxHeader())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AuxHeader

# Register Dwg_AuxHeader in _LibreDWG:
_LibreDWG.Dwg_AuxHeader_swigregister(Dwg_AuxHeader)
class Dwg_SummaryInfo(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TITLE = property(_LibreDWG.Dwg_SummaryInfo_TITLE_get, _LibreDWG.Dwg_SummaryInfo_TITLE_set)
    SUBJECT = property(_LibreDWG.Dwg_SummaryInfo_SUBJECT_get, _LibreDWG.Dwg_SummaryInfo_SUBJECT_set)
    AUTHOR = property(_LibreDWG.Dwg_SummaryInfo_AUTHOR_get, _LibreDWG.Dwg_SummaryInfo_AUTHOR_set)
    KEYWORDS = property(_LibreDWG.Dwg_SummaryInfo_KEYWORDS_get, _LibreDWG.Dwg_SummaryInfo_KEYWORDS_set)
    COMMENTS = property(_LibreDWG.Dwg_SummaryInfo_COMMENTS_get, _LibreDWG.Dwg_SummaryInfo_COMMENTS_set)
    LASTSAVEDBY = property(_LibreDWG.Dwg_SummaryInfo_LASTSAVEDBY_get, _LibreDWG.Dwg_SummaryInfo_LASTSAVEDBY_set)
    REVISIONNUMBER = property(_LibreDWG.Dwg_SummaryInfo_REVISIONNUMBER_get, _LibreDWG.Dwg_SummaryInfo_REVISIONNUMBER_set)
    HYPERLINKBASE = property(_LibreDWG.Dwg_SummaryInfo_HYPERLINKBASE_get, _LibreDWG.Dwg_SummaryInfo_HYPERLINKBASE_set)
    TDINDWG = property(_LibreDWG.Dwg_SummaryInfo_TDINDWG_get, _LibreDWG.Dwg_SummaryInfo_TDINDWG_set)
    TDCREATE = property(_LibreDWG.Dwg_SummaryInfo_TDCREATE_get, _LibreDWG.Dwg_SummaryInfo_TDCREATE_set)
    TDUPDATE = property(_LibreDWG.Dwg_SummaryInfo_TDUPDATE_get, _LibreDWG.Dwg_SummaryInfo_TDUPDATE_set)
    num_props = property(_LibreDWG.Dwg_SummaryInfo_num_props_get, _LibreDWG.Dwg_SummaryInfo_num_props_set)
    props = property(_LibreDWG.Dwg_SummaryInfo_props_get, _LibreDWG.Dwg_SummaryInfo_props_set)
    unknown1 = property(_LibreDWG.Dwg_SummaryInfo_unknown1_get, _LibreDWG.Dwg_SummaryInfo_unknown1_set)
    unknown2 = property(_LibreDWG.Dwg_SummaryInfo_unknown2_get, _LibreDWG.Dwg_SummaryInfo_unknown2_set)

    def __init__(self):
        _LibreDWG.Dwg_SummaryInfo_swiginit(self, _LibreDWG.new_Dwg_SummaryInfo())
    __swig_destroy__ = _LibreDWG.delete_Dwg_SummaryInfo

# Register Dwg_SummaryInfo in _LibreDWG:
_LibreDWG.Dwg_SummaryInfo_swigregister(Dwg_SummaryInfo)
class Dwg_AppInfo(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    size = property(_LibreDWG.Dwg_AppInfo_size_get, _LibreDWG.Dwg_AppInfo_size_set)
    unknown_bits = property(_LibreDWG.Dwg_AppInfo_unknown_bits_get, _LibreDWG.Dwg_AppInfo_unknown_bits_set)
    class_version = property(_LibreDWG.Dwg_AppInfo_class_version_get, _LibreDWG.Dwg_AppInfo_class_version_set)
    num_strings = property(_LibreDWG.Dwg_AppInfo_num_strings_get, _LibreDWG.Dwg_AppInfo_num_strings_set)
    appinfo_name = property(_LibreDWG.Dwg_AppInfo_appinfo_name_get, _LibreDWG.Dwg_AppInfo_appinfo_name_set)
    version_checksum = property(_LibreDWG.Dwg_AppInfo_version_checksum_get, _LibreDWG.Dwg_AppInfo_version_checksum_set)
    comment_checksum = property(_LibreDWG.Dwg_AppInfo_comment_checksum_get, _LibreDWG.Dwg_AppInfo_comment_checksum_set)
    product_checksum = property(_LibreDWG.Dwg_AppInfo_product_checksum_get, _LibreDWG.Dwg_AppInfo_product_checksum_set)
    version = property(_LibreDWG.Dwg_AppInfo_version_get, _LibreDWG.Dwg_AppInfo_version_set)
    comment = property(_LibreDWG.Dwg_AppInfo_comment_get, _LibreDWG.Dwg_AppInfo_comment_set)
    product_info = property(_LibreDWG.Dwg_AppInfo_product_info_get, _LibreDWG.Dwg_AppInfo_product_info_set)

    def __init__(self):
        _LibreDWG.Dwg_AppInfo_swiginit(self, _LibreDWG.new_Dwg_AppInfo())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AppInfo

# Register Dwg_AppInfo in _LibreDWG:
_LibreDWG.Dwg_AppInfo_swigregister(Dwg_AppInfo)
class Dwg_FileDepList(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num_features = property(_LibreDWG.Dwg_FileDepList_num_features_get, _LibreDWG.Dwg_FileDepList_num_features_set)
    features = property(_LibreDWG.Dwg_FileDepList_features_get, _LibreDWG.Dwg_FileDepList_features_set)
    num_files = property(_LibreDWG.Dwg_FileDepList_num_files_get, _LibreDWG.Dwg_FileDepList_num_files_set)
    files = property(_LibreDWG.Dwg_FileDepList_files_get, _LibreDWG.Dwg_FileDepList_files_set)

    def __init__(self):
        _LibreDWG.Dwg_FileDepList_swiginit(self, _LibreDWG.new_Dwg_FileDepList())
    __swig_destroy__ = _LibreDWG.delete_Dwg_FileDepList

# Register Dwg_FileDepList in _LibreDWG:
_LibreDWG.Dwg_FileDepList_swigregister(Dwg_FileDepList)
class Dwg_Security(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    unknown_1 = property(_LibreDWG.Dwg_Security_unknown_1_get, _LibreDWG.Dwg_Security_unknown_1_set)
    unknown_2 = property(_LibreDWG.Dwg_Security_unknown_2_get, _LibreDWG.Dwg_Security_unknown_2_set)
    unknown_3 = property(_LibreDWG.Dwg_Security_unknown_3_get, _LibreDWG.Dwg_Security_unknown_3_set)
    crypto_id = property(_LibreDWG.Dwg_Security_crypto_id_get, _LibreDWG.Dwg_Security_crypto_id_set)
    crypto_name = property(_LibreDWG.Dwg_Security_crypto_name_get, _LibreDWG.Dwg_Security_crypto_name_set)
    algo_id = property(_LibreDWG.Dwg_Security_algo_id_get, _LibreDWG.Dwg_Security_algo_id_set)
    key_len = property(_LibreDWG.Dwg_Security_key_len_get, _LibreDWG.Dwg_Security_key_len_set)
    encr_size = property(_LibreDWG.Dwg_Security_encr_size_get, _LibreDWG.Dwg_Security_encr_size_set)
    encr_buffer = property(_LibreDWG.Dwg_Security_encr_buffer_get, _LibreDWG.Dwg_Security_encr_buffer_set)

    def __init__(self):
        _LibreDWG.Dwg_Security_swiginit(self, _LibreDWG.new_Dwg_Security())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Security

# Register Dwg_Security in _LibreDWG:
_LibreDWG.Dwg_Security_swigregister(Dwg_Security)
class Dwg_VBAProject(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    size = property(_LibreDWG.Dwg_VBAProject_size_get, _LibreDWG.Dwg_VBAProject_size_set)
    unknown_bits = property(_LibreDWG.Dwg_VBAProject_unknown_bits_get, _LibreDWG.Dwg_VBAProject_unknown_bits_set)

    def __init__(self):
        _LibreDWG.Dwg_VBAProject_swiginit(self, _LibreDWG.new_Dwg_VBAProject())
    __swig_destroy__ = _LibreDWG.delete_Dwg_VBAProject

# Register Dwg_VBAProject in _LibreDWG:
_LibreDWG.Dwg_VBAProject_swigregister(Dwg_VBAProject)
class Dwg_AppInfoHistory(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    size = property(_LibreDWG.Dwg_AppInfoHistory_size_get, _LibreDWG.Dwg_AppInfoHistory_size_set)
    unknown_bits = property(_LibreDWG.Dwg_AppInfoHistory_unknown_bits_get, _LibreDWG.Dwg_AppInfoHistory_unknown_bits_set)

    def __init__(self):
        _LibreDWG.Dwg_AppInfoHistory_swiginit(self, _LibreDWG.new_Dwg_AppInfoHistory())
    __swig_destroy__ = _LibreDWG.delete_Dwg_AppInfoHistory

# Register Dwg_AppInfoHistory in _LibreDWG:
_LibreDWG.Dwg_AppInfoHistory_swigregister(Dwg_AppInfoHistory)
class Dwg_RevHistory(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    class_version = property(_LibreDWG.Dwg_RevHistory_class_version_get, _LibreDWG.Dwg_RevHistory_class_version_set)
    class_minor = property(_LibreDWG.Dwg_RevHistory_class_minor_get, _LibreDWG.Dwg_RevHistory_class_minor_set)
    num_histories = property(_LibreDWG.Dwg_RevHistory_num_histories_get, _LibreDWG.Dwg_RevHistory_num_histories_set)
    histories = property(_LibreDWG.Dwg_RevHistory_histories_get, _LibreDWG.Dwg_RevHistory_histories_set)

    def __init__(self):
        _LibreDWG.Dwg_RevHistory_swiginit(self, _LibreDWG.new_Dwg_RevHistory())
    __swig_destroy__ = _LibreDWG.delete_Dwg_RevHistory

# Register Dwg_RevHistory in _LibreDWG:
_LibreDWG.Dwg_RevHistory_swigregister(Dwg_RevHistory)
class Dwg_ObjFreeSpace(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    zero = property(_LibreDWG.Dwg_ObjFreeSpace_zero_get, _LibreDWG.Dwg_ObjFreeSpace_zero_set)
    numhandles = property(_LibreDWG.Dwg_ObjFreeSpace_numhandles_get, _LibreDWG.Dwg_ObjFreeSpace_numhandles_set)
    TDUPDATE = property(_LibreDWG.Dwg_ObjFreeSpace_TDUPDATE_get, _LibreDWG.Dwg_ObjFreeSpace_TDUPDATE_set)
    objects_address = property(_LibreDWG.Dwg_ObjFreeSpace_objects_address_get, _LibreDWG.Dwg_ObjFreeSpace_objects_address_set)
    numnums = property(_LibreDWG.Dwg_ObjFreeSpace_numnums_get, _LibreDWG.Dwg_ObjFreeSpace_numnums_set)
    max32 = property(_LibreDWG.Dwg_ObjFreeSpace_max32_get, _LibreDWG.Dwg_ObjFreeSpace_max32_set)
    max64 = property(_LibreDWG.Dwg_ObjFreeSpace_max64_get, _LibreDWG.Dwg_ObjFreeSpace_max64_set)
    maxtbl = property(_LibreDWG.Dwg_ObjFreeSpace_maxtbl_get, _LibreDWG.Dwg_ObjFreeSpace_maxtbl_set)
    maxrl = property(_LibreDWG.Dwg_ObjFreeSpace_maxrl_get, _LibreDWG.Dwg_ObjFreeSpace_maxrl_set)
    max32_hi = property(_LibreDWG.Dwg_ObjFreeSpace_max32_hi_get, _LibreDWG.Dwg_ObjFreeSpace_max32_hi_set)
    max64_hi = property(_LibreDWG.Dwg_ObjFreeSpace_max64_hi_get, _LibreDWG.Dwg_ObjFreeSpace_max64_hi_set)
    maxtbl_hi = property(_LibreDWG.Dwg_ObjFreeSpace_maxtbl_hi_get, _LibreDWG.Dwg_ObjFreeSpace_maxtbl_hi_set)
    maxrl_hi = property(_LibreDWG.Dwg_ObjFreeSpace_maxrl_hi_get, _LibreDWG.Dwg_ObjFreeSpace_maxrl_hi_set)

    def __init__(self):
        _LibreDWG.Dwg_ObjFreeSpace_swiginit(self, _LibreDWG.new_Dwg_ObjFreeSpace())
    __swig_destroy__ = _LibreDWG.delete_Dwg_ObjFreeSpace

# Register Dwg_ObjFreeSpace in _LibreDWG:
_LibreDWG.Dwg_ObjFreeSpace_swigregister(Dwg_ObjFreeSpace)
class Dwg_Template(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    description = property(_LibreDWG.Dwg_Template_description_get, _LibreDWG.Dwg_Template_description_set)
    MEASUREMENT = property(_LibreDWG.Dwg_Template_MEASUREMENT_get, _LibreDWG.Dwg_Template_MEASUREMENT_set)

    def __init__(self):
        _LibreDWG.Dwg_Template_swiginit(self, _LibreDWG.new_Dwg_Template())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Template

# Register Dwg_Template in _LibreDWG:
_LibreDWG.Dwg_Template_swigregister(Dwg_Template)
class Dwg_SecondHeader_Sections(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    nr = property(_LibreDWG.Dwg_SecondHeader_Sections_nr_get, _LibreDWG.Dwg_SecondHeader_Sections_nr_set)
    address = property(_LibreDWG.Dwg_SecondHeader_Sections_address_get, _LibreDWG.Dwg_SecondHeader_Sections_address_set)
    size = property(_LibreDWG.Dwg_SecondHeader_Sections_size_get, _LibreDWG.Dwg_SecondHeader_Sections_size_set)

    def __init__(self):
        _LibreDWG.Dwg_SecondHeader_Sections_swiginit(self, _LibreDWG.new_Dwg_SecondHeader_Sections())
    __swig_destroy__ = _LibreDWG.delete_Dwg_SecondHeader_Sections

# Register Dwg_SecondHeader_Sections in _LibreDWG:
_LibreDWG.Dwg_SecondHeader_Sections_swigregister(Dwg_SecondHeader_Sections)
class Dwg_SecondHeader_Handles(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num_hdl = property(_LibreDWG.Dwg_SecondHeader_Handles_num_hdl_get, _LibreDWG.Dwg_SecondHeader_Handles_num_hdl_set)
    nr = property(_LibreDWG.Dwg_SecondHeader_Handles_nr_get, _LibreDWG.Dwg_SecondHeader_Handles_nr_set)
    hdl = property(_LibreDWG.Dwg_SecondHeader_Handles_hdl_get, _LibreDWG.Dwg_SecondHeader_Handles_hdl_set)
    name = property(_LibreDWG.Dwg_SecondHeader_Handles_name_get, _LibreDWG.Dwg_SecondHeader_Handles_name_set)

    def __init__(self):
        _LibreDWG.Dwg_SecondHeader_Handles_swiginit(self, _LibreDWG.new_Dwg_SecondHeader_Handles())
    __swig_destroy__ = _LibreDWG.delete_Dwg_SecondHeader_Handles

# Register Dwg_SecondHeader_Handles in _LibreDWG:
_LibreDWG.Dwg_SecondHeader_Handles_swigregister(Dwg_SecondHeader_Handles)
class Dwg_SecondHeader(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    size = property(_LibreDWG.Dwg_SecondHeader_size_get, _LibreDWG.Dwg_SecondHeader_size_set)
    address = property(_LibreDWG.Dwg_SecondHeader_address_get, _LibreDWG.Dwg_SecondHeader_address_set)
    version = property(_LibreDWG.Dwg_SecondHeader_version_get, _LibreDWG.Dwg_SecondHeader_version_set)
    maint_rel_version = property(_LibreDWG.Dwg_SecondHeader_maint_rel_version_get, _LibreDWG.Dwg_SecondHeader_maint_rel_version_set)
    zero_one_or_three = property(_LibreDWG.Dwg_SecondHeader_zero_one_or_three_get, _LibreDWG.Dwg_SecondHeader_zero_one_or_three_set)
    dwg_versions = property(_LibreDWG.Dwg_SecondHeader_dwg_versions_get, _LibreDWG.Dwg_SecondHeader_dwg_versions_set)
    codepage = property(_LibreDWG.Dwg_SecondHeader_codepage_get, _LibreDWG.Dwg_SecondHeader_codepage_set)
    num_sections = property(_LibreDWG.Dwg_SecondHeader_num_sections_get, _LibreDWG.Dwg_SecondHeader_num_sections_set)
    sections = property(_LibreDWG.Dwg_SecondHeader_sections_get, _LibreDWG.Dwg_SecondHeader_sections_set)
    num_handles = property(_LibreDWG.Dwg_SecondHeader_num_handles_get, _LibreDWG.Dwg_SecondHeader_num_handles_set)
    handles = property(_LibreDWG.Dwg_SecondHeader_handles_get, _LibreDWG.Dwg_SecondHeader_handles_set)
    crc = property(_LibreDWG.Dwg_SecondHeader_crc_get, _LibreDWG.Dwg_SecondHeader_crc_set)
    junk_r14 = property(_LibreDWG.Dwg_SecondHeader_junk_r14_get, _LibreDWG.Dwg_SecondHeader_junk_r14_set)

    def __init__(self):
        _LibreDWG.Dwg_SecondHeader_swiginit(self, _LibreDWG.new_Dwg_SecondHeader())
    __swig_destroy__ = _LibreDWG.delete_Dwg_SecondHeader

# Register Dwg_SecondHeader in _LibreDWG:
_LibreDWG.Dwg_SecondHeader_swigregister(Dwg_SecondHeader)
class Dwg_Data(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    header = property(_LibreDWG.Dwg_Data_header_get, _LibreDWG.Dwg_Data_header_set)
    num_classes = property(_LibreDWG.Dwg_Data_num_classes_get, _LibreDWG.Dwg_Data_num_classes_set)
    dwg_class = property(_LibreDWG.Dwg_Data_dwg_class_get, _LibreDWG.Dwg_Data_dwg_class_set)
    num_objects = property(_LibreDWG.Dwg_Data_num_objects_get, _LibreDWG.Dwg_Data_num_objects_set)
    num_alloced_objects = property(_LibreDWG.Dwg_Data_num_alloced_objects_get, _LibreDWG.Dwg_Data_num_alloced_objects_set)
    object = property(_LibreDWG.Dwg_Data_object_get, _LibreDWG.Dwg_Data_object_set)
    num_entities = property(_LibreDWG.Dwg_Data_num_entities_get, _LibreDWG.Dwg_Data_num_entities_set)
    num_object_refs = property(_LibreDWG.Dwg_Data_num_object_refs_get, _LibreDWG.Dwg_Data_num_object_refs_set)
    cur_index = property(_LibreDWG.Dwg_Data_cur_index_get, _LibreDWG.Dwg_Data_cur_index_set)
    last_viewport_id = property(_LibreDWG.Dwg_Data_last_viewport_id_get, _LibreDWG.Dwg_Data_last_viewport_id_set)
    object_ref = property(_LibreDWG.Dwg_Data_object_ref_get, _LibreDWG.Dwg_Data_object_ref_set)
    object_map = property(_LibreDWG.Dwg_Data_object_map_get, _LibreDWG.Dwg_Data_object_map_set)
    dirty_refs = property(_LibreDWG.Dwg_Data_dirty_refs_get, _LibreDWG.Dwg_Data_dirty_refs_set)
    opts = property(_LibreDWG.Dwg_Data_opts_get, _LibreDWG.Dwg_Data_opts_set)
    header_vars = property(_LibreDWG.Dwg_Data_header_vars_get, _LibreDWG.Dwg_Data_header_vars_set)
    thumbnail = property(_LibreDWG.Dwg_Data_thumbnail_get, _LibreDWG.Dwg_Data_thumbnail_set)
    fhdr = property(_LibreDWG.Dwg_Data_fhdr_get)
    mspace_block = property(_LibreDWG.Dwg_Data_mspace_block_get, _LibreDWG.Dwg_Data_mspace_block_set)
    pspace_block = property(_LibreDWG.Dwg_Data_pspace_block_get, _LibreDWG.Dwg_Data_pspace_block_set)
    block_control = property(_LibreDWG.Dwg_Data_block_control_get, _LibreDWG.Dwg_Data_block_control_set)
    auxheader = property(_LibreDWG.Dwg_Data_auxheader_get, _LibreDWG.Dwg_Data_auxheader_set)
    secondheader = property(_LibreDWG.Dwg_Data_secondheader_get, _LibreDWG.Dwg_Data_secondheader_set)
    summaryinfo = property(_LibreDWG.Dwg_Data_summaryinfo_get, _LibreDWG.Dwg_Data_summaryinfo_set)
    appinfo = property(_LibreDWG.Dwg_Data_appinfo_get, _LibreDWG.Dwg_Data_appinfo_set)
    filedeplist = property(_LibreDWG.Dwg_Data_filedeplist_get, _LibreDWG.Dwg_Data_filedeplist_set)
    security = property(_LibreDWG.Dwg_Data_security_get, _LibreDWG.Dwg_Data_security_set)
    vbaproject = property(_LibreDWG.Dwg_Data_vbaproject_get, _LibreDWG.Dwg_Data_vbaproject_set)
    appinfohistory = property(_LibreDWG.Dwg_Data_appinfohistory_get, _LibreDWG.Dwg_Data_appinfohistory_set)
    revhistory = property(_LibreDWG.Dwg_Data_revhistory_get, _LibreDWG.Dwg_Data_revhistory_set)
    objfreespace = property(_LibreDWG.Dwg_Data_objfreespace_get, _LibreDWG.Dwg_Data_objfreespace_set)
    Template = property(_LibreDWG.Dwg_Data_Template_get, _LibreDWG.Dwg_Data_Template_set)
    acds = property(_LibreDWG.Dwg_Data_acds_get, _LibreDWG.Dwg_Data_acds_set)
    layout_type = property(_LibreDWG.Dwg_Data_layout_type_get, _LibreDWG.Dwg_Data_layout_type_set)
    num_acis_sab_hdl = property(_LibreDWG.Dwg_Data_num_acis_sab_hdl_get, _LibreDWG.Dwg_Data_num_acis_sab_hdl_set)
    acis_sab_hdl = property(_LibreDWG.Dwg_Data_acis_sab_hdl_get, _LibreDWG.Dwg_Data_acis_sab_hdl_set)
    next_hdl = property(_LibreDWG.Dwg_Data_next_hdl_get, _LibreDWG.Dwg_Data_next_hdl_set)
    prev_entity_index = property(_LibreDWG.Dwg_Data_prev_entity_index_get, _LibreDWG.Dwg_Data_prev_entity_index_set)
    object_ordered_ref = property(_LibreDWG.Dwg_Data_object_ordered_ref_get, _LibreDWG.Dwg_Data_object_ordered_ref_set)
    num_object_ordered_refs = property(_LibreDWG.Dwg_Data_num_object_ordered_refs_get, _LibreDWG.Dwg_Data_num_object_ordered_refs_set)

    def __init__(self):
        _LibreDWG.Dwg_Data_swiginit(self, _LibreDWG.new_Dwg_Data())
    __swig_destroy__ = _LibreDWG.delete_Dwg_Data

# Register Dwg_Data in _LibreDWG:
_LibreDWG.Dwg_Data_swigregister(Dwg_Data)
class _dwg_struct_fhdr(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    r2004_header = property(_LibreDWG._dwg_struct_fhdr_r2004_header_get, _LibreDWG._dwg_struct_fhdr_r2004_header_set)
    r2007_file_header = property(_LibreDWG._dwg_struct_fhdr_r2007_file_header_get, _LibreDWG._dwg_struct_fhdr_r2007_file_header_set)

    def __init__(self):
        _LibreDWG._dwg_struct_fhdr_swiginit(self, _LibreDWG.new__dwg_struct_fhdr())
    __swig_destroy__ = _LibreDWG.delete__dwg_struct_fhdr

# Register _dwg_struct_fhdr in _LibreDWG:
_LibreDWG._dwg_struct_fhdr_swigregister(_dwg_struct_fhdr)
DWG_OPTS_LOGLEVEL = _LibreDWG.DWG_OPTS_LOGLEVEL
DWG_OPTS_MINIMAL = _LibreDWG.DWG_OPTS_MINIMAL
DWG_OPTS_DXFB = _LibreDWG.DWG_OPTS_DXFB
DWG_OPTS_JSONFIRST = _LibreDWG.DWG_OPTS_JSONFIRST
DWG_OPTS_INDXF = _LibreDWG.DWG_OPTS_INDXF
DWG_OPTS_INJSON = _LibreDWG.DWG_OPTS_INJSON
DWG_OPTS_IN = _LibreDWG.DWG_OPTS_IN
DWG_VT_INVALID = _LibreDWG.DWG_VT_INVALID
DWG_VT_STRING = _LibreDWG.DWG_VT_STRING
DWG_VT_POINT3D = _LibreDWG.DWG_VT_POINT3D
DWG_VT_REAL = _LibreDWG.DWG_VT_REAL
DWG_VT_INT16 = _LibreDWG.DWG_VT_INT16
DWG_VT_INT32 = _LibreDWG.DWG_VT_INT32
DWG_VT_INT8 = _LibreDWG.DWG_VT_INT8
DWG_VT_BINARY = _LibreDWG.DWG_VT_BINARY
DWG_VT_HANDLE = _LibreDWG.DWG_VT_HANDLE
DWG_VT_OBJECTID = _LibreDWG.DWG_VT_OBJECTID
DWG_VT_BOOL = _LibreDWG.DWG_VT_BOOL
DWG_VT_INT64 = _LibreDWG.DWG_VT_INT64

def dwg_read_file(filename, dwg):
    return _LibreDWG.dwg_read_file(filename, dwg)

def dxf_read_file(filename, dwg):
    return _LibreDWG.dxf_read_file(filename, dwg)

def dwg_write_file(filename, dwg):
    return _LibreDWG.dwg_write_file(filename, dwg)

def dwg_bmp(dwg, size, typep):
    return _LibreDWG.dwg_bmp(dwg, size, typep)

def dxf_cvt_lweight(value):
    return _LibreDWG.dxf_cvt_lweight(value)

def dxf_revcvt_lweight(lw):
    return _LibreDWG.dxf_revcvt_lweight(lw)

def dwg_ctrl_table(dwg, table):
    return _LibreDWG.dwg_ctrl_table(dwg, table)

def dwg_find_tablehandle(dwg, name, table):
    return _LibreDWG.dwg_find_tablehandle(dwg, name, table)

def dwg_find_tablehandle_index(dwg, index, table):
    return _LibreDWG.dwg_find_tablehandle_index(dwg, index, table)

def dwg_handle_name(dwg, table, handle):
    return _LibreDWG.dwg_handle_name(dwg, table, handle)

def dwg_find_table_control(dwg, table):
    return _LibreDWG.dwg_find_table_control(dwg, table)

def dwg_find_dictionary(dwg, name):
    return _LibreDWG.dwg_find_dictionary(dwg, name)

def dwg_find_dicthandle(dwg, dict, name):
    return _LibreDWG.dwg_find_dicthandle(dwg, dict, name)

def dwg_find_dicthandle_objname(dwg, dict, name):
    return _LibreDWG.dwg_find_dicthandle_objname(dwg, dict, name)

def dwg_find_table_extname(dwg, obj):
    return _LibreDWG.dwg_find_table_extname(dwg, obj)

def dwg_variable_dict(dwg, name):
    return _LibreDWG.dwg_variable_dict(dwg, name)

def dwg_model_x_min(arg1):
    return _LibreDWG.dwg_model_x_min(arg1)

def dwg_model_x_max(arg1):
    return _LibreDWG.dwg_model_x_max(arg1)

def dwg_model_y_min(arg1):
    return _LibreDWG.dwg_model_y_min(arg1)

def dwg_model_y_max(arg1):
    return _LibreDWG.dwg_model_y_max(arg1)

def dwg_model_z_min(arg1):
    return _LibreDWG.dwg_model_z_min(arg1)

def dwg_model_z_max(arg1):
    return _LibreDWG.dwg_model_z_max(arg1)

def dwg_page_x_min(arg1):
    return _LibreDWG.dwg_page_x_min(arg1)

def dwg_page_x_max(arg1):
    return _LibreDWG.dwg_page_x_max(arg1)

def dwg_page_y_min(arg1):
    return _LibreDWG.dwg_page_y_min(arg1)

def dwg_page_y_max(arg1):
    return _LibreDWG.dwg_page_y_max(arg1)

def dwg_block_control(dwg):
    return _LibreDWG.dwg_block_control(dwg)

def dwg_model_space_ref(dwg):
    return _LibreDWG.dwg_model_space_ref(dwg)

def dwg_paper_space_ref(dwg):
    return _LibreDWG.dwg_paper_space_ref(dwg)

def dwg_model_space_object(dwg):
    return _LibreDWG.dwg_model_space_object(dwg)

def dwg_paper_space_object(dwg):
    return _LibreDWG.dwg_paper_space_object(dwg)

def dwg_get_layer_count(dwg):
    return _LibreDWG.dwg_get_layer_count(dwg)

def dwg_get_layers(dwg):
    return _LibreDWG.dwg_get_layers(dwg)

def dwg_get_num_objects(dwg):
    return _LibreDWG.dwg_get_num_objects(dwg)

def dwg_get_object_num_objects(dwg):
    return _LibreDWG.dwg_get_object_num_objects(dwg)

def dwg_class_is_entity(klass):
    return _LibreDWG.dwg_class_is_entity(klass)

def dwg_obj_is_control(obj):
    return _LibreDWG.dwg_obj_is_control(obj)

def dwg_obj_is_table(obj):
    return _LibreDWG.dwg_obj_is_table(obj)

def dwg_obj_is_subentity(obj):
    return _LibreDWG.dwg_obj_is_subentity(obj)

def dwg_obj_has_subentity(obj):
    return _LibreDWG.dwg_obj_has_subentity(obj)

def dwg_obj_is_3dsolid(obj):
    return _LibreDWG.dwg_obj_is_3dsolid(obj)

def dwg_obj_is_acsh(obj):
    return _LibreDWG.dwg_obj_is_acsh(obj)

def dwg_get_num_entities(arg1):
    return _LibreDWG.dwg_get_num_entities(arg1)

def dwg_get_entities(arg1):
    return _LibreDWG.dwg_get_entities(arg1)

def dwg_get_entity_layer(arg1):
    return _LibreDWG.dwg_get_entity_layer(arg1)

def dwg_next_object(obj):
    return _LibreDWG.dwg_next_object(obj)

def dwg_next_entity(obj):
    return _LibreDWG.dwg_next_entity(obj)

def dwg_next_handle(dwg):
    return _LibreDWG.dwg_next_handle(dwg)

def dwg_next_handseed(dwg):
    return _LibreDWG.dwg_next_handseed(dwg)

def dwg_ref_object(dwg, ref):
    return _LibreDWG.dwg_ref_object(dwg, ref)

def dwg_ref_object_relative(dwg, ref, obj):
    return _LibreDWG.dwg_ref_object_relative(dwg, ref, obj)

def dwg_ref_object_silent(dwg, ref):
    return _LibreDWG.dwg_ref_object_silent(dwg, ref)

def get_first_owned_entity(hdr):
    return _LibreDWG.get_first_owned_entity(hdr)

def get_next_owned_entity(hdr, current):
    return _LibreDWG.get_next_owned_entity(hdr, current)

def get_first_owned_subentity(owner):
    return _LibreDWG.get_first_owned_subentity(owner)

def get_next_owned_subentity(owner, current):
    return _LibreDWG.get_next_owned_subentity(owner, current)

def get_first_owned_block(hdr):
    return _LibreDWG.get_first_owned_block(hdr)

def get_last_owned_block(hdr):
    return _LibreDWG.get_last_owned_block(hdr)

def get_next_owned_block(hdr, current):
    return _LibreDWG.get_next_owned_block(hdr, current)

def get_next_owned_block_entity(hdr, current):
    return _LibreDWG.get_next_owned_block_entity(hdr, current)

def dwg_get_first_object(dwg, type):
    return _LibreDWG.dwg_get_first_object(dwg, type)

def dwg_get_next_object(dwg, type, index):
    return _LibreDWG.dwg_get_next_object(dwg, type, index)

def dwg_resolve_handle(dwg, absref):
    return _LibreDWG.dwg_resolve_handle(dwg, absref)

def dwg_resolve_handle_silent(dwg, absref):
    return _LibreDWG.dwg_resolve_handle_silent(dwg, absref)

def dwg_resolve_handleref(ref, obj):
    return _LibreDWG.dwg_resolve_handleref(ref, obj)

def dwg_resolve_jump(obj):
    return _LibreDWG.dwg_resolve_jump(obj)

def dwg_section_type(name):
    return _LibreDWG.dwg_section_type(name)

def dwg_section_wtype(wname):
    return _LibreDWG.dwg_section_wtype(wname)

def dwg_section_name(dwg, sec_id):
    return _LibreDWG.dwg_section_name(dwg, sec_id)

def dwg_resbuf_value_type(gc):
    return _LibreDWG.dwg_resbuf_value_type(gc)

def dwg_free(dwg):
    return _LibreDWG.dwg_free(dwg)

def dwg_free_object(obj):
    return _LibreDWG.dwg_free_object(obj)

def dwg_new_ref(dwg):
    return _LibreDWG.dwg_new_ref(dwg)

def dwg_add_handle(hdl, code, value, obj):
    return _LibreDWG.dwg_add_handle(hdl, code, value, obj)

def dwg_add_handleref(dwg, code, value, obj):
    return _LibreDWG.dwg_add_handleref(dwg, code, value, obj)

def dwg_dup_handleref(dwg, ref):
    return _LibreDWG.dwg_dup_handleref(dwg, ref)

def dwg_add_handleref_free(code, absref):
    return _LibreDWG.dwg_add_handleref_free(code, absref)

def dwg_version_type(version):
    return _LibreDWG.dwg_version_type(version)

def dwg_version_as(version):
    return _LibreDWG.dwg_version_as(version)

def dwg_version_hdr_type(hdr):
    return _LibreDWG.dwg_version_hdr_type(hdr)

def dwg_supports_eed(dwg):
    return _LibreDWG.dwg_supports_eed(dwg)

def dwg_supports_obj(dwg, obj):
    return _LibreDWG.dwg_supports_obj(dwg, obj)

def dwg_errstrings(error):
    return _LibreDWG.dwg_errstrings(error)

def dwg_encrypt_SAT1(blocksize, acis_data, acis_data_offset):
    return _LibreDWG.dwg_encrypt_SAT1(blocksize, acis_data, acis_data_offset)

def dwg_convert_SAB_to_SAT1(_obj):
    return _LibreDWG.dwg_convert_SAB_to_SAT1(_obj)
class Dwg_RGB_Palette(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    r = property(_LibreDWG.Dwg_RGB_Palette_r_get, _LibreDWG.Dwg_RGB_Palette_r_set)
    g = property(_LibreDWG.Dwg_RGB_Palette_g_get, _LibreDWG.Dwg_RGB_Palette_g_set)
    b = property(_LibreDWG.Dwg_RGB_Palette_b_get, _LibreDWG.Dwg_RGB_Palette_b_set)

    def __init__(self):
        _LibreDWG.Dwg_RGB_Palette_swiginit(self, _LibreDWG.new_Dwg_RGB_Palette())
    __swig_destroy__ = _LibreDWG.delete_Dwg_RGB_Palette

# Register Dwg_RGB_Palette in _LibreDWG:
_LibreDWG.Dwg_RGB_Palette_swigregister(Dwg_RGB_Palette)

def dwg_rgb_palette():
    return _LibreDWG.dwg_rgb_palette()

def dwg_rgb_palette_index(index):
    return _LibreDWG.dwg_rgb_palette_index(index)

def dwg_find_color_index(rgb):
    return _LibreDWG.dwg_find_color_index(rgb)

def dwg_add_object(dwg):
    return _LibreDWG.dwg_add_object(dwg)

def dwg_object_name(name, dxfnamep, typep, is_entp, stabilityp):
    return _LibreDWG.dwg_object_name(name, dxfnamep, typep, is_entp, stabilityp)

def dwg_setup__3DFACE(obj):
    return _LibreDWG.dwg_setup__3DFACE(obj)

def dwg_setup__3DSOLID(obj):
    return _LibreDWG.dwg_setup__3DSOLID(obj)

def dwg_setup_ARC(obj):
    return _LibreDWG.dwg_setup_ARC(obj)

def dwg_setup_ATTDEF(obj):
    return _LibreDWG.dwg_setup_ATTDEF(obj)

def dwg_setup_ATTRIB(obj):
    return _LibreDWG.dwg_setup_ATTRIB(obj)

def dwg_setup_BLOCK(obj):
    return _LibreDWG.dwg_setup_BLOCK(obj)

def dwg_setup_BODY(obj):
    return _LibreDWG.dwg_setup_BODY(obj)

def dwg_setup_CIRCLE(obj):
    return _LibreDWG.dwg_setup_CIRCLE(obj)

def dwg_setup_DIMENSION_ALIGNED(obj):
    return _LibreDWG.dwg_setup_DIMENSION_ALIGNED(obj)

def dwg_setup_DIMENSION_ANG2LN(obj):
    return _LibreDWG.dwg_setup_DIMENSION_ANG2LN(obj)

def dwg_setup_DIMENSION_ANG3PT(obj):
    return _LibreDWG.dwg_setup_DIMENSION_ANG3PT(obj)

def dwg_setup_DIMENSION_DIAMETER(obj):
    return _LibreDWG.dwg_setup_DIMENSION_DIAMETER(obj)

def dwg_setup_DIMENSION_LINEAR(obj):
    return _LibreDWG.dwg_setup_DIMENSION_LINEAR(obj)

def dwg_setup_DIMENSION_ORDINATE(obj):
    return _LibreDWG.dwg_setup_DIMENSION_ORDINATE(obj)

def dwg_setup_DIMENSION_RADIUS(obj):
    return _LibreDWG.dwg_setup_DIMENSION_RADIUS(obj)

def dwg_setup_ELLIPSE(obj):
    return _LibreDWG.dwg_setup_ELLIPSE(obj)

def dwg_setup_ENDBLK(obj):
    return _LibreDWG.dwg_setup_ENDBLK(obj)

def dwg_setup_INSERT(obj):
    return _LibreDWG.dwg_setup_INSERT(obj)

def dwg_setup_LEADER(obj):
    return _LibreDWG.dwg_setup_LEADER(obj)

def dwg_setup_LINE(obj):
    return _LibreDWG.dwg_setup_LINE(obj)

def dwg_setup_LOAD(obj):
    return _LibreDWG.dwg_setup_LOAD(obj)

def dwg_setup_MINSERT(obj):
    return _LibreDWG.dwg_setup_MINSERT(obj)

def dwg_setup_MLINE(obj):
    return _LibreDWG.dwg_setup_MLINE(obj)

def dwg_setup_MTEXT(obj):
    return _LibreDWG.dwg_setup_MTEXT(obj)

def dwg_setup_OLEFRAME(obj):
    return _LibreDWG.dwg_setup_OLEFRAME(obj)

def dwg_setup_POINT(obj):
    return _LibreDWG.dwg_setup_POINT(obj)

def dwg_setup_POLYLINE_2D(obj):
    return _LibreDWG.dwg_setup_POLYLINE_2D(obj)

def dwg_setup_POLYLINE_3D(obj):
    return _LibreDWG.dwg_setup_POLYLINE_3D(obj)

def dwg_setup_POLYLINE_MESH(obj):
    return _LibreDWG.dwg_setup_POLYLINE_MESH(obj)

def dwg_setup_POLYLINE_PFACE(obj):
    return _LibreDWG.dwg_setup_POLYLINE_PFACE(obj)

def dwg_setup_PROXY_ENTITY(obj):
    return _LibreDWG.dwg_setup_PROXY_ENTITY(obj)

def dwg_setup_RAY(obj):
    return _LibreDWG.dwg_setup_RAY(obj)

def dwg_setup_REGION(obj):
    return _LibreDWG.dwg_setup_REGION(obj)

def dwg_setup_SEQEND(obj):
    return _LibreDWG.dwg_setup_SEQEND(obj)

def dwg_setup_SHAPE(obj):
    return _LibreDWG.dwg_setup_SHAPE(obj)

def dwg_setup_SOLID(obj):
    return _LibreDWG.dwg_setup_SOLID(obj)

def dwg_setup_SPLINE(obj):
    return _LibreDWG.dwg_setup_SPLINE(obj)

def dwg_setup_TEXT(obj):
    return _LibreDWG.dwg_setup_TEXT(obj)

def dwg_setup_TOLERANCE(obj):
    return _LibreDWG.dwg_setup_TOLERANCE(obj)

def dwg_setup_TRACE(obj):
    return _LibreDWG.dwg_setup_TRACE(obj)

def dwg_setup_UNKNOWN_ENT(obj):
    return _LibreDWG.dwg_setup_UNKNOWN_ENT(obj)

def dwg_setup_VERTEX_2D(obj):
    return _LibreDWG.dwg_setup_VERTEX_2D(obj)

def dwg_setup_VERTEX_3D(obj):
    return _LibreDWG.dwg_setup_VERTEX_3D(obj)

def dwg_setup_VERTEX_MESH(obj):
    return _LibreDWG.dwg_setup_VERTEX_MESH(obj)

def dwg_setup_VERTEX_PFACE(obj):
    return _LibreDWG.dwg_setup_VERTEX_PFACE(obj)

def dwg_setup_VERTEX_PFACE_FACE(obj):
    return _LibreDWG.dwg_setup_VERTEX_PFACE_FACE(obj)

def dwg_setup_VIEWPORT(obj):
    return _LibreDWG.dwg_setup_VIEWPORT(obj)

def dwg_setup_XLINE(obj):
    return _LibreDWG.dwg_setup_XLINE(obj)

def dwg_setup_APPID(obj):
    return _LibreDWG.dwg_setup_APPID(obj)

def dwg_setup_APPID_CONTROL(obj):
    return _LibreDWG.dwg_setup_APPID_CONTROL(obj)

def dwg_setup_BLOCK_CONTROL(obj):
    return _LibreDWG.dwg_setup_BLOCK_CONTROL(obj)

def dwg_setup_BLOCK_HEADER(obj):
    return _LibreDWG.dwg_setup_BLOCK_HEADER(obj)

def dwg_setup_DICTIONARY(obj):
    return _LibreDWG.dwg_setup_DICTIONARY(obj)

def dwg_setup_DIMSTYLE(obj):
    return _LibreDWG.dwg_setup_DIMSTYLE(obj)

def dwg_setup_DIMSTYLE_CONTROL(obj):
    return _LibreDWG.dwg_setup_DIMSTYLE_CONTROL(obj)

def dwg_setup_DUMMY(obj):
    return _LibreDWG.dwg_setup_DUMMY(obj)

def dwg_setup_LAYER(obj):
    return _LibreDWG.dwg_setup_LAYER(obj)

def dwg_setup_LAYER_CONTROL(obj):
    return _LibreDWG.dwg_setup_LAYER_CONTROL(obj)

def dwg_setup_LONG_TRANSACTION(obj):
    return _LibreDWG.dwg_setup_LONG_TRANSACTION(obj)

def dwg_setup_LTYPE(obj):
    return _LibreDWG.dwg_setup_LTYPE(obj)

def dwg_setup_LTYPE_CONTROL(obj):
    return _LibreDWG.dwg_setup_LTYPE_CONTROL(obj)

def dwg_setup_MLINESTYLE(obj):
    return _LibreDWG.dwg_setup_MLINESTYLE(obj)

def dwg_setup_STYLE(obj):
    return _LibreDWG.dwg_setup_STYLE(obj)

def dwg_setup_STYLE_CONTROL(obj):
    return _LibreDWG.dwg_setup_STYLE_CONTROL(obj)

def dwg_setup_UCS(obj):
    return _LibreDWG.dwg_setup_UCS(obj)

def dwg_setup_UCS_CONTROL(obj):
    return _LibreDWG.dwg_setup_UCS_CONTROL(obj)

def dwg_setup_UNKNOWN_OBJ(obj):
    return _LibreDWG.dwg_setup_UNKNOWN_OBJ(obj)

def dwg_setup_VIEW(obj):
    return _LibreDWG.dwg_setup_VIEW(obj)

def dwg_setup_VIEW_CONTROL(obj):
    return _LibreDWG.dwg_setup_VIEW_CONTROL(obj)

def dwg_setup_VPORT(obj):
    return _LibreDWG.dwg_setup_VPORT(obj)

def dwg_setup_VPORT_CONTROL(obj):
    return _LibreDWG.dwg_setup_VPORT_CONTROL(obj)

def dwg_setup_VX_CONTROL(obj):
    return _LibreDWG.dwg_setup_VX_CONTROL(obj)

def dwg_setup_VX_TABLE_RECORD(obj):
    return _LibreDWG.dwg_setup_VX_TABLE_RECORD(obj)

def dwg_setup__3DLINE(obj):
    return _LibreDWG.dwg_setup__3DLINE(obj)

def dwg_setup_CAMERA(obj):
    return _LibreDWG.dwg_setup_CAMERA(obj)

def dwg_setup_DGNUNDERLAY(obj):
    return _LibreDWG.dwg_setup_DGNUNDERLAY(obj)

def dwg_setup_DWFUNDERLAY(obj):
    return _LibreDWG.dwg_setup_DWFUNDERLAY(obj)

def dwg_setup_ENDREP(obj):
    return _LibreDWG.dwg_setup_ENDREP(obj)

def dwg_setup_HATCH(obj):
    return _LibreDWG.dwg_setup_HATCH(obj)

def dwg_setup_IMAGE(obj):
    return _LibreDWG.dwg_setup_IMAGE(obj)

def dwg_setup_JUMP(obj):
    return _LibreDWG.dwg_setup_JUMP(obj)

def dwg_setup_LIGHT(obj):
    return _LibreDWG.dwg_setup_LIGHT(obj)

def dwg_setup_LWPOLYLINE(obj):
    return _LibreDWG.dwg_setup_LWPOLYLINE(obj)

def dwg_setup_MESH(obj):
    return _LibreDWG.dwg_setup_MESH(obj)

def dwg_setup_MULTILEADER(obj):
    return _LibreDWG.dwg_setup_MULTILEADER(obj)

def dwg_setup_OLE2FRAME(obj):
    return _LibreDWG.dwg_setup_OLE2FRAME(obj)

def dwg_setup_PDFUNDERLAY(obj):
    return _LibreDWG.dwg_setup_PDFUNDERLAY(obj)

def dwg_setup_REPEAT(obj):
    return _LibreDWG.dwg_setup_REPEAT(obj)

def dwg_setup_SECTIONOBJECT(obj):
    return _LibreDWG.dwg_setup_SECTIONOBJECT(obj)

def dwg_setup_WIPEOUT(obj):
    return _LibreDWG.dwg_setup_WIPEOUT(obj)

def dwg_setup_ACSH_BOOLEAN_CLASS(obj):
    return _LibreDWG.dwg_setup_ACSH_BOOLEAN_CLASS(obj)

def dwg_setup_ACSH_BOX_CLASS(obj):
    return _LibreDWG.dwg_setup_ACSH_BOX_CLASS(obj)

def dwg_setup_ACSH_CONE_CLASS(obj):
    return _LibreDWG.dwg_setup_ACSH_CONE_CLASS(obj)

def dwg_setup_ACSH_CYLINDER_CLASS(obj):
    return _LibreDWG.dwg_setup_ACSH_CYLINDER_CLASS(obj)

def dwg_setup_ACSH_FILLET_CLASS(obj):
    return _LibreDWG.dwg_setup_ACSH_FILLET_CLASS(obj)

def dwg_setup_ACSH_HISTORY_CLASS(obj):
    return _LibreDWG.dwg_setup_ACSH_HISTORY_CLASS(obj)

def dwg_setup_ACSH_SPHERE_CLASS(obj):
    return _LibreDWG.dwg_setup_ACSH_SPHERE_CLASS(obj)

def dwg_setup_ACSH_TORUS_CLASS(obj):
    return _LibreDWG.dwg_setup_ACSH_TORUS_CLASS(obj)

def dwg_setup_ACSH_WEDGE_CLASS(obj):
    return _LibreDWG.dwg_setup_ACSH_WEDGE_CLASS(obj)

def dwg_setup_ASSOCGEOMDEPENDENCY(obj):
    return _LibreDWG.dwg_setup_ASSOCGEOMDEPENDENCY(obj)

def dwg_setup_ASSOCNETWORK(obj):
    return _LibreDWG.dwg_setup_ASSOCNETWORK(obj)

def dwg_setup_BLOCKALIGNMENTGRIP(obj):
    return _LibreDWG.dwg_setup_BLOCKALIGNMENTGRIP(obj)

def dwg_setup_BLOCKALIGNMENTPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKALIGNMENTPARAMETER(obj)

def dwg_setup_BLOCKBASEPOINTPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKBASEPOINTPARAMETER(obj)

def dwg_setup_BLOCKFLIPACTION(obj):
    return _LibreDWG.dwg_setup_BLOCKFLIPACTION(obj)

def dwg_setup_BLOCKFLIPGRIP(obj):
    return _LibreDWG.dwg_setup_BLOCKFLIPGRIP(obj)

def dwg_setup_BLOCKFLIPPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKFLIPPARAMETER(obj)

def dwg_setup_BLOCKGRIPLOCATIONCOMPONENT(obj):
    return _LibreDWG.dwg_setup_BLOCKGRIPLOCATIONCOMPONENT(obj)

def dwg_setup_BLOCKLINEARGRIP(obj):
    return _LibreDWG.dwg_setup_BLOCKLINEARGRIP(obj)

def dwg_setup_BLOCKLOOKUPGRIP(obj):
    return _LibreDWG.dwg_setup_BLOCKLOOKUPGRIP(obj)

def dwg_setup_BLOCKMOVEACTION(obj):
    return _LibreDWG.dwg_setup_BLOCKMOVEACTION(obj)

def dwg_setup_BLOCKROTATEACTION(obj):
    return _LibreDWG.dwg_setup_BLOCKROTATEACTION(obj)

def dwg_setup_BLOCKROTATIONGRIP(obj):
    return _LibreDWG.dwg_setup_BLOCKROTATIONGRIP(obj)

def dwg_setup_BLOCKSCALEACTION(obj):
    return _LibreDWG.dwg_setup_BLOCKSCALEACTION(obj)

def dwg_setup_BLOCKVISIBILITYGRIP(obj):
    return _LibreDWG.dwg_setup_BLOCKVISIBILITYGRIP(obj)

def dwg_setup_CELLSTYLEMAP(obj):
    return _LibreDWG.dwg_setup_CELLSTYLEMAP(obj)

def dwg_setup_DETAILVIEWSTYLE(obj):
    return _LibreDWG.dwg_setup_DETAILVIEWSTYLE(obj)

def dwg_setup_DICTIONARYVAR(obj):
    return _LibreDWG.dwg_setup_DICTIONARYVAR(obj)

def dwg_setup_DICTIONARYWDFLT(obj):
    return _LibreDWG.dwg_setup_DICTIONARYWDFLT(obj)

def dwg_setup_DYNAMICBLOCKPURGEPREVENTER(obj):
    return _LibreDWG.dwg_setup_DYNAMICBLOCKPURGEPREVENTER(obj)

def dwg_setup_FIELD(obj):
    return _LibreDWG.dwg_setup_FIELD(obj)

def dwg_setup_FIELDLIST(obj):
    return _LibreDWG.dwg_setup_FIELDLIST(obj)

def dwg_setup_GEODATA(obj):
    return _LibreDWG.dwg_setup_GEODATA(obj)

def dwg_setup_GROUP(obj):
    return _LibreDWG.dwg_setup_GROUP(obj)

def dwg_setup_IDBUFFER(obj):
    return _LibreDWG.dwg_setup_IDBUFFER(obj)

def dwg_setup_IMAGEDEF(obj):
    return _LibreDWG.dwg_setup_IMAGEDEF(obj)

def dwg_setup_IMAGEDEF_REACTOR(obj):
    return _LibreDWG.dwg_setup_IMAGEDEF_REACTOR(obj)

def dwg_setup_INDEX(obj):
    return _LibreDWG.dwg_setup_INDEX(obj)

def dwg_setup_LAYERFILTER(obj):
    return _LibreDWG.dwg_setup_LAYERFILTER(obj)

def dwg_setup_LAYER_INDEX(obj):
    return _LibreDWG.dwg_setup_LAYER_INDEX(obj)

def dwg_setup_LAYOUT(obj):
    return _LibreDWG.dwg_setup_LAYOUT(obj)

def dwg_setup_PLACEHOLDER(obj):
    return _LibreDWG.dwg_setup_PLACEHOLDER(obj)

def dwg_setup_PLOTSETTINGS(obj):
    return _LibreDWG.dwg_setup_PLOTSETTINGS(obj)

def dwg_setup_RASTERVARIABLES(obj):
    return _LibreDWG.dwg_setup_RASTERVARIABLES(obj)

def dwg_setup_RENDERENVIRONMENT(obj):
    return _LibreDWG.dwg_setup_RENDERENVIRONMENT(obj)

def dwg_setup_SCALE(obj):
    return _LibreDWG.dwg_setup_SCALE(obj)

def dwg_setup_SECTIONVIEWSTYLE(obj):
    return _LibreDWG.dwg_setup_SECTIONVIEWSTYLE(obj)

def dwg_setup_SECTION_MANAGER(obj):
    return _LibreDWG.dwg_setup_SECTION_MANAGER(obj)

def dwg_setup_SORTENTSTABLE(obj):
    return _LibreDWG.dwg_setup_SORTENTSTABLE(obj)

def dwg_setup_SPATIAL_FILTER(obj):
    return _LibreDWG.dwg_setup_SPATIAL_FILTER(obj)

def dwg_setup_SUN(obj):
    return _LibreDWG.dwg_setup_SUN(obj)

def dwg_setup_TABLEGEOMETRY(obj):
    return _LibreDWG.dwg_setup_TABLEGEOMETRY(obj)

def dwg_setup_VBA_PROJECT(obj):
    return _LibreDWG.dwg_setup_VBA_PROJECT(obj)

def dwg_setup_VISUALSTYLE(obj):
    return _LibreDWG.dwg_setup_VISUALSTYLE(obj)

def dwg_setup_WIPEOUTVARIABLES(obj):
    return _LibreDWG.dwg_setup_WIPEOUTVARIABLES(obj)

def dwg_setup_XRECORD(obj):
    return _LibreDWG.dwg_setup_XRECORD(obj)

def dwg_setup_PDFDEFINITION(obj):
    return _LibreDWG.dwg_setup_PDFDEFINITION(obj)

def dwg_setup_DGNDEFINITION(obj):
    return _LibreDWG.dwg_setup_DGNDEFINITION(obj)

def dwg_setup_DWFDEFINITION(obj):
    return _LibreDWG.dwg_setup_DWFDEFINITION(obj)

def dwg_setup_ARC_DIMENSION(obj):
    return _LibreDWG.dwg_setup_ARC_DIMENSION(obj)

def dwg_setup_HELIX(obj):
    return _LibreDWG.dwg_setup_HELIX(obj)

def dwg_setup_LARGE_RADIAL_DIMENSION(obj):
    return _LibreDWG.dwg_setup_LARGE_RADIAL_DIMENSION(obj)

def dwg_setup_LAYOUTPRINTCONFIG(obj):
    return _LibreDWG.dwg_setup_LAYOUTPRINTCONFIG(obj)

def dwg_setup_PLANESURFACE(obj):
    return _LibreDWG.dwg_setup_PLANESURFACE(obj)

def dwg_setup_POINTCLOUD(obj):
    return _LibreDWG.dwg_setup_POINTCLOUD(obj)

def dwg_setup_POINTCLOUDEX(obj):
    return _LibreDWG.dwg_setup_POINTCLOUDEX(obj)

def dwg_setup_ACSH_BREP_CLASS(obj):
    return _LibreDWG.dwg_setup_ACSH_BREP_CLASS(obj)

def dwg_setup_ACSH_CHAMFER_CLASS(obj):
    return _LibreDWG.dwg_setup_ACSH_CHAMFER_CLASS(obj)

def dwg_setup_ACSH_PYRAMID_CLASS(obj):
    return _LibreDWG.dwg_setup_ACSH_PYRAMID_CLASS(obj)

def dwg_setup_ALDIMOBJECTCONTEXTDATA(obj):
    return _LibreDWG.dwg_setup_ALDIMOBJECTCONTEXTDATA(obj)

def dwg_setup_ASSOC2DCONSTRAINTGROUP(obj):
    return _LibreDWG.dwg_setup_ASSOC2DCONSTRAINTGROUP(obj)

def dwg_setup_ASSOCACTION(obj):
    return _LibreDWG.dwg_setup_ASSOCACTION(obj)

def dwg_setup_ASSOCACTIONPARAM(obj):
    return _LibreDWG.dwg_setup_ASSOCACTIONPARAM(obj)

def dwg_setup_ASSOCARRAYACTIONBODY(obj):
    return _LibreDWG.dwg_setup_ASSOCARRAYACTIONBODY(obj)

def dwg_setup_ASSOCASMBODYACTIONPARAM(obj):
    return _LibreDWG.dwg_setup_ASSOCASMBODYACTIONPARAM(obj)

def dwg_setup_ASSOCBLENDSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_setup_ASSOCBLENDSURFACEACTIONBODY(obj)

def dwg_setup_ASSOCCOMPOUNDACTIONPARAM(obj):
    return _LibreDWG.dwg_setup_ASSOCCOMPOUNDACTIONPARAM(obj)

def dwg_setup_ASSOCDEPENDENCY(obj):
    return _LibreDWG.dwg_setup_ASSOCDEPENDENCY(obj)

def dwg_setup_ASSOCDIMDEPENDENCYBODY(obj):
    return _LibreDWG.dwg_setup_ASSOCDIMDEPENDENCYBODY(obj)

def dwg_setup_ASSOCEXTENDSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_setup_ASSOCEXTENDSURFACEACTIONBODY(obj)

def dwg_setup_ASSOCEXTRUDEDSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_setup_ASSOCEXTRUDEDSURFACEACTIONBODY(obj)

def dwg_setup_ASSOCFACEACTIONPARAM(obj):
    return _LibreDWG.dwg_setup_ASSOCFACEACTIONPARAM(obj)

def dwg_setup_ASSOCFILLETSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_setup_ASSOCFILLETSURFACEACTIONBODY(obj)

def dwg_setup_ASSOCLOFTEDSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_setup_ASSOCLOFTEDSURFACEACTIONBODY(obj)

def dwg_setup_ASSOCNETWORKSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_setup_ASSOCNETWORKSURFACEACTIONBODY(obj)

def dwg_setup_ASSOCOBJECTACTIONPARAM(obj):
    return _LibreDWG.dwg_setup_ASSOCOBJECTACTIONPARAM(obj)

def dwg_setup_ASSOCOFFSETSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_setup_ASSOCOFFSETSURFACEACTIONBODY(obj)

def dwg_setup_ASSOCOSNAPPOINTREFACTIONPARAM(obj):
    return _LibreDWG.dwg_setup_ASSOCOSNAPPOINTREFACTIONPARAM(obj)

def dwg_setup_ASSOCPATCHSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_setup_ASSOCPATCHSURFACEACTIONBODY(obj)

def dwg_setup_ASSOCPATHACTIONPARAM(obj):
    return _LibreDWG.dwg_setup_ASSOCPATHACTIONPARAM(obj)

def dwg_setup_ASSOCPLANESURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_setup_ASSOCPLANESURFACEACTIONBODY(obj)

def dwg_setup_ASSOCPOINTREFACTIONPARAM(obj):
    return _LibreDWG.dwg_setup_ASSOCPOINTREFACTIONPARAM(obj)

def dwg_setup_ASSOCREVOLVEDSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_setup_ASSOCREVOLVEDSURFACEACTIONBODY(obj)

def dwg_setup_ASSOCTRIMSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_setup_ASSOCTRIMSURFACEACTIONBODY(obj)

def dwg_setup_ASSOCVALUEDEPENDENCY(obj):
    return _LibreDWG.dwg_setup_ASSOCVALUEDEPENDENCY(obj)

def dwg_setup_ASSOCVARIABLE(obj):
    return _LibreDWG.dwg_setup_ASSOCVARIABLE(obj)

def dwg_setup_ASSOCVERTEXACTIONPARAM(obj):
    return _LibreDWG.dwg_setup_ASSOCVERTEXACTIONPARAM(obj)

def dwg_setup_BLKREFOBJECTCONTEXTDATA(obj):
    return _LibreDWG.dwg_setup_BLKREFOBJECTCONTEXTDATA(obj)

def dwg_setup_BLOCKALIGNEDCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKALIGNEDCONSTRAINTPARAMETER(obj)

def dwg_setup_BLOCKANGULARCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKANGULARCONSTRAINTPARAMETER(obj)

def dwg_setup_BLOCKARRAYACTION(obj):
    return _LibreDWG.dwg_setup_BLOCKARRAYACTION(obj)

def dwg_setup_BLOCKDIAMETRICCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKDIAMETRICCONSTRAINTPARAMETER(obj)

def dwg_setup_BLOCKHORIZONTALCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKHORIZONTALCONSTRAINTPARAMETER(obj)

def dwg_setup_BLOCKLINEARCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKLINEARCONSTRAINTPARAMETER(obj)

def dwg_setup_BLOCKLINEARPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKLINEARPARAMETER(obj)

def dwg_setup_BLOCKLOOKUPACTION(obj):
    return _LibreDWG.dwg_setup_BLOCKLOOKUPACTION(obj)

def dwg_setup_BLOCKLOOKUPPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKLOOKUPPARAMETER(obj)

def dwg_setup_BLOCKPARAMDEPENDENCYBODY(obj):
    return _LibreDWG.dwg_setup_BLOCKPARAMDEPENDENCYBODY(obj)

def dwg_setup_BLOCKPOINTPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKPOINTPARAMETER(obj)

def dwg_setup_BLOCKPOLARGRIP(obj):
    return _LibreDWG.dwg_setup_BLOCKPOLARGRIP(obj)

def dwg_setup_BLOCKPOLARPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKPOLARPARAMETER(obj)

def dwg_setup_BLOCKPOLARSTRETCHACTION(obj):
    return _LibreDWG.dwg_setup_BLOCKPOLARSTRETCHACTION(obj)

def dwg_setup_BLOCKRADIALCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKRADIALCONSTRAINTPARAMETER(obj)

def dwg_setup_BLOCKREPRESENTATION(obj):
    return _LibreDWG.dwg_setup_BLOCKREPRESENTATION(obj)

def dwg_setup_BLOCKROTATIONPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKROTATIONPARAMETER(obj)

def dwg_setup_BLOCKSTRETCHACTION(obj):
    return _LibreDWG.dwg_setup_BLOCKSTRETCHACTION(obj)

def dwg_setup_BLOCKUSERPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKUSERPARAMETER(obj)

def dwg_setup_BLOCKVERTICALCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKVERTICALCONSTRAINTPARAMETER(obj)

def dwg_setup_BLOCKVISIBILITYPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKVISIBILITYPARAMETER(obj)

def dwg_setup_BLOCKXYGRIP(obj):
    return _LibreDWG.dwg_setup_BLOCKXYGRIP(obj)

def dwg_setup_BLOCKXYPARAMETER(obj):
    return _LibreDWG.dwg_setup_BLOCKXYPARAMETER(obj)

def dwg_setup_DATALINK(obj):
    return _LibreDWG.dwg_setup_DATALINK(obj)

def dwg_setup_DBCOLOR(obj):
    return _LibreDWG.dwg_setup_DBCOLOR(obj)

def dwg_setup_DIMASSOC(obj):
    return _LibreDWG.dwg_setup_DIMASSOC(obj)

def dwg_setup_EVALUATION_GRAPH(obj):
    return _LibreDWG.dwg_setup_EVALUATION_GRAPH(obj)

def dwg_setup_FCFOBJECTCONTEXTDATA(obj):
    return _LibreDWG.dwg_setup_FCFOBJECTCONTEXTDATA(obj)

def dwg_setup_GRADIENT_BACKGROUND(obj):
    return _LibreDWG.dwg_setup_GRADIENT_BACKGROUND(obj)

def dwg_setup_GROUND_PLANE_BACKGROUND(obj):
    return _LibreDWG.dwg_setup_GROUND_PLANE_BACKGROUND(obj)

def dwg_setup_IBL_BACKGROUND(obj):
    return _LibreDWG.dwg_setup_IBL_BACKGROUND(obj)

def dwg_setup_IMAGE_BACKGROUND(obj):
    return _LibreDWG.dwg_setup_IMAGE_BACKGROUND(obj)

def dwg_setup_LEADEROBJECTCONTEXTDATA(obj):
    return _LibreDWG.dwg_setup_LEADEROBJECTCONTEXTDATA(obj)

def dwg_setup_LIGHTLIST(obj):
    return _LibreDWG.dwg_setup_LIGHTLIST(obj)

def dwg_setup_MATERIAL(obj):
    return _LibreDWG.dwg_setup_MATERIAL(obj)

def dwg_setup_MENTALRAYRENDERSETTINGS(obj):
    return _LibreDWG.dwg_setup_MENTALRAYRENDERSETTINGS(obj)

def dwg_setup_MLEADERSTYLE(obj):
    return _LibreDWG.dwg_setup_MLEADERSTYLE(obj)

def dwg_setup_MTEXTOBJECTCONTEXTDATA(obj):
    return _LibreDWG.dwg_setup_MTEXTOBJECTCONTEXTDATA(obj)

def dwg_setup_OBJECT_PTR(obj):
    return _LibreDWG.dwg_setup_OBJECT_PTR(obj)

def dwg_setup_PARTIAL_VIEWING_INDEX(obj):
    return _LibreDWG.dwg_setup_PARTIAL_VIEWING_INDEX(obj)

def dwg_setup_POINTCLOUDCOLORMAP(obj):
    return _LibreDWG.dwg_setup_POINTCLOUDCOLORMAP(obj)

def dwg_setup_POINTCLOUDDEF(obj):
    return _LibreDWG.dwg_setup_POINTCLOUDDEF(obj)

def dwg_setup_POINTCLOUDDEFEX(obj):
    return _LibreDWG.dwg_setup_POINTCLOUDDEFEX(obj)

def dwg_setup_POINTCLOUDDEF_REACTOR(obj):
    return _LibreDWG.dwg_setup_POINTCLOUDDEF_REACTOR(obj)

def dwg_setup_POINTCLOUDDEF_REACTOR_EX(obj):
    return _LibreDWG.dwg_setup_POINTCLOUDDEF_REACTOR_EX(obj)

def dwg_setup_PROXY_OBJECT(obj):
    return _LibreDWG.dwg_setup_PROXY_OBJECT(obj)

def dwg_setup_RAPIDRTRENDERSETTINGS(obj):
    return _LibreDWG.dwg_setup_RAPIDRTRENDERSETTINGS(obj)

def dwg_setup_RENDERENTRY(obj):
    return _LibreDWG.dwg_setup_RENDERENTRY(obj)

def dwg_setup_RENDERGLOBAL(obj):
    return _LibreDWG.dwg_setup_RENDERGLOBAL(obj)

def dwg_setup_RENDERSETTINGS(obj):
    return _LibreDWG.dwg_setup_RENDERSETTINGS(obj)

def dwg_setup_SECTION_SETTINGS(obj):
    return _LibreDWG.dwg_setup_SECTION_SETTINGS(obj)

def dwg_setup_SKYLIGHT_BACKGROUND(obj):
    return _LibreDWG.dwg_setup_SKYLIGHT_BACKGROUND(obj)

def dwg_setup_SOLID_BACKGROUND(obj):
    return _LibreDWG.dwg_setup_SOLID_BACKGROUND(obj)

def dwg_setup_SPATIAL_INDEX(obj):
    return _LibreDWG.dwg_setup_SPATIAL_INDEX(obj)

def dwg_setup_TABLESTYLE(obj):
    return _LibreDWG.dwg_setup_TABLESTYLE(obj)

def dwg_setup_TEXTOBJECTCONTEXTDATA(obj):
    return _LibreDWG.dwg_setup_TEXTOBJECTCONTEXTDATA(obj)

def dwg_setup_ASSOCARRAYMODIFYPARAMETERS(obj):
    return _LibreDWG.dwg_setup_ASSOCARRAYMODIFYPARAMETERS(obj)

def dwg_setup_ASSOCARRAYPATHPARAMETERS(obj):
    return _LibreDWG.dwg_setup_ASSOCARRAYPATHPARAMETERS(obj)

def dwg_setup_ASSOCARRAYPOLARPARAMETERS(obj):
    return _LibreDWG.dwg_setup_ASSOCARRAYPOLARPARAMETERS(obj)

def dwg_setup_ASSOCARRAYRECTANGULARPARAMETERS(obj):
    return _LibreDWG.dwg_setup_ASSOCARRAYRECTANGULARPARAMETERS(obj)
class Dwg_DYNAPI_field(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    name = property(_LibreDWG.Dwg_DYNAPI_field_name_get)
    type = property(_LibreDWG.Dwg_DYNAPI_field_type_get)
    size = property(_LibreDWG.Dwg_DYNAPI_field_size_get)
    offset = property(_LibreDWG.Dwg_DYNAPI_field_offset_get)
    is_indirect = property(_LibreDWG.Dwg_DYNAPI_field_is_indirect_get)
    is_malloc = property(_LibreDWG.Dwg_DYNAPI_field_is_malloc_get)
    is_string = property(_LibreDWG.Dwg_DYNAPI_field_is_string_get)
    dxf = property(_LibreDWG.Dwg_DYNAPI_field_dxf_get)

    def __init__(self):
        _LibreDWG.Dwg_DYNAPI_field_swiginit(self, _LibreDWG.new_Dwg_DYNAPI_field())
    __swig_destroy__ = _LibreDWG.delete_Dwg_DYNAPI_field

# Register Dwg_DYNAPI_field in _LibreDWG:
_LibreDWG.Dwg_DYNAPI_field_swigregister(Dwg_DYNAPI_field)

def is_dwg_entity(name):
    return _LibreDWG.is_dwg_entity(name)

def is_dwg_object(name):
    return _LibreDWG.is_dwg_object(name)

def dwg_dynapi_header_value(dwg, fieldname, out, fp):
    return _LibreDWG.dwg_dynapi_header_value(dwg, fieldname, out, fp)

def dwg_dynapi_entity_value(entity, dxfname, fieldname, out, fp):
    return _LibreDWG.dwg_dynapi_entity_value(entity, dxfname, fieldname, out, fp)

def dwg_dynapi_common_value(_obj, fieldname, out, fp):
    return _LibreDWG.dwg_dynapi_common_value(_obj, fieldname, out, fp)

def dwg_dynapi_subclass_value(ptr, subclass, fieldname, out, fp):
    return _LibreDWG.dwg_dynapi_subclass_value(ptr, subclass, fieldname, out, fp)

def dwg_dynapi_header_utf8text(dwg, fieldname, textp, isnew, fp):
    return _LibreDWG.dwg_dynapi_header_utf8text(dwg, fieldname, textp, isnew, fp)

def dwg_dynapi_entity_utf8text(entity, name, fieldname, textp, isnew, fp):
    return _LibreDWG.dwg_dynapi_entity_utf8text(entity, name, fieldname, textp, isnew, fp)

def dwg_dynapi_common_utf8text(_obj, fieldname, textp, isnew, fp):
    return _LibreDWG.dwg_dynapi_common_utf8text(_obj, fieldname, textp, isnew, fp)

def dwg_dynapi_header_set_value(dwg, fieldname, value, is_utf8):
    return _LibreDWG.dwg_dynapi_header_set_value(dwg, fieldname, value, is_utf8)

def dwg_dynapi_entity_set_value(entity, dxfname, fieldname, value, is_utf8):
    return _LibreDWG.dwg_dynapi_entity_set_value(entity, dxfname, fieldname, value, is_utf8)

def dwg_dynapi_common_set_value(_obj, fieldname, value, is_utf8):
    return _LibreDWG.dwg_dynapi_common_set_value(_obj, fieldname, value, is_utf8)

def dwg_dynapi_handle_name(dwg, hdl, alloced):
    return _LibreDWG.dwg_dynapi_handle_name(dwg, hdl, alloced)

def dwg_dynapi_header_field(fieldname):
    return _LibreDWG.dwg_dynapi_header_field(fieldname)

def dwg_dynapi_entity_field(name, fieldname):
    return _LibreDWG.dwg_dynapi_entity_field(name, fieldname)

def dwg_dynapi_subclass_field(name, fieldname):
    return _LibreDWG.dwg_dynapi_subclass_field(name, fieldname)

def dwg_dynapi_common_entity_field(fieldname):
    return _LibreDWG.dwg_dynapi_common_entity_field(fieldname)

def dwg_dynapi_common_object_field(fieldname):
    return _LibreDWG.dwg_dynapi_common_object_field(fieldname)

def dwg_dynapi_entity_fields(name):
    return _LibreDWG.dwg_dynapi_entity_fields(name)

def dwg_dynapi_common_entity_fields():
    return _LibreDWG.dwg_dynapi_common_entity_fields()

def dwg_dynapi_common_object_fields():
    return _LibreDWG.dwg_dynapi_common_object_fields()

def dwg_dynapi_subclass_fields(name):
    return _LibreDWG.dwg_dynapi_subclass_fields(name)

def dwg_dynapi_fields_size(name):
    return _LibreDWG.dwg_dynapi_fields_size(name)

def dwg_object_tablectrl_get_entries(obj, error):
    return _LibreDWG.dwg_object_tablectrl_get_entries(obj, error)

def dwg_ref_get_table_name(ref, error):
    return _LibreDWG.dwg_ref_get_table_name(ref, error)

def dwg_ent_get_layer_name(ent, error):
    return _LibreDWG.dwg_ent_get_layer_name(ent, error)

def dwg_obj_layer_get_name(layer, error):
    return _LibreDWG.dwg_obj_layer_get_name(layer, error)

def dwg_obj_table_get_name(obj, error):
    return _LibreDWG.dwg_obj_table_get_name(obj, error)

def dwg_ent_to_object(obj, error):
    return _LibreDWG.dwg_ent_to_object(obj, error)

def dwg_object_to_object(obj, error):
    return _LibreDWG.dwg_object_to_object(obj, error)

def dwg_obj_obj_to_object(obj, error):
    return _LibreDWG.dwg_obj_obj_to_object(obj, error)

def dwg_object_to_entity(obj, error):
    return _LibreDWG.dwg_object_to_entity(obj, error)

def dwg_object_get_type(obj):
    return _LibreDWG.dwg_object_get_type(obj)

def dwg_object_get_fixedtype(obj):
    return _LibreDWG.dwg_object_get_fixedtype(obj)

def dwg_object_get_dxfname(obj):
    return _LibreDWG.dwg_object_get_dxfname(obj)

def dwg_ref_get_absref(ref, error):
    return _LibreDWG.dwg_ref_get_absref(ref, error)

def dwg_ref_get_object(ref, error):
    return _LibreDWG.dwg_ref_get_object(ref, error)

def dwg_absref_get_object(dwg, absref):
    return _LibreDWG.dwg_absref_get_object(dwg, absref)

def dwg_get_num_classes(dwg):
    return _LibreDWG.dwg_get_num_classes(dwg)

def dwg_get_class(dwg, index):
    return _LibreDWG.dwg_get_class(dwg, index)

def dwg_get_object(dwg, index):
    return _LibreDWG.dwg_get_object(dwg, index)

def dwg_obj_generic_to_object(obj, error):
    return _LibreDWG.dwg_obj_generic_to_object(obj, error)

def dwg_obj_generic_parent(obj, error):
    return _LibreDWG.dwg_obj_generic_parent(obj, error)

def dwg_object_to_DIMENSION(obj):
    return _LibreDWG.dwg_object_to_DIMENSION(obj)

def dwg_getall__3DFACE(hdr):
    return _LibreDWG.dwg_getall__3DFACE(hdr)

def dwg_getall__3DLINE(hdr):
    return _LibreDWG.dwg_getall__3DLINE(hdr)

def dwg_getall__3DSOLID(hdr):
    return _LibreDWG.dwg_getall__3DSOLID(hdr)

def dwg_getall_ARC(hdr):
    return _LibreDWG.dwg_getall_ARC(hdr)

def dwg_getall_ATTDEF(hdr):
    return _LibreDWG.dwg_getall_ATTDEF(hdr)

def dwg_getall_ATTRIB(hdr):
    return _LibreDWG.dwg_getall_ATTRIB(hdr)

def dwg_getall_BLOCK(hdr):
    return _LibreDWG.dwg_getall_BLOCK(hdr)

def dwg_getall_BODY(hdr):
    return _LibreDWG.dwg_getall_BODY(hdr)

def dwg_getall_CAMERA(hdr):
    return _LibreDWG.dwg_getall_CAMERA(hdr)

def dwg_getall_CIRCLE(hdr):
    return _LibreDWG.dwg_getall_CIRCLE(hdr)

def dwg_getall_DGNUNDERLAY(hdr):
    return _LibreDWG.dwg_getall_DGNUNDERLAY(hdr)

def dwg_getall_DIMENSION_ALIGNED(hdr):
    return _LibreDWG.dwg_getall_DIMENSION_ALIGNED(hdr)

def dwg_getall_DIMENSION_ANG2LN(hdr):
    return _LibreDWG.dwg_getall_DIMENSION_ANG2LN(hdr)

def dwg_getall_DIMENSION_ANG3PT(hdr):
    return _LibreDWG.dwg_getall_DIMENSION_ANG3PT(hdr)

def dwg_getall_DIMENSION_DIAMETER(hdr):
    return _LibreDWG.dwg_getall_DIMENSION_DIAMETER(hdr)

def dwg_getall_DIMENSION_LINEAR(hdr):
    return _LibreDWG.dwg_getall_DIMENSION_LINEAR(hdr)

def dwg_getall_DIMENSION_ORDINATE(hdr):
    return _LibreDWG.dwg_getall_DIMENSION_ORDINATE(hdr)

def dwg_getall_DIMENSION_RADIUS(hdr):
    return _LibreDWG.dwg_getall_DIMENSION_RADIUS(hdr)

def dwg_getall_DWFUNDERLAY(hdr):
    return _LibreDWG.dwg_getall_DWFUNDERLAY(hdr)

def dwg_getall_ELLIPSE(hdr):
    return _LibreDWG.dwg_getall_ELLIPSE(hdr)

def dwg_getall_ENDBLK(hdr):
    return _LibreDWG.dwg_getall_ENDBLK(hdr)

def dwg_getall_ENDREP(hdr):
    return _LibreDWG.dwg_getall_ENDREP(hdr)

def dwg_getall_HATCH(hdr):
    return _LibreDWG.dwg_getall_HATCH(hdr)

def dwg_getall_IMAGE(hdr):
    return _LibreDWG.dwg_getall_IMAGE(hdr)

def dwg_getall_INSERT(hdr):
    return _LibreDWG.dwg_getall_INSERT(hdr)

def dwg_getall_JUMP(hdr):
    return _LibreDWG.dwg_getall_JUMP(hdr)

def dwg_getall_LEADER(hdr):
    return _LibreDWG.dwg_getall_LEADER(hdr)

def dwg_getall_LIGHT(hdr):
    return _LibreDWG.dwg_getall_LIGHT(hdr)

def dwg_getall_LINE(hdr):
    return _LibreDWG.dwg_getall_LINE(hdr)

def dwg_getall_LOAD(hdr):
    return _LibreDWG.dwg_getall_LOAD(hdr)

def dwg_getall_LWPOLYLINE(hdr):
    return _LibreDWG.dwg_getall_LWPOLYLINE(hdr)

def dwg_getall_MESH(hdr):
    return _LibreDWG.dwg_getall_MESH(hdr)

def dwg_getall_MINSERT(hdr):
    return _LibreDWG.dwg_getall_MINSERT(hdr)

def dwg_getall_MLINE(hdr):
    return _LibreDWG.dwg_getall_MLINE(hdr)

def dwg_getall_MTEXT(hdr):
    return _LibreDWG.dwg_getall_MTEXT(hdr)

def dwg_getall_MULTILEADER(hdr):
    return _LibreDWG.dwg_getall_MULTILEADER(hdr)

def dwg_getall_OLE2FRAME(hdr):
    return _LibreDWG.dwg_getall_OLE2FRAME(hdr)

def dwg_getall_OLEFRAME(hdr):
    return _LibreDWG.dwg_getall_OLEFRAME(hdr)

def dwg_getall_PDFUNDERLAY(hdr):
    return _LibreDWG.dwg_getall_PDFUNDERLAY(hdr)

def dwg_getall_POINT(hdr):
    return _LibreDWG.dwg_getall_POINT(hdr)

def dwg_getall_POLYLINE_2D(hdr):
    return _LibreDWG.dwg_getall_POLYLINE_2D(hdr)

def dwg_getall_POLYLINE_3D(hdr):
    return _LibreDWG.dwg_getall_POLYLINE_3D(hdr)

def dwg_getall_POLYLINE_MESH(hdr):
    return _LibreDWG.dwg_getall_POLYLINE_MESH(hdr)

def dwg_getall_POLYLINE_PFACE(hdr):
    return _LibreDWG.dwg_getall_POLYLINE_PFACE(hdr)

def dwg_getall_PROXY_ENTITY(hdr):
    return _LibreDWG.dwg_getall_PROXY_ENTITY(hdr)

def dwg_getall_RAY(hdr):
    return _LibreDWG.dwg_getall_RAY(hdr)

def dwg_getall_REGION(hdr):
    return _LibreDWG.dwg_getall_REGION(hdr)

def dwg_getall_REPEAT(hdr):
    return _LibreDWG.dwg_getall_REPEAT(hdr)

def dwg_getall_SECTIONOBJECT(hdr):
    return _LibreDWG.dwg_getall_SECTIONOBJECT(hdr)

def dwg_getall_SEQEND(hdr):
    return _LibreDWG.dwg_getall_SEQEND(hdr)

def dwg_getall_SHAPE(hdr):
    return _LibreDWG.dwg_getall_SHAPE(hdr)

def dwg_getall_SOLID(hdr):
    return _LibreDWG.dwg_getall_SOLID(hdr)

def dwg_getall_SPLINE(hdr):
    return _LibreDWG.dwg_getall_SPLINE(hdr)

def dwg_getall_TEXT(hdr):
    return _LibreDWG.dwg_getall_TEXT(hdr)

def dwg_getall_TOLERANCE(hdr):
    return _LibreDWG.dwg_getall_TOLERANCE(hdr)

def dwg_getall_TRACE(hdr):
    return _LibreDWG.dwg_getall_TRACE(hdr)

def dwg_getall_UNKNOWN_ENT(hdr):
    return _LibreDWG.dwg_getall_UNKNOWN_ENT(hdr)

def dwg_getall_VERTEX_2D(hdr):
    return _LibreDWG.dwg_getall_VERTEX_2D(hdr)

def dwg_getall_VERTEX_3D(hdr):
    return _LibreDWG.dwg_getall_VERTEX_3D(hdr)

def dwg_getall_VERTEX_MESH(hdr):
    return _LibreDWG.dwg_getall_VERTEX_MESH(hdr)

def dwg_getall_VERTEX_PFACE(hdr):
    return _LibreDWG.dwg_getall_VERTEX_PFACE(hdr)

def dwg_getall_VERTEX_PFACE_FACE(hdr):
    return _LibreDWG.dwg_getall_VERTEX_PFACE_FACE(hdr)

def dwg_getall_VIEWPORT(hdr):
    return _LibreDWG.dwg_getall_VIEWPORT(hdr)

def dwg_getall_WIPEOUT(hdr):
    return _LibreDWG.dwg_getall_WIPEOUT(hdr)

def dwg_getall_XLINE(hdr):
    return _LibreDWG.dwg_getall_XLINE(hdr)

def dwg_getall_ARC_DIMENSION(hdr):
    return _LibreDWG.dwg_getall_ARC_DIMENSION(hdr)

def dwg_getall_HELIX(hdr):
    return _LibreDWG.dwg_getall_HELIX(hdr)

def dwg_getall_LARGE_RADIAL_DIMENSION(hdr):
    return _LibreDWG.dwg_getall_LARGE_RADIAL_DIMENSION(hdr)

def dwg_getall_LAYOUTPRINTCONFIG(hdr):
    return _LibreDWG.dwg_getall_LAYOUTPRINTCONFIG(hdr)

def dwg_getall_PLANESURFACE(hdr):
    return _LibreDWG.dwg_getall_PLANESURFACE(hdr)

def dwg_getall_POINTCLOUD(hdr):
    return _LibreDWG.dwg_getall_POINTCLOUD(hdr)

def dwg_getall_POINTCLOUDEX(hdr):
    return _LibreDWG.dwg_getall_POINTCLOUDEX(hdr)

def dwg_getall_ACSH_BOOLEAN_CLASS(dwg):
    return _LibreDWG.dwg_getall_ACSH_BOOLEAN_CLASS(dwg)

def dwg_getall_ACSH_BOX_CLASS(dwg):
    return _LibreDWG.dwg_getall_ACSH_BOX_CLASS(dwg)

def dwg_getall_ACSH_CONE_CLASS(dwg):
    return _LibreDWG.dwg_getall_ACSH_CONE_CLASS(dwg)

def dwg_getall_ACSH_CYLINDER_CLASS(dwg):
    return _LibreDWG.dwg_getall_ACSH_CYLINDER_CLASS(dwg)

def dwg_getall_ACSH_FILLET_CLASS(dwg):
    return _LibreDWG.dwg_getall_ACSH_FILLET_CLASS(dwg)

def dwg_getall_ACSH_HISTORY_CLASS(dwg):
    return _LibreDWG.dwg_getall_ACSH_HISTORY_CLASS(dwg)

def dwg_getall_ACSH_SPHERE_CLASS(dwg):
    return _LibreDWG.dwg_getall_ACSH_SPHERE_CLASS(dwg)

def dwg_getall_ACSH_TORUS_CLASS(dwg):
    return _LibreDWG.dwg_getall_ACSH_TORUS_CLASS(dwg)

def dwg_getall_ACSH_WEDGE_CLASS(dwg):
    return _LibreDWG.dwg_getall_ACSH_WEDGE_CLASS(dwg)

def dwg_getall_APPID(dwg):
    return _LibreDWG.dwg_getall_APPID(dwg)

def dwg_getall_APPID_CONTROL(dwg):
    return _LibreDWG.dwg_getall_APPID_CONTROL(dwg)

def dwg_getall_ASSOCGEOMDEPENDENCY(dwg):
    return _LibreDWG.dwg_getall_ASSOCGEOMDEPENDENCY(dwg)

def dwg_getall_ASSOCNETWORK(dwg):
    return _LibreDWG.dwg_getall_ASSOCNETWORK(dwg)

def dwg_getall_BLOCKALIGNMENTGRIP(dwg):
    return _LibreDWG.dwg_getall_BLOCKALIGNMENTGRIP(dwg)

def dwg_getall_BLOCKALIGNMENTPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKALIGNMENTPARAMETER(dwg)

def dwg_getall_BLOCKBASEPOINTPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKBASEPOINTPARAMETER(dwg)

def dwg_getall_BLOCKFLIPACTION(dwg):
    return _LibreDWG.dwg_getall_BLOCKFLIPACTION(dwg)

def dwg_getall_BLOCKFLIPGRIP(dwg):
    return _LibreDWG.dwg_getall_BLOCKFLIPGRIP(dwg)

def dwg_getall_BLOCKFLIPPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKFLIPPARAMETER(dwg)

def dwg_getall_BLOCKGRIPLOCATIONCOMPONENT(dwg):
    return _LibreDWG.dwg_getall_BLOCKGRIPLOCATIONCOMPONENT(dwg)

def dwg_getall_BLOCKLINEARGRIP(dwg):
    return _LibreDWG.dwg_getall_BLOCKLINEARGRIP(dwg)

def dwg_getall_BLOCKLOOKUPGRIP(dwg):
    return _LibreDWG.dwg_getall_BLOCKLOOKUPGRIP(dwg)

def dwg_getall_BLOCKMOVEACTION(dwg):
    return _LibreDWG.dwg_getall_BLOCKMOVEACTION(dwg)

def dwg_getall_BLOCKROTATEACTION(dwg):
    return _LibreDWG.dwg_getall_BLOCKROTATEACTION(dwg)

def dwg_getall_BLOCKROTATIONGRIP(dwg):
    return _LibreDWG.dwg_getall_BLOCKROTATIONGRIP(dwg)

def dwg_getall_BLOCKSCALEACTION(dwg):
    return _LibreDWG.dwg_getall_BLOCKSCALEACTION(dwg)

def dwg_getall_BLOCKVISIBILITYGRIP(dwg):
    return _LibreDWG.dwg_getall_BLOCKVISIBILITYGRIP(dwg)

def dwg_getall_BLOCK_CONTROL(dwg):
    return _LibreDWG.dwg_getall_BLOCK_CONTROL(dwg)

def dwg_getall_BLOCK_HEADER(dwg):
    return _LibreDWG.dwg_getall_BLOCK_HEADER(dwg)

def dwg_getall_CELLSTYLEMAP(dwg):
    return _LibreDWG.dwg_getall_CELLSTYLEMAP(dwg)

def dwg_getall_DETAILVIEWSTYLE(dwg):
    return _LibreDWG.dwg_getall_DETAILVIEWSTYLE(dwg)

def dwg_getall_DICTIONARY(dwg):
    return _LibreDWG.dwg_getall_DICTIONARY(dwg)

def dwg_getall_DICTIONARYVAR(dwg):
    return _LibreDWG.dwg_getall_DICTIONARYVAR(dwg)

def dwg_getall_DICTIONARYWDFLT(dwg):
    return _LibreDWG.dwg_getall_DICTIONARYWDFLT(dwg)

def dwg_getall_DIMSTYLE(dwg):
    return _LibreDWG.dwg_getall_DIMSTYLE(dwg)

def dwg_getall_DIMSTYLE_CONTROL(dwg):
    return _LibreDWG.dwg_getall_DIMSTYLE_CONTROL(dwg)

def dwg_getall_DUMMY(dwg):
    return _LibreDWG.dwg_getall_DUMMY(dwg)

def dwg_getall_DYNAMICBLOCKPURGEPREVENTER(dwg):
    return _LibreDWG.dwg_getall_DYNAMICBLOCKPURGEPREVENTER(dwg)

def dwg_getall_FIELD(dwg):
    return _LibreDWG.dwg_getall_FIELD(dwg)

def dwg_getall_FIELDLIST(dwg):
    return _LibreDWG.dwg_getall_FIELDLIST(dwg)

def dwg_getall_GEODATA(dwg):
    return _LibreDWG.dwg_getall_GEODATA(dwg)

def dwg_getall_GROUP(dwg):
    return _LibreDWG.dwg_getall_GROUP(dwg)

def dwg_getall_IDBUFFER(dwg):
    return _LibreDWG.dwg_getall_IDBUFFER(dwg)

def dwg_getall_IMAGEDEF(dwg):
    return _LibreDWG.dwg_getall_IMAGEDEF(dwg)

def dwg_getall_IMAGEDEF_REACTOR(dwg):
    return _LibreDWG.dwg_getall_IMAGEDEF_REACTOR(dwg)

def dwg_getall_INDEX(dwg):
    return _LibreDWG.dwg_getall_INDEX(dwg)

def dwg_getall_LAYER(dwg):
    return _LibreDWG.dwg_getall_LAYER(dwg)

def dwg_getall_LAYERFILTER(dwg):
    return _LibreDWG.dwg_getall_LAYERFILTER(dwg)

def dwg_getall_LAYER_CONTROL(dwg):
    return _LibreDWG.dwg_getall_LAYER_CONTROL(dwg)

def dwg_getall_LAYER_INDEX(dwg):
    return _LibreDWG.dwg_getall_LAYER_INDEX(dwg)

def dwg_getall_LAYOUT(dwg):
    return _LibreDWG.dwg_getall_LAYOUT(dwg)

def dwg_getall_LONG_TRANSACTION(dwg):
    return _LibreDWG.dwg_getall_LONG_TRANSACTION(dwg)

def dwg_getall_LTYPE(dwg):
    return _LibreDWG.dwg_getall_LTYPE(dwg)

def dwg_getall_LTYPE_CONTROL(dwg):
    return _LibreDWG.dwg_getall_LTYPE_CONTROL(dwg)

def dwg_getall_MLINESTYLE(dwg):
    return _LibreDWG.dwg_getall_MLINESTYLE(dwg)

def dwg_getall_PLACEHOLDER(dwg):
    return _LibreDWG.dwg_getall_PLACEHOLDER(dwg)

def dwg_getall_PLOTSETTINGS(dwg):
    return _LibreDWG.dwg_getall_PLOTSETTINGS(dwg)

def dwg_getall_RASTERVARIABLES(dwg):
    return _LibreDWG.dwg_getall_RASTERVARIABLES(dwg)

def dwg_getall_RENDERENVIRONMENT(dwg):
    return _LibreDWG.dwg_getall_RENDERENVIRONMENT(dwg)

def dwg_getall_SCALE(dwg):
    return _LibreDWG.dwg_getall_SCALE(dwg)

def dwg_getall_SECTIONVIEWSTYLE(dwg):
    return _LibreDWG.dwg_getall_SECTIONVIEWSTYLE(dwg)

def dwg_getall_SECTION_MANAGER(dwg):
    return _LibreDWG.dwg_getall_SECTION_MANAGER(dwg)

def dwg_getall_SORTENTSTABLE(dwg):
    return _LibreDWG.dwg_getall_SORTENTSTABLE(dwg)

def dwg_getall_SPATIAL_FILTER(dwg):
    return _LibreDWG.dwg_getall_SPATIAL_FILTER(dwg)

def dwg_getall_STYLE(dwg):
    return _LibreDWG.dwg_getall_STYLE(dwg)

def dwg_getall_STYLE_CONTROL(dwg):
    return _LibreDWG.dwg_getall_STYLE_CONTROL(dwg)

def dwg_getall_SUN(dwg):
    return _LibreDWG.dwg_getall_SUN(dwg)

def dwg_getall_TABLEGEOMETRY(dwg):
    return _LibreDWG.dwg_getall_TABLEGEOMETRY(dwg)

def dwg_getall_UCS(dwg):
    return _LibreDWG.dwg_getall_UCS(dwg)

def dwg_getall_UCS_CONTROL(dwg):
    return _LibreDWG.dwg_getall_UCS_CONTROL(dwg)

def dwg_getall_UNKNOWN_OBJ(dwg):
    return _LibreDWG.dwg_getall_UNKNOWN_OBJ(dwg)

def dwg_getall_VBA_PROJECT(dwg):
    return _LibreDWG.dwg_getall_VBA_PROJECT(dwg)

def dwg_getall_VIEW(dwg):
    return _LibreDWG.dwg_getall_VIEW(dwg)

def dwg_getall_VIEW_CONTROL(dwg):
    return _LibreDWG.dwg_getall_VIEW_CONTROL(dwg)

def dwg_getall_VISUALSTYLE(dwg):
    return _LibreDWG.dwg_getall_VISUALSTYLE(dwg)

def dwg_getall_VPORT(dwg):
    return _LibreDWG.dwg_getall_VPORT(dwg)

def dwg_getall_VPORT_CONTROL(dwg):
    return _LibreDWG.dwg_getall_VPORT_CONTROL(dwg)

def dwg_getall_VX_CONTROL(dwg):
    return _LibreDWG.dwg_getall_VX_CONTROL(dwg)

def dwg_getall_VX_TABLE_RECORD(dwg):
    return _LibreDWG.dwg_getall_VX_TABLE_RECORD(dwg)

def dwg_getall_WIPEOUTVARIABLES(dwg):
    return _LibreDWG.dwg_getall_WIPEOUTVARIABLES(dwg)

def dwg_getall_XRECORD(dwg):
    return _LibreDWG.dwg_getall_XRECORD(dwg)

def dwg_getall_PDFDEFINITION(dwg):
    return _LibreDWG.dwg_getall_PDFDEFINITION(dwg)

def dwg_getall_DGNDEFINITION(dwg):
    return _LibreDWG.dwg_getall_DGNDEFINITION(dwg)

def dwg_getall_DWFDEFINITION(dwg):
    return _LibreDWG.dwg_getall_DWFDEFINITION(dwg)

def dwg_getall_ACSH_BREP_CLASS(dwg):
    return _LibreDWG.dwg_getall_ACSH_BREP_CLASS(dwg)

def dwg_getall_ACSH_CHAMFER_CLASS(dwg):
    return _LibreDWG.dwg_getall_ACSH_CHAMFER_CLASS(dwg)

def dwg_getall_ACSH_PYRAMID_CLASS(dwg):
    return _LibreDWG.dwg_getall_ACSH_PYRAMID_CLASS(dwg)

def dwg_getall_ALDIMOBJECTCONTEXTDATA(dwg):
    return _LibreDWG.dwg_getall_ALDIMOBJECTCONTEXTDATA(dwg)

def dwg_getall_ASSOC2DCONSTRAINTGROUP(dwg):
    return _LibreDWG.dwg_getall_ASSOC2DCONSTRAINTGROUP(dwg)

def dwg_getall_ASSOCACTION(dwg):
    return _LibreDWG.dwg_getall_ASSOCACTION(dwg)

def dwg_getall_ASSOCACTIONPARAM(dwg):
    return _LibreDWG.dwg_getall_ASSOCACTIONPARAM(dwg)

def dwg_getall_ASSOCARRAYACTIONBODY(dwg):
    return _LibreDWG.dwg_getall_ASSOCARRAYACTIONBODY(dwg)

def dwg_getall_ASSOCASMBODYACTIONPARAM(dwg):
    return _LibreDWG.dwg_getall_ASSOCASMBODYACTIONPARAM(dwg)

def dwg_getall_ASSOCBLENDSURFACEACTIONBODY(dwg):
    return _LibreDWG.dwg_getall_ASSOCBLENDSURFACEACTIONBODY(dwg)

def dwg_getall_ASSOCCOMPOUNDACTIONPARAM(dwg):
    return _LibreDWG.dwg_getall_ASSOCCOMPOUNDACTIONPARAM(dwg)

def dwg_getall_ASSOCDEPENDENCY(dwg):
    return _LibreDWG.dwg_getall_ASSOCDEPENDENCY(dwg)

def dwg_getall_ASSOCDIMDEPENDENCYBODY(dwg):
    return _LibreDWG.dwg_getall_ASSOCDIMDEPENDENCYBODY(dwg)

def dwg_getall_ASSOCEXTENDSURFACEACTIONBODY(dwg):
    return _LibreDWG.dwg_getall_ASSOCEXTENDSURFACEACTIONBODY(dwg)

def dwg_getall_ASSOCEXTRUDEDSURFACEACTIONBODY(dwg):
    return _LibreDWG.dwg_getall_ASSOCEXTRUDEDSURFACEACTIONBODY(dwg)

def dwg_getall_ASSOCFACEACTIONPARAM(dwg):
    return _LibreDWG.dwg_getall_ASSOCFACEACTIONPARAM(dwg)

def dwg_getall_ASSOCFILLETSURFACEACTIONBODY(dwg):
    return _LibreDWG.dwg_getall_ASSOCFILLETSURFACEACTIONBODY(dwg)

def dwg_getall_ASSOCLOFTEDSURFACEACTIONBODY(dwg):
    return _LibreDWG.dwg_getall_ASSOCLOFTEDSURFACEACTIONBODY(dwg)

def dwg_getall_ASSOCNETWORKSURFACEACTIONBODY(dwg):
    return _LibreDWG.dwg_getall_ASSOCNETWORKSURFACEACTIONBODY(dwg)

def dwg_getall_ASSOCOBJECTACTIONPARAM(dwg):
    return _LibreDWG.dwg_getall_ASSOCOBJECTACTIONPARAM(dwg)

def dwg_getall_ASSOCOFFSETSURFACEACTIONBODY(dwg):
    return _LibreDWG.dwg_getall_ASSOCOFFSETSURFACEACTIONBODY(dwg)

def dwg_getall_ASSOCOSNAPPOINTREFACTIONPARAM(dwg):
    return _LibreDWG.dwg_getall_ASSOCOSNAPPOINTREFACTIONPARAM(dwg)

def dwg_getall_ASSOCPATCHSURFACEACTIONBODY(dwg):
    return _LibreDWG.dwg_getall_ASSOCPATCHSURFACEACTIONBODY(dwg)

def dwg_getall_ASSOCPATHACTIONPARAM(dwg):
    return _LibreDWG.dwg_getall_ASSOCPATHACTIONPARAM(dwg)

def dwg_getall_ASSOCPLANESURFACEACTIONBODY(dwg):
    return _LibreDWG.dwg_getall_ASSOCPLANESURFACEACTIONBODY(dwg)

def dwg_getall_ASSOCPOINTREFACTIONPARAM(dwg):
    return _LibreDWG.dwg_getall_ASSOCPOINTREFACTIONPARAM(dwg)

def dwg_getall_ASSOCREVOLVEDSURFACEACTIONBODY(dwg):
    return _LibreDWG.dwg_getall_ASSOCREVOLVEDSURFACEACTIONBODY(dwg)

def dwg_getall_ASSOCTRIMSURFACEACTIONBODY(dwg):
    return _LibreDWG.dwg_getall_ASSOCTRIMSURFACEACTIONBODY(dwg)

def dwg_getall_ASSOCVALUEDEPENDENCY(dwg):
    return _LibreDWG.dwg_getall_ASSOCVALUEDEPENDENCY(dwg)

def dwg_getall_ASSOCVARIABLE(dwg):
    return _LibreDWG.dwg_getall_ASSOCVARIABLE(dwg)

def dwg_getall_ASSOCVERTEXACTIONPARAM(dwg):
    return _LibreDWG.dwg_getall_ASSOCVERTEXACTIONPARAM(dwg)

def dwg_getall_BLKREFOBJECTCONTEXTDATA(dwg):
    return _LibreDWG.dwg_getall_BLKREFOBJECTCONTEXTDATA(dwg)

def dwg_getall_BLOCKALIGNEDCONSTRAINTPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKALIGNEDCONSTRAINTPARAMETER(dwg)

def dwg_getall_BLOCKANGULARCONSTRAINTPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKANGULARCONSTRAINTPARAMETER(dwg)

def dwg_getall_BLOCKARRAYACTION(dwg):
    return _LibreDWG.dwg_getall_BLOCKARRAYACTION(dwg)

def dwg_getall_BLOCKDIAMETRICCONSTRAINTPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKDIAMETRICCONSTRAINTPARAMETER(dwg)

def dwg_getall_BLOCKHORIZONTALCONSTRAINTPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKHORIZONTALCONSTRAINTPARAMETER(dwg)

def dwg_getall_BLOCKLINEARCONSTRAINTPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKLINEARCONSTRAINTPARAMETER(dwg)

def dwg_getall_BLOCKLINEARPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKLINEARPARAMETER(dwg)

def dwg_getall_BLOCKLOOKUPACTION(dwg):
    return _LibreDWG.dwg_getall_BLOCKLOOKUPACTION(dwg)

def dwg_getall_BLOCKLOOKUPPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKLOOKUPPARAMETER(dwg)

def dwg_getall_BLOCKPARAMDEPENDENCYBODY(dwg):
    return _LibreDWG.dwg_getall_BLOCKPARAMDEPENDENCYBODY(dwg)

def dwg_getall_BLOCKPOINTPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKPOINTPARAMETER(dwg)

def dwg_getall_BLOCKPOLARGRIP(dwg):
    return _LibreDWG.dwg_getall_BLOCKPOLARGRIP(dwg)

def dwg_getall_BLOCKPOLARPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKPOLARPARAMETER(dwg)

def dwg_getall_BLOCKPOLARSTRETCHACTION(dwg):
    return _LibreDWG.dwg_getall_BLOCKPOLARSTRETCHACTION(dwg)

def dwg_getall_BLOCKRADIALCONSTRAINTPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKRADIALCONSTRAINTPARAMETER(dwg)

def dwg_getall_BLOCKREPRESENTATION(dwg):
    return _LibreDWG.dwg_getall_BLOCKREPRESENTATION(dwg)

def dwg_getall_BLOCKROTATIONPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKROTATIONPARAMETER(dwg)

def dwg_getall_BLOCKSTRETCHACTION(dwg):
    return _LibreDWG.dwg_getall_BLOCKSTRETCHACTION(dwg)

def dwg_getall_BLOCKUSERPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKUSERPARAMETER(dwg)

def dwg_getall_BLOCKVERTICALCONSTRAINTPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKVERTICALCONSTRAINTPARAMETER(dwg)

def dwg_getall_BLOCKVISIBILITYPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKVISIBILITYPARAMETER(dwg)

def dwg_getall_BLOCKXYGRIP(dwg):
    return _LibreDWG.dwg_getall_BLOCKXYGRIP(dwg)

def dwg_getall_BLOCKXYPARAMETER(dwg):
    return _LibreDWG.dwg_getall_BLOCKXYPARAMETER(dwg)

def dwg_getall_DATALINK(dwg):
    return _LibreDWG.dwg_getall_DATALINK(dwg)

def dwg_getall_DBCOLOR(dwg):
    return _LibreDWG.dwg_getall_DBCOLOR(dwg)

def dwg_getall_DIMASSOC(dwg):
    return _LibreDWG.dwg_getall_DIMASSOC(dwg)

def dwg_getall_EVALUATION_GRAPH(dwg):
    return _LibreDWG.dwg_getall_EVALUATION_GRAPH(dwg)

def dwg_getall_FCFOBJECTCONTEXTDATA(dwg):
    return _LibreDWG.dwg_getall_FCFOBJECTCONTEXTDATA(dwg)

def dwg_getall_GRADIENT_BACKGROUND(dwg):
    return _LibreDWG.dwg_getall_GRADIENT_BACKGROUND(dwg)

def dwg_getall_GROUND_PLANE_BACKGROUND(dwg):
    return _LibreDWG.dwg_getall_GROUND_PLANE_BACKGROUND(dwg)

def dwg_getall_IBL_BACKGROUND(dwg):
    return _LibreDWG.dwg_getall_IBL_BACKGROUND(dwg)

def dwg_getall_IMAGE_BACKGROUND(dwg):
    return _LibreDWG.dwg_getall_IMAGE_BACKGROUND(dwg)

def dwg_getall_LEADEROBJECTCONTEXTDATA(dwg):
    return _LibreDWG.dwg_getall_LEADEROBJECTCONTEXTDATA(dwg)

def dwg_getall_LIGHTLIST(dwg):
    return _LibreDWG.dwg_getall_LIGHTLIST(dwg)

def dwg_getall_MATERIAL(dwg):
    return _LibreDWG.dwg_getall_MATERIAL(dwg)

def dwg_getall_MENTALRAYRENDERSETTINGS(dwg):
    return _LibreDWG.dwg_getall_MENTALRAYRENDERSETTINGS(dwg)

def dwg_getall_MLEADERSTYLE(dwg):
    return _LibreDWG.dwg_getall_MLEADERSTYLE(dwg)

def dwg_getall_MTEXTOBJECTCONTEXTDATA(dwg):
    return _LibreDWG.dwg_getall_MTEXTOBJECTCONTEXTDATA(dwg)

def dwg_getall_OBJECT_PTR(dwg):
    return _LibreDWG.dwg_getall_OBJECT_PTR(dwg)

def dwg_getall_PARTIAL_VIEWING_INDEX(dwg):
    return _LibreDWG.dwg_getall_PARTIAL_VIEWING_INDEX(dwg)

def dwg_getall_POINTCLOUDCOLORMAP(dwg):
    return _LibreDWG.dwg_getall_POINTCLOUDCOLORMAP(dwg)

def dwg_getall_POINTCLOUDDEF(dwg):
    return _LibreDWG.dwg_getall_POINTCLOUDDEF(dwg)

def dwg_getall_POINTCLOUDDEFEX(dwg):
    return _LibreDWG.dwg_getall_POINTCLOUDDEFEX(dwg)

def dwg_getall_POINTCLOUDDEF_REACTOR(dwg):
    return _LibreDWG.dwg_getall_POINTCLOUDDEF_REACTOR(dwg)

def dwg_getall_POINTCLOUDDEF_REACTOR_EX(dwg):
    return _LibreDWG.dwg_getall_POINTCLOUDDEF_REACTOR_EX(dwg)

def dwg_getall_PROXY_OBJECT(dwg):
    return _LibreDWG.dwg_getall_PROXY_OBJECT(dwg)

def dwg_getall_RAPIDRTRENDERSETTINGS(dwg):
    return _LibreDWG.dwg_getall_RAPIDRTRENDERSETTINGS(dwg)

def dwg_getall_RENDERENTRY(dwg):
    return _LibreDWG.dwg_getall_RENDERENTRY(dwg)

def dwg_getall_RENDERGLOBAL(dwg):
    return _LibreDWG.dwg_getall_RENDERGLOBAL(dwg)

def dwg_getall_RENDERSETTINGS(dwg):
    return _LibreDWG.dwg_getall_RENDERSETTINGS(dwg)

def dwg_getall_SECTION_SETTINGS(dwg):
    return _LibreDWG.dwg_getall_SECTION_SETTINGS(dwg)

def dwg_getall_SKYLIGHT_BACKGROUND(dwg):
    return _LibreDWG.dwg_getall_SKYLIGHT_BACKGROUND(dwg)

def dwg_getall_SOLID_BACKGROUND(dwg):
    return _LibreDWG.dwg_getall_SOLID_BACKGROUND(dwg)

def dwg_getall_SPATIAL_INDEX(dwg):
    return _LibreDWG.dwg_getall_SPATIAL_INDEX(dwg)

def dwg_getall_TABLESTYLE(dwg):
    return _LibreDWG.dwg_getall_TABLESTYLE(dwg)

def dwg_getall_TEXTOBJECTCONTEXTDATA(dwg):
    return _LibreDWG.dwg_getall_TEXTOBJECTCONTEXTDATA(dwg)

def dwg_getall_ASSOCARRAYMODIFYPARAMETERS(dwg):
    return _LibreDWG.dwg_getall_ASSOCARRAYMODIFYPARAMETERS(dwg)

def dwg_getall_ASSOCARRAYPATHPARAMETERS(dwg):
    return _LibreDWG.dwg_getall_ASSOCARRAYPATHPARAMETERS(dwg)

def dwg_getall_ASSOCARRAYPOLARPARAMETERS(dwg):
    return _LibreDWG.dwg_getall_ASSOCARRAYPOLARPARAMETERS(dwg)

def dwg_getall_ASSOCARRAYRECTANGULARPARAMETERS(dwg):
    return _LibreDWG.dwg_getall_ASSOCARRAYRECTANGULARPARAMETERS(dwg)

def dwg_object_to__3DFACE(obj):
    return _LibreDWG.dwg_object_to__3DFACE(obj)

def dwg_object_to__3DLINE(obj):
    return _LibreDWG.dwg_object_to__3DLINE(obj)

def dwg_object_to__3DSOLID(obj):
    return _LibreDWG.dwg_object_to__3DSOLID(obj)

def dwg_object_to_ARC(obj):
    return _LibreDWG.dwg_object_to_ARC(obj)

def dwg_object_to_ATTDEF(obj):
    return _LibreDWG.dwg_object_to_ATTDEF(obj)

def dwg_object_to_ATTRIB(obj):
    return _LibreDWG.dwg_object_to_ATTRIB(obj)

def dwg_object_to_BLOCK(obj):
    return _LibreDWG.dwg_object_to_BLOCK(obj)

def dwg_object_to_BODY(obj):
    return _LibreDWG.dwg_object_to_BODY(obj)

def dwg_object_to_CAMERA(obj):
    return _LibreDWG.dwg_object_to_CAMERA(obj)

def dwg_object_to_CIRCLE(obj):
    return _LibreDWG.dwg_object_to_CIRCLE(obj)

def dwg_object_to_DGNUNDERLAY(obj):
    return _LibreDWG.dwg_object_to_DGNUNDERLAY(obj)

def dwg_object_to_DIMENSION_ALIGNED(obj):
    return _LibreDWG.dwg_object_to_DIMENSION_ALIGNED(obj)

def dwg_object_to_DIMENSION_ANG2LN(obj):
    return _LibreDWG.dwg_object_to_DIMENSION_ANG2LN(obj)

def dwg_object_to_DIMENSION_ANG3PT(obj):
    return _LibreDWG.dwg_object_to_DIMENSION_ANG3PT(obj)

def dwg_object_to_DIMENSION_DIAMETER(obj):
    return _LibreDWG.dwg_object_to_DIMENSION_DIAMETER(obj)

def dwg_object_to_DIMENSION_LINEAR(obj):
    return _LibreDWG.dwg_object_to_DIMENSION_LINEAR(obj)

def dwg_object_to_DIMENSION_ORDINATE(obj):
    return _LibreDWG.dwg_object_to_DIMENSION_ORDINATE(obj)

def dwg_object_to_DIMENSION_RADIUS(obj):
    return _LibreDWG.dwg_object_to_DIMENSION_RADIUS(obj)

def dwg_object_to_DWFUNDERLAY(obj):
    return _LibreDWG.dwg_object_to_DWFUNDERLAY(obj)

def dwg_object_to_ELLIPSE(obj):
    return _LibreDWG.dwg_object_to_ELLIPSE(obj)

def dwg_object_to_ENDBLK(obj):
    return _LibreDWG.dwg_object_to_ENDBLK(obj)

def dwg_object_to_ENDREP(obj):
    return _LibreDWG.dwg_object_to_ENDREP(obj)

def dwg_object_to_HATCH(obj):
    return _LibreDWG.dwg_object_to_HATCH(obj)

def dwg_object_to_IMAGE(obj):
    return _LibreDWG.dwg_object_to_IMAGE(obj)

def dwg_object_to_INSERT(obj):
    return _LibreDWG.dwg_object_to_INSERT(obj)

def dwg_object_to_JUMP(obj):
    return _LibreDWG.dwg_object_to_JUMP(obj)

def dwg_object_to_LEADER(obj):
    return _LibreDWG.dwg_object_to_LEADER(obj)

def dwg_object_to_LIGHT(obj):
    return _LibreDWG.dwg_object_to_LIGHT(obj)

def dwg_object_to_LINE(obj):
    return _LibreDWG.dwg_object_to_LINE(obj)

def dwg_object_to_LOAD(obj):
    return _LibreDWG.dwg_object_to_LOAD(obj)

def dwg_object_to_LWPOLYLINE(obj):
    return _LibreDWG.dwg_object_to_LWPOLYLINE(obj)

def dwg_object_to_MESH(obj):
    return _LibreDWG.dwg_object_to_MESH(obj)

def dwg_object_to_MINSERT(obj):
    return _LibreDWG.dwg_object_to_MINSERT(obj)

def dwg_object_to_MLINE(obj):
    return _LibreDWG.dwg_object_to_MLINE(obj)

def dwg_object_to_MTEXT(obj):
    return _LibreDWG.dwg_object_to_MTEXT(obj)

def dwg_object_to_MULTILEADER(obj):
    return _LibreDWG.dwg_object_to_MULTILEADER(obj)

def dwg_object_to_OLE2FRAME(obj):
    return _LibreDWG.dwg_object_to_OLE2FRAME(obj)

def dwg_object_to_OLEFRAME(obj):
    return _LibreDWG.dwg_object_to_OLEFRAME(obj)

def dwg_object_to_PDFUNDERLAY(obj):
    return _LibreDWG.dwg_object_to_PDFUNDERLAY(obj)

def dwg_object_to_POINT(obj):
    return _LibreDWG.dwg_object_to_POINT(obj)

def dwg_object_to_POLYLINE_2D(obj):
    return _LibreDWG.dwg_object_to_POLYLINE_2D(obj)

def dwg_object_to_POLYLINE_3D(obj):
    return _LibreDWG.dwg_object_to_POLYLINE_3D(obj)

def dwg_object_to_POLYLINE_MESH(obj):
    return _LibreDWG.dwg_object_to_POLYLINE_MESH(obj)

def dwg_object_to_POLYLINE_PFACE(obj):
    return _LibreDWG.dwg_object_to_POLYLINE_PFACE(obj)

def dwg_object_to_PROXY_ENTITY(obj):
    return _LibreDWG.dwg_object_to_PROXY_ENTITY(obj)

def dwg_object_to_RAY(obj):
    return _LibreDWG.dwg_object_to_RAY(obj)

def dwg_object_to_REGION(obj):
    return _LibreDWG.dwg_object_to_REGION(obj)

def dwg_object_to_REPEAT(obj):
    return _LibreDWG.dwg_object_to_REPEAT(obj)

def dwg_object_to_SECTIONOBJECT(obj):
    return _LibreDWG.dwg_object_to_SECTIONOBJECT(obj)

def dwg_object_to_SEQEND(obj):
    return _LibreDWG.dwg_object_to_SEQEND(obj)

def dwg_object_to_SHAPE(obj):
    return _LibreDWG.dwg_object_to_SHAPE(obj)

def dwg_object_to_SOLID(obj):
    return _LibreDWG.dwg_object_to_SOLID(obj)

def dwg_object_to_SPLINE(obj):
    return _LibreDWG.dwg_object_to_SPLINE(obj)

def dwg_object_to_TEXT(obj):
    return _LibreDWG.dwg_object_to_TEXT(obj)

def dwg_object_to_TOLERANCE(obj):
    return _LibreDWG.dwg_object_to_TOLERANCE(obj)

def dwg_object_to_TRACE(obj):
    return _LibreDWG.dwg_object_to_TRACE(obj)

def dwg_object_to_UNKNOWN_ENT(obj):
    return _LibreDWG.dwg_object_to_UNKNOWN_ENT(obj)

def dwg_object_to_VERTEX_2D(obj):
    return _LibreDWG.dwg_object_to_VERTEX_2D(obj)

def dwg_object_to_VERTEX_3D(obj):
    return _LibreDWG.dwg_object_to_VERTEX_3D(obj)

def dwg_object_to_VERTEX_MESH(obj):
    return _LibreDWG.dwg_object_to_VERTEX_MESH(obj)

def dwg_object_to_VERTEX_PFACE(obj):
    return _LibreDWG.dwg_object_to_VERTEX_PFACE(obj)

def dwg_object_to_VERTEX_PFACE_FACE(obj):
    return _LibreDWG.dwg_object_to_VERTEX_PFACE_FACE(obj)

def dwg_object_to_VIEWPORT(obj):
    return _LibreDWG.dwg_object_to_VIEWPORT(obj)

def dwg_object_to_WIPEOUT(obj):
    return _LibreDWG.dwg_object_to_WIPEOUT(obj)

def dwg_object_to_XLINE(obj):
    return _LibreDWG.dwg_object_to_XLINE(obj)

def dwg_object_to_ARC_DIMENSION(obj):
    return _LibreDWG.dwg_object_to_ARC_DIMENSION(obj)

def dwg_object_to_HELIX(obj):
    return _LibreDWG.dwg_object_to_HELIX(obj)

def dwg_object_to_LARGE_RADIAL_DIMENSION(obj):
    return _LibreDWG.dwg_object_to_LARGE_RADIAL_DIMENSION(obj)

def dwg_object_to_LAYOUTPRINTCONFIG(obj):
    return _LibreDWG.dwg_object_to_LAYOUTPRINTCONFIG(obj)

def dwg_object_to_PLANESURFACE(obj):
    return _LibreDWG.dwg_object_to_PLANESURFACE(obj)

def dwg_object_to_POINTCLOUD(obj):
    return _LibreDWG.dwg_object_to_POINTCLOUD(obj)

def dwg_object_to_POINTCLOUDEX(obj):
    return _LibreDWG.dwg_object_to_POINTCLOUDEX(obj)

def dwg_object_to_ACSH_BOOLEAN_CLASS(obj):
    return _LibreDWG.dwg_object_to_ACSH_BOOLEAN_CLASS(obj)

def dwg_object_to_ACSH_BOX_CLASS(obj):
    return _LibreDWG.dwg_object_to_ACSH_BOX_CLASS(obj)

def dwg_object_to_ACSH_CONE_CLASS(obj):
    return _LibreDWG.dwg_object_to_ACSH_CONE_CLASS(obj)

def dwg_object_to_ACSH_CYLINDER_CLASS(obj):
    return _LibreDWG.dwg_object_to_ACSH_CYLINDER_CLASS(obj)

def dwg_object_to_ACSH_FILLET_CLASS(obj):
    return _LibreDWG.dwg_object_to_ACSH_FILLET_CLASS(obj)

def dwg_object_to_ACSH_HISTORY_CLASS(obj):
    return _LibreDWG.dwg_object_to_ACSH_HISTORY_CLASS(obj)

def dwg_object_to_ACSH_SPHERE_CLASS(obj):
    return _LibreDWG.dwg_object_to_ACSH_SPHERE_CLASS(obj)

def dwg_object_to_ACSH_TORUS_CLASS(obj):
    return _LibreDWG.dwg_object_to_ACSH_TORUS_CLASS(obj)

def dwg_object_to_ACSH_WEDGE_CLASS(obj):
    return _LibreDWG.dwg_object_to_ACSH_WEDGE_CLASS(obj)

def dwg_object_to_APPID(obj):
    return _LibreDWG.dwg_object_to_APPID(obj)

def dwg_object_to_APPID_CONTROL(obj):
    return _LibreDWG.dwg_object_to_APPID_CONTROL(obj)

def dwg_object_to_ASSOCGEOMDEPENDENCY(obj):
    return _LibreDWG.dwg_object_to_ASSOCGEOMDEPENDENCY(obj)

def dwg_object_to_ASSOCNETWORK(obj):
    return _LibreDWG.dwg_object_to_ASSOCNETWORK(obj)

def dwg_object_to_BLOCKALIGNMENTGRIP(obj):
    return _LibreDWG.dwg_object_to_BLOCKALIGNMENTGRIP(obj)

def dwg_object_to_BLOCKALIGNMENTPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKALIGNMENTPARAMETER(obj)

def dwg_object_to_BLOCKBASEPOINTPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKBASEPOINTPARAMETER(obj)

def dwg_object_to_BLOCKFLIPACTION(obj):
    return _LibreDWG.dwg_object_to_BLOCKFLIPACTION(obj)

def dwg_object_to_BLOCKFLIPGRIP(obj):
    return _LibreDWG.dwg_object_to_BLOCKFLIPGRIP(obj)

def dwg_object_to_BLOCKFLIPPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKFLIPPARAMETER(obj)

def dwg_object_to_BLOCKGRIPLOCATIONCOMPONENT(obj):
    return _LibreDWG.dwg_object_to_BLOCKGRIPLOCATIONCOMPONENT(obj)

def dwg_object_to_BLOCKLINEARGRIP(obj):
    return _LibreDWG.dwg_object_to_BLOCKLINEARGRIP(obj)

def dwg_object_to_BLOCKLOOKUPGRIP(obj):
    return _LibreDWG.dwg_object_to_BLOCKLOOKUPGRIP(obj)

def dwg_object_to_BLOCKMOVEACTION(obj):
    return _LibreDWG.dwg_object_to_BLOCKMOVEACTION(obj)

def dwg_object_to_BLOCKROTATEACTION(obj):
    return _LibreDWG.dwg_object_to_BLOCKROTATEACTION(obj)

def dwg_object_to_BLOCKROTATIONGRIP(obj):
    return _LibreDWG.dwg_object_to_BLOCKROTATIONGRIP(obj)

def dwg_object_to_BLOCKSCALEACTION(obj):
    return _LibreDWG.dwg_object_to_BLOCKSCALEACTION(obj)

def dwg_object_to_BLOCKVISIBILITYGRIP(obj):
    return _LibreDWG.dwg_object_to_BLOCKVISIBILITYGRIP(obj)

def dwg_object_to_BLOCK_CONTROL(obj):
    return _LibreDWG.dwg_object_to_BLOCK_CONTROL(obj)

def dwg_object_to_BLOCK_HEADER(obj):
    return _LibreDWG.dwg_object_to_BLOCK_HEADER(obj)

def dwg_object_to_CELLSTYLEMAP(obj):
    return _LibreDWG.dwg_object_to_CELLSTYLEMAP(obj)

def dwg_object_to_DETAILVIEWSTYLE(obj):
    return _LibreDWG.dwg_object_to_DETAILVIEWSTYLE(obj)

def dwg_object_to_DICTIONARY(obj):
    return _LibreDWG.dwg_object_to_DICTIONARY(obj)

def dwg_object_to_DICTIONARYVAR(obj):
    return _LibreDWG.dwg_object_to_DICTIONARYVAR(obj)

def dwg_object_to_DICTIONARYWDFLT(obj):
    return _LibreDWG.dwg_object_to_DICTIONARYWDFLT(obj)

def dwg_object_to_DIMSTYLE(obj):
    return _LibreDWG.dwg_object_to_DIMSTYLE(obj)

def dwg_object_to_DIMSTYLE_CONTROL(obj):
    return _LibreDWG.dwg_object_to_DIMSTYLE_CONTROL(obj)

def dwg_object_to_DUMMY(obj):
    return _LibreDWG.dwg_object_to_DUMMY(obj)

def dwg_object_to_DYNAMICBLOCKPURGEPREVENTER(obj):
    return _LibreDWG.dwg_object_to_DYNAMICBLOCKPURGEPREVENTER(obj)

def dwg_object_to_FIELD(obj):
    return _LibreDWG.dwg_object_to_FIELD(obj)

def dwg_object_to_FIELDLIST(obj):
    return _LibreDWG.dwg_object_to_FIELDLIST(obj)

def dwg_object_to_GEODATA(obj):
    return _LibreDWG.dwg_object_to_GEODATA(obj)

def dwg_object_to_GROUP(obj):
    return _LibreDWG.dwg_object_to_GROUP(obj)

def dwg_object_to_IDBUFFER(obj):
    return _LibreDWG.dwg_object_to_IDBUFFER(obj)

def dwg_object_to_IMAGEDEF(obj):
    return _LibreDWG.dwg_object_to_IMAGEDEF(obj)

def dwg_object_to_IMAGEDEF_REACTOR(obj):
    return _LibreDWG.dwg_object_to_IMAGEDEF_REACTOR(obj)

def dwg_object_to_INDEX(obj):
    return _LibreDWG.dwg_object_to_INDEX(obj)

def dwg_object_to_LAYER(obj):
    return _LibreDWG.dwg_object_to_LAYER(obj)

def dwg_object_to_LAYERFILTER(obj):
    return _LibreDWG.dwg_object_to_LAYERFILTER(obj)

def dwg_object_to_LAYER_CONTROL(obj):
    return _LibreDWG.dwg_object_to_LAYER_CONTROL(obj)

def dwg_object_to_LAYER_INDEX(obj):
    return _LibreDWG.dwg_object_to_LAYER_INDEX(obj)

def dwg_object_to_LAYOUT(obj):
    return _LibreDWG.dwg_object_to_LAYOUT(obj)

def dwg_object_to_LONG_TRANSACTION(obj):
    return _LibreDWG.dwg_object_to_LONG_TRANSACTION(obj)

def dwg_object_to_LTYPE(obj):
    return _LibreDWG.dwg_object_to_LTYPE(obj)

def dwg_object_to_LTYPE_CONTROL(obj):
    return _LibreDWG.dwg_object_to_LTYPE_CONTROL(obj)

def dwg_object_to_MLINESTYLE(obj):
    return _LibreDWG.dwg_object_to_MLINESTYLE(obj)

def dwg_object_to_PLACEHOLDER(obj):
    return _LibreDWG.dwg_object_to_PLACEHOLDER(obj)

def dwg_object_to_PLOTSETTINGS(obj):
    return _LibreDWG.dwg_object_to_PLOTSETTINGS(obj)

def dwg_object_to_RASTERVARIABLES(obj):
    return _LibreDWG.dwg_object_to_RASTERVARIABLES(obj)

def dwg_object_to_RENDERENVIRONMENT(obj):
    return _LibreDWG.dwg_object_to_RENDERENVIRONMENT(obj)

def dwg_object_to_SCALE(obj):
    return _LibreDWG.dwg_object_to_SCALE(obj)

def dwg_object_to_SECTIONVIEWSTYLE(obj):
    return _LibreDWG.dwg_object_to_SECTIONVIEWSTYLE(obj)

def dwg_object_to_SECTION_MANAGER(obj):
    return _LibreDWG.dwg_object_to_SECTION_MANAGER(obj)

def dwg_object_to_SORTENTSTABLE(obj):
    return _LibreDWG.dwg_object_to_SORTENTSTABLE(obj)

def dwg_object_to_SPATIAL_FILTER(obj):
    return _LibreDWG.dwg_object_to_SPATIAL_FILTER(obj)

def dwg_object_to_STYLE(obj):
    return _LibreDWG.dwg_object_to_STYLE(obj)

def dwg_object_to_STYLE_CONTROL(obj):
    return _LibreDWG.dwg_object_to_STYLE_CONTROL(obj)

def dwg_object_to_SUN(obj):
    return _LibreDWG.dwg_object_to_SUN(obj)

def dwg_object_to_TABLEGEOMETRY(obj):
    return _LibreDWG.dwg_object_to_TABLEGEOMETRY(obj)

def dwg_object_to_UCS(obj):
    return _LibreDWG.dwg_object_to_UCS(obj)

def dwg_object_to_UCS_CONTROL(obj):
    return _LibreDWG.dwg_object_to_UCS_CONTROL(obj)

def dwg_object_to_UNKNOWN_OBJ(obj):
    return _LibreDWG.dwg_object_to_UNKNOWN_OBJ(obj)

def dwg_object_to_VBA_PROJECT(obj):
    return _LibreDWG.dwg_object_to_VBA_PROJECT(obj)

def dwg_object_to_VIEW(obj):
    return _LibreDWG.dwg_object_to_VIEW(obj)

def dwg_object_to_VIEW_CONTROL(obj):
    return _LibreDWG.dwg_object_to_VIEW_CONTROL(obj)

def dwg_object_to_VISUALSTYLE(obj):
    return _LibreDWG.dwg_object_to_VISUALSTYLE(obj)

def dwg_object_to_VPORT(obj):
    return _LibreDWG.dwg_object_to_VPORT(obj)

def dwg_object_to_VPORT_CONTROL(obj):
    return _LibreDWG.dwg_object_to_VPORT_CONTROL(obj)

def dwg_object_to_VX_CONTROL(obj):
    return _LibreDWG.dwg_object_to_VX_CONTROL(obj)

def dwg_object_to_VX_TABLE_RECORD(obj):
    return _LibreDWG.dwg_object_to_VX_TABLE_RECORD(obj)

def dwg_object_to_WIPEOUTVARIABLES(obj):
    return _LibreDWG.dwg_object_to_WIPEOUTVARIABLES(obj)

def dwg_object_to_XRECORD(obj):
    return _LibreDWG.dwg_object_to_XRECORD(obj)

def dwg_object_to_PDFDEFINITION(obj):
    return _LibreDWG.dwg_object_to_PDFDEFINITION(obj)

def dwg_object_to_DGNDEFINITION(obj):
    return _LibreDWG.dwg_object_to_DGNDEFINITION(obj)

def dwg_object_to_DWFDEFINITION(obj):
    return _LibreDWG.dwg_object_to_DWFDEFINITION(obj)

def dwg_object_to_ACSH_BREP_CLASS(obj):
    return _LibreDWG.dwg_object_to_ACSH_BREP_CLASS(obj)

def dwg_object_to_ACSH_CHAMFER_CLASS(obj):
    return _LibreDWG.dwg_object_to_ACSH_CHAMFER_CLASS(obj)

def dwg_object_to_ACSH_PYRAMID_CLASS(obj):
    return _LibreDWG.dwg_object_to_ACSH_PYRAMID_CLASS(obj)

def dwg_object_to_ALDIMOBJECTCONTEXTDATA(obj):
    return _LibreDWG.dwg_object_to_ALDIMOBJECTCONTEXTDATA(obj)

def dwg_object_to_ASSOC2DCONSTRAINTGROUP(obj):
    return _LibreDWG.dwg_object_to_ASSOC2DCONSTRAINTGROUP(obj)

def dwg_object_to_ASSOCACTION(obj):
    return _LibreDWG.dwg_object_to_ASSOCACTION(obj)

def dwg_object_to_ASSOCACTIONPARAM(obj):
    return _LibreDWG.dwg_object_to_ASSOCACTIONPARAM(obj)

def dwg_object_to_ASSOCARRAYACTIONBODY(obj):
    return _LibreDWG.dwg_object_to_ASSOCARRAYACTIONBODY(obj)

def dwg_object_to_ASSOCASMBODYACTIONPARAM(obj):
    return _LibreDWG.dwg_object_to_ASSOCASMBODYACTIONPARAM(obj)

def dwg_object_to_ASSOCBLENDSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_object_to_ASSOCBLENDSURFACEACTIONBODY(obj)

def dwg_object_to_ASSOCCOMPOUNDACTIONPARAM(obj):
    return _LibreDWG.dwg_object_to_ASSOCCOMPOUNDACTIONPARAM(obj)

def dwg_object_to_ASSOCDEPENDENCY(obj):
    return _LibreDWG.dwg_object_to_ASSOCDEPENDENCY(obj)

def dwg_object_to_ASSOCDIMDEPENDENCYBODY(obj):
    return _LibreDWG.dwg_object_to_ASSOCDIMDEPENDENCYBODY(obj)

def dwg_object_to_ASSOCEXTENDSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_object_to_ASSOCEXTENDSURFACEACTIONBODY(obj)

def dwg_object_to_ASSOCEXTRUDEDSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_object_to_ASSOCEXTRUDEDSURFACEACTIONBODY(obj)

def dwg_object_to_ASSOCFACEACTIONPARAM(obj):
    return _LibreDWG.dwg_object_to_ASSOCFACEACTIONPARAM(obj)

def dwg_object_to_ASSOCFILLETSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_object_to_ASSOCFILLETSURFACEACTIONBODY(obj)

def dwg_object_to_ASSOCLOFTEDSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_object_to_ASSOCLOFTEDSURFACEACTIONBODY(obj)

def dwg_object_to_ASSOCNETWORKSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_object_to_ASSOCNETWORKSURFACEACTIONBODY(obj)

def dwg_object_to_ASSOCOBJECTACTIONPARAM(obj):
    return _LibreDWG.dwg_object_to_ASSOCOBJECTACTIONPARAM(obj)

def dwg_object_to_ASSOCOFFSETSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_object_to_ASSOCOFFSETSURFACEACTIONBODY(obj)

def dwg_object_to_ASSOCOSNAPPOINTREFACTIONPARAM(obj):
    return _LibreDWG.dwg_object_to_ASSOCOSNAPPOINTREFACTIONPARAM(obj)

def dwg_object_to_ASSOCPATCHSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_object_to_ASSOCPATCHSURFACEACTIONBODY(obj)

def dwg_object_to_ASSOCPATHACTIONPARAM(obj):
    return _LibreDWG.dwg_object_to_ASSOCPATHACTIONPARAM(obj)

def dwg_object_to_ASSOCPLANESURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_object_to_ASSOCPLANESURFACEACTIONBODY(obj)

def dwg_object_to_ASSOCPOINTREFACTIONPARAM(obj):
    return _LibreDWG.dwg_object_to_ASSOCPOINTREFACTIONPARAM(obj)

def dwg_object_to_ASSOCREVOLVEDSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_object_to_ASSOCREVOLVEDSURFACEACTIONBODY(obj)

def dwg_object_to_ASSOCTRIMSURFACEACTIONBODY(obj):
    return _LibreDWG.dwg_object_to_ASSOCTRIMSURFACEACTIONBODY(obj)

def dwg_object_to_ASSOCVALUEDEPENDENCY(obj):
    return _LibreDWG.dwg_object_to_ASSOCVALUEDEPENDENCY(obj)

def dwg_object_to_ASSOCVARIABLE(obj):
    return _LibreDWG.dwg_object_to_ASSOCVARIABLE(obj)

def dwg_object_to_ASSOCVERTEXACTIONPARAM(obj):
    return _LibreDWG.dwg_object_to_ASSOCVERTEXACTIONPARAM(obj)

def dwg_object_to_BLKREFOBJECTCONTEXTDATA(obj):
    return _LibreDWG.dwg_object_to_BLKREFOBJECTCONTEXTDATA(obj)

def dwg_object_to_BLOCKALIGNEDCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKALIGNEDCONSTRAINTPARAMETER(obj)

def dwg_object_to_BLOCKANGULARCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKANGULARCONSTRAINTPARAMETER(obj)

def dwg_object_to_BLOCKARRAYACTION(obj):
    return _LibreDWG.dwg_object_to_BLOCKARRAYACTION(obj)

def dwg_object_to_BLOCKDIAMETRICCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKDIAMETRICCONSTRAINTPARAMETER(obj)

def dwg_object_to_BLOCKHORIZONTALCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKHORIZONTALCONSTRAINTPARAMETER(obj)

def dwg_object_to_BLOCKLINEARCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKLINEARCONSTRAINTPARAMETER(obj)

def dwg_object_to_BLOCKLINEARPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKLINEARPARAMETER(obj)

def dwg_object_to_BLOCKLOOKUPACTION(obj):
    return _LibreDWG.dwg_object_to_BLOCKLOOKUPACTION(obj)

def dwg_object_to_BLOCKLOOKUPPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKLOOKUPPARAMETER(obj)

def dwg_object_to_BLOCKPARAMDEPENDENCYBODY(obj):
    return _LibreDWG.dwg_object_to_BLOCKPARAMDEPENDENCYBODY(obj)

def dwg_object_to_BLOCKPOINTPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKPOINTPARAMETER(obj)

def dwg_object_to_BLOCKPOLARGRIP(obj):
    return _LibreDWG.dwg_object_to_BLOCKPOLARGRIP(obj)

def dwg_object_to_BLOCKPOLARPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKPOLARPARAMETER(obj)

def dwg_object_to_BLOCKPOLARSTRETCHACTION(obj):
    return _LibreDWG.dwg_object_to_BLOCKPOLARSTRETCHACTION(obj)

def dwg_object_to_BLOCKRADIALCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKRADIALCONSTRAINTPARAMETER(obj)

def dwg_object_to_BLOCKREPRESENTATION(obj):
    return _LibreDWG.dwg_object_to_BLOCKREPRESENTATION(obj)

def dwg_object_to_BLOCKROTATIONPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKROTATIONPARAMETER(obj)

def dwg_object_to_BLOCKSTRETCHACTION(obj):
    return _LibreDWG.dwg_object_to_BLOCKSTRETCHACTION(obj)

def dwg_object_to_BLOCKUSERPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKUSERPARAMETER(obj)

def dwg_object_to_BLOCKVERTICALCONSTRAINTPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKVERTICALCONSTRAINTPARAMETER(obj)

def dwg_object_to_BLOCKVISIBILITYPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKVISIBILITYPARAMETER(obj)

def dwg_object_to_BLOCKXYGRIP(obj):
    return _LibreDWG.dwg_object_to_BLOCKXYGRIP(obj)

def dwg_object_to_BLOCKXYPARAMETER(obj):
    return _LibreDWG.dwg_object_to_BLOCKXYPARAMETER(obj)

def dwg_object_to_DATALINK(obj):
    return _LibreDWG.dwg_object_to_DATALINK(obj)

def dwg_object_to_DBCOLOR(obj):
    return _LibreDWG.dwg_object_to_DBCOLOR(obj)

def dwg_object_to_DIMASSOC(obj):
    return _LibreDWG.dwg_object_to_DIMASSOC(obj)

def dwg_object_to_EVALUATION_GRAPH(obj):
    return _LibreDWG.dwg_object_to_EVALUATION_GRAPH(obj)

def dwg_object_to_FCFOBJECTCONTEXTDATA(obj):
    return _LibreDWG.dwg_object_to_FCFOBJECTCONTEXTDATA(obj)

def dwg_object_to_GRADIENT_BACKGROUND(obj):
    return _LibreDWG.dwg_object_to_GRADIENT_BACKGROUND(obj)

def dwg_object_to_GROUND_PLANE_BACKGROUND(obj):
    return _LibreDWG.dwg_object_to_GROUND_PLANE_BACKGROUND(obj)

def dwg_object_to_IBL_BACKGROUND(obj):
    return _LibreDWG.dwg_object_to_IBL_BACKGROUND(obj)

def dwg_object_to_IMAGE_BACKGROUND(obj):
    return _LibreDWG.dwg_object_to_IMAGE_BACKGROUND(obj)

def dwg_object_to_LEADEROBJECTCONTEXTDATA(obj):
    return _LibreDWG.dwg_object_to_LEADEROBJECTCONTEXTDATA(obj)

def dwg_object_to_LIGHTLIST(obj):
    return _LibreDWG.dwg_object_to_LIGHTLIST(obj)

def dwg_object_to_MATERIAL(obj):
    return _LibreDWG.dwg_object_to_MATERIAL(obj)

def dwg_object_to_MENTALRAYRENDERSETTINGS(obj):
    return _LibreDWG.dwg_object_to_MENTALRAYRENDERSETTINGS(obj)

def dwg_object_to_MLEADERSTYLE(obj):
    return _LibreDWG.dwg_object_to_MLEADERSTYLE(obj)

def dwg_object_to_MTEXTOBJECTCONTEXTDATA(obj):
    return _LibreDWG.dwg_object_to_MTEXTOBJECTCONTEXTDATA(obj)

def dwg_object_to_OBJECT_PTR(obj):
    return _LibreDWG.dwg_object_to_OBJECT_PTR(obj)

def dwg_object_to_PARTIAL_VIEWING_INDEX(obj):
    return _LibreDWG.dwg_object_to_PARTIAL_VIEWING_INDEX(obj)

def dwg_object_to_POINTCLOUDCOLORMAP(obj):
    return _LibreDWG.dwg_object_to_POINTCLOUDCOLORMAP(obj)

def dwg_object_to_POINTCLOUDDEF(obj):
    return _LibreDWG.dwg_object_to_POINTCLOUDDEF(obj)

def dwg_object_to_POINTCLOUDDEFEX(obj):
    return _LibreDWG.dwg_object_to_POINTCLOUDDEFEX(obj)

def dwg_object_to_POINTCLOUDDEF_REACTOR(obj):
    return _LibreDWG.dwg_object_to_POINTCLOUDDEF_REACTOR(obj)

def dwg_object_to_POINTCLOUDDEF_REACTOR_EX(obj):
    return _LibreDWG.dwg_object_to_POINTCLOUDDEF_REACTOR_EX(obj)

def dwg_object_to_PROXY_OBJECT(obj):
    return _LibreDWG.dwg_object_to_PROXY_OBJECT(obj)

def dwg_object_to_RAPIDRTRENDERSETTINGS(obj):
    return _LibreDWG.dwg_object_to_RAPIDRTRENDERSETTINGS(obj)

def dwg_object_to_RENDERENTRY(obj):
    return _LibreDWG.dwg_object_to_RENDERENTRY(obj)

def dwg_object_to_RENDERGLOBAL(obj):
    return _LibreDWG.dwg_object_to_RENDERGLOBAL(obj)

def dwg_object_to_RENDERSETTINGS(obj):
    return _LibreDWG.dwg_object_to_RENDERSETTINGS(obj)

def dwg_object_to_SECTION_SETTINGS(obj):
    return _LibreDWG.dwg_object_to_SECTION_SETTINGS(obj)

def dwg_object_to_SKYLIGHT_BACKGROUND(obj):
    return _LibreDWG.dwg_object_to_SKYLIGHT_BACKGROUND(obj)

def dwg_object_to_SOLID_BACKGROUND(obj):
    return _LibreDWG.dwg_object_to_SOLID_BACKGROUND(obj)

def dwg_object_to_SPATIAL_INDEX(obj):
    return _LibreDWG.dwg_object_to_SPATIAL_INDEX(obj)

def dwg_object_to_TABLESTYLE(obj):
    return _LibreDWG.dwg_object_to_TABLESTYLE(obj)

def dwg_object_to_TEXTOBJECTCONTEXTDATA(obj):
    return _LibreDWG.dwg_object_to_TEXTOBJECTCONTEXTDATA(obj)

def dwg_object_to_ASSOCARRAYMODIFYPARAMETERS(obj):
    return _LibreDWG.dwg_object_to_ASSOCARRAYMODIFYPARAMETERS(obj)

def dwg_object_to_ASSOCARRAYPATHPARAMETERS(obj):
    return _LibreDWG.dwg_object_to_ASSOCARRAYPATHPARAMETERS(obj)

def dwg_object_to_ASSOCARRAYPOLARPARAMETERS(obj):
    return _LibreDWG.dwg_object_to_ASSOCARRAYPOLARPARAMETERS(obj)

def dwg_object_to_ASSOCARRAYRECTANGULARPARAMETERS(obj):
    return _LibreDWG.dwg_object_to_ASSOCARRAYRECTANGULARPARAMETERS(obj)

